# Domain Name Service

## Functions

- [DnsAttachThread](#dnsattachthread-function)
- [DnsCancel](#dnscancel-function)
- [DnsCloseHandle](#dnsclosehandle-function)
- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsDisableTrace](#dnsdisabletrace-function)
- [DnsEnableTrace](#dnsenabletrace-function)
- [DnsEnumHostAliases](#dnsenumhostaliases-function)
- [DnsEnumMailExchanges](#dnsenummailexchanges-function)
- [DnsFormatAddress](#dnsformataddress-function)
- [DnsGetAddress](#dnsgetaddress-function)
- [DnsGetAddressFamily](#dnsgetaddressfamily-function)
- [DnsGetDefaultHostFile](#dnsgetdefaulthostfile-function)
- [DnsGetDefaultServicesFile](#dnsgetdefaultservicesfile-function)
- [DnsGetErrorString](#dnsgeterrorstring-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostFile](#dnsgethostfile-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetLastError](#dnsgetlasterror-function)
- [DnsGetLocalAddress](#dnsgetlocaladdress-function)
- [DnsGetLocalDomain](#dnsgetlocaldomain-function)
- [DnsGetLocalName](#dnsgetlocalname-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsGetResolverOptions](#dnsgetresolveroptions-function)
- [DnsGetRetryCount](#dnsgetretrycount-function)
- [DnsGetServerAddress](#dnsgetserveraddress-function)
- [DnsGetServerPort](#dnsgetserverport-function)
- [DnsGetServiceName](#dnsgetservicename-function)
- [DnsGetServicePort](#dnsgetserviceport-function)
- [DnsGetTimeout](#dnsgettimeout-function)
- [DnsHostNameToUnicode](#dnshostnametounicode-function)
- [DnsInitialize](#dnsinitialize-function)
- [DnsMatchHostName](#dnsmatchhostname-function)
- [DnsNormalizeHostName](#dnsnormalizehostname-function)
- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsReset](#dnsreset-function)
- [DnsSetHostFile](#dnssethostfile-function)
- [DnsSetLastError](#dnssetlasterror-function)
- [DnsSetLocalDomain](#dnssetlocaldomain-function)
- [DnsSetResolverOptions](#dnssetresolveroptions-function)
- [DnsSetRetryCount](#dnssetretrycount-function)
- [DnsSetTimeout](#dnssettimeout-function)
- [DnsUninitialize](#dnsuninitialize-function)
- [DnsUnregisterServer](#dnsunregisterserver-function)
## Structures

- [HOSTENT](#hostent-structure)
- [INITDATA](#initdata-structure)
- [INTERNET_ADDRESS](#internet_address-structure)

---

## DnsAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI DnsAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The DnsAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should DnsAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the DnsCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the DnsUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsCancel](#dnscancel-function)
- [DnsCloseHandle](#dnsclosehandle-function)
- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsUninitialize](#dnsuninitialize-function)

---

## DnsCancel Function

Cancel an outstanding nameserver query

### Prototype

```c
INT WINAPI DnsCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)

---

## DnsCloseHandle Function

Close the current client session

### Prototype

```c
INT WINAPI DnsCloseHandle(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsUninitialize](#dnsuninitialize-function)
- [DnsUnregisterServer](#dnsunregisterserver-function)

---

## DnsCreateHandle Function

Create a new handle for the current client session

### Prototype

```c
HCLIENT WINAPI DnsCreateHandle(
    INT nTimeout,
    INT nRetries,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTimeout` | `INT` | The number of seconds that the client will wait for a response before failing the query. The constant DNS_TIMEOUT can be used to specify the default timeout period of 15 seconds. |
| `nRetries` | `INT` | The number of attempts the client will make to resolve a query after a timeout. The constant DNS_RETRIES can be used to specify the default retry count of 4. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more resolver options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `DNS_OPTION_NONE`: No additional resolver options specified. This is the default value, and it is recommended that most applications do not specify additional options unless the implications of doing so are understood.
    - `DNS_OPTION_PROMOTE`: Promotes the server that successfully completed the last query to the first server that will be used to resolve subsequent queries. This option can improve performance in some cases where one or more of the registered servers are non-responsive. This option takes precedence over the DNS_OPTION_ROTATE option.
    - `DNS_OPTION_ROTATE`: Enables a round-robin selection of nameservers when performing queries. Normally each nameserver is queried in the same order. This option rotates the available nameservers so a different server is used with each query.
    - `DNS_OPTION_AUTHONLY`: Require the answer from the nameserver to be authoritative, not from the server's cache. This option is included for future expansion as most servers do not support this feature and will ignore it.
    - `DNS_OPTION_PRIMARY`: Queries are only accepted from the primary nameserver. This option is included for future expansion as most servers do not support this feature and will ignore it.
    - `DNS_OPTION_NORECURSE`: Disable the sending of recursive queries to the nameserver. Specifying this option will disable the bit in the DNS request header that specifies recursion is desired.
    - `DNS_OPTION_NOSEARCH`: Disable additional queries of higher domains in the search list if the host name cannot be resolved. If this option is specified, and the host name cannot be resolved using the local domain name an error is returned immediately. This option is ignored if no local domain has been specified or if the DNS_OPTION_NOSUFFIX option has been specified.
    - `DNS_OPTION_NOSUFFIX`: Disable additional queries using the local domain name if the host name is not a fully qualified domain name and cannot be resolved. This option is ignored if no local domain has been specified.
    - `DNS_OPTION_NONAMECHECK`: Disable checking the host name for invalid characters, such as the underscore and control characters. By default, host names are checked to ensure they're valid before submitting a query to the nameserver.
    - `DNS_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsCreateHandle function returns a handle to the application which can be used to perform nameserver queries. By default, the library will use the nameservers which are configured for the local host. To determine the IP addresses of those nameservers, use the DnsGetServerAddress function.
- To specify alternate nameservers, either as replacements for or additions to the default nameservers, use the DnsRegisterServer function.
- It is recommended that most applications do not specify any resolver options and use the default behavior. Specifying these options without understanding how they can affect standard queries can result in unexpected failures. In particular, caution should be used when specifying the DNS_OPTION_NORECURSE and DNS_OPTION_NOSEARCH options as they change the normal process of resolving a host name.
- Specifying the DNS_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsCloseHandle](#dnsclosehandle-function)
- [DnsGetResolverOptions](#dnsgetresolveroptions-function)
- [DnsGetServerAddress](#dnsgetserveraddress-function)
- [DnsInitialize](#dnsinitialize-function)
- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsSetLocalDomain](#dnssetlocaldomain-function)
- [DnsSetResolverOptions](#dnssetresolveroptions-function)

---

## DnsDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI DnsDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsEnableTrace](#dnsenabletrace-function)

---

## DnsEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI DnsEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the remote host.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsDisableTrace](#dnsdisabletrace-function)

---

## DnsEnumHostAliases Function

Enumerate the aliases for the specified host name or address

### Prototype

```c
INT WINAPI DnsEnumHostAliases(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    LPTSTR lpszHostAlias,
    INT nMaxAliases
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string that contains the hostname to be resolved. If a fully qualified domain name is not provided, the default local domain will be used. This value may also be an IP address. |
| `lpszHostAlias` | `LPTSTR` | Pointer to an array of string pointers which specify one or more host aliases. If the application needs to store these values, a local copy should be made because they are invalidated when another host name is resolved. |
| `nMaxAliases` | `INT` | The maximum number of aliases in the array. This parameter must have a value of at least one, or an error will be returned. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of host aliases. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The application must never attempt to modify the host aliases or free any of the values. This function uses an internal data structure to store the host information and only one copy of this structure is allocated per thread. The application must copy any information it needs before issuing any other function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostName](#dnsgethostname-function)

---

## DnsEnumMailExchanges Function

Return a list of mail exchanges for the specified host name or IP address

### Prototype

```c
INT WINAPI DnsEnumMailExchanges(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    LPTSTR lpszMailExchanges,
    INT nMaxMailExchanges
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string that contains the hostname or domain name to be queried. |
| `lpszMailExchanges` | `LPTSTR` | Pointer to an array of string pointers which specify one or more mail exchanges. If the application needs to store these values, a local copy should be made because they are invalidated when another host name is resolved. The list of mail exchange records is sorted in priority order, from highest (i.e., those whose preference value is smallest) to lowest. |
| `nMaxMailExchanges` | `INT` | The maximum number of mail exchanges in the array. If this parameter is 0, then the function will return the number of mail exchanges, but the list of mail exchanges will not be output in lpszMailExchanges. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of mail exchanges. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The application must never attempt to modify the mail exchange host names or free any of the values. This function uses an internal data structure to store the host information and only one copy of this structure is allocated per thread. The application must copy any information it needs before issuing any other function calls.

### Example

```cpp
// Count the number of mail exchanges
if ((nMX = DnsEnumMailExchanges(hClient,szHostName, NULL, 0)) == DNS_ERROR)
{
	dwError = DnsGetLastError();
	DnsGetErrorString(dwError, szError, BUFSIZE);
	printf("DnsEnumMailExchanges failed for %s: %s\n", szHostName, szError);
}
else
{
	int nIndex;
	printf("%d MX for %s\n",nMX, szHostName);

	// Allocate memory for the list of mail exchanges
	lpszMailExchanges = (LPTSTR *)malloc(nMX * sizeof(LPTSTR));

	// Retrieve the list of mail exchanges
	nMX = DnsEnumMailExchanges(hClient, szHostName, lpszMailExchanges, nMX);

	for (nIndex = 0; nIndex < nMX; nIndex++)
		printf("#%d: %s\n", nIndex+1, *lpszMailExchanges++);

	free(lpszMailExchanges);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetMailExchange](#dnsgetmailexchange-function)

---

## DnsFormatAddress Function

Convert a numeric IPv4 address to a string

### Prototype

```c
INT WINAPI DnsFormatAddress(
    LPINTERNET_ADDRESS lpAddress,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpAddress` | `LPINTERNET_ADDRESS` | A pointer to an INTERNET_ADDRESS structure which specifies the numeric IPv4 or IPv6 address to be converted into a string. |
| `lpszAddress` | `LPTSTR` | A pointer to a null-terminated array of characters which will contain the converted IPv4 address in dot-notation. This string should be at least 16 characters in length. |
| `nMaxLength` | `INT` | The maximum number of characters which may be copied in to the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the string buffer. If the function fails, the return value is zero. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetAddress](#dnsgetaddress-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [INTERNET_ADDRESS](#internet_address-function)

---

## DnsGetAddress Function

Convert an IPv4 address string in dotted notation to a numeric IP address

### Prototype

```c
INT WINAPI DnsGetAddress(
    LPCTSTR lpszAddress,
    INT nAddressFamily,
    LPINTERNET_ADDRESS lpAddress
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAddress` | `LPCTSTR` | A pointer to a string which specifies an IPv4 address in dotted notation. |
| `nAddressFamily` | `INT` | An integer value which specifies the type of IP address. If this parameter is zero, the address family will be determined automatically based on the format of the address string. If this parameter is DNS_ADDRESS_IPV4, the address must be in IPv4 format, and if it is DNS_ADDRESS_IPV6, the address must be in IPv6 format. |
| `lpAddress` | `LPINTERNET_ADDRESS` | A pointer to an INTERNET_ADDRESS structure that will contain the numeric form of the IPv4 or IPv6 address in network byte order when the function returns. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the address family for the IP address. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- This function will only accept a string that is in the proper format for an IP address, and cannot be used to resolve a host name. To perform host name resolution, use the DnsGetHostAddress function. To convert a numeric address to an address string, use the DnsFormatAddress function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsFormatAddress](#dnsformataddress-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [INTERNET_ADDRESS](#internet_address-function)

---

## DnsGetAddressFamily Function

Return the address family for the specified IP address

### Prototype

```c
INT WINAPI DnsGetAddressFamily(
    LPCTSTR lpszAddress
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAddress` | `LPCTSTR` | A pointer to a string which specifies an IPv4 or IPv6 address. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is DNS_ADDRESS_IPV4 if the address is in IPv4 format, or DNS_ADDRESS_IPV6 if the address is in IPv6 format. If the address string is not in a recognized format, it returns DNS_ADDRESS_ANY.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsFormatAddress](#dnsformataddress-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)

---

## DnsGetDefaultHostFile Function

Return the full path to the default host file on the local system

### Prototype

```c
INT WINAPI DnsGetDefaultHostFile(
    LPTSTR lpszFileName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFileName` | `LPTSTR` | Pointer to a string buffer that will contain the fully qualified path to the default host file. It is recommended that this buffer be at least MAX_PATH characters in size. This parameter may be NULL, in which case the function will return the length of the string, not including the terminating null byte. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied to the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the string buffer, not including the terminating null character. A return value of zero indicates that the default host file could not be determined for the current platform. To get extended error information, call DnsGetLastError.

### Remarks

- This function returns the location of the host file and does not determine if the file actually exists. The default location for this file is in a protected area of the Windows operating system and can only be modified by a process with administrative privileges.
- The default host file is processed before performing a nameserver lookup when resolving a hostname into an IP address, or an IP address into a hostname. To specify an alternate local host file, use the DnsSetHostFile function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetDefaultServicesFile](#dnsgetdefaultservicesfile-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostFile](#dnsgethostfile-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsSetHostFile](#dnssethostfile-function)

---

## DnsGetDefaultServicesFile Function

Return the full path the default services file on the local system

### Prototype

```c
INT WINAPI DnsGetDefaultServicesFile(
    LPTSTR lpszFileName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFileName` | `LPTSTR` | Pointer to a string buffer that will contain the fully qualified path to the default services file. It is recommended that this buffer be at least MAX_PATH characters in size. This parameter may be NULL, in which case the function will return the length of the string, not including the terminating null byte. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied to the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the string buffer, not including the terminating null character. A return value of zero indicates that the default services file could not be determined for the current platform. To get extended error information, call DnsGetLastError.

### Remarks

- This function returns the location of the services file and does not determine if the file actually exists. The default location for this file is in a protected area of the Windows operating system and can only be modified by a process with administrative privileges.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetDefaultHostFile](#dnsgetdefaulthostfile-function)
- [DnsGetServiceName](#dnsgetservicename-function)
- [DnsGetServicePort](#dnsgetserviceport-function)

---

## DnsGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI DnsGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The last-error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is 0, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetLastError](#dnsgetlasterror-function)
- [DnsSetLastError](#dnssetlasterror-function)

---

## DnsGetHostAddress Function

Return the IP address for the specified hostname

### Prototype

```c
INT WINAPI DnsGetHostAddress(
    HCLIENT hClient,
    INT nAddressFamily,
    LPCTSTR lpszHostName,
    LPTSTR lpszHostAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nAddressFamily` | `INT` | An integer value which specifies the type of address that should be returned. A value of DNS_ADDRESS_IPV4 specifies that the IPv4 address for the host should be returned. A value of DNS_ADDRESS_IPV6 specifies that the IPv6 address for the host should be returned. A value of DNS_ADDRESS_ANY specifies that if the host only has an IPv6 address, that value should be returned, otherwise return the IPv4 address for the host. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string that contains the hostname to be resolved. If a fully qualified domain name is not provided, the default local domain will be used. |
| `lpszHostAddress` | `LPTSTR` | Pointer to the buffer that will contain the IP address, stored as a string in dot notation. This buffer should be at least 48 characters in length to accommodate both IPv4 and IPv6 addresses. The format of the address is determined by the address family specified. |
| `nMaxLength` | `INT` | The maximum length of the string buffer. The maximum length of the buffer must include the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the host address buffer. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsGetHostAddress function may return an address in either IPv4 or IPv6 format, depending on the address family that is specified and what records exist for the host. If your application does not support the IPv6 address format, you must specify the nAddressFamily parameter as DNS_ADDRESS_IPV4 to prevent the possibility of an IPv6 address being returned.
- If the nAddressFamily parameter is specified as DNS_ADDRESS_ANY, this function will first check for an IPv4 address record for the host. If it exists, it will return that address. If the host does not have an IPv4 address, it will then check for an IPv6 address record and return that address. This gives preference to IPv4 addresses, but your application should never depend on this behavior. In the future, this function may change to give preference to IPv6 addresses.
- To determine what format an address is in, use the DnsGetAddressFamily function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsEnumHostAliases](#dnsenumhostaliases-function)
- [DnsGetAddressFamily](#dnsgetaddressfamily-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)

---

## DnsGetHostByAddress Function

Return a pointer to data for the specified host IP address

### Prototype

```c
LPHOSTENT WINAPI DnsGetHostByAddress(
    HCLIENT hClient,
    LPVOID lpvAddress,
    INT cbAddress,
    INT nAddressFamily
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpvAddress` | `LPVOID` | Pointer to an integer IPv4 address in network byte order. |
| `cbAddress` | `INT` | The length of the address in bytes; this value should always be 4. |
| `nAddressFamily` | `INT` | The type of address being resolved; this value should always be DNS_ADDRESS_IPV4 as defined in the Windows Sockets header file. |

### Return Value

- **Type**: `LPHOSTENT`
- If the function succeeds, the return value is a pointer to a HOSTENT structure. If the function fails, the return value is NULL. To get extended error information, call DnsGetLastError.

### Remarks

- The application must never attempt to modify this structure or to free any of its components. Only one copy of this structure is allocated per thread, so the application should copy any information it needs before issuing any other function calls. To convert an IPv4 address string in dotted notation to a 32-bit IP address, use the DnsGetAddress function.
- This function is included for compatibility with existing applications which already use the HOSTENT structure. Because this function returns a pointer to a complex structure, it may not be suitable for some programming languages. There is no Unicode version of the HOSTENT structure and members will return pointers to ANSI strings even if the application is compiled to use Unicode.
- This function is not compatible with IPv6 addresses. For applications that must support both IPv4 and IPv6 address formats, use the DnsGetHostAddress and DnsGetHostName functions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)

---

## DnsGetHostByName Function

Return a pointer to data for the specified host name

### Prototype

```c
LPHOSTENT WINAPI DnsGetHostByName(
    HCLIENT hClient,
    LPCTSTR lpszHostName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string that contains the hostname to be resolved. If a fully qualified domain name is not provided, the default local domain will be used. |

### Return Value

- **Type**: `LPHOSTENT`
- If the function succeeds, the return value is a pointer to a HOSTENT structure. If the function fails, the return value is NULL. To get extended error information, call DnsGetLastError.

### Remarks

- The application must never attempt to modify this structure or to free any of its components. Only one copy of this structure is allocated per thread, so the application should copy any information it needs before issuing any other function calls. This function will automatically resolve an IP address passed as a string, converting it to numeric form and calling the DnsGetHostByAddress function.
- This function is included for compatibility with existing applications which already use the HOSTENT structure. Because this function returns a pointer to a complex structure, it may not be suitable for some programming languages. There is no Unicode version of the HOSTENT structure and members will return pointers to ANSI strings even if the application is compiled to use Unicode.
- This function is not compatible with IPv6 addresses. For applications that must support both IPv4 and IPv6 address formats, use the DnsGetHostAddress and DnsGetHostName functions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)

---

## DnsGetHostFile Function

Return the name of the current host file

### Prototype

```c
INT WINAPI DnsGetHostFile(
    HCLIENT hClient,
    LPTSTR lpszFileName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszFileName` | `LPTSTR` | Pointer to a string buffer that will contain the host file name. It is recommended that this buffer be at least MAX_PATH characters in size. This parameter may be NULL, in which case the function will return the length of the string, not including the terminating null character. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied to the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the string buffer, not including the terminating null character. A return value of zero indicates no host file has been specified or the function was unable to determine the file name. To get extended error information, call DnsGetLastError.

### Remarks

- This function returns the name of the host file that is cached in memory for the current thread. If an alternate host file has not been specified, this function will return the path to the default host file. The contents of the file on the disk may have changed after the file was loaded into memory. To reload the host file or clear the cache, call the DnsSetHostFile function.
- If a host file has been specified, it is processed before the default host file when resolving a hostname into an IP address, or an IP address into a hostname. If the host name or address is not found, or no host file has been specified, a nameserver lookup is performed.
- The host file returned by this function may be different than the default host file for the local system. To determine the file name for the default host file, use the DnsGetDefaultHostFile function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetDefaultHostFile](#dnsgetdefaulthostfile-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsSetHostFile](#dnssethostfile-function)

---

## DnsGetHostInfo Function

Return additional information for the specified host

### Prototype

```c
INT WINAPI DnsGetHostInfo(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string which specifies the host name that information will be returned for. |
| `lpszBuffer` | `LPTSTR` | Pointer to the buffer which will contain the host information returned by the nameserver. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the length of the host information buffer is returned. A return value of zero indicates that no information is available for the specified host. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- Many systems to do not maintain HINFO records for a site since that information can potentially be used to compromise system security. The information is typically used for administrative purposes with internal networks.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)

---

## DnsGetHostName Function

Return the host name for the specified IP address

### Prototype

```c
INT WINAPI DnsGetHostName(
    HCLIENT hClient,
    LPCTSTR lpszHostAddress,
    LPTSTR lpszHostName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostAddress` | `LPCTSTR` | Pointer to a string that specifies an IPv4 or IPv6 formatted address. |
| `lpszHostName` | `LPTSTR` | Pointer to the buffer that will contain the fully qualified domain name for the specified host. This buffer should be at least 64 characters in length. |
| `nMaxLength` | `INT` | The maximum length of the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the host name buffer. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsGetHostName function first looks to see if there is an entry in the local host file for the specified IP address, and if one exists, it will return the host name for that address. If you do not want to use the local host file at all, and only return an host name if a DNS query resolves the address, use the DnsGetRecord function and specify a record type of DNS_RECORD_PTR.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsEnumHostAliases](#dnsenumhostaliases-function)
- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)

---

## DnsGetHostServices Function

Return a list of services supported by the specified host

### Prototype

```c
INT WINAPI DnsGetHostServices(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    INT nProtocol,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string which specifies the host name that information will be returned for. |
| `nProtocol` | `INT` | The protocol for those services that information should be returned about. The following protocols are recognized: |
| `lpszBuffer` | `LPTSTR` | Pointer to the buffer which will contain the host information returned by the nameserver. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Options

- **nProtocol** Parameter

    - `INET_PROTOCOL_TCP`: Services that use the Transmission Control Protocol (TCP)
    - `INET_PROTOCOL_UDP`: Services that use the User Datagram Protocol (UDP)

### Return Value

- **Type**: `INT`
- If the function succeeds, the length of the host services buffer is returned. A return value of zero indicates that no information is available for the specified host. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- Many systems to do not maintain complete services records for a site since that information can potentially be used to compromise system security. An application should not depend on this information being available for any given record.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)

---

## DnsGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI DnsGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the calling thread's last-error code value. Functions set this value by calling the DnsSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last-error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the DnsGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call DnsSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or DNS_ERROR. Those functions which call DnsSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetErrorString](#dnsgeterrorstring-function)
- [DnsSetLastError](#dnssetlasterror-function)

---

## DnsGetLocalAddress Function

Return the IP address for the local host

### Prototype

```c
INT WINAPI DnsGetLocalAddress(
    HCLIENT hClient,
    INT nAddressFamily,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nAddressFamily` | `INT` | An integer value which specifies the type of address that should be returned. A value of DNS_ADDRESS_IPV4 specifies that the IPv4 address for the host should be returned. A value of DNS_ADDRESS_IPV6 specifies that the IPv6 address for the host should be returned. A value of DNS_ADDRESS_ANY specifies that if the host only has an IPv6 address, that value should be returned, otherwise return the IPv4 address for the host. |
| `lpszAddress` | `LPTSTR` | Pointer to the buffer that will contain the IP address, stored as a string in dot notation. This buffer should be at least 40 characters in length to accommodate both IPv4 and IPv6 addresses. |
| `nMaxLength` | `INT` | The maximum length of the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the host address buffer. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsGetLocalAddress function may return an address in either IPv4 or IPv6 format, depending on the address family that is specified and what records exist for the host. If your application does not support the IPv6 address format, you must specify the nAddressFamily parameter as DNS_ADDRESS_IPV4 to prevent the possibility of an IPv6 address being returned.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)

---

## DnsGetLocalDomain Function

Return the local domain name for the current session

### Prototype

```c
INT WINAPI DnsGetLocalDomain(
    HCLIENT hClient,
    LPTSTR lpszDomain,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszDomain` | `LPTSTR` | Pointer to the buffer that is used to store the local domain name. If no local domain name has been set, this buffer will be set to zero length. |
| `nMaxLength` | `INT` | The maximum number of bytes to copy into the buffer, including the null character terminator. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the domain name string. A return value of zero indicates that no local domain name has been set. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsSetLocalDomain](#dnssetlocaldomain-function)

---

## DnsGetLocalName Function

Return the local host name

### Prototype

```c
INT WINAPI DnsGetLocalName(
    HCLIENT hClient,
    LPTSTR lpszLocalName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszLocalName` | `LPTSTR` | Pointer to a string buffer that will contain the local host name. It is recommended that this buffer be at least 64 characters in size. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied to the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the local host name. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)

---

## DnsGetMailExchange Function

Return the host that processes mail for the specified domain

### Prototype

```c
INT WINAPI DnsGetMailExchange(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    LPINT lpnPreference,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string which specifies the host name that information will be returned for. |
| `lpnPreference` | `LPINT` | Pointer to the integer which will contain the preference for the specified mail exchange host. |
| `lpszBuffer` | `LPTSTR` | Pointer to the buffer which will contain the host information returned by the nameserver. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the length of the buffer is returned. A return value of zero indicates that no information is available for the specified host. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The mail exchange record is typically used by mail delivery agents to determine what system is responsible for accepting mail addressed to a given domain. This function will return the first MX record provided by the server. Note that some domains may have multiple mail servers. To enumerate all of the mail exchange records for a domain, use the DnsEnumMailExchanges function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsEnumMailExchanges](#dnsenummailexchanges-function)

---

## DnsGetRecord Function

Return record data for the current host

### Prototype

```c
INT WINAPI DnsGetRecord(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    INT nRecordType,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | Pointer to the string which specifies the host name that information will be returned for. |
| `nRecordType` | `INT` | The record type for the information that should be returned. The following record types are recognized: |
| `lpszBuffer` | `LPTSTR` | Pointer to the buffer which will contain the host information returned by the nameserver. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Options

- **nRecordType** Parameter

    - `DNS_RECORD_NONE`: No record type
    - `DNS_RECORD_ADDRESS`: IPv4 host address
    - `DNS_RECORD_NS`: Authoritative nameserver
    - `DNS_RECORD_CNAME`: Canonical host name (alias)
    - `DNS_RECORD_SOA`: Start of Authority
    - `DNS_RECORD_WKS`: Well known services
    - `DNS_RECORD_PTR`: Domain name
    - `DNS_RECORD_HINFO`: Host information
    - `DNS_RECORD_MINFO`: Mailbox information
    - `DNS_RECORD_MX`: Mail exchange host
    - `DNS_RECORD_TXT`: Text strings
    - `DNS_RECORD_AAAA`: IPv6 host address
    - `DNS_RECORD_LOC`: Location information
    - `DNS_RECORD_UINFO`: User information
    - `DNS_RECORD_UID`: User ID
    - `DNS_RECORD_GID`: Group ID

### Return Value

- **Type**: `INT`
- If the function succeeds, the length of the information buffer is returned. A return value of zero indicates that no information for that record is available for the specified host. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsGetRecord function can be used to resolve a host name into an IPv4 address using the record type DNS_RECORD_ADDRESS, or an IPv6 address using the record type DNS_RECORD_AAAA. It can also be used to perform a reverse lookup and resolve an IP address into a host name by using the record type DNS_RECORD_PTR.
- To determine the host that serves as the primary or master DNS for a zone, the record name should be specified as the domain name (e.g.: microsoft.com) and the record type should be DNS_RECORD_SOA. The value returned will the fully qualified domain name for host.
- Note that this function does not reference a local host file when resolving host names or addresses. If the record lookup fails, this function will return an error even if there's an entry for the host in the file that has been specified by a call to DnsSetHostFile.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)

---

## DnsGetResolverAddress Function

Return address of last nameserver that resolved query

### Prototype

```c
INT WINAPI DnsGetResolverAddress(
    HCLIENT hClient,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszAddress` | `LPTSTR` | A pointer to a string buffer that will contain the address of the nameserver when the function returns. This buffer should be large enough to store both IPv4 and IPv6 addresses, with a minimum length of 40 characters. If this parameter is NULL, it will be ignored. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied into the string buffer. If this value is zero, the lpszAddress parameter will be ignored and the function will return the length of the address. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the address, not including the terminating null character. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostByAddress](#dnsgethostbyaddress-function)
- [DnsGetHostByName](#dnsgethostbyname-function)
- [DnsGetHostInfo](#dnsgethostinfo-function)
- [DnsGetHostServices](#dnsgethostservices-function)
- [DnsGetMailExchange](#dnsgetmailexchange-function)
- [DnsGetRecord](#dnsgetrecord-function)
- [DnsRegisterServer](#dnsregisterserver-function)

---

## DnsGetResolverOptions Function

Return the current resolver options for the specified client session

### Prototype

```c
DWORD WINAPI DnsGetResolverOptions(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the resolver options set for the client session. If the client handle is invalid or no resolver options have been specified, the function will return zero.

### Remarks

- The DnsGetResolverOptions function can be used to determine which resolver options have been specified for the client session. For a list of the available options, refer to the documentation for the DnsSetResolverOptions function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsSetResolverOptions](#dnssetresolveroptions-function)

---

## DnsGetRetryCount Function

Get the number of times the query is sent to each server

### Prototype

```c
INT WINAPI DnsGetRetryCount(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the retry count. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The retry count determines the amount of time the client will wait for a response from each query, the effective amount of time the client will wait increases with each nameserver and the total number of retries specified. For example, two nameservers registered with the client, with a default of 4 retries per nameserver and a timeout value of 10 seconds, would cause the client to wait a total of 80 seconds until it returns an error indicating that it was unable to resolve the query.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetTimeout](#dnsgettimeout-function)
- [DnsSetRetryCount](#dnssetretrycount-function)
- [DnsSetTimeout](#dnssettimeout-function)

---

## DnsGetServerAddress Function

Return the address of the specified nameserver

### Prototype

```c
INT WINAPI DnsGetServerAddress(
    HCLIENT hClient,
    INT nServer,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle the client session. |
| `nServer` | `INT` | The index into the client's nameserver table. This index is the same value that is passed to the DnsRegisterServer function when the nameserver is registered. |
| `lpszAddress` | `LPTSTR` | A pointer to a string buffer that will contain the address of the nameserver when the function returns. This buffer should be large enough to store both IPv4 and IPv6 addresses, with a minimum length of 40 characters. If this parameter is NULL, it will be ignored. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied into the string buffer. If this value is zero, the lpszAddress parameter will be ignored and the function will return the length of the address. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the address, not including the terminating null character. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsGetServerPort](#dnsgetserverport-function)
- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsUnregisterServer](#dnsunregisterserver-function)

---

## DnsGetServerPort Function

Return the port of the specified nameserver

### Prototype

```c
INT WINAPI DnsGetServerPort(
    HCLIENT hClient,
    INT nServer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle the client session. |
| `nServer` | `INT` | The index into the client's nameserver table. This index is the same value that is passed to the DnsRegisterServer function when the nameserver is registered. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the specified port number. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetResolverAddress](#dnsgetresolveraddress-function)
- [DnsGetServerAddress](#dnsgetserveraddress-function)
- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsUnregisterServer](#dnsunregisterserver-function)

---

## DnsGetServiceName Function

Return the name of a service assigned to the specified port number

### Prototype

```c
INT WINAPI DnsGetServiceName(
    HCLIENT hClient,
    INT nServicePort,
    INT nServiceType,
    LPTSTR lpszServiceName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nServicePort` | `INT` | An integer which specifies the port number. This value must be a valid port number in the range of 1 through 65535. |
| `nServiceType` | `INT` | An integer which specifies the protocol associated with the service. This may be one of the following values: |
| `lpszServiceName` | `LPTSTR` | A pointer to a string buffer which will contain the name of the service assigned to the port number when the function returns. This parameter cannot be NULL and the buffer must be large enough to store the complete service name, including the terminating null character. |
| `nMaxLength` | `INT` | An integer which specifies the maximum number of characters which can be copied into the string buffer. |

### Options

- **nServiceType** Parameter

    - `DNS_SERVICE_ANY`: Match the first service entry for the specified port number regardless of the protocol used.
    - `DNS_SERVICE_TCP`: Match a service entry for the specified port number which uses the TCP protocol.
    - `DNS_SERVICE_UDP`: Match a service entry for the specified port number which uses the UDP protocol.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the string buffer, not including the terminating null character. If the function fails, the return value is zero. To get extended error information, call DnsGetLastError.

### Remarks

- Port numbers can be assigned human readable service names to make them easier to reference. This function will return the name assigned to the specified port number. If the port number does not have an assigned name, this function will return zero and the last error code will be set to ST_ERROR_INVALID_SERVICE_NAME.
- Most applications should use DNS_SERVICE_ANY to return the service name assigned to a port number. Services typically use the same name and port number regardless of the protocol used to establish a connection with them. However, specifying a protocol can enable your application to determine if the service supports the protocol. For example, calling this function with port number 21 and DNS_SERVICE_UDP would result in an error because FTP only supports TCP connections.
- Service names are assigned by the Internet Assigned Numbers Authority (IANA) and the database of recognized service names can be returned by calling the DnsGetDefaultServicesFile function. The default location for this file is in a protected area of the Windows operating system and can only be modified by a process with administrative privileges.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetDefaultHostFile](#dnsgetdefaulthostfile-function)
- [DnsGetDefaultServicesFile](#dnsgetdefaultservicesfile-function)
- [DnsGetHostFile](#dnsgethostfile-function)
- [DnsGetServicePort](#dnsgetserviceport-function)

---

## DnsGetServicePort Function

Return the port number assigned to the specified service name

### Prototype

```c
INT WINAPI DnsGetServicePort(
    HCLIENT hClient,
    LPCTSTR lpszServiceName,
    INT nServiceType
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszServiceName` | `LPCTSTR` | A pointer to a null terminated string which specifies the service name. Service names are not case sensitive. |
| `nServiceType` | `INT` | An integer which specifies the protocol associated with the service. This may be one of the following values: |

### Options

- **nServiceType** Parameter

    - `DNS_SERVICE_ANY`: Match the first service entry for the specified port number regardless of the protocol used.
    - `DNS_SERVICE_TCP`: Match a service entry for the specified port number which uses the TCP protocol.
    - `DNS_SERVICE_UDP`: Match a service entry for the specified port number which uses the UDP protocol.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the port number assigned to the service name. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- Port numbers can be assigned human readable service names to make them easier to reference. This function will return the port number assigned to the specified service name. If the service name does not exist, this function will return DNS_ERROR and the last error code will be set to ST_ERROR_INVALID_SERVICE_NAME.
- Most applications should use DNS_SERVICE_ANY to return the port number assigned to a service. Services typically use the same name and port number regardless of the protocol used to establish a connection with them. However, specifying a protocol can enable your application to determine if the service supports the protocol. For example, calling this function with the service name "ftp" and DNS_SERVICE_UDP would result in an error because FTP only supports TCP connections.
- Service names are assigned by the Internet Assigned Numbers Authority (IANA) and the database of recognized service names can be returned by calling the DnsGetDefaultServicesFile function. The default location for this file is in a protected area of the Windows operating system and can only be modified by a process with administrative privileges.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetDefaultHostFile](#dnsgetdefaulthostfile-function)
- [DnsGetDefaultServicesFile](#dnsgetdefaultservicesfile-function)
- [DnsGetHostFile](#dnsgethostfile-function)
- [DnsGetServicePort](#dnsgetserviceport-function)

---

## DnsGetTimeout Function

Get the number of seconds until a query times out

### Prototype

```c
INT WINAPI DnsGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The timeout value determines the amount of time the client will wait for a response from each query, the effective amount of time the client will wait increases with each nameserver and the total number of retries specified. For example, two nameservers registered with the client, with a default of 4 retries per nameserver and a timeout value of 10 seconds, would cause the client to wait a total of 80 seconds until it returns an error indicating that it was unable to resolve the query.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetRetryCount](#dnsgetretrycount-function)
- [DnsSetRetryCount](#dnssetretrycount-function)
- [DnsSetTimeout](#dnssettimeout-function)

---

## DnsHostNameToUnicode Function

Converts the canonical form of a host name to its Unicode version

### Prototype

```c
INT WINAPI DnsHostNameToUnicode(
    LPCTSTR lpszHostName,
    LPTSTR lpszUnicodeName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | Pointer to the host name as a null-terminated string. This parameter cannot be a NULL pointer or a zero length string. |
| `lpszUnicodeName` | `LPTSTR` | Pointer to the string buffer that will contain the original Unicode version of the host name, including the terminating null character. It is recommended that this buffer be at least 256 characters in size. This parameter cannot be a NULL pointer. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied to the lpszUnicodeName string buffer. This parameter cannot be zero, and must include the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the string buffer. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsHostNameToUnicode function will convert the encoded ASCII version of a host name to its Unicode version. Although any valid host name is accepted by this function, it is intended to convert a Punycode encoded host name to its original Unicode character encoding.
- If the Unicode version of this function is used, the value returned in lpszUnicodeName will be a Unicode string using UTF-16 encoding. If the ANSI version of this function, the value returned will be a Unicode string using UTF-8 encoding. To display a UTF-8 encoded host name, your application will need to convert it to UTF-16 using the MultiByteToWideChar function.
- Although this function performs checks to ensure that the lpszHostName parameter is in the correct format and does not contain any illegal characters or malformed encoding, it does not validate the existence of the domain name. To check if the host name exists and has a valid IP address, use the DnsGetHostAddress function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsNormalizeHostName](#dnsnormalizehostname-function)

---

## DnsInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI DnsInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call DnsGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that DnsInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to DnsUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the DnsInitialize or DnsUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsCloseHandle](#dnsclosehandle-function)
- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsUninitialize](#dnsuninitialize-function)

---

## DnsMatchHostName Function

Match a host name against of list of addresses including wildcards

### Prototype

```c
BOOL WINAPI DnsMatchHostName(
    HCLIENT hClient,
    LPCTSTR lpszHostName,
    LPCTSTR lpszHostMask,
    BOOL bResolve
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszHostName` | `LPCTSTR` | A pointer to a string which specifies the host name or IP address to match. |
| `lpszHostMask` | `LPCTSTR` | A pointer to a string which specifies one or more values to match against the host name. The asterisk character can be used to match any number of characters in the host name, and the question mark can be used to match any single character. Multiple values may be specified by separating them with a semicolon. |
| `bResolve` | `BOOL` | A boolean value which specifies if the host name or IP address should be resolved when matching the host against the mask string. If this parameter is non-zero, two checks against the host mask string will be performed; once for the host name specified and once for its IP address. If this parameter is zero, then the match is made only against the host name string provided. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsMatchHostName function provides a convenient way for an application to determine if a given host name matches one or more mask strings which may contain wildcard characters. For example, the host name could be "www.microsoft.com" and the host mask string could be "*.microsoft.com". In this example, the function would return a non-zero value indicating the host name matched the mask. However, if the mask string was "*.net" then the function would return zero, indicating that there was no match. Multiple mask values can be combined by separating them with a semicolon; for example, the mask "*.com;*.org" would match any host name in either the .com or .org top-level domains.
- If an internationalized domain name (IDN) is specified, it will be converted internally to an ASCII string using Punycode encoding. The host mask will be matched against this encoded version of the host name, not its Unicode version.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetAddress](#dnsgetaddress-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsGetLocalAddress](#dnsgetlocaladdress-function)

---

## DnsNormalizeHostName Function

Return the canonical form of a host name

### Prototype

```c
INT WINAPI DnsNormalizeHostName(
    LPCTSTR lpszHostName,
    LPTSTR lpszNormalized,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | Pointer to the host name as a null-terminated string. This parameter cannot be a NULL pointer or a zero length string. |
| `lpszNormalized` | `LPTSTR` | Pointer to the string buffer that will contain the canonical form of the host name, including the terminating null character. It is recommended that this buffer be at least 256 characters in size. This parameter cannot be a NULL pointer. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied to the lpszNormalized string buffer. This parameter cannot be zero, and must include the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the string buffer. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsNormalizeHostName function will remove all leading and trailing whitespace characters from the host name and fold all upper-case characters to lower-case. If an internationalized domain name (IDN) containing Unicode characters is passed to this function, it will be converted to an ASCII compatible format for domain names.
- If the Unicode version of this function is used, the host name will be converted from UTF-16 to UTF-8 and then processed. If you are unsure if an internationalized domain name will be specified as the host name, it is recommended that you use the Unicode version.
- Although this function performs checks to ensure that the lpszHostName parameter is in the correct format and does not contain any illegal characters or malformed encoding, it does not validate the existence of the domain name. To check if the host name exists and has a valid IP address, use the DnsGetHostAddress function.
- It is recommended that you use this function if your application needs to store the host name, and if accepts a host name as user input. It is not necessary to call this function prior to calling the other DNS API functions that accept a host name as a parameter. Those functions already normalize the host name and perform checks to ensure it is in the correct format.
- If the lpszHostName parameter specifies a valid IPv4 or IPv6 address string instead of a host name, this function will return a copy of that IP address in the buffer provided by the caller. This allows the function to be used in cases where a user may input either a host name or IP address. To determine if the IP address has a corresponding host name, use the DnsGetHostName method.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostName](#dnsgethostname-function)
- [DnsHostNameToUnicode](#dnshostnametounicode-function)

---

## DnsRegisterServer Function

Add a nameserver address to the current session

### Prototype

```c
INT WINAPI DnsRegisterServer(
    HCLIENT hClient,
    INT nServer,
    LPCTSTR lpszHostAddress,
    INT nPort
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nServer` | `INT` | The index into the client nameserver table. This index, starting at 0, is used to specify which slot in the client's nameserver table will be used to store the nameserver information. |
| `lpszHostAddress` | `LPCTSTR` | A pointer to a string that specifies the IP address of the nameserver to be registered. Note that hostnames cannot be specified. |
| `nPort` | `INT` | The port number that the specified nameserver is accepting queries on. This value may be set to zero, in which case it will use the default port value. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsGetServerAddress](#dnsgetserveraddress-function)
- [DnsGetServerPort](#dnsgetserverport-function)
- [DnsUnregisterServer](#dnsunregisterserver-function)

---

## DnsReset Function

Reset the current client state

### Prototype

```c
INT WINAPI DnsReset(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsSetLocalDomain](#dnssetlocaldomain-function)
- [DnsSetRetryCount](#dnssetretrycount-function)
- [DnsSetTimeout](#dnssettimeout-function)
- [DnsUnregisterServer](#dnsunregisterserver-function)

---

## DnsSetHostFile Function

Specify the name of an alternate file to use when resolving hostnames and IP addresses

### Prototype

```c
INT WINAPI DnsSetHostFile(
    HCLIENT hClient,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string that specifies the name of the file. If the parameter is NULL, then the current host file is cleared from the cache and only the default host file will be used to resolve hostnames and addresses. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of entries in the host file. A return value of DNS_ERROR indicates failure. To get extended error information, call DnsGetLastError.

### Remarks

- This function loads the file into memory allocated for the current thread. If the contents of the file have changed after the function has been called, those changes will not be reflected when resolving hostnames or addresses. To reload the host file from disk, call this function again with the same file name. To remove the alternate host file from memory, specify a NULL pointer as the parameter.
- If a host file has been specified, it is processed before the default host file when resolving a hostname into an IP address, or an IP address into a hostname. If the host name or address is not found, or no host file has been specified, a nameserver lookup is performed.
- To determine if an alternate host file has been specified, use the DnsGetHostFile function. A return value of zero indicates that no alternate host file has been cached for the current thread.
- A system may have a default host file, which is used to resolve hostnames before performing a nameserver lookup. To determine the name of this file, use the DnsGetDefaultHostFile function. It is not necessary to specify this default host file, since it is always used to resolve host names and addresses.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetDefaultHostFile](#dnsgetdefaulthostfile-function)
- [DnsGetHostAddress](#dnsgethostaddress-function)
- [DnsGetHostFile](#dnsgethostfile-function)
- [DnsGetHostName](#dnsgethostname-function)

---

## DnsSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI DnsSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the error code for the current thread. A value of zero clears the last error code. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or DNS_ERROR. Those functions which call DnsSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the DnsGetLastError function. The use of DnsGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetErrorString](#dnsgeterrorstring-function)
- [DnsGetLastError](#dnsgetlasterror-function)

---

## DnsSetLocalDomain Function

Set the local domain name for the current session

### Prototype

```c
INT WINAPI DnsSetLocalDomain(
    HCLIENT hClient,
    LPCTSTR lpszDomain
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszDomain` | `LPCTSTR` | Pointer to the string which contains the local domain name. This is used as a default value when a query does not explicitly specify a domain name. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DnsGetLocalDomain](#dnsgetlocaldomain-function)
- [DnsGetRetryCount](#dnsgetretrycount-function)
- [DnsGetTimeout](#dnsgettimeout-function)
- [DnsReset](#dnsreset-function)
- [DnsSetRetryCount](#dnssetretrycount-function)
- [DnsSetTimeout](#dnssettimeout-function)

---

## DnsSetResolverOptions Function

Set the resolver options for the specified client session

### Prototype

```c
DWORD WINAPI DnsSetResolverOptions(
    HCLIENT hClient,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more resolver options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `DNS_OPTION_NONE`: No additional resolver options specified. This is the default value, and it is recommended that most applications do not specify additional options unless the implications of doing so are understood.
    - `DNS_OPTION_PROMOTE`: Promotes the server that successfully completed the last query to the first server that will be used to resolve subsequent queries. This option can improve performance in some cases where one or more of the registered servers are non-responsive. This option takes precedence over the DNS_OPTION_ROTATE option.
    - `DNS_OPTION_ROTATE`: Enables a round-robin selection of nameservers when performing queries. Normally each nameserver is queried in the same order. This option rotates the available nameservers so a different server is used with each query.
    - `DNS_OPTION_AUTHONLY`: Require the answer from the nameserver to be authoritative, not from the server's cache. This option is included for future expansion as most servers do not support this feature and will ignore it.
    - `DNS_OPTION_PRIMARY`: Queries are only accepted from the primary nameserver. This option is included for future expansion as most servers do not support this feature and will ignore it.
    - `DNS_OPTION_NORECURSE`: Disable the sending of recursive queries to the nameserver. Specifying this option will disable the bit in the DNS request header that specifies recursion is desired.
    - `DNS_OPTION_NOSEARCH`: Disable additional queries of higher domains in the search list if the host name cannot be resolved. If this option is specified, and the host name cannot be resolved using the local domain name an error is returned immediately. This option is ignored if no local domain has been specified or if the DNS_OPTION_NOSUFFIX option has been specified.
    - `DNS_OPTION_NOSUFFIX`: Disable additional queries using the local domain name if the host name is not a fully qualified domain name and cannot be resolved. This option is ignored if no local domain has been specified.
    - `DNS_OPTION_NONAMECHECK`: Disable checking the host name for invalid characters, such as the underscore and control characters. By default, host names are checked to ensure they're valid before submitting a query to the nameserver.

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call DnsGetLastError.

### Remarks

- The DnsSetResolverOptions function changes the resolver options for the specified client session, modifying how nameserver queries are processed. It is recommended that most applications do not specify any resolver options and use the default behavior. Specifying these options without understanding how they can affect standard queries can result in unexpected failures. In particular, caution should be used when specifying the DNS_OPTION_NORECURSE and DNS_OPTION_NOSEARCH options as they change the normal process of resolving a host name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsCreateHandle](#dnscreatehandle-function)
- [DnsGetResolverOptions](#dnsgetresolveroptions-function)

---

## DnsSetRetryCount Function

Set the number of times the query is sent to each server

### Prototype

```c
INT WINAPI DnsSetRetryCount(
    HCLIENT hClient,
    INT nRetries
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nRetries` | `INT` | The number of attempts the client will make, per nameserver, to resolve a query. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The retry count determines the amount of time the client will wait for a response from each query, the effective amount of time the client will wait increases with each nameserver and the total number of retries specified. For example, two nameservers registered with the client, with a default of 4 retries per nameserver and a timeout value of 10 seconds, would cause the client to wait a total of 80 seconds until it returns an error indicating that it was unable to resolve the query.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetRetryCount](#dnsgetretrycount-function)
- [DnsGetTimeout](#dnsgettimeout-function)
- [DnsSetTimeout](#dnssettimeout-function)

---

## DnsSetTimeout Function

Set the number of seconds until a query times out

### Prototype

```c
INT WINAPI DnsSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds until the client times out waiting for a response from a nameserver. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Remarks

- The timeout value determines the amount of time the client will wait for a response from each query, the effective amount of time the client will wait increases with each nameserver and the total number of retries specified. For example, with two nameservers registered with the client, with a default of 4 retries per nameserver and a timeout value of 10 seconds, would cause the client to wait a total of 80 seconds until it returns an error indicating that it was unable to resolve the query.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetRetryCount](#dnsgetretrycount-function)
- [DnsGetTimeout](#dnsgettimeout-function)
- [DnsSetRetryCount](#dnssetretrycount-function)

---

## DnsUninitialize Function

Terminates the use of the library

### Prototype

```c
VOID WINAPI DnsUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful DnsInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call DnsUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to DnsUninitialize for every successful call to DnsInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsInitialize](#dnsinitialize-function)
- [DnsCloseHandle](#dnsclosehandle-function)
- [DnsCreateHandle](#dnscreatehandle-function)

---

## DnsUnregisterServer Function

Remove a nameserver address from the current session

### Prototype

```c
INT WINAPI DnsUnregisterServer(
    HCLIENT hClient,
    INT nServer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nServer` | `INT` | The index into the client's nameserver table. This index is the same value that is passed to the DnsRegisterServer function when the nameserver is registered. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is DNS_ERROR. To get extended error information, call DnsGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csdnsv11.lib`

### See Also

- [DnsGetServerAddress](#dnsgetserveraddress-function)
- [DnsGetServerPort](#dnsgetserverport-function)
- [DnsRegisterServer](#dnsregisterserver-function)
- [DnsReset](#dnsreset-function)

---

## HOSTENT Structure

The HOSTENT structure contains information about a domain name, including its IP address and any aliases which may be defined.

### Definition

```c
typedef struct _HOSTENT
{
    short h_addrtype;
    short h_length;
} HOSTENT, *LPHOSTENT;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `h_addrtype` | `short` | The type of address being returned. |
| `h_length` | `short` | The length, in bytes, of each address. |

### Remarks

- This structure is intended to be used for backwards compatibility with legacy applications and should not be used with new applications. You cannot use this structure to obtain information for a host which is assigned an IPv6 address. Applications should use the DnsGetHostAddress and DnsGetHostName functions which support both IPv4 and IPv6 addresses.
- The application must never attempt to modify this structure or free any of its members. There is no Unicode version of this structure, string pointers will always specify ANSI strings, even if the application is compiled to use Unicode. Only one copy of this structure is allocated per thread, so the application should copy any information it needs before calling any other functions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `winsock2.h.`

### See Also

- [DnsGetHostAddress](#dnsgethostaddress-structure)
- [DnsGetHostByAddress](#dnsgethostbyaddress-structure)
- [DnsGetHostByName](#dnsgethostbyname-structure)
- [DnsGetHostName](#dnsgethostname-structure)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## INTERNET_ADDRESS Structure

This structure represents a numeric IPv4 or IPv6 address in network byte order.

### Definition

```c
typedef struct _INTERNET_ADDRESS
{
    INT ipFamily;
    BYTE ipNumber;
} INTERNET_ADDRESS, *LPINTERNET_ADDRESS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `ipFamily` | `INT` | An integer which identifies the type of IP address. It will be one of the following values: |
| `ipNumber` | `BYTE` | A byte array which contains the numeric form of the IP address. This array is large enough to store both IPv4 (32 bit) and IPv6 (128 bit) addresses. The values are stored in network byte order. |

### Remarks

- The INTERNET_ADDRESS structure is used by some functions to represent an Internet address in a binary format that is compatible with both IPv4 and IPv6 addresses. Applications that use this structure should consider it to be opaque, and should not modify the contents of the structure directly.
- For compatibility with legacy applications that expect an IP address to be 32 bits and stored in an unsigned integer, you can copy the first four bytes of the ipNumber array using the CopyMemory function or equivalent. Note that if this is done, your application should always check the ipFamily member first to make sure that it is actually an IPv4 address.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

