# Internet Control Message Protocol

## Functions

- [IcmpCancel](#icmpcancel-function)
- [IcmpCloseHandle](#icmpclosehandle-function)
- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpDisableEvents](#icmpdisableevents-function)
- [IcmpDisableTrace](#icmpdisabletrace-function)
- [IcmpEcho](#icmpecho-function)
- [IcmpEnableEvents](#icmpenableevents-function)
- [IcmpEnableTrace](#icmpenabletrace-function)
- [IcmpEventProc](#icmpeventproc-function)
- [IcmpFormatAddress](#icmpformataddress-function)
- [IcmpFreezeEvents](#icmpfreezeevents-function)
- [IcmpGetErrorString](#icmpgeterrorstring-function)
- [IcmpGetHostAddress](#icmpgethostaddress-function)
- [IcmpGetHostName](#icmpgethostname-function)
- [IcmpGetLastError](#icmpgetlasterror-function)
- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeout](#icmpgettimeout-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpInitialize](#icmpinitialize-function)
- [IcmpIsBlocking](#icmpisblocking-function)
- [IcmpRecvEcho](#icmprecvecho-function)
- [IcmpRegisterEvent](#icmpregisterevent-function)
- [IcmpReset](#icmpreset-function)
- [IcmpResolveAddress](#icmpresolveaddress-function)
- [IcmpSendEcho](#icmpsendecho-function)
- [IcmpSetHostAddress](#icmpsethostaddress-function)
- [IcmpSetHostName](#icmpsethostname-function)
- [IcmpSetLastError](#icmpsetlasterror-function)
- [IcmpSetPacketSize](#icmpsetpacketsize-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeout](#icmpsettimeout-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)
- [IcmpTraceRoute](#icmptraceroute-function)
- [IcmpUninitialize](#icmpuninitialize-function)
## Structures

- [ICMPTIME](#icmptime-structure)
- [ICMPTRACE](#icmptrace-structure)
- [INITDATA](#initdata-structure)
- [INTERNET_ADDRESS](#internet_address-structure)

---

## IcmpCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI IcmpCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- When the IcmpCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpIsBlocking](#icmpisblocking-function)

---

## IcmpCloseHandle Function

Close the current client session

### Prototype

```c
INT WINAPI IcmpCloseHandle(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpInitialize](#icmpinitialize-function)
- [IcmpUninitialize](#icmpuninitialize-function)

---

## IcmpCreateHandle Function

Create a new handle for the current client session

### Prototype

```c
HCLIENT WINAPI IcmpCreateHandle(
    UINT nPacketSize,
    UINT nTimeToLive,
    DWORD dwTimeout,
    DWORD dwReserved,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nPacketSize` | `UINT` | An unsigned integer which specifies the default packet size used when generating ICMP echo datagrams. The minimum packet size is 32 bytes and the maximum size is 65,535 bytes. |
| `nTimeToLive` | `UINT` | An unsigned integer which specifies the default time-to-live for ICMP echo datagrams. This determines the maximum number of times that a packet will be routed from one system to another while en route to its destination. The minimum time-to-live value is 1, the maximum is 255. The recommend value for this parameter is 255, and typical applications should use a time-to-live value of at least 30. |
| `dwTimeout` | `DWORD` | An unsigned integer which specifies the maximum number of milliseconds to wait before the current operation times out. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |
| `hEventWnd` | `HWND` | The handle to an asynchronous notification window. This window receives messages which notify the client when asynchronous network events occur. If asynchronous event notification is not required, this parameter may be NULL. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous network event occurs. This value should be greater than WM_USER as defined in the Windows header files. If the hEventWnd parameter is NULL, this parameter should be specified as WM_NULL. |

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to the client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call IcmpGetLastError.

### Remarks

- The IcmpCreateHandle function creates a client handle that is used with subsequent calls to the library. This library uses a special type of socket called a raw socket, which is created to send and receive ICMP echo datagrams. Raw socket support is optional under the Windows Sockets specification, and may not be available if a non-standard networking libraries are used or may only be available to privileged accounts.
- If the hEventWnd parameter is not NULL, the client operates in asynchronous mode and messages will be posted to the notification window when a network event occurs. When a message is posted to the window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpCloseHandle](#icmpclosehandle-function)
- [IcmpInitialize](#icmpinitialize-function)
- [IcmpUninitialize](#icmpuninitialize-function)

---

## IcmpDisableEvents Function

Disable asynchronous event notification

### Prototype

```c
INT WINAPI IcmpDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- This function affects both event notification and event callbacks. Any outstanding events in the message queue should be ignored by the client application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpEnableEvents](#icmpenableevents-function)
- [IcmpFreezeEvents](#icmpfreezeevents-function)
- [IcmpRegisterEvent](#icmpregisterevent-function)

---

## IcmpDisableTrace Function

Disable logging of socket-level calls to the trace log

### Prototype

```c
BOOL WINAPI IcmpDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpEnableTrace](#icmpenabletrace-function)

---

## IcmpEcho Function

Send one or more echo datagrams to the specified host

### Prototype

```c
INT WINAPI IcmpEcho(
    LPCTSTR lpszHostName,
    UINT nIterations,
    UINT nPacketSize,
    DWORD dwTimeout,
    DWORD dwReserved,
    LPICMPTIME lpTime
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | A pointer to a string which specifies the fully qualified domain name of the remote host, or the IP address in dotted notation. |
| `nIterations` | `UINT` | An unsigned integer value which specifies the number of echo datagrams that will be sent to the remote host. The minimum value for this parameter is 1 and the maximum value is 512. |
| `nPacketSize` | `UINT` | An unsigned integer value which specifies the size of the echo datagram in bytes. The minimum size is 1 byte and the maximum size is 65,535 bytes. It is recommended that most applications use the minimum size of 32 bytes for this parameter. |
| `dwTimeout` | `DWORD` | An unsigned integer which specifies the number of milliseconds the function will wait for a response to an echo datagram. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |
| `lpTime` | `LPICMPTIME` | A pointer to an ICMPTIME structure which will contain the minimum, maximum and average round trip times for the echo datagrams sent and received. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of replies received from the remote host. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The IcmpEcho function sends a series of ICMP echo datagrams to the specified host, providing a simplified interface for pinging a remote system. If the function returns the same value as the number of iterations, then replies were received for all of the echo datagrams that were sent. This would typically indicate that the client can establish a reliable connection to the server; the values returned in the ICMPTIME structure provide information on the latency between the two hosts. Higher average time values would indicate greater latency and reduced throughput between the systems. If the function returns a value less than the specified number of iterations, this indicates that replies were not received for one or more of the echo datagrams. This may indicate that there are problems routing data between the local and remote host. A return value of zero indicates that there was no response to the echo datagrams. The remote host may not exist or may not be available.
- The failure for a host to respond to an ICMP echo datagram may not indicate a problem with the remote system. In some cases, a router between the local and remote host may be malfunctioning or discarding the datagrams. Systems can also be configured to specifically ignore ICMP echo datagrams and not respond to them; this is often a security measure to prevent certain kinds of Denial of Service attacks.
- The ability to send ICMP datagrams may be restricted to users with administrative privileges, depending on the policies and configuration of the local system. If you are unable to send or receive any ICMP datagrams, it is recommended that you check the firewall settings and any third-party security software that could impact the normal operation of this component.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpRecvEcho](#icmprecvecho-function)
- [IcmpSendEcho](#icmpsendecho-function)
- [IcmpTraceRoute](#icmptraceroute-function)

---

## IcmpEnableEvents Function

Enable asynchronous event notification

### Prototype

```c
INT WINAPI IcmpEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the event notification window. This window receives a user-defined message which specifies the event that has occurred. If this value is NULL, event notification is disabled. |
| `uEventMsg` | `UINT` | An unsigned integer which specifies the user-defined message that is sent when an event occurs. This parameter's value must be greater than the value of WM_USER. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpDisableEvents](#icmpdisableevents-function)
- [IcmpFreezeEvents](#icmpfreezeevents-function)
- [IcmpRegisterEvent](#icmpregisterevent-function)

---

## IcmpEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI IcmpEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the remote host.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpDisableTrace](#icmpdisabletrace-function)

---

## IcmpEventProc Function

Callback function that processes events generated by the client

### Prototype

```c
VOID __cdecl IcmpEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | The handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the IcmpRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the IcmpRegisterEvent function. The IcmpEventProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpDisableEvents](#icmpdisableevents-function)
- [IcmpEnableEvents](#icmpenableevents-function)
- [IcmpFreezeEvents](#icmpfreezeevents-function)
- [IcmpRegisterEvent](#icmpregisterevent-function)

---

## IcmpFormatAddress Function

Convert a numeric IP address to a string

### Prototype

```c
INT WINAPI IcmpFormatAddress(
    DWORD dwAddress,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwAddress` | `DWORD` | A unsigned 32-bit integer which specifies the IP address to be converted into a string. |
| `lpszAddress` | `LPTSTR` | A pointer to a null-terminated array of characters which will contain the converted IP address in dot-notation. This string should be at least 16 characters in length. |
| `nMaxLength` | `INT` | The maximum number of characters which may be copied in to the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the string buffer. If the function fails, the return value is zero. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpGetHostAddress](#icmpgethostaddress-function)
- [IcmpGetHostName](#icmpgethostname-function)
- [IcmpResolveAddress](#icmpresolveaddress-function)

---

## IcmpFreezeEvents Function

Suspend or resume event handling by the calling process

### Prototype

```c
INT WINAPI IcmpFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four read events occur, once event handling is resumed only one of those read events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpDisableEvents](#icmpdisableevents-function)
- [IcmpEnableEvents](#icmpenableevents-function)
- [IcmpRegisterEvent](#icmpregisterevent-function)

---

## IcmpGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI IcmpGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is 0, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpGetLastError](#icmpgetlasterror-function)
- [IcmpSetLastError](#icmpsetlasterror-function)

---

## IcmpGetHostAddress Function

Return the current host IP address

### Prototype

```c
INT WINAPI IcmpGetHostAddress(
    HCLIENT hClient,
    LPTSTR lpszHostAddress,
    INT cbHostAddress
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `lpszHostAddress` | `LPTSTR` | A pointer to the buffer that will contain the IP address of the current remote host in dot-notation. This buffer should be at least 16 characters in length. |
| `cbHostAddress` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the address string. If the return value is zero, this indicates that no remote host has been specified. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpFormatAddress](#icmpformataddress-function)
- [IcmpGetHostName](#icmpgethostname-function)
- [IcmpResolveAddress](#icmpresolveaddress-function)
- [IcmpSetHostAddress](#icmpsethostaddress-function)
- [IcmpSetHostName](#icmpsethostname-function)

---

## IcmpGetHostName Function

Return the current host name

### Prototype

```c
INT WINAPI IcmpGetHostName(
    HCLIENT hClient,
    LPTSTR lpszHostName,
    INT cbHostName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `lpszHostName` | `LPTSTR` | A pointer to the buffer that will contain the name of the current remote host. This buffer should be at least 64 characters in length. |
| `cbHostName` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the host name string. If the return value is zero, this indicates that no remote host has been specified. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpFormatAddress](#icmpformataddress-function)
- [IcmpGetHostAddress](#icmpgethostaddress-function)
- [IcmpResolveAddress](#icmpresolveaddress-function)
- [IcmpSetHostAddress](#icmpsethostaddress-function)
- [IcmpSetHostName](#icmpsethostname-function)

---

## IcmpGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI IcmpGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the IcmpSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the IcmpGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call IcmpSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or ICMP_ERROR. Those functions which call IcmpSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetErrorString](#icmpgeterrorstring-function)
- [IcmpSetLastError](#icmpsetlasterror-function)

---

## IcmpGetPacketSize Function

Return the ICMP datagram packet size

### Prototype

```c
UINT WINAPI IcmpGetPacketSize(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the size of the datagram. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetPacketSize](#icmpsetpacketsize-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpGetRecvCount Function

Return the number of packets received

### Prototype

```c
INT WINAPI IcmpGetRecvCount(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of replies. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpGetSendCount Function

Return the number of packets sent

### Prototype

```c
INT WINAPI IcmpGetSendCount(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of datagrams sent by the client. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpGetSequenceId Function

Return the current sequence identifier

### Prototype

```c
INT WINAPI IcmpGetSequenceId(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the sequence identifier. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpGetTimeout Function

Return the number of milliseconds until an operation times out

### Prototype

```c
DWORD WINAPI IcmpGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the timeout period in milliseconds. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpIsBlocking](#icmpisblocking-function)
- [IcmpSetTimeout](#icmpsettimeout-function)

---

## IcmpGetTimeToLive Function

Return the current time-to-live for the ICMP datagram

### Prototype

```c
INT WINAPI IcmpGetTimeToLive(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the time-to-live for the last datagram. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The time-to-live (TTL) value is specified in the IP header of a datagram, and is used to control the number of routers that the datagram is passed through. Each router that handles the datagram decrements the TTL value by one. When it drops to zero, a datagram is returned to the sender, specifying that the TTL has been exceeded. The default value is 255.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpGetTripTime Function

Return the current trip time statistics

### Prototype

```c
INT WINAPI IcmpGetTripTime(
    HCLIENT hClient,
    LPICMPTIME lpIcmpTime
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `lpIcmpTime` | `LPICMPTIME` | A pointer to an ICMPTIME data structure which will contain the round-trip statistics for the current remote host. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI IcmpInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call IcmpGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that IcmpInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to IcmpUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the IcmpInitialize or IcmpUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpCloseHandle](#icmpclosehandle-function)
- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpUninitialize](#icmpuninitialize-function)

---

## IcmpIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI IcmpIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns a non-zero value. If the client is not performing a blocking operation, or the client handle is invalid, the function returns zero.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The IcmpIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action (such as warning the user that they must wait for the operation to complete).

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpCancel](#icmpcancel-function)

---

## IcmpRecvEcho Function

Read an ICMP datagram returned by the remote host

### Prototype

```c
INT WINAPI IcmpRecvEcho(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpSendEcho](#icmpsendecho-function)

---

## IcmpRegisterEvent Function

Register an event callback function

### Prototype

```c
INT WINAPI IcmpRegisterEvent(
    HCLIENT hClient,
    UINT nEventId,
    INETEVENTPROC lpfnEvent,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to client session. |
| `nEventId` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One or more of the following values may be used: |
| `lpfnEvent` | `INETEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the IcmpEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEventId** Parameter

    - `ICMP_EVENT_ECHO`: The client has generated an ICMP echo datagram and has sent it to the specified host. If the datagram is received, the remote host should generate a reply and return it to the sender.
    - `ICMP_EVENT_REPLY`: The client has received an ICMP echo reply datagram from the remote host. At this point the client can collect statistical information.
    - `ICMP_EVENT_TIMEOUT`: The network operation has exceeded the specified timeout period. The client application may attempt to retry the operation.
    - `ICMP_EVENT_CANCEL`: The current operation has been canceled. The client application may attempt to retry the operation or close the handle.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The IcmpRegisterEvent function associates a callback function with a specific event. The event handler is an IcmpEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpDisableEvents](#icmpdisableevents-function)
- [IcmpEnableEvents](#icmpenableevents-function)
- [IcmpEventProc](#icmpeventproc-function)
- [IcmpFreezeEvents](#icmpfreezeevents-function)

---

## IcmpReset Function

Reset the current client state

### Prototype

```c
INT WINAPI IcmpReset(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpSetHostAddress](#icmpsethostaddress-function)
- [IcmpSetHostName](#icmpsethostname-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpResolveAddress Function

Resolve an IP address into a fully qualified host name

### Prototype

```c
INT WINAPI IcmpResolveAddress(
    DWORD dwAddress,
    LPTSTR lpszHostName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwAddress` | `DWORD` | The IP address to be resolved, specified as an unsigned 32-bit integer in network byte order. |
| `lpszHostName` | `LPTSTR` | A pointer to a buffer that will contain a null-terminated string that specifies the fully qualified domain name for the host. It is recommended that this buffer be at least 64 characters in length. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied into the string buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the host name. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The IcmpResolveAddress function is used to perform a reverse DNS lookup, converting a numeric IP address into a fully qualified domain name. This is useful for functions like IcmpTraceRoute, which return host addresses as numeric values in network byte order. If the reverse DNS lookup fails because there is no PTR record for the given IP address, a printable form of the address in dotted notation will be returned in the string buffer.
- Calling this function will cause the thread to block until the IP address is resolved, or until the DNS query times out because there is no reverse record for the address. In some cases, a reverse lookup can take several seconds.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpGetHostAddress](#icmpgethostaddress-function)
- [IcmpGetHostName](#icmpgethostname-function)
- [IcmpSetHostAddress](#icmpsethostaddress-function)
- [IcmpSetHostName](#icmpsethostname-function)
- [IcmpTraceRoute](#icmptraceroute-function)

---

## IcmpSendEcho Function

Send an ICMP datagram to the specified host

### Prototype

```c
INT WINAPI IcmpSendEcho(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- For asynchronous client sessions, this function returns immediately. Otherwise, the client enters a blocking state and waits for a reply from the remote host. The function returns when a reply has been received, or the operation times-out.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpEcho](#icmpecho-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpRecvEcho](#icmprecvecho-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)
- [IcmpTraceRoute](#icmptraceroute-function)

---

## IcmpSetHostAddress Function

Set the IP address of the host to receive the next datagram

### Prototype

```c
INT WINAPI IcmpSetHostAddress(
    HCLIENT hClient,
    DWORD dwAddress
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `dwAddress` | `DWORD` | The IP address of the remote host as a 32-bit integer value, specified in network byte order. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- To specify a remote host name or an IP address as a string in dot notation, use the IcmpSetHostName function instead.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetHostAddress](#icmpgethostaddress-function)
- [IcmpGetHostName](#icmpgethostname-function)
- [IcmpSetHostName](#icmpsethostname-function)

---

## IcmpSetHostName Function

Set the name of the host to receive the next datagram

### Prototype

```c
INT WINAPI IcmpSetHostName(
    HCLIENT hClient,
    LPCTSTR lpszHostName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `lpszHostName` | `LPCTSTR` | A pointer to a string which specifies the name, or IP address in dot-notation, of the remote host. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpGetHostAddress](#icmpgethostaddress-function)
- [IcmpGetHostName](#icmpgethostname-function)
- [IcmpSetHostAddress](#icmpsethostaddress-function)

---

## IcmpSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI IcmpSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or ICMP_ERROR. Those functions which call IcmpSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the IcmpGetLastError function. The use of IcmpGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetErrorString](#icmpgeterrorstring-function)
- [IcmpGetLastError](#icmpgetlasterror-function)

---

## IcmpSetPacketSize Function

Set the ICMP datagram packet size

### Prototype

```c
UINT WINAPI IcmpSetPacketSize(
    HCLIENT hClient,
    UINT nPacketSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nPacketSize` | `UINT` | Size of the ICMP datagram packet in bytes. The minimum packet size is 1 byte and the maximum packet size is 65,535 bytes. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the previous ICMP datagram packet size. If the function fails, the return value is zero. To get extended error information, call IcmpGetLastError.

### Remarks

- Note that packet sizes over 512 bytes may not be supported by your networking software or configuration. It is recommended that most applications use the minimum packet size of 32 bytes.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpSetSequenceId Function

Set the sequence identifier for the next datagram

### Prototype

```c
INT WINAPI IcmpSetSequenceId(
    HCLIENT hClient,
    INT nSequenceId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `nSequenceId` | `INT` | The sequence identifier for the next datagram sent by the client. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the previous sequence identifier. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetPacketSize](#icmpsetpacketsize-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpSetTimeout Function

Set the number of milliseconds until an operation times out

### Prototype

```c
DWORD WINAPI IcmpSetTimeout(
    HCLIENT hClient,
    DWORD dwTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwTimeout` | `DWORD` | The number of milliseconds until a blocking operation fails. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the value of the timeout before the function was called. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The timeout value is only used with blocking client connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpGetTimeout](#icmpgettimeout-function)

---

## IcmpSetTimeToLive Function

Set the time-to-live for the next datagram

### Prototype

```c
INT WINAPI IcmpSetTimeToLive(
    HCLIENT hClient,
    INT nTimeToLive
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |
| `nTimeToLive` | `INT` | The time-to-live value for the next ICMP echo datagram. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the previous time-to-live value. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The time-to-live (TTL) value is specified in the IP header of a datagram, and is used to control the number of routers that the datagram is passed through. Each router that handles the datagram decrements the TTL value by one. When it drops to zero, a datagram is returned to the sender, specifying that the TTL has been exceeded.
- Calling this function changes the default TTL value for all subsequent ICMP datagrams sent by the library, with the default value being 255. Note that not all Windows Sockets implementations support setting the time-to-live value.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpGetPacketSize](#icmpgetpacketsize-function)
- [IcmpGetRecvCount](#icmpgetrecvcount-function)
- [IcmpGetSendCount](#icmpgetsendcount-function)
- [IcmpGetSequenceId](#icmpgetsequenceid-function)
- [IcmpGetTimeToLive](#icmpgettimetolive-function)
- [IcmpGetTripTime](#icmpgettriptime-function)
- [IcmpSetPacketSize](#icmpsetpacketsize-function)
- [IcmpSetSequenceId](#icmpsetsequenceid-function)

---

## IcmpTraceRoute Function

Trace the route from the local system to a remote host

### Prototype

```c
INT WINAPI IcmpTraceRoute(
    LPCTSTR lpszHostName,
    UINT nMaxHops,
    DWORD dwTimeout,
    DWORD dwReserved,
    LPICMPTRACE lpTrace,
    LPDWORD lpdwSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | A pointer to a string which specifies the fully qualified domain name of the remote host, or the IP address in dotted notation. |
| `nMaxHops` | `UINT` | An unsigned integer which specifies the maximum number of routers the datagram will be forwarded through (the number of hops) to the remote host. The minimum value is 1 and the maximum value is 255. It is recommended that most applications specify a value of at least 30. |
| `dwTimeout` | `DWORD` | An unsigned integer which specifies the number of milliseconds the function will wait for a response to an echo datagram. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |
| `lpTrace` | `LPICMPTRACE` | A pointer to an array of ICMPTRACE structures which will contain information about each intermediate host between the local and remote system. The number of structures must be at least the same as the maximum number of hops specified by the nMaxHops parameter. To determine the total number of bytes that must be allocated for the structure array, this parameter can be passed as NULL and the total size will be returned in the lpdwSize parameter. |
| `lpdwSize` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the size of the ICMPTRACE array passed to the function. If the lpTrace parameter is NULL, the function will calculate the size of the array buffer that must be allocated and return that value. If both the lpTrace and lpdwSize parameters are NULL, no statistical information about the intermediate hosts will be collected. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of intermediate hosts between the local and remote system. If the function fails, the return value is ICMP_ERROR. To get extended error information, call IcmpGetLastError.

### Remarks

- The IcmpTraceRoute function sends a series of ICMP echo datagrams to the specified host, adjusting the time-to-live value to determine the intermediate hosts that route the packet. The function returns the number of hops to the remote host.
- It is important to note that the failure of an intermediate host to respond to an ICMP echo datagram may not indicate a problem with the remote system. Systems can be configured to specifically ignore ICMP echo datagrams and not respond to them; this is often a security measure to prevent certain kinds of Denial of Service attacks.
- The ability to send ICMP datagrams may be restricted to users with administrative privileges, depending on the policies and configuration of the local system. If you are unable to send or receive any ICMP datagrams, it is recommended that you check the firewall settings and any third-party security software that could impact the normal operation of this component.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpEcho](#icmpecho-function)
- [IcmpRecvEcho](#icmprecvecho-function)
- [IcmpResolveAddress](#icmpresolveaddress-function)
- [IcmpSendEcho](#icmpsendecho-function)
- [IcmpSetTimeToLive](#icmpsettimetolive-function)

---

## IcmpUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI IcmpUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful IcmpInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call IcmpUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to IcmpUninitialize for every successful call to IcmpInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csicmv11.lib`

### See Also

- [IcmpCloseHandle](#icmpclosehandle-function)
- [IcmpCreateHandle](#icmpcreatehandle-function)
- [IcmpInitialize](#icmpinitialize-function)

---

## ICMPTIME Structure

This structure is used by the IcmpGetTripTime function to return the round-trip times of an ICMP datagram.

### Definition

```c
typedef struct _ICMPTIME
{
    DWORD dwTripAverage;
    DWORD dwTripMaximum;
    DWORD dwTripMinimum;
    DWORD dwTripTime;
} ICMPTIME, *LPICMPTIME;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwTripAverage` | `DWORD` | The average round-trip time in milliseconds for all datagrams sent to the current host. |
| `dwTripMaximum` | `DWORD` | The maximum round-trip time in milliseconds for all datagrams sent to current host. |
| `dwTripMinimum` | `DWORD` | The minimum round-trip time in milliseconds for all datagrams sent to the current host. |
| `dwTripTime` | `DWORD` | The current round-trip time in milliseconds for the last datagram sent to the current host. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## ICMPTRACE Structure

This structure is used by the IcmpTraceRoute function to return the route of an ICMP datagram.

### Definition

```c
typedef struct _ICMPTRACE
{
    UINT nDistance;
    DWORD dwHostAddress;
    DWORD dwTripAverage;
    DWORD dwTripMaximum;
    DWORD dwTripMinimum;
} ICMPTRACE, *LPICMPTRACE;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nDistance` | `UINT` | The distance from the local host to the remote host for this route. |
| `dwHostAddress` | `DWORD` | An unsigned integer which specifies the IP address of the remote host in network byte order. |
| `dwTripAverage` | `DWORD` | The average round-trip time in milliseconds for all datagrams sent to the specified host. |
| `dwTripMaximum` | `DWORD` | The maximum round-trip time in milliseconds for all datagrams sent to specified host. |
| `dwTripMinimum` | `DWORD` | The minimum round-trip time in milliseconds for all datagrams sent to the specified host. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## INTERNET_ADDRESS Structure

This structure represents a numeric IPv4 or IPv6 address in network byte order.

### Definition

```c
typedef struct _INTERNET_ADDRESS
{
    INT ipFamily;
    BYTE ipNumber;
} INTERNET_ADDRESS, *LPINTERNET_ADDRESS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `ipFamily` | `INT` | An integer which identifies the type of IP address. It will be one of the following values: |
| `ipNumber` | `BYTE` | A byte array which contains the numeric form of the IP address. This array is large enough to store both IPv4 (32 bit) and IPv6 (128 bit) addresses. The values are stored in network byte order. |

### Remarks

- The INTERNET_ADDRESS structure is used by some functions to represent an Internet address in a binary format that is compatible with both IPv4 and IPv6 addresses. Applications that use this structure should consider it to be opaque, and should not modify the contents of the structure directly.
- For compatibility with legacy applications that expect an IP address to be 32 bits and stored in an unsigned integer, you can copy the first four bytes of the ipNumber array using the CopyMemory function or equivalent. Note that if this is done, your application should always check the ipFamily member first to make sure that it is actually an IPv4 address.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

