# Internet Message Access Protocol

## Functions

- [ImapAsyncConnect](#imapasyncconnect-function)
- [ImapAttachThread](#imapattachthread-function)
- [ImapCancel](#imapcancel-function)
- [ImapCheckMailbox](#imapcheckmailbox-function)
- [ImapCloseMessage](#imapclosemessage-function)
- [ImapCommand](#imapcommand-function)
- [ImapConnect](#imapconnect-function)
- [ImapCopyMessage](#imapcopymessage-function)
- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapCreateMessage](#imapcreatemessage-function)
- [ImapCreateSecurityCredentials](#imapcreatesecuritycredentials-function)
- [ImapDecodeMailboxName](#imapdecodemailboxname-function)
- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapDeleteSecurityCredentials](#imapdeletesecuritycredentials-function)
- [ImapDisableEvents](#imapdisableevents-function)
- [ImapDisableTrace](#imapdisabletrace-function)
- [ImapDisconnect](#imapdisconnect-function)
- [ImapEnableEvents](#imapenableevents-function)
- [ImapEnableTrace](#imapenabletrace-function)
- [ImapEncodeMailboxName](#imapencodemailboxname-function)
- [ImapEnumMessages](#imapenummessages-function)
- [ImapEventProc](#imapeventproc-function)
- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapExpungeMailbox](#imapexpungemailbox-function)
- [ImapFreezeEvents](#imapfreezeevents-function)
- [ImapGetCapability](#imapgetcapability-function)
- [ImapGetCurrentMailbox](#imapgetcurrentmailbox-function)
- [ImapGetDeletedMessages](#imapgetdeletedmessages-function)
- [ImapGetErrorString](#imapgeterrorstring-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetHeaderValueEx](#imapgetheadervalueex-function)
- [ImapGetIdleThreadId](#imapgetidlethreadid-function)
- [ImapGetLastError](#imapgetlasterror-function)
- [ImapGetMailboxDelimiter](#imapgetmailboxdelimiter-function)
- [ImapGetMailboxSize](#imapgetmailboxsize-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetMailboxUid](#imapgetmailboxuid-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageCountEx](#imapgetmessagecountex-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageHeadersEx](#imapgetmessageheadersex-function)
- [ImapGetMessageId](#imapgetmessageid-function)
- [ImapGetMessageParts](#imapgetmessageparts-function)
- [ImapGetMessageSender](#imapgetmessagesender-function)
- [ImapGetMessageSize](#imapgetmessagesize-function)
- [ImapGetMessageUid](#imapgetmessageuid-function)
- [ImapGetNewMessages](#imapgetnewmessages-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapGetResultCode](#imapgetresultcode-function)
- [ImapGetResultString](#imapgetresultstring-function)
- [ImapGetSecurityInformation](#imapgetsecurityinformation-function)
- [ImapGetStatus](#imapgetstatus-function)
- [ImapGetTimeout](#imapgettimeout-function)
- [ImapGetTransferStatus](#imapgettransferstatus-function)
- [ImapGetUnseenMessages](#imapgetunseenmessages-function)
- [ImapIdle](#imapidle-function)
- [ImapIdleProc](#imapidleproc-function)
- [ImapInitialize](#imapinitialize-function)
- [ImapIsBlocking](#imapisblocking-function)
- [ImapIsConnected](#imapisconnected-function)
- [ImapIsReadable](#imapisreadable-function)
- [ImapIsWritable](#imapiswritable-function)
- [ImapLogin](#imaplogin-function)
- [ImapOpenMessage](#imapopenmessage-function)
- [ImapOpenMessageEx](#imapopenmessageex-function)
- [ImapRead](#imapread-function)
- [ImapRegisterEvent](#imapregisterevent-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapReselectMailbox](#imapreselectmailbox-function)
- [ImapSearchMailbox](#imapsearchmailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapSetLastError](#imapsetlasterror-function)
- [ImapSetMessageFlags](#imapsetmessageflags-function)
- [ImapSetTimeout](#imapsettimeout-function)
- [ImapStoreMessage](#imapstoremessage-function)
- [ImapSubscribeMailbox](#imapsubscribemailbox-function)
- [ImapUndeleteMessage](#imapundeletemessage-function)
- [ImapUninitialize](#imapuninitialize-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)
- [ImapUnsubscribeMailbox](#imapunsubscribemailbox-function)
- [ImapWrite](#imapwrite-function)
## Structures

- [IMAPMAILBOX](#imapmailbox-structure)
- [IMAPMAILBOXSTATUS](#imapmailboxstatus-structure)
- [IMAPMESSAGE](#imapmessage-structure)
- [IMAPTRANSFERSTATUS](#imaptransferstatus-structure)
- [INITDATA](#initdata-structure)
- [SECURITYCREDENTIALS](#securitycredentials-structure)
- [SECURITYINFO](#securityinfo-structure)

---

## ImapAsyncConnect Function

Connect asynchronously to the specified server

### Prototype

```c
HCLIENT WINAPI ImapAsyncConnect(
    LPCTSTR lpszHostName,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPSECURITYCREDENTIALS lpCredentials,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the host name or IP address of the IMAP server. |
| `nRemotePort` | `UINT` | The port number the client should use to establish the connection. A value of zero specifies that default port 143 should be used, which is the standard port number assigned to the IMAP service. If the secure port number is specified, an implicit TLS connection will be established by default. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation |
| `dwOptions` | `DWORD` | An unsigned integer value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which is used to establish the client credentials for a secure connection to the server. The function ImapCreateSecurityCredentials can be used to create this structure if necessary. If a standard non-secure connection is being established, or client credentials are not required by the server, this parameter can be NULL. |
| `hEventWnd` | `HWND` | The handle to the asynchronous notification window. This window receives messages which notify the client of various asynchronous client events that occur. Specifying this parameter and a message identifier causes the connection to be non-blocking. If this parameter is NULL, then a blocking connection is established. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous client event occurs. This value should be greater than WM_USER as defined in the Windows header files. If the hEventWnd parameter is NULL, this parameter should be specified as WM_NULL. |

### Options

- **dwOptions** Parameter

    - `IMAP_OPTION_NONE`: No connection options specified. A standard connection to the server will be established using the specified host name and port number.
    - `IMAP_OPTION_IDENTIFY`: This option specifies the client should identify itself to the server. If enabled, the client will send the ID command to the server as defined in RFC 2971. This option has no effect if the server does not support the ID command.
    - `IMAP_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `IMAP_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `IMAP_OPTION_SECURE`: This option specifies that a secure connection should be established with the server and requires that the server support the TLS protocol. This option is the same as specifying IMAP_OPTION_SECURE_EXPLICIT, which initiates the secure session using the STARTTLS command.
    - `IMAP_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol and the STARTTLS command.
    - `IMAP_OPTION_SECURE_IMPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol, and the secure session must be negotiated immediately after the connection has been established.
    - `IMAP_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `HTTP_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `IMAP_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call ImapGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDisableEvents](#imapdisableevents-function)
- [ImapDisconnect](#imapdisconnect-function)
- [ImapEnableEvents](#imapenableevents-function)
- [ImapInitialize](#imapinitialize-function)
- [ImapUninitialize](#imapuninitialize-function)

---

## ImapAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI ImapAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The ImapAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should ImapAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the ImapCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the ImapUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDisconnect](#imapdisconnect-function)
- [ImapUninitialize](#imapuninitialize-function)

---

## ImapCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI ImapCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- When the ImapCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapIsBlocking](#imapisblocking-function)

---

## ImapCheckMailbox Function

Check the current mailbox for new messages

### Prototype

```c
INT WINAPI ImapCheckMailbox(
    HCLIENT hClient,
    UINT lpnMessages,
    UINT lpnUnseen
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpnMessages` | `UINT` | A pointer to an unsigned integer value which will contain the number of messages in the currently selected mailbox when the function returns. |
| `lpnUnseen` | `UINT` | A pointer to an unsigned integer value which will contain the number of unseen messages in the currently selected mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- When the client requests a checkpoint, the server may perform implementation-dependent housekeeping for that mailbox, such updating the mailbox on disk with the current state of the mailbox in memory. On some systems this command has no effect other than to update the client with the current number of messages in the mailbox.
- This function actually sends two IMAP commands. The first is the CHECK command, followed by the NOOP command to poll for any new messages that have arrived. In addition to polling the server for new messages, this command can also be used to ensure the idle timer on the server does not expire and force a disconnect from the client.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)

---

## ImapCloseMessage Function

Close the current message

### Prototype

```c
INT WINAPI ImapCloseMessage(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapCloseMessage function closes the current message. If there is any remaining data left to be read from the message, it will be read and discarded.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCreateMessage](#imapcreatemessage-function)
- [ImapOpenMessage](#imapopenmessage-function)
- [ImapOpenMessageEx](#imapopenmessageex-function)

---

## ImapCommand Function

Send a command to the server

### Prototype

```c
INT WINAPI ImapCommand(
    HCLIENT hClient,
    LPCTSTR lpszCommand,
    LPCTSTR lpszParameter
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszCommand` | `LPCTSTR` | The command which will be executed by the server. |
| `lpszParameter` | `LPCTSTR` | An optional command parameter. If the command requires more than one parameter, then they should be combined into a single string, with a space separating each parameter. If the command does not accept any parameters, this value may be NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns an IMAP result code. If the command was successful, it returns IMAP_RESULT_OK. A return value of IMAP_RESULT_CONTINUE indicates the command was accepted and the caller should proceed with the next command. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- A list of valid commands can be found in the technical specification for the protocol. The current version of the protocol which is supported by this library is version 4rev1 as defined in RFC 3501.
- Use the ImapGetResultCode function to determine the result of the command that was sent to the server.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetResultCode](#imapgetresultcode-function)
- [ImapGetResultString](#imapgetresultstring-function)

---

## ImapConnect Function

Connect to the specified server

### Prototype

```c
HCLIENT WINAPI ImapConnect(
    LPCTSTR lpszHostName,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPSECURITYCREDENTIALS lpCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the host name or IP address of the IMAP server. |
| `nRemotePort` | `UINT` | The port number the client should use to establish the connection. A value of zero specifies that default port 143 should be used, which is the standard port number assigned to the IMAP service. If the secure port number is specified, an implicit TLS connection will be established by default. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which is used to establish the client credentials for a secure connection to the server. The function ImapCreateSecurityCredentials can be used to create this structure if necessary. If a standard non-secure connection is being established, or client credentials are not required by the server, this parameter can be NULL. |

### Options

- **dwOptions** Parameter

    - `IMAP_OPTION_NONE`: No connection options specified. A standard connection to the server will be established using the specified host name and port number.
    - `IMAP_OPTION_IDENTIFY`: This option specifies the client should identify itself to the server. If enabled, the client will send the ID command to the server as defined in RFC 2971. This option has no effect if the server does not support the ID command.
    - `IMAP_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `IMAP_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `IMAP_OPTION_SECURE`: This option specifies that a secure connection should be established with the server and requires that the server support the TLS protocol. This option is the same as specifying IMAP_OPTION_SECURE_EXPLICIT, which initiates the secure session using the STARTTLS command.
    - `IMAP_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. Note that the server must support secure connections using the TLS protocol.
    - `IMAP_OPTION_SECURE_IMPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol, and the secure session must be negotiated immediately after the connection has been established.
    - `IMAP_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `IMAP_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `IMAP_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call ImapGetLastError.

### Remarks

- This function will block the current thread until a connection has been established or the timeout period has elapsed. To prevent it from blocking the main user interface thread, the application should create a background worker thread and establish a connection by calling ImapConnect in that thread. If the application requires multiple simultaneous connections, it is recommended you create a worker thread for each connection.
- The dwOptions argument can be used to specify the threading model that is used by the library when a connection is established. By default, the handle is initially attached to the thread that created it. From that point on, until the it is released, only the owner may call functions using that handle. The ownership of the handle may be transferred from one thread to another using the ImapAttachThread function.
- Specifying the IMAP_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDisconnect](#imapdisconnect-function)
- [ImapInitialize](#imapinitialize-function)
- [ImapLogin](#imaplogin-function)

---

## ImapCopyMessage Function

Copy a message from the current mailbox to another mailbox

### Prototype

```c
INT WINAPI ImapCopyMessage(
    HCLIENT hClient,
    UINT nMessageId,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | The message identifier which specifies which message is to be copied to the mailbox. This value must be greater than zero and specify a valid message number. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a string which specifies the name of the mailbox that the message will be copied to. The mailbox must already exist, and the client must have the appropriate access rights to modify the mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns a value of zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- If the mailbox does not exist, the function will fail. To create a new mailbox, use the ImapCreateMailbox function. A message can be copied within the same mailbox, in which case the server may flag it as a new message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapGetResultCode](#imapgetresultcode-function)
- [ImapGetResultString](#imapgetresultstring-function)

---

## ImapCreateMailbox Function

Create a new mailbox on the server

### Prototype

```c
INT WINAPI ImapCreateMailbox(
    HCLIENT hClient,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a null terminated string which specifies the new mailbox to be created. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns a value of zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- If the mailbox name is suffixed with the server's hierarchy delimiter, this indicates to the server that the client intends to create mailbox names under the specified name in the hierarchy. If superior hierarchical names are specified in the mailbox name, then the server may automatically create them as needed. For example, if the mailbox name "Mail/Office/Projects" is specified and "Mail/Office" does not exist, it may be automatically created by the server.
- The special mailbox name INBOX is reserved, and cannot be created. It is recommended that mailbox names only consist of printable ASCII characters, and the special characters "*" and "%" should be avoided.
- If you need to create a mailbox which contains Unicode characters or symbols, it is recommended you always use the Unicode version of this function. The mailbox name will automatically be converted to modified UTF-7 encoding as defined by the IMAP4 standard. The ANSI version of this function will only accept mailbox names using either modified UTF-7 encoding or UTF-8 encoding.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDecodeMailboxName](#imapdecodemailboxname-function)
- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapEncodeMailboxName](#imapencodemailboxname-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)

---

## ImapCreateMessage Function

Create a new message in the specified mailbox

### Prototype

```c
INT WINAPI ImapCreateMessage(
    HCLIENT hClient,
    LPCTSTR lpszMailbox,
    DWORD dwReserved,
    LPVOID lpMessage,
    DWORD dwMessageSize,
    DWORD dwFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a string which specifies the name of the mailbox that the message will be created in. The mailbox must already exist, and the client must have the appropriate access rights to modify the mailbox. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |
| `lpMessage` | `LPVOID` | A pointer to the buffer which contains the message to be created. |
| `dwMessageSize` | `DWORD` | An unsigned integer value which specifies the size of the message in bytes. If this value is zero, then it is assumed that the lpMessage parameter points to a string. |
| `dwFlags` | `DWORD` | An unsigned integer that specifies one or more message flags. This parameter is constructed by using a bitwise operator with any of the following values: |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- If the mailbox does not exist, the function will fail. To create a new mailbox, use the ImapCreateMailbox function. If the message is created in the mailbox that is currently selected, the server may flag it as recent.
- This function is typically used by applications to store messages which have already been sent to a user. After a message has been delivered using the SMTP protocol, that same message may be created in a mailbox on the IMAP server so that the user has access to those messages.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCopyMessage](#imapcopymessage-function)
- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapSetMessageFlags](#imapsetmessageflags-function)

---

## ImapCreateSecurityCredentials Function

Allocate a structure to establish client security credentials

### Prototype

```c
BOOL WINAPI ImapCreateSecurityCredentials(
    DWORD dwProtocol,
    DWORD dwOptions,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    LPCTSTR lpszCertStore,
    LPCTSTR lpszCertName,
    LPVOID lpvReserved,
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. This parameter is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This member is constructed by using a bitwise operator with any of the following values: |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's username. A value of NULL specifies that no username is required. Currently this parameter is not used and any value specified will be ignored. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's password. A value of NULL specifies that no password is required. This parameter is only used if a PKCS #12 (PFX) certificate file is specified and that certificate has been secured with a password. This value will be ignored the current user or local machine certificate store is specified. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The function will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the function will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the function will return an error indicating that the certificate could not be found. |
| `lpvReserved` | `LPVOID` | Pointer reserved for future use. Set it to NULL when using this function. |
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. The memory for the credentials structure will be allocated by this function and must be released by calling the ImapDeleteSecurityCredentials function when it is no longer needed. The pointer value must be set to NULL before the function is called. It is important to note that this is a pointer to a pointer variable, not a pointer to the SECURITYCREDENTIALS structure itself. |

### Options

- **dwProtocol** Parameter

    - `SECURITY_PROTOCOL_DEFAULT`: The default security protocol will be used when establishing a connection. This option will always request the latest version of the preferred security protocol and is the recommended value. Currently the default security protocol is TLS 1.2.
    - `SECURITY_PROTOCOL_TLS`: Either the TLS 1.2 or 1.3 protocol should be used when establishing a secure connection. The correct protocol is automatically selected, based on what version of the protocol is supported by the server. If this is the only protocol specified, SSL will be excluded from the list of supported protocols. This may be necessary for some servers which reject any attempt to use the older SSL protocol and require that only TLS be used.
    - `SECURITY_PROTOCOL_TLS10`: The TLS 1.0 protocol should be used when establishing a secure connection. This version is supported on all Windows platforms, however some servers may reject connections using version 1.0 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS11`: The TLS 1.1 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. Some servers may reject connections using version 1.1 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS12`: The TLS 1.2 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. This version of TLS offers the broadest compatibility with most servers.
    - `SECURITY_PROTOCOL_TLS13`: The TLS 1.3 protocol should be used when establishing a secure connection. This is the newest version of the protocol and is only supported on Windows 11, Windows Server 2022 and later versions of Windows. If this version is not supported by the operating system, TLS 1.2 will be used instead.
- **dwOptions** Parameter

    - `CREDENTIAL_STORE_CURRENT_USER`: The library will attempt to open a store for the current user using the certificate store name provided in this structure. If no options are specified, then this is the default behavior.
    - `CREDENTIAL_STORE_LOCAL_MACHINE`: The library will attempt to open a local machine store using the certificate store name provided in this structure. If this option is specified, the library will always search for a local machine store before searching for the store by that name for the current user.
    - `CREDENTIAL_STORE_FILENAME`: The certificate store name is a file that contains the certificate and private key that will be used.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError.

### Remarks

- The structure that is created by this function may be used as client credentials when establishing a secure connection. This is particularly useful for programming languages other than C/C++ which may not support C structures or pointers. The pointer to the SECURITYCREDENTIALS structure can be declared as an unsigned integer variable which is passed by reference to this function, and then passed by value to the ImapConnect function.
- A client application only needs to call this function if the server requires that the client provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions parameter should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore parameter should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12.These certificate files typically have an extension of .pfx or .p12. If a password was specified when the certificate was created, it must be provided in the lpszPassword parameter or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword parameters are values which are used to access the certificate store or client certificate. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session. In most cases, these parameters will be NULL.

### Example

```cpp
LPSECURITYCREDENTIALS lpSecCred = NULL;
ImapCreateSecurityCredentials(SECURITY_PROTOCOL_DEFAULT,
							0,
							NULL,
							NULL,
							lpszCertStore,
							lpszCertName,
							NULL,
							&lpSecCred);

hClient = ImapConnect(lpszHostName,
					IMAP_PORT_SECURE,
					IMAP_TIMEOUT,
					IMAP_OPTION_SECURE,
					lpSecCred);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDeleteSecurityCredentials](#imapdeletesecuritycredentials-function)
- [ImapGetSecurityInformation](#imapgetsecurityinformation-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)

---

## ImapDecodeMailboxName Function

Decode a UTF-7 encoded mailbox name using the specified code page

### Prototype

```c
INT WINAPI ImapDecodeMailboxName(
    INT nCodePage,
    LPCSTR lpszMailboxName,
    LPTSTR lpszDecodedName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCodePage` | `INT` | An integer value which specifies the code page which should be used when decoding the mailbox name. If the ANSI version of this function is called, this parameter can be any valid code page, or the constant CP_ACP which specifies the current active code page for the locale. If the Unicode version of this function is called, this parameter should always be zero. |
| `lpszMailboxName` | `LPCSTR` | A pointer to a null terminated string which contains the UTF-7 encoded name of the mailbox. The encoded string must use the modified UTF-7 format defined in the IMAP4 protocol standard. This parameter cannot be a NULL pointer or an empty string. |
| `lpszDecodedName` | `LPTSTR` | A pointer to a string buffer which will contain the decoded mailbox name. This parameter cannot be NULL and must be large enough to contain the entire name. |
| `nMaxLength` | `INT` | An integer value which specifies the maximum number of characters which can be copied into the lpszDecodedName string buffer. This value must be greater than zero and the buffer must be large enough to store the entire name. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the number of characters in the decoded mailbox name. If an error occurs, the function returns zero. To get extended error information, call ImapGetLastError.

### Remarks

- Mailbox names which contain non-ASCII characters must be converted to a modified version of UTF-7 encoding as defined by the IMAP4 protocol. This encoding allows applications to reference mailbox names which contain Unicode characters and provides support for languages other than English. This function is used to convert a UTF-7 encoded mailbox name using the current locale, primarily for display purposes.
- For example, if the lpszMailboxName parameter specifies the string "&A5QDvwO6A7kDvAOu-", this function will convert it to the Greek word "Δοκιμή" and return that value in the lpszDecodedName buffer. It is recommended you use the Unicode version of this function whenever possible to ensure the mailbox name is decoded correctly.
- The ANSI version of this function should only be used with legacy applications which do not support Unicode. The decoded mailbox name will be converted using the code page specified by the nCodePage parameter. If the code page does not support the characters used in the mailbox name, this can result in an incorrect conversion. Using the previous example, the mailbox name will always decode correctly if the Unicode version of the function is used, but the ANSI version of the function will only decode it correctly if the code page is specified as 1253 (Greek).
- This function should not be used to convert strings between Unicode and UTF-7. The encoding format is specifically intended for use with the IMAP4 protocol and should never be used for general purpose encoding or decoding. If you need to convert a UTF-7 encoded string to Unicode, use the Win32 API function MultiByteToWideChar.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapEncodeMailboxName](#imapencodemailboxname-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)

---

## ImapDeleteMailbox Function

Delete the specified mailbox from the server

### Prototype

```c
INT WINAPI ImapDeleteMailbox(
    HCLIENT hClient,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a null terminated string which specifies the mailbox to be deleted. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns value of zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- A mailbox cannot be deleted if it contains inferior hierarchical names and has the IMAP_FLAG_NOSELECT attribute. On most systems this is the case when the mailbox name references a directory on the server, and that directory contains other subdirectories or mailboxes. To remove the mailbox, you must first delete any child mailboxes that exist.
- If the mailbox that is deleted is the currently selected mailbox, it will be automatically unselected and any messages marked for deletion will be expunged before the mailbox is removed. If the delete operation fails, the client will remain in an unselected state until either the ImapExamineMailbox or ImapSelectMailbox function is called.
- The special mailbox name INBOX is reserved, and cannot be deleted.
- If you need to delete a mailbox which contains Unicode characters or symbols, it is recommended you always use the Unicode version of this function. The mailbox name will automatically be converted to modified UTF-7 encoding as defined by the IMAP4 standard. The ANSI version of this function will accept mailbox names using either modified UTF-7 encoding or UTF-8 encoding.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)

---

## ImapDeleteMessage Function

Delete the specified message from the mailbox

### Prototype

```c
INT WINAPI ImapDeleteMessage(
    HCLIENT hClient,
    UINT nMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessage` | `UINT` | Number of message to delete from the server. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- This function only marks the message for deletion. The message is not actually deleted until the mailbox is expunged or another mailbox is selected. This function will return an error if the current mailbox is in read-only mode, such as if it was selected using the ImapExamineMailbox function.
- It is important to note that unlike the POP3 protocol, a message that is marked for deletion is still accessible on the IMAP server until the mailbox is expunged. This means, for example, that a deleted message can still be retrieved using the ImapGetMessage function.
- To determine if a message has been marked for deletion, use the ImapGetMessageFlags function and check if the IMAP_FLAG_DELETED bit flag has been set. To list all of the deleted messages in the current mailbox, use the ImapGetDeletedMessages function.
- To remove the deletion flag from the message, use the ImapUndeleteMessage function. To prevent all messages in the current mailbox from being expunged, use the ImapReselectMailbox function to reset the current mailbox state.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetDeletedMessages](#imapgetdeletedmessages-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapReselectMailbox](#imapreselectmailbox-function)
- [ImapUndeleteMessage](#imapundeletemessage-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapDeleteSecurityCredentials Function

Delete the specified client security credentials

### Prototype

```c
VOID WINAPI ImapDeleteSecurityCredentials(
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. On exit from the function, the pointer will be NULL. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- This function can be used to release the memory allocated to the client or server credentials after a secure connection has been terminated.

### Example

```cpp
if (lpSecCred)
	ImapDeleteSecurityCredentials(&lpSecCred);

ImapUninitialize();
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateSecurityCredentials](#imapcreatesecuritycredentials-function)
- [ImapUninitialize](#imapuninitialize-function)

---

## ImapDisableEvents Function

Disable the event notification mechanism

### Prototype

```c
INT WINAPI ImapDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapDisableEvents function is used to disable event message posting for the specified client session. Although this will immediately prevent any new events from being generated, it is possible that messages could be waiting in the message queue. Therefore, an application must be prepared to handle client event messages after this function has been called.
- This function is automatically called if the client has event notification enabled, and the ImapDisconnect function is called. The same issues regarding outstanding event messages also applies in this situation, requiring that the application handle event messages that may reference a client handle that is no longer valid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapEnableEvents](#imapenableevents-function)
- [ImapRegisterEvent](#imapregisterevent-function)

---

## ImapDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI ImapDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapEnableTrace](#imapenabletrace-function)

---

## ImapDisconnect Function

Disconnect from the current server

### Prototype

```c
INT WINAPI ImapDisconnect(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- If a mailbox is selected at the time that ImapDisconnect is called, the current mailbox will be unselected and any messages that are marked for deletion will be expunged. To prevent any deleted messages from being removed from the mailbox, use the ImapUnselectMailbox function to unselect the current mailbox prior to disconnecting from the server.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapUninitialize](#imapuninitialize-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapEnableEvents Function

Enable the client event notification mechanism

### Prototype

```c
INT WINAPI ImapEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the window which will receive the client notification messages. This parameter must specify a valid window handle. If a NULL handle is specified, event notification will be disabled. |
| `uEventMsg` | `UINT` | The message that is received when a client event occurs. To avoid conflict with standard Windows messages, this value must be greater than WM_USER (1024) or an error will be returned. If the hEventWnd parameter is NULL, this value should be WM_NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapEnableEvents function is used to request that notification messages be posted to the specified window whenever a client event occurs. This allows an application to monitor the status of different client operations, such as a file transfer. The client must create a window message handler, which processes the various events. The wParam argument will contain the client handle, the low word of the lParam argument will contain the event identifier, and the high word will contain any error code. If no error has occurred, the high word will have a value of zero. The following events may be generated:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDisableEvents](#imapdisableevents-function)
- [ImapRegisterEvent](#imapregisterevent-function)

---

## ImapEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI ImapEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | Name of the trace log file. If this parameter is NULL or empty,the file CSTRACE.LOG is used. The directory for CSTRACE.LOG is given by the TEMP environment variable, if it is defined; otherwise, the directory given by the TMP environment variable is used, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_DEFAULT`: All function calls and return values are written to the trace file. The actual data being sent or received will not be logged. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file. Those errors which are not fatal and only indicate a warning will not be logged.
    - `TRACE_WARNING`: Only those function calls which fail, or return values which indicate a warning, are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError.

### Remarks

- When trace logging is enabled, the logfile is opened, appended to and closed for each socket function call. Using the same logfile name, you can do the same in your application to add additional information to the logfile if needed. This can provide an application-level context for the entries made by the library. Make sure that the logfile is closed after the data has been written.
- The TRACE_HEXDUMP option can produce very large logfiles, since all data that is being sent and received by the application is logged. To reduce the size of the file, you can enable and disable logging around limited sections of code that you wish to analyze.
- All of the SocketTools networking components that use the Windows Sockets API support logging. If you are using multiple components, you only need to enable tracing once in your application or once per thread in a multithreaded application.
- To redistribute an application that includes logging functionality, the cstrcv11.dll library must be included as part of the installation package. This library provides the trace logging features, and if it is not available the ImapEnableTrace function will fail. Note that the trace logging library is a standard Windows DLL and does not need to be registered, it only needs to be redistributed with your application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDisableTrace](#imapdisabletrace-function)

---

## ImapEncodeMailboxName Function

Encode a mailbox name using modified UTF-7 encoding

### Prototype

```c
INT WINAPI ImapEncodeMailboxName(
    INT nCodePage,
    LPCTSTR lpszMailboxName,
    LPTSTR lpszEncodedName,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCodePage` | `INT` | An integer value which specifies the code page which should be used when converting the mailbox name to Unicode. If the ANSI version of this function is called, this parameter can be any valid code page or a value of zero, which specifies the current active code page for the locale. If the Unicode version of this function is called, no conversion is required and this parameter should always be zero. |
| `lpszMailboxName` | `LPCTSTR` | A pointer to a null terminated string which contains the name of the mailbox. If the ANSI version of this function is called, this string is automatically converted to Unicode using the code page specified by the nCodePage parameter. This parameter cannot be a NULL pointer or an empty string. |
| `lpszEncodedName` | `LPTSTR` | A pointer to a string buffer which will contain the UTF-7 encoded mailbox name. This parameter cannot be NULL and must be large enough to contain the entire name. The string will be null terminated and will only contain printable ASCII characters. |
| `nMaxLength` | `INT` | An integer value which specifies the maximum number of characters which can be copied into the lpszEncodedName string buffer. This value must be greater than zero and the buffer must be large enough to store the entire name. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the number of characters in the encoded mailbox name. If an error occurs, the function returns zero. To get extended error information, call ImapGetLastError.

### Remarks

- Mailbox names which contain non-ASCII characters must be converted to a modified version of UTF-7 encoding as defined by the IMAP4 protocol. This encoding allows applications to reference mailbox names which contain Unicode characters and provides support for languages other than English. This function is used to convert a mailbox name which contains non-ASCII characters to the UTF-7 encoded format required by IMAP servers.
- For example, if the lpszMailboxName parameter specifies the string "Δοκιμή", this function will encode it as "&A5QDvwO6A7kDvAOu-" and return that value in the lpszEncodedName buffer. It is recommended you use the Unicode version of this function whenever possible to ensure the mailbox name is encoded correctly.
- The ANSI version of this function should only be used with legacy applications which do not support Unicode. The mailbox name will be converted to Unicode using the code page specified by the nCodePage parameter. If the code page does not support the characters used in the mailbox name, this can result in an incorrect conversion. Using the previous example, the mailbox name will always encode correctly if the Unicode version of the function is used, but the ANSI version of the function will only encode it correctly if the code page is specified as 1253 (Greek).
- If the ANSI version of this function is used and the nCodePage parameter is zero, this function will attempt to determine if the lpszMailboxName parameter specifies UTF-8 text as the mailbox name. If it does, the name will be converted from UTF-8 to modified UTF-7; otherwise, the name will be encoded as modified UTF-7 using the current active code page for the process.
- This function should not be used to convert strings between Unicode and UTF-7. The encoding format is specifically intended for use with the IMAP4 protocol and should never be used for general purpose encoding or decoding. If you need to convert a Unicode string to UTF-7, use the Win32 API function WideCharToMultiByte.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapDecodeMailboxName](#imapdecodemailboxname-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)

---

## ImapEnumMessages Function

Enumerate the messages in the current mailbox

### Prototype

```c
INT WINAPI ImapEnumMessages(
    HCLIENT hClient,
    UINT nFirstMessageId,
    UINT nLastMessageId,
    DWORD dwMessageFlags,
    DWORD dwReserved,
    LPIMAPMESSAGE lpMessageList,
    INT nMaxMessages
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nFirstMessageId` | `UINT` | An unsigned integer value which specifies the first message to enumerate. This value must be greater than zero and specify a valid message identifier. The first message in the mailbox has a value of one. |
| `nLastMessageId` | `UINT` | An unsigned integer value which specifies the last message to enumerate. This value must be greater than or equal to the value of the nFirstMessageId parameter. A special value of 0xFFFFFFFF can be used to specify the last message in the mailbox. |
| `dwMessageFlags` | `DWORD` | This parameter is used to determine which messages are enumerated. If the value is zero, then all applicable messages will be enumerated. If the value is non-zero, only those messages which have at least one of the specified flags will be returned. More than one flag can be specified by using a bitwise operator. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be set to zero. |
| `lpMessageList` | `LPIMAPMESSAGE` | A pointer to an array of IMAPMESSAGE structures which will contain information about each of the messages returned by the server. This parameter cannot be NULL. |
| `nMaxMessages` | `INT` | An integer value which specifies the maximum size of the IMAPMESSAGE array that was passed to the function. This value must be at least one. |

### Options

- **dwMessageFlags** Parameter

    - `IMAP_FLAG_ANSWERED`: Return only those messages which have been answered.
    - `IMAP_FLAG_DELETED`: Return only those messages which have been marked for deletion.
    - `IMAP_FLAG_DRAFT`: Return only those messages which have been marked as draft copies.
    - `IMAP_FLAG_URGENT`: Return only those messages which have been flagged for urgent or special attention.
    - `IMAP_FLAG_RECENT`: Return only those messages which have been recently added to the mailbox.
    - `IMAP_FLAG_SEEN`: Return only those messages which have been read.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of messages that were enumerated. If no messages match the specified criteria, the function will return a value of zero. If an error is encountered, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- If the message UID is being stored locally by the client to identify the message over multiple sessions, it must also store the mailbox UID. Only the combination of the mailbox name, mailbox UID and message UID can be used to uniquely identify a given message on the server. Although IMAP server implementations are encouraged to maintain persistent message UIDs, they are not required to do so and those values may change if the mailbox UID changes.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageSize](#imapgetmessagesize-function)
- [ImapGetMessageUid](#imapgetmessageuid-function)

---

## ImapEventProc Function

Callback function that processes events generated by the client

### Prototype

```c
VOID __cdecl ImapEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the ImapRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the ImapRegisterEvent function. The ImapEventProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDisableEvents](#imapdisableevents-function)
- [ImapEnableEvents](#imapenableevents-function)
- [ImapFreezeEvents](#imapfreezeevents-function)
- [ImapRegisterEvent](#imapregisterevent-function)

---

## ImapExamineMailbox Function

Select the specified mailbox in read-only mode

### Prototype

```c
INT WINAPI ImapExamineMailbox(
    HCLIENT hClient,
    LPCTSTR lpszMailbox,
    LPIMAPMAILBOX lpMailboxInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a string which specifies the new mailbox to be examined. |
| `lpMailboxInfo` | `LPIMAPMAILBOX` | A pointer to an IMAPMAILBOX structure which contains information about the mailbox when the function returns. This parameter may be NULL if the caller does not require any information about the mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapExamineMailbox function is used to select a mailbox in read-only mode. Messages can be read, but they cannot be modified or deleted from the mailbox and new messages will not lose their status as new messages if they are accessed.
- If the client has a different mailbox currently selected, that mailbox will be closed and any messages marked for deletion will be expunged. To prevent deleted messages from being removed from the previous mailbox, use the ImapUnselectMailbox function prior to examining the new mailbox.
- If an application wishes to update the information returned in the IMAPMAILBOX structure for the current mailbox, simply call ImapExamineMailbox again with the same mailbox name.
- The special case-insensitive mailbox name INBOX is used for new messages. Other mailbox names may or may not be case-sensitive depending on the IMAP server's operating system and implementation.
- To access a mailbox in read-write mode, use the ImapSelectMailbox function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapReselectMailbox](#imapreselectmailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapExpungeMailbox Function

Remove messages that have been marked for deletion from the current mailbox

### Prototype

```c
INT WINAPI ImapExpungeMailbox(
    HCLIENT hClient,
    LPIMAPMAILBOX lpMailboxInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpMailboxInfo` | `LPIMAPMAILBOX` | A pointer to an IMAPMAILBOX structure which contains information about the mailbox when the function returns. This parameter may be NULL if the caller does not require any information about the mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapExpungeMailbox function causes all messages marked for deletion to be removed from the mailbox. Note that this can cause the mailbox's UID to change, and potentially invalidate the current message UIDs. It is recommended that applications use the information returned in the IMAPMAILBOX structure to update any internal state information stored on the local system.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)

---

## ImapFreezeEvents Function

Suspend and resume event handling by the client

### Prototype

```c
INT WINAPI ImapFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four events of the same type occur, once event handling is resumed only one of those events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDisableEvents](#imapdisableevents-function)
- [ImapEnableEvents](#imapenableevents-function)
- [ImapRegisterEvent](#imapregisterevent-function)

---

## ImapGetCapability Function

Return a string which identifies the capabilities of the server

### Prototype

```c
INT WINAPI ImapGetCapability(
    HCLIENT hClient,
    LPTSTR lpszCapability,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszCapability` | `LPTSTR` | A pointer to a null terminated string buffer that will contain one or more tokens separated by spaces which identify the capabilities of the IMAP server. |
| `nMaxLength` | `INT` | The maximum length of the capability string, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the capability string, not including the terminating null character. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- This function returns a string that contains one or more tokens separated by whitespace. Each token identifies a capability of the server. The following table lists some of the common capabilities:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDisconnect](#imapdisconnect-function)

---

## ImapGetCurrentMailbox Function

Return the name of the currently selected mailbox

### Prototype

```c
INT WINAPI ImapGetCurrentMailbox(
    HCLIENT hClient,
    LPTSTR lpszMailbox,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPTSTR` | A pointer to a null terminated string buffer that will contain the current mailbox name. If the Unicode version of this function is called, the mailbox name will automatically be converted from modified UTF-7 encoding to Unicode. If the ANSI version of this function is called, the name will be returned using modified UTF-7 encoding. This parameter cannot be NULL and the buffer must be large enough to store the complete mailbox name. |
| `nMaxLength` | `INT` | The maximum length of the mailbox string, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the current mailbox name, not including the terminating null character. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- It is recommended you use the Unicode version of this function, particularly if it is possible that mailbox names will contain Unicode characters or symbols. If the ANSI version of this function is called and a mailbox name contains Unicode characters, the name will be returned using a modified UTF-7 encoding format as defined by the IMAP4 standard. The mailbox name will not be returned as UTF-8 Unicode or as ANSI text converted to the current locale. If the Unicode version of this function is called, mailbox names will be automatically converted from modified UTF-7 to UTF-16 Unicode.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapGetDeletedMessages Function

Return the messages that have been marked for deletion

### Prototype

```c
INT WINAPI ImapGetDeletedMessages(
    HCLIENT hClient,
    UINT lpnMessageIds,
    INT nMaxMessages
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpnMessageIds` | `UINT` | A pointer to an array of unsigned integers that will contain the message identifiers of those messages which have been marked for deletion in the current mailbox. This parameter may be NULL, in which case the function will return the number of deleted messages but will not return their identifiers. |
| `nMaxMessages` | `INT` | The maximum number of message identifiers that may be stored in the lpnMessageIds array. If the lpnMessageIds parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of messages marked for deletion in the current mailbox. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The message identifiers returned by this function are only valid until the mailbox is expunged or another mailbox is selected.
- To remove the deleted flag from a message, use the ImapUndeleteMessage function. To prevent all messages in the current mailbox from being expunged, use the ImapReselectMailbox function to reset the current mailbox state.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetNewMessages](#imapgetnewmessages-function)
- [ImapGetUnseenMessages](#imapgetunseenmessages-function)
- [ImapSearchMailbox](#imapsearchmailbox-function)

---

## ImapGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI ImapGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description should be returned. If this value is zero, then the description of the last error will be returned. If the last error code is zero, indicating no error, then this function will return zero. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. It is recommended that this buffer be at least 128 characters in length. If a NULL pointer is specified, then no message will be returned but the function will return the length of the error string, not including the terminating null byte. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is 0, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetLastError](#imapgetlasterror-function)
- [ImapGetResultCode](#imapgetresultcode-function)
- [ImapGetResultString](#imapgetresultstring-function)
- [ImapSetLastError](#imapsetlasterror-function)

---

## ImapGetFirstMailbox Function

Return the first mailbox according to specified criteria

### Prototype

```c
INT WINAPI ImapGetFirstMailbox(
    HCLIENT hClient,
    LPCTSTR lpszReference,
    LPCTSTR lpszWildcard,
    DWORD dwOptions,
    LPVOID lpvReserved,
    LPTSTR lpszMailbox,
    INT nMaxLength,
    LPDWORD lpdwFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszReference` | `LPCTSTR` | A pointer to a string which specifies the reference name. An empty string or NULL pointer specifies that the default mailbox hierarchy for the current user is returned. If the reference name is provided, this must be the name of a mailbox or a level of the mailbox hierarchy which provides the context in which the mailbox name is interpreted. |
| `lpszWildcard` | `LPCTSTR` | A pointer to a null terminated string which specifies the mailbox name to match. The wildcard character "*" may be used to match any portion of the mailbox hierarchy, including the delimiter. The wildcard character "%" matches any portion of the mailbox name, but does not match the mailbox delimiter. An empty string or NULL pointer specifies that all mailboxes in the context of the lpszReference parameter should be returned. |
| `dwOptions` | `DWORD` | Specifies one or more options which controls how mailboxes are returned by the function. The options are bit flags which may be combined using a bitwise operator. One or more of the following values may be used: |
| `lpvReserved` | `LPVOID` | A reserved parameter. This value should always be set to NULL. |
| `lpszMailbox` | `LPTSTR` | A pointer to a string buffer which will contain the first matching mailbox. This parameter cannot be NULL. A minimum buffer size of at least 128 character is recommended. If the Unicode version of this function is called, the mailbox name will automatically be converted from modified UTF-7 encoding to Unicode. If the ANSI version of this function is called, the mailbox name will be returned using modified UTF-7 encoding. |
| `nMaxLength` | `INT` | Specifies the maximum length of the string buffer. The maximum length of the buffer should be large enough to accommodate most path names on the IMAP server. |
| `lpdwFlags` | `LPDWORD` | A pointer to an unsigned integer which will contain the mailbox flags for the first matching mailbox. This parameter may be NULL, in which case the mailbox flags are not returned. Otherwise, one or more of the following bit flags may be returned: |

### Options

- **dwOptions** Parameter

    - `IMAP_LIST_DEFAULT`: This option specifies that all regular, selectable mailboxes should be returned.
    - `IMAP_LIST_SUBSCRIBED`: This option specifies that only subscribed mailboxes should be returned.
    - `IMAP_LIST_FOLDERS`: This option specifies that non-selectable mailbox folders should also be returned.
    - `IMAP_LIST_HIDDEN`: This option specifies that hidden mailboxes should be returned.
    - `IMAP_LIST_INFERIOR`: This option specifies that inferior mailboxes should be returned if an explicit wildcard mask is not specified.
- **lpdwFlags** Parameter

    - `IMAP_FLAG_NOINFERIORS`: The mailbox does not contain any sub-mailboxes. In the IMAP protocol, these are referred to as inferior hierarchical mailbox names.
    - `IMAP_FLAG_NOSELECT`: The mailbox cannot be selected or examined. This flag is typically used by servers to indicate that the mailbox name refers to a directory on the server, not a mailbox file.
    - `IMAP_FLAG_MARKED`: The mailbox is marked as being of interest to a client. If this flag is used, it typically means that the mailbox contains messages. An application should not depend on this flag being present for any given mailbox. Some IMAP servers do not support marked or unmarked flags for mailboxes.
    - `IMAP_FLAG_UNMARKED`: The mailbox is marked as not being of interest to a client. If this flag is used, it typically means that the mailbox does not contain any messages. An application should not depend on this flag being present for any given mailbox. Some IMAP servers do not support marked or unmarked flags for mailboxes.

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the mailbox name. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetFirstMailbox function is used to begin enumerating the available mailboxes for the current user on the IMAP server. Subsequent mailbox names are returned by calling ImapGetNextMailbox until the function returns IMAP_ERROR with an error code of ST_ERROR_NO_MORE_MAILBOXES.
- It is recommended you use the Unicode version of this function, particularly if it is possible that mailbox names will contain Unicode characters or symbols. If the ANSI version of this function is called and a mailbox name contains Unicode characters, the name will be returned using a modified UTF-7 encoding format as defined by the IMAP4 standard. The mailbox name will not be returned as UTF-8 Unicode or as ANSI text converted to the current locale. If the Unicode version of this function is called, mailbox names will be automatically converted from modified UTF-7 to UTF-16 Unicode.
- The function of the lpszReference and lpszWildcard parameters are implementation dependent and generally are tied to the underlying operating system. On a UNIX based system, it can be helpful to think of the reference name as the directory where mailbox folders are stored, and the mailbox name as the name to search for in that directory and any subdirectories, if applicable. If the reference name is an empty string or NULL pointer, this typically refers to the current user's home directory.
- Generally speaking, a reference name should only be specified if you or the user of the application knows the directory structure on the IMAP server. Incorrectly using a reference name can have serious negative side-effects. For example, specifying a reference name of "/" on a UNIX based system could cause the IMAP server to return search every directory on the system for a matching mailbox name. Similarly, the IMAP server may be unable to distinguish between regular files in the user's home directory and mailboxes. For this reason, most IMAP clients require that the user specify the directory on the server where their mailboxes are stored. Typically this is subdirectory named "mail" or "Mail" under the user's home directory. For non-UNIX servers, the mailbox hierarchy may represented differently, including a flat hierarchy.
- Hidden mailboxes are those mailboxes which use the UNIX convention of the name beginning with a period. Therefore, a mailbox named ".secrets" would not normally be returned by the ImapGetFirstMailbox and ImapGetNextMailbox functions. The IMAP_LIST_HIDDEN option causes all mailboxes to be returned.
- The IMAP_LIST_INFERIOR option will return inferior mailboxes (mailboxes located in folders or subdirectories) if a wildcard mask is not specified. If a wildcard mask is specified, this option has no effect and only those mailboxes which match the wildcard will be returned.
- Subscribed mailboxes are those which were specified using the ImapSubscribeMailbox function. Marked mailboxes are typically those which have some special importance to the user.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapGetCurrentMailbox](#imapgetcurrentmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)

---

## ImapGetHeaderValue Function

Return the value of the specified header field

### Prototype

```c
INT WINAPI ImapGetHeaderValue(
    HCLIENT hClient,
    UINT nMessageId,
    LPCTSTR lpszHeader,
    LPTSTR lpszValue,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve header value from. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszHeader` | `LPCTSTR` | Pointer to a string which specifies the message header to retrieve. The colon should not be included in this string. |
| `lpszValue` | `LPTSTR` | Pointer to a string buffer that will contain the value of the specified message header. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the header field value. If the header field is not present in the message, the function will return a value of zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetHeaderValue function returns the value of header field from the specified message. This allows an application to be able to easily determine the value of a header such as the sender, or the subject of the message. Any header field, including non-standard extensions, may be returned by this function.
- To return the value of a header field from a specific part of a multipart MIME message, use the ImapGetHeaderValueEx function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetHeaderValueEx](#imapgetheadervalueex-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageId](#imapgetmessageid-function)
- [ImapGetMessageSender](#imapgetmessagesender-function)

---

## ImapGetHeaderValueEx Function

Return the value of a header field in the specified message part

### Prototype

```c
INT WINAPI ImapGetHeaderValueEx(
    HCLIENT hClient,
    UINT nMessageId,
    UINT nMessagePart,
    LPVOID lpvReserved,
    LPCTSTR lpszHeader,
    LPTSTR lpszValue,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve header value from. This value must be greater than zero. The first message in the mailbox is message number one. |
| `nMessagePart` | `UINT` | The message part that the header value will be be retrieved from. Message parts start with a value of one. A value of zero specifies that the RFC822 header field for the message will be used. |
| `lpvReserved` | `LPVOID` | A reserved parameter. This value should always be NULL. |
| `lpszHeader` | `LPCTSTR` | Pointer to a string which specifies the message header to retrieve. The colon should not be included in this string. |
| `lpszValue` | `LPTSTR` | Pointer to a string buffer that will contain the value of the specified message header. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the header field value. If the header field is not present in the message, the function will return a value of zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetHeaderValueEx function returns the value of a header field from the specified message part. This allows an application to be able to easily determine the value of a header such as the sender, or the subject of the message. Any header field, including non-standard extensions, may be returned by this function.
- To determine if a message is a multipart MIME message, use the ImapGetMessageParts function. The return value specifies the number of parts in the message, with a value greater than one indicating that it is a multipart message.
- Note that unlike the SocketTools MIME API which considers the first message part to be zero, the IMAP protocol defines the first message part to be one.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageId](#imapgetmessageid-function)
- [ImapGetMessageSender](#imapgetmessagesender-function)

---

## ImapGetIdleThreadId Function

Return the ID of the thread created to monitor the client session

### Prototype

```c
DWORD WINAPI ImapGetIdleThreadId(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, it returns the ID of the thread that is checking for update notifications from the server. If there is no active thread monitoring the client session, the function will return zero.

### Remarks

- The worker thread that monitors the client connection in the background can terminate if an IMAP command is sent to the server, if the ImapCancel function is called or if the client disconnects from the server. The ImapGetIdleThreadId function enables the application to determine if this background thread is still active or not.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapIdle](#imapidle-function)
- [ImapIdleProc](#imapidleproc-function)

---

## ImapGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI ImapGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the calling thread's last error code value. Functions set this value by calling the ImapSetLastError function. The return value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- You should call the ImapGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call ImapSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value error code such as FALSE, NULL, INVALID_CLIENT or IMAP_ERROR. Those functions which call ImapSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetErrorString](#imapgeterrorstring-function)
- [ImapSetLastError](#imapsetlasterror-function)

---

## ImapGetMailboxDelimiter Function

Get the path delimiter for the specified mailbox hierarchy

### Prototype

```c
INT WINAPI ImapGetMailboxDelimiter(
    HCLIENT hClient,
    LPCTSTR lpszMailbox,
    LPTSTR lpszDelimiter,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a null terminated string which specifies the mailbox name. This parameter may be NULL or an empty string, in which case the default delimiter will be returned. |
| `lpszDelimiter` | `LPTSTR` | A pointer to a null terminated string buffer that will contain the mailbox delimiter. |
| `nMaxLength` | `INT` | The maximum length of the delimiter string, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the delimiter for the specified mailbox, not including the terminating null character. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- If the IMAP server supports multiple levels of mailboxes, then a special character or sequence of characters are used as delimiters between different levels of the mailbox hierarchy. On most systems, including most UNIX and Windows platforms, the delimiter is the forward slash "/" character.
- It is possible that an IMAP server may only support a flat namespace, in which case this function will return a value of zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapGetMailboxSize Function

Return the size of the specified mailbox

### Prototype

```c
DWORD WINAPI ImapGetMailboxSize(
    HCLIENT hClient,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a string which specifies the mailbox name. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, it returns the size of the mailbox. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMailboxSize function may require a significant amount of time to calculate the mailbox size if there are a large number of messages in the mailbox. If the specified mailbox is not currently selected, then the current mailbox is unselected, the new mailbox is selected and the size calculated, and then the original mailbox is re-selected. This will have the side-effect of causing any messages marked for deletion to be expunged from the mailbox.
- Because it can potentially result in long delays, it is not recommended that an application calculate the mailbox size unless it is absolutely necessary.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)

---

## ImapGetMailboxStatus Function

Return the status of the specified mailbox

### Prototype

```c
INT WINAPI ImapGetMailboxStatus(
    HCLIENT hClient,
    LPCTSTR lpszMailbox,
    LPIMAPMAILBOXSTATUS lpMailboxStatus
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a null terminated string that specifies the mailbox to return information about. |
| `lpMailboxStatus` | `LPIMAPMAILBOXSTATUS` | A pointer to an IMAPMAILBOXSTATUS structure which contains status information about the specified mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMailboxStatus function enables an application to obtain status information about a mailbox without having to select another mailbox or open a second connection to the IMAP server to examine the mailbox. The information returned is a subset of the information returned when a mailbox is selected.
- If you need to check the status of a mailbox which contains Unicode characters or symbols, it is recommended you use the Unicode version of this function. The mailbox name will automatically be converted to modified UTF-7 encoding as defined by the IMAP4 standard. The ANSI version of this function will only accept mailbox names using modified UTF-7 encoding or UTF-8 encoding.
- Requesting information for a mailbox may be a slow operation, requiring the server to open the mailbox in read-only mode internally in order to obtain some of the status information. For this reason, this function should not be used to check for new messages; use the ImapCheckMailbox function instead.
- Some IMAP servers may return an error if you attempt to obtain status information about the currently selected mailbox. The protocol standard states that clients should not use this method on the currently selected mailbox, and should instead use the information returned by the ImapSelectMailbox or ImapExamineMailbox functions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCheckMailbox](#imapcheckmailbox-function)
- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)

---

## ImapGetMailboxUid Function

Return the unique identifier for the specified mailbox

### Prototype

```c
DWORD WINAPI ImapGetMailboxUid(
    HCLIENT hClient,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a null terminated string that specifies the mailbox name. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, it returns a non-zero value. If no unique identifier is assigned to the mailbox, the function will return zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMailboxUid function returns an unsigned 32-bit value which uniquely identifies the mailbox and corresponds to the UIDVALIDITY value returned by the IMAP server. The actual value is determined by the server and should be considered opaque. The protocol specification requires that a mailbox's UID must not change unless the mailbox contents are modified or existing messages in the mailbox have been assigned new UIDs.
- An application can use the mailbox UID value in combination with the message UID in order to uniquely identify a message on the server. However, the application must take into consideration that the IMAP server can reassign new message UIDs when the mailbox is modified. If the mailbox and message UIDs are being stored on the local system to track what messages have been retrieved from the server, the application must check the UID of the mailbox whenever it is selected. If the mailbox UID has changed, this means that the UIDs for the messages in that mailbox may have changed. The client should resynchronize with the server, and update it's local copy of that mailbox.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetCurrentMailbox](#imapgetcurrentmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetMessageUid](#imapgetmessageuid-function)

---

## ImapGetMessage Function

Retrieve the specified message from the server

### Prototype

```c
INT WINAPI ImapGetMessage(
    HCLIENT hClient,
    UINT nMessageId,
    UINT nMessagePart,
    LPVOID lpvMessage,
    LPDWORD lpdwLength,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve from the server. This value must be greater than zero. The first message in the mailbox is message number one. |
| `nMessagePart` | `UINT` | The message part that will be retrieved. A value of zero specifies that the complete message should be returned. If the message is a multipart MIME message, message parts start with a value of one. |
| `lpvMessage` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvMessage parameter. If the lpvMessage parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the message that was downloaded. |
| `dwOptions` | `DWORD` | An unsigned integer value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `IMAP_SECTION_DEFAULT`: All headers and the complete body of the specified message or message part are to be retrieved. The client application is responsible for parsing the header block. If the message is a MIME multipart message and the complete message is returned, the application is responsible for parsing the individual message parts if necessary.
    - `IMAP_SECTION_HEADER`: All headers for the specified message or message part are to be retrieved. The client application is responsible for parsing the header block.
    - `IMAP_SECTION_MIMEHEADER`: The MIME headers for the specified message or message are to be retrieved. Only those header fields which are used in MIME messages, such as Content-Type will be returned to the client. This is typically useful when processing the header for a multipart message which contains file attachments. The client application is responsible for parsing the header block.
    - `IMAP_SECTION_BODY`: The body of the specified message or message part is to be retrieved. For a MIME formatted message, this may include data that is uuencoded or base64 encoded. The application is responsible for decoding this data.
    - `IMAP_SECTION_PREVIEW`: The message header or body is being previewed and should not be marked as read by the server. This prevents the message from having the IMAP_FLAG_SEEN flag from being automatically set when the message data is retrieved.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessage function is used to retrieve a message from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the message. In this case, the lpvMessage parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvMessage parameter point to a global memory handle which will contain the message data when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur.
- This function will cause the current thread to block until the transfer completes, a timeout occurs or the transfer is canceled. During the transfer, the IMAP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling ImapEnableEvents, or by registering a callback function using the ImapRegisterEvent function.
- To determine if a message is a multipart MIME message, use the ImapGetMessageParts function. The return value specifies the number of parts in the message, with a value greater than one indicating that it is a multipart message. Combining the IMAP_SECTION_HEADER and IMAP_SECTION_BODY options will only return the header and body for the specified message if the nMessagePart parameter is zero. Due to a limitation of the IMAP FETCH command, if a message part is specified then only the body of that message part will be returned.
- Note that unlike the SocketTools MIME API which considers the first message part to be zero, the IMAP protocol defines the first message part to be one.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageParts](#imapgetmessageparts-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapGetMessageCount Function

Return the number of messages available in the mailbox

### Prototype

```c
INT WINAPI ImapGetMessageCount(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the number of messages in the currently selected mailbox. If no messages are available, this function will return zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessageCount function does not distinguish between messages that are marked for deletion and those which are not. This is because messages that are marked for deletion on an IMAP server can still be accessed until the mailbox is expunged or unselected. This differs from the POP3 protocol, where messages cannot be accessed once they have been marked for deletion.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCountEx](#imapgetmessagecountex-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapGetMessageCountEx Function

Return the number of messages available in the mailbox

### Prototype

```c
INT WINAPI ImapGetMessageCountEx(
    HCLIENT hClient,
    UINT lpnLastMessage,
    DWORD lpdwMailboxSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpnLastMessage` | `UINT` | Address of a variable that receives the number of the last valid message in the mailbox. If a NULL value is specified, this argument is ignored. |
| `lpdwMailboxSize` | `DWORD` | Address of a variable that receives the current size of the mailbox. If a NULL value is specified, this argument is ignored. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the number of messages that are currently available. If no messages are available, this function will return zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessageCountEx function is provided for compatibility with the POP3 API. The lpnLastMessage parameter will always contain the same value returned by the function since there is no distinction between the message count and the last available message. This is because messages that are marked for deletion on an IMAP server can still be accessed until the mailbox is expunged or unselected. This differs from the POP3 protocol, where messages cannot be accessed once they have been marked for deletion.
- If the lpdwMailboxSize parameter is specified, this function will call ImapGetMailboxSize to determine the size of the currently selected mailbox. Unlike the POP3 protocol, calculating the mailbox size may require a significant amount of time if there are a large number of messages in the mailbox. It is not recommended that an application request the mailbox size unless it is absolutely necessary.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMailboxSize](#imapgetmailboxsize-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapGetMessageFlags Function

Return the status flags for the specified message

### Prototype

```c
BOOL WINAPI ImapGetMessageFlags(
    HCLIENT hClient,
    UINT nMessageId,
    LPDWORD lpdwMessageFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to obtain the message flags for. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpdwMessageFlags` | `LPDWORD` | Pointer to an unsigned integer that will contain the message flags for the specified message. The value may be zero, or one or more of the following values: |

### Options

- **lpdwMessageFlags** Parameter

    - `IMAP_FLAG_ANSWERED`: The message has been answered.
    - `IMAP_FLAG_DELETED`: The message has been marked for deletion.
    - `IMAP_FLAG_DRAFT`: The message has not been completed and is marked as a draft copy.
    - `IMAP_FLAG_URGENT`: The message has been flagged for urgent or special attention.
    - `IMAP_FLAG_RECENT`: The message has been added to the mailbox recently.
    - `IMAP_FLAG_SEEN`: The message has been read.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero and the lpdwMessageFlags parameter contains the status flags for the specified message. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapSetMessageFlags](#imapsetmessageflags-function)

---

## ImapGetMessageHeaders Function

Retrieve the specified message header from the server

### Prototype

```c
INT WINAPI ImapGetMessageHeaders(
    HCLIENT hClient,
    UINT nMessageId,
    LPVOID lpvHeaders,
    LPDWORD lpdwLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve from the server. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpvHeaders` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvHeaders parameter. If the lpvHeaders parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the message that was downloaded. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessagHeaders function is used to retrieve a message header block from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the file. In this case, the lpvHeaders parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvHeaders parameter point to a global memory handle which will contain the file data when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur.
- This function will cause the current thread to block until the file transfer completes, a timeout occurs or the transfer is canceled. During the transfer, the IMAP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling ImapEnableEvents, or by registering a callback function using the ImapRegisterEvent function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageParts](#imapgetmessageparts-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapGetMessageHeadersEx Function

Retrieve the specified message header, UID and flags from the server

### Prototype

```c
INT WINAPI ImapGetMessageHeadersEx(
    HCLIENT hClient,
    UINT nMessageId,
    DWORD dwReserved,
    LPDWORD lpdwMessageUID,
    LPDWORD lpdwMessageFlags,
    LPVOID lpvHeaders,
    LPDWORD lpdwLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve from the server. This value must be greater than zero. The first message in the mailbox is message number one. |
| `dwReserved` | `DWORD` | A reserved parameter, this value should always be zero. |
| `lpdwMessageUID` | `LPDWORD` | A pointer to an unsigned integer which will contain the unique ID assigned to the message by the server when the function returns. This parameter may be NULL if this information is not required. |
| `lpdwMessageFlags` | `LPDWORD` | A pointer to an unsigned integer which will contain the message flags when the function returns. This parameter may be NULL if this information is not required, otherwise the value may be zero or one or more of the following bit flags: |
| `lpvHeaders` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvHeaders parameter. If the lpvHeaders parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the message that was downloaded. |

### Options

- **lpdwMessageFlags** Parameter

    - `IMAP_FLAG_ANSWERED`: The message has been answered.
    - `IMAP_FLAG_DELETED`: The message has been marked for deletion.
    - `IMAP_FLAG_DRAFT`: The message has not been completed and is marked as a draft copy.
    - `IMAP_FLAG_URGENT`: The message has been flagged for urgent or special attention.
    - `IMAP_FLAG_RECENT`: The message has been added to the mailbox recently.
    - `IMAP_FLAG_SEEN`: The message has been read.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessagHeadersEx function is used to retrieve a message header block from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the file. In this case, the lpvHeaders parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvHeaders parameter point to a global memory handle which will contain the file data when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur.
- If the lpdwMessageUID and/or lpdwMessageFlags parameters are NULL, they will be ignored. Otherwise, the function will always initialize the values to zero and only sets them if the header block for the message can be retrieved.
- This function will cause the current thread to block until the file transfer completes, a timeout occurs or the transfer is canceled. During the transfer, the IMAP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling ImapEnableEvents, or by registering a callback function using the ImapRegisterEvent function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetMessageParts](#imapgetmessageparts-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapGetMessageId Function

Return the message ID string for the specified message

### Prototype

```c
INT WINAPI ImapGetMessageId(
    HCLIENT hClient,
    UINT nMessageId,
    LPTSTR lpszMessageId,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve the unique identifier for. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszMessageId` | `LPTSTR` | Address of a string buffer to receive the message identifier. This should be at least 64 bytes in length. |
| `nMaxLength` | `INT` | The maximum length of the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the unique identifier string. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessageId function returns the message identifier from the Message-ID header of the specified message. The returned value is a string which can be used to identify a specific message, regardless if the message is moved to a different mailbox.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageSender](#imapgetmessagesender-function)

---

## ImapGetMessageParts Function

Return the number of MIME message parts in the specified message

### Prototype

```c
INT WINAPI ImapGetMessageParts(
    HCLIENT hClient,
    UINT nMessageId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve the part count for. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of parts in the specified message. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessageParts function can be used to determine if a message on the server contains multiple parts. A multipart MIME message typically contains file attachments or multiple representations of the message, such as a version of the message in plain text and another using HTML markup.
- If the function returns a value of one, then the message does not contain multiple parts and is a standard RFC822 formatted message. A value greater than one indicates that the message does have multiple parts. The ImapGetMessageEx function may be used to retrieve the data for a specific part of the message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageId](#imapgetmessageid-function)
- [ImapGetMessageSender](#imapgetmessagesender-function)

---

## ImapGetMessageSender Function

Return the address of the message sender

### Prototype

```c
INT WINAPI ImapGetMessageSender(
    HCLIENT hClient,
    UINT nMessageId,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve header value from. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszAddress` | `LPTSTR` | Pointer to a string buffer that will contain the address of the message sender. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the address. If the sender cannot be determined, the function will return a value of zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessageSender function returns the email address specified in the Return-Path header field. This allows an application to be able to easily determine the sender, without parsing the header or downloading the contents of the message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageId](#imapgetmessageid-function)

---

## ImapGetMessageSize Function

Return the size of the specified message

### Prototype

```c
DWORD WINAPI ImapGetMessageSize(
    HCLIENT hClient,
    UINT nMessageId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve size of. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the size of the specified message in bytes. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetHeaderValue](#imapgetheadervalue-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetMessageId](#imapgetmessageid-function)
- [ImapGetMessageSender](#imapgetmessagesender-function)

---

## ImapGetMessageUid Function

Return the unique identifier for the specified message

### Prototype

```c
DWORD WINAPI ImapGetMessageUid(
    HCLIENT hClient,
    UINT nMessageId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to obtain the unique identifier for. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, it returns a non-zero value. If no unique identifier is assigned to the message, the function will return zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetMessageUid function returns an unsigned integer value which specifies a unique identifier for this message. The actual value is determined by the server and should be considered opaque.
- An application can use the message UID value in combination with the mailbox UID in order to uniquely identify a message on the server. However, the application must take into consideration that the IMAP server can reassign new message UIDs when the mailbox is modified. If the mailbox and message UIDs are being stored on the local system to track what messages have been retrieved from the server, the application must check the UID of the mailbox whenever it is selected. If the mailbox UID has changed, this means that the UIDs for the messages in that mailbox may have changed. The client should resynchronize with the server, and update it's local copy of that mailbox.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetCurrentMailbox](#imapgetcurrentmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetMailboxUid](#imapgetmailboxuid-function)

---

## ImapGetNewMessages Function

Return a list of the new messages in the current mailbox

### Prototype

```c
INT WINAPI ImapGetNewMessages(
    HCLIENT hClient,
    UINT lpnMessageIds,
    INT nMaxMessages
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpnMessageIds` | `UINT` | A pointer to an array of unsigned integers that will contain the message identifiers of those messages that have recently been added to the mailbox and have not been read. This parameter may be NULL, in which case the function will return the number of new messages but will not return their identifiers. |
| `nMaxMessages` | `INT` | The maximum number of message identifiers that may be stored in the lpnMessageIds array. If the lpnMessageIds parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of new messages in the current mailbox. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The message identifiers returned by this function are only valid until the mailbox is expunged or another mailbox is selected. Once a message has been read using ImapGetMessage or ImapStoreMessage, it is no longer considered to be a new message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetDeletedMessages](#imapgetdeletedmessages-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetUnseenMessages](#imapgetunseenmessages-function)
- [ImapSearchMailbox](#imapsearchmailbox-function)

---

## ImapGetNextMailbox Function

Return the next mailbox name on the server

### Prototype

```c
INT WINAPI ImapGetNextMailbox(
    HCLIENT hClient,
    LPTSTR lpszMailbox,
    INT nMaxLength,
    LPDWORD lpdwFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPTSTR` | A pointer to a string buffer which will contain the next matching mailbox. This parameter cannot be NULL. A minimum buffer size of at least 128 character is recommended. If the Unicode version of this function is called, the mailbox name will automatically be converted from modified UTF-7 encoding to Unicode. If the ANSI version of this function is called, the mailbox name will be returned using modified UTF-7 encoding. |
| `nMaxLength` | `INT` | Specifies the maximum length of the string buffer. The maximum length of the buffer should be large enough to accommodate most path names on the IMAP server. |
| `lpdwFlags` | `LPDWORD` | A pointer to an unsigned integer which will contain the mailbox flags for the next matching mailbox. This parameter may be NULL, in which case the mailbox flags are not returned. Otherwise, one or more of the following bit flags may be returned: |

### Options

- **lpdwFlags** Parameter

    - `IMAP_FLAG_NOINFERIORS`: The mailbox does not contain any sub-mailboxes. In the IMAP protocol, these are referred to as inferior hierarchical mailbox names.
    - `IMAP_FLAG_NOSELECT`: The mailbox cannot be selected or examined. This flag is typically used by servers to indicate that the mailbox name refers to a directory on the server, not a mailbox file.
    - `IMAP_FLAG_MARKED`: The mailbox is marked as being of interest to a client. If this flag is used, it typically means that the mailbox contains messages. An application should not depend on this flag being present for any given mailbox. Some IMAP servers do not support marked or unmarked flags for mailboxes.
    - `IMAP_FLAG_UNMARKED`: The mailbox is marked as not being of interest to a client. If this flag is used, it typically means that the mailbox does not contain any messages. An application should not depend on this flag being present for any given mailbox. Some IMAP servers do not support marked or unmarked flags for mailboxes.

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the length of the mailbox name. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetNextMailbox function returns the next matching mailbox name. When the last mailbox has been returned, the next call to this function will result in an error, with the last error code set to ST_ERROR_NO_MORE_MAILBOXES.
- It is recommended you use the Unicode version of this function, particularly if it is possible that mailbox names will contain Unicode characters or symbols. If the ANSI version of this function is called and a mailbox name contains Unicode characters, the name will be returned using a modified UTF-7 encoding format as defined by the IMAP4 standard. The mailbox name will not be returned as UTF-8 Unicode or as ANSI text converted to the current locale. If the Unicode version of this function is called, mailbox names will be automatically converted from modified UTF-7 to UTF-16 Unicode.
- Subscribed mailboxes are those which were specified using the ImapSubscribeMailbox function. Marked mailboxes are typically those which have some special importance to the user.
- For more information about enumerating the available mailboxes on the IMAP server, refer to the ImapGetFirstMailbox function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapGetCurrentMailbox](#imapgetcurrentmailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)

---

## ImapGetResultCode Function

Return the result code from the previous command

### Prototype

```c
INT WINAPI ImapGetResultCode(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the result code. If the function fails, it returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The following result codes may be returned by the IMAP server:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCommand](#imapcommand-function)
- [ImapGetResultString](#imapgetresultstring-function)

---

## ImapGetResultString Function

Return the result string from the previous command

### Prototype

```c
INT WINAPI ImapGetResultString(
    HCLIENT hClient,
    LPTSTR lpszResult,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszResult` | `LPTSTR` | A pointer to the buffer that will contain the result string returned by the server. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the result string buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the result string. If a value of zero is returned, this means that no result string was sent by the server. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetResultString function is most useful when an error occurs because the server will typically include a brief description of the cause of the error. This can then be parsed by the application or displayed to the user. The result string is updated each time the client sends a command to the server and then calls ImapGetResultCode to obtain the result code for the operation.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCommand](#imapcommand-function)
- [ImapGetResultCode](#imapgetresultcode-function)

---

## ImapGetSecurityInformation Function

Return security information about the current client connection

### Prototype

```c
BOOL WINAPI ImapGetSecurityInformation(
    HCLIENT hClient,
    LPSECURITYINFO lpSecurityInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpSecurityInfo` | `LPSECURITYINFO` | A pointer to a SECURITYINFO structure which contains information about the current client connection. The dwSize member of this structure must be initialized to the size of the structure before passing the address of the structure to this function. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError.

### Remarks

- This function is used to obtain security related information about the current client connection to the server. It can be used to determine if a secure connection has been established, what security protocol was selected, and information about the server certificate. Note that a secure connection has not been established, the dwProtocol structure member will contain the value SECURITY_PROTOCOL_NONE.

### Example

```cpp
// Notifies the user if the connection is secure or not
SECURITYINFO securityInfo;

securityInfo.dwSize = sizeof(SECURITYINFO);
if (ImapGetSecurityInformation(hClient, &securityInfo))
{
	if (securityInfo.dwProtocol == SECURITY_PROTOCOL_NONE)
	{
		MessageBox(NULL, _T("The connection is not secure"),
						_T("Connection"), MB_OK);
	}
	else
	{
		if (securityInfo.dwCertStatus == SECURITY_CERTIFICATE_VALID)
		{
			MessageBox(NULL, _T("The connection is secure"),
							_T("Connection"), MB_OK);
		}
		else
		{
			MessageBox(NULL, _T("The server certificate not valid"),
							_T("Connection"), MB_OK);
		}
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapCreateSecurityCredentials](#imapcreatesecuritycredentials-function)
- [ImapDeleteSecurityCredentials](#imapdeletesecuritycredentials-function)

---

## ImapGetStatus Function

Return the current status of the client

### Prototype

```c
INT WINAPI ImapGetStatus(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the client status code. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetStatus function returns a numeric code which identifies the current state of the client session. The following values may be returned:
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapIsBlocking](#imapisblocking-function)
- [ImapIsConnected](#imapisconnected-function)
- [ImapIsReadable](#imapisreadable-function)
- [ImapIsWritable](#imapiswritable-function)

---

## ImapGetTimeout Function

Return the number of seconds until an operation times out

### Prototype

```c
INT WINAPI ImapGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapIsReadable](#imapisreadable-function)
- [ImapIsWritable](#imapiswritable-function)
- [ImapRead](#imapread-function)
- [ImapSetTimeout](#imapsettimeout-function)
- [ImapWrite](#imapwrite-function)

---

## ImapGetTransferStatus Function

Return data transfer statistics

### Prototype

```c
INT WINAPI ImapGetTransferStatus(
    HCLIENT hClient,
    LPIMAPTRANSFERSTATUS lpStatus
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpStatus` | `LPIMAPTRANSFERSTATUS` | A pointer to an IMAPTRANSFERSTATUS structure which contains information about the status of the current file transfer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapGetTransferStatus function returns information about the current file transfer, including the average number of bytes transferred per second and the estimated amount of time until the transfer completes. If there is no file currently being transferred, this function will return the status of the last successful transfer made by the client.
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapEnableEvents](#imapenableevents-function)
- [ImapGetStatus](#imapgetstatus-function)
- [ImapRegisterEvent](#imapregisterevent-function)

---

## ImapGetUnseenMessages Function

Return a list of messages from the current mailbox that have not been read

### Prototype

```c
INT WINAPI ImapGetUnseenMessages(
    HCLIENT hClient,
    UINT lpnMessageIds,
    INT nMaxMessages
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpnMessageIds` | `UINT` | A pointer to an array of unsigned integers that will contain the message identifiers of those messages that have not been read. This parameter may be NULL, in which case the function will return the number of unseen messages but will not return their identifiers. |
| `nMaxMessages` | `INT` | The maximum number of message identifiers that may be stored in the lpnMessageIds array. If the lpnMessageIds parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of unseen messages in the current mailbox. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The message identifiers returned by this function are only valid until the mailbox is expunged or another mailbox is selected. Once a message has been read using ImapGetMessage or ImapStoreMessage, it is no longer considered to be an unseen message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetDeletedMessages](#imapgetdeletedmessages-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetNewMessages](#imapgetnewmessages-function)
- [ImapSearchMailbox](#imapsearchmailbox-function)

---

## ImapIdle Function

Enables mailbox status monitoring for the client session

### Prototype

```c
INT WINAPI ImapIdle(
    HCLIENT hClient,
    UINT nTimeout,
    DWORD dwOptions,
    IMAPIDLEPROC lpfnIdleProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `UINT` | Specifies the timeout period in seconds to wait for a notification from the server. This parameter is only used when the IMAP_IDLE_WAIT option has been specified. |
| `dwOptions` | `DWORD` | Specifies the options which should be used when enabling idle monitoring. The following options are supported: |
| `lpfnIdleProc` | `IMAPIDLEPROC` | A pointer to an IMAPIDLEPROC callback function that will be invoked whenever the server sends an update notification to the client. This parameter must specify a valid function address and cannot be a NULL pointer. |
| `dwParam` | `DWORD_PTR` | A user-defined value that is passed back to the caller whenever the callback function is invoked. This can be used to provide additional state information to the client. If it is not needed, the caller should use a value of zero. |

### Options

- **dwOptions** Parameter

    - `IMAP_IDLE_NOWAIT`: The function should return immediately after idle processing has been enabled. When this option is used, the application may continue to perform other functions while the client session is monitored for status updates sent by the server. The client will continue to monitor status changes until an IMAP command issued or the client disconnects from the server.
    - `IMAP_IDLE_WAIT`: The function should wait until the server sends a status update, or until the timeout period is reached. The client will stop monitoring status changes when the function returns. If this option is used in a single-threaded application, normal message processing can be impeded, causing the application to appear non-responsive until the timeout period is reached. It is strongly recommended that single-threaded applications with a user interface specify the IMAP_IDLE_NOWAIT option instead.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- Many IMAP servers support the ability to asynchronously send status updates to the client, rather than have the client periodically poll the server. The client enables this feature by calling ImapIdle and providing the address of a callback function that will be invoked whenever the server sends an update notification to the client. Typically these updates inform the client that a new message has arrived or that a message has been expunged from the mailbox.
- The ImapIdle function can operate in two modes, based on the options specified by the caller. If the option IMAP_IDLE_NOWAIT is specified, the function begins monitoring the client session and returns control immediately to the caller. If the server sends a update notification, the callback function will be invoked with information about the status change. If the option IMAP_IDLE_WAIT is specified, the function will block waiting for the server to send a notification message to the client. The function will return when either a message is received or the timeout period is exceeded.
- Sending an IMAP command to the server will cause the client to stop monitoring the session for status changes. To explicitly stop monitoring the session, use the ImapCancel function. To determine if the current client session is being monitored, use the ImapGetIdleThreadId function. A non-zero return value indicates that the client session is idle and being monitored.
- This function works by sending the IDLE command to the server and starting a worker thread which monitors the connection and looks for untagged responses issued by the server. Callbacks will be invoked for EXISTS, EXPUNGE and RECENT messages. Note that some servers may periodically send untagged OK messages to the client, indicating that the connection is still active; these messages are explicitly ignored. Because the monitoring is performed in a different thread, the callback function is invoked in the context of that thread. Client event notifications are disabled while inside the callback function, and the ImapIdle function cannot be used to restart monitoring from within a callback function.
- Applications should not perform any operation that takes a significant amount of time or updates the user interface from within the callback function. Instead, use flags or send application defined messages to indicate a change in state. For example, if the server sends a notification that a new email message has arrived, the application should not attempt to read the new message and update the user interface from within the callback function. Instead, it could use the PostMessage function to send an application-defined message to the UI thread indicating the change in state. The application would have a message handler for that Windows message and update the user interface, indicating that a new message has arrived.
- An application should never make an assumption about how a particular server may send update notifications to the client. Servers can be configured to use different intervals at which notifications are sent. For example, a server may send new message notifications immediately, but may periodically notify the client when a message has been expunged. Alternatively, a server may only send notifications at fixed intervals, in which case the client would not be notified of any new messages until the interval period is reached. It is not possible for a client to know what a particular server's update interval is. Applications that require that degree of control should not use ImapIdle and should poll the server instead.

### Example

```cpp
// Begin monitoring the client session for status changes; when a new
// message arrives or a message is expunged, the UpdateHandler callback
// function will be invoked.
BOOL MonitorUpdates(HWND hwndNotify)
{
	INT nResult;      

	nResult = ImapIdle(hClient, 0, IMAP_IDLE_NOWAIT, UpdateHandler, hwndNotify);

	if (nResult == IMAP_ERROR)
	{
		ShowError(ImapGetLastError());
		return FALSE;
	}

	return TRUE;
}

// This function is called whenever the server notifies the client
// that a new message has arrived or a message has been expunged from
// the current mailbox
#define WM_APP_NEWMESSAGE (WM_APP+1)
#define WM_APP_EXPUNGED   (WM_APP+2)

VOID CALLBACK UpdateHandler(HCLIENT hClient, UINT nUpdateId, UINT nMessageId, DWORD_PTR dwParam)
{
	switch (nUpdateId)
	{
	case IMAP_UPDATE_MESSAGE:
		{
			// Send a message indicating that a new message has arrived
			PostMessage((HWND)dwParam, WM_APP_NEWMESSAGE, nMessageId, 0);
		}
		break;

	case IMAP_UPDATE_EXPUNGE:
		{
			// Send a message indicating that a message has been expunged
			PostMessage((HWND)dwParam, WM_APP_EXPUNGED, nMessageId, 0);
		}
		break;
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetIdleThreadId](#imapgetidlethreadid-function)
- [ImapIdleProc](#imapidleproc-function)

---

## ImapIdleProc Function

Callback function that receives update notifications from the server

### Prototype

```c
VOID __cdecl ImapIdleProc(
    HCLIENT hClient,
    UINT nUpdateId,
    UINT nMessageId,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nUpdateId` | `UINT` | An unsigned integer which specifies the type of update notification that has been sent by the server. It may be one of the following values: |
| `nMessageId` | `UINT` | An unsigned integer which specifies the message number associated with the status change. Note that this argument is not used with the IMAP_UPDATE_MAILBOX notification and will contain a value of zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the ImapIdle function was called. |

### Options

- **nUpdateId** Parameter

    - `IMAP_UPDATE_UNKNOWN`: The server has sent an unrecognized notification message. The value of the nMessageId argument is undefined for this type of notification. This does not necessarily reflect an error condition, as some servers may send additional notification messages beyond the standard EXISTS, EXPUNGE and RECENT messages. Most applications should ignore this type of notification.
    - `IMAP_UPDATE_MESSAGE`: The server has sent notification message to the client indicating that a new message has arrived. The nMessageId argument will contain the message number for the new message. Typically this update notification occurs shortly after the new message has been stored in the current mailbox.
    - `IMAP_UPDATE_EXPUNGE`: The server has sent a notification message to the client indicating that a message has been removed from the current mailbox. The nMessageId argument will contain the message number for the message that has been removed. It is recommended that the application re-examine the mailbox when this notification is received. Typically this notification is only sent periodically by the server, and may not be sent immediately after a message has been expunged from the mailbox.
    - `IMAP_UPDATE_MAILBOX`: The server has sent notification message to the client indicating that the state of the mailbox has changed. The nMessageId argument is not used with this notification. This message is sent periodically by the server and may not be sent immediately after a new message arrives or a message is flagged as unread. It is recommended that the application re-examine the mailbox when this notification is received.

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must enable this callback function by passing its address to the ImapIdle function. The ImapIdleProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetIdleThreadId](#imapgetidlethreadid-function)
- [ImapIdle](#imapidle-function)

---

## ImapInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI ImapInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that ImapInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to ImapUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the ImapInitialize or ImapUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDisconnect](#imapdisconnect-function)
- [ImapUninitialize](#imapuninitialize-function)

---

## ImapIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI ImapIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns a non-zero value. If the client is not performing a blocking operation, or the client handle is invalid, the function function returns zero.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The ImapIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action such as warning the user that they must wait for the operation to complete.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCancel](#imapcancel-function)
- [ImapIsConnected](#imapisconnected-function)
- [ImapIsReadable](#imapisreadable-function)
- [ImapIsWritable](#imapiswritable-function)

---

## ImapIsConnected Function

Determine if the client is connected to the server

### Prototype

```c
BOOL WINAPI ImapIsConnected(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is connected to a server, the function returns a non-zero value. If the client is not connected, or the client handle is invalid, the function returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapIsBlocking](#imapisblocking-function)
- [ImapIsReadable](#imapisreadable-function)
- [ImapIsWritable](#imapiswritable-function)

---

## ImapIsReadable Function

Determine if data can be read from the server

### Prototype

```c
BOOL WINAPI ImapIsReadable(
    HCLIENT hClient,
    INT nTimeout,
    LPDWORD lpdwAvail
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |
| `lpdwAvail` | `LPDWORD` | A pointer to an unsigned integer which will contain the number of bytes available to read. This parameter may be NULL if this information is not required. |

### Return Value

- **Type**: `BOOL`
- If the client can read data from the server within the specified timeout period, the function returns a non-zero value. If the client cannot read any data, the function returns zero.

### Remarks

- On some platforms, this value will not exceed the size of the receive buffer (typically 64K bytes). Because of differences between TCP/IP stack implementations, it is not recommended that your application exclusively depend on this value to determine the exact number of bytes available. Instead, it should be used as a general indicator that there is data available to be read.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetStatus](#imapgetstatus-function)
- [ImapIsBlocking](#imapisblocking-function)
- [ImapIsConnected](#imapisconnected-function)
- [ImapIsWritable](#imapiswritable-function)
- [ImapRead](#imapread-function)

---

## ImapIsWritable Function

Determine if data can be written to the server

### Prototype

```c
BOOL WINAPI ImapIsWritable(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |

### Return Value

- **Type**: `BOOL`
- If the client can write data to the server within the specified timeout period, the function returns a non-zero value. If the client cannot write any data, the function returns zero.

### Remarks

- Although this function can be used to determine if some amount of data can be sent to the remote process it does not indicate the amount of data that can be written without blocking the client. In most cases, it is recommended that large amounts of data be broken into smaller logical blocks, typically some multiple of 512 bytes in length.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetStatus](#imapgetstatus-function)
- [ImapIsBlocking](#imapisblocking-function)
- [ImapIsConnected](#imapisconnected-function)
- [ImapIsReadable](#imapisreadable-function)
- [ImapWrite](#imapwrite-function)

---

## ImapLogin Function

Login to the server

### Prototype

```c
INT WINAPI ImapLogin(
    HCLIENT hClient,
    UINT nAuthType,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nAuthType` | `UINT` | Identifies the type of authentication that should be used when the client logs in to the mail server. The following authentication methods are supported: |
| `lpszUserName` | `LPCTSTR` | A null terminated string which specifies the user name to be used to authenticate the current client session. |
| `lpszPassword` | `LPCTSTR` | A null terminated string which specifies the password to be used when authenticating the current client session. If you are using the IMAP_AUTH_XOAUTH2 or IMAP_AUTH_BEARER authentication methods, this parameter is not a password, instead it specifies the OAuth 2.0 bearer token provided by the mail service. |

### Options

- **nAuthType** Parameter

    - `IMAP_AUTH_LOGIN`: Standard cleartext username and password is sent to the server. This authentication method is supported by all servers. Note that some servers may only support LOGIN authentication if a secure connection has been established.
    - `IMAP_AUTH_PLAIN`: Login using the PLAIN authentication mechanism as defined in RFC 4959. This authentication method is supported by most servers, although some may require that client establish a secure connection.
    - `IMAP_AUTH_XOAUTH2`: This authentication type will use the XOAUTH2 method to authenticate the client session. This authentication method does not require the user password, instead the lpszPassword parameter must specify the OAuth 2.0 bearer token issued by the service provider. The application must provide a valid access token which has not expired, or this function will fail.
    - `IMAP_AUTH_BEARER`: This authentication type will use the OAUTHBEARER method to authenticate the client session as defined in RFC 7628. This authentication method does not require the user password, instead the lpszPassword parameter must specify the OAuth 2.0 bearer token issued by the service provider. The application must provide a valid access token which has not expired, or this function will fail.
    - `IMAP_AUTH_ANONYMOUS`: Login using the anonymous Simple Authentication and Security Layer (SASL) mechanism as defined in RFC 4505. If this authentication method is specified, the lpszUserName parameter should specify a name or email address acceptable to the mail server. The lpszPassword parameter is ignored and may be NULL.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- In some cases, the user may be pre-authenticated by the server. In this case, the function will fail with the last error set to ST_ERROR_ALREADY_AUTHENTICATED. If a particular authentication method is not supported by the server, the last error will be set to ST_ERROR_INVALID_AUTHENTICATION_TYPE. For compatibility with the greatest number of servers, it is recommended that you use IMAP_AUTH_LOGIN as the authentication method.
- You should only use an OAuth 2.0 authentication method if you understand the process of how to request the access token. Obtaining an access token requires registering your application with the mail service provider (e.g.: Microsoft or Google), getting a unique client ID associated with your application and then requesting the access token using the appropriate scope for the service. Obtaining the initial token will typically involve interactive confirmation on the part of the user, requiring they grant permission to your application to access their mail account.
- The IMAP_AUTH_XOAUTH2 and IMAP_AUTH_BEARER authentication methods are similar, but they are not interchangeable. Both use an OAuth 2.0 bearer token to authenticate the client session, but they differ in how the token is presented to the server. It is currently preferable to use the XOAUTH2 method because it is more widely available and some service providers do not yet support the OAUTHBEARER method.
- Your application should not store an OAuth 2.0 bearer token for later use. They have a relatively short lifespan, typically about an hour, and are designed to be used with that session. You should specify offline access as part of the OAuth 2.0 scope if necessary and store the refresh token provided by the service. The refresh token has a much longer validity period and can be used to obtain a new bearer token when needed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDisconnect](#imapdisconnect-function)
- [ImapInitialize](#imapinitialize-function)

---

## ImapOpenMessage Function

Open the specified message for reading on the server

### Prototype

```c
INT WINAPI ImapOpenMessage(
    HCLIENT hClient,
    UINT nMessageId,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve. This value must be greater than zero. The first message in the mailbox is message number one. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapOpenMessage function uses the FETCH command to access the specified message on the server. The client can then use the the ImapRead function to read the contents of the message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCloseMessage](#imapclosemessage-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapOpenMessageEx](#imapopenmessageex-function)
- [ImapRead](#imapread-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapOpenMessageEx Function

Open the specified message for reading on the server, with additional options

### Prototype

```c
INT WINAPI ImapOpenMessageEx(
    HCLIENT hClient,
    UINT nMessageId,
    UINT nMessagePart,
    DWORD dwOffset,
    LPDWORD lpdwLength,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve. This value must be greater than zero. The first message in the mailbox is message number one. |
| `nMessagePart` | `UINT` | The message part that will be retrieved. A value of zero specifies that the complete message should be returned. If the message is a multipart MIME message, message parts start with a value of one. |
| `dwOffset` | `DWORD` | The byte offset into the message. This parameter can be used in conjunction with the lpdwLength parameter to return a specific part of a message. A value of zero specifies the beginning of the message. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer value which should be initialized to the maximum number of bytes to be read, and will contain the size of the message when the function returns. To specify the entire message, from the offset specified by the dwOffset parameter to the end of the message, initialize the lpdwLength parameter to a value of -1. This parameter may be NULL if the message size is not needed. |
| `dwOptions` | `DWORD` | The low order word of this parameter specifies how the message data will be returned. It may be one of the following values: |

### Options

- **dwOptions** Parameter

    - `IMAP_SECTION_DEFAULT`: All headers and the complete body of the specified message or message part are to be retrieved. The client application is responsible for parsing the header block. If the message is a MIME multipart message and the complete message is returned, the application is responsible for parsing the individual message parts if necessary.
    - `IMAP_SECTION_HEADER`: All headers for the specified message or message part are to be retrieved. The client application is responsible for parsing the header block.
    - `IMAP_SECTION_MIMEHEADER`: The MIME headers for the specified message or message are to be retrieved. Only those header fields which are used in MIME messages, such as Content-Type will be returned to the client. This is typically useful when processing the header for a multipart message which contains file attachments. The client application is responsible for parsing the header block.
    - `IMAP_SECTION_BODY`: The body of the specified message or message part is to be retrieved. For a MIME formatted message, this may include data that is uuencoded or base64 encoded. The application is responsible for decoding this data.
    - `IMAP_SECTION_PREVIEW`: The message header or body is being previewed and should not be marked as read by the server. This prevents the message from having the IMAP_FLAG_SEEN flag from being automatically set when the message data is retrieved.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapOpenMessageEx function uses the FETCH command to access the specified message on the server. The client can then use the ImapRead function to read the contents of the message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCloseMessage](#imapclosemessage-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapOpenMessage](#imapopenmessage-function)
- [ImapRead](#imapread-function)
- [ImapStoreMessage](#imapstoremessage-function)

---

## ImapRead Function

Read data returned by the server

### Prototype

```c
INT WINAPI ImapRead(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer in which the data will be copied. |
| `cbBuffer` | `INT` | The maximum number of bytes to read and copy into the specified buffer. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually read. A return value of zero indicates that there is no more data available to be read. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- When ImapRead is called and the client is in non-blocking mode, it is possible that the function will fail because there is no available data to read at that time. This should not be considered a fatal error. Instead, the application should simply wait to receive the next asynchronous notification that data is available.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapIsBlocking](#imapisblocking-function)

---

## ImapRegisterEvent Function

Register an event handler for the specified event

### Prototype

```c
INT WINAPI ImapRegisterEvent(
    HCLIENT hClient,
    UINT nEvent,
    IMAPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One of the following values may be used: |
| `lpEventProc` | `IMAPEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the ImapEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEvent** Parameter

    - `IMAP_EVENT_CONNECT`: The connection to the server has completed.
    - `IMAP_EVENT_DISCONNECT`: The server has closed the connection to the client. The client should read any remaining data and disconnect.
    - `IMAP_EVENT_READ`: Data is available to read by the client. No additional messages will be posted until the client has read at least some of the data. This event is only generated if the calling process is in asynchronous mode.
    - `IMAP_EVENT_WRITE`: The client can now write data. This notification is sent after a connection has been established, or after a previous attempt to write data has failed because it would result in a blocking operation. This event is only generated if the client is in asynchronous mode.
    - `IMAP_EVENT_TIMEOUT`: The client has timed out while waiting for a response from the server. Note that under some circumstances this event can be generated for a non-blocking connection, such as when the client is establishing a secure connection.
    - `IMAP_EVENT_CANCEL`: The client has canceled the current operation.
    - `IMAP_EVENT_COMMAND`: The client has processed a command that was sent to the server. The result code and result string can be used to determine if the response to the command.
    - `IMAP_EVENT_PROGRESS`: This event notification is sent periodically during lengthy blocking operations, such as retrieving a complete message from the server.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapRegisterEvent function associates a callback function with a specific event. The event handler is an ImapEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- This function is typically used to register an event handler that is invoked while a message is being retrieved. The IMAP_EVENT_PROGRESS event will only be generated periodically during the transfer to ensure the application is not flooded with event notifications. It is guaranteed that at least one IMAP_EVENT_PROGRESS notification will occur at the beginning of the transfer, and one at the end of the transfer when it has completed.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDisableEvents](#imapdisableevents-function)
- [ImapEnableEvents](#imapenableevents-function)
- [ImapEventProc](#imapeventproc-function)
- [ImapFreezeEvents](#imapfreezeevents-function)

---

## ImapRenameMailbox Function

Rename the specified mailbox

### Prototype

```c
INT WINAPI ImapRenameMailbox(
    HCLIENT hClient,
    LPCTSTR lpszOldMailbox,
    LPCTSTR lpszNewMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszOldMailbox` | `LPCTSTR` | A pointer to a string which specifies the mailbox to be renamed. |
| `lpszNewMailbox` | `LPCTSTR` | A pointer to a string which specifies the new mailbox name. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapRenameMailbox function renames an existing mailbox on the server. The new mailbox name cannot exist on the server, or the function will fail.
- If the existing mailbox name contains inferior hierarchical names (mailboxes under the specified mailbox) then those mailboxes will also be renamed. For example, if the mailbox "Mail/Pictures" contains two mailboxes, "Personal" and "Work" and it is renamed to "Mail/Images" then the two mailboxes under it would be automatically renamed to "Mail/Images/Personal" and "Mail/Images/Work".
- If the mailbox being renamed is the currently selected mailbox, the current mailbox will be unselected and any messages marked for deletion will be expunged. The new mailbox name will then automatically be re-selected. To prevent deleted messages from being removed from the mailbox prior to being renamed, use the ImapUnselectMailbox function to unselect the current mailbox before calling ImapRenameMailbox. Note that if the rename operation fails, the client may be left in an unselected state.
- It is permitted to rename the special mailbox INBOX. In this case, the messages will be moved from the INBOX mailbox to the new mailbox. If the INBOX mailbox is currently selected, the new mailbox will not automatically be selected. INBOX will remain the selected mailbox.
- If you need to delete a mailbox which contains Unicode characters or symbols, it is recommended you always use the Unicode version of this function. The mailbox name will automatically be converted to modified UTF-7 encoding as defined by the IMAP4 standard. The ANSI version of this function will accept mailbox names using either modified UTF-7 encoding or UTF-8 encoding.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapCreateMailbox](#imapcreatemailbox-function)
- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)

---

## ImapReselectMailbox Function

Reselect the current mailbox and return updated status information

### Prototype

```c
BOOL WINAPI ImapReselectMailbox(
    HCLIENT hClient,
    LPIMAPMAILBOX lpMailboxInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpMailboxInfo` | `LPIMAPMAILBOX` | A pointer to an IMAPMAILBOX structure which contains updated information about the mailbox when the function returns. This parameter may be NULL if the caller does not require any information about the mailbox. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapReselectMailbox function forces the current mailbox to be reselected and returns updated information about the status of the mailbox. Deleted messages are not expunged from the mailbox and remain marked for deletion.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapSearchMailbox Function

Search the mailbox according to specified criteria

### Prototype

```c
LONG WINAPI ImapSearchMailbox(
    HCLIENT hClient,
    LPCTSTR lpszCriteria,
    LPCTSTR lpszCharacterSet,
    DWORD dwReserved,
    UINT lpnMessageIds,
    LONG nMaxMessages
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszCriteria` | `LPCTSTR` | A pointer to a string which specifies the search criteria. |
| `lpszCharacterSet` | `LPCTSTR` | A pointer to a string which specifies the character set to use when searching the mailbox. If this parameter is NULL or an empty string, the default UTF-8 character set will be used. |
| `dwReserved` | `DWORD` | A reserved parameter which should be set to the value 0. |
| `lpnMessageIds` | `UINT` | A pointer to an array of unsigned integers that will contain the message identifiers of those messages which match the search criteria in the current mailbox. This parameter may be NULL, in which case the function will return the number of matching messages but will not return their identifiers. |
| `nMaxMessages` | `LONG` | The maximum number of message identifiers that may be stored in the lpnMessageIds array. If the lpnMessageIds parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the number of messages that meet the search criteria in the current mailbox. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapSearchMailbox function is used to search a mailbox for messages which match a given criteria and return a list of the matching message identifiers. The search criteria is composed of one or more search keywords and and optional value to match against. String searches are not case sensitive and partial matches in the message are returned.
- The following search keywords are recognized:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetDeletedMessages](#imapgetdeletedmessages-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapGetNewMessages](#imapgetnewmessages-function)
- [ImapGetUnseenMessages](#imapgetunseenmessages-function)

---

## ImapSelectMailbox Function

Select the specified mailbox in read-write mode

### Prototype

```c
INT WINAPI ImapSelectMailbox(
    HCLIENT hClient,
    LPCTSTR lpszMailbox,
    LPIMAPMAILBOX lpMailboxInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a null terminated string which specifies the new mailbox to be selected. If this parameter is NULL or an empty string, the default INBOX mailbox will be selected. |
| `lpMailboxInfo` | `LPIMAPMAILBOX` | A pointer to an IMAPMAILBOX structure which contains information about the mailbox when the function returns. This parameter may be NULL if the caller does not require any information about the mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapSelectMailbox function is used to select a mailbox in read-write mode. If the client has a different mailbox currently selected, that mailbox will be closed and any messages marked for deletion will be expunged. To prevent deleted messages from being removed from the previous mailbox, use the ImapUnselectMailbox function prior to selecting the new mailbox.
- If you need to select a mailbox which contains Unicode characters or symbols, it is recommended you use the Unicode version of this function. The mailbox name will automatically be converted to modified UTF-7 encoding as defined by the IMAP4 standard. The ANSI version of this function will only accept mailbox names using modified UTF-7 encoding or UTF-8 encoding.
- If an application wishes to update the information returned in the IMAPMAILBOX structure for the current mailbox, simply call ImapSelectMailbox again with the same mailbox name. Note that this will not cause any messages marked for deletion to be expunged.
- The special case-insensitive mailbox name INBOX is used for new messages. Other mailbox names may or may not be case-sensitive depending on the IMAP server's operating system and implementation.
- To access a mailbox in read-only mode, use the ImapExamineMailbox function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapReselectMailbox](#imapreselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapSetLastError Function

Set the last error code for the current thread

### Prototype

```c
VOID WINAPI ImapSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the caller. A value of zero clears the last error code. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value error code such as FALSE, NULL, INVALID_CLIENT or IMAP_ERROR. Those functions which call ImapSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the ImapGetLastError function. The use of ImapGetLastError is optional. An application can call it to find out the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapGetErrorString](#imapgeterrorstring-function)
- [ImapGetLastError](#imapgetlasterror-function)

---

## ImapSetMessageFlags Function

Set one or more status flags for the specified message

### Prototype

```c
INT WINAPI ImapSetMessageFlags(
    HCLIENT hClient,
    UINT nMessageId,
    UINT nMode,
    DWORD dwMessageFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to obtain the message flags for. This value must be greater than zero. The first message in the mailbox is message number one. |
| `nMode` | `UINT` | An unsigned integer value which specifies one of the following modes which determines how the message flags are set: |
| `dwMessageFlags` | `DWORD` | An unsigned integer value which specifies one or more message flags. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **nMode** Parameter

    - `IMAP_FLAGS_REPLACE`: All message flags are replaced with the flags specified by the dwMessageFlags parameter.
    - `IMAP_FLAGS_ADD`: The message flags specified by the dwMessageFlags parameter will be set for the message. Message flags that have been previously set will remain unmodified.
    - `IMAP_FLAGS_REMOVE`: The message flags specified by the dwMessageFlags parameter will be removed from the message. Message flags that are not specified will remain unmodified.
- **dwMessageFlags** Parameter

    - `IMAP_FLAG_ANSWERED`: The message has been answered.
    - `IMAP_FLAG_DELETED`: The message has been marked for deletion.
    - `IMAP_FLAG_DRAFT`: The message has not been completed and is marked as a draft copy.
    - `IMAP_FLAG_URGENT`: The message has been flagged for urgent or special attention.
    - `IMAP_FLAG_RECENT`: The message has been added to the mailbox recently.
    - `IMAP_FLAG_SEEN`: The message has been read.

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns IMAP_RESULT_OK. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)

---

## ImapSetTimeout Function

Set the number of seconds until an operation times out

### Prototype

```c
INT WINAPI ImapSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds to wait for a blocking operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The timeout value is only used with blocking network operations.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapGetTimeout](#imapgettimeout-function)
- [ImapRead](#imapread-function)

---

## ImapStoreMessage Function

Store the contents of a message to the specified file

### Prototype

```c
INT WINAPI ImapStoreMessage(
    HCLIENT hClient,
    UINT nMessageId,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of the message to retrieve. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string which specifies the file that the message will be stored in. If an empty string or NULL pointer is passed as an argument, the message is copied to the system clipboard. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapStoreMessage function provides a method of retrieving and storing a message on the local system. The contents of the message is stored as a text file, using the specified file name. This function always causes the caller to block until the entire message has been retrieved, even if the client has been put in asynchronous mode.
- If event handling is enabled, the IMAP_EVENT_PROGRESS event will fire periodically during the transfer of the message to the local system. An application can determine how much of the message has been retrieved by calling the ImapGetTransferStatus function.
- To retrieve the message into a global memory buffer so that it can be passed to the MIME or SMTP libraries, use the ImapGetMessage function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageHeaders](#imapgetmessageheaders-function)
- [ImapGetTransferStatus](#imapgettransferstatus-function)

---

## ImapSubscribeMailbox Function

Subscribe to the specified mailbox

### Prototype

```c
INT WINAPI ImapSubscribeMailbox(
    HCLIENT hClient,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a string which specifies the mailbox to subscribe to. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapSubscribeMailbox function adds the specified mailbox to the current user's list of active or subscribed mailboxes. The user will remain subscribed to the mailbox across multiple sessions, until the ImapUnsubscribeMailbox function is called to remove the mailbox from the subscription list.
- To list those mailboxes which the user has subscribed to, use the ImapGetFirstMailbox function and specify the IMAP_LIST_SUBSCRIBED option.
- Note that if a user subscribes to a mailbox and that mailbox is later renamed or deleted, the mailbox will not be automatically removed from the user's subscription list. An application must not assume that because a mailbox name is included in the list of subscribed mailboxes, it exists and can be selected. To check if the mailbox exists, use the ImapGetMailboxStatus function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)
- [ImapUnsubscribeMailbox](#imapunsubscribemailbox-function)

---

## ImapUndeleteMessage Function

Undelete the specified message from the current mailbox

### Prototype

```c
INT WINAPI ImapUndeleteMessage(
    HCLIENT hClient,
    UINT nMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessage` | `UINT` | Number of message to undelete from the server. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDeleteMessage](#imapdeletemessage-function)
- [ImapGetDeletedMessages](#imapgetdeletedmessages-function)
- [ImapGetMessage](#imapgetmessage-function)
- [ImapGetMessageCount](#imapgetmessagecount-function)
- [ImapGetMessageFlags](#imapgetmessageflags-function)
- [ImapReselectMailbox](#imapreselectmailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI ImapUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful ImapInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call ImapUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to ImapUninitialize for every successful call to ImapInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapConnect](#imapconnect-function)
- [ImapDisconnect](#imapdisconnect-function)
- [ImapInitialize](#imapinitialize-function)

---

## ImapUnselectMailbox Function

Unselect the current mailbox and expunge any deleted messages

### Prototype

```c
INT WINAPI ImapUnselectMailbox(
    HCLIENT hClient,
    BOOL bExpunge
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bExpunge` | `BOOL` | A boolean flag which determines if deleted messages will be expunged from the mailbox. A non-zero value specifies that messages that have been marked for deletion will be removed from the mailbox. A zero value specifies that no messages will be removed from the mailbox. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapDeleteMailbox](#imapdeletemailbox-function)
- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetMailboxStatus](#imapgetmailboxstatus-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapRenameMailbox](#imaprenamemailbox-function)
- [ImapReselectMailbox](#imapreselectmailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)

---

## ImapUnsubscribeMailbox Function

Unsubscribe from the specified mailbox

### Prototype

```c
INT WINAPI ImapUnsubscribeMailbox(
    HCLIENT hClient,
    LPCTSTR lpszMailbox
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailbox` | `LPCTSTR` | A pointer to a string which specifies the mailbox to unsubscribe from. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns zero. If an error occurs, the function returns IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The ImapUnsubscribeMailbox function removes the specified mailbox from the current user's list of active or subscribed mailboxes.
- To list those mailboxes which the user has subscribed to, use the ImapGetFirstMailbox function and specify the IMAP_LIST_SUBSCRIBED option.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ImapExamineMailbox](#imapexaminemailbox-function)
- [ImapGetFirstMailbox](#imapgetfirstmailbox-function)
- [ImapGetNextMailbox](#imapgetnextmailbox-function)
- [ImapSelectMailbox](#imapselectmailbox-function)
- [ImapSubscribeMailbox](#imapsubscribemailbox-function)
- [ImapUnselectMailbox](#imapunselectmailbox-function)

---

## ImapWrite Function

Write data to the server

### Prototype

```c
INT WINAPI ImapWrite(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | The pointer to the buffer which contains the data that is to be sent to the server. |
| `cbBuffer` | `INT` | The number of bytes to send from the specified buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually written. If the function fails, the return value is IMAP_ERROR. To get extended error information, call ImapGetLastError.

### Remarks

- The return value may be less than the number of bytes specified by the cbBuffer parameter. In this case, the data has been partially written and it is the responsibility of the client application to send the remaining data at some later point. For non-blocking clients, the client must wait for the IMAP_EVENT_WRITE asynchronous notification message before it resumes sending data.
- The ImapWrite function should not be used to send commands to the IMAP server. Use the ImapCommand function instead.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmapv11.lib`

### See Also

- [ImapCommand](#imapcommand-function)
- [ImapIsBlocking](#imapisblocking-function)
- [ImapIsWritable](#imapiswritable-function)
- [ImapRead](#imapread-function)

---

## IMAPMAILBOX Structure

This structure contains information about a selected mailbox.

### Definition

```c
typedef struct _IMAPMAILBOX
{
    UINT nMessages;
    UINT nRecentMessages;
    UINT nUnseenMessageId;
    DWORD dwMailboxUID;
    DWORD dwNextMessageUID;
    DWORD dwFlags;
    DWORD dwPermanentFlags;
    DWORD dwAccessMode;
    DWORD dwReserved1;
    DWORD dwReserved2;
} IMAPMAILBOX, *LPIMAPMAILBOX;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nMessages` | `UINT` | A value specifies the total number of messages in the mailbox. |
| `nRecentMessages` | `UINT` | A value which specifies the number of new messages that have recently arrived in the mailbox. |
| `nUnseenMessageId` | `UINT` | A value which specifies the message ID of the first unseen message in the mailbox. |
| `dwMailboxUID` | `DWORD` | A value which specifies a unique identifier for this mailbox which corresponds to the UIDVALIDITY value returned by the IMAP server. The actual value is determined by the server and should be considered opaque. The protocol specification requires that a mailbox's UID must not change unless the mailbox contents are modified or existing messages in the mailbox have been assigned new UIDs. |
| `dwNextMessageUID` | `DWORD` | A value which specifies the predicted unique identifier that will be assigned to a new message in the mailbox. This corresponds to the UIDNEXT value returned by the IMAP server. The protocol specification requires that as long as the mailbox UID is unchanged, messages that are added to the mailbox will be assigned a UID greater than or equal to the next UID value. |
| `dwFlags` | `DWORD` | A value which specifies one or more mailbox flags. One or more of the following values may be specified: |
| `dwPermanentFlags` | `DWORD` | A value which specifies the message flags that a client can change permanently. If this value is zero, then no permanent flags are defined for the mailbox and the client may assume that all message flags may be set permanently. Otherwise, one or more of the following values may be specified: |
| `dwAccessMode` | `DWORD` | A value which specifies the access mode for the mailbox. It may be one of the following values: |
| `dwReserved1` | `DWORD` | A reserved value that is undefined. |
| `dwReserved2` | `DWORD` | A reserved value that is undefined. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## IMAPMAILBOXSTATUS Structure

This structure contains information about a mailbox.

### Definition

```c
typedef struct _IMAPMAILBOXSTATUS
{
    UINT nMessages;
    UINT nRecentMessages;
    UINT nUnseenMessages;
    DWORD dwMailboxUID;
    DWORD dwNextMessageUID;
    DWORD dwReserved;
} IMAPMAILBOXSTATUS, *LPIMAPMAILBOXSTATUS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nMessages` | `UINT` | A value specifies the total number of messages in the mailbox. |
| `nRecentMessages` | `UINT` | A value which specifies the number of new messages that have recently arrived in the mailbox. |
| `nUnseenMessages` | `UINT` | A value which specifies the number of unread messages in the mailbox. |
| `dwMailboxUID` | `DWORD` | A value which specifies a unique identifier for this mailbox which corresponds to the UIDVALIDITY value returned by the IMAP server. The actual value is determined by the server and should be considered opaque. The protocol specification requires that a mailbox's UID must not change unless the mailbox contents are modified or existing messages in the mailbox have been assigned new UIDs. |
| `dwNextMessageUID` | `DWORD` | A value which specifies the predicted unique identifier that will be assigned to a new message in the mailbox. This corresponds to the UIDNEXT value returned by the IMAP server. The protocol specification requires that as long as the mailbox UID is unchanged, messages that are added to the mailbox will be assigned a UID greater than or equal to the next UID value. |
| `dwReserved` | `DWORD` | A reserved value that is undefined. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## IMAPMESSAGE Structure

This structure contains information about a message.

### Definition

```c
typedef struct _IMAPMESSAGE
{
    UINT nMessageId;
    DWORD dwMessageUID;
    DWORD dwSize;
    DWORD dwFlags;
    DWORD dwTimestamp;
    DWORD dwReserved;
} IMAPMESSAGE, *LPIMAPMESSAGE;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nMessageId` | `UINT` | An integer value which identifies the message. The message identifier is only valid while the mailbox is selected and no messages marked for deletion have been expunged. To maintain a persistent identifier for the message, use a combination of the mailbox UID and message UID. |
| `dwMessageUID` | `DWORD` | An integer value which specifies a unique identifier for this message. The actual value is determined by the server and should be considered opaque. If the client application stores the message UID on the local system, it should also store the UID for the mailbox that contains the message. If the mailbox UID changes, the message UID may no longer be valid. |
| `dwSize` | `DWORD` | Specifies the size of the message in bytes. |
| `dwFlags` | `DWORD` | A value which specifies one or more message flags. One or more of the following values may be specified: |
| `dwTimestamp` | `DWORD` | An integer value which specifies the date and time that the message was created in the mailbox. The value is expressed as the number of seconds since midnight, 1 January 1970 and is the same value that is used for the standard C runtime library time functions. Note that the date and time used is the message's internal date from the mail server, not the value of the Date header field. |
| `dwReserved` | `DWORD` | A reserved value that is undefined. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## IMAPTRANSFERSTATUS Structure

This structure is used by the ImapGetTransferStatus function to return information about a file transfer in progress.

### Definition

```c
typedef struct _IMAPTRANSFERSTATUS
{
    UINT nMessageId;
    DWORD dwBytesTotal;
    DWORD dwBytesCopied;
    DWORD dwBytesPerSecond;
    DWORD dwTimeElapsed;
    DWORD dwTimeEstimated;
} IMAPTRANSFERSTATUS, *LPIMAPTRANSFERSTATUS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nMessageId` | `UINT` | The message ID of the current message that is being transferred. |
| `dwBytesTotal` | `DWORD` | The total number of bytes that will be transferred. If the file is being copied from the server to the local host, this is the size of the remote file. If the file is being copied from the local host to the server, it is the size of the local file. If the file size cannot be determined, this value will be zero. |
| `dwBytesCopied` | `DWORD` | The total number of bytes that have been copied. |
| `dwBytesPerSecond` | `DWORD` | The average number of bytes that have been copied per second. |
| `dwTimeElapsed` | `DWORD` | The number of seconds that have elapsed since the file transfer started. |
| `dwTimeEstimated` | `DWORD` | The estimated number of seconds until the file transfer is completed. This is based on the average number of bytes transferred per second. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYCREDENTIALS Structure

The SECURITYCREDENTIALS structure specifies the information needed by the library to specify additional security credentials, such as a client certificate, when establishing a secure connection.

### Definition

```c
typedef struct _SECURITYCREDENTIALS
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwOptions;
    DWORD dwReserved;
    LPCTSTR lpszHostName;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
    LPCTSTR lpszCertStore;
    LPCTSTR lpszCertName;
    LPCTSTR lpszKeyFile;
} SECURITYCREDENTIALS, *LPSECURITYCREDENTIALS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. If the structure is being allocated dynamically, this member must be set to the size of the structure and all other unused structure members must be initialized to a value of zero or NULL. |
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. The value of this structure member is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or options. This member is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | This structure member is reserved for future use and should always be initialized to zero. |
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the hostname that will be used when validating the server certificate. If this member is NULL, then the server certificate will be validated against the hostname used to establish the connection. |
| `lpszUserName` | `LPCTSTR` | A pointer to a null terminated string which identifies the owner of client certificate. Currently this member is not used by the library and should always be initialized as a NULL pointer. |
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string which specifies the password needed to access the certificate. Currently this member is only used if the CREDENTIAL_STORE_FILENAME option has been specified. If there is no password associated with the certificate, then this member should be initialized as a NULL pointer. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The library will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the library will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the library will return an error indicating that the certificate could not be found. If the SECURITY_PROTOCOL_SSH protocol has been specified, this member should be NULL. |
| `lpszKeyFile` | `LPCTSTR` | This member is not used and should always be NULL. The private key cannot be specified separately and must be associated with the certificate. |

### Remarks

- An application only needs to create this structure if the server requires the client to provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions member should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore member should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12. These certificate files typically have an extension of .pfx or .p12. Note that if a password was specified when the certificate file was created, it must be provided in the lpszPassword member or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword members are values which are used to access the certificate store or private key file. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYINFO Structure

The SECURITYINFO structure contains security related information about the connection, including the protocol, cipher suite and certificate.

### Definition

```c
typedef struct _SECURITYINFO
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwCipher;
    DWORD dwCipherStrength;
    DWORD dwHash;
    DWORD dwHashStrength;
    DWORD dwKeyExchange;
    DWORD dwCertStatus;
    SYSTEMTIME stCertIssued;
    SYSTEMTIME stCertExpires;
    LPCTSTR lpszCertIssuer;
    LPCTSTR lpszCertSubject;
    LPCTSTR lpszFingerprint;
} SECURITYINFO, *LPSECURITYINFO;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Specifies the size of the data structure. This member must always be initialized to the size of the structure in bytes prior to passing the address of this structure to the function. If this this member is not initialized, an error will be returned indicating that an invalid parameter has been passed to the function. |
| `dwProtocol` | `DWORD` | A numeric value which specifies the security protocol selected when establishing the connection. |
| `dwCipher` | `DWORD` | A numeric value which specifies the cipher that was selected when establishing the secure connection. |
| `dwCipherStrength` | `DWORD` | A numeric value which specifies the strength (the length of the cipher key in bits) of the cipher that was selected. Typically this value will be 128 or 256. 40-bit and 56-bit key lengths are considered weak encryption, and subject to brute force attacks. 128-bit and 256-bit key lengths are considered to be secure, and are the recommended key length for secure communications. |
| `dwHash` | `DWORD` | A numeric value which specifies the hash algorithm which was selected. |
| `dwHashStrength` | `DWORD` | A numeric value which specifies the strength (the length in bits) of the message digest that was selected. |
| `dwKeyExchange` | `DWORD` | A numeric value which specifies the key exchange algorithm which was selected. |
| `dwCertStatus` | `DWORD` | A numeric value which specifies the status of the certificate returned by the secure server. |
| `stCertIssued` | `SYSTEMTIME` | A structure which contains the date and time that the certificate was issued by the certificate authority. If the issue date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `stCertExpires` | `SYSTEMTIME` | A structure which contains the date and time that the certificate expires. If the expiration date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `lpszCertIssuer` | `LPCTSTR` | A pointer to a string which contains information about the organization that issued the certificate. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate issuer could not be determined, this member may be NULL. |
| `lpszCertSubject` | `LPCTSTR` | A pointer to a string which contains information about the organization that the certificate was issued to. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate subject could not be determined, this member may be NULL. |
| `lpszFingerprint` | `LPCTSTR` | A pointer to a string which contains a sequence of hexadecimal values that uniquely identify the server. This member may be NULL. |

### Remarks

- The size of this structure is typically determined using the sizeof operator in C/C++ and most other languages have a similar method for obtaining the size of a data structure or user-defined type. It is important to note the size of this structure will be different on 32-bit and 64-bit platforms due to the difference in the length of the structure members which are pointers.
- When evaluating structure members which are string pointers, such as lpszCertIssuer and lpszCertSubject, you should always check to ensure their values are not NULL (zero) before attempting to dereference them. If those values are undefined, it is not guaranteed they will point to zero-length strings.
- There are additional protocols and ciphers defined which are not listed above because they are considered insecure and are no longer supported. It is important to note that most servers will reject attempts to use unsupported protocols such as SSL 3.0 and will either abort the connection or issue an error message and then immediately disconnect.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

