# Multipurpose Internet Mail Extensions

## Functions

- [MimeAddMessageHeaders](#mimeaddmessageheaders-function)
- [MimeAppendMessageText](#mimeappendmessagetext-function)
- [MimeAttachData](#mimeattachdata-function)
- [MimeAttachFile](#mimeattachfile-function)
- [MimeAttachImage](#mimeattachimage-function)
- [MimeClearMessageText](#mimeclearmessagetext-function)
- [MimeCloseMessageStore](#mimeclosemessagestore-function)
- [MimeCompareMessageText](#mimecomparemessagetext-function)
- [MimeComposeMessage](#mimecomposemessage-function)
- [MimeCopyMessageStore](#mimecopymessagestore-function)
- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeCreateMessageEx](#mimecreatemessageex-function)
- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeCreateMessagePartEx](#mimecreatemessagepartex-function)
- [MimeDecodeText](#mimedecodetext-function)
- [MimeDecodeTextEx](#mimedecodetextex-function)
- [MimeDeleteMessage](#mimedeletemessage-function)
- [MimeDeleteMessageHeader](#mimedeletemessageheader-function)
- [MimeDeleteMessagePart](#mimedeletemessagepart-function)
- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)
- [MimeEncodeText](#mimeencodetext-function)
- [MimeEncodeTextEx](#mimeencodetextex-function)
- [MimeEnumAttachments](#mimeenumattachments-function)
- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeEnumMessageRecipients](#mimeenummessagerecipients-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExportMessageEx](#mimeexportmessageex-function)
- [MimeExtractAllFiles](#mimeextractallfiles-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeExtractFileEx](#mimeextractfileex-function)
- [MimeFindAttachment](#mimefindattachment-function)
- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeFormatDate](#mimeformatdate-function)
- [MimeGetAllHeaders](#mimegetallheaders-function)
- [MimeGetAllRecipients](#mimegetallrecipients-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)
- [MimeGetContentDigest](#mimegetcontentdigest-function)
- [MimeGetContentLength](#mimegetcontentlength-function)
- [MimeGetErrorString](#mimegeterrorstring-function)
- [MimeGetExportOptions](#mimegetexportoptions-function)
- [MimeGetFileContentType](#mimegetfilecontenttype-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetLastError](#mimegetlasterror-function)
- [MimeGetMessageBoundary](#mimegetmessageboundary-function)
- [MimeGetMessageDate](#mimegetmessagedate-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetMessageHeaderEx](#mimegetmessageheaderex-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetMessagePartCount](#mimegetmessagepartcount-function)
- [MimeGetMessageSender](#mimegetmessagesender-function)
- [MimeGetMessageSize](#mimegetmessagesize-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeGetMessageVersion](#mimegetmessageversion-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeGetStoredMessageCount](#mimegetstoredmessagecount-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)
- [MimeInitialize](#mimeinitialize-function)
- [MimeLocalizeText](#mimelocalizetext-function)
- [MimeOpenMessageStore](#mimeopenmessagestore-function)
- [MimeParseAddress](#mimeparseaddress-function)
- [MimeParseBuffer](#mimeparsebuffer-function)
- [MimeParseDate](#mimeparsedate-function)
- [MimeParseHeader](#mimeparseheader-function)
- [MimePurgeMessageStore](#mimepurgemessagestore-function)
- [MimeReplaceStoredMessage](#mimereplacestoredmessage-function)
- [MimeResetMessage](#mimeresetmessage-function)
- [MimeSetExportOptions](#mimesetexportoptions-function)
- [MimeSetFileContentType](#mimesetfilecontenttype-function)
- [MimeSetLastError](#mimesetlasterror-function)
- [MimeSetMessageDate](#mimesetmessagedate-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessageHeaderEx](#mimesetmessageheaderex-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)
- [MimeSetMessageText](#mimesetmessagetext-function)
- [MimeSetMessageVersion](#mimesetmessageversion-function)
- [MimeStoreMessage](#mimestoremessage-function)
- [MimeUninitialize](#mimeuninitialize-function)
## Structures

- [INITDATA](#initdata-structure)

---

## MimeAddMessageHeaders Function

Add one or more header values to the specified message

### Prototype

```c
BOOL WINAPI MimeAddMessageHeaders(
    HMESSAGE hMessage,
    LPCTSTR lpszHeaderList
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeaderList` | `LPCTSTR` | Points to a null-terminated string which specifies one or more header values which should be set for the specified message. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the client handle is invalid, the function returns a value of zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeAddMessageHeaders function enables your application to set one or more header values by providing a list of name/value pairs separated by a colon. Multiple header values may be provided by separating them with a newline character. This function is similar to calling the MimeSetMessageHeader function for each value. When the list of header values is parsed, extraneous whitespace is ignored; however, if the header list contains invalid text (for example, a missing colon separating a header name from its value) the function will fail and an error will be returned.
- This function will only add or update header values in the main header block for the message. It cannot be used to update header values in a specific section of a multipart message. If you need to add or change a header value in a specific part of the message, use the MimeSetMessageHeaderEx function.
- When adding custom, application-specific header values you should always prefix them with "X-" to avoid conflicting with standard headers.

### Example

```cpp
// Define a list of header values which should be included with the
// main header block for the message
LPCTSTR lpszHeaderList = _T("X-App-Sender: someuser@domain.tld\n") \
						_T("X-App-Version: 1.5\n") \
						_T("X-App-Mailer: AppMail (Win64)\n")

if (!MimeAddMessageHeaders(hMessage, lpszHeaderList))
{
	// Unable to add headers for this message
	return;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessageHeaderEx](#mimesetmessageheaderex-function)

---

## MimeAppendMessageText Function

Append text to the body of the current message part

### Prototype

```c
LONG WINAPI MimeAppendMessageText(
    HMESSAGE hMessage,
    LPCTSTR lpszText
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszText` | `LPCTSTR` | A pointer to a string which specifies the text to be appended to the current message part. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the number of bytes copied into the message. A return value of zero indicates that no text could be appended to the current message part. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeClearMessageText](#mimeclearmessagetext-function)
- [MimeCompareMessageText](#mimecomparemessagetext-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeAttachData Function

Attach the contents of a buffer to the specified message

### Prototype

```c
BOOL WINAPI MimeAttachData(
    HMESSAGE hMessage,
    LPBYTE lpBuffer,
    LONG cbBuffer,
    LPCTSTR lpszContentName,
    LPCTSTR lpszContentType,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpBuffer` | `LPBYTE` | Pointer to a byte buffer which contains the data to be attached to the message. This parameter may be NULL, in which case no data is attached, but an additional empty message part will be created. |
| `cbBuffer` | `LONG` | An unsigned integer which specifies the number of bytes of data in the buffer pointed to by the lpBuffer parameter. If the lpBuffer parameter is NULL, this value must be zero. |
| `lpszContentName` | `LPCTSTR` | Pointer to a string which specifies a name for the data being attached to the message. This typically is used as a file name by the mail client to store the data in. If this parameter is NULL or an empty string then no name is defined and the data is attached as inline content. Note that if a file name is specified with a path, only the base name will be used. |
| `lpszContentType` | `LPCTSTR` | Pointer to a string which specifies the type of data being attached. The value must be a valid MIME content type. If this parameter is NULL or an empty string, then the buffer will be examined to determine what kind of data it contains. If there is only text characters, then the content type will be specified as "text/plain". If the buffer contains binary data, then the content type will be specified as "application/octet-stream", which is appropriate for any type of data. |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `MIME_ATTACH_DEFAULT`: The data encoding is based on the content type. Text data is not encoded, and binary data is encoded using the standard base64 encoding algorithm.
    - `MIME_ATTACH_BASE64`: The data is always encoded using the standard base64 algorithm, even if the buffer only contains printable text characters.
    - `MIME_ATTACH_UUCODE`: The data is always encoded using the uuencode algorithm, even if the buffer only contains printable text characters.
    - `MIME_ATTACH_QUOTED`: The data is always encoded using the quoted-printable algorithm. This encoding should only be used if the data contains 8-bit text characters.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeAttachData function allows an application to attach data to the message as either a file attachment or as inline content. The recipient of the message will see the attached data in the same way that they would see a file attached to the message using the MimeAttachFile function.
- If the specified message is not a multipart message, it is marked as multipart and the attached file is appended to the message. If the message is already a multipart message, an additional part is created and the attachment is added to the message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)
- [MimeGetFileContentType](#mimegetfilecontenttype-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeSetFileContentType](#mimesetfilecontenttype-function)

---

## MimeAttachFile Function

Attach a file to the specified message

### Prototype

```c
BOOL WINAPI MimeAttachFile(
    HMESSAGE hMessage,
    LPCTSTR lpszFileName,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string which specifies the name of the file to be attached to the message. |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `MIME_ATTACH_DEFAULT`: The file attachment encoding is based on the file content type. Text files are not encoded, and binary files are encoded using the standard base64 encoding algorithm. This is the default option for file attachments.
    - `MIME_ATTACH_BASE64`: The file attachment is always encoded using the standard base64 algorithm, even if the attached file is a plain text file.
    - `MIME_ATTACH_UUCODE`: The file attachment is always encoded using the uuencode algorithm, even if the attached file is a plain text file.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- If the specified message is not a multipart message, it is marked as multipart and the attached file is appended to the message. If the message is already a multipart message, an additional part is created and the attachment is added to the message.
- To attach data that is stored in a memory buffer rather than a file, use the MimeAttachData function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachData](#mimeattachdata-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)
- [MimeGetFileContentType](#mimegetfilecontenttype-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeSetFileContentType](#mimesetfilecontenttype-function)

---

## MimeAttachImage Function

Attach an inline image to the specified message

### Prototype

```c
BOOL WINAPI MimeAttachImage(
    HMESSAGE hMessage,
    LPCTSTR lpszImageFile,
    LPCTSTR lpszContentId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszImageFile` | `LPCTSTR` | Pointer to a string which specifies the name of the image file to be attached to the message. This parameter cannot be NULL and must specify the name of an existing file. |
| `lpszContentId` | `LPCTSTR` | A pointer to a null-terminated string which specifies the content ID associated with the inline image. If this parameter is NULL, a random content ID string will be automatically generated. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeAttachImage function enables an application to attach an inline image to the message. Unlike a normal file attachment, this function is designed to be used with HTML formatted email messages that display images attached to the message. The lpszContentId parameter specifies the content ID string that is used with the HTML image tag to reference that image.
- If the specified message is not a multipart message, it is marked as multipart and the attached image file is appended to the message. If the message is already a multipart message, an additional part is created and the attachment is added to the message.
- To attach regular files to the message, use the MimeAttachFile function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachData](#mimeattachdata-function)
- [MimeAttachFile](#mimeattachfile-function)
- [MimeComposeMessage](#mimecomposemessage-function)

---

## MimeClearMessageText Function

Clear the body of the current message part

### Prototype

```c
BOOL WINAPI MimeClearMessageText(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeAppendMessageText](#mimeappendmessagetext-function)
- [MimeCompareMessageText](#mimecomparemessagetext-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeCloseMessageStore Function

Close the specified message storage file

### Prototype

```c
BOOL WINAPI MimeCloseMessageStore(
    HMESSAGESTORE hStorage,
    BOOL bPurgeMessages
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `bPurgeMessages` | `BOOL` | An integer value which specifies if deleted messages are purged from the message store. A non-zero value specifies that all messages marked for deletion will be removed from the message store. A value of zero specifies that deleted messages will not be removed from the store. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeCloseMessageStore function closes the storage file that was previously opened, releasing all of the memory allocated for the message store and optionally purging all deleted messages. This function must be called when the application has finished accessing the messages in the message store. Failure to call this function will leave the storage file open and potentially locked by the process.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCopyMessageStore](#mimecopymessagestore-function)
- [MimeOpenMessageStore](#mimeopenmessagestore-function)
- [MimePurgeMessageStore](#mimepurgemessagestore-function)

---

## MimeCompareMessageText Function

Compare text in the body of the current message part

### Prototype

```c
BOOL WINAPI MimeCompareMessageText(
    HMESSAGE hMessage,
    LONG lpnOffset,
    LPCTSTR lpszBuffer,
    LONG cchBuffer,
    BOOL bCaseSensitive
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpnOffset` | `LONG` | Pointer to a long integer which specifies the offset in the message at which to begin the comparison. This value will be updated when the function returns to indicate the offset position in the message where the comparison ended. |
| `lpszBuffer` | `LPCTSTR` | Pointer to a string buffer which contains the text that is to be compared against the body of the message. |
| `cchBuffer` | `LONG` | The number of characters in the buffer that should be compared against the body of the message. |
| `bCaseSensitive` | `BOOL` | Boolean flag which specifies that the comparison should be case sensitive. |

### Return Value

- **Type**: `BOOL`
- If the text buffer matches the contents of the current message body, the function will return a non-zero value, and the lpnOffset argument will be set to position in the buffer where the match terminated. If the text buffer does not match, the function will return a value of zero, and the lpnOffset argument will be set to the position of the first non-matching character. The function will also return zero if one of the arguments is invalid. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAppendMessageText](#mimeappendmessagetext-function)
- [MimeClearMessageText](#mimeclearmessagetext-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeComposeMessage Function

Compose a new message using the specified parameters

### Prototype

```c
HMESSAGE WINAPI MimeComposeMessage(
    LPCTSTR lpszFrom,
    LPCTSTR lpszTo,
    LPCTSTR lpszCc,
    LPCTSTR lpszSubject,
    LPCTSTR lpszMessageText,
    LPCTSTR lpszMessageHTML,
    UINT nCharacterSet,
    UINT nEncodingType
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFrom` | `LPCTSTR` | A pointer to a string which specifies the sender's email address. This parameter may be NULL, in which case no sender address will be included in the message header. |
| `lpszTo` | `LPCTSTR` | A pointer to a string which specifies one or more recipient addresses. If multiple addresses are specified, each address must be separated by a comma. This parameter may be NULL, in which case no recipient addresses will be included in the message header. |
| `lpszCc` | `LPCTSTR` | A pointer to a string which specifies one or more addresses that will receive a copy of the message in addition to the listed recipients. If multiple addresses are specified, each address must be separated by a comma. This parameter may be NULL, in which case no carbon-copy addresses will be included in the message header. |
| `lpszSubject` | `LPCTSTR` | A pointer to a string which specifies the subject of the message. This parameter may be NULL, in which case no subject will be included in the message. |
| `lpszMessageText` | `LPCTSTR` | A pointer to a string which contains the body of the message as plain text. Each line of text contained in the string should be terminated with a carriage-return and linefeed (CRLF) pair, which is recognized as the end-of-line. If this parameter is NULL or points to an empty string, then the message will have an empty body unless the lpszMessageHTML parameter is not NULL. |
| `lpszMessageHTML` | `LPCTSTR` | A pointer to a string which contains the message using HTML formatting. If the lpszMessageText parameter is not NULL, then a multipart message will be created with both plain text and HTML text as the alternative. This allows mail clients to select which message body they wish to display. If the lpszMessageText argument is NULL or points to an empty string, then the message will only contain HTML. Although this is supported, it is not recommended because older mail clients may be unable to display the message correctly. |
| `nCharacterSet` | `UINT` | A numeric identifier which specifies the character set to use when composing the message. A value of zero specifies that the default UTF-8 character set should be used. It is recommended that you always use UTF-8 when composing a new message or creating a new message part. |
| `nEncodingType` | `UINT` | A numeric identifier which specifies the encoding type to use when composing the message. A value of zero specifies that default 7bit encoding should be used. The following values may also be used: |

### Options

- **nEncodingType** Parameter

    - `MIME_ENCODING_7BIT`: Each character is encoded in one or more bytes, with each byte being 8 bits long, with the most significant bit cleared. This encoding is most commonly used with plain text using the US-ASCII character set, where each character is represented by a single byte in the range of 20h to 7Eh.
    - `MIME_ENCODING_8BIT`: Each character is encoded in one or more bytes, with each byte being 8 bits long and all bits are used. 8-bit encoding is typically used with multibyte character sets and is the default encoding used with Unicode text.
    - `MIME_ENCODING_QUOTED`: Quoted-printable encoding is designed for textual messages where most of the characters are represented by the ASCII character set and is generally human-readable. Non-printable characters or 8-bit characters with the high bit set are encoded as hexadecimal values and represented as 7-bit text. Quoted-printable encoding is typically used for messages which use character sets such as ISO-8859-1, as well as those which use HTML.
    - `MIME_ENCODING_BASE64`: Base64 encoding converts binary or text data to ASCII by translating it so each base64 digit represents 6 bits of data. This encoding method is commonly used with messages that contain binary data (such as binary file attachments), or when text uses extended characters that cannot be represented by 7-bit ASCII. It is recommended that you use base64 encoding with Unicode text.

### Return Value

- **Type**: `HMESSAGE`
- If the function succeeds, the return value is a handle to the message. If the function fails, the return value is INVALID_MESSAGE. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeComposeMessage function composes a new message and returns a handle which can be used to further modify or export the message. To create an empty message without any predefined header values, call the MimeCreateMessage function.
- email addresses may be specified as simple addresses, or as commented addresses that include the sender's name or other information. For example, any one of these address formats are acceptable:
- To specify multiple addresses, you should separate each address by a comma or semi-colon. Note that the lpszFrom parameter cannot specify multiple addresses, however it is permitted with the lpszTo, lpszCc and lpszBcc parameters.
- To send a message that contains HTML, it is recommended that you provide both a plain text version of the message body and an HTML formatted version. While it is permitted to send a message that only contains HTML, some older mail clients may not be capable of displaying the message correctly. In some cases, anti-spam software will increase the spam score of messages that do not contain a plain text message body. This can result in your message being rejected or quarantined by the mail server.
- Call the MimeDeleteMessage function to free the memory allocated for this message when it is no longer needed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeDeleteMessage](#mimedeletemessage-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeInitialize](#mimeinitialize-function)

---

## MimeCopyMessageStore Function

Duplicate the contents of the specified message store in a new file

### Prototype

```c
BOOL WINAPI MimeCopyMessageStore(
    HMESSAGESTORE hStorage,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `lpszFileName` | `LPCTSTR` | A pointer to a string which specifies the name of the file that the messages will be copied to. This parameter cannot be NULL and must specify a valid file path and name. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeCopyMessageStore function is used to create a copy of the specified message store in a new file. If the file does not exist, it will be created. If the file already exists, then the contents will be overwritten with the contents of the message store.
- Messages that have been marked for deletion are not copied to the new message store file.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCloseMessageStore](#mimeclosemessagestore-function)
- [MimeOpenMessageStore](#mimeopenmessagestore-function)
- [MimePurgeMessageStore](#mimepurgemessagestore-function)

---

## MimeCreateMessage Function

Create a new empty message

### Prototype

```c
HMESSAGE __cdecl MimeCreateMessage();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `HMESSAGE`
- If the function succeeds, the return value is a handle to the message. If the function fails, the return value is INVALID_MESSAGE. To get extended error information, call MimeGetLastError.

### Remarks

- To create a message with defined values for the sender, recipient, subject and body it is recommended that you use the MimeComposeMessage function.
- Call the MimeDeleteMessage function to free the memory allocated for this message when it is no longer needed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeComposeMessage](#mimecomposemessage-function)
- [MimeCreateMessageEx](#mimecreatemessageex-function)
- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeDeleteMessage](#mimedeletemessage-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeCreateMessageEx Function

Create a new message from the contents of a file or a message in memory

### Prototype

```c
HMESSAGE WINAPI MimeCreateMessageEx(
    DWORD dwImportMode,
    DWORD dwImportOptions,
    LPVOID lpvMessage,
    DWORD dwMessageSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwImportMode` | `DWORD` | An unsigned integer which specifies how the message contents will be imported. It may be one of the following values: |
| `dwImportOptions` | `DWORD` | An unsigned integer which specifies how the message will be imported: |
| `lpvMessage` | `LPVOID` | A pointer to a string, a byte buffer or a global memory handle. The dwImportMode parameter determines how this pointer is used by the function. |
| `dwMessageSize` | `DWORD` | An unsigned integer value which specifies the size of the message to import. This parameter is only used when importing a message from a memory buffer. The message size is determined automatically when the message is imported from a file or the system clipboard. |

### Options

- **dwImportMode** Parameter

    - `MIME_IMPORT_DEFAULT`: The default import mode. If the lpvMessage parameter is NULL, the function will return a handle to an empty message. Otherwise, the lpvMessage parameter is a pointer to a string which specifies the name of a file that contains the message. The dwMessageSize parameter is ignored.
    - `MIME_IMPORT_FILE`: The lpvMessage parameter is a pointer to a string which specifies the name of a file that contains the message. If the file does not exist, or is not a regular text file, an error will occur. The dwMessageSize parameter is ignored.
    - `MIME_IMPORT_CLIPBOARD`: The contents of the message is imported from the system clipboard. The lpvMessage parameter is ignored. The dwMessageSize parameter is ignored.
    - `MIME_IMPORT_MEMORY`: The contents of the message is imported from a local buffer. The lpvMessage parameter must point to a byte array which contains the message to be imported. The dwMessageSize parameter specifies the number of bytes to copy from the buffer. If this value is zero, it is assumed that the end of the message data in the buffer is terminated with a null character and the length is calculated automatically.
    - `MIME_IMPORT_HGLOBAL`: The contents of the message is imported from a global memory buffer. The lpvMessage parameter must be a global memory handle which contains the message. The dwMessageSize parameter specifies the number of bytes to copy from the buffer. If this value is zero, it is assumed that the end of the message data in the buffer is terminated with a null character and the length is calculated automatically.
- **dwImportOptions** Parameter

    - `MIME_OPTION_DEFAULT`: The default import options. Currently this is the only valid value for this parameter and applications should always specify this constant.

### Return Value

- **Type**: `HMESSAGE`
- If the function succeeds, the return value is a handle to the message. If the function fails, the return value is INVALID_MESSAGE. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeCreateMessageEx function creates a new message from the contents of a file or memory, and returns a handle to the parsed message. When the message is no longer needed by the application, it should call MimeDeleteMessage to free the memory allocated for the message and release the handle.
- To create a new message with predefined header values and content, use the MimeComposeMessage function.

### Example

```cpp
// Creates a message using an HGLOBAL handle which references
// a block of memory containing an email message
hMessage = MimeCreateMessageEx(MIME_IMPORT_HGLOBAL,
							MIME_OPTION_DEFAULT,
							(LPVOID)hgblMessage,
							0);

if (hMessage != INVALID_MESSAGE)
{
	// The message has been successfully created
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeComposeMessage](#mimecomposemessage-function)
- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeDeleteMessage](#mimedeletemessage-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeCreateMessagePart Function

Create a new message part for the specified message

### Prototype

```c
INT WINAPI MimeCreateMessagePart(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the new message part number. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeDeleteMessagePart](#mimedeletemessagepart-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetMessagePartCount](#mimegetmessagepartcount-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeCreateMessagePartEx Function

Create a new message part for the specified message

### Prototype

```c
INT WINAPI MimeCreateMessagePartEx(
    HMESSAGE hMessage,
    UINT nCharacterSet,
    UINT nEncodingType,
    LPCTSTR lpszText
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nCharacterSet` | `UINT` | A numeric identifier which specifies the character set to use when composing the message. A value of zero specifies the character set should be the same character set used to initially compose the message. It is recommended that you always use UTF-8 when composing a new message or creating a new message part. |
| `nEncodingType` | `UINT` | A numeric identifier which specifies the encoding type to use when composing the message. A value of zero specifies that default 7bit encoding should be used. The following values may also be used: |
| `lpszText` | `LPCTSTR` | A pointer to a string which specifies the text to be included in the body of the new message part. If this parameter is NULL or points to an empty string, no text is added to the message part. |

### Options

- **nEncodingType** Parameter

    - `MIME_ENCODING_7BIT`: Each character is encoded in one or more bytes, with each byte being 8 bits long, with the most significant bit cleared. This encoding is most commonly used with plain text using the US-ASCII character set, where each character is represented by a single byte in the range of 20h to 7Eh.
    - `MIME_ENCODING_8BIT`: Each character is encoded in one or more bytes, with each byte being 8 bits long and all bits are used. 8-bit encoding is typically used with multibyte character sets and is the default encoding used with Unicode text.
    - `MIME_ENCODING_QUOTED`: Quoted-printable encoding is designed for textual messages where most of the characters are represented by the ASCII character set and is generally human-readable. Non-printable characters or 8-bit characters with the high bit set are encoded as hexadecimal values and represented as 7-bit text. Quoted-printable encoding is typically used for messages which use Latin character sets such as ISO-8859-1, as well as those which use HTML.
    - `MIME_ENCODING_BASE64`: Base64 encoding converts binary or text data to ASCII by translating it so each base64 digit represents 6 bits of data. This encoding method is commonly used with messages that contain binary data (such as binary file attachments), or when text uses extended characters that cannot be represented by 7-bit ASCII. It is recommended that you use base64 encoding with Unicode text.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the new message part number. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeComposeMessage](#mimecomposemessage-function)
- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeDeleteMessagePart](#mimedeletemessagepart-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetMessagePartCount](#mimegetmessagepartcount-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeDecodeText Function

Decode a base64 or quoted-printable encoded string

### Prototype

```c
LONG WINAPI MimeDecodeText(
    UINT nEncodingType,
    LPCTSTR lpszInput,
    LONG cchInput,
    LPTSTR lpszOutput,
    LONG cchOutput
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nEncodingType` | `UINT` | An integer value that specifies the encoding method used. It may be zero or one of the following values: |
| `lpszInput` | `LPCTSTR` | A pointer to a null terminated string which contains the encoded text. This parameter cannot be a NULL pointer. |
| `cchInput` | `LONG` | An integer value which specifies the number of characters of text in the input string which should be decoded. If this value is -1, the entire length of the string up to the terminating null will be decoded. |
| `lpszOutput` | `LPTSTR` | A pointer to a string buffer that will contain the decoded text. This buffer must be large enough to store all of the characters in the decoded text, including the terminating null character. This parameter cannot be NULL. |
| `cchOutput` | `LONG` | An integer value which specifies the maximum number of characters which can be copied into the output string buffer. The buffer must be large enough to store all of the decoded text and terminating null character. This value must be greater than zero. |

### Options

- **nEncodingType** Parameter

    - `MIME_ENCODING_QUOTED`: Quoted-printable encoding is designed for textual messages where most of the characters are represented by the ASCII character set and is generally human-readable. Non-printable characters or 8-bit characters with the high bit set are encoded as hexadecimal values and represented as 7-bit text. Quoted-printable encoding is typically used for messages which use character sets such as ISO-8859-1, as well as those which use HTML.
    - `MIME_ENCODING_BASE64`: Base64 encoding converts binary or text data to ASCII by translating it so each base64 digit represents 6 bits of data. This encoding method is commonly used with messages that contain binary data (such as binary file attachments), or when text uses extended characters that cannot be represented by 7-bit ASCII. It is recommended that you use base64 encoding with Unicode text. This is the default encoding type used by this function.

### Return Value

- **Type**: `LONG`
- If the input buffer can be successfully decoded, the return value is the length of the decoded output string. If the function returns zero, then no text was decoded and the output string buffer will be empty. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function provides a means to decode text that was previously encoded using either base64 or quoted-printable encoding. In most cases, it is not necessary to use this function because the message parser will automatically decode the message text if necessary.
- This function and the MimeEncodeText function use the UTF-8 character set. If the Unicode version of this function is called, the output text will be decoded as UTF-8, then converted to UTF-16 and returned to the caller. If the ANSI version of this function is used, the decoded output will always be returned to the caller using the UTF-8 character set.
- If an unsupported encoding type is specified, this function will return MIME_ERROR and the output text string will be empty. In most cases, it is preferable to use MIME_ENCODING_BASE64 as the encoding method, with quoted-printable encoding only used for legacy support.
- If the original text was encoded using a different character set, use the MimeDecodeTextEx function, which enables you to specify an alternate character set.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeDecodeTextEx](#mimedecodetextex-function)
- [MimeEncodeText](#mimeencodetext-function)
- [MimeEncodeTextEx](#mimeencodetextex-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeLocalizeText](#mimelocalizetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeDecodeTextEx Function

Decode a base64 or quoted-printable encoded string using a specified character set

### Prototype

```c
LONG WINAPI MimeDecodeTextEx(
    UINT nCharacterSet,
    UINT nEncodingType,
    DWORD dwReserved,
    LPCTSTR lpszInput,
    LONG cchInput,
    LPTSTR lpszOutput,
    LONG cchOutput
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCharacterSet` | `UINT` | A numeric identifier which specifies the character set to use when decoding the input text. A value of zero specifies the character set is undefined and no Unicode text conversion is performed when the input string is decoded. If this value does not match the character set used when the text was originally encoded, the resulting output text may be invalid. |
| `nEncodingType` | `UINT` | An integer value that specifies the encoding method used. It may be zero or one of the following values: |
| `dwReserved` | `DWORD` | An integer value reserved for internal use. This value must always be zero. |
| `lpszInput` | `LPCTSTR` | A pointer to a null terminated string which contains the encoded text. This parameter cannot be a NULL pointer. |
| `cchInput` | `LONG` | An integer value which specifies the number of characters of text in the input string which should be decoded. If this value is -1, the entire length of the string up to the terminating null will be decoded. |
| `lpszOutput` | `LPTSTR` | A pointer to a string buffer that will contain the decoded text. This buffer must be large enough to store all of the characters in the decoded text, including the terminating null character. This parameter cannot be NULL. |
| `cchOutput` | `LONG` | An integer value which specifies the maximum number of characters which can be copied into the output string buffer. The buffer must be large enough to store all of the decoded text and terminating null character. This value must be greater than zero. |

### Options

- **nEncodingType** Parameter

    - `MIME_ENCODING_QUOTED`: Quoted-printable encoding is designed for textual messages where most of the characters are represented by the ASCII character set and is generally human-readable. Non-printable characters or 8-bit characters with the high bit set are encoded as hexadecimal values and represented as 7-bit text. Quoted-printable encoding is typically used for messages which use character sets such as ISO-8859-1, as well as those which use HTML.
    - `MIME_ENCODING_BASE64`: Base64 encoding converts binary or text data to ASCII by translating it so each base64 digit represents 6 bits of data. This encoding method is commonly used with messages that contain binary data (such as binary file attachments), or when text uses extended characters that cannot be represented by 7-bit ASCII. It is recommended that you use base64 encoding with Unicode text. This is the default encoding type used by this function.

### Return Value

- **Type**: `LONG`
- If the input buffer can be successfully decoded, the return value is the length of the decoded output string. If the function returns zero, then no text was decoded and the output string buffer will be empty. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function provides a means to decode text that was previously encoded using either base64 or quoted-printable encoding. In most cases, it is not necessary to use this function because the message parser will detect which character set and encoding was used, then automatically decode the message text if necessary.
- The value of the nCharacterSet parameter does not affect the resulting output text, it is only used when decoding the input text. If the Unicode version of this function is called, the output text will be converted to UTF-16 and returned to the caller. If the ANSI version of this function is used, the decoded output will always be returned to the caller using the UTF-8 character set.
- If the nCharacterSet parameter is specified as MIME_CHARSET_UTF16, the encoding type must be MIME_ENCODING_BASE64. Other encoding methods are not supported for Unicode strings and will cause the function to fail. In most cases, it is preferable to use MIME_ENCODING_BASE64 as the encoding method, with quoted-printable encoding only used for legacy support.
- If an unsupported encoding type is specified, this function will return MIME_ERROR and the output text string will be empty. This function cannot be used to decode uuencoded text.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeDecodeText](#mimedecodetext-function)
- [MimeEncodeText](#mimeencodetext-function)
- [MimeEncodeTextEx](#mimeencodetextex-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeLocalizeText](#mimelocalizetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeDeleteMessage Function

Delete the specified message

### Prototype

```c
BOOL WINAPI MimeDeleteMessage(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- Once the message has been deleted, the handle is invalid and should not be referenced.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeDeleteMessagePart](#mimedeletemessagepart-function)
- [MimeResetMessage](#mimeresetmessage-function)

---

## MimeDeleteMessageHeader Function

Delete the specified header field from the message

### Prototype

```c
BOOL WINAPI MimeDeleteMessageHeader(
    HMESSAGE hMessage,
    LPCTSTR lpszHeader
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeader` | `LPCTSTR` | Pointer to a string which specifies the header field that will be deleted. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeDeleteMessagePart Function

Delete the specified message part

### Prototype

```c
BOOL WINAPI MimeDeleteMessagePart(
    HMESSAGE hMessage,
    INT nMessagePart
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nMessagePart` | `INT` | The message part index to delete. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function cannot be used to delete part zero, which is the main body of the message. Instead use the MimeResetMessage function to clear the entire message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeCreateMessagePartEx](#mimecreatemessagepartex-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetMessagePartCount](#mimegetmessagepartcount-function)
- [MimeResetMessage](#mimeresetmessage-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeDeleteStoredMessage Function

Remove the specified message from the message store

### Prototype

```c
BOOL WINAPI MimeDeleteStoredMessage(
    HMESSAGESTORE hStorage,
    LONG nMessageId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `nMessageId` | `LONG` | An integer value which identifies the message that is to be removed from the message store. Message numbers begin at one and increment for each message in the store. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeDeleteStoredMessage function marks the specified message for deletion from the storage file. When the message store is closed or purged, the message is removed from the file. Once a message has been marked for deletion, it may no longer be referenced by the application. For example, you cannot access the contents of a message that has been deleted.
- The message store must be opened with write access. This function will fail if you attempt to delete a message from a storage file that has been opened for read-only access. If the application needs to delete messages in the message store, it is recommended that the file be opened for exclusive access using the MIME_STORAGE_LOCK option.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCloseMessageStore](#mimeclosemessagestore-function)
- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeGetStoredMessageCount](#mimegetstoredmessagecount-function)
- [MimePurgeMessageStore](#mimepurgemessagestore-function)

---

## MimeEncodeText Function

Encode a string using base64 or quoted-printable encoding

### Prototype

```c
LONG WINAPI MimeEncodeText(
    UINT nEncodingType,
    LPCTSTR lpszInput,
    LONG cchInput,
    LPTSTR lpszOutput,
    LONG cchOutput
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nEncodingType` | `UINT` | An integer value that specifies the encoding method used. It may be zero or one of the following values: |
| `lpszInput` | `LPCTSTR` | A pointer to a null terminated string which contains the encoded text. This parameter cannot be a NULL pointer. |
| `cchInput` | `LONG` | An integer value which specifies the number of characters of text in the input string which should be encoded. If this value is -1, the entire length of the string up to the terminating null will be encoded. |
| `lpszOutput` | `LPTSTR` | A pointer to a string buffer that will contain the encoded text. This buffer must be large enough to store all of the characters in the encoded text, including the terminating null character. This parameter cannot be NULL. |
| `cchOutput` | `LONG` | An integer value which specifies the maximum number of characters which can be copied into the output string buffer. The buffer must be large enough to store all of the encoded text and terminating null character. This value must be greater than zero. |

### Options

- **nEncodingType** Parameter

    - `MIME_ENCODING_QUOTED`: Quoted-printable encoding is designed for textual messages where most of the characters are represented by the ASCII character set and is generally human-readable. Non-printable characters or 8-bit characters with the high bit set are encoded as hexadecimal values and represented as 7-bit text. Quoted-printable encoding is typically used for messages which use character sets such as ISO-8859-1, as well as those which use HTML.
    - `MIME_ENCODING_BASE64`: Base64 encoding converts binary or text data to ASCII by translating it so each base64 digit represents 6 bits of data. This encoding method is commonly used with messages that contain binary data (such as binary file attachments), or when text uses extended characters that cannot be represented by 7-bit ASCII. It is recommended that you use base64 encoding with Unicode text. This is the default encoding type used by this function.

### Return Value

- **Type**: `LONG`
- If the input buffer can be successfully encoded, the return value is the length of the encoded output string. If the function returns zero, then no text was encoded and the output string buffer will be empty. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function provides a means to encode text using either base64 or quoted-printable encoding. It is not necessary to use this function to encode text when using the MimeSetMessageText function. The library will automatically encode message text which contains non-ASCII characters using the character set specified when the message is created.
- This function and the MimeDecodeText function use the UTF-8 character set. If the Unicode version of this function is called, the input text will be converted to UTF-8 and then encoded. If the ANSI version of this function is used, the input text is expected to be UTF-8 or ASCII text. If you are encoding text which uses non-ASCII characters, it is recommended that you use the Unicode version of this function.
- If an unsupported encoding type is specified, this function will return MIME_ERROR and the output text string will be empty. This function cannot be used to create uuencoded text. In most cases, it is preferable to use MIME_ENCODING_BASE64 as the encoding method, with quoted-printable encoding only used for legacy support.
- If you want the text to be encoded using a different character set, you can use the MimeEncodeTextEx function. In most cases, your application should use the default UTF-8 character set when encoding and decoding text which contains non-ASCII characters, rather than using specific character sets.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeDecodeText](#mimedecodetext-function)
- [MimeDecodeTextEx](#mimedecodetextex-function)
- [MimeEncodeTextEx](#mimeencodetextex-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeEncodeTextEx Function

Encode a string using base64 or quoted-printable encoding using a specified character set

### Prototype

```c
LONG WINAPI MimeEncodeTextEx(
    UINT nCharacterSet,
    UINT nEncodingType,
    DWORD dwReserved,
    LPCTSTR lpszInput,
    LONG cchInput,
    LPTSTR lpszOutput,
    LONG cchOutput
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCharacterSet` | `UINT` | A numeric identifier which specifies the character set to use when encoding the input text. A value of zero specifies the character set is undefined and no Unicode text conversion is performed when the input string is encoded. |
| `nEncodingType` | `UINT` | An integer value that specifies the encoding method used. It may be one of the following values: |
| `dwReserved` | `DWORD` | An integer value reserved for internal use. This value must always be zero. |
| `lpszInput` | `LPCTSTR` | A pointer to a null terminated string which contains the encoded text. This parameter cannot be a NULL pointer. |
| `cchInput` | `LONG` | An integer value which specifies the number of characters of text in the input string which should be encoded. If this value is -1, the entire length of the string up to the terminating null will be encoded. |
| `lpszOutput` | `LPTSTR` | A pointer to a string buffer that will contain the encoded text. This buffer must be large enough to store all of the characters in the encoded text, including the terminating null character. This parameter cannot be NULL. |
| `cchOutput` | `LONG` | An integer value which specifies the maximum number of characters which can be copied into the output string buffer. The buffer must be large enough to store all of the encoded text and terminating null character. This value must be greater than zero. |

### Options

- **nEncodingType** Parameter

    - `MIME_ENCODING_QUOTED`: Quoted-printable encoding is designed for textual messages where most of the characters are represented by the ASCII character set and is generally human-readable. Non-printable characters or 8-bit characters with the high bit set are encoded as hexadecimal values and represented as 7-bit text. Quoted-printable encoding is typically used for messages which use character sets such as ISO-8859-1, as well as those which use HTML.
    - `MIME_ENCODING_BASE64`: Base64 encoding converts binary or text data to ASCII by translating it so each base64 digit represents 6 bits of data. This encoding method is commonly used with messages that contain binary data (such as binary file attachments), or when text uses extended characters that cannot be represented by 7-bit ASCII. It is recommended that you use base64 encoding with Unicode text.

### Return Value

- **Type**: `LONG`
- If the input buffer can be successfully encoded, the return value is the length of the encoded output string. If the function returns zero, then no text was encoded and the output string buffer will be empty. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function provides a means to encode text using either base64 or quoted-printable encoding. It is not necessary to use this function to encode text when using the MimeSetMessageText function. The library will automatically encode message text which contains non-ASCII characters using the character set specified when the message is created.
- If the nCharacterSet parameter parameter is non-zero, the function will encode the text using the specified character set. If the Unicode version of this function is called, the input text is converted to ANSI using the code page associated with the character set. If the ANSI version of this function is called, the input text is converted to Unicode using the system default code page, and then back to ANSI using the specified character set.
- If the nCharacterSet parameter specifies the MIME_CHARSET_UTF16 character set, you must specify MIME_ENCODING_BASE64 as the encoding method. Other encoding methods are not supported for Unicode strings and will cause the function to fail. It is not recommended you encode text as UTF-16 unless there is a specific requirement to use that character set.
- It is recommended that you use the MIME_CHARSET_UTF8 character set whenever possible. It is capable of encoding all Unicode code points, and is a standard for virtually all modern Internet applications. In most cases, it is preferable to use MIME_ENCODING_BASE64 as the encoding method, with quoted-printable encoding only used for legacy support.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeDecodeText](#mimedecodetext-function)
- [MimeEncodeText](#mimeencodetext-function)
- [MimeEncodeTextEx](#mimeencodetextex-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeEnumAttachments Function

Enumerate all file attachments in the specified message

### Prototype

```c
INT WINAPI MimeEnumAttachments(
    HMESSAGE hMessage,
    LPCTSTR lpFileNames,
    INT nMaxFiles
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpFileNames` | `LPCTSTR` | A pointer to a an array of null-terminated strings that contain the names of the files attached to the message. If this parameter is NULL, the function will only return the number of files attached to the message. |
| `nMaxFiles` | `INT` | An integer value that specifies the maximum size of the array of string pointers specified by the lpFileNames parameter. If this value is zero, the lpFileNames parameter is ignored and the function will only return the number of files attached to the message. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of files attached to the message. If the message does not contain any file attachments, this function will return a value of zero. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function can be used to obtain the names of the files attached to the message. If this function is used with languages other than C/C++, it is important to note that the lpFileNames parameter is not a pointer to a string, but rather a pointer to an array of pointers to null-terminated strings. It is not recommended that you use this function with languages that do not support C-style arrays. For example, Visual Basic represents arrays as SAFEARRAY structures and is not compatible with this function.

### Example

```cpp
LPCTSTR lpszFiles[MAXFILES];

INT nFiles = MimeEnumAttachments(hMessage, lpszFiles, MAXFILES);
if (nFiles == MIME_ERROR)
{
	DWORD dwError = MimeGetLastError();
	_tprintf(_T("Unable to enumerate attachments, error 0x%08lx\n"), dwError);
	return;
}    

_tprintf(_T("There are %d files attached to the message\n"), nFiles);
for (INT nIndex = 0; nIndex < nFiles; nIndex++)
{
	BOOL bResult = MimeExtractFileEx(hMessage, -1, lpszFiles[nIndex], NULL, 0);
	if (bResult)
		_tprintf(_T("%d: extracted attachment \"%s\"\n"), nIndex, lpszFiles[nIndex]);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractAllFiles](#mimeextractallfiles-function)
- [MimeExtractFileEx](#mimeextractfileex-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeEnumMessageHeaders Function

Enumerate all header fields in the current message part

### Prototype

```c
INT WINAPI MimeEnumMessageHeaders(
    HMESSAGE hMessage,
    LPCTSTR lpHeaderList,
    INT nMaxHeaders
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpHeaderList` | `LPCTSTR` | Pointer to an array of pointers to null terminated header field names. If this parameter is NULL, the function only returns the number of headers in the current message part. |
| `nMaxHeaders` | `INT` | The maximum number of header fields which may be returned in the lpHeaderList parameter. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the total number of headers that are defined in the current message part. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The values returned in the header list array must not be directly modified by the application. There is no specific order in which the header fields are enumerated by this function. The header fields from an imported message may not be returned in the same order as which they appear in the message. An application should never make an assumption about the order in which one or more header fields are defined.

### Example

```cpp
// Determine the total number of headers in the current
// message part

nHeaders = MimeEnumMessageHeaders(hMessage, NULL, 0);
if (nHeaders > 0)
{

	// Allocate memory for the list of headers

	lpHeaderList = (LPCTSTR *)LocalAlloc(LPTR, nHeaders * sizeof(LPCTSTR));
	assert(lpHeaderList != NULL);

	// Retrieve the list of headers in the current
	// message part, and get their values

	MimeEnumMessageHeaders(hMessage, lpHeaderList, nHeaders);
	for (nIndex = 0; nIndex < nHeaders; nIndex++)
	{

		LPCTSTR lpszValue;
		lpszValue = MimeGetMessageHeader(hMessage, lpHeaderList[nIndex]);
		assert(lpszValue != NULL);

		printf("%s: %s\n", lpHeaderList[nIndex], lpszValue);

	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeEnumMessageRecipients Function

Enumerate addresses of all message recipients

### Prototype

```c
INT WINAPI MimeEnumMessageRecipients(
    HMESSAGE hMessage,
    LPCTSTR lpszExtraAddress,
    LPTSTR lpBuffer,
    LPDWORD lpcchBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszExtraAddress` | `LPCTSTR` | A pointer to a string which contains one or more additional email addresses that should be included in the list, in addition to those found in the message. If more than one address is specified, each address should be separated by a comma. This parameter may be NULL if there are no extra addresses to include in the recipient list. |
| `lpBuffer` | `LPTSTR` | Pointer to buffer which will contain zero or more null-terminated strings. The end of the string list is indicated by an additional terminating null. If this parameter is NULL, the function will calculate the minimum number of bytes required to store the addresses and return the value in the lpcbBuffer parameter. |
| `lpcchBuffer` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of characters that can be copied into the buffer specified by the lpBuffer parameter. When the function returns, it will be updated to contain the actual number of characters copied into the buffer. If the lpBuffer parameter is NULL, then this value will contain the minimum number of characters required to store all of the recipient addresses in the current message. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the total number of recipients for the current message. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeEnumMessageRecipients function returns a list of recipient email addresses for the specified message, with each address being terminated by a null character. The end of the list is indicated by an additional null character. To determine the size of the buffer you should pass to this function, you can specify the lpBuffer parameter as NULL and initialize the value of the lpcchBuffer parameter to zero.
- This function is primarily designed for use with C/C++ and languages that can easily use C-style strings and pointers to string values. In many cases, it may be preferable to use the MimeGetAllRecipients function which returns a comma-separated list of recipient addresses in a string buffer.
- If you are using the Unicode version of this function, keep in mind that the lpcchBuffer parameter returns the number of characters in the output buffer, not the number of bytes. If you are dynamically allocating the buffer, make sure you allocate enough memory to store Unicode strings, which are two bytes per character.

### Example

```cpp
LPTSTR lpRecipients = NULL;
DWORD cchRecipients = 0;
INT nRecipients = 0;

// Determine the number of characters that should be allocated to store
// all of the recipient addresses in the current message

nRecipients = MimeEnumMessageRecipients(hMessage,
										NULL,
										NULL,
										&cchRecipients);

// Allocate the memory for the string buffer that will contain all
// of the recipient addresses and call MimeEnumMessageRecipients
// again to store those addresses in the buffer

if (nRecipients > 0 && cchRecipients > 0)
{
	lpRecipients = (LPTSTR)LocalAlloc(LPTR, cchRecipients * sizeof(TCHAR));
	if (lpRecipients == NULL)
		return; // Virtual memory exhausted

	nRecipients = MimeEnumMessageRecipients(hMessage,
											NULL,
											lpRecipients,
											&cchRecipients);
}

// Move through the buffer, processing each recipient address
// that was returned

if (nRecipients > 0)
{
	LPTSTR lpszAddress = lpRecipients;
	INT cchAddress;

	while (lpszAddress != NULL)
	{
		if ((cchAddress = lstrlen(lpszAddress)) == 0)
			break;

		// lpszAddress specifies a recipient address
		// Advance to the next address string in the buffer
		lpszAddress += cchAddress + 1;
	}
}

if (lpRecipients)
{
	LocalFree((HLOCAL)lpRecipients);
	lpRecipients = NULL;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetAllRecipients](#mimegetallrecipients-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeExportMessage Function

Export the current message to a text file

### Prototype

```c
BOOL WINAPI MimeExportMessage(
    HMESSAGE hMessage,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string which specifies the name of the file to be created from the message. If this parameter is NULL, the contents of the message will be copied to the system clipboard. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function is provided for backwards compatibility with previous versions of the API. To export the contents of the message to a buffer in memory, use the MimeExportMessageEx function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeExportMessageEx](#mimeexportmessageex-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)
- [MimeSetExportOptions](#mimesetexportoptions-function)

---

## MimeExportMessageEx Function

Export the current message to a file, clipboard or memory buffer

### Prototype

```c
BOOL WINAPI MimeExportMessageEx(
    HMESSAGE hMessage,
    DWORD dwExportMode,
    DWORD dwExportOptions,
    LPVOID lpvMessage,
    LPDWORD lpdwMessageSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `dwExportMode` | `DWORD` | An unsigned integer which specifies how the message contents will be exported. It may be one of the following values: |
| `dwExportOptions` | `DWORD` | An unsigned integer which specifies how the message will be exported. The following values may be combined using a bitwise Or operator: |
| `lpvMessage` | `LPVOID` | A pointer to a string, a byte buffer or a global memory handle. The dwExportMode parameter determines how this pointer is used by the function. |
| `lpdwMessageSize` | `LPDWORD` | A pointer to an unsigned integer value which will contain the size of the message when the function exits. This parameter may be NULL if you do not require this information, except if the dwExportMode parameter is MIME_EXPORT_MEMORY. In this case, the parameter must point to a value initialized with the maximum size of the byte buffer that has been passed to the function. |

### Options

- **dwExportMode** Parameter

    - `MIME_EXPORT_DEFAULT`: The default export mode. If the lpvMessage parameter is NULL, then the contents of the message will be copied to the system clipboard. Otherwise, the lpvMessage parameter is a pointer to a string which specifies the name of a file to store the message in.
    - `MIME_EXPORT_FILE`: The lpvMessage parameter is a pointer to a null-terminated string which specifies the name of a file to store the message in. If the file does not exist, it will be created. Otherwise the file will be overwritten with the contents of the message.
    - `MIME_EXPORT_CLIPBOARD`: The contents of the message is copied to the system clipboard. The lpvMessage parameter is ignored.
    - `MIME_EXPORT_MEMORY`: The contents of the message is copied to a local buffer. The lpvMessage parameter must point to a byte array which will contain the message contents when the function returns. The lpdwMessageSize parameter must be initialized to the maximum size of the buffer and will contain the number of bytes copied to the buffer when the function returns.
    - `MIME_EXPORT_HGLOBAL`: The contents of the message is stored in a global memory buffer. The lpvMessage parameter must point to a global memory handle which will reference the message buffer when the function returns. The lpdwMessageSize parameter will contain the number of bytes allocated for the message. The client application is responsible for releasing the memory handle when the message contents are no longer needed.
- **dwExportOptions** Parameter

    - `MIME_OPTION_DEFAULT`: The default export options. The headers for the message are written out in a specific consistent order, with custom headers written to the end of the header block regardless of the order in which they were set or imported from another message. If the message contains Bcc, Received, Return-Path, Status or X400-Received header fields, they will not be exported.
    - `MIME_OPTION_KEEPORDER`: The original order in which the message header fields were set or imported are preserved when the message is exported.
    - `MIME_OPTION_ALLHEADERS`: All headers, including the Received, Return-Path, Status and X400-Received header fields will be exported. Normally these headers are not exported because they are only used by the mail transport system. This option can be useful when exporting a message to be stored on the local system, but should not be used when exporting a message to be delivered to another user.
    - `MIME_OPTION_NOHEADERS`: When exporting a message, the main header block will not be included. This can be useful when creating a multipart message for services which expect MIME formatted data, such as HTTP POST requests. This option should never be used for email messages being submitted using SMTP.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- If the MIME_EXPORT_MEMORY mode is used to export the message to a pre-allocated buffer, the lpdwMessageSize parameter must be initialized to the size of the buffer before being passed to the function by reference. If this value is not properly initialized by the application, it can result in an error or a general protection exception. If the function succeeds, the message will be copied to the buffer. If the function fails, the previous contents of the buffer will not be preserved. It is not guaranteed that the buffer will be terminated with a null byte. If the buffer is not large enough to store the entire message, the function will fail. The MimeGetMessageSize function can be used to determine the minimum size of the buffer required to store the message.
- If the MIME_EXPORT_HGLOBAL mode is used, a global memory handle will be returned to the caller which contains the message contents. This handle must be dereferenced using the GlobalLock function. No changes should be made to this copy of the message. If you wish to modify the contents of the message, allocate a local buffer and copy the message contents, or use the MIME_EXPORT_MEMORY option instead. Your application is responsible for calling GlobalUnlock and GlobalFree to unlock and free the handle when it is no longer needed.

### Example

```cpp
// Exports the message contents to a global buffer
HGLOBAL hgblMessage = NULL;
DWORD dwMessageSize = 0;

bResult = MimeExportMessageEx(hMessage,
							MIME_EXPORT_HGLOBAL,
							MIME_OPTION_DEFAULT,
							&hgblMessage,
							&dwMessageSize);

if (bResult)
{
	LPBYTE lpMessage = (LPBYTE)GlobalLock(hgblMessage);

	if (lpMessage != NULL)
	{
		// Process the contents of the message
	}

	GlobalUnlock(hgblMessage);
	GlobalFree(hgblMessage);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeGetMessageSize](#mimegetmessagesize-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)
- [MimeSetExportOptions](#mimesetexportoptions-function)

---

## MimeExtractAllFiles Function

Extract all file attachments in the message and store them in the specified directory

### Prototype

```c
INT WINAPI MimeExtractAllFiles(
    HMESSAGE hMessage,
    LPCTSTR lpszDirectory,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszDirectory` | `LPCTSTR` | A pointer to a string which specifies the name of the directory where the file attachments should be stored. If this parameter is NULL or points to an empty string, the attached files will be stored in the current working directory on the local system. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This parameter must always have a value of zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of file attachments which were extracted from the message. If the message does not contain any file attachments, this function will return a value of zero. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function will extract all of the files that are attached to the message and store them in the specified directory. The directory must exist and the current user must have the appropriate permissions to create files there. If a file with the same name as the attachment already exists, it will be overwritten with the contents of the attachment. If the file attachment was encoded using base64 or uuencode, this function will automatically decode the contents of the attachment.
- To determine the file names for each of the attachments in a message, use the MimeEnumAttachments function. To store a file attachment on the local system using a name that is different than the file name of the attachment, use the MimeExtractFile function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeEnumAttachments](#mimeenumattachments-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractFileEx](#mimeextractfileex-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeExtractFile Function

Extract the file attachment from the current message part

### Prototype

```c
BOOL WINAPI MimeExtractFile(
    HMESSAGE hMessage,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPCTSTR` | A pointer to a string which specifies the name of a file on the local system. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function will store the contents of a file attachment in the current message part to the specified file on the local system. If a path is specified as part of the file name, it must exist and the current user must have the appropriate permissions to create the file. If a file with the same name already exists, it will be overwritten with the contents of the attachment. If the file attachment was encoded using base64 or uuencode, this function will automatically decode the contents of the attachment.
- To determine if the current message part contains a file attachment, use the MimeGetAttachedFileName function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractFileEx](#mimeextractfileex-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeExtractFileEx Function

Extract a file attachment from the message with additional options

### Prototype

```c
BOOL WINAPI MimeExtractFileEx(
    HMESSAGE hMessage,
    INT nMessagePart,
    LPCTSTR lpszAttachment,
    LPCTSTR lpszFileName,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nMessagePart` | `INT` | An integer value that specifies the message part that contains the file attachment. This value may be -1, in which case the current message part will be used, or the function will search for an attachment with a file name that matches the value of the lpszAttachment parameter. |
| `lpszAttachment` | `LPCTSTR` | A pointer to a string that specifies the file name for the attachment in the message. If the file name of the attachment is not known, this parameter can be NULL or point to an empty string. This parameter is ignored if the nMessagePart parameter has a value greater than -1. |
| `lpszFileName` | `LPCTSTR` | A pointer to a string that specifies the name of a file on the local system. If this parameter is NULL or points to an empty string, the value of the lpszAttachment parameter will specify the name of the file in the current working directory. If both the lpszAttachment and lpszFileName parameters are NULL, the function will fail. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This parameter must always have a value of zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function will store the contents of a file attachment in the current message part to the specified file on the local system. If a path is specified as part of the file name, it must exist and the current user must have the appropriate permissions to create the file. If a file with the same name already exists, it will be overwritten with the contents of the attachment. If the file attachment was encoded using base64 or uuencode, this function will automatically decode the contents of the attachment.
- If the nMessagePart parameter has a value of -1 and the lpszAttachment parameter specifies a file name, this function will search the entire message for an attachment with the same file name. The search is not case-sensitive, however it must match the attachment file name completely. This function will not match partial file names or names that include wildcard characters. If a match is found, the contents of that attachment will be stored in the file specified by the lpszFileName parameter.
- If the nMessagePart parameter has a value of -1 and the lpszAttachment parameter is NULL or points to an empty string, then the attachment in the current message part will be stored in the specified file. If the current message part does not contain a file attachment, this function will fail. Calling this function in this manner is effectively the same as calling the MimeExtractFile function.
- To extract all of the files attached to a message in a single function call, use the MimeExtractAllFiles function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeEnumAttachments](#mimeenumattachments-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractAllFiles](#mimeextractallfiles-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeFindAttachment](#mimefindattachment-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeFindAttachment Function

Search for a file attachment in the specified message

### Prototype

```c
INT WINAPI MimeFindAttachment(
    HMESSAGE hMessage,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string that specifies the name of the file attachment to search for. This parameter should only specify a base file name; it should not include a file path and cannot be NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the message part number that contains an attachment that matches the specified file name. If the message does not contain an attachment with the specified file name, the function will return MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function will search the message for a attachment that matches the specified file name. The search is not case-sensitive, however it must match the attachment file name completely. This function will not match partial file names or names that include wildcard characters. To obtain a list of all of the files attached to a message, use the MimeEnumAttachments function.

### Example

```cpp
// The name of the file attachment to search for
LPCTSTR lpszFileName = _T("MyProject.docx");

// Search for the attached file and store it on the local system
INT nMessagePart = MimeFindAttachment(hMessage, lpszFileName);
if (nMessagePart != MIME_ERROR)
{
	MimeSetMessagePart(hMessage, nMessagePart);

	if (MimeExtractFile(hMessage, lpszFileName) != MIME_ERROR)
		_tprintf(_T("Saved file attachment %s\n"), lpszFileName);
	else
	{
		_tprintf(_T("Unable to save file attachment %s\n"), lpszFileName);
		return;
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeExtractAllFiles](#mimeextractallfiles-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeExtractFileEx](#mimeextractfileex-function)
- [MimeGetAttachedFileName](#mimegetattachedfilename-function)

---

## MimeFindStoredMessage Function

Search for a message in the specified message store

### Prototype

```c
LONG WINAPI MimeFindStoredMessage(
    HMESSAGESTORE hStorage,
    LONG nMessageId,
    LPCTSTR lpszHeaderName,
    LPCTSTR lpszHeaderValue,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `nMessageId` | `LONG` | An integer value which specifies the message number that should be used when starting the search. The first message in the message store has a value of one. |
| `lpszHeaderName` | `LPCTSTR` | A pointer to the string which specifies the name of the header field that should be searched. The header field name is not case sensitive. This parameter cannot be NULL. |
| `lpszHeaderValue` | `LPCTSTR` | A pointer to the string which specifies the header value that should be searched for. The search options can be used to specify if the search is case-sensitive, and whether the search should return partial matches to the string. This parameter cannot be NULL. |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `MIME_SEARCH_DEFAULT`: Perform a complete match against the specified header value. The comparison is not case-sensitive.
    - `MIME_SEARCH_CASE_SENSITIVE`: The header field value comparison will be case-sensitive. Note that this does not affect header field names. Matches for header names are always case-insensitive.
    - `MIME_SEARCH_PARTIAL_MATCH`: Perform a partial match against the specified header value. It recommended that this option be used when searching for matches to email addresses.
    - `MIME_SEARCH_DECODE_HEADERS`: Decode any encoded message headers before comparing them to the specified value. This option can increase the amount of time required to search the message store and should only be used when necessary.

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the number for the message which matches the search criteria. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeFindStoredMessage function is used to search the message store for a message which matches a specific header field value. For example, it can be used to find every message which is addressed to a specific recipient or has a subject which matches a particular string value.

### Example

```cpp
HMESSAGE hMessage = INVALID_MESSAGE;
LPCTSTR lpszHeader = _T("From");
LPCTSTR lpszAddress = _T("jsmith@example.com");
LONG nMessageId = 1; 

// Begin searching for messages from the specified sender
while (nMessageId != MIME_ERROR)
{
	nMessageId = MimeFindStoredMessage(hStorage,
									nMessageId,
									lpszHeader,
									lpszAddress,
									MIME_SEARCH_PARTIAL_MATCH);

	if (nMessageId != MIME_ERROR)
	{
		// Get a handle to the message that was found
		hMessage = MimeGetStoredMessage(hStorage, nMessageId, 0);

		if (hMessage != INVALID_MESSAGE)
		{
			// Store the message in a file
			TCHAR szFileName[MAX_PATH];
			BOOL bExported;

			// Create a filename based on the message number
			wsprintf(szFileName, _T("msg%05ld.tmp"), nMessageId);

			// Export the message to a file
			bExported = MimeExportMessage(hMessage, szFileName);
		}

		// Increase the message ID to resume the search at the next message
		nMessageId++;
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeGetStoredMessageCount](#mimegetstoredmessagecount-function)
- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)

---

## MimeFormatDate Function

Return a standard RFC 822 formatted date string

### Prototype

```c
LPTSTR WINAPI MimeFormatDate(
    LONG nSeconds,
    LONG nTimezone,
    LPTSTR lpszDate,
    INT cchMaxDate
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nSeconds` | `LONG` | A long integer which specifies the number of seconds since 1 January 1970 00:00:00 UTC. This date is commonly called the epoch, and is the base date used by the standard C time functions. If the value of this parameter is zero, the current date and time is used. |
| `nTimezone` | `LONG` | A pointer to a long integer which is set to the difference in seconds between the specified date's timezone and Coordinated Universal Time. A value of zero specifies Coordinated Universal Time (UTC), while a positive value specifies a timezone west of UTC and a negative value specifies a timezone east of UTC. For example, Eastern Standard Time would be specified as 18000 and Pacific Standard Time would be 28800. |
| `lpszDate` | `LPTSTR` | A pointer to a string buffer which will contain the formatted date. This parameter cannot be a NULL pointer. |
| `cchMaxDate` | `INT` | The maximum number of characters, including the terminating null character, which may be copied into the date string buffer. |

### Return Value

- **Type**: `LPTSTR`
- If the function succeeds, a pointer to the date string buffer is returned. If the function fails, a NULL pointer is returned. To get extended error information, call MimeGetLastError.

### Remarks

- The date string is returned in a standard format as outlined in RFC 822, the document which describes the basic structure of Internet email messages. This format is as follows:
- www, dd mmm yyyy hh:mm:ss [-]zzzz
- Each part of the date string is defined as follows:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeParseDate](#mimeparsedate-function)
- [MimeGetMessageDate](#mimegetmessagedate-function)
- [MimeSetMessageDate](#mimesetmessagedate-function)

---

## MimeGetAllHeaders Function

Return the complete RFC 822 header values in a string buffer

### Prototype

```c
INT WINAPI MimeGetAllHeaders(
    HMESSAGE hMessage,
    LPTSTR lpszHeaders,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeaders` | `LPTSTR` | Pointer to string buffer which will contain the header values for the specified message. This parameter may be NULL, in which case the function will calculate the number of characters needed to store the complete header block. |
| `nMaxLength` | `INT` | An integer value which specifies the maximum number of characters that can be stored in the lpszHeaders string. If the lpszHeaders parameter is NULL, this value must be zero. If the lpszHeaders parameter is not NULL, this value must be large enough to store the entire list of addresses. |

### Return Value

- **Type**: `INT`
- If the function succeeds and the lpszHeaders parameter is NULL, the return value is the minimum number of characters that should be allocated to store all of the header values, including the terminating null character. If the lpszHeaders parameter is not NULL, then the return value is the number of characters copied into the string, not including the terminating null character. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeGetAllHeaders function will return all of the RFC 822 header values in a string buffer. This includes the message headers that are most commonly referred to, such as the To, From and Subject headers. Each header and its value are separated by a colon, and terminated with a carriage return and linefeed (CRLF) pair.
- The headers and their values returned by this function will not be identical to the header block in the original message. If a header value is split across multiple lines, this function will fold the text, returning the complete header value on a single line of text and removing any extraneous whitespace. If the header value has been encoded by the mail client, this function will return the decoded value, not the original encoded value.

### Example

```cpp
LPTSTR lpszHeaders = NULL;
INT nLength;

// Determine the number of characters that should be allocated to store
// the RFC822 headers

nLength = MimeGetAllHeaders(hMessage, NULL, 0);

// Allocate the memory for the string buffer that is large enough and
// call MimeGetAllHeaders again

if (nLength > 0)
{
	lpszHeaders = (LPTSTR)LocalAlloc(LPTR, nLength * sizeof(TCHAR));
	if (lpszHeaders == NULL)
		return; // Virtual memory exhausted

	nLength = MimeGetAllHeaders(hMessage, lpszHeaders, nLength);  
}

// The lpszHeaders string now contains all of the RFC822 headers for the
// message, with each header terminated by a CRLF sequence

if (lpszHeaders != NULL)
{
	LocalFree((HLOCAL)lpszHeaders);
	lpszHeaders = NULL;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeGetAllRecipients Function

Return a comma-separated list of recipient addresses in a string buffer

### Prototype

```c
INT WINAPI MimeGetAllRecipients(
    HMESSAGE hMessage,
    LPCTSTR lpszExtraAddress,
    LPTSTR lpszRecipients,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszExtraAddress` | `LPCTSTR` | A pointer to a string which contains one or more additional email addresses that should be included in the list, in addition to those found in the message. If more than one address is specified, each address should be separated by a comma. This parameter may be NULL if there are no extra addresses to include in the recipient list. |
| `lpszRecipients` | `LPTSTR` | Pointer to string buffer which will contain a comma-separated list of email addresses when the function returns. This parameter may be NULL, in which case the function will calculate the number of characters needed to store the complete list. |
| `nMaxLength` | `INT` | An integer value which specifies the maximum number of characters that can be stored in the lpszRecipients string. If the lpszRecipients parameter is NULL, this value must be zero. If the lpszRecipients parameter is not NULL, this value must be large enough to store the entire list of addresses. |

### Return Value

- **Type**: `INT`
- If the function succeeds and the lpszRecipients parameter is NULL, the return value is the minimum number of characters that should be allocated to store the list recipient addresses, including the terminating null character. If the lpszRecipients parameter is not NULL, then the return value is the number of characters copied into the buffer, not including the terminating null character. If the function returns a value of zero, then the specified message has no recipients. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeGetAllRecipients function is useful for creating a list of message recipients that can be passed to functions like SmtpSendMessage. If you wish to dynamically allocate the string buffer that will contain the list of recipients, then the lpszRecipients parameter should be NULL and the nMaxLength parameter should have a value of zero. The function will then return the recommended size of the buffer that should be allocated. This value is guaranteed to be large enough to store the entire list of message recipients, including the terminating null character.

### Example

```cpp
LPTSTR lpszRecipients = NULL;
INT nLength;

// Determine the number of characters that should be allocated to store
// a list of the recipient addresses in the message

nLength = MimeGetAllRecipients(hMessage, NULL, NULL, 0);

// Allocate the memory for the string buffer that is large enough and
// call MimeGetAllRecipients again

if (nLength > 0)
{
	lpszRecipients = (LPTSTR)LocalAlloc(LPTR, nLength * sizeof(TCHAR));
	if (lpRecipients == NULL)
		return; // Virtual memory exhausted

	nLength = MimeGetAllRecipients(hMessage, NULL, lpszRecipients, nLength);  
}

// The lpszRecipients string now contains a comma-separated list of
// each recipient address in the message

if (lpszRecipients != NULL)
{
	LocalFree((HLOCAL)lpszRecipients);
	lpszRecipients = NULL;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageRecipients](#mimeenummessagerecipients-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeGetAttachedFileName Function

Return the name of the file attachment for the current part

### Prototype

```c
BOOL WINAPI MimeGetAttachedFileName(
    HMESSAGE hMessage,
    LPTSTR lpszFileName,
    INT cchFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPTSTR` | Pointer to a buffer that will contain the current attachment file name as a string. |
| `cchFileName` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The function will first try to get the filename from the Content-Disposition header field. If this field does not exist, it then attempts to get the name from the Content-Type header field. If neither field exists, the function will fail.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeEnumAttachments](#mimeenumattachments-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeFindAttachment](#mimefindattachment-function)

---

## MimeGetContentDigest Function

Return encoded digest of message's content

### Prototype

```c
INT WINAPI MimeGetContentDigest(
    HMESSAGE hMessage,
    LPTSTR lpszDigest,
    INT cchDigest
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Message handle. |
| `lpszDigest` | `LPTSTR` | Pointer to a string buffer to contain the MD5 digest for the specified message. |
| `cchDigest` | `INT` | Maximum length of the digest string, in bytes. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the message digest string. A value of zero specifies that there is no MD5 digest for the current message. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function returns the value of the Content-MD5 header field in the main body of the message. If the header exists, it contains the MD5 digest for the message as defined in RFC 1864.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

---

## MimeGetContentLength Function

Return the length of the current message part content

### Prototype

```c
LONG WINAPI MimeGetContentLength(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the content length. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function will return the size of the content in the current message part as the number of bytes and does not account for any Unicode conversion of text. Exercise caution when using this function to determine the size of the buffer that should be allocated for a function like MimeGetMessageText. You should always allocate enough memory to accommodate any potential text conversion and decoding which may occur.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI MimeGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetLastError](#mimegetlasterror-function)
- [MimeSetLastError](#mimesetlasterror-function)

---

## MimeGetExportOptions Function

Return a bitmask that describes current message export options

### Prototype

```c
BOOL WINAPI MimeGetExportOptions(
    HMESSAGE hMessage,
    LPDWORD lpdwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpdwOptions` | `LPDWORD` | Pointer to mask of attribute options. The mask is a combination of the following values: |

### Options

- **lpdwOptions** Parameter

    - `MIME_OPTION_DEFAULT`: The default export options. The headers for the message are written out in a specific consistent order, with custom headers written to the end of the header block regardless of the order in which they were set or imported from another message. If the message contains Bcc, Received, Return-Path, Status or X400-Received header fields, they will not be exported.
    - `MIME_OPTION_KEEPORDER`: The original order in which the message header fields were set or imported are preserved when the message is exported.
    - `MIME_OPTION_ALLHEADERS`: All headers, including the Bcc, Received, Return-Path, Status and X400-Received header fields will be exported. Normally these headers are not exported because they are only used by the mail transport system. This option can be useful when exporting a message to be stored on the local system, but should not be used when exporting a message to be delivered to another user.
    - `MIME_OPTION_NOHEADERS`: When exporting a message, the main header block will not be included. This can be useful when creating a multipart message for services which expect MIME formatted data, such as HTTP POST requests. This option should never be used for email messages being submitted using SMTP.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is 0. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeExportMessage](#mimeexportmessage-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeSetExportOptions](#mimesetexportoptions-function)

---

## MimeGetFileContentType Function

Return the content type for a specified file

### Prototype

```c
UINT WINAPI MimeGetFileContentType(
    HMESSAGE hMessage,
    LPCTSTR lpszFileName,
    LPTSTR lpszContentType,
    INT cchContentType
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string which specifies the name of the file for which content type information is returned. |
| `lpszContentType` | `LPTSTR` | Pointer to a string buffer that will contain the MIME type for the specified file. This may be a NULL pointer, in which case this parameter is ignored. If a buffer is provided, it is recommended that it be at least 64 characters in length. |
| `cchContentType` | `INT` | An integer which specifies the maximum number of characters, including the terminating null character, which may be copied into the string buffer. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the content type of the specified file. If the function fails, the return value is MIME_CONTENT_UNKNOWN. To get extended error information, call MimeGetLastError.

### Remarks

- The content type for a given file is determined based on the file name extension, or if the extension is not recognized, the actual contents of the file. On 32-bit platforms, the system registry is used to determine the default content type values for a given extension. In all cases, file types that are explicitly set using the MimeSetFileContentType function will override the default system values.
- The content type string which is copied to the string buffer is the standard MIME content type description, which specifies a primary type and a subtype, separated by a slash. For example, a plain text file would have a content type of text/plain, while an HTML document would have a content type of text/html. Binary files may be associated with a specific application. For example, the content type for a Microsoft Word document is application/msword. Those binary files which are not associated with a specific application, or have an unrecognized file name extension, have a content type of application/octet-stream.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachData](#mimeattachdata-function)
- [MimeAttachFile](#mimeattachfile-function)
- [MimeSetFileContentType](#mimesetfilecontenttype-function)

---

## MimeGetFirstMessageHeader Function

Return the first header field and value in the current message part

### Prototype

```c
BOOL WINAPI MimeGetFirstMessageHeader(
    HMESSAGE hMessage,
    LPTSTR lpszHeader,
    INT cchMaxHeader,
    LPTSTR lpszValue,
    INT cchMaxValue
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeader` | `LPTSTR` | A pointer to a string buffer that will contain the name of the first header in the current message part. This parameter cannot be a NULL pointer. |
| `cchMaxHeader` | `INT` | An integer which specifies the maximum number of characters which may be copied into the buffer, including the terminating null character. This parameter must have a value greater than zero. |
| `lpszValue` | `LPTSTR` | A pointer to a string buffer that will contain the value of the first header in the current message part. This parameter may be a NULL pointer, in which case the value of the header field is ignored. |
| `cchMaxValue` | `INT` | An integer which specifies the maximum number of characters which may be copied into the buffer, including the terminating null character. If the lpszValue parameter is NULL, this parameter should have a value of zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If no headers exist for the current message part, or the handle to the message is invalid, the function will return zero. To get extended error information, call MimeGetLastError.

### Remarks

- Each part in a multipart message has one or more header fields. To obtain header values for the main message, rather than the message attachments, the current part number must be set to zero using the MimeSetMessagePart function.
- The header fields from an imported message may not be returned in the same order as which they appear in the message. An application should never make an assumption about the order in which one or more header fields are defined.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeParseHeader](#mimeparseheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI MimeGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the calling thread's last error code value. Functions set this value by calling the MimeSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last-error code.

### Remarks

- You should call the MimeGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call MimeSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_MESSAGE or MIME_ERROR. Those functions which call MimeSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeGetErrorString](#mimegeterrorstring-function)
- [MimeSetLastError](#mimesetlasterror-function)

---

## MimeGetMessageBoundary Function

Return the multipart message boundary string

### Prototype

```c
LPCTSTR WINAPI MimeGetMessageBoundary(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `LPCTSTR`
- If the function succeeds, the return value is a pointer to the boundary string. If the function fails, a NULL pointer is returned. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)

---

## MimeGetMessageDate Function

Return the date and time from the message header

### Prototype

```c
LPCTSTR WINAPI MimeGetMessageDate(
    HMESSAGE hMessage,
    BOOL bLocalize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `bLocalize` | `BOOL` | Boolean flag which specifies if the date and time should be localized for the current timezone. |

### Return Value

- **Type**: `LPCTSTR`
- If the function succeeds, the return value is a pointer to the date and time string. If the function fails, it will return a NULL pointer. To get extended error information, call MimeGetLastError.

### Remarks

- If no date has been specified in the message, the Date header field will be set to the current date and time, and that value will be returned. The date string returned by this function should never be directly modified by the application. Each call to this function will invalidate the previous value that was returned, so if you wish to save or modify the value, you should first make a private copy of the string.
- To convert this date string to a long integer value that can be used with the standard C time functions, use the MimeParseDate function. Refer to the MimeFormatDate function for information on the format of the date string.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeFormatDate](#mimeformatdate-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeParseDate](#mimeparsedate-function)
- [MimeSetMessageDate](#mimesetmessagedate-function)

---

## MimeGetMessageHeader Function

Return the value of a specified header from the message

### Prototype

```c
LPCTSTR WINAPI MimeGetMessageHeader(
    HMESSAGE hMessage,
    LPCTSTR lpszHeader
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeader` | `LPCTSTR` | A pointer to a string which specifies the message header. This parameter cannot be a NULL pointer. |

### Return Value

- **Type**: `LPCTSTR`
- If the function succeeds, the return value is a pointer to the header value. If the header field does not exist, a NULL pointer is returned. To get extended error information, call MimeGetLastError.

### Remarks

- Each part in a multipart message has one or more header fields. To obtain header values for the main message, rather than the message attachments, the current part number must be set to zero using the MimeSetMessagePart function.
- For languages other than C/C++, it may be preferable to use the MimeGetMessageHeaderEx function which copies the header value into a string buffer rather than returning a pointer to the header value itself.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeaderEx](#mimegetmessageheaderex-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeParseHeader](#mimeparseheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetMessageHeaderEx Function

Copy the value of specified header to a string buffer

### Prototype

```c
INT WINAPI MimeGetMessageHeaderEx(
    HMESSAGE hMessage,
    INT nMessagePart,
    LPCTSTR lpszHeader,
    LPTSTR lpszValue,
    INT nMaxValue,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nMessagePart` | `INT` | An integer value which specifies which part of the message to return the header value from. A value of zero returns a header value from the main message header, while a value greater than zero returns the header value from that specific part of a multipart message. A value of -1 specifies that the header value should be returned from the current message part. |
| `lpszHeader` | `LPCTSTR` | A pointer to a string which specifies the message header. This parameter cannot be a NULL pointer. |
| `lpszValue` | `LPTSTR` | A pointer to a string buffer which will contain the value of the specified header when the function returns. If this parameter is NULL, the function will return the length of the header value without copying the data. This is useful for determining the length of a header value so that a string buffer can be allocated and passed to a subsequent call to the function. |
| `nMaxValue` | `INT` | An integer value which specifies the maximum number of characters that can be copied to the string buffer, including the terminating null character. If the lpszValue parameter is NULL, this value should be zero. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes copied into the string buffer, not including the terminating null byte. If the lpszValue parameter is NULL, the return value is the length of the header value. If the header field does not exist, a value of zero is returned. If an invalid pointer or message part is specified, a value of MIME_ERROR is returned. To get extended error information, call MimeGetLastError.

### Example

```cpp
LPTSTR lpszValue = NULL;
INT cchValue = 0;

// Determine the length of the To header field
cchValue = MimeGetMessageHeaderEx(hMessage, 0, _T("To"), NULL, 0);

// If the header field exists, allocate a string buffer
// and copy the value into the buffer
if (cchValue > 0)
{
	lpszValue = (LPTSTR)LocalAlloc(LPTR, cchValue + 1);
	MimeGetMessageHeaderEx(hMessage,
						0,
						_T("To"),
						lpszValue,
						cchValue + 1);
}

// After the string buffer has been used, release the
// memory that was allocated for it
if (lpszValue)
{
	LocalFree((HLOCAL)lpszValue);
	lpszValue = NULL;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeParseHeader](#mimeparseheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetMessagePart Function

Return the current message part index

### Prototype

```c
INT WINAPI MimeGetMessagePart(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the message part index. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeDeleteMessagePart](#mimedeletemessagepart-function)
- [MimeGetMessagePartCount](#mimegetmessagepartcount-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetMessagePartCount Function

Return the total number of message parts

### Prototype

```c
INT WINAPI MimeGetMessagePartCount(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of message parts. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCreateMessagePart](#mimecreatemessagepart-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetMessageSender Function

Return the email address of the message sender

### Prototype

```c
INT WINAPI MimeGetMessageSender(
    HMESSAGE hMessage,
    LPCTSTR lpszSender,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszSender` | `LPCTSTR` | A pointer to a string buffer that will contain the email address of the message sender when the function returns. |
| `nMaxLength` | `INT` | An integer value that specifies the maximum number of characters that can be copied to the string buffer, including the terminating null character. It is recommended that the maximum length be at least 64 characters. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes copied to the string buffer. If an error occurs, the function will return MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- This function attempts to determine the email address of the sender that originated the message. It will first check for the presence of a Sender or X-Sender header value. If these headers are not defined, it will use the value of the From header field. It will only return successfully if a valid email address can be found.
- If the function succeeds, the string buffer that is provided will only contain an email address. It will not contain the display name of the user associated with the address or any extraneous comments that are included in quotes or parenthesis.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeaderEx](#mimegetmessageheaderex-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeParseHeader](#mimeparseheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetMessageSize Function

Return the size of the complete message in bytes

### Prototype

```c
BOOL WINAPI MimeGetMessageSize(
    HMESSAGE hMessage,
    DWORD dwOptions,
    LPDWORD lpdwMessageSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `dwOptions` | `DWORD` | An unsigned integer which specifies how the size of the message should be calculated, based on what header fields should be included. These are the same options used when exporting a message to a file or memory buffer. The following values may be combined using a bitwise Or operator: |
| `lpdwMessageSize` | `LPDWORD` | A pointer to an unsigned integer value which will contain the size of the message if the function succeeds, or a value of zero if the function fails. This parameter cannot be NULL. |

### Options

- **dwOptions** Parameter

    - `MIME_OPTION_DEFAULT`: The default export options. The headers for the message are written out in a specific consistent order, with custom headers written to the end of the header block regardless of the order in which they were set or imported from another message. If the message contains Bcc, Received, Return-Path, Status or X400-Received header fields, they will not be exported.
    - `MIME_OPTION_ALLHEADERS`: All headers, including the Received, Return-Path, Status and X400-Received header fields will be exported. Normally these headers are not exported because they are only used by the mail transport system. This option can be useful when exporting a message to be stored on the local system, but should not be used when exporting a message to be delivered to another user.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If an error occurs, the function will return zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function returns the size of the complete message, including all headers, the message body and any attachments. It can be used to determine the minimum amount of memory that should be allocated to export the message to a memory buffer using the ExportMessage method.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExportMessageEx](#mimeexportmessageex-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)

---

## MimeGetMessageText Function

Return the text of the current message part

### Prototype

```c
LONG WINAPI MimeGetMessageText(
    HMESSAGE hMessage,
    LONG nOffset,
    LPTSTR lpszBuffer,
    LONG nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nOffset` | `LONG` | The byte offset from the beginning of the message. A value of zero specifies the first character in the body of the message part. |
| `lpszBuffer` | `LPTSTR` | A pointer to the string buffer that will contain a copy of the message text when the function returns. Note that the buffer will typically contain multiple lines of text. |
| `nMaxLength` | `LONG` | The maximum number of bytes to copy into the buffer. The size of the buffer provided must be larger than the content length for the current message part. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is number of bytes copied into the buffer. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- If your project targets a multi-byte character set, this function will always return the message contents as UTF-8 text, regardless of the original character set specified in the message itself. This ensures that characters in the original text are preserved, regardless of the default ANSI code page on the local system. It is recommended you build your project to use Unicode whenever possible. If your application must use ANSI, you can use the MimeLocalizeText method to convert the Unicode text to a specific character set.
- You should not determine the maximum size of the output buffer using the MimeGetContentLength function. That function returns the content size in bytes as it is stored in the message, and does not account for any character encoding or Unicode conversion which may be required. The content length can be used to estimate the amount of text stored in the message part, but you should always allocate a buffer which is larger than the length specified in the message.
- If the nMaxLength parameter does not specify a buffer size large enough to store the contents of the current message part, this function will fail and the last error code will be set to ST_ERROR_BUFFER_TOO_SMALL. Your application must ensure the buffer is large enough to contain the complete message text and a terminating NUL character.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAppendMessageText](#mimeappendmessagetext-function)
- [MimeClearMessageText](#mimeclearmessagetext-function)
- [MimeCompareMessageText](#mimecomparemessagetext-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeGetContentLength](#mimegetcontentlength-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeLocalizeText](#mimelocalizetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeGetMessageVersion Function

Return the MIME version from the message header

### Prototype

```c
LPCTSTR WINAPI MimeGetMessageVersion(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `LPCTSTR`
- If the function succeeds, the return value is a pointer to the MIME version. If the version is not specified in the current message, the function will return NULL. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeSetMessageVersion](#mimesetmessageversion-function)

---

## MimeGetNextMessageHeader Function

Return the next header field and value in the current message part

### Prototype

```c
BOOL WINAPI MimeGetNextMessageHeader(
    HMESSAGE hMessage,
    LPTSTR lpszHeader,
    INT cchMaxHeader,
    LPTSTR lpszValue,
    INT cchMaxValue
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeader` | `LPTSTR` | A pointer to the string buffer that will contain the name of the next header in the current message part when the function returns. This parameter cannot be a NULL pointer. |
| `cchMaxHeader` | `INT` | An integer which specifies the maximum number of characters which may be copied into the buffer, including the terminating null character. This parameter must have a value greater than zero. |
| `lpszValue` | `LPTSTR` | A pointer to the string buffer that will contain the value of the next header in the current message part when the function returns. This parameter may be a NULL pointer, in which case the value of the header field is ignored. |
| `cchMaxValue` | `INT` | An integer which specifies the maximum number of characters which may be copied into the buffer, including the terminating null character. If the lpszValue parameter is NULL, this parameter should have a value of zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If no more headers exist for the current message part, or the handle to the message is invalid, the function will return zero. To get extended error information, call MimeGetLastError.

### Remarks

- Each part in a multipart message has one or more header fields. To obtain header values for the main message, rather than the message attachments, the current part number must be set to zero using the MimeSetMessagePart function.
- The header fields from an imported message may not be returned in the same order as which they appear in the message. An application should never make an assumption about the order in which one or more header fields are defined, with the following exception:
- If an imported message has multiple Received headers, then those headers will be returned by MimeGetNextMessageHeader in the order in which they appeared in the original message. Note that if MimeGetMessageHeader is used to retrieve the Received header, the first Received header in the message will be returned.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeParseHeader](#mimeparseheader-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeGetStoredMessage Function

Retrieve a message from the specified message store

### Prototype

```c
HMESSAGE WINAPI MimeGetStoredMessage(
    HMESSAGESTORE hStorage,
    LONG nMessageId,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `nMessageId` | `LONG` | An integer value which specifies the message number that should be retrieved. The first message in the message store has a value of one. |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `MIME_COPY_STORED_MESSAGE`: A message handle is allocated for a copy of the message that is retrieved from the message store.

### Return Value

- **Type**: `HMESSAGE`
- If the function succeeds, the return value is a handle to the message. If the function fails, the return value is INVALID_MESSAGE. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeGetStoredMessage function returns a message handle for the specified message in the message store. If no options are specified, a temporary message handle is returned that is only valid until the next message is retrieved. The application can use message handle with any function except for the MimeDeleteMessage function because it does not own the message handle. If a multithreaded application changes the contents of the temporary message, it will change for all other threads that have obtained a message handle using this function.
- If the application must have a unique copy of the message, the MIME_COPY_STORED_MESSAGE option should be specified. Instead of returning a handle to a shared message, the message is duplicated and a handle to that copy of the message is returned. If this option is used, the application must call MimeDeleteMessage to release the memory allocated for the message.

### Example

```cpp
HMESSAGE hMessage = INVALID_MESSAGE;
LPCTSTR lpszHeader = _T("From");
LPCTSTR lpszAddress = _T("jsmith@example.com");
LONG nMessageId = 1; 

// Begin searching for messages from the specified sender
while (nMessageId != MIME_ERROR)
{
	nMessageId = MimeFindStoredMessage(hStorage,
									nMessageId,
									lpszHeader,
									lpszAddress,
									MIME_SEARCH_PARTIAL_MATCH);

	if (nMessageId != MIME_ERROR)
	{
		// Get a handle to the message that was found
		hMessage = MimeGetStoredMessage(hStorage, nMessageId, 0);

		if (hMessage != INVALID_MESSAGE)
		{
			// Store the message in a file
			TCHAR szFileName[MAX_PATH];
			BOOL bExported;

			// Create a filename based on the message number
			wsprintf(szFileName, _T("msg%05ld.tmp"), nMessageId);

			// Export the message to a file
			bExported = MimeExportMessage(hMessage, szFileName);
		}

		// Increase the message ID to resume the search at the next message
		nMessageId++;
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessageCount](#mimegetstoredmessagecount-function)
- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)
- [MimeStoreMessage](#mimestoremessage-function)

---

## MimeGetStoredMessageCount Function

Return the number of messages in a message store

### Prototype

```c
LONG WINAPI MimeGetStoredMessageCount(
    HMESSAGESTORE hStorage,
    LPLONG lpnLastMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `lpnLastMessage` | `LPLONG` | A pointer to an integer which will contain the message number for the last message in the storage file. If this information is not required, a NULL pointer may be specified. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the number of messages in the message store. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeGetStoredMessageCount function returns the number of messages in the message store. It is important to note that does not count those messages which have been marked for deletion. This means that the value returned by this function will decrease as messages are deleted.
- The message number returned in the lpnLastMessage parameter will specify the total number of messages in the message store, including deleted messages.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)

---

## MimeImportMessage Function

Import a message from the specified text file

### Prototype

```c
BOOL WINAPI MimeImportMessage(
    HMESSAGE hMessage,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string which specifies the name of the text file to import. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function will delete the contents of the current message before importing the new message from the specified file.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachFile](#mimeattachfile-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExportMessageEx](#mimeexportmessageex-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)
- [MimeInitialize](#mimeinitialize-function)
- [MimeParseBuffer](#mimeparsebuffer-function)

---

## MimeImportMessageEx Function

Import a message from a file, clipboard or memory buffer

### Prototype

```c
BOOL WINAPI MimeImportMessageEx(
    HMESSAGE hMessage,
    DWORD dwImportMode,
    DWORD dwImportOptions,
    LPVOID lpvMessage,
    DWORD dwMessageSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `dwImportMode` | `DWORD` | An unsigned integer which specifies how the message contents will be imported. It may be one of the following values: |
| `dwImportOptions` | `DWORD` | An unsigned integer which specifies how the message will be imported: |
| `lpvMessage` | `LPVOID` | A pointer to a string, a byte buffer or a global memory handle. The dwImportMode parameter determines how this pointer is used by the function. |
| `dwMessageSize` | `DWORD` | An unsigned integer value which specifies the size of the message to import. This parameter is only used when importing a message from a memory buffer. The message size is determined automatically when the message is imported from a file or the system clipboard. |

### Options

- **dwImportMode** Parameter

    - `MIME_IMPORT_DEFAULT`: The default import mode. If the lpvMessage parameter is NULL, then the contents of the message will be imported from the system clipboard. Otherwise, the lpvMessage parameter is a pointer to a string which specifies the name of a file that contains the message. The dwMessageSize parameter is ignored.
    - `MIME_IMPORT_FILE`: The lpvMessage parameter is a pointer to a string which specifies the name of the file that contains the message. If the file does not exist, or is not a regular text file, an error will occur. The dwMessageSize parameter is ignored.
    - `MIME_IMPORT_CLIPBOARD`: The contents of the message is imported from the system clipboard. The lpvMessage parameter is ignored. The dwMessageSize parameter is ignored.
    - `MIME_IMPORT_MEMORY`: The contents of the message is imported from a local buffer. The lpvMessage parameter must point to a byte array which contains the message to be imported. The dwMessageSize parameter specifies the number of bytes to copy from the buffer. If this value is zero, it is assumed that the end of the message data in the buffer is terminated with a null character and the length is calculated automatically.
    - `MIME_IMPORT_HGLOBAL`: The contents of the message is imported from a global memory buffer. The lpvMessage parameter must be a global memory handle which contains the message. The dwMessageSize parameter specifies the number of bytes to copy from the buffer. If this value is zero, it is assumed that the end of the message data in the buffer is terminated with a null character and the length is calculated automatically.
- **dwImportOptions** Parameter

    - `MIME_OPTION_DEFAULT`: The default import options. Currently this is the only valid value for this parameter and applications should always specify this constant.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Example

```cpp
// Imports a message using an HGLOBAL handle which references
// a block of memory containing an email message
bResult = MimeImportMessageEx(hMessage,
							MIME_IMPORT_HGLOBAL,
							MIME_OPTION_DEFAULT,
							(LPVOID)hgblMessage,
							0);

if (bResult)
{
	// The message has been successfully imported
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExportMessageEx](#mimeexportmessageex-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeInitialize Function

Initialize the library for use by the client

### Prototype

```c
BOOL WINAPI MimeInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that MimeInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to MimeUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the MimeInitialize or MimeUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeComposeMessage](#mimecomposemessage-function)
- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeDeleteMessage](#mimedeletemessage-function)
- [MimeUninitialize](#mimeuninitialize-function)

---

## MimeLocalizeText Function

Localize Unicode text to ANSI using a specific character set

### Prototype

```c
LONG WINAPI MimeLocalizeText(
    UINT nCharacterSet,
    LPCTSTR lpszInput,
    LONG cchInput,
    LPBYTE lpOutput,
    LONG cbOutput
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCharacterSet` | `UINT` | A numeric identifier which specifies the character set to use when localizing the text. A value of zero specifies the locale for the current thread should be used when localizing the text. |
| `lpszInput` | `LPCTSTR` | A pointer to a null terminated string which contains the Unicode text which should be localized. If the ANSI version of this function is called, the input text must be in UTF-8 format or the function will fail. This parameter cannot be a NULL pointer. |
| `cchInput` | `LONG` | An integer value which specifies the number of characters of text in the input string which should be localized. If this value is -1, the entire length of the string up to the terminating null will be decoded. |
| `lpOutput` | `LPBYTE` | A pointer to a byte buffer which will contain the localized ANSI version of the input text. This parameter cannot be a NULL pointer. |
| `cbOutput` | `LONG` | An integer value which specifies the maximum number of bytes which can be copied into the output buffer. The buffer must be large enough to store all of the localized text. This value must be greater than zero. |

### Return Value

- **Type**: `LONG`
- If the input text can be successfully localized, the return value is the number of bytes copied into the output buffer. If the function returns zero, then no text was localized. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeLocalizeText function enables the application to localize a Unicode string, returning the ANSI version of that text using the specified character set. Because library handles all text as Unicode internally, the ANSI functions in this library will always return UTF-8 encoded text. This function allows you to easily convert that UTF-8 text to another supported character set.
- If the ANSI version of this function is called, the input text must use UTF-8 character encoding or the function will fail with the last error set to ST_ERROR_INVALID_CHARACTER_SET.
- If the nCharacterSet parameter is MIME_CHARSET_DEFAULT or MIME_CHARSET_UNKNOWN the input text will be converted to the default ANSI code page for the current thread locale. If there are characters in the Unicode input text which cannot be converted to an ANSI equivalent using the specified character set, those characters will be replaced by the default character for your locale, typically a question mark symbol. You cannot specify MIME_CHARSET_UTF16 as the character set.
- This function will always attempt to ensure that the output buffer is terminated with an extra null byte so it is easier to work with as a standard C-style null terminated string. However, if the output buffer is not large enough to accommodate the extra null byte, it will not be copied. It is always recommended that your output buffer be somewhat larger than the length of the original input text to account for multi-byte character sequences. If the output buffer is not large enough to contain the entire localized text, no bytes will be copied to the output buffer and the function will fail with the last error set to ST_ERROR_BUFFER_TOO_SMALL.
- This function is only required if your application needs to localize the UTF-8 text returned by another function and convert it to a specific 8-bit ANSI character set. For example, if you have an application which calls the ANSI version of MimeGetMessageText, it will return the message contents as UTF-8 text. If you need to display that text as localized ANSI, you can call this function to convert the UTF-8 text to your current locale or a specific character set.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeDecodeTextEx](#mimedecodetextex-function)
- [MimeEncodeText](#mimeencodetext-function)
- [MimeEncodeTextEx](#mimeencodetextex-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeOpenMessageStore Function

Open the specified message storage file

### Prototype

```c
HMESSAGESTORE WINAPI MimeOpenMessageStore(
    LPCTSTR lpszFileName,
    DWORD dwOpenMode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFileName` | `LPCTSTR` | A pointer to a string which specifies the name of the storage file. |
| `dwOpenMode` | `DWORD` | A value which specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOpenMode** Parameter

    - `MIME_STORAGE_READ`: The message store will be opened for read access. The contents of the message store can be accessed, but cannot be modified by the process unless it has also been opened for writing.
    - `MIME_STORAGE_WRITE`: The message store will be opened for writing. This mode also implies read access and must be specified if the application needs to modify the contents of the message store.
    - `MIME_STORAGE_CREATE`: The message store will be created if the storage file does not exist. If the file exists, it will be truncated. This mode implies read and write access.
    - `MIME_STORAGE_LOCK`: The message store will be opened so that it may only be accessed and modified by the current process.
    - `MIME_STORAGE_COMPRESS`: The contents of the message store are compressed. This option is automatically enabled if a compressed message store is opened for reading or writing.
    - `MIME_STORAGE_MAILBOX`: The message store should use the UNIX mbox format when reading and storing messages. This option is provided for backwards compatibility and is not recommended for general use.

### Return Value

- **Type**: `HMESSAGESTORE`
- If the function succeeds, the return value is a handle to the message store. If the function fails, the return value is INVALID_MESSAGESTORE. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeOpenMessageStore function opens a message storage file which contains one or more messages. If the storage file is opened for read access, the application can search the message store and extract messages but it cannot add or delete messages. To add new messages or delete existing messages from the store, it must be opened with write access.
- The message store is designed to be a simple, effective way to store multiple messages together in a single file. When the message store is opened, the contents are indexed in memory. Although there is no specific limit to the number of messages that can be stored, there must be sufficient memory available to build an index of each message and its headers. If the application must store and manage a very large number of messages, it is recommended that you use a database rather than a flat-file message store.
- Message Store Format Each message is prefixed by a control sequence of five ASCII 01 characters followed by an ASCII 10 and ASCII 13 character. The messages themselves are stored unmodified in their original text format. The length of each message is calculated based on the location of the control sequence that delimits each message, and explicit message lengths are not stored in the file. This means that it is safe to manually change the message contents, as long as the message delimiters are preserved.
- If the message store is compressed, the contents of the storage file are expanded when the file is opened and then re-compressed when the storage file is closed. Using the MIME_STORAGE_COMPRESS option reduces the size of the storage file and prevents the contents of the message store from being read using a text file editor. However, enabling compression will increase the amount of memory allocated by the library and can increase the amount of time that it takes to open and close the storage file.
- The library also has a backwards compatibility mode where it will recognize storage files that use the UNIX mbox format. While this format is supported for accessing existing files, it is not recommended that you use it when creating new message stores or adding messages to an existing store. There are a number of different variants on the mbox format that have been used by different Mail Transfer Agents (MTAs) on the UNIX platform. For example, the mboxrd variant looks identical to the mboxcl2 variant, and they are programmatically indistinguishable from one another, but they are not compatible. For this reason, the use of the mbox format is strongly discouraged.

### Example

```cpp
HMESSAGE hMessage;

// Compose a new message
hMessage = MimeComposeMessage(lpszSender,
							lpszRecipient,
							NULL,
							lpszSubject,
							lpszMessage,
							NULL,
							MIME_CHARSET_DEFAULT,
							MIME_ENCODING_DEFAULT);

if (hMessage != INVALID_MESSAGE)
{
	HMESSAGESTORE hStorage;

	// Open the message storage file
	hStorage = MimeOpenMessageStore(lpszFileName, MIME_STORAGE_WRITE);

	if (hStorage == INVALID_MESSAGESTORE)
	{
		// Delete the message and return if we are unable to
		// open the storage file
		MimeDeleteMessage(hMessage);
		return;
	}

	// Store a copy of the message in the message store
	nMessageId = MimeStoreMessage(hStorage, hMessage, 0);

	if (nMessageId == MIME_ERROR)
	{
		// We were unable to store the message
	}

	// Close the message store
	MimeCloseMessageStore(hStorage);

	// Destroy the message that was created
	MimeDeleteMessage(hMessage);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeCloseMessageStore](#mimeclosemessagestore-function)
- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeGetStoredMessageCount](#mimegetstoredmessagecount-function)
- [MimePurgeMessageStore](#mimepurgemessagestore-function)

---

## MimeParseAddress Function

Parse the specified email address

### Prototype

```c
INT WINAPI MimeParseAddress(
    LPCTSTR lpszString,
    LPCTSTR lpszDomain,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszString` | `LPCTSTR` | A pointer to a string which contains the email address to parse. |
| `lpszDomain` | `LPCTSTR` | A pointer to a string which specifies a default domain for the address. This parameter may be NULL or point to an empty string if no default domain is required. |
| `lpszAddress` | `LPTSTR` | A pointer to a string buffer which will contain the parsed email address when the function returns. It is recommended that this buffer be at least 128 characters in length. |
| `nMaxLength` | `INT` | The maximum number of characters which can be copied into the string buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the address. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeParseAddress function is useful for parsing the email addresses that may be specified in various header fields in the message. In many cases, the addresses have additional comment characters which are not part of the address itself. For example, one common format used is as follows:
- "User Name" <user@domain.com>
- In this case, the email address is enclosed in angle brackets and the name outside of the brackets is considered to be a comment which is not part of the address itself. Another common format is:
- user@domain.com (User Name)
- In this case, there is the address followed by a comment which is enclosed in parenthesis. The MimeParseAddress function recognizes both formats, and when passed either string, would return the following address:
- user@domain.com
- If there was no domain specified in the address, that is just a user name was specified, then the value the lpszDomain parameter is added to the address.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExtractFile](#mimeextractfile-function)
- [MimeImportMessage](#mimeimportmessage-function)

---

## MimeParseBuffer Function

Parse the specified text and add to the current message

### Prototype

```c
BOOL WINAPI MimeParseBuffer(
    HMESSAGE hMessage,
    LPCTSTR lpszBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszBuffer` | `LPCTSTR` | Pointer to a buffer that contains the text to be added to the message contents. |
| `cbBuffer` | `INT` | The length of the specified buffer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function is useful when the application needs to parse an arbitrary block of text and add it to the specified message. If the buffer contains header fields, the values will be added to the message header. Once the end of the header block is detected, all subsequent text is added to the body of the message.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeImportMessage](#mimeimportmessage-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)
- [MimeParseHeader](#mimeparseheader-function)

---

## MimeParseDate Function

Parse the specified RFC 822 formatted date string

### Prototype

```c
BOOL WINAPI MimeParseDate(
    LPCTSTR lpszDate,
    BOOL bLocalize,
    LPLONG lpnSeconds,
    LPLONG lpnTimezone
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszDate` | `LPCTSTR` | A pointer to a string which specifies the date to be parsed. The date string must be in the standard format defined by RFC 822. |
| `bLocalize` | `BOOL` | A boolean flag which determines if the date should be localized to the current timezone, regardless of the timezone specified in the date string. A non-zero value specifies the timezone for the local system will be used, adjusted for daylight savings time if applicable. |
| `lpnSeconds` | `LPLONG` | A pointer to a long integer which is set to the number of seconds since 1 January 1970 00:00:00 UTC. This date is commonly called the epoch, and is the base date used by the standard C time functions. This pointer may be NULL, in which case the parameter is ignored. |
| `lpnTimezone` | `LPLONG` | A pointer to a long integer which is set to the difference in seconds between the specified date's timezone and coordinated universal time (also known as Greenwich Mean Time). This pointer may be NULL, in which case the parameter is ignored. |

### Return Value

- **Type**: `BOOL`
- If the date could be successfully parsed, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This is not a general purpose date parsing function, and it may not be capable of parsing dates for a specific locale. The date and time should be in a standard format as outlined in RFC 822, which describes the basic structure of Internet email messages. For a description of the date string format, refer to the MimeFormatDate function.
- If the date and time does not include any timezone information, Coordinated Universal Time (UTC) will be used by default. This is an important consideration if you use this function to parse input from a user, because in most cases they will not provide a timezone and will assume the date and time they enter is for their current timezone.
- The value of the bLocalize parameter will only change the number of seconds offset by the current timezone and does not affect the value returned in the lpnSeconds parameter. If the date and time is localized, the timezone offset will be adjusted for daylight savings if it was in effect at the time.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeFormatDate](#mimeformatdate-function)
- [MimeGetMessageDate](#mimegetmessagedate-function)
- [MimeSetMessageDate](#mimesetmessagedate-function)

---

## MimeParseHeader Function

Parse the specified text and add to message header

### Prototype

```c
BOOL WINAPI MimeParseHeader(
    HMESSAGE hMessage,
    LPCTSTR lpszBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszBuffer` | `LPCTSTR` | Pointer to a string which contains the header and value to be added to the message. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- This function is used to parse a line of text that is part of a message header. The string must consist of a header name, followed by a colon, followed by the header value. The header name may only consist of printable characters, and may not contain whitespace (space, tab, carriage return or linefeed characters).

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeImportMessage](#mimeimportmessage-function)
- [MimeParseBuffer](#mimeparsebuffer-function)

---

## MimePurgeMessageStore Function

Purge all deleted messages from the specified message store

### Prototype

```c
BOOL WINAPI MimePurgeMessageStore(
    HMESSAGESTORE hStorage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimePurgeMessageStore function purges all deleted messages from the message store. If the storage file has been opened in read-only mode or there are no messages marked for deletion, this function will take no action.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCopyMessageStore](#mimecopymessagestore-function)
- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)

---

## MimeReplaceStoredMessage Function

Replace the specified message in the message store

### Prototype

```c
BOOL WINAPI MimeReplaceStoredMessage(
    HMESSAGESTORE hStorage,
    LONG nMessageId,
    HMESSAGE hMessage,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `nMessageId` | `LONG` | An integer value which specifies the message number that should be replaced. |
| `hMessage` | `HMESSAGE` | Handle to the message that will be stored. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must always be zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeReplaceStoredMessage function replaces the specified message with a new message. The message number may be a message that has been previously marked for deletion. It is important to note that the change will not be reflected in the physical storage file until it has been closed. If the application needs to replace messages in the message store, it is recommended that the file be opened for exclusive access using the MIME_STORAGE_LOCK option.

### Example

```cpp
HMESSAGE hMessage;

// Compose a new message
hMessage = MimeComposeMessage(lpszSender,
							lpszRecipient,
							NULL,
							lpszSubject,
							lpszMessage,
							NULL,
							MIME_CHARSET_DEFAULT,
							MIME_ENCODING_DEFAULT);

if (hMessage != INVALID_MESSAGE)
{
	HMESSAGESTORE hStorage;
	LONG nLastMessage;
	BOOL bResult;

	// Open the message storage file
	hStorage = MimeOpenMessageStore(lpszFileName, MIME_STORAGE_WRITE);

	if (hStorage == INVALID_MESSAGESTORE)
	{
		// Delete the message and return if we are unable to
		// open the storage file
		MimeDeleteMessage(hMessage);
		return;
	}

	if (MimeGetStoredMessageCount(hStorage, &nLastMessage) < 1)
	{
		// No messages are stored in the file
		MimeCloseMessageStore(hStorage);
		MimeDeleteMessage(hMessage);
		return;
	}

	// Replace the last message in the message store
	bResult = MimeReplaceStoredMessage(hStorage, nLastMessage, hMessage, 0);

	if (bResult == FALSE)
	{
		// We were unable to replace the message
	}

	// Close the message store
	MimeCloseMessageStore(hStorage);

	// Destroy the message that was created
	MimeDeleteMessage(hMessage);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)
- [MimeStoreMessage](#mimestoremessage-function)

---

## MimeResetMessage Function

Clear the specified message, deleting all message parts

### Prototype

```c
BOOL WINAPI MimeResetMessage(
    HMESSAGE hMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeCreateMessage](#mimecreatemessage-function)
- [MimeDeleteMessage](#mimedeletemessage-function)

---

## MimeSetExportOptions Function

Specify a bitmask that describes current message export options

### Prototype

```c
BOOL WINAPI MimeSetExportOptions(
    HMESSAGE hMessage,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `dwOptions` | `DWORD` | Mask of attribute options. The mask is a combination of the following values: |

### Options

- **dwOptions** Parameter

    - `MIME_OPTION_DEFAULT`: The default export options. The headers for the message are written out in a specific consistent order, with custom headers written to the end of the header block regardless of the order in which they were set or imported from another message. If the message contains Bcc, Received, Return-Path, Status or X400-Received header fields, they will not be exported.
    - `MIME_OPTION_KEEPORDER`: The original order in which the message header fields were set or imported are preserved when the message is exported.
    - `MIME_OPTION_ALLHEADERS`: All headers, including the Bcc, Received, Return-Path, Status and X400-Received header fields will be exported. Normally these headers are not exported because they are only used by the mail transport system. This option can be useful when exporting a message to be stored on the local system, but should not be used when exporting a message to be delivered to another user.
    - `MIME_OPTION_NOHEADERS`: When exporting a message, the main header block will not be included. This can be useful when creating a multipart message for services which expect MIME formatted data, such as HTTP POST requests. This option should never be used for email messages being submitted using SMTP.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- By default, the Received and Return-Path headers are not exported. In addition, the order of the headers in an exported message is undefined. This is reasonable behavior for most mail clients, but may not be appropriate for applications which need access to all of the header fields.

### Example

```cpp
MimeSetExportOptions(hMessage, MIME_EXPORT_OPTIONS_ALL |
							MIME_EXPORT_OPTIONS_KEEP_ORDER);

if (MimeImportMessage(hMessage, lpszFileName))
{
	// Process the message and make any modifications
	// then write the message back out, preserving all
	// of the headers in their original order
	bResult = MimeExportMessage(hMessage, lpszFileName);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeExportMessage](#mimeexportmessage-function)
- [MimeExportMessageEx](#mimeexportmessageex-function)
- [MimeImportMessage](#mimeimportmessage-function)
- [MimeImportMessageEx](#mimeimportmessageex-function)
- [MimeGetExportOptions](#mimegetexportoptions-function)

---

## MimeSetFileContentType Function

Set the content type for a specific file name extension

### Prototype

```c
BOOL WINAPI MimeSetFileContentType(
    HMESSAGE hMessage,
    UINT nContentType,
    LPCTSTR lpszSubtype,
    LPCTSTR lpszExtension
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nContentType` | `UINT` | An identifier which specifies the primary content type. It may be one of the following values: |
| `lpszSubtype` | `LPCTSTR` | A pointer to a string which specifies the MIME subtype. This parameter may be NULL if the content type association is being deleted. |
| `lpszExtension` | `LPCTSTR` | A pointer to a string which specifies the file name extension that will be associated with the MIME content type. |

### Options

- **nContentType** Parameter

    - `MIME_CONTENT_UNKNOWN`: The default content type for the specified extension should be used. This value should only be used to delete a previously defined content type.
    - `MIME_CONTENT_APPLICATION`: The file content is application specific. Examples of this type of file would be a Microsoft Word document or an executable program. This is also the default type for files which have an unrecognized file name extension and contain binary data.
    - `MIME_CONTENT_AUDIO`: The file is an audio file in one of several standard formats. Examples of this type of file would be a Windows (.wav) file or MPEG3 (.mp3) file.
    - `MIME_CONTENT_IMAGE`: The file is an image file in one of several standard formats. Examples of this type of file would be a GIF or JPEG image file.
    - `MIME_CONTENT_TEXT`: The file is a text file. This is also the default type for files which have an unrecognized file name extension and contain only printable text data.
    - `MIME_CONTENT_VIDEO`: The file is a video file in one of several standard formats. Examples of this type of file would be a Windows (.avi) or Quicktime (.mov) video file.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeSetFileContentType function allows an application to specify a content type for a given file extension, and is typically used to define custom content types for file attachments. The content type will override any default content types associated with the extension, as well as allow new content types to be defined for application-specific files.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAttachData](#mimeattachdata-function)
- [MimeAttachFile](#mimeattachfile-function)
- [MimeGetFileContentType](#mimegetfilecontenttype-function)

---

## MimeSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI MimeSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the caller. A value of zero clears the last error code. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value error code such as FALSE, NULL, INVALID_MESSAGE or MIME_ERROR. Those functions which call MimeSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the MimeGetLastError function. The use of MimeGetLastError is optional. An application can call it to find out the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeGetErrorString](#mimegeterrorstring-function)
- [MimeGetLastError](#mimegetlasterror-function)

---

## MimeSetMessageDate Function

Set the current date in the header for the specified message

### Prototype

```c
BOOL WINAPI MimeSetMessageDate(
    HMESSAGE hMessage,
    LPCTSTR lpszDate,
    BOOL bLocalize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszDate` | `LPCTSTR` | Pointer to a string which specifies the date and time. If this parameter specifies a zero-length string or a NULL pointer, the current date and time will be used. |
| `bLocalize` | `BOOL` | Boolean flag that specifies the date and time should be localized for the current timezone. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Remarks

- The date string should be in a standard format as outlined in RFC 822, the document which describes the basic structure of Internet email messages. For a description of the date string format, refer to the MimeFormatDate function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeFormatDate](#mimeformatdate-function)
- [MimeGetMessageDate](#mimegetmessagedate-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeParseDate](#mimeparsedate-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeSetMessageHeader Function

Create or update a header field in the specified message

### Prototype

```c
BOOL WINAPI MimeSetMessageHeader(
    HMESSAGE hMessage,
    LPCTSTR lpszHeader,
    LPCTSTR lpszValue
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszHeader` | `LPCTSTR` | Pointer to a string which specifies the header field that will be added or updated. |
| `lpszValue` | `LPCTSTR` | Pointer to a string which specifies the value for the header field. This pointer may be NULL, which causes the header field to be removed from the message. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAddMessageHeaders](#mimeaddmessageheaders-function)
- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeSetMessageHeaderEx](#mimesetmessageheaderex-function)

---

## MimeSetMessageHeaderEx Function

Create or update a header field, with additional options

### Prototype

```c
BOOL WINAPI MimeSetMessageHeaderEx(
    HMESSAGE hMessage,
    INT nMessagePart,
    LPCTSTR lpszHeader,
    LPCTSTR lpszValue,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nMessagePart` | `INT` | An integer value which specifies which part of the message the header should be set or modified in. A value of zero sets a header value in the main message header block, while a value greater than zero sets the header value in that specific part of a multipart message. A value of -1 specifies that the header value should be set in the current message part. |
| `lpszHeader` | `LPCTSTR` | Pointer to a string which specifies the header field that will be added or updated. |
| `lpszValue` | `LPCTSTR` | Pointer to a string which specifies the value for the header field. This pointer may be NULL, which causes the header field to be removed from the message. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAddMessageHeaders](#mimeaddmessageheaders-function)
- [MimeEnumMessageHeaders](#mimeenummessageheaders-function)
- [MimeGetFirstMessageHeader](#mimegetfirstmessageheader-function)
- [MimeGetMessageHeader](#mimegetmessageheader-function)
- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetNextMessageHeader](#mimegetnextmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)
- [MimeSetMessageText](#mimesetmessagetext-function)

---

## MimeSetMessagePart Function

Set the current message part index for the specified message

### Prototype

```c
INT WINAPI MimeSetMessagePart(
    HMESSAGE hMessage,
    INT nNewPart
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nNewPart` | `INT` | The new message part index. A value of zero specifies the main message part. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the previous message part index. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeGetMessagePart](#mimegetmessagepart-function)
- [MimeGetMessagePartCount](#mimegetmessagepartcount-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeSetMessageText Function

Create or update the specified message body

### Prototype

```c
LONG WINAPI MimeSetMessageText(
    HMESSAGE hMessage,
    LONG nOffset,
    LPCTSTR lpszText
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `nOffset` | `LONG` | The offset into the body of the message part. A value of -1 specifies that the text will be appended to the message body. |
| `lpszText` | `LPCTSTR` | A pointer to a string which specifies the text to be copied to the current message part at the given offset. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the number of bytes copied into the message. A return value of zero indicates that no text could be copied into the current message part. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeAppendMessageText](#mimeappendmessagetext-function)
- [MimeClearMessageText](#mimeclearmessagetext-function)
- [MimeCompareMessageText](#mimecomparemessagetext-function)
- [MimeGetMessageText](#mimegetmessagetext-function)
- [MimeResetMessage](#mimeresetmessage-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)
- [MimeSetMessagePart](#mimesetmessagepart-function)

---

## MimeSetMessageVersion Function

Set the MIME version for the specified message

### Prototype

```c
BOOL WINAPI MimeSetMessageVersion(
    HMESSAGE hMessage,
    LPCTSTR lpszVersion
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hMessage` | `HMESSAGE` | Handle to the message. |
| `lpszVersion` | `LPCTSTR` | Pointer to a string which specifies the MIME version. A value of NULL sets the version to the default value of 1.0. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call MimeGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [MimeGetMessageVersion](#mimegetmessageversion-function)
- [MimeSetMessageHeader](#mimesetmessageheader-function)

---

## MimeStoreMessage Function

Store the specified message in a message store

### Prototype

```c
LONG WINAPI MimeStoreMessage(
    HMESSAGESTORE hStorage,
    HMESSAGE hMessage,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HMESSAGESTORE` | Handle to the message store. |
| `hMessage` | `HMESSAGE` | Handle to the message that will be stored. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must always be zero. |

### Return Value

- **Type**: `LONG`
- If the function succeeds, the return value is the message number for the message that was just stored. If the function fails, the return value is MIME_ERROR. To get extended error information, call MimeGetLastError.

### Remarks

- The MimeStoreMessage function will always append the specified message to the storage file. If you want to replace a message in the message store, you should use the MimeReplaceStoredMessage function.

### Example

```cpp
HMESSAGE hMessage;

// Compose a new message
hMessage = MimeComposeMessage(lpszSender,
							lpszRecipient,
							NULL,
							lpszSubject,
							lpszMessage,
							NULL,
							MIME_CHARSET_DEFAULT,
							MIME_ENCODING_DEFAULT);

if (hMessage != INVALID_MESSAGE)
{
	HMESSAGESTORE hStorage;

	// Open the message storage file
	hStorage = MimeOpenMessageStore(lpszFileName, MIME_STORAGE_WRITE);

	if (hStorage == INVALID_MESSAGESTORE)
	{
		// Delete the message and return if we are unable to
		// open the storage file
		MimeDeleteMessage(hMessage);
		return;
	}

	// Store a copy of the message in the message store
	nMessageId = MimeStoreMessage(hStorage, hMessage, 0);

	if (nMessageId == MIME_ERROR)
	{
		// We were unable to store the message
	}

	// Close the message store
	MimeCloseMessageStore(hStorage);

	// Destroy the message that was created
	MimeDeleteMessage(hMessage);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeDeleteStoredMessage](#mimedeletestoredmessage-function)
- [MimeFindStoredMessage](#mimefindstoredmessage-function)
- [MimeGetStoredMessage](#mimegetstoredmessage-function)
- [MimeReplaceStoredMessage](#mimereplacestoredmessage-function)

---

## MimeUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI MimeUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful MimeInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call MimeUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to MimeUninitialize for every successful call to MimeInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmsgv11.lib`

### See Also

- [MimeInitialize](#mimeinitialize-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

