# Simple Mail Transfer Protocol

## Functions

- [SmtpAddRecipient](#smtpaddrecipient-function)
- [SmtpAppendMessage](#smtpappendmessage-function)
- [SmtpAsyncConnect](#smtpasyncconnect-function)
- [SmtpAsyncSendMessage](#smtpasyncsendmessage-function)
- [SmtpAsyncSubmitMessage](#smtpasyncsubmitmessage-function)
- [SmtpAttachThread](#smtpattachthread-function)
- [SmtpAuthenticate](#smtpauthenticate-function)
- [SmtpCancel](#smtpcancel-function)
- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpCommand](#smtpcommand-function)
- [SmtpConnect](#smtpconnect-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)
- [SmtpCreateSecurityCredentials](#smtpcreatesecuritycredentials-function)
- [SmtpDeleteSecurityCredentials](#smtpdeletesecuritycredentials-function)
- [SmtpDisableEvents](#smtpdisableevents-function)
- [SmtpDisableTrace](#smtpdisabletrace-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SmtpEnableEvents](#smtpenableevents-function)
- [SmtpEnableTrace](#smtpenabletrace-function)
- [SmtpEnumTasks](#smtpenumtasks-function)
- [SmtpEventProc](#smtpeventproc-function)
- [SmtpExpandAddress](#smtpexpandaddress-function)
- [SmtpFreezeEvents](#smtpfreezeevents-function)
- [SmtpGetCurrentDate](#smtpgetcurrentdate-function)
- [SmtpGetDeliveryOptions](#smtpgetdeliveryoptions-function)
- [SmtpGetErrorString](#smtpgeterrorstring-function)
- [SmtpGetExtendedOptions](#smtpgetextendedoptions-function)
- [SmtpGetLastError](#smtpgetlasterror-function)
- [SmtpGetResultCode](#smtpgetresultcode-function)
- [SmtpGetResultString](#smtpgetresultstring-function)
- [SmtpGetSecurityInformation](#smtpgetsecurityinformation-function)
- [SmtpGetStatus](#smtpgetstatus-function)
- [SmtpGetTaskError](#smtpgettaskerror-function)
- [SmtpGetTaskId](#smtpgettaskid-function)
- [SmtpGetTimeout](#smtpgettimeout-function)
- [SmtpGetTransferStatus](#smtpgettransferstatus-function)
- [SmtpInitialize](#smtpinitialize-function)
- [SmtpIsBlocking](#smtpisblocking-function)
- [SmtpIsConnected](#smtpisconnected-function)
- [SmtpIsReadable](#smtpisreadable-function)
- [SmtpIsWritable](#smtpiswritable-function)
- [SmtpRegisterEvent](#smtpregisterevent-function)
- [SmtpReset](#smtpreset-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpSetDeliveryOptions](#smtpsetdeliveryoptions-function)
- [SmtpSetLastError](#smtpsetlasterror-function)
- [SmtpSetTimeout](#smtpsettimeout-function)
- [SmtpSubmitMessage](#smtpsubmitmessage-function)
- [SmtpSubmitMessageEx](#smtpsubmitmessageex-function)
- [SmtpTaskAbort](#smtptaskabort-function)
- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)
- [SmtpUninitialize](#smtpuninitialize-function)
- [SmtpVerifyAddress](#smtpverifyaddress-function)
- [SmtpWrite](#smtpwrite-function)
## Structures

- [INITDATA](#initdata-structure)
- [SECURITYCREDENTIALS](#securitycredentials-structure)
- [SECURITYINFO](#securityinfo-structure)
- [SMTPMESSAGE](#smtpmessage-structure)
- [SMTPMESSAGEEX](#smtpmessageex-structure)
- [SMTPSERVER](#smtpserver-structure)
- [SMTPTRANSFERSTATUS](#smtptransferstatus-structure)

---

## SmtpAddRecipient Function

Add an address to the recipient list

### Prototype

```c
INT WINAPI SmtpAddRecipient(
    HCLIENT hClient,
    LPCTSTR lpszAddress
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszAddress` | `LPCTSTR` | Points to a string which specifies the address to be added to the recipient list. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the server result code. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)
- [SmtpExpandAddress](#smtpexpandaddress-function)
- [SmtpVerifyAddress](#smtpverifyaddress-function)

---

## SmtpAppendMessage Function

Append contents of specified file to the current message

### Prototype

```c
INT WINAPI SmtpAppendMessage(
    HCLIENT hClient,
    LPVOID lpvMessage,
    DWORD dwMessageSize,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpvMessage` | `LPVOID` | Pointer to a buffer which contains the message data to be appended, or a pointer to the name of the file which contains the data to be written to the data stream. The use of this parameter depends on the value of the dwOptions parameter. |
| `dwMessageSize` | `DWORD` | An unsigned integer which specifies the length of the message in bytes. |
| `dwOptions` | `DWORD` | Specifies the source of the message data that will be written to the data stream; it may be one of the following values: |

### Options

- **dwOptions** Parameter

    - `SMTP_MESSAGE_MEMORY`: The lpvMessage parameter specifies a pointer to an array of characters. If the value of dwMessageSize is zero, then it is assumed to be a pointer to a string.
    - `SMTP_MESSAGE_HGLOBAL`: The lpvMessage parameter specifies an HGLOBAL which contains the data to be written to the data stream. If the value of the dwMessageSize parameter is zero, then the data is assumed to be null-terminated.
    - `SMTP_MESSAGE_FILE`: The lpvMessage parameter specifies a pointer to a string which contains the name of a file. The file is opened and the contents of the file are written to the data stream. The value of the dwMessageSize parameter is ignored when this option is specified.
    - `SMTP_MESSAGE_CLIPBOARD`: The lpvMessage and dwMessageSize parameters are ignored. The current contents of the clipboard are written to the data stream.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpAppendMessage function is used to append the contents of a memory buffer, file or the system clipboard to the current message that is being composed for delivery. To send a complete RFC 822 formatted message, refer to the SmtpSendMessage function.
- This function will cause the current thread to block until the complete message has been written, a timeout occurs or the operation is canceled. During the transfer, the SMTP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling SmtpEnableEvents, or by registering a callback function using the SmtpRegisterEvent function.
- To determine the current status of a transfer while it is in progress, use the SmtpGetTransferStatus function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)
- [SmtpSendMessage](#smtpsendmessage-function)

---

## SmtpAsyncConnect Function

Establish an asynchronous connection with a server

### Prototype

```c
HCLIENT WINAPI SmtpAsyncConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPCTSTR lpszLocalName,
    LPSECURITYCREDENTIALS lpCredentials,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number should be used. For standard connections, the default port number is 25. An alternative port is 587, which is commonly used by authenticated clients to submit messages for delivery. For secure connections, the default port number is 465. If the secure port number is specified, an implicit TLS connection will be established by default. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpszLocalName` | `LPCTSTR` | A pointer to a string which specifies the domain name of the local host. This parameter can be NULL or point to an empty string, in which case the local domain name is determined automatically from the system configuration. This parameter should be used if the mail server only accepts messages from a client that identifies itself using a specific domain name. |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which is used to establish the client credentials for a secure connection to the server. The function SmtpCreateSecurityCredentials can be used to create this structure if necessary. If a standard non-secure connection is being established, or client credentials are not required by the server, this parameter can be NULL. |
| `hEventWnd` | `HWND` | The handle to the event notification window. This window receives messages which notify the client of various asynchronous socket events that occur. If this parameter is NULL, a blocking connection is established with the server. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous socket event occurs. This value should be greater than WM_USER as defined in the Windows header files. If the hEventWnd parameter is NULL, this parameter must be specified as WM_NULL. |

### Options

- **dwOptions** Parameter

    - `SMTP_OPTION_NONE`: No additional options are specified when establishing a connection with the server. A standard, non-secure connection will be used and the client will not attempt to use extended (ESMTP) features of the protocol. Note that if the mail server requires authentication, the SMTP_OPTION_EXTENDED option must be specified.
    - `SMTP_OPTION_EXTENDED`: Extended SMTP commands should be used if possible. This option enables features such as authentication and delivery status notification. If this option is not specified, the library will not attempt to use any extended features. This option is automatically enabled if the connection is established on port 587 because submitting messages for delivery using this port typically requires client authentication.
    - `SMTP_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `SMTP_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `SMTP_OPTION_SECURE`: This option specifies that a secure connection should be established with the server and requires that the server support the TLS protocol. This option is the same as specifying SMTP_OPTION_SECURE_EXPLICIT, which initiates the secure session using the STARTTLS command.
    - `SMTP_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server using the STARTTLS command. Note that the server must support secure connections using the TLS protocol.
    - `SMTP_OPTION_SECURE_IMPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol, and the secure session must be negotiated immediately after the connection has been established.
    - `SMTP_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `SMTP_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `SMTP_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call SmtpGetLastError.

### Remarks

- The lpszLocalName argument should only specify a domain name if it is absolutely necessary. In most cases, it is preferable to pass this parameter as NULL or an empty string and allow the library to automatically determine the correct domain name to use. Providing an invalid domain name may cause the mail server to reject the connection.
- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAuthenticate](#smtpauthenticate-function)
- [SmtpConnect](#smtpconnect-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SmtpInitialize](#smtpinitialize-function)
- [SmtpUninitialize](#smtpuninitialize-function)

---

## SmtpAsyncSendMessage Function

Send message to the specified recipient

### Prototype

```c
UINT WINAPI SmtpAsyncSendMessage(
    HCLIENT hClient,
    LPCTSTR lpszFrom,
    LPCTSTR lpszRecipient,
    LPVOID lpvMessage,
    DWORD dwMessageSize,
    DWORD dwOptions,
    SMTPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszFrom` | `LPCTSTR` | Pointer to a string which specifies the email address of the sender. |
| `lpszRecipient` | `LPCTSTR` | Pointer to a string which specifies the recipient of the message. Multiple recipients may be specified by separating each address with a comma. |
| `lpvMessage` | `LPVOID` | Pointer to a buffer which contains the message to be delivered, or a pointer to the name of the file which contains the data to be written to the data stream. The use of this parameter depends on the value of the dwOptions parameter. |
| `dwMessageSize` | `DWORD` | An unsigned integer which specifies the length of the message in bytes. |
| `dwOptions` | `DWORD` | Specifies the source of the message data that will be written to the data stream; it may be one of the following values: |
| `lpEventProc` | `SMTPEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the SmtpEventProc callback function. This parameter may be NULL if you do not wish to implement an event handler. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. This parameter is ignored if the lpEventProc parameter is NULL. |

### Options

- **dwOptions** Parameter

    - `SMTP_MESSAGE_MEMORY`: The lpvMessage parameter specifies a pointer to an array of characters. If the value of dwMessageSize is zero, then it is assumed to be a pointer to a string.
    - `SMTP_MESSAGE_HGLOBAL`: The lpvMessage parameter specifies an HGLOBAL which contains the data to be written to the data stream. If the value of the dwMessageSize parameter is zero, then the data is assumed to be null-terminated.
    - `SMTP_MESSAGE_FILE`: The lpvMessage parameter specifies a pointer to a string which contains the name of a file. The file is opened and the contents of the file are written to the data stream. The value of the dwMessageSize parameter is ignored when this option is specified.
    - `SMTP_MESSAGE_CLIPBOARD`: The lpvMessage and dwMessageSize parameters are ignored. The current contents of the clipboard are written to the data stream.

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is non-zero integer value that specifies a unique asynchronous task identifier. If the function fails, the return value is zero. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpAsyncSendMessage function is used to send the contents of a memory buffer, file or the system clipboard to the specified recipients. This function is similar to the SmtpSendMessage function, however it uses a background worker thread and does not block the current working thread. This enables the application to continue to perform other operations while the message is being submitted to mail server for delivery.
- Because this function works asynchronously, it is important that the memory allocated for the message is not released before the asynchronous task completes. If you provide a buffer that is allocated on the stack, ensure that your code does not return from the function while the message is being submitted. This can be achieved by calling the SmtpTaskWait function or periodically calling the SmtpTaskDone function to determine if the background task has completed. If you wish to return from the calling function immediately, then you must dynamically allocate memory for the lpvMessage parameter on the heap and free that memory after the task has completed.
- If the address of an event handler is provided to this function, it is guaranteed that the handler will be invoked with the SMTP_EVENT_CONNECT event after the connection has been established, and the SMTP_EVENT_DISCONNECT event after the message has been submitted. This enables your application to know when the message is being submitted to the mail server, and immediately before the worker thread is terminated. The worker thread creates a secondary connection to the server with its own session handle. This ensures that the asynchronous operation will not interfere with the current client session. Your application can interact with this background worker thread using the client handle that is passed to the event handler.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAsyncSubmitMessage](#smtpasyncsubmitmessage-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpSubmitMessage](#smtpsubmitmessage-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpAsyncSubmitMessage Function

Compose and submit a message for delivery to the specified mail server

### Prototype

```c
UINT WINAPI SmtpAsyncSubmitMessage(
    LPSMTPSERVER lpServer,
    LPSMTPMESSAGE lpMessage,
    SMTPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpServer` | `LPSMTPSERVER` | A pointer to an SMTPSERVER structure that contains information about the mail server that the message will be submitted to for delivery. This parameter cannot be NULL and the structure members must be properly initialized prior to calling this function. |
| `lpMessage` | `LPSMTPMESSAGE` | A pointer to an SMTPMESSAGE structure that contains information about the message, including the sender, recipients and the body of the message. This parameter cannot be NULL and the structure members must be property initialized prior to calling this function. |
| `lpEventProc` | `SMTPEVENTPROC` | A pointer to the procedure-instance address of an application defined callback function. For more information about event handling and the callback function, see the description of the SmtpEventProc callback function. If this parameter is NULL, event notification is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the lpEventProc parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is non-zero integer value that specifies a unique asynchronous task identifier. If the function fails, the return value is zero. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpAsyncSubmitMessage function provides a high-level interface that enables an application to send an email message with a single function call. This function is similar to the SmtpSubmitMessage function; however, it uses a background worker thread and does not block the current working thread. This enables the application to continue to perform other operations while the message is being submitted to the mail server.
- The SMTPSERVER and SMTPMESSAGE structures are used to provide the function with information about the mail server that will accept the message and the contents of the message itself. Note that this function does not require a client session handle, and therefore it is not required that you call the SmtpConnect function prior to calling this function.
- If the address of an event handler is provided to this function, it is guaranteed that the handler will be invoked with the SMTP_EVENT_CONNECT event after the connection has been established, and the SMTP_EVENT_DISCONNECT event after the message has been submitted. This enables your application to know when the message is being submitted to the mail server, and immediately before the worker thread is terminated. The worker thread creates a secondary connection to the server with its own session handle. This ensures that the asynchronous operation will not interfere with the current client session. Your application can interact with this background worker thread using the client handle that is passed to the event handler.

### Example

```cpp
SMTPSERVER mailServer;
ZeroMemory(&mailServer, sizeof(mailServer));
mailServer.lpszHostName = _T("smtp.gmail.com");
mailServer.nHostPort = SMTP_PORT_SUBMIT;
mailServer.lpszUserName = m_strSender;
mailServer.lpszPassword = m_strPassword;
mailServer.dwOptions = SMTP_OPTION_SECURE;

SMTPMESSAGE mailMessage;
ZeroMemory(&mailMessage, sizeof(mailMessage));
mailMessage.lpszFrom = m_strSender;
mailMessage.lpszTo = m_strRecipients;
mailMessage.lpszSubject = m_strSubject;
mailMessage.lpszText = m_strMessage;

UINT nTaskId = SmtpAsyncSubmitMessage(&mailServer, &mailMessage, NULL, 0);

if (nTaskId != 0)
{
	DWORD dwError = NO_ERROR;
	DWORD dwElapsed = 0;

	// Wait for the message to be submitted
	SmtpTaskWait(nTaskId, INFINTE, &dwElapsed, &dwError);

	if (dwError == NO_ERROR)
		_tprintf(_T("SmtpAsyncSubmitMessage was successful\n"));
}
else
{
	DWORD dwError = SmtpGetLastError();
	_tprintf(_T("SmtpSubmitMessage failed with error 0x%08lx\n"), dwError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAsyncSendMessage](#smtpasyncsendmessage-function)
- [SmtpEventProc](#smtpeventproc-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpSubmitMessage](#smtpsubmitmessage-function)
- [SmtpTaskWait](#smtptaskwait-function)
- [SMTPMESSAGE](#smtpmessage-function)
- [SMTPSERVER](#smtpserver-function)

---

## SmtpAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI SmtpAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The SmtpAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should SmtpAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the SmtpCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the SmtpUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SmtpUninitialize](#smtpuninitialize-function)

---

## SmtpAuthenticate Function

Authenticate the client session with a user name and password

### Prototype

```c
INT WINAPI SmtpAuthenticate(
    HCLIENT hClient,
    UINT nAuthType,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | The handle to the client session. |
| `nAuthType` | `UINT` | An integer value which specifies which method the library should use to authenticate the client session. This parameter should be set to one of the following values: |
| `lpszUserName` | `LPCTSTR` | A null terminated string which specifies the account name for the user authorized to send mail through the server. |
| `lpszPassword` | `LPCTSTR` | A null terminated string which specifies the password to be used when authenticating the current client session. If you are using the SMTP_AUTH_XOAUTH2 or SMTP_AUTH_BEARER authentication methods, this parameter is not a password, instead it specifies the OAuth 2.0 access token provided by the mail service. |

### Options

- **nAuthType** Parameter

    - `SMTP_AUTH_LOGIN`: The client will authenticate using the AUTH LOGIN command. This encodes the username and password, however the credentials are not encrypted and it is recommended you use a secure connection. This is the default method accepted by most mail servers and is the preferred authentication type for most clients.
    - `SMTP_AUTH_PLAIN`: The client will authenticate using the AUTH PLAIN command. This encodes the username and password, however the credentials are not encrypted and it is recommended you use a secure connection. The server must support the PLAIN Simple Authentication and Security Layer (SASL) mechanism as defined in RFC 4616.
    - `SMTP_AUTH_XOAUTH2`: The client will authenticate using the AUTH XOAUTH2 command. This authentication method does not require the user password, instead the lpszPassword parameter must specify the OAuth 2.0 bearer token issued by the service provider. The application must provide a valid access token which has not expired, or this function will fail.
    - `SMTP_AUTH_BEARER`: The client will authenticate using the AUTH OAUTHBEARER command as defined in RFC 7628. This authentication method does not require the user password, instead the lpszPassword parameter must specify the OAuth 2.0 bearer token issued by the service provider. The application must provide a valid access token which has not expired, or this function will fail.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the command result code. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- To submit a mail message for delivery, virtually all public mail servers require clients to authenticate and will only accept messages from authorized users. In some cases, they may also require the sender email address match the account being used to authenticate the session. It is also typical for most public mail servers to reject authentication attempts over a standard (non-secure) connection. You should always use a secure connection whenever possible.
- All authentication methods require the mail server to support the standard service extensions for authentication as specified in the RFC 4954. The server must support the ESMTP protocol extensions and the AUTH command. A user name and password are required for authentication. If you wish to authenticate without a user password, you must use one of the OAuth 2.0 authentication methods.
- The default authentication method is SMTP_AUTH_LOGIN and this is accepted by most mail servers. It is common for mail servers to allow the SMTP_AUTH_PLAIN method as well, however it is recommended you explicitly check whether the server supports the desired authentication method by calling the SmtpGetExtendedOptions function. If you attempt to use an authentication method which is not supported by the server, this function will fail and the last error code will be set to ST_ERROR_INVALID_AUTHENTICATION_TYPE.
- You should only use an OAuth 2.0 authentication method if you understand the process of how to request the access token. Obtaining an access token requires registering your application with the mail service provider (e.g.: Microsoft or Google), getting a unique client ID associated with your application and then requesting the access token using the appropriate scope for the service. Obtaining the initial token will typically involve interactive confirmation on the part of the user, requiring they grant permission to your application to access their mail account.
- The SMTP_AUTH_XOAUTH2 and SMTP_AUTH_BEARER authentication methods are similar, but they are not interchangeable. Both use an OAuth 2.0 bearer token to authenticate the client session, but they differ in how the token is presented to the server. It is currently preferable to use the XOAUTH2 method because it is more widely available and some service providers do not yet support the OAUTHBEARER method.
- Your application should not store an OAuth 2.0 bearer token for later use. They have a relatively short lifespan, typically about an hour, and are designed to be used with that session. You should specify offline access as part of the OAuth 2.0 scope if necessary and store the refresh token provided by the service. The refresh token has a much longer validity period and can be used to obtain a new bearer token when needed.

### Example

```cpp
BOOL bExtended = FALSE;
DWORD dwOptions = 0;

// Determine which extended options and authentication methods
// are supported by this server

bExtended = SmtpGetExtendedOptions(hClient, &dwOptions);

if (bUseBearerToken)
{
	if (bExtended && (dwOptions & SMTP_EXTOPT_XOAUTH2))
	{
		INT nResult = SmtpAuthenticate(hClient, SMTP_AUTH_XOAUTH2, lpszUserName, lpszBearerToken);

		if (nResult == SMTP_ERROR)
		{
			// An error occurred during authentication; when using an
			// OAuth 2.0 bearer token, this typically means that the token
			// has expired and must be refreshed
			return;
		}
	}
	else
	{
		// The server does not support XOAUTH2
		return;
	}
}
else
{
	if (bExtended && (dwOptions & SMTP_EXTOPT_AUTHLOGIN))
	{
		INT nResult = SmtpAuthenticate(hClient, SMTP_AUTH_LOGIN, lpszUserName, lpszPassword);

		if (nResult == SMTP_ERROR)
		{
			// An error occurred during authentication
			return;
		}
	}
	else
	{
		// The server does not support AUTH LOGIN
		return;
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpGetExtendedOptions](#smtpgetextendedoptions-function)

---

## SmtpCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI SmtpCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- When the SmtpCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpIsBlocking](#smtpisblocking-function)

---

## SmtpCloseMessage Function

Close the message being composed and submit for delivery

### Prototype

```c
INT WINAPI SmtpCloseMessage(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpCloseMessage function should be called after all of the message data has been written to the data stream.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpAppendMessage](#smtpappendmessage-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)
- [SmtpWrite](#smtpwrite-function)

---

## SmtpCommand Function

Send a command to the server

### Prototype

```c
INT WINAPI SmtpCommand(
    HCLIENT hClient,
    LPCTSTR lpszCommand,
    LPCTSTR lpszParameter
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszCommand` | `LPCTSTR` | The command which will be executed by the server. |
| `lpszParameter` | `LPCTSTR` | An optional command parameter. If the command requires more than one parameter, then they should be combined into a single string, with a space separating each parameter. If the command does not accept any parameters, this value may be NULL. |

### Return Value

- **Type**: `INT`
- If the command was successful, the function returns the result code. If the command failed, the function returns SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- A list of valid commands can be found in the technical specification for the protocol. Many servers will list supported commands when the HELP command is used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpGetResultCode](#smtpgetresultcode-function)
- [SmtpGetResultString](#smtpgetresultstring-function)

---

## SmtpConnect Function

Establish a connection with a server

### Prototype

```c
HCLIENT WINAPI SmtpConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPCTSTR lpszLocalName,
    LPSECURITYCREDENTIALS lpCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number should be used. For standard connections, the default port number is 25. An alternative port is 587, which is commonly used by authenticated clients to submit messages for delivery. For implicit TLS connections, the default port number is 465. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpszLocalName` | `LPCTSTR` | A pointer to a string which specifies the domain name of the local host. This parameter can be NULL or point to an empty string, in which case the local domain name is determined automatically from the system configuration. This parameter should be used if the mail server only accepts messages from a client that identifies itself using a specific domain name. |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which is used to establish the client credentials for a secure connection to the server. The function SmtpCreateSecurityCredentials can be used to create this structure if necessary. If a standard non-secure connection is being established, or client credentials are not required by the server, this parameter can be NULL. |

### Options

- **dwOptions** Parameter

    - `SMTP_OPTION_NONE`: No additional options are specified when establishing a connection with the server. A standard, non-secure connection will be used and the client will not attempt to use extended (ESMTP) features of the protocol. Note that if the mail server requires authentication, the SMTP_OPTION_EXTENDED option must be specified.
    - `SMTP_OPTION_EXTENDED`: Extended SMTP commands should be used if possible. This option enables features such as authentication and delivery status notification. If this option is not specified, the library will not attempt to use any extended features. This option is automatically enabled if the connection is established on port 587 because submitting messages for delivery using this port typically requires client authentication.
    - `SMTP_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `SMTP_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `SMTP_OPTION_SECURE`: This option specifies that a secure connection should be established with the server and requires that the server support the TLS protocol. The client will initiate the secure session using the STARTTLS command.
    - `SMTP_OPTION_SECURE_IMPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol, and the secure session must be negotiated immediately after the connection has been established.
    - `SMTP_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `SMTP_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `SMTP_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call SmtpGetLastError.

### Remarks

- This function will block the current thread until a connection has been established or the timeout period has elapsed. To prevent it from blocking the main user interface thread, the application should create a background worker thread and establish a connection by calling SmtpConnect in that thread. If the application requires multiple simultaneous connections, it is recommended you create a worker thread for each connection.
- The lpszLocalName argument should only point to a specific domain name if it is absolutely necessary. In most cases, it is preferable to pass this parameter as NULL or an empty string and allow the library to automatically determine the correct domain name to use. Providing an invalid domain name may cause the mail server to reject the connection.
- The dwOptions argument can be used to specify the threading model that is used by the library when a connection is established. By default, the handle is initially attached to the thread that created it. From that point on, until the it is released, only the owner may call functions using that handle. The ownership of the handle may be transferred from one thread to another using the SmtpAttachThread function.
- Specifying the SMTP_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAuthenticate](#smtpauthenticate-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SmtpInitialize](#smtpinitialize-function)
- [SmtpUninitialize](#smtpuninitialize-function)

---

## SmtpCreateMessage Function

Create a new message

### Prototype

```c
INT WINAPI SmtpCreateMessage(
    HCLIENT hClient,
    LPCTSTR lpszSender,
    DWORD dwMessageSize,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszSender` | `LPCTSTR` | A pointer to a string which specifies the email address of the user sending the message. This typically corresponds to the address in the From header of the message, but it is not required that they be the same. |
| `dwMessageSize` | `DWORD` | An unsigned integer which specifies the size of the message in bytes. If the size of the message is unknown, this value should be zero. This parameter is ignored if the server does not support extended features. If the message size is larger than what the server will accept, this function will fail. Most Internet Service Providers impose a limit on the size of an email message, typically between 5 and 10 megabytes. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpCreateMessage function begins the composition of a new message to be submitted to the mail server for delivery. There are several steps that must be followed when dynamically composing a message for delivery:
- For applications that do not need to dynamically compose the message and already have the message contents stored in a file or memory buffer, the SmtpSendMessage function is the preferred method of submitting a message for delivery.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAddRecipient](#smtpaddrecipient-function)
- [SmtpAppendMessage](#smtpappendmessage-function)
- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpWrite](#smtpwrite-function)

---

## SmtpCreateSecurityCredentials Function

Allocate a structure to establish client security credentials

### Prototype

```c
BOOL WINAPI SmtpCreateSecurityCredentials(
    DWORD dwProtocol,
    DWORD dwOptions,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    LPCTSTR lpszCertStore,
    LPCTSTR lpszCertName,
    LPVOID lpvReserved,
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. This parameter is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This member is constructed by using a bitwise operator with any of the following values: |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's username. A value of NULL specifies that no username is required. Currently this parameter is not used and any value specified will be ignored. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's password. A value of NULL specifies that no password is required. This parameter is only used if a PKCS #12 (PFX) certificate file is specified and that certificate has been secured with a password. This value will be ignored the current user or local machine certificate store is specified. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The function will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the function will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the function will return an error indicating that the certificate could not be found. |
| `lpvReserved` | `LPVOID` | Pointer reserved for future use. Set it to NULL when using this function. |
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. The memory for the credentials structure will be allocated by this function and must be released by calling the SmtpDeleteSecurityCredentials function when it is no longer needed. The pointer value must be set to NULL before the function is called. It is important to note that this is a pointer to a pointer variable, not a pointer to the SECURITYCREDENTIALS structure itself. |

### Options

- **dwProtocol** Parameter

    - `SECURITY_PROTOCOL_DEFAULT`: The default security protocol will be used when establishing a connection. This option will always request the latest version of the preferred security protocol and is the recommended value. Currently the default security protocol is TLS 1.2.
    - `SECURITY_PROTOCOL_TLS`: Either the TLS 1.2 or 1.3 protocol should be used when establishing a secure connection. The correct protocol is automatically selected, based on what version of the protocol is supported by the server. If this is the only protocol specified, SSL will be excluded from the list of supported protocols. This may be necessary for some servers which reject any attempt to use the older SSL protocol and require that only TLS be used.
    - `SECURITY_PROTOCOL_TLS10`: The TLS 1.0 protocol should be used when establishing a secure connection. This version is supported on all Windows platforms, however some servers may reject connections using version 1.0 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS11`: The TLS 1.1 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. Some servers may reject connections using version 1.1 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS12`: The TLS 1.2 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. This version of TLS offers the broadest compatibility with most servers.
    - `SECURITY_PROTOCOL_TLS13`: The TLS 1.3 protocol should be used when establishing a secure connection. This is the newest version of the protocol and is only supported on Windows 11, Windows Server 2022 and later versions of Windows. If this version is not supported by the operating system, TLS 1.2 will be used instead.
- **dwOptions** Parameter

    - `CREDENTIAL_STORE_CURRENT_USER`: The library will attempt to open a store for the current user using the certificate store name provided in this structure. If no options are specified, then this is the default behavior.
    - `CREDENTIAL_STORE_LOCAL_MACHINE`: The library will attempt to open a local machine store using the certificate store name provided in this structure. If this option is specified, the library will always search for a local machine store before searching for the store by that name for the current user.
    - `CREDENTIAL_STORE_FILENAME`: The certificate store name is a file that contains the certificate and private key that will be used.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The structure that is created by this function may be used as client credentials when establishing a secure connection. This is particularly useful for programming languages other than C/C++ which may not support C structures or pointers. The pointer to the SECURITYCREDENTIALS structure can be declared as an unsigned integer variable which is passed by reference to this function, and then passed by value to the SmtpConnect function.
- A client application only needs to call this function if the server requires that the client provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions parameter should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore parameter should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12.These certificate files typically have an extension of .pfx or .p12. If a password was specified when the certificate was created, it must be provided in the lpszPassword parameter or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword parameters are values which are used to access the certificate store or client certificate. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session. In most cases, these parameters will be NULL.

### Example

```cpp
LPSECURITYCREDENTIALS lpSecCred = NULL;
SmtpCreateSecurityCredentials(SECURITY_PROTOCOL_DEFAULT,
							0,
							NULL,
							NULL,
							lpszCertStore,
							lpszCertName,
							NULL,
							&lpSecCred);

hClient = SmtpConnect(lpszHostName,
					SMTP_PORT_SECURE,
					SMTP_TIMEOUT,
					SMTP_OPTION_EXTENDED | SMTP_OPTION_SECURE,
					lpSecCred);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpDeleteSecurityCredentials](#smtpdeletesecuritycredentials-function)
- [SmtpGetSecurityInformation](#smtpgetsecurityinformation-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)

---

## SmtpDeleteSecurityCredentials Function

Delete the specified client security credentials

### Prototype

```c
VOID WINAPI SmtpDeleteSecurityCredentials(
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. On exit from the function, the pointer will be NULL. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- This function can be used to release the memory allocated to the client or server credentials after a secure connection has been terminated.

### Example

```cpp
if (lpSecCred)
	SmtpDeleteSecurityCredentials(&lpSecCred);

SmtpUninitialize();
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpCreateSecurityCredentials](#smtpcreatesecuritycredentials-function)
- [SmtpUninitialize](#smtpuninitialize-function)

---

## SmtpDisableEvents Function

Disable all event notification, including event callbacks

### Prototype

```c
INT WINAPI SmtpDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpDisableEvents function is used to disable event message posting for the specified client session. Although this will immediately prevent any new events from being generated, it is possible that messages could be waiting in the message queue. Therefore, an application must be prepared to handle client event messages after this function has been called.
- This function is automatically called if the client has event notification enabled, and the SmtpDisconnect function is called. The same issues regarding outstanding event messages also applies in this situation, requiring that the application handle event messages that may reference a client handle that is no longer valid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpEnableEvents](#smtpenableevents-function)
- [SmtpRegisterEvent](#smtpregisterevent-function)

---

## SmtpDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI SmtpDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpEnableTrace](#smtpenabletrace-function)

---

## SmtpDisconnect Function

Disconnect from the current server

### Prototype

```c
INT WINAPI SmtpDisconnect(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpUninitialize](#smtpuninitialize-function)

---

## SmtpEnableEvents Function

Enable event handling by the library

### Prototype

```c
INT WINAPI SmtpEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the window which will receive the client notification messages. This parameter must specify a valid window handle. If a NULL handle is specified, event notification will be disabled. |
| `uEventMsg` | `UINT` | The message that is received when a client event occurs. To avoid conflict with standard Windows messages, this value must be greater than WM_USER (1024) or an error will be returned. If the hEventWnd parameter is NULL, this value should be WM_NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpEnableEvents function is used to request that notification messages be posted to the specified window whenever a client event occurs. This allows an application to monitor the status of different client operations, such as a file transfer. The client must create a window message handler, which processes the various events. The wParam argument will contain the client handle, the low word of the lParam argument will contain the event identifier, and the high word will contain any error code. If no error has occurred, the high word will have a value of zero. The following events may be generated:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpDisableEvents](#smtpdisableevents-function)
- [SmtpRegisterEvent](#smtpregisterevent-function)

---

## SmtpEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI SmtpEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | Name of the trace log file. If this parameter is NULL or empty, the file CSTRACE.LOG is used. The directory for CSTRACE.LOG is given by the TEMP environment variable, if it is defined; otherwise, the directory given by the TMP environment variable is used, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_DEFAULT`: All function calls and return values are written to the trace file. The actual data being sent or received will not be logged. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file. Those errors which are not fatal and only indicate a warning will not be logged.
    - `TRACE_WARNING`: Only those function calls which fail, or return values which indicate a warning, are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError.

### Remarks

- When trace logging is enabled, the logfile is opened, appended to and closed for each socket function call. Using the same logfile name, you can do the same in your application to add additional information to the logfile if needed. This can provide an application-level context for the entries made by the library. Make sure that the logfile is closed after the data has been written.
- The TRACE_HEXDUMP option can produce very large logfiles, since all data that is being sent and received by the application is logged. To reduce the size of the file, you can enable and disable logging around limited sections of code that you wish to analyze.
- All of the SocketTools networking components that use the Windows Sockets API support logging. If you are using multiple components, you only need to enable tracing once in your application or once per thread in a multithreaded application.
- To redistribute an application that includes logging functionality, the cstrcv11.dll library must be included as part of the installation package. This library provides the trace logging features, and if it is not available the SmtpEnableTrace function will fail. Note that the trace logging library is a standard Windows DLL and does not need to be registered, it only needs to be redistributed with your application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpDisableTrace](#smtpdisabletrace-function)

---

## SmtpEnumTasks Function

Return a list of asynchronous tasks

### Prototype

```c
INT WINAPI SmtpEnumTasks(
    UINT lpTasks,
    INT nMaxTasks,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpTasks` | `UINT` | A pointer to an array of unsigned integer values that will contain unique task identifiers when the function returns. If this parameter is NULL, the function will return the number of tasks. |
| `nMaxTasks` | `INT` | An integer value that specifies the maximum number of task identifiers that may be copied into the lpTasks array. If the lpTasks parameter is NULL, this value must be zero. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies the type of asynchronous tasks that may be returned by this function. It may be a combination of the following values: |

### Options

- **dwOptions** Parameter

    - `SMTP_TASK_DEFAULT`: The list of asynchronous task IDs should include both active and suspended tasks. This option is the same as specifying both the SMTP_TASK_ACTIVE and SMTP_TASK_SUSPENDED options.
    - `SMTP_TASK_ACTIVE`: The list of asynchronous task IDs should include those tasks which are currently active. An active task represents a background connection to a server that is in the process of performing the requested action, such as uploading or downloading a file.
    - `SMTP_TASK_SUSPENDED`: The list of asynchronous task IDs should include those tasks which have been suspended. A suspended task represents a background connection that has been established, but the worker thread is not scheduled for execution.
    - `SMTP_TASK_FINISHED`: The list of asynchronous task IDs should include those tasks which have completed recently.

### Return Value

- **Type**: `INT`
- If the function is successful, the return value is the number of task identifiers copied into the provided array. If there are no tasks which match the requested criteria, the return value is zero. A return value of SMTP_ERROR indicates an error has occurred. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpEnumTasks function can be used to obtain a list of numeric identifiers that represent the asynchronous tasks that have been started or those that have completed. These task IDs are used by other functions to reference the background worker thread that has been created and obtain status information for the task. For example, the SmtpTaskDone function can be used to determine if a particular task has completed, and the SmtpTaskWait function can be used to wait for a task to complete and return an error status code if the background operation failed.
- There is an internal limit of 128 asynchronous tasks per process that may be active at any one time. When a task completes, the status information about that task is maintained for period of time after the task has completed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpEventProc Function

Process events generated by the client

### Prototype

```c
VOID __cdecl SmtpEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the SmtpRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the SmtpRegisterEvent function. This callback function is also used by asynchronous tasks to notify the application when the task has started and completed. The SmtpEventProc function is a placeholder for the application-defined function name.
- If the callback function is invoked by an asynchronous task, it will execute in the context of the worker thread that is managing the client session. You must ensure that any access to global or static variables are synchronized, otherwise the results may be unpredictable. It is recommended that you do not declare any static variables within the callback function itself.
- If the application has a graphical user interface, you should never attempt to directly modify a UI control from within the callback function for an asynchronous task. Controls should only be modified by the same UI thread that created their window. One common approach to resolve this issue is to post a user-defined message to the main window to signal that the user interface needs to be updated. The message handler would then process the user-defined message and update the user interface as needed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpDisableEvents](#smtpdisableevents-function)
- [SmtpEnableEvents](#smtpenableevents-function)
- [SmtpFreezeEvents](#smtpfreezeevents-function)
- [SmtpRegisterEvent](#smtpregisterevent-function)

---

## SmtpExpandAddress Function

Expand the specified address

### Prototype

```c
INT WINAPI SmtpExpandAddress(
    HCLIENT hClient,
    LPCTSTR lpszMailingList,
    LPTSTR lpszAddresses,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszMailingList` | `LPCTSTR` | Points to a string which specifies the mailing list that the server should expand into full addresses. |
| `lpszAddresses` | `LPTSTR` | Points to a buffer that the expanded addresses will be copied into. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the server result code. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpExpandAddress function requests that the server expand the specified email address. Typically this is used to expand aliases which refer to a mailing list, returning all of the members of that list. A server may not support this command, or may restrict its usage. An application should not depend on the ability to expand addresses.
- This function cannot be called while a mail message is being composed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAddRecipient](#smtpaddrecipient-function)
- [SmtpVerifyAddress](#smtpverifyaddress-function)

---

## SmtpFreezeEvents Function

Suspend and resume event handling by the client

### Prototype

```c
INT WINAPI SmtpFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four events of the same type occur, once event handling is resumed only one of those events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpDisableEvents](#smtpdisableevents-function)
- [SmtpEnableEvents](#smtpenableevents-function)
- [SmtpRegisterEvent](#smtpregisterevent-function)

---

## SmtpGetCurrentDate Function

Return the current date and time

### Prototype

```c
INT WINAPI SmtpGetCurrentDate(
    LPTSTR lpszDateString,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszDateString` | `LPTSTR` | Pointer to a string buffer that will contain the current date and time when the function returns. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied into the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the buffer, not including the null-terminator. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The date value that is returned is adjusted for the local timezone.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)

---

## SmtpGetDeliveryOptions Function

Return the delivery options for the current session

### Prototype

```c
BOOL WINAPI SmtpGetDeliveryOptions(
    HCLIENT hClient,
    LPDWORD lpdwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpdwOptions` | `LPDWORD` | Address of a variable that will be set to the current delivery options. This bitmask is created by combining one or more of the following values with a bitwise Or operator: |

### Options

- **lpdwOptions** Parameter

    - `SMTP_NOTIFY_NEVER`: Never return information about the success or failure of the message delivery process.
    - `SMTP_NOTIFY_SUCCESS`: Return a message to the sender if the message has been successfully delivered to the recipient's mail server.
    - `SMTP_NOTIFY_FAILURE`: Return a message to the sender if the message could not be delivered to the recipient's mail server.
    - `SMTP_NOTIFY_DELAY`: Return a message to the sender if delivery of the message was delayed.
    - `SMTP_RETURN_HEADERS`: Return only the message headers to the sender.
    - `SMTP_RETURN_MESSAGE`: Return the complete message headers and body to the sender.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero value. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpGetDeliveryOptions function returns the current delivery options for the client session. Note that delivery options are only available on those mail servers which support delivery status notification (DSN) using the extended SMTP protocol. The client must connect specifying SMTP_OPTION_EXTENDED in order to use extended server options.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpGetExtendedOptions](#smtpgetextendedoptions-function)
- [SmtpSetDeliveryOptions](#smtpsetdeliveryoptions-function)

---

## SmtpGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI SmtpGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description should be returned. If this value is zero, then the description of the last error will be returned. If the last error code is zero, indicating no error, then this function will return zero. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. It is recommended that this buffer be at least 128 characters in length. If a NULL pointer is specified, then no message will be returned but the function will return the length of the error string, not including the terminating null byte. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is 0, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpGetLastError](#smtpgetlasterror-function)
- [SmtpGetResultCode](#smtpgetresultcode-function)
- [SmtpGetResultString](#smtpgetresultstring-function)
- [SmtpSetLastError](#smtpsetlasterror-function)

---

## SmtpGetExtendedOptions Function

Return the extended options supported by the server

### Prototype

```c
BOOL WINAPI SmtpGetExtendedOptions(
    HCLIENT hClient,
    LPDWORD lpdwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpdwOptions` | `LPDWORD` | Address of a variable that will be set to the current server options. This bitmask is created by combining one or more of the following values with a bitwise Or operator: |

### Options

- **lpdwOptions** Parameter

    - `SMTP_EXTOPT_EXPN`: The server supports address expansion using the EXPN command. The SmtpExpandAddress function can be used to expand addresses, which typically returns the email addresses associated with a mailing list. Most public mail servers restrict or disable this functionality because it can present a security risk. If a server does permit the use of the command, it is often limited to specific authorized users.
    - `SMTP_EXTOPT_VRFY`: The server supports verification of addresses using the VRFY command. The SmtpVerifyAddress function can be used to verify addresses. Most public mail servers restrict the ability for clients to verify email addresses to prevent potential abuse. If a server does permit the use of the command, it is often limited to specific authorized users.
    - `SMTP_EXTOPT_DSN`: The server supports delivery status notification (DSN) which allows the sender to be notified when a message has been delivered, or when an error occurs during the delivery process. The SmtpSetDeliveryOptions function can be used to specify the delivery options to be used in the current session.
    - `SMTP_EXTOPT_SIZE`: The server supports the use of the SIZE parameter, which enables the client to determine the maximum message size that may be delivered through the server. Most public mail servers impose a limit of on the total size of a message, including any encoded attachments.
    - `SMTP_EXTOPT_ETRN`: The server supports the use of the ETRN command, instructing the server to start processing its message queues for a specific host. Most public mail servers do not support this capability and its use has been deprecated.
    - `SMTP_EXTOPT_8BITMIME`: The server supports the delivery of messages that contain characters with the high bit set. Most servers support this option, however it is recommended that you encode any message text which contains non-ASCII characters to ensure the broadest compatibility with other servers and clients.
    - `SMTP_EXTOPT_STARTTLS`: The server supports explicit TLS sessions. This extended option is used internally to determine how secure connections should be established, and if a secure connection can be made using the standard submission port.
    - `SMTP_EXTOPT_UTF8`: The server supports UTF-8 encoding in email addresses and the message envelope. Not all mail servers will have this extended capability enabled, and applications should not depend on being able to provide internationalized user and domain names unless this option bit flag has been set.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero value. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpGetExtendedOptions function returns the extended options supported by the server. The use of extended options requires that the server support the ESMTP protocol, and that the client connect using the SMTP_OPTION_EXTENDED option.
- You should check these options prior to calling SmtpAuthenticate to determine which authentication methods are acceptable to the server. If you wish to use an OAuth 2.0 bearer token, always check to make sure either the SMTP_EXTOPT_XOAUTH2 or SMTP_EXTOPT_BEARER bit flags are set in the options value returned by this function.

### Example

```cpp
BOOL bExtended = FALSE;
DWORD dwOptions = 0;

// Determine which extended options and authentication methods
// are supported by this server

bExtended = SmtpGetExtendedOptions(hClient, &dwOptions);

if (bExtended && (dwOptions & SMTP_EXTOPT_XOAUTH2))
{
	INT nResult = SmtpAuthenticate(hClient, SMTP_AUTH_XOAUTH2, lpszUserName, lpszBearerToken);

	if (nResult == SMTP_ERROR)
	{
		// An error occurred during authentication; when using an
		// OAuth 2.0 bearer token, this typically means that the token
		// has expired and must be refreshed
		return;
	}
}
else
{
	// The server does not support XOAUTH2
	return;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpAuthenticate](#smtpauthenticate-function)
- [SmtpConnect](#smtpconnect-function)
- [SmtpGetDeliveryOptions](#smtpgetdeliveryoptions-function)
- [SmtpSetDeliveryOptions](#smtpsetdeliveryoptions-function)

---

## SmtpGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI SmtpGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the calling thread's last error code value. Functions set this value by calling the SmtpSetLastError function. The return value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- You should call the SmtpGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call SmtpSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value error code such as FALSE, NULL, INVALID_CLIENT or SMTP_ERROR. Those functions which call SmtpSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetErrorString](#smtpgeterrorstring-function)
- [SmtpSetLastError](#smtpsetlasterror-function)

---

## SmtpGetResultCode Function

Return the result code from the previous command

### Prototype

```c
INT WINAPI SmtpGetResultCode(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the result code. If the function fails, it returns SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The following result codes may be returned by the SMTP server:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpCommand](#smtpcommand-function)
- [SmtpGetResultString](#smtpgetresultstring-function)

---

## SmtpGetResultString Function

Return the result string from the previous command

### Prototype

```c
INT WINAPI SmtpGetResultString(
    HCLIENT hClient,
    LPTSTR lpszResult,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszResult` | `LPTSTR` | A pointer to the buffer that will contain the result string returned by the server. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the result string buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the result string. If a value of zero is returned, this means that no result string was sent by the server. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpGetResultString function is most useful when an error occurs because the server will typically include a brief description of the cause of the error. This can then be parsed by the application or displayed to the user. The result string is updated each time the client sends a command to the server and then calls SmtpGetResultCode to obtain the result code for the operation.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpCommand](#smtpcommand-function)
- [SmtpGetResultCode](#smtpgetresultcode-function)

---

## SmtpGetSecurityInformation Function

Return security information about the current client connection

### Prototype

```c
BOOL WINAPI SmtpGetSecurityInformation(
    HCLIENT hClient,
    LPSECURITYINFO lpSecurityInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpSecurityInfo` | `LPSECURITYINFO` | A pointer to a SECURITYINFO structure which contains information about the current client connection. The dwSize member of this structure must be initialized to the size of the structure before passing the address of the structure to this function. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError.

### Remarks

- This function is used to obtain security related information about the current client connection to the server. It can be used to determine if a secure connection has been established, what security protocol was selected, and information about the server certificate. Note that a secure connection has not been established, the dwProtocol structure member will contain the value SECURITY_PROTOCOL_NONE.

### Example

```cpp
// Notifies the user if the connection is secure or not
SECURITYINFO securityInfo;

securityInfo.dwSize = sizeof(SECURITYINFO);
if (SmtpGetSecurityInformation(hClient, &securityInfo))
{
	if (securityInfo.dwProtocol == SECURITY_PROTOCOL_NONE)
	{
		MessageBox(NULL, _T("The connection is not secure"),
						_T("Connection"), MB_OK);
	}
	else
	{
		if (securityInfo.dwCertStatus == SECURITY_CERTIFICATE_VALID)
		{
			MessageBox(NULL, _T("The connection is secure"),
							_T("Connection"), MB_OK);
		}
		else
		{
			MessageBox(NULL, _T("The server certificate not valid"),
							_T("Connection"), MB_OK);
		}
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SECURITYINFO](#securityinfo-function)

---

## SmtpGetStatus Function

Return the current status of the client

### Prototype

```c
INT WINAPI SmtpGetStatus(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the client status code. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpGetStatus function returns a numeric code which identifies the current state of the client session. The following values may be returned:
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpIsBlocking](#smtpisblocking-function)
- [SmtpIsConnected](#smtpisconnected-function)
- [SmtpIsReadable](#smtpisreadable-function)
- [SmtpIsWritable](#smtpiswritable-function)

---

## SmtpGetTaskError Function

Return the last error code for the specified asynchronous task

### Prototype

```c
DWORD WINAPI SmtpGetTaskError(
    UINT nTaskId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTaskId` | `UINT` | The task identifier. |

### Return Value

- **Type**: `DWORD`
- If the asynchronous task has completed successfully, this function returns a value of zero. A non-zero return value indicates an error has occurred.

### Remarks

- The SmtpGetTaskError function returns the last error code associated with the specified asynchronous task. If the task completed successfully, the return value will be zero. If the task is still active, the function will return the error ST_ERROR_TASK_ACTIVE. If the task has been suspended, the function will return ST_ERROR_TASK_SUSPENDED. Any other value indicates that the task completed, but the operation has failed and the error code will specify the cause of the failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskAbort](#smtptaskabort-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpGetTaskId Function

Return the unique task identifier associated with the specified client session

### Prototype

```c
UINT WINAPI SmtpGetTaskId(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is non-zero integer value that specifies a unique asynchronous task identifier. If the client handle is not associated with an asynchronous task, the function will return a value of zero.

### Remarks

- The SmtpGetTaskId function will return the task ID that is associated with a client session. This is a unique unsigned integer value that references the worker thread that was created to manage the asynchronous client session. This function should only be called within an event handler that is invoked by a background task that has been started using a function such as SmtpAsyncGetFile.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskAbort](#smtptaskabort-function)
- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpGetTimeout Function

Return the number of seconds until an operation times out

### Prototype

```c
INT WINAPI SmtpGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpSetTimeout](#smtpsettimeout-function)

---

## SmtpGetTransferStatus Function

Return data transfer statistics

### Prototype

```c
INT WINAPI SmtpGetTransferStatus(
    HCLIENT hClient,
    LPSMTPTRANSFERSTATUS lpStatus
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpStatus` | `LPSMTPTRANSFERSTATUS` | A pointer to an SMTPTRANSFERSTATUS structure which contains information about the status of the message being submitted for delivery. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpGetTransferStatus function returns information about the current message being submitted, including the average number of bytes transferred per second and the estimated amount of time until the transfer completes. If there is no message currently being submitted, this function will return the status of the last successful submission made by the client.
- In a multithreaded application, any thread in the current process may call this function to obtain the status of a submission for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpEnableEvents](#smtpenableevents-function)
- [SmtpGetStatus](#smtpgetstatus-function)
- [SmtpRegisterEvent](#smtpregisterevent-function)

---

## SmtpInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI SmtpInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that SmtpInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to SmtpUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the SmtpInitialize or SmtpUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SmtpUninitialize](#smtpuninitialize-function)

---

## SmtpIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI SmtpIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns a non-zero value. If the client is not performing a blocking operation, or the client handle is invalid, the function function returns zero.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The SmtpIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action such as warning the user that they must wait for the operation to complete.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpCancel](#smtpcancel-function)
- [SmtpIsConnected](#smtpisconnected-function)
- [SmtpIsReadable](#smtpisreadable-function)
- [SmtpIsWritable](#smtpiswritable-function)

---

## SmtpIsConnected Function

Determine if the client is connected to the server

### Prototype

```c
BOOL WINAPI SmtpIsConnected(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is connected to a server, the function returns a non-zero value. If the client is not connected, or the client handle is invalid, the function returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpIsBlocking](#smtpisblocking-function)
- [SmtpIsReadable](#smtpisreadable-function)
- [SmtpIsWritable](#smtpiswritable-function)

---

## SmtpIsReadable Function

Determine if data can be read from the server

### Prototype

```c
BOOL WINAPI SmtpIsReadable(
    HCLIENT hClient,
    INT nTimeout,
    LPDWORD lpdwAvail
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |
| `lpdwAvail` | `LPDWORD` | A pointer to an unsigned integer which will contain the number of bytes available to read. This parameter may be NULL if this information is not required. |

### Return Value

- **Type**: `BOOL`
- If the client can read data from the server within the specified timeout period, the function returns a non-zero value. If the client cannot read any data, the function returns zero.

### Remarks

- On some platforms, this value will not exceed the size of the receive buffer (typically 64K bytes). Because of differences between TCP/IP stack implementations, it is not recommended that your application exclusively depend on this value to determine the exact number of bytes available. Instead, it should be used as a general indicator that there is data available to be read.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetStatus](#smtpgetstatus-function)
- [SmtpIsBlocking](#smtpisblocking-function)
- [SmtpIsConnected](#smtpisconnected-function)
- [SmtpIsWritable](#smtpiswritable-function)
- [SmtpWrite](#smtpwrite-function)

---

## SmtpIsWritable Function

Determine if data can be written to the server

### Prototype

```c
BOOL WINAPI SmtpIsWritable(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |

### Return Value

- **Type**: `BOOL`
- If the client can write data to the server within the specified timeout period, the function returns a non-zero value. If the client cannot write any data, the function returns zero.

### Remarks

- Although this function can be used to determine if some amount of data can be sent to the remote process it does not indicate the amount of data that can be written without blocking the client. In most cases, it is recommended that large amounts of data be broken into smaller logical blocks, typically some multiple of 512 bytes in length.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetStatus](#smtpgetstatus-function)
- [SmtpIsBlocking](#smtpisblocking-function)
- [SmtpIsConnected](#smtpisconnected-function)
- [SmtpIsReadable](#smtpisreadable-function)
- [SmtpWrite](#smtpwrite-function)

---

## SmtpRegisterEvent Function

Register an event handler for the specified event

### Prototype

```c
INT WINAPI SmtpRegisterEvent(
    HCLIENT hClient,
    UINT nEvent,
    SMTPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One of the following values may be used: |
| `lpEventProc` | `SMTPEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the SmtpEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEvent** Parameter

    - `SMTP_EVENT_CONNECT`: The connection to the server has completed.
    - `SMTP_EVENT_DISCONNECT`: The server has closed the connection to the client. The client should read any remaining data and disconnect.
    - `SMTP_EVENT_READ`: Data is available to read by the client. No additional messages will be posted until the client has read at least some of the data. This event is only generated if the calling process is in asynchronous mode.
    - `SMTP_EVENT_WRITE`: The client can now write data. This notification is sent after a connection has been established, or after a previous attempt to write data has failed because it would result in a blocking operation. This event is only generated if the client is in asynchronous mode.
    - `SMTP_EVENT_TIMEOUT`: The client has timed out while waiting for a response from the server. Note that under some circumstances this event can be generated for a non-blocking connection, such as when the client is establishing a secure connection.
    - `SMTP_EVENT_CANCEL`: The client has canceled the current operation.
    - `SMTP_EVENT_COMMAND`: The client has processed a command that was sent to the server. The result code and result string can be used to determine if the response to the command.
    - `SMTP_EVENT_PROGRESS`: This event notification is sent periodically during lengthy blocking operations, such as retrieving a complete message from the server.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpRegisterEvent function associates a callback function with a specific event. The event handler is an SmtpEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- This function is typically used to register an event handler that is invoked while a message is being submitted to the server for delivery. The SMTP_EVENT_PROGRESS event will only be generated periodically during the transfer to ensure the application is not flooded with event notifications. It is guaranteed that at least one SMTP_EVENT_PROGRESS notification will occur at the beginning of the transfer, and one at the end of the transfer when it has completed.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpDisableEvents](#smtpdisableevents-function)
- [SmtpEnableEvents](#smtpenableevents-function)
- [SmtpEventProc](#smtpeventproc-function)
- [SmtpFreezeEvents](#smtpfreezeevents-function)

---

## SmtpReset Function

Reset the client and return to a command state

### Prototype

```c
INT WINAPI SmtpReset(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The client cannot be reset while in a blocked state.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpCancel](#smtpcancel-function)
- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)
- [SmtpIsBlocking](#smtpisblocking-function)

---

## SmtpSendMessage Function

Send message to the specified recipient

### Prototype

```c
INT WINAPI SmtpSendMessage(
    HCLIENT hClient,
    LPCTSTR lpszFrom,
    LPCTSTR lpszRecipient,
    LPVOID lpvMessage,
    DWORD dwMessageSize,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszFrom` | `LPCTSTR` | Pointer to a string which specifies the email address of the sender. |
| `lpszRecipient` | `LPCTSTR` | Pointer to a string which specifies the recipient of the message. Multiple recipients may be specified by separating each address with a comma. |
| `lpvMessage` | `LPVOID` | Pointer to a buffer which contains the message to be delivered, or a pointer to the name of the file which contains the data to be written to the data stream. The use of this parameter depends on the value of the dwOptions parameter. |
| `dwMessageSize` | `DWORD` | An unsigned integer which specifies the length of the message in bytes. |
| `dwOptions` | `DWORD` | Specifies the source of the message data that will be written to the data stream; it may be one of the following values: |

### Options

- **dwOptions** Parameter

    - `SMTP_MESSAGE_MEMORY`: The lpvMessage parameter specifies a pointer to an array of characters. If the value of dwMessageSize is zero, then it is assumed to be a pointer to a string.
    - `SMTP_MESSAGE_HGLOBAL`: The lpvMessage parameter specifies an HGLOBAL which contains the data to be written to the data stream. If the value of the dwMessageSize parameter is zero, then the data is assumed to be null-terminated.
    - `SMTP_MESSAGE_FILE`: The lpvMessage parameter specifies a pointer to a string which contains the name of a file. The file is opened and the contents of the file are written to the data stream. The value of the dwMessageSize parameter is ignored when this option is specified.
    - `SMTP_MESSAGE_CLIPBOARD`: The lpvMessage and dwMessageSize parameters are ignored. The current contents of the clipboard are written to the data stream.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the result code from the server. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpSendMessage function is used to send the contents of a memory buffer, file or the system clipboard to the specified recipients. The message must be in the standard format as described in RFC 822 or a MIME multipart message. The MIME API can be used to compose and export a message in the correct format.
- This protocol is only concerned with the delivery of a message and not its contents. Header fields in the message are not parsed to determine the recipients. This recipient parameter should be a concatenation of all recipients, including carbon copies and blind carbon copies, with each address separated with a comma.
- This function will cause the current thread to block until the complete message has been delivered, a timeout occurs or the operation is canceled. During the transfer, the SMTP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling SmtpEnableEvents, or by registering a callback function using the SmtpRegisterEvent function.
- To determine the current status of the transaction while it is in progress, use the SmtpGetTransferStatus function.
- An alternative approach to creating a message without using the MIME API is the SmtpSubmitMessage function. It accepts two structure parameters which define the message contents and the connection information for the mail server. This enables the application to compose the message and submit it for delivery in a single function call.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAppendMessage](#smtpappendmessage-function)
- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpGetTransferStatus](#smtpgettransferstatus-function)
- [SmtpSubmitMessage](#smtpsubmitmessage-function)

---

## SmtpSetDeliveryOptions Function

Set the delivery options for the current session

### Prototype

```c
BOOL WINAPI SmtpSetDeliveryOptions(
    HCLIENT hClient,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwOptions` | `DWORD` | A bitmask that defines the current delivery options. This value is created by combining one or more of the following constants with a bitwise Or operator: |

### Options

- **dwOptions** Parameter

    - `SMTP_NOTIFY_NEVER`: Never return information about the success or failure of the message delivery process.
    - `SMTP_NOTIFY_SUCCESS`: Return a message to the sender if the message has been successfully delivered to the recipient's mail server.
    - `SMTP_NOTIFY_FAILURE`: Return a message to the sender if the message could not be delivered to the recipient's mail server.
    - `SMTP_NOTIFY_DELAY`: Return a message to the sender if delivery of the message was delayed.
    - `SMTP_RETURN_HEADERS`: Return only the message headers to the sender.
    - `SMTP_RETURN_MESSAGE`: Return the complete message headers and body to the sender.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero value. If the function fails, the return value is zero. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpSetDeliveryOptions function sets the current delivery options for the client session. Note that delivery options are only available on those mail servers which support delivery status notification (DSN) using the extended SMTP protocol. The client must connect specifying SMTP_OPTION_EXTENDED in order to use extended server options.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetDeliveryOptions](#smtpgetdeliveryoptions-function)
- [SmtpGetExtendedOptions](#smtpgetextendedoptions-function)

---

## SmtpSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI SmtpSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the caller. A value of zero clears the last error code. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value error code such as FALSE, NULL, INVALID_CLIENT or SMTP_ERROR. Those functions which call SmtpSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the SmtpGetLastError function. The use of SmtpGetLastError is optional. An application can call it to find out the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetErrorString](#smtpgeterrorstring-function)
- [SmtpGetLastError](#smtpgetlasterror-function)

---

## SmtpSetTimeout Function

Set the number of seconds until an operation times out

### Prototype

```c
INT WINAPI SmtpSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds to wait for a blocking operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The timeout value is only used with blocking client connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetTimeout](#smtpgettimeout-function)

---

## SmtpSubmitMessage Function

Compose and submit a message for delivery to the specified mail server

### Prototype

```c
INT WINAPI SmtpSubmitMessage(
    LPSMTPSERVER lpServer,
    LPSMTPMESSAGE lpMessage,
    SMTPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpServer` | `LPSMTPSERVER` | A pointer to an SMTPSERVER structure that contains information about the mail server that the message will be submitted to for delivery. This parameter cannot be NULL and the structure members must be properly initialized prior to calling this function. |
| `lpMessage` | `LPSMTPMESSAGE` | A pointer to an SMTPMESSAGE structure that contains information about the message, including the sender, recipients and the body of the message. This parameter cannot be NULL and the structure members must be property initialized prior to calling this function. |
| `lpEventProc` | `SMTPEVENTPROC` | A pointer to the procedure-instance address of an application defined callback function. For more information about event handling and the callback function, see the description of the SmtpEventProc callback function. If this parameter is NULL, event notification is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the lpEventProc parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpSubmitMessage function provides a high-level interface that enables an application to send an email message with a single function call. The SMTPSERVER and SMTPMESSAGE structures are used to provide the function with information about the mail server that will accept the message and the contents of the message itself. Note that this function does not require a client session handle, and therefore it is not required that you call the SmtpConnect function prior to calling this function.
- If you need to specify additional custom headers in the message that is submitted for delivery, you should use the SmtpSubmitMessageEx function. That version of the function uses an extended version of the message structure which will allow you to define custom headers to be included in the message.
- This function will cause the calling thread to block until the message has been submitted for delivery, an error occurs or the connection to the mail server times out. If an event handler is specified, then the callback function will be periodically invoked as the message is being sent. For large messages, the SMTP_EVENT_PROGRESS event can be used to monitor the submission process and update the user interface. The SmtpGetTransferStatus function can be used within the callback function to obtain information about the current status of the submission.

### Example

```cpp
SMTPSERVER mailServer;
ZeroMemory(&mailServer, sizeof(mailServer));
mailServer.lpszHostName = _T("smtp.gmail.com");
mailServer.nHostPort = SMTP_PORT_SUBMIT;
mailServer.lpszUserName = m_strSender;
mailServer.lpszPassword = m_strPassword;
mailServer.dwOptions = SMTP_OPTION_SECURE;

SMTPMESSAGE mailMessage;
ZeroMemory(&mailMessage, sizeof(mailMessage));
mailMessage.lpszFrom = m_strSender;
mailMessage.lpszTo = m_strRecipients;
mailMessage.lpszSubject = m_strSubject;
mailMessage.lpszText = m_strMessage;

INT nResult = SmtpSubmitMessage(&mailServer, &mailMessage, NULL, 0);

if (nResult != SMTP_ERROR)
	_tprintf(_T("SmtpSubmitMessage was successful\n"));
else
{
	DWORD dwError = SmtpGetLastError();
	_tprintf(_T("SmtpSubmitMessage failed with error 0x%08lx\n"), dwError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpEventProc](#smtpeventproc-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpSubmitMessageEx](#smtpsubmitmessageex-function)
- [SMTPMESSAGE](#smtpmessage-function)
- [SMTPSERVER](#smtpserver-function)

---

## SmtpSubmitMessageEx Function

Compose and submit a message for delivery with additional options

### Prototype

```c
INT WINAPI SmtpSubmitMessageEx(
    LPSMTPSERVER lpServer,
    LPSMTPMESSAGEEX lpMessageEx,
    DWORD dwReserved,
    SMTPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpServer` | `LPSMTPSERVER` | A pointer to an SMTPSERVER structure that contains information about the mail server that the message will be submitted to for delivery. This parameter cannot be NULL and the structure members must be properly initialized prior to calling this function. |
| `lpMessageEx` | `LPSMTPMESSAGEEX` | A pointer to an SMTPMESSAGEEX structure that contains information about the message, including the sender, recipients and the body of the message. This parameter cannot be NULL and the structure members must be property initialized prior to calling this function. |
| `dwReserved` | `DWORD` | An integer value that is reserved for future use. This value must be zero or the function will fail. |
| `lpEventProc` | `SMTPEVENTPROC` | A pointer to the procedure-instance address of an application defined callback function. For more information about event handling and the callback function, see the description of the SmtpEventProc callback function. If this parameter is NULL, event notification is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the lpEventProc parameter is NULL, this value should be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpSubmitMessageEx function provides a high-level interface that enables an application to send an email message with a single function call. The SMTPSERVER and SMTPMESSAGEEX structures are used to provide the function with information about the mail server that will accept the message and the contents of the message itself. Note that this function does not require a client session handle, and therefore it is not required that you call the SmtpConnect function prior to calling this function.
- This function will cause the calling thread to block until the message has been submitted for delivery, an error occurs or the connection to the mail server times out. If an event handler is specified, then the callback function will be periodically invoked as the message is being sent. For large messages, the SMTP_EVENT_PROGRESS event can be used to monitor the submission process and update the user interface. The SmtpGetTransferStatus function can be used within the callback function to obtain information about the current status of the submission.

### Example

```cpp
SMTPSERVER mailServer;
ZeroMemory(&mailServer, sizeof(mailServer));
mailServer.lpszHostName = _T("smtp.gmail.com");
mailServer.nHostPort = SMTP_PORT_SUBMIT;
mailServer.lpszUserName = m_strSender;
mailServer.lpszPassword = m_strPassword;
mailServer.dwOptions = SMTP_OPTION_SECURE;

SMTPMESSAGEEX mailMessageEx;
ZeroMemory(&mailMessageEx, sizeof(mailMessageEx));
mailMessageEx.dwSize = sizeof(mailMessageEx);
mailMessageEx.lpszFrom = m_strSender;
mailMessageEx.lpszTo = m_strRecipients;
mailMessageEx.lpszSubject = m_strSubject;
mailMessageEx.lpszText = m_strMessage;

INT nResult = SmtpSubmitMessageEx(&mailServer, &mailMessageEx, 0, NULL, 0);

if (nResult != SMTP_ERROR)
	_tprintf(_T("SmtpSubmitMessageEx was successful\n"));
else
{
	DWORD dwError = SmtpGetLastError();
	_tprintf(_T("SmtpSubmitMessageEx failed with error 0x%08lx\n"), dwError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpEventProc](#smtpeventproc-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpSubmitMessage](#smtpsubmitmessage-function)
- [SMTPMESSAGEEX](#smtpmessageex-function)
- [SMTPSERVER](#smtpserver-function)

---

## SmtpTaskAbort Function

Abort the specified asynchronous task

### Prototype

```c
BOOL WINAPI SmtpTaskAbort(
    UINT nTaskId,
    DWORD dwMilliseconds
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTaskId` | `UINT` | The task identifier. |
| `dwMilliseconds` | `DWORD` | An unsigned integer that specifies the number of milliseconds to wait for the background task to abort. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds and the worker thread has terminated, the return value is non-zero. A return value of zero indicates that the worker thread is still running or an error has occurred. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpTaskAbort function signals the background worker thread associated with the task ID to abort the current operation and terminate as soon as possible. If the dwMilliseconds parameter has a value of zero, the function returns immediately after the background thread has been signaled. If the dwMilliseconds parameter is non-zero, the function will wait that amount of time for the background thread to terminate.
- This function should never be called from within the event handler for an asynchronous task because it can cause the process to deadlock. To abort a file transfer within an event handler, use the SmtpCancel function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpTaskDone Function

Determine if an asynchronous task has completed

### Prototype

```c
BOOL WINAPI SmtpTaskDone(
    UINT nTaskId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTaskId` | `UINT` | The task identifier. |

### Return Value

- **Type**: `BOOL`
- If the asynchronous task has completed, this function returns a non-zero value. A return value of zero indicates that the worker thread is still running or an error has occurred. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpTaskDone function is used to determine if the specified asynchronous task has completed. If you use this function to poll the status of a background task from within the main UI thread, you must ensure that Windows messages are processed so that the application remains responsive to the end-user. To check if a background transfer has completed, it is recommended that you use a timer to periodically call this function rather than calling it repeatedly within a loop.
- To determine if the task completed successfully, the SmtpGetTaskError function will return the last error code associated with the task. A return value of zero indicates success, while a non-zero return value specifies an error code that indicates the cause of the failure. The last error code for the task can also be retrieved using the SmtpTaskWait function, which causes the application to wait for the asynchronous task to complete.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpGetTaskError](#smtpgettaskerror-function)
- [SmtpTaskAbort](#smtptaskabort-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpTaskResume Function

Resume execution of an asynchronous task

### Prototype

```c
BOOL WINAPI SmtpTaskResume(
    UINT nTaskId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTaskId` | `UINT` | The task identifier. |

### Return Value

- **Type**: `BOOL`
- If the asynchronous task has resumed, this function returns a non-zero value. A return value of zero indicates that an error has occurred. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpTaskResume function resumes execution of the background worker thread that was previously suspended using the SmtpTaskSuspend function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskAbort](#smtptaskabort-function)
- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpTaskSuspend Function

Suspend execution of an asynchronous task

### Prototype

```c
BOOL WINAPI SmtpTaskSuspend(
    UINT nTaskId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTaskId` | `UINT` | The task identifier. |

### Return Value

- **Type**: `BOOL`
- If the asynchronous task has resumed, this function returns a non-zero value. A return value of zero indicates that an error has occurred. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpTaskSuspend function will suspend execution of the background worker thread associated with the task. Once the task has been suspended, it will no longer be scheduled for execution, however the client session will remain active and the task may be resumed using the SmtpTaskResume function. Note that if a task is suspended for a long period of time, the background operation may fail because it has exceeded the timeout period imposed by the server.
- This function should never be called from within the event handler for an asynchronous task because it can cause the process to deadlock.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskAbort](#smtptaskabort-function)
- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpTaskWait Function

Wait for an asynchronous task to complete

### Prototype

```c
BOOL WINAPI SmtpTaskWait(
    UINT nTaskId,
    DWORD dwMilliseconds,
    DWORD dwReserved,
    LPDWORD lpdwElapsed,
    LPDWORD lpdwError
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nTaskId` | `UINT` | The task identifier. |
| `dwMilliseconds` | `DWORD` | An unsigned integer that specifies the number of milliseconds to wait for the background task to complete. |
| `dwReserved` | `DWORD` | An unsigned integer reserved for future use. This value should always be zero. |
| `lpdwElapsed` | `LPDWORD` | A pointer to an unsigned integer that will contain elapsed time in milliseconds when the function returns. If this information is not required, this parameter may be NULL. |
| `lpdwError` | `LPDWORD` | A pointer to an unsigned integer that will contain the error code associated with the completed task. If this information is not required, this parameter may be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds and the worker thread has terminated, the return value is non-zero. A return value of zero indicates that the worker thread is still running or an error has occurred. To get extended error information, call the SmtpGetLastError function.

### Remarks

- The SmtpTaskWait function waits for the specified task to complete. If the task is active and the dwMilliseconds parameter is non-zero, this function will cause the current working thread to block until the task completes or the amount of time exceeds the number of milliseconds specified by the caller. If the dwMilliseconds parameter is zero, then this function will poll the status of the task and return immediately to the caller.
- If the specified task has already completed at the time this function is called, the function will return immediately without causing the current thread to block. If the lpdwElapsed parameter is not NULL, it will contain the number of milliseconds that it took for the task to complete. If the lpdwError parameter is not NULL, it will contain the last error code value that was set by the worker thread before it terminated. If this value is zero, that means that the background operation was successful and no error occurred. A non-zero value will indicate that the background operation has failed.
- You should not call this function from the main UI thread with a long timeout period to wait for a background task to complete. Windows messages will not be processed while this function is blocked waiting for the background task to complete, and this can cause your application to appear non-responsive to the end-user. If you have a GUI application and you need to periodically check to see if a task has completed, create a timer to periodically call the SmtpTaskDone function. When it returns a non-zero value (indicating that the task has completed), you can safely call SmtpTaskWait to obtain the elapsed time and last error code without blocking the current thread.
- This function should never be called from within the event handler for an asynchronous task because it can cause the process to deadlock.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpTaskDone](#smtptaskdone-function)
- [SmtpTaskResume](#smtptaskresume-function)
- [SmtpTaskSuspend](#smtptasksuspend-function)
- [SmtpTaskWait](#smtptaskwait-function)

---

## SmtpUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI SmtpUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful SmtpInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call SmtpUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to SmtpUninitialize for every successful call to SmtpInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpConnect](#smtpconnect-function)
- [SmtpDisconnect](#smtpdisconnect-function)
- [SmtpInitialize](#smtpinitialize-function)

---

## SmtpVerifyAddress Function

Verify that the specified address is valid

### Prototype

```c
INT WINAPI SmtpVerifyAddress(
    HCLIENT hClient,
    LPCTSTR lpszAddress,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszAddress` | `LPCTSTR` | Points to a string which specifies the address that the server should verify. |
| `lpszBuffer` | `LPTSTR` | Points to a buffer that the verified address will be copied into. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the server result code. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The SmtpVerifyAddress function requests that the server verify the specified email address. Typically this is used to verify that a recipient address is valid, and return a fully qualified email address for that recipient. A server may not support this command, or may restrict its usage. An application should not depend on the ability to verify addresses.
- This function cannot be called while a mail message is being composed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmtpAddRecipient](#smtpaddrecipient-function)
- [SmtpExpandAddress](#smtpexpandaddress-function)

---

## SmtpWrite Function

Write data to the server

### Prototype

```c
INT WINAPI SmtpWrite(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | The pointer to the buffer which contains the data that is to be sent to the server. |
| `cbBuffer` | `INT` | The number of bytes to send from the specified buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually written. If the function fails, the return value is SMTP_ERROR. To get extended error information, call SmtpGetLastError.

### Remarks

- The return value may be less than the number of bytes specified by the cbBuffer parameter. In this case, the data has been partially written and it is the responsibility of the client application to send the remaining data at some later point. For non-blocking clients, the client must wait for the SMTP_EVENT_WRITE asynchronous notification message before it resumes sending data.
- If the SmtpWrite function is used to send the message contents to the server, the application must first call the SmtpCreateMessage function to specify the sender and the length of the message, followed by one or more calls to the SmtpAddRecipient function to specify each recipient of the message. When all of the message text has been submitted to the server, the application must call the SmtpCloseMessage function.
- The message text is filtered by the SmtpWrite function, and it will automatically normalize end-of-line character sequences to ensure the message meets the protocol requirements. The message itself must be in a standard RFC 822 or multi-part MIME message format, or the server may reject the message. Binary data, such as file attachments, should always be encoded. The MIME API can be used to compose and export a message in the correct format, which can then be submitted to the server.
- It is recommended that most applications use the SmtpSendMessage function to submit the message for delivery.
- An alternative approach to creating a message without using the MIME API is the SmtpSubmitMessage function. It accepts two structure parameters which define the message contents and the connection information for the mail server. This enables the application to compose the message and submit it for delivery in a single function call.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csmtpv11.lib`

### See Also

- [SmtpAddRecipient](#smtpaddrecipient-function)
- [SmtpAppendMessage](#smtpappendmessage-function)
- [SmtpCloseMessage](#smtpclosemessage-function)
- [SmtpCreateMessage](#smtpcreatemessage-function)
- [SmtpSendMessage](#smtpsendmessage-function)
- [SmtpSubmitMessage](#smtpsubmitmessage-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYCREDENTIALS Structure

The SECURITYCREDENTIALS structure specifies the information needed by the library to specify additional security credentials, such as a client certificate, when establishing a secure connection.

### Definition

```c
typedef struct _SECURITYCREDENTIALS
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwOptions;
    DWORD dwReserved;
    LPCTSTR lpszHostName;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
    LPCTSTR lpszCertStore;
    LPCTSTR lpszCertName;
    LPCTSTR lpszKeyFile;
} SECURITYCREDENTIALS, *LPSECURITYCREDENTIALS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. If the structure is being allocated dynamically, this member must be set to the size of the structure and all other unused structure members must be initialized to a value of zero or NULL. |
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. The value of this structure member is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or options. This member is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | This structure member is reserved for future use and should always be initialized to zero. |
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the hostname that will be used when validating the server certificate. If this member is NULL, then the server certificate will be validated against the hostname used to establish the connection. |
| `lpszUserName` | `LPCTSTR` | A pointer to a null terminated string which identifies the owner of client certificate. Currently this member is not used by the library and should always be initialized as a NULL pointer. |
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string which specifies the password needed to access the certificate. Currently this member is only used if the CREDENTIAL_STORE_FILENAME option has been specified. If there is no password associated with the certificate, then this member should be initialized as a NULL pointer. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The library will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the library will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the library will return an error indicating that the certificate could not be found. If the SECURITY_PROTOCOL_SSH protocol has been specified, this member should be NULL. |
| `lpszKeyFile` | `LPCTSTR` | This member is not used and should always be NULL. The private key cannot be specified separately and must be associated with the certificate. |

### Remarks

- An application only needs to create this structure if the server requires the client to provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions member should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore member should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12. These certificate files typically have an extension of .pfx or .p12. Note that if a password was specified when the certificate file was created, it must be provided in the lpszPassword member or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword members are values which are used to access the certificate store or private key file. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYINFO Structure

The SECURITYINFO structure contains security related information about the connection, including the protocol, cipher suite and certificate.

### Definition

```c
typedef struct _SECURITYINFO
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwCipher;
    DWORD dwCipherStrength;
    DWORD dwHash;
    DWORD dwHashStrength;
    DWORD dwKeyExchange;
    DWORD dwCertStatus;
    SYSTEMTIME stCertIssued;
    SYSTEMTIME stCertExpires;
    LPCTSTR lpszCertIssuer;
    LPCTSTR lpszCertSubject;
    LPCTSTR lpszFingerprint;
} SECURITYINFO, *LPSECURITYINFO;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Specifies the size of the data structure. This member must always be initialized to the size of the structure in bytes prior to passing the address of this structure to the function. If this this member is not initialized, an error will be returned indicating that an invalid parameter has been passed to the function. |
| `dwProtocol` | `DWORD` | A numeric value which specifies the security protocol selected when establishing the connection. |
| `dwCipher` | `DWORD` | A numeric value which specifies the cipher that was selected when establishing the secure connection. |
| `dwCipherStrength` | `DWORD` | A numeric value which specifies the strength (the length of the cipher key in bits) of the cipher that was selected. Typically this value will be 128 or 256. 40-bit and 56-bit key lengths are considered weak encryption, and subject to brute force attacks. 128-bit and 256-bit key lengths are considered to be secure, and are the recommended key length for secure communications. |
| `dwHash` | `DWORD` | A numeric value which specifies the hash algorithm which was selected. |
| `dwHashStrength` | `DWORD` | A numeric value which specifies the strength (the length in bits) of the message digest that was selected. |
| `dwKeyExchange` | `DWORD` | A numeric value which specifies the key exchange algorithm which was selected. |
| `dwCertStatus` | `DWORD` | A numeric value which specifies the status of the certificate returned by the secure server. |
| `stCertIssued` | `SYSTEMTIME` | A structure which contains the date and time that the certificate was issued by the certificate authority. If the issue date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `stCertExpires` | `SYSTEMTIME` | A structure which contains the date and time that the certificate expires. If the expiration date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `lpszCertIssuer` | `LPCTSTR` | A pointer to a string which contains information about the organization that issued the certificate. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate issuer could not be determined, this member may be NULL. |
| `lpszCertSubject` | `LPCTSTR` | A pointer to a string which contains information about the organization that the certificate was issued to. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate subject could not be determined, this member may be NULL. |
| `lpszFingerprint` | `LPCTSTR` | A pointer to a string which contains a sequence of hexadecimal values that uniquely identify the server. This member may be NULL. |

### Remarks

- The size of this structure is typically determined using the sizeof operator in C/C++ and most other languages have a similar method for obtaining the size of a data structure or user-defined type. It is important to note the size of this structure will be different on 32-bit and 64-bit platforms due to the difference in the length of the structure members which are pointers.
- When evaluating structure members which are string pointers, such as lpszCertIssuer and lpszCertSubject, you should always check to ensure their values are not NULL (zero) before attempting to dereference them. If those values are undefined, it is not guaranteed they will point to zero-length strings.
- There are additional protocols and ciphers defined which are not listed above because they are considered insecure and are no longer supported. It is important to note that most servers will reject attempts to use unsupported protocols such as SSL 3.0 and will either abort the connection or issue an error message and then immediately disconnect.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SMTPMESSAGE Structure

This structure provides information about the contents of a message and is used by the SmtpSubmitMessage function.

### Definition

```c
typedef struct _SMTPMESSAGE
{
    LPCTSTR lpszFrom;
    LPCTSTR lpszTo;
    LPCTSTR lpszCc;
    LPCTSTR lpszBcc;
    LPCTSTR lpszSubject;
    LPCTSTR lpszText;
    LPCTSTR lpszHTML;
    LPCTSTR lpszAttach;
    UINT nCharSet;
    UINT nEncType;
    DWORD dwReserved;
} SMTPMESSAGE, *LPSMTPMESSAGE;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `lpszFrom` | `LPCTSTR` | A pointer to a string that specifies the email address of the person sending the message. This structure member must point to a valid address and cannot be NULL. |
| `lpszTo` | `LPCTSTR` | A pointer to a string that specifies the email addresses of one or more recipients. If multiple addresses are provided, they must be separated by commas or semi-colons. This structure member must point to at least one valid address and cannot be NULL. |
| `lpszCc` | `LPCTSTR` | A pointer to a string that specifies the email addresses of one or more recipients that will receive copies of the message. If multiple addresses are provided, they must be separated by commas or semi-colons. This structure member may be NULL or point to an empty string. |
| `lpszBcc` | `LPCTSTR` | A pointer to a string that specifies the email addresses of one or more recipients that will receive blind copies of the message. If multiple addresses are provided, they must be separated by commas or semi-colons. This structure member may be NULL or point to an empty string. Unlike the recipients specified by the lpszTo and lpszCc members, any addresses specified by this member will not be included in the header of the email message. |
| `lpszSubject` | `LPCTSTR` | A pointer to a string that specifies the subject of the message. This structure member may be NULL, in which case no subject will be included in the message. |
| `lpszText` | `LPCTSTR` | A pointer to a string which contains the body of the message as plain text. Each line of text contained in the string should be terminated with a carriage-return and linefeed (CRLF) pair, which is recognized as the end-of-line. If this structure member is NULL or points to an empty string, then the lpszHTML member must specify the body of the message. |
| `lpszHTML` | `LPCTSTR` | A pointer to a string which contains the message using HTML formatting. If the lpszText member is not NULL, then a multipart message will be created with both plain text and HTML text as the alternative. This allows mail clients to select which message body they wish to display. If the lpszText member is NULL or points to an empty string, then the message will only contain HTML. Although this is supported, it is not recommended because older mail clients may be unable to display the message correctly. |
| `lpszAttach` | `LPCTSTR` | A pointer to a string which specifies one or more file attachments for the message. If multiple files are to be attached to the message, each file name must be separated by a semi-colon. It is recommended that you provide the complete path to the file. If this structure member is NULL or points to an empty string, the message will be created without attachments. |
| `nCharSet` | `UINT` | A integer value which specifies the character set to use when composing the message. A value of zero specifies that the default USASCII character set should be used. The following values may also be used: |
| `nEncType` | `UINT` | A numeric identifier which specifies the encoding type to use when composing the message. A value of zero specifies that default 7bit encoding should be used. The following values may also be used: |
| `dwReserved` | `DWORD` | An unsigned integer value reserved for future use. This structure member should always have a value of zero. |

### Remarks

- This structure is used to define the contents of a message that will be submitted for delivery using the SmtpSubmitMessage function. It is required that you specify a sender, at least one recipient and a message body. All other structure members may be NULL or have a value of zero to indicate that either the value is not required, or that a default should be used. It is recommended that you initialize all of the structure members to a value of zero using the ZeroMemory function prior to populating the structure.
- email addresses may be specified as simple addresses, or as commented addresses that include the sender's name or other information. For example, any one of these address formats are acceptable:
- To specify multiple addresses, you should separate each address by a comma or semi-colon. Note that the lpszFrom member cannot specify multiple addresses, however it is permitted with the lpszTo, lpszCc and lpszBcc structure members. Each message must have at least one valid recipient, or the message cannot be submitted for delivery.
- To send a message that contains HTML, it is recommended that you provide both a plain text version of the message body and an HTML formatted version. While it is permitted to send a message that only contains HTML, some older mail clients may not be capable of displaying the message correctly. In some cases, anti-spam software will increase the spam score of messages that do not contain a plain text message body. This can result in your message being rejected or quarantined by the mail server.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SMTPSERVER](#smtpserver-structure)

---

## SMTPMESSAGEEX Structure

This structure provides information about the contents of a message and is used by the SmtpSubmitMessageEx function.

### Definition

```c
typedef struct _SMTPMESSAGEEX
{
    LPCTSTR lpszFrom;
    LPCTSTR lpszTo;
    LPCTSTR lpszCc;
    LPCTSTR lpszBcc;
    LPCTSTR lpszDate;
    LPCTSTR lpszSubject;
    LPCTSTR lpszHeaders;
    LPCTSTR lpszText;
    LPCTSTR lpszHTML;
    LPCTSTR lpszAttach;
    UINT nCharSet;
    UINT nEncType;
    DWORD dwReserved;
} SMTPMESSAGEEX, *LPSMTPMESSAGEEX;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `lpszFrom` | `LPCTSTR` | A pointer to a string that specifies the email address of the person sending the message. This structure member must point to a valid address and cannot be NULL. |
| `lpszTo` | `LPCTSTR` | A pointer to a string that specifies the email addresses of one or more recipients. If multiple addresses are provided, they must be separated by commas or semi-colons. This structure member must point to at least one valid address and cannot be NULL. |
| `lpszCc` | `LPCTSTR` | A pointer to a string that specifies the email addresses of one or more recipients that will receive copies of the message. If multiple addresses are provided, they must be separated by commas or semi-colons. This structure member may be NULL or point to an empty string. |
| `lpszBcc` | `LPCTSTR` | A pointer to a string that specifies the email addresses of one or more recipients that will receive blind copies of the message. If multiple addresses are provided, they must be separated by commas or semi-colons. This structure member may be NULL or point to an empty string. Unlike the recipients specified by the lpszTo and lpszCc members, any addresses specified by this member will not be included in the header of the email message. |
| `lpszDate` | `LPCTSTR` | A pointer to a string that specifies the date and time for the message. This structure member may be NULL or point to an empty string. If the date is not specified, then the current date and time will be used by default. If a date is specified, it should be in the standard format as defined by RFC822. |
| `lpszSubject` | `LPCTSTR` | A pointer to a string that specifies the subject of the message. This structure member may be NULL, in which case no subject will be included in the message. |
| `lpszHeaders` | `LPCTSTR` | A pointer to a string that specifies additional headers that should be included in the message. Header names should be separated from values by a colon, and multiple headers may be defined by separating them with a newline character. This structure member may be NULL, in which case no additional headers will be included in the message. |
| `lpszText` | `LPCTSTR` | A pointer to a string which contains the body of the message as plain text. Each line of text contained in the string should be terminated with a carriage-return and linefeed (CRLF) pair, which is recognized as the end-of-line. If this structure member is NULL or points to an empty string, then the lpszHTML member must specify the body of the message. |
| `lpszHTML` | `LPCTSTR` | A pointer to a string which contains the message using HTML formatting. If the lpszText member is not NULL, then a multipart message will be created with both plain text and HTML text as the alternative. This allows mail clients to select which message body they wish to display. If the lpszText member is NULL or points to an empty string, then the message will only contain HTML. Although this is supported, it is not recommended because older mail clients may be unable to display the message correctly. |
| `lpszAttach` | `LPCTSTR` | A pointer to a string which specifies one or more file attachments for the message. If multiple files are to be attached to the message, each file name must be separated by a semi-colon. It is recommended that you provide the complete path to the file. If this structure member is NULL or points to an empty string, the message will be created without attachments. |
| `nCharSet` | `UINT` | A integer value which specifies the character set to use when composing the message. A value of zero specifies that the default USASCII character set should be used. The following values may also be used: |
| `nEncType` | `UINT` | A numeric identifier which specifies the encoding type to use when composing the message. A value of zero specifies that default 7bit encoding should be used. The following values may also be used: |
| `dwReserved` | `DWORD` | An unsigned integer value reserved for future use. This structure member should always have a value of zero. |

### Remarks

- This structure is used to define the contents of a message that will be submitted for delivery using the SmtpSubmitMessageEx function. It is required that you specify a sender, at least one recipient and a message body. Other structure members may be NULL or have a value of zero to indicate that either the value is not required, or that a default should be used. It is recommended that you initialize all of the structure members to a value of zero using the ZeroMemory function prior to populating the structure.
- Note that you must explicitly define the size of the structure by setting the value of the dwSize member variable. This ensures that the correct version of the structure is being passed to the function. This structure is not compatible with the SmtpSubmitMessage function and must only be used with SmtpSubmitMessageEx.
- Email addresses may be specified as simple addresses, or as commented addresses that include the sender's name or other information. For example, any one of these address formats are acceptable:
- To specify multiple addresses, you should separate each address by a comma or semi-colon. Note that the lpszFrom member cannot specify multiple addresses, however it is permitted with the lpszTo, lpszCc and lpszBcc structure members. Each message must have at least one valid recipient, or the message cannot be submitted for delivery.
- If you specify a message date by assigning a value to the lpszDate member, and it does not include any timezone information, Coordinated Universal Time (UTC) will be used by default. This is an important consideration if you provide input from a user, because in most cases they will not include the timezone and will assume the date and time they enter is for their current timezone.
- If you wish to include additional headers in the message, you can specify them in a string. Each header consists of a name and value, separated by a colon (":") character. If you wish to define multiple headers, then you can separate them with a newline (e.g.: a linefeed character or combination of a carriage-return and linefeed). Extraneous leading and trailing whitespace are trimmed from header names and values. Invalid names or values will be ignored and will not generate an error.
- To send a message that contains HTML, it is recommended that you provide both a plain text version of the message body and an HTML formatted version. While it is permitted to send a message that only contains HTML, some older mail clients may not be capable of displaying the message correctly. In some cases, anti-spam software will increase the spam score of messages that do not contain a plain text message body. This can result in your message being rejected or quarantined by the mail server.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SMTPSERVER](#smtpserver-structure)

---

## SMTPSERVER Structure

This structure provides connection information for a mail server and is used by the SmtpSubmitMessage function.

### Definition

```c
typedef struct _SMTPSERVER
{
    LPCTSTR lpszHostName;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
    UINT nHostPort;
    UINT nTimeout;
    DWORD dwOptions;
    DWORD dwReserved;
} SMTPSERVER, *LPSMTPSERVER;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `lpszHostName` | `LPCTSTR` | A pointer to a string that specifies the host name or IP address of the mail server. This structure member cannot be NULL. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string that specifies the username that will be used to authenticate the client session. If the mail server does not require authentication, this structure member can be NULL or point to an empty string. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string that specifies the password that will be used to authenticate the client session. If the mail server does not require authentication, this structure member can be NULL or point to an empty string. |
| `nHostPort` | `UINT` | An integer value that specifies the port number used to establish the connection. A value of zero specifies that the default port number should be used. For standard connections, the default port number is 25. An alternative port is 587, which is commonly used by authenticated clients to submit messages for delivery. For implicit TLS connections, the default port number is 465. |
| `nTimeout` | `UINT` | An integer value that specifies the number of seconds that the client will wait for a response from the server before failing the operation. A value of zero specifies the default timeout period of 20 seconds. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. The value of this structure member is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | An unsigned integer value reserved for future use. This structure member should always have a value of zero. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SMTPMESSAGE](#smtpmessage-structure)

---

## SMTPTRANSFERSTATUS Structure

This structure is used by the SmtpGetTransferStatus function to return information about a message being submitted for delivery.

### Definition

```c
typedef struct _SMTPTRANSFERSTATUS
{
    DWORD dwBytesTotal;
    DWORD dwBytesCopied;
    DWORD dwBytesPerSecond;
    DWORD dwTimeElapsed;
    DWORD dwTimeEstimated;
} SMTPTRANSFERSTATUS, *LPSMTPTRANSFERSTATUS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwBytesTotal` | `DWORD` | The total number of bytes that will be transferred. If the size of the message cannot be determined, this value will be zero. |
| `dwBytesCopied` | `DWORD` | The total number of bytes that have been copied. |
| `dwBytesPerSecond` | `DWORD` | The average number of bytes that have been copied per second. |
| `dwTimeElapsed` | `DWORD` | The number of seconds that have elapsed since the file transfer started. |
| `dwTimeEstimated` | `DWORD` | The estimated number of seconds until the transfer is completed. This is based on the average number of bytes transferred per second. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

