# Encoding and Compression

## Functions

- [AesDecryptBuffer](#aesdecryptbuffer-function)
- [AesDecryptFile](#aesdecryptfile-function)
- [AesDecryptString](#aesdecryptstring-function)
- [AesEncryptBuffer](#aesencryptbuffer-function)
- [AesEncryptFile](#aesencryptfile-function)
- [AesEncryptString](#aesencryptstring-function)
- [CompareFile](#comparefile-function)
- [CompareFileEx](#comparefileex-function)
- [CompressBuffer](#compressbuffer-function)
- [CompressFile](#compressfile-function)
- [CompressFileEx](#compressfileex-function)
- [DecodeBuffer](#decodebuffer-function)
- [DecodeFile](#decodefile-function)
- [EncodeBuffer](#encodebuffer-function)
- [EncodeFile](#encodefile-function)
- [ExpandBuffer](#expandbuffer-function)
- [ExpandFile](#expandfile-function)
- [ExpandFileEx](#expandfileex-function)
- [GetMessageDigest](#getmessagedigest-function)
- [IsUnicodeText](#isunicodetext-function)
- [UnicodeDecodeText](#unicodedecodetext-function)
- [UnicodeEncodeText](#unicodeencodetext-function)
## Structures


---

## AesDecryptBuffer Function

Decrypt a buffer previously encrypted using the AesEncryptBuffer function

### Prototype

```c
BOOL WINAPI AesDecryptBuffer(
    LPCTSTR lpszPassword,
    LPVOID lpvInputBuffer,
    DWORD dwInputSize,
    LPVOID lpvOutputBuffer,
    LPDWORD lpdwOutputSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string of characters that will be used to generate the decryption key. This parameter may be NULL or a zero-length string, in which case a default internal hash value is used to decrypt the data. Password strings that exceed 215 characters will be truncated. |
| `lpvInputBuffer` | `LPVOID` | A pointer to the buffer which contains the data to be decrypted. |
| `dwInputSize` | `DWORD` | The number of bytes in the input buffer. This value must be greater than zero. |
| `lpvOutputBuffer` | `LPVOID` | A pointer to the buffer which will contain the decrypted data when the function returns. This memory must be allocated by the application and be large enough to contain all of the decrypted data. If the output buffer is not large enough, the function will fail. This parameter cannot be NULL. |
| `lpdwOutputSize` | `LPDWORD` | A pointer to the number of bytes that may be copied into the output buffer. This parameter must be initialized to a non-zero value. When the function returns, the actual number of bytes of decrypted data is returned in this parameter. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the data was successfully decrypted. A zero value indicates that the data could not be decrypted. To get extended error information, call GetLastError.

### Remarks

- The AesDecryptBuffer function will decrypt a block of memory using a 256-bit AES (Advanced Encryption Standard) algorithm and returns a copy of the decrypted data to the caller. The password (or passphrase) provided by the caller is used to generate a SHA-256 hash value which is used as part of the decryption process. The lpszPassword value must be identical to the value used to encrypt the data using the AesEncryptBuffer function.
- Due to how the SHA-256 hash is generated, this function cannot be used to decrypt data that was encrypted using another third-party library. It can only be used to decrypt data that was previously encrypted using AesEncryptBuffer.
- If you wish to decrypt the contents of a file, use the AesDecryptFile function.
- This function uses the Microsoft CryptoAPI and the RSA AES cryptographic provider. This provider may not be available in some languages, countries or regions. The availability of this provider may also be constrained by cryptography export restrictions imposed by the United States or other countries. If the required cryptographic provider is not available, the function will fail.

### Example

```cpp
BOOL bIsDecrypted = FALSE;
DWORD dwOutputSize = MAX_BUFFER_SIZE;
BYTE *lpOutputBuffer = (BYTE *)LocalAlloc(LPTR, dwOutputSize);

if (lpOutputBuffer != NULL)
{
	bIsDecrypted = AesDecryptBuffer(lpszPassword,
									lpInputBuffer,
									dwInputSize,
									lpOutputBuffer,
									&dwOutputSize);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [AesDecryptFile](#aesdecryptfile-function)
- [AesDecryptString](#aesdecryptstring-function)
- [AesEncryptBuffer](#aesencryptbuffer-function)
- [CompressBuffer](#compressbuffer-function)

---

## AesDecryptFile Function

Decrypt a file previously encrypted using the AesEncryptFile function

### Prototype

```c
BOOL WINAPI AesDecryptFile(
    LPCTSTR lpszPassword,
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string of characters that will be used to generate the decryption key. This parameter may be NULL or a zero-length string, in which case a default internal hash value is used to decrypt the data. Password strings that exceed 215 characters will be truncated. |
| `lpszInputFile` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the file to be decrypted. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. |
| `lpszOutputFile` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the file that will contain the decrypted data. If the file exists, it will be overwritten. It must be a regular file that can be opened for writing by the current process. An error will be returned if a character device, such as CON: is specified as the file name. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the string was successfully encrypted. A zero value indicates that the string could not be encrypted. To get extended error information, call GetLastError.

### Remarks

- The AesDecryptFile function will decrypt the contents of a file using a 256-bit AES (Advanced Encryption Standard) algorithm and stores the decrypted data in the specified output file. The password (or passphrase) provided by the caller is used to generate a SHA-256 hash value which is used as part of the decryption process. The lpszPassword value must be identical to the value used to encrypt the data using the AesEncryptFile function.
- Due to how the SHA-256 hash is generated, this function cannot be used to decrypt files that were encrypted using another third-party library. It can only be used to decrypt data that was previously encrypted using AesEncryptFile.
- A temporary file is created during the decryption process and the output file is created or overwritten only if the input file could be successfully decrypted. If the decryption fails, no output file will be created.
- If you wish to decrypt the contents of a memory buffer or string, use the AesDecryptBuffer or AesDecryptString functions.
- This function uses the Microsoft CryptoAPI and the RSA AES cryptographic provider. This provider may not be available in some languages, countries or regions. The availability of this provider may also be constrained by cryptography export restrictions imposed by the United States or other countries. If the required cryptographic provider is not available, the function will fail.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [AesDecryptBuffer](#aesdecryptbuffer-function)
- [AesDecryptString](#aesdecryptstring-function)
- [AesEncryptFile](#aesencryptfile-function)

---

## AesDecryptString Function

Decrypt a string previously encrypted using the AesEncryptString function

### Prototype

```c
BOOL WINAPI AesDecryptString(
    LPCTSTR lpszPassword,
    LPCTSTR lpszInputString,
    LPCTSTR lpszOutputString,
    LONG nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string of characters that will be used to generate the decryption key. This parameter may be NULL or a zero-length string, in which case a default internal hash value is used to decrypt the data. Password strings that exceed 215 characters will be truncated. |
| `lpszInputString` | `LPCTSTR` | A pointer to a null terminated string which contains the data to be decrypted. The encrypted input data must be base64 encoded and identical to the encrypted string returned by the AesEncryptString function. If this parameter is NULL or points to an empty string the function will fail. |
| `lpszOutputString` | `LPCTSTR` | A pointer to the buffer which will contain the decrypted string data. This parameter cannot be NULL. |
| `nMaxLength` | `LONG` | The maximum number of characters that can be copied to the output string buffer. The output buffer must be large enough to store the complete decrypted string and is terminated with a null character. This value must be greater than zero. If the output string buffer is not large enough, the function will fail. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the string was successfully encrypted. A zero value indicates that the string could not be decrypted. To get extended error information, call GetLastError.

### Remarks

- The AesDecryptString function will decrypt a string using a 256-bit AES (Advanced Encryption Standard) algorithm and returns a copy of the decrypted string o the caller. The password (or passphrase) provided by the caller is used to generate a SHA-256 hash value which is used as part of the decryption process. The lpszPassword value must be identical to the value used to encrypt the data using the AesEncryptString function.
- Due to how the SHA-256 hash is generated, this function cannot be used to decrypt strings that were encrypted using another third-party library. It can only be used to decrypt strings that were previously encrypted using AesEncryptString.
- If you wish to decrypt the contents of a file, use the AesDecryptFile function.
- This function uses the Microsoft CryptoAPI and the RSA AES cryptographic provider. This provider may not be available in some languages, countries or regions. The availability of this provider may also be constrained by cryptography export restrictions imposed by the United States or other countries. If the required cryptographic provider is not available, the function will fail.

### Example

```cpp
BOOL bIsDecrypted = FALSE;
LPCTSTR lpszPassword = _T("NFr-E{Ki3_1w0iV+LI@z");
TCHAR szDecryptedText[MAX_STRING_LENGTH];

bIsDecrypted = AesDecryptString(lpszPassword,
								szEncryptedText,
								szDecryptedText,
								MAX_STRING_LENGTH);

if (bIsDecrypted)
{
	_tprintf(_T("The decrypted string is \"%s\"\n"), szDecryptedText);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [AesDecryptBuffer](#aesdecryptbuffer-function)
- [AesDecryptFile](#aesdecryptfile-function)
- [AesEncryptString](#aesencryptstring-function)

---

## AesEncryptBuffer Function

Encrypt a memory buffer using 256-bit AES encryption

### Prototype

```c
BOOL WINAPI AesEncryptBuffer(
    LPCTSTR lpszPassword,
    LPVOID lpvInputBuffer,
    DWORD dwInputSize,
    LPVOID lpvOutputBuffer,
    LPDWORD lpdwOutputSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string of characters that will be used to generate the encryption key. Passwords are case sensitive and must match exactly when decrypting data that was previously encrypted using this function. This parameter may be NULL or a zero-length string, in which case a default internal hash value is used. Password strings that exceed 215 characters will be truncated. |
| `lpvInputBuffer` | `LPVOID` | A pointer to the buffer which contains the data to be encrypted. |
| `dwInputSize` | `DWORD` | The number of bytes in the input buffer. This value must be greater than zero. |
| `lpvOutputBuffer` | `LPVOID` | A pointer to the buffer which will contain the encrypted data when the function returns. This memory must be allocated by the application and be large enough to contain all of the encrypted data. The amount encrypted data returned will always be larger than the amount of data specified by dwInputSize. If the output buffer is not large enough, the function will fail. This parameter cannot be NULL. |
| `lpdwOutputSize` | `LPDWORD` | A pointer to the number of bytes that may be copied into the output buffer. This parameter must be initialized to a non-zero value. When the function returns, the actual number of bytes of encrypted data is returned in this parameter. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the data was successfully encrypted. A zero value indicates that the data could not be encrypted. To get extended error information, call GetLastError.

### Remarks

- The AesEncryptBuffer function will encrypt a block of memory using a 256-bit AES (Advanced Encryption Standard) algorithm and returns a copy of the encrypted data to the caller. The password (or passphrase) provided by the caller is used to generate a SHA-256 hash value which is used as part of the encryption process. The identical password is required to decrypt the data using the AesDecryptBuffer function.
- It is recommended that most applications specify a password value. If the lpszPassword parameter is NULL or specifies a zero-length string, a default internal hash value is used. This means that any other application which uses a NULL password value will be able to decrypt the data. If the Unicode version of this function is called, the lpszPassword value will be encoded using UTF-8 prior to the hash value being generated.
- Due to how the SHA-256 hash is generated, the encrypted data cannot be decrypted using another third-party library with the same password value. It can only be decrypted using the AesDecryptBuffer function.
- The amount of encrypted data returned by this function will always be somewhat larger than original unencrypted data. If your application dynamically allocates a block of memory to store the encrypted data, provide a maximum buffer size that is at least several hundred bytes larger than the unencrypted data. If the output buffer provided is not large enough, the function will fail and the GetLastError function will return ERROR_INSUFFICIENT_BUFFER.
- The encrypted data returned by this function can contain embedded nulls and should be typically handled as unsigned char (byte) values. If you wish to encrypt strings and store the encrypted values, use the AesEncryptString function. It will perform the same 256-bit AES encryption, but return the encrypted data as a base64 encoded string rather than binary data.
- If your application is also using the CompressBuffer function to compress the data, it is recommended that you call CompressBuffer before calling AesEncryptBuffer. You will typically achieve a better compression rate on unencrypted data than than attempting to compress data which has been encrypted with this function.
- If you wish to encrypt the contents of a file, use the AesEncryptFile function.
- This function uses the Microsoft CryptoAPI and the RSA AES cryptographic provider. This provider may not be available in some languages, countries or regions. The availability of this provider may also be constrained by cryptography export restrictions imposed by the United States or other countries. If the required cryptographic provider is not available, the function will fail.

### Example

```cpp
BOOL bIsEncrypted = FALSE;
DWORD dwOutputSize = MAX_BUFFER_SIZE;
BYTE *lpOutputBuffer = (BYTE *)LocalAlloc(LPTR, dwOutputSize);

if (lpOutputBuffer != NULL)
{
	bIsEncrypted = AesEncryptBuffer(lpszPassword,
									lpInputBuffer,
									dwInputSize,
									lpOutputBuffer,
									&dwOutputSize);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [AesDecryptBuffer](#aesdecryptbuffer-function)
- [AesEncryptFile](#aesencryptfile-function)
- [AesEncryptString](#aesencryptstring-function)
- [CompressBuffer](#compressbuffer-function)

---

## AesEncryptFile Function

Encrypt the contents of a file using 256-bit AES encryption

### Prototype

```c
BOOL WINAPI AesEncryptFile(
    LPCTSTR lpszPassword,
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string of characters that will be used to generate the encryption key. Passwords are case sensitive and must match exactly when decrypting data that was previously encrypted using this function. This parameter may be NULL or a zero-length string, in which case a default internal hash value is used. Password strings that exceed 215 characters will be truncated. |
| `lpszInputFile` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the file to be encrypted. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. |
| `lpszOutputFile` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the file that will contain the encrypted data. If the file exists, it will be overwritten. It must be a regular file that can be opened for writing by the current process. An error will be returned if a character device, such as CON: is specified as the file name. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the string was successfully encrypted. A zero value indicates that the string could not be encrypted. To get extended error information, call GetLastError.

### Remarks

- The AesEncryptFile function will encrypt the contents of a file using a 256-bit AES (Advanced Encryption Standard) algorithm and stores the encrypted data in the specified output file. The password (or passphrase) provided by the caller is used to generate a SHA-256 hash value which is used as part of the encryption process. The identical password is required to decrypt the data using the AesDecryptFile function.
- It is recommended that most applications specify a password value. If the lpszPassword parameter is NULL or specifies a zero-length string, a default internal hash value is used. This means that any other application which uses a NULL password value will be able to decrypt the data. If the Unicode version of this function is called, the lpszPassword value will be encoded using UTF-8 prior to the hash value being generated.
- Due to how the SHA-256 hash is generated, the encrypted data cannot be decrypted using another third-party library with the same password value. It can only be decrypted using the AesDecryptFile function.
- A temporary file is created during the encryption process and the output file is created or overwritten only if the input file could be successfully encrypted. If the encryption fails, no output file will be created.
- The input file contents will always be processed as a binary data stream. If you use this function to encrypt a text file, the output file will contain binary characters, not printable text. If you wish to transfer or store the encrypted data as text, it should be encoded using the EncodeFile function after calling AesEncryptFile.
- If your application is also using the CompressFile function to compress the data, it is recommended that you call CompressFile before calling AesEncryptFile. You will typically achieve a better compression rate on unencrypted data than than attempting to compress data which has been encrypted with this function.
- If you wish to encrypt the contents of a memory buffer or string, use the AesEncryptBuffer or AesEncryptString functions.
- This function uses the Microsoft CryptoAPI and the RSA AES cryptographic provider. This provider may not be available in some languages, countries or regions. The availability of this provider may also be constrained by cryptography export restrictions imposed by the United States or other countries. If the required cryptographic provider is not available, the function will fail.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [AesDecryptFile](#aesdecryptfile-function)
- [AesEncryptBuffer](#aesencryptbuffer-function)
- [AesEncryptString](#aesencryptstring-function)
- [CompressFile](#compressfile-function)
- [EncodeFile](#encodefile-function)

---

## AesEncryptString Function

Encrypt a null-terminated string using 256-bit AES encryption

### Prototype

```c
BOOL WINAPI AesEncryptString(
    LPCTSTR lpszPassword,
    LPCTSTR lpszInputString,
    LONG nInputLength,
    LPCTSTR lpszOutputString,
    LONG nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string of characters that will be used to generate the encryption key. Passwords are case sensitive and must match exactly when decrypting data that was previously encrypted using this function. This parameter may be NULL or a zero-length string, in which case a default internal hash value is used. Password strings that exceed 215 characters will be truncated. |
| `lpszInputString` | `LPCTSTR` | A pointer to a null terminated string which contains the text to be encrypted. If the Unicode version of this function is called, the string is automatically UTF-8 encoded prior to being encrypted. |
| `nInputLength` | `LONG` | The number of characters in the input string which will be encrypted. If this value is -1, the length of the input string is determined automatically by counting the number of of characters up to the terminating null byte. Note that with Unicode strings, this value must represent the number of characters in the string, not the number of bytes allocated for the string. |
| `lpszOutputString` | `LPCTSTR` | A pointer to the buffer which will contain a base64 encoded encrypted string when the function returns. This memory must be allocated by the application and be large enough to contain all of the encrypted data. The amount encrypted data returned will always be larger than the amount of data specified by nInputLength. If the output string buffer is not large enough, the function will fail. This parameter cannot be NULL. |
| `nMaxLength` | `LONG` | The maximum number of characters that can be copied to the output string buffer. The output buffer must be large enough to store the complete encrypted string which is encoded using base64 and terminated with a null character. This value must be greater than zero. If the output string buffer is not large enough, the function will fail. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the string was successfully encrypted. A zero value indicates that the string could not be encrypted. To get extended error information, call GetLastError.

### Remarks

- The AesEncryptString function will encrypt a string using a 256-bit AES (Advanced Encryption Standard) algorithm and returns a copy of the encrypted data as a base64 encoded string to the caller. The password (or passphrase) provided by the caller is used to generate a SHA-256 hash value which is used as part of the encryption process. The identical password is required to decrypt the data using the AesDecryptString function.
- It is recommended that most applications specify a password value. If the lpszPassword parameter is NULL or specifies a zero-length string, a default internal hash value is used. This means that any other application which uses a NULL password value will be able to decrypt the data. If the Unicode version of this function is called, the lpszPassword value will be encoded using UTF-8 prior to the hash value being generated.
- Due to how the SHA-256 hash is generated, the encrypted data cannot be decrypted using another third-party library with the same password value. It can only be decrypted using the AesDecryptString function.
- The amount of encrypted data returned by this function will always be larger than original unencrypted data. If your application dynamically allocates a block of memory to store the encrypted string, provide a maximum output string size that is at least several hundred bytes larger than the unencrypted data. If the output string is not large enough, the function will fail and the GetLastError function will return ERROR_INSUFFICIENT_BUFFER.
- The string provided to this function cannot contain embedded nulls and and should not be used to encrypt binary data. If you wish to encrypt binary data, use the AesEncryptBuffer function. It will perform the same 256-bit AES encryption and return the encrypted data into an buffer provided by the caller.
- If you wish to encrypt the contents of a file, use the AesEncryptFile function.
- This function uses the Microsoft CryptoAPI and the RSA AES cryptographic provider. This provider may not be available in some languages, countries or regions. The availability of this provider may also be constrained by cryptography export restrictions imposed by the United States or other countries. If the required cryptographic provider is not available, the function will fail.

### Example

```cpp
BOOL bIsEncrypted = FALSE;
LPCTSTR lpszPassword = _T("NFr-E{Ki3_1w0iV+LI@z");
LPCTSTR lpszPlainText = _T("The quick brown fox jumped over the lazy dog.");
TCHAR szEncryptedText[MAX_STRING_LENGTH];

bIsEncrypted = AesEncryptString(lpszPassword,
								lpszPlainText,
								-1,
								szEncryptedText,
								MAX_STRING_LENGTH);

if (bIsEncrypted)
{
	_tprintf(_T("The encrypted string is \"%s\"\n"), szEncryptedText);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [AesDecryptString](#aesdecryptstring-function)
- [AesEncryptBuffer](#aesencryptbuffer-function)
- [AesEncryptFile](#aesencryptfile-function)

---

## CompareFile Function

Compare the contents of two files to determine if they are identical

### Prototype

```c
BOOL WINAPI CompareFile(
    LPCTSTR lpszFile1,
    LPCTSTR lpszFile2
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFile1` | `LPCTSTR` | A pointer to a null-terminated string which specifies the name of the first file to compare. This parameter must specify the name of a file which exists and can be opened for read access by the current process. If the file name specifies a directory or device name, the function will fail. This parameter cannot be NULL or a zero-length string. It is permitted to use environment variables in the file name by surrounding the variable name with the percent symbol. If the file name contains any leading or trailing space characters after the variable expansion, they will be removed. |
| `lpszFile2` | `LPCTSTR` | A pointer to a null-terminated string which specifies the name of the second file to compare. This parameter must specify the name of a file which exists and can be opened for read access by the current process. If the file name specifies a directory or device name, the function will fail. This parameter cannot be NULL or a zero-length string. It is permitted to use environment variables in the file name by surrounding the variable name with the percent symbol. If the file name contains any leading or trailing space characters after the variable expansion, they will be removed. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the two files are identical. A value of zero indicates the files are different or an error has occurred. Check the value returned by the GetLastError function to determine the reason the file comparison was not successful. If the files are identical, this function will clear the last error code for the current thread.

### Remarks

- The CompareFile function can be used compare the contents of two files to determine if they are the same. If you need to perform a more complex file comparison, such as comparing the file creation or modification timestamps or verifying the file contents match a computed hash value, use the CompareFileEx function.
- If the lpszFile1 and lpszFile2 file names resolve to the same file on the disk, this function will succeed without checking the contents of the file. This check is performed immediately after the file names have been expanded and converted to absolute paths. The function will still return an error if the file not does actually exist or is not a regular file.
- To avoid potential problems with relative pathing in multithreaded applications, it is recommended you use absolute paths whenever possible. If you do not specify an absolute path, the file locations will be considered relative to the current working directory for the process. Because this is shared between all threads, it is possible for one thread to change the current working directory at any time and this can create inconsistent results. If you must use relative paths in a multithreaded application, use a mutex or critical section to ensure other threads cannot change the current working directory for the process while performing the file comparison.
- This function will normalize the file names provided by the caller and perform checks to make sure the names do not contain illegal characters. If the name includes quotes, wildcard characters or other invalid symbols the function will fail and return zero with the last error code set to ST_ERROR_INVALID_FILE_NAME. It is permitted to use forward slashes in path names and these will be converted to the standard backslash character used with Windows paths. This function cannot be used to compare the contents of NTFS alternate data streams.
- This function is defined using an alias which automatically selects an ANSI or Unicode version based on the value of the UNICODE preprocessor macro. If you use the ANSI version of this function, the expanded file names will always be considered UTF-8 encoded strings, regardless of the current locale of the application. If your application uses file names which contain non-ASCII characters, you should always use the Unicode version of the function, even if your project is configured to use a multi-byte character set.

### Example

```cpp
// Compare the contents of two files
LPCTSTR lpszFile1 = _T("%USERPROFILE%\\AppName\\ProjectData.db");
LPCTSTR lpszFile2 = _T("%TEMP%\\BackupData.db");

BOOL bIdentical = CompareFile(lpszFile1, lpszFile2);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompareFileEx](#comparefileex-function)
- [CompressFile](#compressfile-function)
- [GetMessageDigest](#getmessagedigest-function)
- [ExpandFile](#expandfile-function)

---

## CompareFileEx Function

Compare the contents of two files or the contents of a file against message digest

### Prototype

```c
BOOL WINAPI CompareFileEx(
    LPCTSTR lpszFile1,
    LPCTSTR lpszFile2,
    DWORD dwOptions,
    LPCTSTR lpszDigest
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFile1` | `LPCTSTR` | A pointer to a null-terminated string which specifies the name of the first file to compare. This parameter must specify the name of a file which exists and can be opened for read access by the current process. If the file name specifies a directory or device name, the function will fail. This parameter cannot be NULL or a zero-length string. It is permitted to use environment variables in the file name by surrounding the variable name with the percent symbol. If the file name contains any leading or trailing space characters after the variable expansion, they will be removed. |
| `lpszFile2` | `LPCTSTR` | A pointer to a null-terminated string which specifies the name of the second file to compare. If this parameter is NULL or a zero-length string, the lpszDigest parameter must specify a valid message digest and the contents of the first file will be compared against that digest. If this parameter specifies a file name, the file must exist and the current process must be able to open it for read access. If the file name specifies a directory or device name, the function will fail. It is permitted to use environment variables in the file name by surrounding the variable name with the percent symbol. If the file name contains any leading or trailing space characters after the variable expansion, they will be removed. |
| `dwOptions` | `DWORD` | An unsigned integer value which specifies one or more option flags which will control how the files are compared. It may be one or more of the following values: |
| `lpszDigest` | `LPCTSTR` | A pointer to a null-terminated string which specifies a sequence of hexadecimal numbers which are used to compare against the computed hash value for the file. If this parameter is not NULL and no option is specified to indicate the hash type, it will be inferred from the length of the string. If this parameter is NULL or a zero-length string, it will be ignored. The case of the hexadecimal digits in the string are ignored and it is permitted, but not required, to prefix the string with either "0x" or "&H" to indicate it is a hexadecimal value. If the string does not specify a valid hexadecimal number or is not a value computed using one of the supported hash algorithms, the function will fail. |

### Options

- **dwOptions** Parameter

    - `COMPARE_FILE_DEFAULT`: Use the default comparison option, which is to perform a byte comparison of both files. If the lpszDigest parameter is not NULL or a zero-length string, the function will compare the message digest against the contents of the file. If the lpszDigest parameter has a value and no specific hash algorithm has been specified as one of the options, the hash type will be automatically determined based on its value

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the files are identical using the specified criteria. A value of zero indicates the files are different or an error has occurred. Check the value returned by the GetLastError function to determine the reason the file comparison was not successful. If the files are identical, this function will clear the last error code for the current thread.

### Remarks

- The CompareFileEx function can be used to either compare the contents of two files or compare the contents of a file against a pre-computed hash value to determine if they match. In addition to comparing file contents, the function can also check the file creation and/or modification times to ensure they are the same. You can use the GetMessageDigest function to compute a hash value for a file.
- In most cases, an application will specify either two files to compare or a message digest value. If you call the function using two file names and a message digest, the contents of both files must be identical and the computed hash value must match the specified message digest. If any of the requirements are are not satisfied, the function will return zero (false). For example, if the actual contents of the two files are identical, but the value specified by lpszDigest does not match the computed hash value of files, the function will still return zero.
- This function performs the comparison in a way to minimize potential disk I/O whenever possible. First, the size of the two files are compared and if they are different, no further checks are performed. Next, if either the COMPARE_FILE_TIME_CREATED or COMPARE_FILE_TIME_MODIFIED options have been specified, the file timestamps are compared and no further checks are performed if they do not match. A byte comparison of both files will only be performed after the other comparison criteria has been satisfied.
- Comparing the contents of a file against a message digest value is the most expensive operation in terms of memory and processor utilization. The larger the files are, and the more complex the hash algorithm, the more compute-intensive the operation will be. In most cases, the SHA-1 algorithm provides a good balance between resource utilization and minimizing the possibility of hash collisions where different files could potentially generate the same hash value.
- If it is critically important to minimize the possibility of a hash collision, use the SHA-256 algorithm; however, be aware of the impact it can have on performance when used with very large files. The current thread will block while the hash is being computed and if a large amount of data is being processed, this may cause the application to become non-responsive. In that case, it is recommended you create a background worker thread and call this function from within that worker thread, rather than inside the main UI thread.
- To avoid potential problems with relative pathing in multithreaded applications, it is recommended you use absolute paths whenever possible. If you do not specify an absolute path, the file locations will be considered relative to the current working directory for the process. Because this is shared between all threads, it is possible for one thread to change the current working directory at any time and this can create inconsistent results. If you must use relative paths in a multithreaded application, use a mutex or critical section to ensure other threads cannot change the current working directory for the process while performing the file comparison.
- This function will normalize the file names provided by the caller and perform checks to make sure the names do not contain illegal characters. If the name includes quotes, wildcard characters or other invalid symbols the function will fail and return zero with the last error code set to ST_ERROR_INVALID_FILE_NAME. It is permitted to use forward slashes in path names and these will be converted to the standard backslash character used with Windows paths. This function cannot be used to compare the contents of NTFS alternate data streams.
- This function is defined using an alias which automatically selects an ANSI or Unicode version based on the value of the UNICODE preprocessor macro. If you use the ANSI version of this function, the expanded file names will always be considered UTF-8 encoded strings, regardless of the current locale of the application. If your application uses file names which contain non-ASCII characters, you should always use the Unicode version of the function, even if your project is configured to use a multi-byte character set.

### Example

```cpp
// Compare the contents of a file against an SHA-1 hash value
LPCTSTR lpszFile = _T("%USERPOFILE%\\Documents\\Projects\\MyDocument.docx");
LPCTSTR lpszSHA1 = _T("541E43FDC612E41C02B770A2FD87EBB4A12EA800");
BOOL bIdentical;

bIdentical = CompareFileEx(
				lpszFile,
				NULL,
				COMPARE_FILE_DIGEST_SHA1,
				lpszSHA1);

// Compare the contents of two files
LPCTSTR lpszFile1 = _T("%USERPROFILE%\\AppName\\ProjectData.db");
LPCTSTR lpszFile2 = _T("%TEMP%\\BackupData.db");

bIdentical = CompareFileEx(
				lpszFile1,
				lpszFile2,
				COMPARE_FILE_CONTENTS,
				NULL);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompareFile](#comparefile-function)
- [CompressFile](#compressfile-function)
- [GetMessageDigest](#getmessagedigest-function)
- [ExpandFile](#expandfile-function)

---

## CompressBuffer Function

Compress the contents of the specified buffer

### Prototype

```c
BOOL WINAPI CompressBuffer(
    LPVOID lpvInputBuffer,
    DWORD cbInputBuffer,
    LPVOID lpvOutputBuffer,
    LPDWORD lpcbOutputBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpvInputBuffer` | `LPVOID` | A pointer to the buffer which contains the data to be compressed. |
| `cbInputBuffer` | `DWORD` | The number of bytes in the input buffer. This value must be greater than zero. |
| `lpvOutputBuffer` | `LPVOID` | A pointer to the buffer which will contain the compressed data. This parameter may be NULL, which can be used to determine the size of the of the resulting compressed data prior to allocating memory for it. |
| `lpcbOutputBuffer` | `LPDWORD` | A pointer to the number of bytes that may be copied into the output buffer. If the lpvOutputBuffer parameter is not NULL, this must be initialized to a non-zero value. When the function returns, the actual number of bytes of compressed data is returned in this parameter. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the data was successfully compressed. A zero value indicates that the data could not be compressed.

### Remarks

- This function will use the default Deflate algorithm to compress the contents of the buffer. This is a compression algorithm which combines the LZ77 algorithm for creating common substrings and Huffman coding to process the different frequencies of byte sequences in the data stream. It generally provides a good balance between the data compression ratio and system resources used. The compression ratio achieved by the CompressBuffer function will depend on the type of data being compressed.
- The compressed data can be expanded to its original contents by calling the ExpandBuffer function.

### Example

```cpp
cbOutputBuffer = 0;
bResult = CompressBuffer(lpInputBuffer,
						cbInputBuffer,
						NULL,
						&cbOutputBuffer);

if (bResult)
{
	lpOutputBuffer = (LPBYTE)LocalAlloc(LPTR, cbOutputBuffer);
	bResult = CompressBuffer(lpInputBuffer,
							cbInputBuffer,
							lpOutputBuffer,
							&cbOutputBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`

### See Also

- [CompressFile](#compressfile-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [ExpandBuffer](#expandbuffer-function)
- [ExpandFile](#expandfile-function)

---

## CompressFile Function

Compress the contents of the specified file

### Prototype

```c
BOOL WINAPI CompressFile(
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInputFile` | `LPCTSTR` | A pointer to a string which specifies the name of the file to be compressed. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. An error will be returned if this parameter is NULL or a zero-length string. |
| `lpszOutputFile` | `LPCTSTR` | The name of the file that is to contain the compressed file data. If the file exists, it must be a regular file that can be opened for writing by the current process and will be overwritten. If the file does not exist, it will be created. An error will be returned if a character device, such as CON: is specified as the file name. If this parameter is NULL or a zero-length string, the compression is performed in-place. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the file was successfully compressed. A zero value indicates that the input file could not be read or that the output file could not be created.

### Remarks

- The compression ratio achieved by the CompressFile function depends on the type of file that is being compressed. The compressed file is not stored in an archive format that is recognized by third-party applications such as PKZip or WinZip.
- This function will use the default Deflate algorithm to compress the file. This is a compression algorithm which combines the LZ77 algorithm for creating common substrings and Huffman coding to process the different frequencies of byte sequences in the data stream. It generally provides a good balance between the data compression ratio and system resources used. You can use the CompressFileEx function to specify an alternate compression algorithm if you wish to perform more resource-intensive file compression.
- If the output file already exists, it will be replaced with the compressed contents of the input file. If the compression fails for any reason, the output file will be deleted and the last error code will be updated indicating the cause of the failure. If you want to ensure an existing output file will be preserved if an error occurs, you should create a temporary file and use it as the output file. If this function succeeds, then you can rename or move the temporary file using a function like MoveFileEx.
- It is possible to compress the input file in-place by specifying the lpszOutputFile parameter as a NULL pointer or zero-length string. In this case, the contents of the input file will be compressed to a temporary file and if the compression succeeds, the original input file will be replaced by the temporary file. If an error occurs, the input file will remain unchanged.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompressFileEx](#compressfileex-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [ExpandFile](#expandfile-function)
- [ExpandFileEx](#expandfileex-function)

---

## CompressFileEx Function

Compress the contents of the specified file with additional options

### Prototype

```c
BOOL WINAPI CompressFileEx(
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile,
    DWORD dwCompressionType,
    DWORD dwCompressionLevel,
    LPVOID lpvReserved,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInputFile` | `LPCTSTR` | A pointer to a string which specifies the name of the file to be compressed. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. An error will be returned if this parameter is NULL or a zero-length string. |
| `lpszOutputFile` | `LPCTSTR` | The name of the file that is to contain the compressed file data. If the file exists, it must be a regular file that can be opened for writing by the current process and will be overwritten. If the file does not exist, it will be created. An error will be returned if a character device, such as CON: is specified as the file name. If this parameter is NULL or a zero-length string, the compression is performed in-place. |
| `dwCompressionType` | `DWORD` | A numeric value which determines the algorithm that will be used to compress the data. One of the following values may be specified. By default, the Deflate algorithm is used. |
| `dwCompressionLevel` | `DWORD` | A numeric value which specifies the compression level to use. A value of zero specifies that the default compression level appropriate for the selected algorithm should be used, balancing resource usage and the compression ratio of the data. A value of 1 specifies that the compression should be performed using minimal memory resources, at the expense of the compression ratio. The maximum value of 9 specifies that the algorithm should use more memory to achieve the maximum compression ratio. It is recommended that most applications use the default value of zero. |
| `lpvReserved` | `LPVOID` | A reserved parameter which should always be NULL. |
| `dwReserved` | `DWORD` | A reserved unsigned integer value which should always be zero. |

### Options

- **dwCompressionType** Parameter

    - `COMPRESSION_TYPE_DEFLATE`: A compression algorithm that combines the LZ77 algorithm for creating common substrings and Huffman coding to process the different frequencies of byte sequences in the data stream. Deflate is widely used by compression software and provides a good balance between the data compression ratio and system resources used. This is the default compression method.
    - `COMPRESSION_TYPE_BZIP2`: A compression algorithm that rearranges blocks of data in sorted order and then uses Huffman coding to process different frequencies of data within the block. Burrows-Wheeler compression provides a better compression ratio than the Deflate algorithm, however it requires more resources to perform the compression.
    - `COMPRESSION_TYPE_LZMA`: A compression algorithm that combines the LZ77 algorithm for dictionary-based compression with range encoding to efficiently represent repetitive patterns in the data. LZMA can achieve higher compression ratios than Deflate or the Burrows-Wheeler algorithms; however, it requires more memory and CPU resources due to the larger dictionary sizes and more complex encoding algorithm.

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the file was successfully compressed. A zero value indicates that the input file could not be read or that the output file could not be created.

### Remarks

- The compression ratio achieved by the CompressFileEx function depends on the type of file that is being compressed. The compressed file is not stored in an archive format that is recognized by third-party applications such as PKZip or WinZip.
- If the output file already exists, it will be replaced with the compressed contents of the input file. If the compression fails for any reason, the output file will be deleted and the last error code will be updated indicating the cause of the failure. If you want to ensure an existing output file will be preserved if an error occurs, you should create a temporary file and use it as the output file. If this function succeeds, then you can rename or move the temporary file using a function like MoveFileEx.
- It is possible to compress the input file in-place by specifying the lpszOutputFile parameter as a NULL pointer or zero-length string. In this case, the contents of the input file will be compressed to a temporary file and if the compression succeeds, the original input file will be replaced by the temporary file. If an error occurs, the input file will remain unchanged.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompressBuffer](#compressbuffer-function)
- [CompressFile](#compressfile-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [ExpandBuffer](#expandbuffer-function)
- [ExpandFile](#expandfile-function)
- [ExpandFileEx](#expandfileex-function)

---

## DecodeBuffer Function

Decode an encoded string, storing the result in the specified buffer

### Prototype

```c
LONG WINAPI DecodeBuffer(
    LPCTSTR lpszInput,
    LPBYTE lpOutput,
    LONG cbOutput,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInput` | `LPCTSTR` | A pointer to a string which contains the base64 encoded text. |
| `lpOutput` | `LPBYTE` | A pointer to a byte array buffer which is used to store the decoded data. It is recommended that the buffer be as large as the length of the encoded string. |
| `cbOutput` | `LONG` | A long integer which specifies the maximum number of bytes which may be stored in the buffer. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `DATA_DECODE_BASE64`: Decode a string that was created using the base64 encoding algorithm. This is the encoding method that is used by most modern email client software. Note that this option cannot be combined with other decoding methods.
    - `DATA_DECODE_QUOTED`: Decode a string that was created using the quoted-printable encoding algorithm. Note that this option cannot be combined with the other decoding methods.
    - `DATA_DECODE_URL`: Decode a string that was created using the URL encoding algorithm. Note that this option cannot be combined with the other decoding methods.
    - `DATA_DECODE_UTF7`: Decode a string that was created using the UTF-7 encoding algorithm. Note that this option cannot be combined with the other decoding methods.
    - `DATA_DECODE_UTF8`: Decode a string that was created using the UTF-8 encoding algorithm. Note that this option cannot be combined with the other decoding methods.
    - `DATA_DECODE_COMPRESSED`: The data was compressed prior to being encoded, and should be expanded after the decoding has completed successfully. This option is ignored if the encoding type is not base64. This should only be used if it was specified when the data was encoded.

### Return Value

- **Type**: `LONG`
- If the function succeeds, it will return the number of bytes decoded and stored in the buffer. If the function fails, it will return -1. Failure typically indicates that the encoded string was corrupted or the buffer is not large enough to store the decoded data.

### Remarks

- The DecodeBuffer function is used to decode a block of data that was previously encoded with the EncodeBuffer function. To decode the contents of a file, it is recommended that you use the DecodeFile function instead.
- If you specify either UTF-7 or UTF-8 encoding, the lpOutput parameter must point to a Unicode string and the cbOutput parameter must specify the maximum number of characters, not bytes, that can be copied into the string. The return value will be the number of Unicode characters, not bytes, that were copied into the output buffer. The conversion is always performed using the default system code page.
- It is recommended that you use the UnicodeEncodeText function to convert a string to UTF-8 encoded text, and the UnicodeDecodeText function to convert UTF-8 encoded text to a UTF-16 or localized multi-byte string.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DecodeFile](#decodefile-function)
- [EncodeBuffer](#encodebuffer-function)
- [EncodeFile](#encodefile-function)
- [GetMessageDigest](#getmessagedigest-function)
- [IsUnicodeText](#isunicodetext-function)
- [UnicodeDecodeText](#unicodedecodetext-function)
- [UnicodeEncodeText](#unicodeencodetext-function)

---

## DecodeFile Function

Decode the contents of a file using the specified decoding method

### Prototype

```c
BOOL WINAPI DecodeFile(
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile,
    DWORD dwOptions,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInputFile` | `LPCTSTR` | A pointer to a string which specifies the name of the file to be decoded. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. |
| `lpszOutputFile` | `LPCTSTR` | The name of the file that is to contain the decoded file data. If the file exists, it must be a regular file that can be opened for writing by the current process and will be overwritten. If the file does not exist, it will be created. An error will be returned if a character device, such as CON: is specified as the file name. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | This parameter is reserved and should always be set to zero. |

### Options

- **dwOptions** Parameter

    - `FILE_DECODE_BASE64`: Use the base64 algorithm for decoding the file. This is the encoding method that is used by most modern email client software. Note that this option cannot be combined with other decoding methods.
    - `FILE_DECODE_QUOTED`: Use the quoted-printable algorithm for decoding the file. Note that this option cannot be combined with the other decoding methods.
    - `FILE_DECODE_UUCODE`: Use the uudecode algorithm for decoding the file. This is a common encoding method used in UNIX systems and older email client software. Note that this option cannot be combined with other decoding methods.
    - `FILE_DECODE_YENCODE`: Use the yEnc algorithm for decoding the file. Note that this option cannot be combined with other decoding methods.
    - `FILE_DECODE_COMPRESSED`: The file was compressed prior to being encoded, and should be expanded after the decoding has completed successfully. This option should only be used if it was specified when the file was encoded.

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the file was successfully decoded. A zero value indicates that the file does not exist, the encoded file was damaged or the output file could not be created.

### Remarks

- The DecodeFile function decodes files that were previously encoded through a call to the EncodeFile function or by a third-party application such as an email client. The option to expand a previously compressed file requires that the function be able to create a temporary file on the local system in the directory specified by the TEMP environment variable. This function can only expand a file that was previously compressed with the EncodeFile or CompressFile functions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompressFile](#compressfile-function)
- [EncodeFile](#encodefile-function)
- [ExpandFile](#expandfile-function)
- [GetMessageDigest](#getmessagedigest-function)

---

## EncodeBuffer Function

Encode the contents of a buffer using the specified encoding method

### Prototype

```c
LONG WINAPI EncodeBuffer(
    LPBYTE lpInput,
    LONG cbInput,
    LPTSTR lpszOutput,
    LONG cchOutput,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpInput` | `LPBYTE` | A pointer to a buffer that contains the data to be encoded. |
| `cbInput` | `LONG` | A long integer which specifies the number of characters in the buffer which should be encoded. |
| `lpszOutput` | `LPTSTR` | A pointer to a string that will be used to store the encoded text. The length of the string must be at least 33% larger than the number of bytes being encoded, and will be terminated with a null byte. |
| `cchOutput` | `LONG` | The maximum number of characters that may be stored in the string buffer. If this value is too small to store the encoded text, the function will fail. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwOptions** Parameter

    - `DATA_ENCODE_BASE64`: Use the base64 algorithm for encoding the data. This is the encoding method that is used by most modern email client software. Note that this option cannot be combined with other encoding methods.
    - `DATA_ENCODE_QUOTED`: Use the quoted-printable algorithm for encoding the data. Most printable characters are left as-is, with control characters and 8-bit characters encoded as their hexadecimal value. Note that this option cannot be combined with the other encoding methods.
    - `DATA_ENCODE_URL`: Use the URL encoding algorithm for encoding the data. Letters and numbers are left as-is, with control characters, most punctuation and 8-bit characters encoded as their hexadecimal value. Note that this option cannot be combined with the other encoding methods.
    - `DATA_ENCODE_UTF7`: Encode the data using the Unicode Transformation Format. This converts Unicode text into 7-bit characters. Note that this option cannot be combined with the other encoding methods.
    - `DATA_ENCODE_UTF8`: Encode the data using the Unicode Transformation Format. This converts Unicode text into 8-bit characters. Note that this option cannot be combined with the other encoding methods.
    - `DATA_ENCODE_COMPRESSED`: The data should be compressed before it is encoded. To restore the original data, it must be expanded after it has been decoded. This option is ignored if the encoding type is not base64.
    - `DATA_ENCODE_LINEBREAK`: The encoded data should broken into multiple lines of text if the resulting string is longer than 72 characters. This option is ignored if the encoding type is not base64 or quoted-printable. This option should be specified if the encoded data is going to be included in an email message.

### Return Value

- **Type**: `LONG`
- If the function succeeds, it will return the number of bytes encoded and stored in the string. If the function fails, it will return -1. Failure typically indicates that the buffer is not large enough to store the encoded data.

### Remarks

- The EncodeBuffer function is used to encode a block of data and store it in the specified string buffer as printable text. To encode the contents of a file it is recommended that you use the EncodeFile function instead.
- A common use of this function is to use the Base64 algorithm to obscure a plain text string. This technique is used by some Internet application protocols when passing authentication information over a standard connection. Although it is not a secure method of encrypting data, it does prevent a casual observer from reading the encoded text.
- If you specify either UTF-7 or UTF-8 encoding, the lpInput parameter must point to a Unicode string and the cbInput parameter must specify the number of characters, not bytes, that are to be encoded. The conversion is always performed using the default system code page.
- It is recommended that you use the UnicodeEncodeText function to convert a string to UTF-8 encoded text, and the UnicodeDecodeText function to convert UTF-8 encoded text to a UTF-16 or localized multi-byte string.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DecodeBuffer](#decodebuffer-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [GetMessageDigest](#getmessagedigest-function)
- [IsUnicodeText](#isunicodetext-function)
- [UnicodeDecodeText](#unicodedecodetext-function)
- [UnicodeEncodeText](#unicodeencodetext-function)

---

## EncodeFile Function

Encode the contents of a file using the specified encoding method

### Prototype

```c
BOOL WINAPI EncodeFile(
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile,
    DWORD dwOptions,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInputFile` | `LPCTSTR` | A pointer to a string which specifies the name of the file to be encoded. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. |
| `lpszOutputFile` | `LPCTSTR` | The name of the file that is to contain the encoded file data. If the file exists, it must be a regular file that can be opened for writing by the current process and will be overwritten. If the file does not exist, it will be created. An error will be returned if a character device, such as CON: is specified as the file name. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | This parameter is reserved and should always be set to zero. |

### Options

- **dwOptions** Parameter

    - `FILE_ENCODE_BASE64`: Use the base64 algorithm for encoding the file. This is the encoding method that is used by most modern email client software. Note that this option cannot be combined with the other encoding methods.
    - `FILE_ENCODE_UUCODE`: Use the uuencode algorithm for encoding the file. This is a common encoding method used UNIX systems and older email client software. Note that this option cannot be combined with the other encoding methods.
    - `FILE_ENCODE_QUOTED`: Use the quoted-printable algorithm for encoding the file. Note that this option cannot be combined with the other encoding methods.
    - `FILE_ENCODE_YENCODE`: Use the yEnc algorithm for encoding the file. This is an encoding method that is commonly used when posting files to Usenet newsgroups. Note that this option cannot be combined with other encoding methods.
    - `FILE_ENCODE_COMPRESSED`: The file should be compressed before it is encoded. To restore the original contents of the file, it must be expanded after it has been decoded.

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the file was successfully encoded. A zero value indicates that the input file does not exist or the output file could not be created.

### Remarks

- The EncodeFile function converts binary data files to a format that contains only printable ASCII characters. The option to compress the file requires that the function be able to create a temporary file on the local system in the directory specified by the TEMP environment variables. A compressed file must be expanded with the DecodeFile or ExpandFile functions. The compressed file is not stored in an archive format that is recognized by third-party applications such as PKZip or WinZip.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompressFile](#compressfile-function)
- [DecodeFile](#decodefile-function)
- [ExpandFile](#expandfile-function)
- [GetMessageDigest](#getmessagedigest-function)

---

## ExpandBuffer Function

Expand the contents of a previously compressed buffer

### Prototype

```c
BOOL WINAPI ExpandBuffer(
    LPVOID lpvInputBuffer,
    DWORD cbInputBuffer,
    LPVOID lpvOutputBuffer,
    LPDWORD lpcbOutputBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpvInputBuffer` | `LPVOID` | A pointer to the buffer which contains the compressed data. |
| `cbInputBuffer` | `DWORD` | The number of bytes in the input buffer. This value must be greater than zero. |
| `lpvOutputBuffer` | `LPVOID` | A pointer to the buffer which will contain the expanded data. This parameter may be NULL, which can be used to determine the size of the of the resulting expanded data prior to allocating memory for it. |
| `lpcbOutputBuffer` | `LPDWORD` | A pointer to the number of bytes that may be copied into the output buffer. If the lpvOutputBuffer parameter is not NULL, this must be initialized to a non-zero value. When the function returns, the actual number of bytes of expanded data is returned in this parameter. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the data was successfully expanded. A zero value indicates that the data could not be expanded. Failure typically indicates that the compressed data was not in a recognized format.

### Remarks

- The compressed data buffer passed to the ExpandBuffer function must have been compressed by either the CompressBuffer or CompressFile functions. Data read from files that were compressed using third-party utilities such as WinZip will not be recognized by this function.

### Example

```cpp
cbOutputBuffer = 0;
bResult = ExpandBuffer(lpInputBuffer,
						cbInputBuffer,
						NULL,
						&cbOutputBuffer);

if (bResult)
{
	lpOutputBuffer = (LPBYTE)LocalAlloc(LPTR, cbOutputBuffer);
	bResult = ExpandBuffer(lpInputBuffer,
							cbInputBuffer,
							lpOutputBuffer,
							&cbOutputBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`

### See Also

- [CompressBuffer](#compressbuffer-function)
- [CompressFile](#compressfile-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [ExpandFile](#expandfile-function)

---

## ExpandFile Function

Expand the contents of a previously compressed file

### Prototype

```c
BOOL WINAPI ExpandFile(
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInputFile` | `LPCTSTR` | A pointer to a string which specifies the name of the file to be compressed. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. An error will be returned if this parameter is NULL or a zero-length string. |
| `lpszOutputFile` | `LPCTSTR` | The name of the file that is to contain the expanded file data. If the file exists, it must be a regular file that can be opened for writing by the current process and will be overwritten. If the file does not exist, it will be created. An error will be returned if a character device, such as CON: is specified as the file name. If this parameter is NULL or a zero-length string, the compression is performed in-place. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the file was successfully expanded. A zero value indicates that the input file could not be read or that the output file could not be created.

### Remarks

- The ExpandFile function can only expand files that were previously compressed using the CompressFile function. It cannot expand the contents of a file stored in an archive format used by third-party applications such as PKZip or WinZip.
- This function will use the default Deflate algorithm to expand the contents of the file. If the file was compressed using CompressFileEx using anything other than COMPRESSION_TYPE_DEFLATE, the file must be expanded using the ExpandFileEx function. This function will fail if a different compression algorithm was used to compress the file.
- If the output file already exists, it will be replaced with the expanded contents of the compressed input file. If the input file cannot be expanded for any reason, the output file will be deleted and the last error code will be updated indicating the cause of the failure. If you want to ensure an existing output file will be preserved if an error occurs, you should create a temporary file and use it as the output file. If this function succeeds, then you can rename or move the temporary file using a function like MoveFileEx.
- It is possible to expand the input file in-place by specifying the lpszOutputFile parameter as a NULL pointer or zero-length string. In this case, the contents of the input file will be expanded to a temporary file and if the operation is successful, the original compressed input file will be replaced by the temporary file. If an error occurs, the input file will remain unchanged.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompressFile](#compressfile-function)
- [CompressFileEx](#compressfileex-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [ExpandFileEx](#expandfileex-function)

---

## ExpandFileEx Function

Expand the contents of a previously compressed file with additional options

### Prototype

```c
BOOL WINAPI ExpandFileEx(
    LPCTSTR lpszInputFile,
    LPCTSTR lpszOutputFile,
    DWORD dwCompressionType,
    LPVOID lpvReserved,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszInputFile` | `LPCTSTR` | A pointer to a string which specifies the name of the file which contains the compressed data. The file must exist, and it must be a regular file that can be opened for reading by the current process. An error will be returned if a character device, such as CON: is specified as the file name. An error will be returned if this parameter is NULL or a zero-length string. |
| `lpszOutputFile` | `LPCTSTR` | The name of the file that is to contain the expanded data. If the file exists, it must be a regular file that can be opened for writing by the current process and will be overwritten. If the file does not exist, it will be created. An error will be returned if a character device, such as CON: is specified as the file name. If this parameter is NULL or a zero-length string, the compression is performed in-place. |
| `dwCompressionType` | `DWORD` | A numeric value which determines the algorithm that was used to compress the data. One of the following values may be specified. By default, the Deflate algorithm is used. |
| `lpvReserved` | `LPVOID` | A reserved parameter which should always be NULL. |
| `dwReserved` | `DWORD` | A reserved unsigned integer value which should always be zero. |

### Options

- **dwCompressionType** Parameter

    - `COMPRESSION_TYPE_UNKNOWN`: The function should attempt to determine what compression algorithm was used when the data was compressed. This is done by examining the header block of the file and checking for certain signatures which can identify the algorithm. If the algorithm cannot be determined, the function will fail. It is recommended that most applications explicitly specify the compression algorithm.
    - `COMPRESSION_TYPE_DEFLATE`: A compression algorithm that combines the LZ77 algorithm for creating common substrings and Huffman coding to process the different frequencies of byte sequences in the data stream. Deflate is widely used by compression software and provides a good balance between the data compression ratio and system resources used. This is the default compression method.
    - `COMPRESSION_TYPE_BZIP2`: A compression algorithm that rearranges blocks of data in sorted order and then uses Huffman coding to process different frequencies of data within the block. Burrows-Wheeler compression provides a better compression ratio than the Deflate algorithm, however it requires more resources to perform the compression.
    - `COMPRESSION_TYPE_LZMA`: A compression algorithm that combines the LZ77 algorithm for dictionary-based compression with range encoding to efficiently represent repetitive patterns in the data. LZMA can achieve higher compression ratios than Deflate or the Burrows-Wheeler algorithms; however, it requires more memory and CPU resources due to the larger dictionary sizes and more complex encoding algorithm.

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the file was successfully expanded. A zero value indicates that the input file could not be read or that the output file could not be created.

### Remarks

- The ExpandFileEx function can only expand files that were previously compressed using the CompressFile or CompressFileEx functions. It cannot expand the contents of a file stored in an archive format used by third-party applications such as PKZip or WinZip.
- The value of the dwCompressionType parameter must match the value which was used with the CompressFileEx function. This function will fail if you specify a different compression algorithm than what was used to compress the file. Although your application can use COMPRESSION_TYPE_UNKNOWN and attempt to automatically determine how the file was compressed, this is not always reliable.
- If the output file already exists, it will be replaced with the expanded contents of the compressed input file. If the input file cannot be expanded for any reason, the output file will be deleted and the last error code will be updated indicating the cause of the failure. If you want to ensure an existing output file will be preserved if an error occurs, you should create a temporary file and use it as the output file. If this function succeeds, then you can rename or move the temporary file using a function like MoveFileEx.
- It is possible to expand the input file in-place by specifying the lpszOutputFile parameter as a NULL pointer or zero-length string. In this case, the contents of the input file will be expanded to a temporary file and if the operation is successful, the original compressed input file will be replaced by the temporary file. If an error occurs, the input file will remain unchanged.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [CompressBuffer](#compressbuffer-function)
- [CompressFileEx](#compressfileex-function)
- [DecodeFile](#decodefile-function)
- [EncodeFile](#encodefile-function)
- [ExpandBuffer](#expandbuffer-function)
- [ExpandFile](#expandfile-function)

---

## GetMessageDigest Function

Compute a message digest for a buffer, string or data file

### Prototype

```c
BOOL WINAPI GetMessageDigest(
    LPVOID lpvBuffer,
    DWORD cbBuffer,
    DWORD dwDigestOptions,
    LPTSTR lpszDigest,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpvBuffer` | `LPVOID` | A pointer to the data that will be processed by the message digest algorithm. This parameter may be NULL, in which case the standard null hash value will be returned. Depending on the options specified, this may point to a block of memory, a null-terminated string or the name of a data file. |
| `cbBuffer` | `DWORD` | The number of bytes of data that will be processed by the message digest algorithm. If this value is zero, then the standard null hash value will be returned. If the lpvBuffer parameter is NULL, this value must be zero. If this value is -1 then lpvBuffer is considered to be a pointer to a null-terminated string. |
| `dwDigestOptions` | `DWORD` | A numeric value which specifies the algorithm that will be used to generate the message digest, and may be combined with other option flags. The algorithm may be one of the following values: |
| `lpszDigest` | `LPTSTR` | A pointer to a string that will contain the message digest computed using the contents of the specified buffer. The digest value will consist of upper-case hexadecimal numbers, with the length varying based on the algorithm selected. The string buffer must be large enough to accommodate the entire message digest, including the terminating null character. This parameter cannot be NULL. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the lpszDigest string buffer, including the terminating null character. This value cannot be zero, and must be large enough to accommodate the complete message digest string value. It is recommended that the string buffer be at least MAX_DIGEST_STRING characters in size. |

### Options

- **dwDigestOptions** Parameter

    - `MESSAGE_DIGEST_DEFAULT`: Use the default message digest algorithm, which is the same as specifying MESSAGE_DIGEST_SHA256
    - `MESSAGE_DIGEST_MD5`: Use the MD5 algorithm which generates a 128-bit hash value that is returned as as 32 digit hexadecimal string. This algorithm is typically used as a checksum to verify data integrity, however it is not considered cryptographically secure and hash collision is possible.
    - `MESSAGE_DIGEST_SHA1`: Use the SHA1 algorithm which generates a 160-bit hash value that is returned as a 40 digit hexadecimal string. This algorithm is typically used as a checksum to verify data integrity, however it is not considered cryptographically secure.
    - `MESSAGE_DIGEST_SHA256`: Use the SHA-256 algorithm which generates a 256-bit hash value that is returned as a 64 digit hexadecimal string. This algorithm is used for data integrity validation and various cryptographic uses, such as password storage and document signing.
    - `MESSAGE_DIGEST_SHA384`: Use the SHA-384 algorithm which generates a 384-bit hash value that is returned as a 96 digit hexadecimal string. This is a truncated version of the SHA-512 algorithm that uses different initial values.
    - `MESSAGE_DIGEST_SHA512`: Use the SHA-512 algorithm which generates a 512-bit hash value that is returned as a 128 digit hexadecimal string. This algorithm is used for data integrity validation and various cryptographic uses, such as password storage and document signing.

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the message digest was computed and returned in the string buffer provided by the caller. A zero value indicates that the digest could not be computed, or the string buffer provided by the caller was not large enough to contain the message digest value.

### Remarks

- The GetMessageDigest function accepts an arbitrary block of data, a null-terminated string or a data file and computes a fixed-length value (hash) that can be used to uniquely identify that content. These algorithms are designed to be one-way functions that are deterministic, have a low probability of collision and are difficult or infeasible to reverse. It should be noted that the MD5 and SHA1 algorithms are no longer considered cryptographically secure, however they are still widely used for data integrity validation to protect against accidental or unintended changes.
- If the MESSAGE_DIGEST_STRING option is specified and the Unicode version of this function is called, the string will be automatically converted from UTF-16 to UTF-8 encoding, and the UTF-8 encoded text will be used to generate the message digest. If you need to generate a message digest using the UTF-16 encoded version of the string, use the MESSAGE_DIGEST_MEMORY option instead. Keep in mind that UTF-16 encoding represents each character with two bytes, so the buffer length should be twice the number of characters in the Unicode string.
- If the MESSAGE_DIGEST_DATAFILE option is specified, the file contents will be processed as binary data with no special consideration given for the various end-of-line conventions used in text files, or any Unicode byte order marks in the file. It is permissible for the file to be empty (zero length), in which case the default null-hash value will be returned based on the algorithm selected by the caller.
- The current thread will block while the message digest is being computed. If a large amount of data is being processed, this may cause the application to become non-responsive. In that case, it is recommended that you create a background worker thread and call this function from that worker thread, rather than from the main UI thread.
- The length of the message digest string returned by this function will always be the same for a given algorithm, regardless of the size of the data buffer. For example, the length of the message digest for the SHA-256 algorithm will always be 64 characters long. Each byte of the message digest is represented by two upper-case hexadecimal numbers.
- It is permitted to provide a NULL or zero-length data buffer to this function, in which case the standard null-hash value will be returned. The same null-hash value will also be returned for a zero-length string or an empty file.

### Example

```cpp
// Get a SHA-256 message digest for a string value
LPCTSTR lpszValue = _T("The quick brown fox jumps over the lazy dog");
TCHAR szDigest[MAX_DIGEST_STRING];
BOOL bSuccess;

bSuccess = GetMessageDigest(
			lpszValue,
			(DWORD)-1L,
			MESSAGE_DIGEST_SHA256 | MESSAGE_DIGEST_STRING,
			szDigest,
			MAX_DIGEST_STRING);


// Get an MD5 message digest for the contents of a data file
LPCTSTR lpszFileName = _T("testfile.zip");

bSuccess = GetMessageDigest(
			lpszFileName,
			0,
			MESSAGE_DIGEST_MD5 | MESSAGE_DIGEST_DATAFILE,
			szDigest,
			MAX_DIGEST_STRING);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [DecodeBuffer](#decodebuffer-function)
- [DecodeFile](#decodefile-function)
- [EncodeBuffer](#encodebuffer-function)
- [EncodeFile](#encodefile-function)

---

## IsUnicodeText Function

Determine if a text buffer contains valid Unicode characters

### Prototype

```c
BOOL WINAPI IsUnicodeText(
    LPCTSTR lpString,
    INT nLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpString` | `LPCTSTR` | A pointer to a null terminated string. This parameter cannot be NULL. |
| `nLength` | `INT` | An integer value which specifies the number of characters to check. If this value is -1, the length of the string is determined by counting the number of characters up to the terminating null character. This parameter cannot be zero. |

### Return Value

- **Type**: `BOOL`
- A non-zero value is returned if the string contains valid Unicode characters. A zero value indicates the string contains characters that are not valid Unicode, or the string does not contain any Unicode characters.

### Remarks

- There are two versions of this function, IsUnicodeTextA which checks a multi-byte string to ensure that it contains valid UTF-8 encoded text, and IsUnicodeTextW which checks a wide string to ensure it contains valid UTF-16 text.
- If the value of the nLength parameter is larger than the number of characters in the string, the function will not check beyond the terminating null character. If the length of the string is unknown, specify a length of -1 and the function will check the entire contents of the string up to the terminating null character.
- If there is a byte order mark (BOM) sequence at the beginning of the string, this will be recognized by the function. A string that contains a valid BOM sequence with no corresponding text will be successfully validated by this function.
- This function does not perform checks to ensure the string contains printable characters. It only validates that the Unicode string is structurally valid. The IsUnicodeTextA function will check to make sure there are no invalid UTF-8 encodings. The IsUnicodeTextW function will check to make sure the UTF-16 string does not contain any unpaired surrogates.
- This function does not preserve state information and cannot be used to check the validity of a stream of text or binary data. It should only be used to validate complete Unicode strings that are terminated with a null character.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [UnicodeDecodeText](#unicodedecodetext-function)
- [UnicodeEncodeText](#unicodeencodetext-function)

---

## UnicodeDecodeText Function

Decode UTF-8 encoded text and return a localized string

### Prototype

```c
INT WINAPI UnicodeDecodeText(
    UINT nCodePage,
    LPCSTR lpUtf8Text,
    INT nLength,
    LPTSTR lpString,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCodePage` | `UINT` | A value that specifies the code page used when decoding the text. This parameter can be set to the value of any code page that is available in the operating system. It is recommended that you use CP_ACP which specifies the system default ANSI code page. This parameter is only used when decoding UTF-8 encoded text to a multi-byte string. |
| `lpUtf8Text` | `LPCSTR` | A pointer to a null terminated string that contains the UTF-8 encoded text to be decoded. This parameter must always specify a pointer to an 8-bit character string, regardless if the project is configured to use the Unicode or Multi-Byte character set. This parameter cannot be NULL. |
| `nLength` | `INT` | The number of bytes in the UTF-8 encoded string to be decoded. If this parameter is -1, the length of the UTF-8 encoded text is determined by counting the number of characters up to the terminating null character. |
| `lpString` | `LPTSTR` | A pointer to a string buffer that will contain the decoded UTF-8 text when the function returns. The string buffer must be large enough to contain all of the encoded text, and cannot be a NULL a pointer. |
| `nMaxLength` | `INT` | The maximum number of characters that can be copied into the string buffer. The contents of the lpString buffer will always be null terminated, and the maximum size must be large enough to include the null character. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the UTF-8 encoded text is successfully decoded, the return value is the number of characters copied to the string buffer. If the text cannot be decoded, or the string buffer is not large enough to store all of the decoded text, the function will return zero. To get extended error information, call GetLastError.

### Remarks

- There are two versions of this function, UnicodeDecodeTextA which returns a localized multi-byte string and UnicodeDecodeTextW which converts the UTF-8 encoded text to UTF-16 text. Your project configuration typically determines which version of this function is used by default.
- If the value of the nLength parameter is larger than the number of UTF-8 characters, the function will not check beyond the terminating null character. If the length of the lpUtf8Text string is unknown, specify a length of -1 and the function will decode the entire contents of the string up to the terminating null character.
- When calling UnicodeDecodeTextA to convert UTF-8 encoded text to a localized multi-byte string, it is recommended that you specify CP_ACP (zero) as the code page value unless you know it contains Unicode characters that cannot be represented using the default ANSI code page. Using CP_ACP will ensure the UTF-8 text is decoded using the current locale and language settings.
- When calling UnicodeDecodeTextW to convert UTF-8 encoded text to UTF-16 text, the code page parameter is ignored and should always be a value of zero.
- This function performs a strict check on the UTF-8 encoded text and will fail if the encoding is malformed, or in the case of being converted to a multi-byte string, if it cannot be decoded using the specified code page. It will not simply replace invalid character sequences with a default character.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IsUnicodeText](#isunicodetext-function)
- [UnicodeEncodeText](#unicodeencodetext-function)

---

## UnicodeEncodeText Function

Encode a text buffer and return a UTF-8 encoded string

### Prototype

```c
INT WINAPI UnicodeEncodeText(
    UINT nCodePage,
    LPCTSTR lpString,
    INT nLength,
    LPSTR lpUtf8Text,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `nCodePage` | `UINT` | A value that specifies the code page used when encoding the text. This parameter can be set to the value of any code page that is available in the operating system. It is recommended that you use CP_ACP which specifies the system default ANSI code page. This parameter is only used when encoding a multi-byte string. |
| `lpString` | `LPCTSTR` | A pointer to a null terminated string that contains the text to be encoded. This parameter cannot be NULL. |
| `nLength` | `INT` | The number of characters in the string to be encoded. If this parameter is -1, the length of the string is determined by counting the number of characters up to the terminating null character. |
| `lpUtf8Text` | `LPSTR` | A pointer to a character buffer which will contain the UTF-8 encoded text when the function returns. This parameter must always specify a pointer to an 8-bit character string, regardless if the project is configured to use the Unicode or Multi-Byte character set. This parameter cannot be NULL. |
| `nMaxLength` | `INT` | The maximum number of bytes that can be copied into the UTF-8 text buffer. The contents of the lpUtf8Text buffer will always be null terminated, and the maximum size must be large enough to include the null byte. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the string is successfully encoded, the return value is the number of characters copied to the output buffer. If the text cannot be encoded, or the output buffer is not large enough to store all of the encoded text, the function will return zero. To get extended error information, call GetLastError.

### Remarks

- There are two versions of this function, UnicodeEncodeTextA which converts a multi-byte string to UTF-8 encoded text, and UnicodeEncodeTextW which converts a UTF-16 string to a UTF-8 string. Your project configuration typically determines which version of this function is used by default.
- If the value of the nLength parameter is larger than the number of characters in lpString, the function will not check beyond the terminating null character. If the length of lpString is unknown, specify a length of -1 and the function will encode the entire contents of the string up to the terminating null character.
- When calling UnicodeEncodeTextA to convert a multi-byte string to UTF-8 encoding, it is recommended that you specify CP_ACP (zero) as the code page value unless you know it contains ANSI characters from a different code page. Using CP_ACP will ensure the string is encoded using the current locale and language settings.
- This function performs a strict check on the multi-byte input string and will fail if it contains a malformed multi-byte sequence or characters that cannot be converted to UTF-8 using the specified code page. It will not simply replace invalid character sequences with a default character.
- When calling UnicodeEncodeTextW to convert UTF-16 text to UTF-8 encoded text, the code page parameter is ignored and should always be a value of zero. The text will be normalized prior to being converted to UTF-8 using canonical composition, where decomposed characters are combined to create their canonical precomposed equivalent.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csncdv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [IsUnicodeText](#isunicodetext-function)
- [UnicodeDecodeText](#unicodedecodetext-function)

