# Network Virtual Terminal

## Functions

- [NvtClearDisplay](#nvtcleardisplay-function)
- [NvtConvertDisplayPos](#nvtconvertdisplaypos-function)
- [NvtCopySelectedText](#nvtcopyselectedtext-function)
- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtDeleteChar](#nvtdeletechar-function)
- [NvtDeleteLine](#nvtdeleteline-function)
- [NvtDestroyDisplay](#nvtdestroydisplay-function)
- [NvtEraseChar](#nvterasechar-function)
- [NvtEraseLine](#nvteraseline-function)
- [NvtGetCursorPos](#nvtgetcursorpos-function)
- [NvtGetDisplayAttributes](#nvtgetdisplayattributes-function)
- [NvtGetDisplayCell](#nvtgetdisplaycell-function)
- [NvtGetDisplayCellSize](#nvtgetdisplaycellsize-function)
- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtGetDisplayColorMap](#nvtgetdisplaycolormap-function)
- [NvtGetDisplayDC](#nvtgetdisplaydc-function)
- [NvtGetDisplayEmulation](#nvtgetdisplayemulation-function)
- [NvtGetDisplayFont](#nvtgetdisplayfont-function)
- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtGetDisplayLine](#nvtgetdisplayline-function)
- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtGetDisplayRect](#nvtgetdisplayrect-function)
- [NvtGetDisplayScrollPos](#nvtgetdisplayscrollpos-function)
- [NvtGetDisplaySize](#nvtgetdisplaysize-function)
- [NvtGetDisplayText](#nvtgetdisplaytext-function)
- [NvtGetDisplayWindow](#nvtgetdisplaywindow-function)
- [NvtGetMappedKey](#nvtgetmappedkey-function)
- [NvtGetScrollRegion](#nvtgetscrollregion-function)
- [NvtGetSelectedText](#nvtgetselectedtext-function)
- [NvtGetTextColor](#nvtgettextcolor-function)
- [NvtInitialize](#nvtinitialize-function)
- [NvtInsertChar](#nvtinsertchar-function)
- [NvtInsertLine](#nvtinsertline-function)
- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtResetDisplay](#nvtresetdisplay-function)
- [NvtResetMappedKeys](#nvtresetmappedkeys-function)
- [NvtResizeDisplay](#nvtresizedisplay-function)
- [NvtRestoreCursor](#nvtrestorecursor-function)
- [NvtSaveCursor](#nvtsavecursor-function)
- [NvtScrollDisplay](#nvtscrolldisplay-function)
- [NvtSelectDisplayText](#nvtselectdisplaytext-function)
- [NvtSetCursorPos](#nvtsetcursorpos-function)
- [NvtSetDisplayAttributes](#nvtsetdisplayattributes-function)
- [NvtSetDisplayBackColor](#nvtsetdisplaybackcolor-function)
- [NvtSetDisplayBoldColor](#nvtsetdisplayboldcolor-function)
- [NvtSetDisplayCell](#nvtsetdisplaycell-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)
- [NvtSetDisplayColorMap](#nvtsetdisplaycolormap-function)
- [NvtSetDisplayDC](#nvtsetdisplaydc-function)
- [NvtSetDisplayEmulation](#nvtsetdisplayemulation-function)
- [NvtSetDisplayFocus](#nvtsetdisplayfocus-function)
- [NvtSetDisplayFont](#nvtsetdisplayfont-function)
- [NvtSetDisplayFontName](#nvtsetdisplayfontname-function)
- [NvtSetDisplayForeColor](#nvtsetdisplayforecolor-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtSetDisplayScrollPos](#nvtsetdisplayscrollpos-function)
- [NvtSetDisplaySize](#nvtsetdisplaysize-function)
- [NvtSetDisplayWindow](#nvtsetdisplaywindow-function)
- [NvtSetMappedKey](#nvtsetmappedkey-function)
- [NvtSetScrollRegion](#nvtsetscrollregion-function)
- [NvtSetTextColor](#nvtsettextcolor-function)
- [NvtTranslateMappedKey](#nvttranslatemappedkey-function)
- [NvtUninitialize](#nvtuninitialize-function)
- [NvtUpdateCaret](#nvtupdatecaret-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)
- [NvtWriteDisplay](#nvtwritedisplay-function)
## Structures

- [INITDATA](#initdata-structure)
- [NVTDISPLAYINFO](#nvtdisplayinfo-structure)

---

## NvtClearDisplay Function

Clear the specified display

### Prototype

```c
BOOL WINAPI NvtClearDisplay(
    HDISPLAY hDisplay,
    UINT nMode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nMode` | `UINT` | Mode which specifies how the display will be cleared. The following values may be used: |

### Options

- **nMode** Parameter

    - `NVT_CLEAR_EOS`: The display is cleared from the current cursor position to the end of the display. The cursor position is not changed.
    - `NVT_CLEAR_TOS`: The display is cleared from the beginning of the display to the current cursor position. The cursor position is not changed.
    - `NVT_CLEAR_ALL`: The entire display is cleared and the cursor is repositioned to the upper left corner of the display.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtDeleteChar](#nvtdeletechar-function)
- [NvtDeleteLine](#nvtdeleteline-function)
- [NvtEraseChar](#nvterasechar-function)
- [NvtEraseLine](#nvteraseline-function)
- [NvtResetDisplay](#nvtresetdisplay-function)

---

## NvtConvertDisplayPos Function

Convert between pixel and screen coordinates

### Prototype

```c
BOOL WINAPI NvtConvertDisplayPos(
    HDISPLAY hDisplay,
    INT nMethod,
    INT xPos,
    INT yPos,
    LPPOINT lppt
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nMethod` | `INT` | An integer value which specifies the conversion method to use. This may be one of the following values: |
| `xPos` | `INT` | An integer value which specifies the X position in the virtual display. This may either be the cursor position or a pixel position, based on the value of the nMethod parameter. |
| `yPos` | `INT` | An integer value which specifies the Y position in the virtual display. This may either be the cursor position or a pixel position, based on the value of the nMethod parameter. |
| `lppt` | `LPPOINT` | A pointer to a POINT structure which will contain the converted coordinates. |

### Options

- **nMethod** Parameter

    - `NVT_CURSOR_TO_PIXELS`: Convert cursor X,Y coordinates to the current window X,Y pixel coordinates. An error is returned if the coordinates are out of bounds for the current display.
    - `NVT_PIXELS_TO_CURSOR`: Convert window X,Y pixel coordinates to cursor X,Y coordinates. If the point is outside of the bounds of the display, it is normalized to account for mouse capture.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetCursorPos](#nvtgetcursorpos-function)
- [NvtGetDisplayCellSize](#nvtgetdisplaycellsize-function)
- [NvtGetDisplayScrollPos](#nvtgetdisplayscrollpos-function)
- [NvtGetDisplaySize](#nvtgetdisplaysize-function)
- [NvtSetCursorPos](#nvtsetcursorpos-function)
- [NvtSetDisplayScrollPos](#nvtsetdisplayscrollpos-function)

---

## NvtCopySelectedText Function

Copy the selected text to the clipboard

### Prototype

```c
BOOL WINAPI NvtCopySelectedText(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetSelectedText](#nvtgetselectedtext-function)
- [NvtSelectDisplayText](#nvtselectdisplaytext-function)

---

## NvtCreateDisplay Function

Create a new virtual terminal emulation display

### Prototype

```c
HDISPLAY WINAPI NvtCreateDisplay(
    HWND hWnd,
    HFONT hFont,
    UINT nEmulation,
    UINT nColumns,
    UINT nRows
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hWnd` | `HWND` | Handle to the window that will be used to display the virtual terminal. |
| `hFont` | `HFONT` | Handle to the font that will be used with the virtual terminal. This parameter may be NULL, in which case a default fixed-width font will be used. If a font is specified, it must be fixed-width, otherwise the virtual cursor positioning will be incorrect in some cases. |
| `nEmulation` | `UINT` | Identifies the virtual terminal emulation type. The following emulation types are currently supported. |
| `nColumns` | `UINT` | The maximum number of columns used by the virtual display. This value must be at least 5, and no greater than 255. |
| `nRows` | `UINT` | The maximum number of rows used by the virtual display. This value must be at least 5, and no greater than 127. |

### Options

- **nEmulation** Parameter

    - `NVT_EMULATION_NONE`: The virtual display does not emulate any specific terminal type, and does not process escape sequences.
    - `NVT_EMULATION_ANSI`: The virtual display processes ANSI escape sequences for screen management and cursor positioning. This emulation also supports escape sequences to control the foreground and background color. The default keymap for ANSI function key escape sequences will be selected.
    - `NVT_EMULATION_VT100`: The virtual display processes DEC VT-100 escape sequences for screen management and cursor positioning. The default keymap for a DEC VT-100 terminal will be selected.
    - `NVT_EMULATION_VT220`: The virtual display processes DEC VT-220 escape sequences for screen management and cursor positioning. This emulation also supports DEC VT-320 escape sequences to control the foreground and background color. The default keymap for a DEC VT-220 terminal will be selected.

### Return Value

- **Type**: `HDISPLAY`
- If the function succeeds, the return value is a handle to the virtual display. If the function fails, the return value is INVALID_DISPLAY.

### Remarks

- The default colors for the display is a black background and white foreground. If ANSI terminal emulation is selected, bold characters will be displayed on a white background and blue foreground.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtDestroyDisplay](#nvtdestroydisplay-function)
- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtInitialize](#nvtinitialize-function)
- [NvtUninitialize](#nvtuninitialize-function)

---

## NvtDeleteChar Function

Delete the specified number of characters

### Prototype

```c
BOOL WINAPI NvtDeleteChar(
    HDISPLAY hDisplay,
    INT nChars
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nChars` | `INT` | Number of characters to delete from the display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the current cursor position. To erase characters from the display without affecting the characters that follow, use the NvtEraseChar function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtClearDisplay](#nvtcleardisplay-function)
- [NvtDeleteLine](#nvtdeleteline-function)
- [NvtEraseChar](#nvterasechar-function)
- [NvtEraseLine](#nvteraseline-function)

---

## NvtDeleteLine Function

Delete the current line, shifting remaining lines up

### Prototype

```c
BOOL WINAPI NvtDeleteLine(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the current cursor position. To erase a line from the display without affecting the lines that follow, use the NvtEraseLine function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtClearDisplay](#nvtcleardisplay-function)
- [NvtDeleteChar](#nvtdeletechar-function)
- [NvtEraseChar](#nvterasechar-function)
- [NvtEraseLine](#nvteraseline-function)

---

## NvtDestroyDisplay Function

Destroy the specified virtual display

### Prototype

```c
VOID WINAPI NvtDestroyDisplay(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- If the display font was specified when the display was created, then it is the responsibility of the application to delete the font object using the DeleteObject function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtInitialize](#nvtinitialize-function)
- [NvtUninitialize](#nvtuninitialize-function)

---

## NvtEraseChar Function

Erase the specified number of characters

### Prototype

```c
BOOL WINAPI NvtEraseChar(
    HDISPLAY hDisplay,
    INT nChars
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nChars` | `INT` | Number of characters to erase from the display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the current cursor position. To delete characters from the display and shift the remaining characters to the left, use the NvtDeleteChar function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtClearDisplay](#nvtcleardisplay-function)
- [NvtDeleteChar](#nvtdeletechar-function)
- [NvtDeleteLine](#nvtdeleteline-function)
- [NvtEraseLine](#nvteraseline-function)

---

## NvtEraseLine Function

Erase the current line

### Prototype

```c
BOOL WINAPI NvtEraseLine(
    HDISPLAY hDisplay,
    UINT nMode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nMode` | `UINT` | Mode which specifies how the line will be erased. The following values may be used: |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the current cursor position. To delete a line from the display and shift the remaining lines up, use the NvtDeleteLine function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtClearDisplay](#nvtcleardisplay-function)
- [NvtDeleteChar](#nvtdeletechar-function)
- [NvtDeleteLine](#nvtdeleteline-function)
- [NvtEraseChar](#nvterasechar-function)

---

## NvtGetCursorPos Function

Return the current cursor position

### Prototype

```c
BOOL WINAPI NvtGetCursorPos(
    HDISPLAY hDisplay,
    LPINT lpnCursorX,
    LPINT lpnCursorY
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpnCursorX` | `LPINT` | Address of the variable that will be set to the column of the current cursor position. If this argument is a NULL pointer, the argument is ignored. |
| `lpnCursorY` | `LPINT` | Address of the variable that will be set to the row of the current cursor position. If this argument is a NULL pointer, the argument is ignored. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero value. If the handle to the virtual display is invalid, the function will return zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtConvertDisplayPos](#nvtconvertdisplaypos-function)
- [NvtSetCursorPos](#nvtsetcursorpos-function)

---

## NvtGetDisplayAttributes Function

Return the current display attributes

### Prototype

```c
UINT WINAPI NvtGetDisplayAttributes(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value the current display attributes. If the function fails, the return value is NVT_ERROR.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayCell](#nvtgetdisplaycell-function)
- [NvtSetDisplayAttributes](#nvtsetdisplayattributes-function)

---

## NvtGetDisplayCell Function

Return information about the specified cell in the virtual display

### Prototype

```c
DWORD WINAPI NvtGetDisplayCell(
    HDISPLAY hDisplay,
    INT xPos,
    INT yPos
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `xPos` | `INT` | An integer value which specifies the X cursor position in the display. |
| `yPos` | `INT` | An integer value which specifies the Y cursor position in the display. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the cell character and attributes. If the function fails, it returns NVT_ERROR.

### Remarks

- The value returned by the NvtGetDisplayCell function is an unsigned 32-bit integer value, where the low order word specifies the ANSI character stored at that position and the high order word specifies the display attributes for that cell.
- The character cell attributes may be one or more of the following values:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayAttributes](#nvtgetdisplayattributes-function)
- [NvtGetDisplayCellSize](#nvtgetdisplaycellsize-function)
- [NvtSetDisplayAttributes](#nvtsetdisplayattributes-function)

---

## NvtGetDisplayCellSize Function

Return the size of the display character cells

### Prototype

```c
BOOL WINAPI NvtGetDisplayCellSize(
    HDISPLAY hDisplay,
    LPSIZE lpCellSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpCellSize` | `LPSIZE` | A pointer to a SIZE structure which will contain the size of a character cell in pixels. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- The NvtGetDisplayCellSize function is used to determine the size of a character cell in the display. This can be useful when the application needs to determine where a display cell is physically located within the virtual display window.
- To convert between display and window coordinates, use the NvtConvertDisplayPos function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtConvertDisplayPos](#nvtconvertdisplaypos-function)
- [NvtGetDisplayCell](#nvtgetdisplaycell-function)

---

## NvtGetDisplayColor Function

Get the current virtual display colors

### Prototype

```c
COLORREF WINAPI NvtGetDisplayColor(
    HDISPLAY hDisplay,
    UINT nColorIndex,
    BOOL bForeground
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nColorIndex` | `UINT` | The index into the virtual display color table. It may be one of the following values. |
| `bForeground` | `BOOL` | A boolean flag which specifies if the color is a foreground color, used when displaying text, or a background color. |

### Options

- **nColorIndex** Parameter

    - `NVT_COLOR_NORMAL`: The colors displayed for normal text. These are the default colors used with the display.
    - `NVT_COLOR_REVERSE`: The colors displayed for text with the reverse attribute set. This is only used when emulation is enabled.
    - `NVT_COLOR_BOLD`: The colors displayed for text with the bold attribute set. This is only used when emulation is enabled.
    - `NVT_COLOR_REVERSEBOLD`: The colors displayed for text with the reverse and bold attributes set. This is only used when emulation is enabled.

### Return Value

- **Type**: `COLORREF`
- If the function succeeds, the return value is the RGB value of the specified color. If the function fails, it returns zero.

### Remarks

- The default colors for the display is a black background and white foreground. Bold characters are displayed on a white background and blue foreground.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtSetDisplayBackColor](#nvtsetdisplaybackcolor-function)
- [NvtSetDisplayBoldColor](#nvtsetdisplayboldcolor-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)
- [NvtSetDisplayForeColor](#nvtsetdisplayforecolor-function)

---

## NvtGetDisplayColorMap Function

Return the virtual display color table

### Prototype

```c
BOOL WINAPI NvtGetDisplayColorMap(
    HDISPLAY hDisplay,
    COLORREF lpColor,
    INT nMaxColors
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpColor` | `COLORREF` | A pointer to an array of COLORREF values which will contain the color values currently being used in the virtual display. |
| `nMaxColors` | `INT` | The maximum number of colors which may be stored in the color array. The minimum value for this parameter is 1, and the maximum value is 16. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero value. If the function fails, the return value is zero. Failure indicates that the handle to the virtual display is invalid, the pointer to the color table is NULL or the maximum number of color values is invalid.

### Remarks

- When the emulator processes an escape sequence that changes the current foreground or background color, the actual RGB color value is determined by looking up the value in the virtual display's color table. The NvtGetDisplayColorMap function is useful for determining what values are being used when a color attribute is set. The emulator currently supports a maximum of sixteen (16) color values, and the index into the table corresponds to the color as defined by the standard for ANSI terminals:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtGetTextColor](#nvtgettextcolor-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)
- [NvtSetDisplayColorMap](#nvtsetdisplaycolormap-function)
- [NvtSetTextColor](#nvtsettextcolor-function)

---

## NvtGetDisplayDC Function

Get the current virtual display device context

### Prototype

```c
HDC WINAPI NvtGetDisplayDC(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `HDC`
- If the function succeeds, the return value is a handle to the device context. If the function fails, or no device context has been specified, the return value is NULL.

### Remarks

- Normally there is no device context explicitly set for the display. Instead, the device context is dynamically created and released when the virtual display is updated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtSetDisplayDC](#nvtsetdisplaydc-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtGetDisplayEmulation Function

Get the current terminal emulation type

### Prototype

```c
UINT WINAPI NvtGetDisplayEmulation(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the terminal emulation type and may contain one of the following values:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtResetDisplay](#nvtresetdisplay-function)
- [NvtSetDisplayEmulation](#nvtsetdisplayemulation-function)

---

## NvtGetDisplayFont Function

Get the current virtual display font

### Prototype

```c
HFONT WINAPI NvtGetDisplayFont(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `HFONT`
- If the function succeeds, the return value is a handle to the font. If the function fails, it returns NULL.

### Remarks

- The handle returned by this function should not be released, and the font object should never be directly modified by the application. Functions that return information about the font, such as GetTextMetrics, may be safely called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtSetDisplayFont](#nvtsetdisplayfont-function)
- [NvtSetDisplayFontName](#nvtsetdisplayfontname-function)

---

## NvtGetDisplayInfo Function

Return information about the virtual display

### Prototype

```c
BOOL WINAPI NvtGetDisplayInfo(
    HDISPLAY hDisplay,
    LPNVTDISPLAYINFO lpvdi
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpvdi` | `LPNVTDISPLAYINFO` | Pointer to a NVTDISPLAYINFO structure which contains information about the virtual display, including the display window, font, cursor and scrolling position. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtGetCursorPos](#nvtgetcursorpos-function)
- [NvtGetDisplayFont](#nvtgetdisplayfont-function)
- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtGetDisplayRect](#nvtgetdisplayrect-function)
- [NvtGetDisplayScrollPos](#nvtgetdisplayscrollpos-function)

---

## NvtGetDisplayLine Function

Return a line of text from the virtual display

### Prototype

```c
INT WINAPI NvtGetDisplayLine(
    HDISPLAY hDisplay,
    INT nRow,
    LPTSTR lpszText,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nRow` | `INT` | The row in the virtual display to return the line of text from. The first row in the display is zero. If the value -1 is specified, the row where the cursor is currently located will be used. |
| `lpszText` | `LPTSTR` | Pointer to the buffer that the display text will be copied to, terminated with a null character character. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the buffer, not including the null character terminator. If the function fails, it returns zero.

### Remarks

- The NvtGetDisplayLine function allows the application to copy the contents of the display at a specific row. Note that the buffer must be large enough to accommodate the text and the null character terminator. Unlike the NvtGetDisplayText function, any trailing whitespace in the specified row is not copied to the buffer.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtGetDisplayText](#nvtgetdisplaytext-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)
- [NvtWriteDisplay](#nvtwritedisplay-function)

---

## NvtGetDisplayMode Function

Get the current virtual display mode

### Prototype

```c
UINT WINAPI NvtGetDisplayMode(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the display mode for the virtual display. If the function fails, it returns zero.

### Remarks

- The display mode is a combination of one or more flags which determines how the emulator handles automatic line wrapping, caret display, and other functions. The following values are recognized.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayEmulation](#nvtgetdisplayemulation-function)
- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)

---

## NvtGetDisplayRect Function

Return the rectangle for the display window client area

### Prototype

```c
BOOL WINAPI NvtGetDisplayRect(
    HDISPLAY hDisplay,
    LPRECT lpRect
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpRect` | `LPRECT` | Pointer to a RECT structure which will contain the client rectangle values when the function returns. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtGetDisplaySize](#nvtgetdisplaysize-function)
- [NvtResizeDisplay](#nvtresizedisplay-function)
- [NvtSetDisplaySize](#nvtsetdisplaysize-function)

---

## NvtGetDisplayScrollPos Function

Get the display scroll box position

### Prototype

```c
INT WINAPI NvtGetDisplayScrollPos(
    HDISPLAY hDisplay,
    INT nScrollBar
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nScrollBar` | `INT` | Specifies the scroll bar to return the position for. This parameter can be one of the following values: |

### Options

- **nScrollBar** Parameter

    - `SB_HORZ`: Gets the position of the scroll box in the display's standard horizontal scroll bar.
    - `SB_VERT`: Gets the position of the scroll box in the display's standard vertical scroll bar.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the position of the scroll box. If the function fails, it returns -1.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtSetDisplayScrollPos](#nvtsetdisplayscrollpos-function)

---

## NvtGetDisplaySize Function

Return the current size of the virtual display

### Prototype

```c
BOOL WINAPI NvtGetDisplaySize(
    HDISPLAY hDisplay,
    LPSIZE lpSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpSize` | `LPSIZE` | Pointer to a SIZE structure which will contain the size of the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- The NvtGetDisplaySize function returns the number of columns and rows in the virtual display. To convert this to pixels, use the NvtGetDisplayCellSize function to determine the size of a character cell and multiply that by the number of columns and/or rows.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayCellSize](#nvtgetdisplaycellsize-function)
- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtGetDisplayRect](#nvtgetdisplayrect-function)

---

## NvtGetDisplayText Function

Get the specified block of text from the virtual display

### Prototype

```c
INT WINAPI NvtGetDisplayText(
    HDISPLAY hDisplay,
    INT nOffset,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nOffset` | `INT` | Offset into the virtual display buffer. A value of -1 specifies that the current cursor position should be used as the offset. |
| `lpszBuffer` | `LPTSTR` | Pointer to the buffer that the display text will be copied to, terminated with a null character character. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the buffer, not including the null character terminator. If the function fails, it returns zero.

### Remarks

- The NvtGetDisplayText function allows the application to copy the contents of the display at a specific location. Note that the buffer must be large enough to accommodate the text and the null character terminator. To copy an entire row of text in the display, use the NvtGetDisplayLine function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtGetDisplayLine](#nvtgetdisplayline-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)
- [NvtWriteDisplay](#nvtwritedisplay-function)

---

## NvtGetDisplayWindow Function

Get the current virtual display window

### Prototype

```c
HWND WINAPI NvtGetDisplayWindow(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `HWND`
- If the function succeeds, the return value is a handle to the virtual display window. If the function fails, it returns NULL.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)

---

## NvtGetMappedKey Function

Return the escape sequence for the mapped key

### Prototype

```c
BOOL WINAPI NvtGetMappedKey(
    HDISPLAY hDisplay,
    UINT nMappedKey,
    LPTSTR lpszKeyBuffer,
    UINT cchKeyBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nMappedKey` | `UINT` | The key which is an index into the display key mapping table. This defines the values returned by special (function) keys for the current emulation. |
| `lpszKeyBuffer` | `LPTSTR` | Address of the buffer which will receive the escape sequence mapped to the specified key. |
| `cchKeyBuffer` | `UINT` | The maximum number of characters that may be copied into the key buffer string, including the terminating null character. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero and the escape sequence for the mapped key is copied into the specified buffer. If the key has not been mapped, and there is no default escape sequence defined, then the function will return zero.

### Remarks

- The NvtGetMappedKey function returns the escape sequence that has been mapped to a special key. This function can be used to determine what sequence of characters should be sent in response to a keypress (for example, what sequence should be sent to a server when the user presses the F1 function key). If a sequence has not been explicitly mapped through a call to NvtSetMappedKey, the default sequence for the current emulation will be returned.
- Note that the current display mode, such as whether or not the emulator is in application mode or not, should be considered when determining which mapped key to use. For example, if the emulator is not in application mode and the user presses the up-arrow key, the sequence mapped to the NVT_UP key should be sent to the server. However, if the emulator is in application mode, the sequence mapped to the NVT_APPUP key should be sent instead. This is automatically handled by the NvtTranslateMappedKey function, so it is recommended that it be used when mapping a virtual keypress to the appropriate escape sequence.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtResetMappedKeys](#nvtresetmappedkeys-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtSetMappedKey](#nvtsetmappedkey-function)
- [NvtTranslateMappedKey](#nvttranslatemappedkey-function)

---

## NvtGetScrollRegion Function

Return the current scrolling region

### Prototype

```c
BOOL WINAPI NvtGetScrollRegion(
    HDISPLAY hDisplay,
    LPINT lpnTop,
    LPINT lpnBottom
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpnTop` | `LPINT` | Address of the variable that will be set to the top row of the current scrolling region. If no scrolling region has been defined, the value will be 0, the first row in the virtual display. If a NULL pointer is passed as the value, this argument will be ignored. |
| `lpnBottom` | `LPINT` | Address of the variable that will be set to the bottom row of the current scrolling region. If no scrolling region has been defined, the value will be one less than the maximum number of rows in the virtual display. If a NULL pointer is passed as the value, this argument will be ignored. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, it will return zero. If the handle to the display is invalid, the function will return zero.

### Remarks

- The NvtGetScrollRegion function allows an application to determine the top and bottom rows of the current scrolling region. By default, the scrolling region is the entire virtual display, however this may be changed through a call to NvtSetScrollRegion or an ANSI escape sequence. If the display is in origin mode, note that the cursor cannot be positioned outside of the current scrolling region.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtSetScrollRegion](#nvtsetscrollregion-function)

---

## NvtGetSelectedText Function

Return the currently selected text

### Prototype

```c
INT WINAPI NvtGetSelectedText(
    HDISPLAY hDisplay,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpszBuffer` | `LPTSTR` | Pointer to the buffer that the selected text will be copied to, terminated with a null character character. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied into the specified buffer, including the null character terminator. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of characters copied into the buffer, not including the null character terminator. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtCopySelectedText](#nvtcopyselectedtext-function)
- [NvtSelectDisplayText](#nvtselectdisplaytext-function)

---

## NvtGetTextColor Function

Return the current text foreground or background color

### Prototype

```c
BOOL WINAPI NvtGetTextColor(
    HDISPLAY hDisplay,
    COLORREF lprgbColor,
    BOOL bForeground
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lprgbColor` | `COLORREF` | A pointer to a COLORREF variable which will contain the current foreground or background text color. |
| `bForeground` | `BOOL` | A boolean value which determines if the foreground or background color is returned. A value of TRUE indicates that the foreground color should be returned, otherwise the background color is returned. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. Failure indicates that the handle to the display is invalid, the display does not support text color attributes, or the pointer to the color value is NULL.

### Remarks

- This function is used to return the current foreground or background color, as determined by the text attribute. The RGB color value for a color attribute is determined by the virtual display's color table.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColorMap](#nvtgetdisplaycolormap-function)
- [NvtSetDisplayColorMap](#nvtsetdisplaycolormap-function)
- [NvtSetTextColor](#nvtsettextcolor-function)

---

## NvtInitialize Function

Initialize the library for use by the client

### Prototype

```c
BOOL WINAPI NvtInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call NvtGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that NvtInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to NvtUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the NvtInitialize or NvtUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtDestroyDisplay](#nvtdestroydisplay-function)
- [NvtResetDisplay](#nvtresetdisplay-function)
- [NvtUninitialize](#nvtuninitialize-function)

---

## NvtInsertChar Function

Insert the specified number of characters

### Prototype

```c
BOOL WINAPI NvtInsertChar(
    HDISPLAY hDisplay,
    INT nChars
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nChars` | `INT` | Number of space characters to insert at the current cursor position. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the cursor position. Inserting space characters may cause the virtual display to scroll.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtDeleteChar](#nvtdeletechar-function)
- [NvtEraseChar](#nvterasechar-function)
- [NvtInsertLine](#nvtinsertline-function)

---

## NvtInsertLine Function

Insert a line, shifting the remaining lines down

### Prototype

```c
BOOL WINAPI NvtInsertLine(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the cursor position. Inserting a line may cause the virtual display to scroll.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtDeleteLine](#nvtdeleteline-function)
- [NvtEraseLine](#nvteraseline-function)
- [NvtInsertChar](#nvtinsertchar-function)

---

## NvtRefreshDisplay Function

Refresh the specified display

### Prototype

```c
BOOL WINAPI NvtRefreshDisplay(
    HDISPLAY hDisplay,
    BOOL bUpdate
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `bUpdate` | `BOOL` | Boolean flags which specifies if the display window is to be updated. If set, the window client area is invalidated and the virtual display is redrawn. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtUpdateCaret](#nvtupdatecaret-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)
- [NvtWriteDisplay](#nvtwritedisplay-function)

---

## NvtResetDisplay Function

Reset the virtual display

### Prototype

```c
BOOL WINAPI NvtResetDisplay(
    HDISPLAY hDisplay,
    HWND hWnd,
    HFONT hFont,
    UINT nColumns,
    UINT nRows
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `hWnd` | `HWND` | Handle to the window that will be used to display the virtual terminal. This parameter may be NULL, in which case the current window will be used. |
| `hFont` | `HFONT` | Handle to the font that will be used with the virtual terminal. This parameter may be NULL, in which case the current font will be used. If a font is specified, it must be fixed-width, otherwise the virtual cursor positioning will be incorrect in some cases. |
| `nColumns` | `UINT` | The maximum number of columns used by the virtual display. A value of zero specifies that the same number of columns should be used. If a non-zero value is specified, it must be at least 5, and no greater than 255. |
| `nRows` | `UINT` | The maximum number of rows used by the virtual display. A value of zero specifies that the same number of columns should be used. If a non-zero value is specified, it must be at least 5, and no greater than 127. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Remarks

- This function will clear the display and reset the current text attributes.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtClearDisplay](#nvtcleardisplay-function)
- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtResizeDisplay](#nvtresizedisplay-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtResetMappedKeys Function

Reset the mapped key table to default values

### Prototype

```c
BOOL WINAPI NvtResetMappedKeys(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the handle to the display is invalid, the function will return zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetMappedKey](#nvtgetmappedkey-function)
- [NvtSetMappedKey](#nvtsetmappedkey-function)
- [NvtTranslateMappedKey](#nvttranslatemappedkey-function)

---

## NvtResizeDisplay Function

Resize the virtual display

### Prototype

```c
BOOL WINAPI NvtResizeDisplay(
    HDISPLAY hDisplay,
    INT cxClient,
    INT cyClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `cxClient` | `INT` | New width of the display window in pixels. If this value is zero, the width of the virtual display remains unchanged. |
| `cyClient` | `INT` | New height of the display window in pixels. If this value is zero, the height of the virtual display remains unchanged. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function resizes the virtual display and updates the scrolling information. Typically this function is called when the display window receives a WM_SIZE message, causing the virtual display to match the size of the window's client area.
- This function will not change the size of the display window. To change the size of the display window, use the SetWindowPos function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtGetDisplayRect](#nvtgetdisplayrect-function)
- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtResetDisplay](#nvtresetdisplay-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtRestoreCursor Function

Restore the saved cursor position and text attributes

### Prototype

```c
BOOL WINAPI NvtRestoreCursor(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- Use the NvtSaveCursor function to save the current cursor position and text attributes. This function may only be called once for each time that the cursor position is saved.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetCursorPos](#nvtgetcursorpos-function)
- [NvtSaveCursor](#nvtsavecursor-function)
- [NvtSetCursorPos](#nvtsetcursorpos-function)
- [NvtUpdateCaret](#nvtupdatecaret-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtSaveCursor Function

Save the current cursor position and text attributes

### Prototype

```c
BOOL WINAPI NvtSaveCursor(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- Use the NvtRestoreCursor function to restore the cursor position and text attributes to their previous values.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetCursorPos](#nvtgetcursorpos-function)
- [NvtRestoreCursor](#nvtrestorecursor-function)
- [NvtSetCursorPos](#nvtsetcursorpos-function)
- [NvtUpdateCaret](#nvtupdatecaret-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtScrollDisplay Function

Scroll the virtual display

### Prototype

```c
BOOL WINAPI NvtScrollDisplay(
    HDISPLAY hDisplay,
    BOOL bScrollUp
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `bScrollUp` | `BOOL` | Boolean flag which specifies if the virtual display is scrolled up or down. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function does not change the current cursor position. The NvtRefreshDisplay function should be called to update the window attached to the virtual display.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayScrollPos](#nvtgetdisplayscrollpos-function)
- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtSetDisplayScrollPos](#nvtsetdisplayscrollpos-function)

---

## NvtSelectDisplayText Function

Select a region of the virtual display

### Prototype

```c
BOOL WINAPI NvtSelectDisplayText(
    HDISPLAY hDisplay,
    LPRECT lpRect,
    DWORD dwOptions
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpRect` | `LPRECT` | A pointer to a region of the display to select. The coordinates must be in display cursor coordinates, not pixels. If this parameter is NULL, any selected text in the display is unselected. |
| `dwOptions` | `DWORD` | One or more options which specifies how the region will be selected. These options may be combined using a bitwise Or operator. The following values may be used: |

### Options

- **dwOptions** Parameter

    - `NVT_SELECT_DEFAULT`: The default selection option. If there is a region of the display already selected, it will be cleared and the new region is selected.
    - `NVT_SELECT_CLIPBOARD`: Copy the selected text to the clipboard. If this option is not specified, the selected text is buffered and may be copied at a later point.
    - `NVT_SELECT_NOREFRESH`: The display is not refreshed when the region is selected. This is useful if the application is going to be selecting multiple regions of the display, or combining more than one region, in order to minimize output to the window.
    - `NVT_SELECT_NOBUFFER`: Do not buffer the text in the selected region of the display. The display will show any text as being selected, but it will not be available to be copied by the application. This can be useful if the application is going to select multiple regions and combine them.
    - `NVT_SELECT_COMBINE`: If there is already a region of the display that has been selected, the new region is combined with the previous region, selecting all of the text.
    - `NVT_SELECT_UNSELECT`: Unselect the specified region of the display.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCopySelectedText](#nvtcopyselectedtext-function)
- [NvtGetSelectedText](#nvtgetselectedtext-function)

---

## NvtSetCursorPos Function

Set the current cursor position

### Prototype

```c
BOOL WINAPI NvtSetCursorPos(
    HDISPLAY hDisplay,
    INT nCursorX,
    INT nCursorY
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nCursorX` | `INT` | New cursor column position. If this value is greater than the maximum number of columns, the current position is set to the last column on the display. The first column on the display is zero. |
| `nCursorY` | `INT` | New cursor row position. If this value is greater than the maximum number of rows, the current position is set to the last row on the display. The first row on the display is zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the handle to the virtual display is invalid, the function will return zero.

### Remarks

- The NvtSetCursorPos function sets the current cursor position on the virtual display. If the display is in origin mode (a scrolling region has been set), then the cursor row position is bound by the current region.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetCursorPos](#nvtgetcursorpos-function)

---

## NvtSetDisplayAttributes Function

Set the current display attributes

### Prototype

```c
UINT WINAPI NvtSetDisplayAttributes(
    HDISPLAY hDisplay,
    UINT nAttributes
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nAttributes` | `UINT` | An unsigned integer value which specifies the new display attributes. This may be one or more of the following values: |

### Options

- **nAttributes** Parameter

    - `NVT_ATTRIBUTE_NORMAL`: Normal, default attributes.
    - `NVT_ATTRIBUTE_REVERSE`: Foreground and background cell colors are reversed.
    - `NVT_ATTRIBUTE_BOLD`: The character is displayed using a higher intensity color.
    - `NVT_ATTRIBUTE_DIM`: The character is displayed using a lower intensity color.
    - `NVT_ATTRIBUTE_UNDERLINE`: The character is displayed with an underline.
    - `NVT_ATTRIBUTE_HIDDEN`: The character is stored in display memory, but not shown.
    - `NVT_ATTRIBUTE_PROTECT`: The character is protected and cannot be cleared.

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the previous display attributes. If the function fails, the return value is NVT_ERROR.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayAttributes](#nvtgetdisplayattributes-function)
- [NvtGetDisplayCell](#nvtgetdisplaycell-function)

---

## NvtSetDisplayBackColor Function

Set the background color for the virtual display

### Prototype

```c
BOOL WINAPI NvtSetDisplayBackColor(
    HDISPLAY hDisplay,
    COLORREF rgbBackground
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `rgbBackground` | `COLORREF` | The background color specified as a 32-bit RGB value. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function sets the background color for normal, reverse, bold and reverse-bold text attributes. To set the background color for a specific attribute, use the NvtSetDisplayColor function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtSetDisplayBoldColor](#nvtsetdisplayboldcolor-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)
- [NvtSetDisplayForeColor](#nvtsetdisplayforecolor-function)

---

## NvtSetDisplayBoldColor Function

Set the bold color for the virtual display

### Prototype

```c
BOOL WINAPI NvtSetDisplayBoldColor(
    HDISPLAY hDisplay,
    COLORREF rgbBold
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `rgbBold` | `COLORREF` | The bold attribute color specified as a 32-bit RGB value. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function sets the foreground color for the bold and reverse-bold text attributes. To set the color for a specific attribute, use the NvtSetDisplayColor function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtSetDisplayBackColor](#nvtsetdisplaybackcolor-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)
- [NvtSetDisplayForeColor](#nvtsetdisplayforecolor-function)

---

## NvtSetDisplayCell Function

Set the value of a character cell in the virtual display

### Prototype

```c
DWORD WINAPI NvtSetDisplayCell(
    HDISPLAY hDisplay,
    INT xPos,
    INT yPos,
    DWORD dwCell
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `xPos` | `INT` | An integer value which specifies the cell column in the virtual display. A value of -1 specifies that the current cursor position should be used. |
| `yPos` | `INT` | An integer value which specifies the cell row in the virtual display. A value of -1 specifies that the current cursor position should be used. |
| `dwCell` | `DWORD` | An unsigned integer which specifies the new value for the cell in the virtual display. The low order word of this value should contain the character to be displayed at that location. The high order word should specify the attributes for that cell. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the previous cell value. If the function fails, it returns 0xFFFFFFFF. The previous cell value is the same value returned by the NvtGetDisplayCell function.

### Remarks

- The NvtSetDisplayCell function is used to modify a specific character cell in the virtual display, changing both the character and the attributes for that cell. Unlike the higher level functions such as NvtWriteDisplay which process character strings and escape sequences, the NvtGetDisplayCell and NvtSetDisplayCell functions allow direct, low-level access to the virtual display in memory. After one or more cells are modified using this function, you should call the NvtRefreshDisplay function to redraw the virtual display.
- The character cell attributes may be one or more of the following values:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayAttributes](#nvtgetdisplayattributes-function)
- [NvtGetDisplayCell](#nvtgetdisplaycell-function)
- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtSetDisplayAttributes](#nvtsetdisplayattributes-function)
- [NvtWriteDisplay](#nvtwritedisplay-function)

---

## NvtSetDisplayColor Function

Set the virtual display colors

### Prototype

```c
BOOL WINAPI NvtSetDisplayColor(
    HDISPLAY hDisplay,
    UINT nColorIndex,
    COLORREF dwColor,
    BOOL bForeground
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nColorIndex` | `UINT` | The index into the virtual display color table. It may be one of the following values. |
| `dwColor` | `COLORREF` | The RGB color value that will be used. |
| `bForeground` | `BOOL` | A boolean flag which specifies if the color is a foreground color, used when displaying text, or a background color. |

### Options

- **nColorIndex** Parameter

    - `NVT_COLOR_NORMAL`: The colors displayed for normal text. These are the default colors used with the display.
    - `NVT_COLOR_REVERSE`: The colors displayed for text with the reverse attribute set. This is only used when emulation is enabled.
    - `NVT_COLOR_BOLD`: The colors displayed for text with the bold attribute set. This is only used when emulation is enabled.
    - `NVT_COLOR_REVERSEBOLD`: The colors displayed for text with the reverse and bold attributes set. This is only used when emulation is enabled.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- The default colors for the display is a black background and white foreground. Bold characters are displayed on a white background and blue foreground.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtSetDisplayBackColor](#nvtsetdisplaybackcolor-function)
- [NvtSetDisplayBoldColor](#nvtsetdisplayboldcolor-function)
- [NvtSetDisplayForeColor](#nvtsetdisplayforecolor-function)

---

## NvtSetDisplayColorMap Function

Set the virtual display color table

### Prototype

```c
BOOL WINAPI NvtSetDisplayColorMap(
    HDISPLAY hDisplay,
    COLORREF lpColor,
    INT nColors
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpColor` | `COLORREF` | A pointer to an array of COLORREF values which specifies the values to be used by the emulator when setting a color attribute. If this value is NULL, the default color table will be loaded. |
| `nColors` | `INT` | The number of colors which are stored in the array. The minimum value for this parameter is 1, and the maximum value is 16. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero value. If the function fails, the return value is zero. Failure indicates that the handle to the virtual display is invalid or the number of color values is invalid.

### Remarks

- When the emulator processes an escape sequence that changes the current foreground or background color, the actual RGB color value is determined by looking up the value in the virtual display's color table. The NvtSetDisplayColorMap function is useful for changing what values are being used when a color attribute it set. The emulator currently supports a maximum of sixteen (16) color values, and the index into the table corresponds to the color as defined by the standard for ANSI terminals:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtGetDisplayColorMap](#nvtgetdisplaycolormap-function)
- [NvtGetTextColor](#nvtgettextcolor-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)
- [NvtSetTextColor](#nvtsettextcolor-function)

---

## NvtSetDisplayDC Function

Set the current virtual display device context

### Prototype

```c
BOOL WINAPI NvtSetDisplayDC(
    HDISPLAY hDisplay,
    HDC hDC
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `hDC` | `HDC` | Handle to the device context. This parameter may be NULL, any device context that is currently associated with the virtual display will be removed. Note that the application still has the responsibility for deleting the device context, otherwise a handle leak will occur. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- It is not required that the device context be explicitly set by the application. By default, the library will use a device context created using the window attached to the virtual display. If a device context is specified by the application, it must be released when it is no longer needed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayDC](#nvtgetdisplaydc-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtSetDisplayEmulation Function

Set the current terminal emulation type

### Prototype

```c
BOOL WINAPI NvtSetDisplayEmulation(
    HDISPLAY hDisplay,
    UINT nEmulation
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nEmulation` | `UINT` | Identifies the virtual terminal emulation type. The following emulation types are currently supported. |

### Options

- **nEmulation** Parameter

    - `NVT_EMULATION_NONE`: The virtual display does not emulate any specific terminal type, and does not process any escape sequences.
    - `NVT_EMULATION_ANSI`: The virtual display will process ANSI escape sequences. The default keymap for an ANSI console is loaded.
    - `NVT_EMULATION_VT100`: The virtual display will process DEC VT100 escape sequences. The default keymap for a VT100 terminal is loaded.
    - `NVT_EMULATION_VT220`: The virtual display will process DEC VT220 escape sequences. The default keymap for a VT220 terminal is loaded.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- Changing the emulation type will not affect the current display.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayEmulation](#nvtgetdisplayemulation-function)
- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtResetDisplay](#nvtresetdisplay-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)

---

## NvtSetDisplayFocus Function

Set the focus on the virtual display

### Prototype

```c
BOOL WINAPI NvtSetDisplayFocus(
    HDISPLAY hDisplay,
    BOOL bFocus
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `bFocus` | `BOOL` | A boolean flag which specifies that the display should receive or lose focus. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- When the virtual display receives focus, it updates the current cursor position and displays the caret. When the display loses focus, the caret is hidden. This function should be called in response to the display window receiving the WM_SETFOCUS and WM_KILLFOCUS messages.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtUpdateCaret](#nvtupdatecaret-function)

---

## NvtSetDisplayFont Function

Set the current virtual display font

### Prototype

```c
BOOL WINAPI NvtSetDisplayFont(
    HDISPLAY hDisplay,
    HFONT hFont
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `hFont` | `HFONT` | Handle to the font that will be used with the virtual terminal. This parameter may be NULL, in which case a default fixed-width font will be used. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- If the previous font was a default font created by the library as the result of a NULL font handle being passed to a function, it will be released. However, if the previous font was created by the application, the DeleteObject function must be called to release it.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayFont](#nvtgetdisplayfont-function)
- [NvtSetDisplayFontName](#nvtsetdisplayfontname-function)

---

## NvtSetDisplayFontName Function

Set the current virtual display font by name

### Prototype

```c
BOOL WINAPI NvtSetDisplayFontName(
    HDISPLAY hDisplay,
    LPTSTR lpszFontName,
    INT nFontSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpszFontName` | `LPTSTR` | A pointer to a string which specifies the name of the font that will be loaded. If a NULL pointer or an empty string is passed as the value, the default Terminal font will be used. |
| `nFontSize` | `INT` | The point size of the font that will be loaded. If this value is zero, a default point size will be used. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- The NvtGetDisplayFont function will return the handle to the font created by this function. However, this handle should not be released using the DeleteObject function. Font handles created by calling this function or passing a NULL handle to the NvtSetDisplayFont function will be automatically released by the library when the font is changed or the virtual display is destroyed.
- If the previous display font was created using the CreateFont or CreateFontIndirect functions and then set using NvtSetDisplayFont, the font handle must be released by calling the DeleteObject function after this function has been called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtGetDisplayFont](#nvtgetdisplayfont-function)
- [NvtSetDisplayFont](#nvtsetdisplayfont-function)

---

## NvtSetDisplayForeColor Function

Set the foreground color for the virtual display

### Prototype

```c
BOOL WINAPI NvtSetDisplayForeColor(
    HDISPLAY hDisplay,
    COLORREF rgbForeground
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `rgbForeground` | `COLORREF` | The foreground color specified as a 32-bit RGB value. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function sets the foreground color for the normal and reverse text attributes. To set the foreground color for a specific attribute, use the NvtSetDisplayColor function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColor](#nvtgetdisplaycolor-function)
- [NvtSetDisplayBackColor](#nvtsetdisplaybackcolor-function)
- [NvtSetDisplayBoldColor](#nvtsetdisplayboldcolor-function)
- [NvtSetDisplayColor](#nvtsetdisplaycolor-function)

---

## NvtSetDisplayMode Function

Set the current virtual display mode

### Prototype

```c
BOOL WINAPI NvtSetDisplayMode(
    HDISPLAY hDisplay,
    UINT nMode,
    BOOL bEnable
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nMode` | `UINT` | The virtual display mode bitmask. This value is a combination of one or more flags which determines how the emulator handles automatic line wrapping, caret display, and other functions. The following values may be specified. |
| `bEnable` | `BOOL` | This boolean flag specifies if the specified mode is to be enabled or disabled. |

### Options

- **nMode** Parameter

    - `NVT_MODE_AUTOWRAP`: The emulator will automatically wrap to the next line when a character is written to the last column on the virtual display.
    - `NVT_MODE_SHOWCARET`: The emulator will display a caret when the display receives the focus.
    - `NVT_MODE_BLOCKCARET`: The emulator will display a block carat that is the height of the selected font characters. If this mode is not set, the caret is displayed as an underline.
    - `NVT_MODE_BELL`: The emulator will beep when a BEL character is processed.
    - `NVT_MODE_CRLF`: The cursor will automatically be positioned at the first column when a linefeed character is processed.
    - `NVT_MODE_CRNL`: The cursor will automatically advance to the next row when a carriage return character is processed.
    - `NVT_MODE_APPCURSOR`: The emulator cursor keys are placed in application mode. This mode changes the default key mappings used when the cursor (arrow) keys are translated. This corresponds to the application mode supported by DEC VT terminals.
    - `NVT_MODE_APPKEYPAD`: The emulator keypad keys are placed in application mode. This mode changes the default key mappings used when the keypad keys are translated. This corresponds to the application keypad mode supported by DEC VT terminals.
    - `NVT_MODE_ORIGIN`: The emulator is in origin mode. If enabled, the cursor cannot be positioned outside of the current scrolling region. Otherwise, the cursor can be positioned at any valid location on the virtual display.
    - `NVT_MODE_COLOR`: The emulator supports the use of escape sequences to change the foreground and background colors. This option is enabled by default if emulating an ANSI console or DEC VT220 terminal.
    - `NVT_MODE_TABOVER`: The emulator will clear the character cells between the current cursor position and the next tab stop when the HT (horizontal tab) control sequence is processed. By default this mode is disabled, and the cursor is simply positioned at the next tab stop.
    - `NVT_MODE_HSCROLL`: The emulator will display a horizontal scroll bar if the number of visible columns are less then the total number of columns in the virtual display. Disabling this mode prevents a horizontal scroll bar from being displayed, regardless of the number of visible columns. By default, this mode is enabled.
    - `NVT_MODE_VSCROLL`: The emulator will display a vertical scroll bar if the number of visible rows are less then the total number of rows in the virtual display. Disabling this mode prevents a vertical scroll bar from being displayed, regardless of the number of visible rows. By default, this mode is enabled.
    - `NVT_MODE_NOREFRESH`: The emulator will not automatically refresh the window after any change has been made to the virtual display, including changes in the cursor position or display mode. This allows the caller to make a sequence of changes, and then update the display all at one time to prevent a flicker effect.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtSetDisplayEmulation](#nvtsetdisplayemulation-function)

---

## NvtSetDisplayScrollPos Function

Set the display scroll box position

### Prototype

```c
BOOL WINAPI NvtSetDisplayScrollPos(
    HDISPLAY hDisplay,
    INT nScrollBar,
    INT nScrollPos
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nScrollBar` | `INT` | Specifies the scroll bar to be set. This parameter can be one of the following values: |
| `nScrollPos` | `INT` | Specifies the new row or column of the scroll box. The position must be within the scrolling range. |

### Options

- **nScrollBar** Parameter

    - `SB_HORZ`: Sets the position of the scroll box in the display's standard horizontal scroll bar.
    - `SB_VERT`: Sets the position of the scroll box in the display's standard vertical scroll bar.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function should always be used to change the scroll box position. The NvtSetDisplayScrollPos function will result in unpredictable behavior if used on the virtual display window.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayScrollPos](#nvtgetdisplayscrollpos-function)
- [NvtScrollDisplay](#nvtscrolldisplay-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtSetDisplaySize Function

Set the size of the virtual display

### Prototype

```c
BOOL WINAPI NvtSetDisplaySize(
    HDISPLAY hDisplay,
    LPSIZE lpSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpSize` | `LPSIZE` | Pointer to a SIZE structure which specifies the new size of the virtual display. The width should be specified in columns and the height should be specified in rows. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayInfo](#nvtgetdisplayinfo-function)
- [NvtGetDisplayRect](#nvtgetdisplayrect-function)

---

## NvtSetDisplayWindow Function

Set the current virtual display window

### Prototype

```c
BOOL WINAPI NvtSetDisplayWindow(
    HDISPLAY hDisplay,
    HWND hWnd
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `hWnd` | `HWND` | Handle to the window that will be used by the virtual display. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function will clear the display and reset the current text attributes.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-function)
- [NvtGetDisplayDC](#nvtgetdisplaydc-function)
- [NvtGetDisplayWindow](#nvtgetdisplaywindow-function)
- [NvtResetDisplay](#nvtresetdisplay-function)
- [NvtSetDisplayDC](#nvtsetdisplaydc-function)

---

## NvtSetMappedKey Function

Set the escape sequence for the specified key

### Prototype

```c
BOOL WINAPI NvtSetMappedKey(
    HDISPLAY hDisplay,
    UINT nMappedKey,
    LPCTSTR lpszKeyBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nMappedKey` | `UINT` | The key which is an index into the display key mapping table. This defines the values returned by special (function) keys for the current emulation. |
| `lpszKeyBuffer` | `LPCTSTR` | Pointer to a string which defines the sequence of characters that are to be mapped to the specified key. Passing an empty string or NULL pointer will delete the current sequence mapped to the key and restore the default value, if one has been defined. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero and the escape sequence is mapped to the specified key. If the key value is invalid or could not be mapped, then the function will return zero.

### Remarks

- The NvtSetMappedKey function maps an escape sequence to a special key. This function can be used to specify what sequence of characters should be sent in response to a keypress (for example, what sequence should be sent to a server when the user presses the F1 function key). There are a number of default sequences that are mapped to the function and cursor keys, based on the current emulation. Calling this function will override the default sequence for a key, if one has been defined.
- The following special keys are defined:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtGetMappedKey](#nvtgetmappedkey-function)
- [NvtResetMappedKeys](#nvtresetmappedkeys-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtTranslateMappedKey](#nvttranslatemappedkey-function)

---

## NvtSetScrollRegion Function

Set the current scrolling region

### Prototype

```c
BOOL WINAPI NvtSetScrollRegion(
    HDISPLAY hDisplay,
    INT nTop,
    INT nBottom
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nTop` | `INT` | The top row of the current scrolling region. If this value is greater than the bottom scrolling region row or the total number of rows in the display, it will be silently adjusted. |
| `nBottom` | `INT` | The bottom row of the current scrolling region. If this value is less than zero or the top scrolling region row, it will be silently adjusted. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, it will return zero. If the handle to the display is invalid, the function will return zero.

### Remarks

- The NvtSetScrollRegion function allows an application to set the current scrolling region for the virtual display. This specifies the region (between the top and bottom rows) in which text will normally scroll. If the display is in origin mode, the cursor cannot be positioned outside of the scrolling region.
- The minimum scrolling region that may be defined is two rows. If a scrolling region is specified that is less than two rows, the function will fail and the current scrolling region will remain unchanged. Specifying values of -1 for both arguments will reset the scrolling region to the default values (the full display).
- The DEC STBM escape sequence is used to set or clear the scrolling region of the virtual display.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtSetScrollRegion](#nvtsetscrollregion-function)

---

## NvtSetTextColor Function

Set the current text foreground or background color

### Prototype

```c
BOOL WINAPI NvtSetTextColor(
    HDISPLAY hDisplay,
    COLORREF rgbColor,
    BOOL bForeground
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `rgbColor` | `COLORREF` | A color value which specifies the current foreground or background text color. The RGB macro can be used to specify the red, green and blue values for the color. |
| `bForeground` | `BOOL` | A boolean value which determines if the foreground or background color is changed. A value of TRUE indicates that the foreground color should be changed, otherwise the background color is changed. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. Failure indicates that the handle to the display is invalid or that the current display does not support color text attributes.

### Remarks

- This function is used to change the current foreground or background color, as determined by the text attribute. Note that changing the current foreground or background text color does not affect the virtual display color table. To change how color attributes are mapped to an RGB color value, use the NvtSetDisplayColorMap function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayColorMap](#nvtgetdisplaycolormap-function)
- [NvtGetTextColor](#nvtgettextcolor-function)
- [NvtSetDisplayColorMap](#nvtsetdisplaycolormap-function)

---

## NvtTranslateMappedKey Function

Translate the keypress to a mapped key escape sequence

### Prototype

```c
BOOL WINAPI NvtTranslateMappedKey(
    HDISPLAY hDisplay,
    UINT nKey,
    UINT nFlags,
    UINT lpnMappedKey,
    LPTSTR lpszKeyBuffer,
    UINT cchBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `nKey` | `UINT` | The virtual key code for the specified key. |
| `nFlags` | `UINT` | The scan code, key-transition code, previous key state, and context code for the specified key. |
| `lpnMappedKey` | `UINT` | A pointer to an unsigned integer which will contain the index into the keymap table when the function returns. This is the same value used with the NvtGetMappedKey and NvtSetMappedKey function. If the index into the keymap is not required, this parameter can be NULL. |
| `lpszKeyBuffer` | `LPTSTR` | Address of the buffer to receive the escape sequence mapped to the specified key. If the mapped key string is not required, this parameter can be NULL. |
| `cchBuffer` | `UINT` | The maximum number of characters that can be copied into the key buffer, including the terminating null character. If the lpszKeyBuffer parameter is NULL, this value must be zero. |

### Return Value

- **Type**: `BOOL`
- If the virtual key can be mapped to an escape sequence, the function will return a non-zero value. If the key is not mapped, or one of the arguments is invalid, the function will return zero.

### Remarks

- The NvtTranslateMappedKey function allows an application to map a virtual key code to an escape sequence that is appropriate for the type of terminal that is being emulated. For example, it will return the escape sequence for the F1 function key when passed the VK_F1 key value. This function should be called when the WM_KEYDOWN message is processed by an application so that it may send the correct sequence to the server.
- This function should only be called in response to a keyboard message such as WM_KEYDOWN. To determine if a specific key has been mapped to an escape sequence, use the NvtGetMappedKey function.

### Example

```cpp
case WM_KEYDOWN:
	/* 
	* If the Num Lock key is pressed, then set the terminal into application 
	* keypad mode. This will change how the NvtTranslateMappedKey function will 
	* translate the keypad keys. 
	* 
	* Note that the terminal may also be placed into application keypad mode 
	* if emulating a DEC VT terminal and the DECNKM escape sequence is sent 
	* by the host. 
	*/
	if (wParam == VK_NUMLOCK)
		NvtSetDisplayMode(hDisplay, NVT_MODE_APPKEYPAD, !(GetKeyState(VK_NUMLOCK) & 1));
	else
	{
		BOOL bMapped;
		UINT nMappedKey;
		TCHAR szKey[128];
				
		bMapped = NvtTranslateMappedKey(hDisplay, wParam, HIWORD(lParam),
										&nMappedKey, szKey, 128);

		if (bMapped)
			TelnetWrite(hClient, szKey, lstrlen(szKey));
	} 
	break;
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [NvtGetDisplayMode](#nvtgetdisplaymode-function)
- [NvtGetMappedKey](#nvtgetmappedkey-function)
- [NvtResetMappedKeys](#nvtresetmappedkeys-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtSetMappedKey](#nvtsetmappedkey-function)

---

## NvtUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI NvtUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful NvtInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call NvtUninitialize to allow the library to free any resources allocated on behalf of the process.
- There must be a call to NvtUninitialize for every successful call to NvtInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtDestroyDisplay](#nvtdestroydisplay-function)
- [NvtUninitialize](#nvtuninitialize-function)

---

## NvtUpdateCaret Function

Update the display window caret

### Prototype

```c
BOOL WINAPI NvtUpdateCaret(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtSetCursorPos](#nvtsetcursorpos-function)
- [NvtSetDisplayMode](#nvtsetdisplaymode-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## NvtUpdateDisplay Function

Update the window attached to the virtual display

### Prototype

```c
BOOL WINAPI NvtUpdateDisplay(
    HDISPLAY hDisplay
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function should be called when the display window receives a WM_PAINT message. If the NvtSetDisplayDC function has not been called to explicitly set the display device context, this function will acquire one for the display window. In this case, the application should not create a device context for the window or call the BeginPaint and EndPaint functions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtResizeDisplay](#nvtresizedisplay-function)
- [NvtSetDisplayDC](#nvtsetdisplaydc-function)
- [NvtSetDisplayScrollPos](#nvtsetdisplayscrollpos-function)
- [NvtSetDisplayWindow](#nvtsetdisplaywindow-function)

---

## NvtWriteDisplay Function

Write the specified buffer to the virtual display

### Prototype

```c
BOOL WINAPI NvtWriteDisplay(
    HDISPLAY hDisplay,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hDisplay` | `HDISPLAY` | Handle to the virtual display. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer which contains the data to be written to the virtual display. |
| `cbBuffer` | `INT` | Number of bytes to write to the display. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, it returns zero.

### Remarks

- This function writes the data at the current cursor location. Control characters are recognized by this function and processed accordingly. If ANSI emulation is enabled, embedded escape sequences will also be parsed and processed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csnvtv11.lib`

### See Also

- [NvtGetDisplayText](#nvtgetdisplaytext-function)
- [NvtRefreshDisplay](#nvtrefreshdisplay-function)
- [NvtUpdateCaret](#nvtupdatecaret-function)
- [NvtUpdateDisplay](#nvtupdatedisplay-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## NVTDISPLAYINFO Structure

The NVTDISPLAYINFO structure contains information about the virtual terminal display.

### Definition

```c
typedef struct _NVTDISPLAYINFO
{
    HWND hWnd;
    HFONT hFont;
    INT xPos;
    INT yPos;
    INT cxClient;
    INT cyClient;
    INT cxChar;
    INT cyChar;
    INT nScrollCol;
    INT nScrollRow;
    INT nMaxScrollCol;
    INT nMaxScrollRow;
} NVTDISPLAYINFO, *LPNVTDISPLAYINFO;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `hWnd` | `HWND` | The handle to the terminal emulation display window. |
| `hFont` | `HFONT` | The handle to the current font. |
| `xPos` | `INT` | The current display x coordinate. |
| `yPos` | `INT` | The current display y coordinate. |
| `cxClient` | `INT` | The width of the client window in pixels. |
| `cyClient` | `INT` | The height of the client window in pixels. |
| `cxChar` | `INT` | The width of the current font in pixels. |
| `cyChar` | `INT` | The height of the current font in pixels. |
| `nScrollCol` | `INT` | The current horizontal scrolling column. |
| `nScrollRow` | `INT` | The current vertical scrolling row. |
| `nMaxScrollCol` | `INT` | The maximum horizontal scrolling column. |
| `nMaxScrollRow` | `INT` | The maximum vertical scrolling row. |

### Remarks

- This structure is populated by the NvtGetDisplayInfo function to provide information about the current state of the virtual terminal. Member of this structure should be modified directly by the application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

### See Also

- [NvtCreateDisplay](#nvtcreatedisplay-structure)
- [NvtGetDisplayInfo](#nvtgetdisplayinfo-structure)

