# Post Office Protocol

## Functions

- [PopAsyncConnect](#popasyncconnect-function)
- [PopAttachThread](#popattachthread-function)
- [PopCancel](#popcancel-function)
- [PopChangePassword](#popchangepassword-function)
- [PopCommand](#popcommand-function)
- [PopConnect](#popconnect-function)
- [PopCreateSecurityCredentials](#popcreatesecuritycredentials-function)
- [PopDeleteMessage](#popdeletemessage-function)
- [PopDeleteSecurityCredentials](#popdeletesecuritycredentials-function)
- [PopDisableEvents](#popdisableevents-function)
- [PopDisableTrace](#popdisabletrace-function)
- [PopDisconnect](#popdisconnect-function)
- [PopEnableEvents](#popenableevents-function)
- [PopEnableTrace](#popenabletrace-function)
- [PopEventProc](#popeventproc-function)
- [PopFreezeEvents](#popfreezeevents-function)
- [PopGetErrorString](#popgeterrorstring-function)
- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetLastError](#popgetlasterror-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageCount](#popgetmessagecount-function)
- [PopGetMessageCountEx](#popgetmessagecountex-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetMessageId](#popgetmessageid-function)
- [PopGetMessageSender](#popgetmessagesender-function)
- [PopGetMessageSize](#popgetmessagesize-function)
- [PopGetMessageUid](#popgetmessageuid-function)
- [PopGetMultiLine](#popgetmultiline-function)
- [PopGetResultCode](#popgetresultcode-function)
- [PopGetResultString](#popgetresultstring-function)
- [PopGetSecurityInformation](#popgetsecurityinformation-function)
- [PopGetStatus](#popgetstatus-function)
- [PopGetTimeout](#popgettimeout-function)
- [PopGetTransferStatus](#popgettransferstatus-function)
- [PopInitialize](#popinitialize-function)
- [PopIsBlocking](#popisblocking-function)
- [PopIsConnected](#popisconnected-function)
- [PopIsReadable](#popisreadable-function)
- [PopIsWritable](#popiswritable-function)
- [PopLogin](#poplogin-function)
- [PopOpenMessage](#popopenmessage-function)
- [PopRead](#popread-function)
- [PopRegisterEvent](#popregisterevent-function)
- [PopReset](#popreset-function)
- [PopSendMessage](#popsendmessage-function)
- [PopSetLastError](#popsetlasterror-function)
- [PopSetMultiLine](#popsetmultiline-function)
- [PopSetTimeout](#popsettimeout-function)
- [PopStoreMessage](#popstoremessage-function)
- [PopUninitialize](#popuninitialize-function)
- [PopWrite](#popwrite-function)
## Structures

- [INITDATA](#initdata-structure)
- [POPTRANSFERSTATUS](#poptransferstatus-structure)
- [SECURITYCREDENTIALS](#securitycredentials-structure)
- [SECURITYINFO](#securityinfo-structure)

---

## PopAsyncConnect Function

Connect asynchronously to the specified server

### Prototype

```c
HCLIENT WINAPI PopAsyncConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPSECURITYCREDENTIALS lpCredentials,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number should be used. For standard connections, the default port number is 110. For secure connections, the default port number is 995. If the secure port number is specified, an implicit TLS connection will be established by default. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which is used to establish the client credentials for a secure connection to the server. The function PopCreateSecurityCredentials can be used to create this structure if necessary. If a standard non-secure connection is being established, or client credentials are not required by the server, this parameter can be NULL. |
| `hEventWnd` | `HWND` | The handle to the event notification window. This window receives messages which notify the client of various asynchronous socket events that occur. If this parameter is NULL, a blocking connection is established with the server. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous socket event occurs. This value should be greater than WM_USER as defined in the Windows header files. If the hEventWnd parameter is NULL, this parameter must be specified as WM_NULL. |

### Options

- **dwOptions** Parameter

    - `POP_OPTION_NONE`: No connection options specified. A standard connection to the server will be established using the specified host name and port number.
    - `POP_OPTION_LINEBREAK`: Message data that is received from the server is read as individual lines of text terminated by a carriage return and linefeed control sequence. This option can be useful for applications that need to use the lower level network I/O functions and must process the message text on a line-by-line basis. This option is not recommended for most applications because it can have a negative impact on performance when retrieving large messages from the server.
    - `POP_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `POP_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `POP_OPTION_SECURE`: This option specifies that a secure connection should be established with the server and requires that the server support the TLS protocol. This option is the same as specifying POP_OPTION_SECURE_EXPLICIT, which initiates the secure session using the STLS command.
    - `POP_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol and the STLS command.
    - `POP_OPTION_SECURE_IMPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. It should only be used when the server expects an implicit TLS connection or does not implement RFC 2595 where the STLS command is used to negotiate a secure connection with the server.
    - `POP_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `POP_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `POP_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call PopGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopConnect](#popconnect-function)
- [PopDisconnect](#popdisconnect-function)
- [PopInitialize](#popinitialize-function)
- [PopLogin](#poplogin-function)
- [PopUninitialize](#popuninitialize-function)

---

## PopAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI PopAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The PopAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should PopAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the PopCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the PopUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopConnect](#popconnect-function)
- [PopDisconnect](#popdisconnect-function)
- [PopUninitialize](#popuninitialize-function)

---

## PopCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI PopCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- When the PopCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopIsBlocking](#popisblocking-function)
- [PopReset](#popreset-function)

---

## PopChangePassword Function

Change the specified mail account password

### Prototype

```c
BOOL WINAPI PopChangePassword(
    HCLIENT hClient,
    LPCTSTR lpszUserName,
    LPCTSTR lpszOldPassword,
    LPCTSTR lpszNewPassword
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszUserName` | `LPCTSTR` | Pointer to a string which specifies the user name of the account who's password will be changed. It is not required that this be the same user name that was used to login to the mail server. |
| `lpszOldPassword` | `LPCTSTR` | Pointer to a string which specifies the current account password. |
| `lpszNewPassword` | `LPCTSTR` | Pointer to a string which specifies the new account password. When the function returns, the user's mailbox password will be set to this value. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, it will return a non-zero value. If the function fails, it will return zero. To get extended error information, call PopGetLastError.

### Remarks

- The PopChangePassword function is used to change the password associated with the specified account on the server. The function establishes a connection to a separate service running on the server, and does not use the POP3 protocol. For this function to succeed, the server must be configured to allow password changes using the "poppass" service, running on port 106.
- Because passwords are sent over the network as clear text, this service is considered to be insecure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopConnect](#popconnect-function)
- [PopLogin](#poplogin-function)

---

## PopCommand Function

Send a command to the server

### Prototype

```c
BOOL WINAPI PopCommand(
    HCLIENT hClient,
    LPCTSTR lpszCommand,
    LPCTSTR lpszParameter
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszCommand` | `LPCTSTR` | The command which will be executed by the server. |
| `lpszParameter` | `LPCTSTR` | An optional command parameter. If the command requires more than one parameter, then they should be combined into a single string, with a space separating each parameter. If the command does not accept any parameters, this value may be NULL. |

### Return Value

- **Type**: `BOOL`
- If the command was successful, the function returns a non-zero value. If the command failed, the function returns zero. To get extended error information, call PopGetLastError.

### Remarks

- A list of valid commands can be found in the technical specification for the protocol. Many servers will list supported commands when the HELP command is used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetMultiLine](#popgetmultiline-function)
- [PopGetResultCode](#popgetresultcode-function)
- [PopGetResultString](#popgetresultstring-function)
- [PopSetMultiLine](#popsetmultiline-function)

---

## PopConnect Function

Connect to the specified server

### Prototype

```c
HCLIENT WINAPI PopConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPSECURITYCREDENTIALS lpCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number should be used. For standard connections, the default port number is 110. For secure connections, the default port number is 995. If the secure port number is specified, an implicit TLS connection will be established by default. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which is used to establish the client credentials for a secure connection to the server. The function PopCreateSecurityCredentials can be used to create this structure if necessary. If a standard non-secure connection is being established, or client credentials are not required by the server, this parameter can be NULL. |

### Options

- **dwOptions** Parameter

    - `POP_OPTION_NONE`: No connection options specified. A standard connection to the server will be established using the specified host name and port number.
    - `POP_OPTION_LINEBREAK`: Message data that is received from the server is read as individual lines of text terminated by a carriage return and linefeed control sequence. This option can be useful for applications that need to use the lower level network I/O functions and must process the message text on a line-by-line basis. This option is not recommended for most applications because it can have a negative impact on performance when retrieving large messages from the server.
    - `POP_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `POP_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `POP_OPTION_SECURE`: This option specifies that a secure connection should be established with the server and requires that the server support the TLS protocol. This option is the same as specifying POP_OPTION_SECURE_EXPLICIT, which initiates the secure session using the STLS command.
    - `POP_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. The server must support secure connections using the TLS protocol and the STLS command.
    - `POP_OPTION_SECURE_IMPLICIT`: This option specifies the client should attempt to establish a secure connection with the server. It should only be used when the server expects an implicit TLS connection or does not implement RFC 2595 where the STLS command is used to negotiate a secure connection with the server.
    - `POP_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `POP_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `POP_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call PopGetLastError.

### Remarks

- This function will block the current thread until a connection has been established or the timeout period has elapsed. To prevent it from blocking the main user interface thread, the application should create a background worker thread and establish a connection by calling PopConnect in that thread. If the application requires multiple simultaneous connections, it is recommended you create a worker thread for each connection.
- The dwOptions argument can be used to specify the threading model that is used by the library when a connection is established. By default, the handle is initially attached to the thread that created it. From that point on, until the it is released, only the owner may call functions using that handle. The ownership of the handle may be transferred from one thread to another using the PopAttachThread function.
- Specifying the POP_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopDisconnect](#popdisconnect-function)
- [PopInitialize](#popinitialize-function)
- [PopLogin](#poplogin-function)
- [PopUninitialize](#popuninitialize-function)

---

## PopCreateSecurityCredentials Function

Allocate a structure to establish client security credentials

### Prototype

```c
BOOL WINAPI PopCreateSecurityCredentials(
    DWORD dwProtocol,
    DWORD dwOptions,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    LPCTSTR lpszCertStore,
    LPCTSTR lpszCertName,
    LPVOID lpvReserved,
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. This parameter is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or options. This value should always be zero for connections using SSH. This member is constructed by using a bitwise operator with any of the following values: |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's username. A value of NULL specifies that no username is required. Currently this parameter is not used and any value specified will be ignored. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's password. A value of NULL specifies that no password is required. This parameter is only used if a PKCS #12 (PFX) certificate file is specified and that certificate has been secured with a password. This value will be ignored the current user or local machine certificate store is specified. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The function will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the function will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the function will return an error indicating that the certificate could not be found. |
| `lpvReserved` | `LPVOID` | Pointer reserved for future use. Set it to NULL when using this function. |
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. The memory for the credentials structure will be allocated by this function and must be released by calling the PopDeleteSecurityCredentials function when it is no longer needed. The pointer value must be set to NULL before the function is called. It is important to note that this is a pointer to a pointer variable, not a pointer to the SECURITYCREDENTIALS structure itself. |

### Options

- **dwProtocol** Parameter

    - `SECURITY_PROTOCOL_DEFAULT`: The default security protocol will be used when establishing a connection. This option will always request the latest version of the preferred security protocol and is the recommended value. Currently the default security protocol is TLS 1.2.
    - `SECURITY_PROTOCOL_TLS`: Either the TLS 1.2 or 1.3 protocol should be used when establishing a secure connection. The correct protocol is automatically selected, based on what version of the protocol is supported by the server. If this is the only protocol specified, SSL will be excluded from the list of supported protocols. This may be necessary for some servers which reject any attempt to use the older SSL protocol and require that only TLS be used.
    - `SECURITY_PROTOCOL_TLS10`: The TLS 1.0 protocol should be used when establishing a secure connection. This version is supported on all Windows platforms, however some servers may reject connections using version 1.0 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS11`: The TLS 1.1 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. Some servers may reject connections using version 1.1 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS12`: The TLS 1.2 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. This version of TLS offers the broadest compatibility with most servers.
    - `SECURITY_PROTOCOL_TLS13`: The TLS 1.3 protocol should be used when establishing a secure connection. This is the newest version of the protocol and is only supported on Windows 11, Windows Server 2022 and later versions of Windows. If this version is not supported by the operating system, TLS 1.2 will be used instead.
- **dwOptions** Parameter

    - `CREDENTIAL_STORE_CURRENT_USER`: The library will attempt to open a store for the current user using the certificate store name provided in this structure. If no options are specified, then this is the default behavior.
    - `CREDENTIAL_STORE_LOCAL_MACHINE`: The library will attempt to open a local machine store using the certificate store name provided in this structure. If this option is specified, the library will always search for a local machine store before searching for the store by that name for the current user.
    - `CREDENTIAL_STORE_FILENAME`: The certificate store name is a file that contains the certificate and private key that will be used.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call the PopGetLastError function.

### Remarks

- The structure that is created by this function may be used as client credentials when establishing a secure connection. This is particularly useful for programming languages other than C/C++ which may not support C structures or pointers. The pointer to the SECURITYCREDENTIALS structure can be declared as an unsigned integer variable which is passed by reference to this function, and then passed by value to the PopConnect function.
- A client application only needs to call this function if the server requires that the client provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions parameter should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore parameter should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12.These certificate files typically have an extension of .pfx or .p12. If a password was specified when the certificate was created, it must be provided in the lpszPassword parameter or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword parameters are values which are used to access the certificate store or client certificate. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session. In most cases, these parameters will be NULL.

### Example

```cpp
LPSECURITYCREDENTIALS lpSecCred = NULL;
PopCreateSecurityCredentials(SECURITY_PROTOCOL_DEFAULT,
							0,
							NULL,
							NULL,
							lpszCertStore,
							lpszCertName,
							NULL,
							&lpSecCred);

hClient = PopConnect(lpszHostName,
					POP_PORT_SECURE,
					POP_TIMEOUT,
					POP_OPTION_SECURE,
					lpSecCred);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopConnect](#popconnect-function)
- [PopDeleteSecurityCredentials](#popdeletesecuritycredentials-function)
- [PopGetSecurityInformation](#popgetsecurityinformation-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)

---

## PopDeleteMessage Function

Delete the specified message from the mailbox

### Prototype

```c
INT WINAPI PopDeleteMessage(
    HCLIENT hClient,
    UINT nMessage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessage` | `UINT` | Number of message to delete from the server. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- This function only marks the message for deletion. The message is not actually deleted until the user disconnects from the server. To prevent one or more marked messages from actually being deleted from the mailbox, call the PopReset function to reset the client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageCount](#popgetmessagecount-function)
- [PopReset](#popreset-function)

---

## PopDeleteSecurityCredentials Function

Delete the specified client security credentials

### Prototype

```c
VOID WINAPI PopDeleteSecurityCredentials(
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. On exit from the function, the pointer will be NULL. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- This function can be used to release the memory allocated to the client or server credentials after a secure connection has been terminated.

### Example

```cpp
if (lpSecCred)
	PopDeleteSecurityCredentials(&lpSecCred);

PopUninitialize();
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopCreateSecurityCredentials](#popcreatesecuritycredentials-function)
- [PopUninitialize](#popuninitialize-function)

---

## PopDisableEvents Function

Disable the event notification mechanism

### Prototype

```c
INT WINAPI PopDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- This function affects both event notification and event callbacks. Any outstanding events in the message queue should be ignored by the client application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopEnableEvents](#popenableevents-function)
- [PopFreezeEvents](#popfreezeevents-function)
- [PopRegisterEvent](#popregisterevent-function)

---

## PopDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI PopDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopEnableTrace](#popenabletrace-function)

---

## PopDisconnect Function

Disconnect from the current server

### Prototype

```c
INT WINAPI PopDisconnect(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopConnect](#popconnect-function)
- [PopUninitialize](#popuninitialize-function)

---

## PopEnableEvents Function

Enable the client event notification mechanism

### Prototype

```c
INT WINAPI PopEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the event notification window. This window receives a user-defined message which specifies the event that has occurred. If this value is NULL, event notification is disabled. |
| `uEventMsg` | `UINT` | An unsigned integer which specifies the user-defined message that is sent when an event occurs. This parameter's value must be greater than the value of WM_USER. If the hEventWnd parameter is NULL, this value must be specified as WM_NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDisableEvents](#popdisableevents-function)
- [PopFreezeEvents](#popfreezeevents-function)
- [PopRegisterEvent](#popregisterevent-function)

---

## PopEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI PopEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopDisableTrace](#popdisabletrace-function)

---

## PopEventProc Function

Callback function that processes events generated by the client

### Prototype

```c
VOID __cdecl PopEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the PopRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the PopRegisterEvent function. The PopEventProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDisableEvents](#popdisableevents-function)
- [PopEnableEvents](#popenableevents-function)
- [PopFreezeEvents](#popfreezeevents-function)
- [PopRegisterEvent](#popregisterevent-function)

---

## PopFreezeEvents Function

Suspend and resume event handling by the client

### Prototype

```c
INT WINAPI PopFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four read events occur, once event handling is resumed only one of those read events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDisableEvents](#popdisableevents-function)
- [PopEnableEvents](#popenableevents-function)
- [PopRegisterEvent](#popregisterevent-function)

---

## PopGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI PopGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetLastError](#popgetlasterror-function)
- [PopSetLastError](#popsetlasterror-function)

---

## PopGetHeaderValue Function

Return the value of the specified header field

### Prototype

```c
INT WINAPI PopGetHeaderValue(
    HCLIENT hClient,
    UINT nMessageId,
    LPCTSTR lpszHeader,
    LPTSTR lpszValue,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve header value from. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszHeader` | `LPCTSTR` | Pointer to a string which specifies the message header to retrieve. The colon should not be included in this string. |
| `lpszValue` | `LPTSTR` | Pointer to a string buffer that will contain the value of the specified message header. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the function returns the length of the header field value. If the header field is not present in the message, the function will return a value of zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetHeaderValue function returns the value of a header field from the specified message. This allows an application to be able to easily determine the value of a header (such as the sender, or the subject of the message) without downloading the entire header block or contents of the message.
- This function uses the XTND XLST command, which is an extension to the POP3 protocol. Not all servers support the use of this command. If this command is not supported by the server, the function will attempt to retrieve the entire message header and return the value for the specified header field. This enables an application to use this function even if the server does not support command extensions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetMessageId](#popgetmessageid-function)
- [PopGetMessageSender](#popgetmessagesender-function)

---

## PopGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI PopGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the PopSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the PopGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call PopSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or POP_ERROR. Those functions which call PopSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetErrorString](#popgeterrorstring-function)
- [PopSetLastError](#popsetlasterror-function)

---

## PopGetMessage Function

Retrieve the specified message from the server

### Prototype

```c
INT WINAPI PopGetMessage(
    HCLIENT hClient,
    LONG nMessageId,
    LPVOID lpvBuffer,
    LPDWORD lpdwLength,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `LONG` | Number of message to retrieve from the server. This value must be greater than zero. |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the file that was downloaded. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the server result code. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetMessage function is used to retrieve an message from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the message. In this case, the lpvBuffer parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the message data when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. See the example code below.
- This function will cause the current thread to block until the complete message has been retrieved, a timeout occurs or the operation is canceled. During the transfer, the POP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling PopEnableEvents, or by registering a callback function using the PopRegisterEvent function.
- To determine the current status of a transfer while it is in progress, use the PopGetTransferStatus function.

### Example

```cpp
HGLOBAL hgblBuffer = (HGLOBAL)NULL;
LPBYTE lpBuffer = (LPBYTE)NULL;
DWORD cbBuffer = 0;

// Return the message into block of global memory allocated by
// the GlobalAlloc function; the handle to this memory will be
// returned in the hgblBuffer parameter
nResult = PopGetMessage(hClient,
						nMessageId,
						&hgblBuffer,
						&cbBuffer,
						0);

if (nResult != POP_ERROR)
{
	// Lock the global memory handle, returning a pointer to the
	// message text
	lpBuffer = (LPBYTE)GlobalLock(hgblBuffer);
	
	// After the data has been used, the handle must be unlocked
	// and freed, otherwise a memory leak will occur
	GlobalUnlock(hgblBuffer);
	GlobalFree(hgblBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopEnableEvents](#popenableevents-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetTransferStatus](#popgettransferstatus-function)
- [PopRegisterEvent](#popregisterevent-function)

---

## PopGetMessageCount Function

Return the number of messages available in the mailbox

### Prototype

```c
INT WINAPI PopGetMessageCount(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the number of messages that are currently available. If no messages are available, either because the mailbox is empty or all of the messages have been deleted, this function will return zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDeleteMessage](#popdeletemessage-function)
- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageCountEx](#popgetmessagecountex-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopStoreMessage](#popstoremessage-function)

---

## PopGetMessageCountEx Function

Return the number of messages available in the mailbox

### Prototype

```c
INT WINAPI PopGetMessageCountEx(
    HCLIENT hClient,
    UINT lpnLastMessage,
    DWORD lpdwMailboxSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpnLastMessage` | `UINT` | Address of a variable that receives the number of the last valid message in the mailbox. If a NULL value is specified, this argument is ignored. |
| `lpdwMailboxSize` | `DWORD` | Address of a variable that receives the current size of the mailbox. This value will decrease as messages are deleted. If a NULL value is specified, this argument is ignored. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns the number of messages that are currently available. If no messages are available, either because the mailbox is empty or all of the messages have been deleted, this function will return zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetMessageCountEx function returns the number of messages available in the mailbox, the last valid message number in the mailbox and the current size of the mailbox in bytes.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDeleteMessage](#popdeletemessage-function)
- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopStoreMessage](#popstoremessage-function)

---

## PopGetMessageHeaders Function

Retrieve the specified message header from the server

### Prototype

```c
INT WINAPI PopGetMessageHeaders(
    HCLIENT hClient,
    LONG nMessageId,
    LPVOID lpvHeaders,
    LPDWORD lpdwLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `LONG` | Number of message to retrieve from the server. This value must be greater than zero. |
| `lpvHeaders` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvHeaders parameter. If the lpvHeaders parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the message that was downloaded. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetMessageHeaders function is used to retrieve an message header block from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the file. In this case, the lpvHeaders parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvHeaders parameter point to a global memory handle which will contain the message headers when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur.
- This function will cause the current thread to block until the transfer completes, a timeout occurs or the transfer is canceled. During the transfer, the POP_EVENT_PROGRESS event will be periodically fired, enabling the application to update any user interface controls. Event notification must be enabled, either by calling PopEnableEvents, or by registering a callback function using the PopRegisterEvent function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageId](#popgetmessageid-function)
- [PopOpenMessage](#popopenmessage-function)

---

## PopGetMessageId Function

Return the message ID string for the specified message

### Prototype

```c
INT WINAPI PopGetMessageId(
    HCLIENT hClient,
    UINT nMessageId,
    LPTSTR lpszMessageId,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve the unique identifier for. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszMessageId` | `LPTSTR` | Address of a string buffer to receive the message identifier. This should be at least 64 bytes in length. |
| `nMaxLength` | `INT` | The maximum length of the string buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the unique identifier string. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetMessageId function returns the message identifier from the Message-ID header of the specified message. The returned value is a typically a string which specifies the domain, date and timestamp for the message that is created when the message is submitted to the mail server for delivery. To obtain a unique identifier for the message in the mailbox, it is recommended that you use the PopGetMessageUid function instead.
- This function uses the XTND XLST command to obtain the value of the "Message-ID" header field. If this command is not supported by the server, the function will attempt to retrieve the entire message header and return the value for the specified header field. This enables an application to use this function even if the server does not support command extensions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetMessageSender](#popgetmessagesender-function)
- [PopGetMessageUid](#popgetmessageuid-function)

---

## PopGetMessageSender Function

Return the address of the message sender

### Prototype

```c
INT WINAPI PopGetMessageSender(
    HCLIENT hClient,
    UINT nMessageId,
    LPTSTR lpszAddress,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve header value from. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszAddress` | `LPTSTR` | Pointer to a string buffer that will contain the address of the message sender. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the function returns the length of the address. If the sender cannot be determined, the function will return a value of zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetMessageSender function returns the email address of the user who sent the specified message. This allows an application to be able to easily determine the sender, without downloading the entire header block or contents of the message.
- This function uses the XSENDER command, which is an extension to the POP3 protocol, to determine the address of the authenticated sender of the message. If the command is not supported, or the server was unable to authenticate the sender, the function will use the XTND XLST command to obtain the value of the "From" header field. If this command is not supported by the server, the function will attempt to retrieve the entire message header and return the value for the specified header field. This enables an application to use this function even if the server does not support command extensions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetMessageId](#popgetmessageid-function)
- [PopGetMessageUid](#popgetmessageuid-function)

---

## PopGetMessageSize Function

Return the size of the specified message

### Prototype

```c
DWORD WINAPI PopGetMessageSize(
    HCLIENT hClient,
    UINT nMessageId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve size of. This value must be greater than zero. The first message in the mailbox is message number one. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the size of the specified message in bytes. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetMessageId](#popgetmessageid-function)
- [PopGetMessageSender](#popgetmessagesender-function)
- [PopGetTransferStatus](#popgettransferstatus-function)

---

## PopGetMessageUid Function

Return the unique identifier for the specified message

### Prototype

```c
INT WINAPI PopGetMessageUid(
    HCLIENT hClient,
    UINT nMessageId,
    LPTSTR lpszMessageUID,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of message to retrieve the unique identifier for. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszMessageUID` | `LPTSTR` | Address of a string buffer to receive the unique identifier for the specified message. This should be at least 64 bytes in length. |
| `nMaxLength` | `INT` | The maximum length of the string buffer for the message UID. |

### Return Value

- **Type**: `INT`
- If the function succeeds, it returns a non-zero value. If no unique identifier is assigned to the message, the function will return zero. If an error occurs, the function returns POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetMessageUid function returns the unique message identifier for the specified message. The returned value is a string which can be used to uniquely identify a specific message in the mailbox across multiple client sessions. This is commonly used by mail clients to determine if they have already retrieved a message from the server in a previous session. The UID can also be used as a key or component of the file name to reference the message after it has been stored on the local system.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetHeaderValue](#popgetheadervalue-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetMessageId](#popgetmessageid-function)
- [PopGetMessageSender](#popgetmessagesender-function)

---

## PopGetMultiLine Function

Return the client multi-line output flag

### Prototype

```c
INT WINAPI PopGetMultiLine(
    HCLIENT hClient,
    LPBOOL lpbMultiLine
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpbMultiLine` | `LPBOOL` | A pointer to a boolean variable. This variable will be set to the current value of the client's internal multi-line flag. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The multi-line flag is used by the library to determine if multiple lines of data will be returned by the server as the result of a command. Unlike a single line response, which consists of a result code and result string, a multi-line response consists of one or more lines of text, terminated by a special end-of-data marker.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopCommand](#popcommand-function)
- [PopGetResultCode](#popgetresultcode-function)
- [PopGetResultString](#popgetresultstring-function)
- [PopSetMultiLine](#popsetmultiline-function)

---

## PopGetResultCode Function

Return the result code from the previous command

### Prototype

```c
BOOL WINAPI PopGetResultCode(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the previous command was successful, the function returns a non-zero value. If the previous command failed, the function returns zero. To get extended error information, call PopGetLastError.

### Remarks

- Unlike most other Internet application protocols, the Post Office Protocol does not return numeric result codes to indicate success or failure. If a command is successful, the server will respond with the string "+OK" and this is indicated by the PopGetResultCode function returning a non-zero value. If the command fails, the server will respond with the string "-ERR" along with a description of the error, and this is indicated by the function returning a value of zero. The description of the error returned by the server can be obtained by calling the PopGetResultString function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopCommand](#popcommand-function)
- [PopGetResultString](#popgetresultstring-function)

---

## PopGetResultString Function

Return the result string from the previous command

### Prototype

```c
INT WINAPI PopGetResultString(
    HCLIENT hClient,
    LPTSTR lpszResult,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszResult` | `LPTSTR` | A pointer to the buffer that will contain the result string returned by the server. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the result string buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the result string. If a value of zero is returned, this means that no result string was sent by the server. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetResultString function is most useful when an error occurs because the server will typically include a brief description of the cause of the error. This can then be parsed by the application or displayed to the user. The result string is updated each time the client sends a command to the server and then calls PopGetResultCode to obtain the result code for the operation.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopCommand](#popcommand-function)
- [PopGetResultCode](#popgetresultcode-function)

---

## PopGetSecurityInformation Function

Return security information about the current client connection

### Prototype

```c
BOOL WINAPI PopGetSecurityInformation(
    HCLIENT hClient,
    LPSECURITYINFO lpSecurityInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpSecurityInfo` | `LPSECURITYINFO` | A pointer to a SECURITYINFO structure which contains information about the current client connection. The dwSize member of this structure must be initialized to the size of the structure before passing the address of the structure to this function. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call PopGetLastError.

### Remarks

- This function is used to obtain security related information about the current client connection to the server. It can be used to determine if a secure connection has been established, what security protocol was selected, and information about the server certificate. Note that a secure connection has not been established, the dwProtocol structure member will contain the value SECURITY_PROTOCOL_NONE.

### Example

```cpp
// Notifies the user if the connection is secure or not
SECURITYINFO securityInfo;

securityInfo.dwSize = sizeof(SECURITYINFO);
if (PopGetSecurityInformation(hClient, &securityInfo))
{
	if (securityInfo.dwProtocol == SECURITY_PROTOCOL_NONE)
	{
		MessageBox(NULL, _T("The connection is not secure"),
						_T("Connection"), MB_OK);
	}
	else
	{
		if (securityInfo.dwCertStatus == SECURITY_CERTIFICATE_VALID)
		{
			MessageBox(NULL, _T("The connection is secure"),
							_T("Connection"), MB_OK);
		}
		else
		{
			MessageBox(NULL, _T("The server certificate not valid"),
							_T("Connection"), MB_OK);
		}
	}
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopConnect](#popconnect-function)
- [PopDisconnect](#popdisconnect-function)
- [SECURITYINFO](#securityinfo-function)

---

## PopGetStatus Function

Return the current status of the client

### Prototype

```c
INT WINAPI PopGetStatus(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the client status code. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetStatus function returns a numeric code which identifies the current state of the client session. The following values may be returned:
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopIsBlocking](#popisblocking-function)
- [PopIsConnected](#popisconnected-function)
- [PopIsReadable](#popisreadable-function)
- [PopIsWritable](#popiswritable-function)

---

## PopGetTimeout Function

Return the number of seconds until an operation times out

### Prototype

```c
INT WINAPI PopGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopConnect](#popconnect-function)
- [PopIsReadable](#popisreadable-function)
- [PopIsWritable](#popiswritable-function)
- [PopRead](#popread-function)
- [PopSetTimeout](#popsettimeout-function)

---

## PopGetTransferStatus Function

Return data transfer statistics

### Prototype

```c
INT WINAPI PopGetTransferStatus(
    HCLIENT hClient,
    LPPOPTRANSFERSTATUS lpStatus
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpStatus` | `LPPOPTRANSFERSTATUS` | A pointer to an POPTRANSFERSTATUS structure which contains information about the status of the current file transfer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopGetTransferStatus function returns information about the current file transfer, including the average number of bytes transferred per second and the estimated amount of time until the transfer completes. If there is no file currently being transferred, this function will return the status of the last successful transfer made by the client.
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopEnableEvents](#popenableevents-function)
- [PopGetStatus](#popgetstatus-function)
- [PopRegisterEvent](#popregisterevent-function)

---

## PopInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI PopInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call PopGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that PopInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to PopUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the PopInitialize or PopUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopConnect](#popconnect-function)
- [PopDisconnect](#popdisconnect-function)
- [PopUninitialize](#popuninitialize-function)

---

## PopIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI PopIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns a non-zero value. If the client is not performing a blocking operation, or the client handle is invalid, the function returns zero.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The PopIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action (such as warning the user that they must wait for the operation to complete).

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopCancel](#popcancel-function)

---

## PopIsConnected Function

Determine if the client is connected to the server

### Prototype

```c
BOOL WINAPI PopIsConnected(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is connected to a server, the function returns a non-zero value. If the client is not connected, or the client handle is invalid, the function returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopIsBlocking](#popisblocking-function)
- [PopIsReadable](#popisreadable-function)
- [PopIsWritable](#popiswritable-function)

---

## PopIsReadable Function

Determine if data can be read from the server

### Prototype

```c
BOOL WINAPI PopIsReadable(
    HCLIENT hClient,
    INT nTimeout,
    LPDWORD lpdwAvail
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |
| `lpdwAvail` | `LPDWORD` | A pointer to an unsigned integer which will contain the number of bytes available to read. This parameter may be NULL if this information is not required. |

### Return Value

- **Type**: `BOOL`
- If the client can read data from the server within the specified timeout period, the function returns a non-zero value. If the client cannot read any data, the function returns zero.

### Remarks

- On some platforms, this value will not exceed the size of the receive buffer (typically 64K bytes). Because of differences between TCP/IP stack implementations, it is not recommended that your application exclusively depend on this value to determine the exact number of bytes available. Instead, it should be used as a general indicator that there is data available to be read.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetStatus](#popgetstatus-function)
- [PopIsBlocking](#popisblocking-function)
- [PopIsConnected](#popisconnected-function)
- [PopIsWritable](#popiswritable-function)
- [PopRead](#popread-function)

---

## PopIsWritable Function

Determine if data can be written to the server

### Prototype

```c
BOOL WINAPI PopIsWritable(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |

### Return Value

- **Type**: `BOOL`
- If the client can write data to the server within the specified timeout period, the function returns a non-zero value. If the client cannot write any data, the function returns zero.

### Remarks

- Although this function can be used to determine if some amount of data can be sent to the remote process it does not indicate the amount of data that can be written without blocking the client. In most cases, it is recommended that large amounts of data be broken into smaller logical blocks, typically some multiple of 512 bytes in length.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetStatus](#popgetstatus-function)
- [PopIsBlocking](#popisblocking-function)
- [PopIsConnected](#popisconnected-function)
- [PopIsReadable](#popisreadable-function)
- [PopWrite](#popwrite-function)

---

## PopLogin Function

Login to the server

### Prototype

```c
INT WINAPI PopLogin(
    HCLIENT hClient,
    UINT nAuthType,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nAuthType` | `UINT` | Identifies the type of authentication that should be used when the client logs in to the mail server. The following authentication methods are supported: |
| `lpszUserName` | `LPCTSTR` | A null terminated string which specifies the user name to be used to authenticate the current client session. For many service providers, the user name is the full email address of the user which owns the mailbox. In some cases, this may only be the portion of their email address before the domain name. |
| `lpszPassword` | `LPCTSTR` | A null terminated string which specifies the password to be used when authenticating the current client session. If you are using the POP_AUTH_XOAUTH2 or POP_AUTH_BEARER authentication methods, this parameter is not a password, instead it specifies the OAuth 2.0 bearer token provided by the mail service. |

### Options

- **nAuthType** Parameter

    - `POP_AUTH_DEFAULT`: The default authentication scheme which sends the username and password as cleartext to the server. Because the user credentials are not encrypted, this method should only be used over a secure connection. This is the same as specifying POP_AUTH_PASS as the authentication method.
    - `POP_AUTH_PASS`: The username and password is sent to the server using the USER and PASS commands. This authentication method is supported by most servers and is the default authentication type. The credentials are not encrypted and this method should only be used over secure connections.
    - `POP_AUTH_APOP`: The APOP authentication method which uses an MD5 digest of the password. This method has been deprecated is not supported by all servers. It should only be used if required by legacy mail servers which do not support the SASL authentication methods.
    - `POP_AUTH_LOGIN`: This authentication type will use the LOGIN method to authenticate the client session. This encodes the username and password in a specific format, but the credentials are not encrypted and should only be used over a secure connection. The server must support the Simple Authentication and Security Layer (SASL) mechanism as defined in RFC 4422.
    - `POP_AUTH_PLAIN`: This authentication type will use the PLAIN method to authenticate the client session. This encodes the username and password in a specific format, but the credentials are not encrypted and should only be used over a secure connection. The server must support the PLAIN Simple Authentication and Security Layer (SASL) mechanism as defined in RFC 4616.
    - `POP_AUTH_XOAUTH2`: This authentication type will use the XOAUTH2 method to authenticate the client session. This authentication method does not require the user password, instead the lpszPassword parameter must specify the OAuth 2.0 bearer token issued by the service provider. The application must provide a valid access token which has not expired, or this function will fail.
    - `POP_AUTH_BEARER`: This authentication type will use the OAUTHBEARER method to authenticate the client session as defined in RFC 7628. This authentication method does not require the user password, instead the lpszPassword parameter must specify the OAuth 2.0 bearer token issued by the service provider. The application must provide a valid access token which has not expired, or this function will fail.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The POP_AUTH_LOGIN and POP_AUTH_PLAIN authentication methods require the mail server support the Simple Authentication and Security Layer (SASL) AUTH command as defined in RFC 5034. Most modern mail servers do support one or both of these methods, and they are generally preferred over the POP_AUTH_PASS method when possible. However, for backwards compatibility with legacy servers, the API will default to using POP_AUTH_PASS for client authentication.
- You should only use an OAuth 2.0 authentication method if you understand the process of how to request the access token. Obtaining an access token requires registering your application with the mail service provider (e.g.: Microsoft or Google), getting a unique client ID associated with your application and then requesting the access token using the appropriate scope for the service. Obtaining the initial token will typically involve interactive confirmation on the part of the user, requiring they grant permission to your application to access their mail account.
- The POP_AUTH_XOAUTH2 and POP_AUTH_BEARER authentication methods are similar, but they are not interchangeable. Both use an OAuth 2.0 bearer token to authenticate the client session, but they differ in how the token is presented to the server. It is currently preferable to use the XOAUTH2 method because it is more widely available and some service providers do not yet support the OAUTHBEARER method.
- Your application should not store an OAuth 2.0 bearer token for later use. They have a relatively short lifespan, typically about an hour, and are designed to be used with that session. You should specify offline access as part of the OAuth 2.0 scope if necessary and store the refresh token provided by the service. The refresh token has a much longer validity period and can be used to obtain a new bearer token when needed.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopConnect](#popconnect-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageCountEx](#popgetmessagecountex-function)
- [PopInitialize](#popinitialize-function)

---

## PopOpenMessage Function

Open the specified message for reading on the server

### Prototype

```c
INT WINAPI PopOpenMessage(
    HCLIENT hClient,
    UINT nMessageId,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number that specifies which message to open. This value must be greater than zero. The first message in the mailbox is message one. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopOpenMessage function is used to begin the process of reading the contents of a message from the from the server. Similar to how a file is opened and read, this method is followed by one or more calls to the PopRead method. When the entire contents of the message has been read, the PopCloseMessage method is used to close the message, completing the transaction on the server.
- This is a lower-level function which enables the application to process the message as the contents are being returned by the server. In general, it is recommended that most applications use the PopGetMessage method instead, which provides a simpler method for retrieving the contents of a message.
- It is important to note that you cannot use this function to read the partial contents of a message. Opening a message on the server begins a process where the entire message contents must be read and the message closed before the next command can be issued to the server. If you only want to obtain the headers for a message, use the PopGetMessageHeaders function instead.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopCloseMessage](#popclosemessage-function)
- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopIsReadable](#popisreadable-function)
- [PopRead](#popread-function)

---

## PopRead Function

Read data returned by the server

### Prototype

```c
INT WINAPI PopRead(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer in which the data will be copied. |
| `cbBuffer` | `INT` | The maximum number of bytes to read and copy into the specified buffer. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually read. A return value of zero indicates that there is no more data available to be read. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- When PopRead is called and the client is in non-blocking mode, it is possible that the function will fail because there is no available data to read at that time. This should not be considered a fatal error. Instead, the application should simply wait to receive the next asynchronous notification that data is available.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopIsBlocking](#popisblocking-function)
- [PopIsReadable](#popisreadable-function)
- [PopOpenMessage](#popopenmessage-function)

---

## PopRegisterEvent Function

Register an event handler for the specified event

### Prototype

```c
INT WINAPI PopRegisterEvent(
    HCLIENT hClient,
    UINT nEvent,
    POPEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One of the following values may be used: |
| `lpEventProc` | `POPEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the PopEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEvent** Parameter

    - `POP_EVENT_CONNECT`: The connection to the server has completed.
    - `POP_EVENT_DISCONNECT`: The server has closed the connection to the client. The client should read any remaining data and disconnect.
    - `POP_EVENT_READ`: Data is available to read by the calling process. No additional messages will be posted until the client has read at least some of the data. This event is only generated if the client is in asynchronous mode.
    - `POP_EVENT_WRITE`: The client can now write data. This notification is sent after a connection has been established, or after a previous attempt to write data has failed because it would result in a blocking operation. This event is only generated if the client is in asynchronous mode.
    - `POP_EVENT_TIMEOUT`: The network operation has exceeded the specified timeout period. The client application may attempt to retry the operation, or may disconnect from the server and report an error to the user.
    - `POP_EVENT_CANCEL`: The current operation has been canceled. Under most circumstances the client should disconnect from the server and re-connect if needed. After an operation has been canceled, the server may abort the connection or refuse to accept further commands from the client.
    - `POP_EVENT_COMMAND`: A command has been issued by the client and the server response has been received and processed. This event can be used to log the result codes and messages returned by the server in response to actions taken by the client.
    - `POP_EVENT_PROGRESS`: The client is in the process of sending or receiving data from the server. This event is called periodically during a transfer so that the client can update any user interface components such as a status control or progress bar.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopRegisterEvent function associates a callback function with a specific event. The event handler is an PopEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- This function is typically used to register an event handler that is invoked while a message is being retrieved. The POP_EVENT_PROGRESS event will only be generated periodically during the transfer to ensure the application is not flooded with event notifications. It is guaranteed that at least one POP_EVENT_PROGRESS notification will occur at the beginning of the transfer, and one at the end of the transfer when it has completed.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDisableEvents](#popdisableevents-function)
- [PopEnableEvents](#popenableevents-function)
- [PopEventProc](#popeventproc-function)
- [PopFreezeEvents](#popfreezeevents-function)

---

## PopReset Function

Reset the client and return to a command state

### Prototype

```c
BOOL WINAPI PopReset(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call PopGetLastError.

### Remarks

- This function will prevent any messages marked for deletion from actually being deleted from the mailbox. The client cannot be reset while the client is in a blocked state.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopCancel](#popcancel-function)
- [PopDeleteMessage](#popdeletemessage-function)
- [PopIsBlocking](#popisblocking-function)

---

## PopSendMessage Function

Send a message through the mail server

### Prototype

```c
INT WINAPI PopSendMessage(
    HCLIENT hClient,
    LPVOID lpMessage,
    DWORD dwMessageSize,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpMessage` | `LPVOID` | Pointer to a buffer which contains the message to be submitted to the mail server for delivery. |
| `dwMessageSize` | `DWORD` | The length of the buffer in bytes. This specifies the number of bytes to be written to the mail server. This value must be greater than zero. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopSendMessage function sends a message through the POP3 server using the XTND XMIT command. The specified file must be in the standard format as described in RFC 822, with the recipient addresses specified in the To: and Cc: header fields. Some servers may support blind carbon copies by using addresses specified in a Bcc: header field, and then removing those addresses from the header before delivering the message.
- Note that not all POP3 servers support this command, and it is recommended that you use the Simple Mail Transfer Protocol (SMTP) for general mail delivery purposes.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetMessage](#popgetmessage-function)
- [PopStoreMessage](#popstoremessage-function)

---

## PopSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI PopSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or POP_ERROR. Those functions which call PopSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the PopGetLastError function. The use of PopGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopGetErrorString](#popgeterrorstring-function)
- [PopGetLastError](#popgetlasterror-function)

---

## PopSetMultiLine Function

Set the client multi-line output flag

### Prototype

```c
INT WINAPI PopSetMultiLine(
    HCLIENT hClient,
    BOOL bMultiLine
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bMultiLine` | `BOOL` | A boolean flag which determines if the client is processing multiple lines of data as the result of a command. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The multi-line flag is used by the library to determine if multiple lines of data will be returned by the server as the result of a command. Unlike a single line response, which consists of a result code and result string, a multi-line response consists of one or more lines of text, terminated by a special end-of-data marker.
- The PopSetMultiLine function should only be used in conjunction with the PopCommand function. If a command is issued which would result in multiple lines of output, the multi-line flag must be set TRUE. The multi-line flag must be set after each command, since it is reset to FALSE with each command that is sent to the server.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopCommand](#popcommand-function)
- [PopGetMultiLine](#popgetmultiline-function)
- [PopGetResultCode](#popgetresultcode-function)
- [PopGetResultString](#popgetresultstring-function)

---

## PopSetTimeout Function

Set the number of seconds until an operation times out

### Prototype

```c
INT WINAPI PopSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds to wait for a blocking operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The timeout value is only used with blocking client connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopConnect](#popconnect-function)
- [PopGetTimeout](#popgettimeout-function)
- [PopRead](#popread-function)

---

## PopStoreMessage Function

Store the contents of a message in the specified file

### Prototype

```c
INT WINAPI PopStoreMessage(
    HCLIENT hClient,
    UINT nMessageId,
    LPCTSTR lpszFileName
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMessageId` | `UINT` | Number of the message to retrieve. This value must be greater than zero. The first message in the mailbox is message number one. |
| `lpszFileName` | `LPCTSTR` | Pointer to a string which specifies the file that the message will be stored in. If an empty string or NULL pointer is passed as an argument, the message is copied to the system clipboard. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The PopStoreMessage function provides a method of retrieving and storing a message on the local system. The contents of the message is stored as a text file, using the specified file name. This function always causes the caller to block until the entire message has been retrieved, even if the client has been put in asynchronous mode.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [PopGetMessage](#popgetmessage-function)
- [PopGetMessageHeaders](#popgetmessageheaders-function)
- [PopGetTransferStatus](#popgettransferstatus-function)
- [PopSendMessage](#popsendmessage-function)

---

## PopUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI PopUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful PopInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call PopUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to PopUninitialize for every successful call to PopInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopDisconnect](#popdisconnect-function)
- [PopInitialize](#popinitialize-function)

---

## PopWrite Function

Write data to the server

### Prototype

```c
INT WINAPI PopWrite(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | The pointer to the buffer which contains the data that is to be sent to the server. |
| `cbBuffer` | `INT` | The number of bytes to send from the specified buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually written. If the function fails, the return value is POP_ERROR. To get extended error information, call PopGetLastError.

### Remarks

- The return value may be less than the number of bytes specified by the cbBuffer parameter. In this case, the data has been partially written and it is the responsibility of the client application to send the remaining data at some later point. For non-blocking clients, the client must wait for the POP_EVENT_WRITE asynchronous notification message before it resumes sending data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cspopv11.lib`

### See Also

- [PopIsBlocking](#popisblocking-function)
- [PopIsWritable](#popiswritable-function)
- [PopRead](#popread-function)
- [PopSendMessage](#popsendmessage-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## POPTRANSFERSTATUS Structure

This structure is used by the PopGetTransferStatus function to return information about a message transfer in progress.

### Definition

```c
typedef struct _POPTRANSFERSTATUS
{
    UINT nMessageId;
    DWORD dwBytesTotal;
    DWORD dwBytesCopied;
    DWORD dwBytesPerSecond;
    DWORD dwTimeElapsed;
    DWORD dwTimeEstimated;
} POPTRANSFERSTATUS, *LPPOPTRANSFERSTATUS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nMessageId` | `UINT` | The message ID of the current message that is being transferred. |
| `dwBytesTotal` | `DWORD` | The total number of bytes that will be transferred. If the message is being copied from the server to the local host, this is the size of the message on the server. If the message is being posted to the server, it is the size of message on the local system. If the message size cannot be determined, this value will be zero. |
| `dwBytesCopied` | `DWORD` | The total number of bytes that have been copied. |
| `dwBytesPerSecond` | `DWORD` | The average number of bytes that have been copied per second. |
| `dwTimeElapsed` | `DWORD` | The number of seconds that have elapsed since the transfer started. |
| `dwTimeEstimated` | `DWORD` | The estimated number of seconds until the transfer is completed. This is based on the average number of bytes transferred per second. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## SECURITYCREDENTIALS Structure

The SECURITYCREDENTIALS structure specifies the information needed by the library to specify additional security credentials, such as a client certificate, when establishing a secure connection.

### Definition

```c
typedef struct _SECURITYCREDENTIALS
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwOptions;
    DWORD dwReserved;
    LPCTSTR lpszHostName;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
    LPCTSTR lpszCertStore;
    LPCTSTR lpszCertName;
    LPCTSTR lpszKeyFile;
} SECURITYCREDENTIALS, *LPSECURITYCREDENTIALS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. If the structure is being allocated dynamically, this member must be set to the size of the structure and all other unused structure members must be initialized to a value of zero or NULL. |
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. The value of this structure member is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or options. This member is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | This structure member is reserved for future use and should always be initialized to zero. |
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the hostname that will be used when validating the server certificate. If this member is NULL, then the server certificate will be validated against the hostname used to establish the connection. |
| `lpszUserName` | `LPCTSTR` | A pointer to a null terminated string which identifies the owner of client certificate. Currently this member is not used by the library and should always be initialized as a NULL pointer. |
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string which specifies the password needed to access the certificate. Currently this member is only used if the CREDENTIAL_STORE_FILENAME option has been specified. If there is no password associated with the certificate, then this member should be initialized as a NULL pointer. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The library will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the library will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the library will return an error indicating that the certificate could not be found. If the SECURITY_PROTOCOL_SSH protocol has been specified, this member should be NULL. |
| `lpszKeyFile` | `LPCTSTR` | This member is not used and should always be NULL. The private key cannot be specified separately and must be associated with the certificate. |

### Remarks

- An application only needs to create this structure if the server requires the client to provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions member should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore member should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12. These certificate files typically have an extension of .pfx or .p12. Note that if a password was specified when the certificate file was created, it must be provided in the lpszPassword member or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword members are values which are used to access the certificate store or private key file. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYINFO Structure

The SECURITYINFO structure contains security related information about the connection, including the protocol, cipher suite and certificate.

### Definition

```c
typedef struct _SECURITYINFO
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwCipher;
    DWORD dwCipherStrength;
    DWORD dwHash;
    DWORD dwHashStrength;
    DWORD dwKeyExchange;
    DWORD dwCertStatus;
    SYSTEMTIME stCertIssued;
    SYSTEMTIME stCertExpires;
    LPCTSTR lpszCertIssuer;
    LPCTSTR lpszCertSubject;
    LPCTSTR lpszFingerprint;
} SECURITYINFO, *LPSECURITYINFO;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Specifies the size of the data structure. This member must always be initialized to the size of the structure in bytes prior to passing the address of this structure to the function. If this this member is not initialized, an error will be returned indicating that an invalid parameter has been passed to the function. |
| `dwProtocol` | `DWORD` | A numeric value which specifies the security protocol selected when establishing the connection. |
| `dwCipher` | `DWORD` | A numeric value which specifies the cipher that was selected when establishing the secure connection. |
| `dwCipherStrength` | `DWORD` | A numeric value which specifies the strength (the length of the cipher key in bits) of the cipher that was selected. Typically this value will be 128 or 256. 40-bit and 56-bit key lengths are considered weak encryption, and subject to brute force attacks. 128-bit and 256-bit key lengths are considered to be secure, and are the recommended key length for secure communications. |
| `dwHash` | `DWORD` | A numeric value which specifies the hash algorithm which was selected. |
| `dwHashStrength` | `DWORD` | A numeric value which specifies the strength (the length in bits) of the message digest that was selected. |
| `dwKeyExchange` | `DWORD` | A numeric value which specifies the key exchange algorithm which was selected. |
| `dwCertStatus` | `DWORD` | A numeric value which specifies the status of the certificate returned by the secure server. |
| `stCertIssued` | `SYSTEMTIME` | A structure which contains the date and time that the certificate was issued by the certificate authority. If the issue date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `stCertExpires` | `SYSTEMTIME` | A structure which contains the date and time that the certificate expires. If the expiration date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `lpszCertIssuer` | `LPCTSTR` | A pointer to a string which contains information about the organization that issued the certificate. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate issuer could not be determined, this member may be NULL. |
| `lpszCertSubject` | `LPCTSTR` | A pointer to a string which contains information about the organization that the certificate was issued to. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate subject could not be determined, this member may be NULL. |
| `lpszFingerprint` | `LPCTSTR` | A pointer to a string which contains a sequence of hexadecimal values that uniquely identify the server. This member may be NULL. |

### Remarks

- The size of this structure is typically determined using the sizeof operator in C/C++ and most other languages have a similar method for obtaining the size of a data structure or user-defined type. It is important to note the size of this structure will be different on 32-bit and 64-bit platforms due to the difference in the length of the structure members which are pointers.
- When evaluating structure members which are string pointers, such as lpszCertIssuer and lpszCertSubject, you should always check to ensure their values are not NULL (zero) before attempting to dereference them. If those values are undefined, it is not guaranteed they will point to zero-length strings.
- There are additional protocols and ciphers defined which are not listed above because they are considered insecure and are no longer supported. It is important to note that most servers will reject attempts to use unsupported protocols such as SSL 3.0 and will either abort the connection or issue an error message and then immediately disconnect.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

