# Remote Command Protocol

## Functions

- [RshAsyncExecute](#rshasyncexecute-function)
- [RshAsyncLogin](#rshasynclogin-function)
- [RshAttachThread](#rshattachthread-function)
- [RshCancel](#rshcancel-function)
- [RshDisableEvents](#rshdisableevents-function)
- [RshDisableTrace](#rshdisabletrace-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshEnableEvents](#rshenableevents-function)
- [RshEnableTrace](#rshenabletrace-function)
- [RshEventProc](#rsheventproc-function)
- [RshExecute](#rshexecute-function)
- [RshFreezeEvents](#rshfreezeevents-function)
- [RshGetErrorString](#rshgeterrorstring-function)
- [RshGetLastError](#rshgetlasterror-function)
- [RshGetStatus](#rshgetstatus-function)
- [RshGetTimeout](#rshgettimeout-function)
- [RshInitialize](#rshinitialize-function)
- [RshIsBlocking](#rshisblocking-function)
- [RshIsConnected](#rshisconnected-function)
- [RshIsReadable](#rshisreadable-function)
- [RshIsWritable](#rshiswritable-function)
- [RshLogin](#rshlogin-function)
- [RshRead](#rshread-function)
- [RshRegisterEvent](#rshregisterevent-function)
- [RshSearch](#rshsearch-function)
- [RshSetLastError](#rshsetlasterror-function)
- [RshSetTimeout](#rshsettimeout-function)
- [RshUninitialize](#rshuninitialize-function)
- [RshWrite](#rshwrite-function)
## Structures

- [INITDATA](#initdata-structure)

---

## RshAsyncExecute Function

Execute a command on the specified server

### Prototype

```c
HCLIENT WINAPI RshAsyncExecute(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    LPCTSTR lpszCommand,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. One of the following values should be used: |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the username used to authenticate the client session. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password used to authenticate the client session. This parameter is only used when connecting to the RSH_PORT_REXEC port. If the password is not required, this parameter may be NULL. |
| `lpszCommand` | `LPCTSTR` | A pointer to a string which specifies the command to execute on the server. |
| `hEventWnd` | `HWND` | The handle to the event notification window. This window receives messages which notify the client of various asynchronous network events that occur. If this parameter is NULL, then a synchronous (blocking) connection will be established with the server. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous network event occurs. This value should be greater than WM_USER as defined in the Windows header files. If the hEventWnd parameter is NULL, this parameter should be specified as WM_NULL. |

### Options

- **nRemotePort** Parameter

    - `RSH_PORT_REXEC`: A connection is established with the server using port 512, the rexec service. This service requires that the client provide a username and password to execute the specified command.
    - `RSH_PORT_RSHELL`: A connection is established with the server using port 514, the rshell service. This service uses host equivalence to authenticate the user. With host equivalence, the server considers the client to be equivalent to itself, and as long as the specified user exists on the server, the client is permitted to execute commands on behalf of the user without requiring a password. Host equivalence is configured by the server administrator.
- **dwOptions** Parameter

    - `RSH_OPTION_RESERVEDPORT`: This option specifies that a reserved port should be used to establish the connection. Reserved ports are those port numbers which are less than 1024. This option should be specified when connecting on the RSH_PORT_RSHELL port.
    - `RSH_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `RSH_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call RshGetLastError.

### Remarks

- This protocol should not be used when connecting to a server on a public network because the user credentials are sent as unencrypted text. For secure remote command execution and interactive terminal sessions, it is recommended you use the Secure Shell (SSH) library.
- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshAsyncLogin](#rshasynclogin-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshExecute](#rshexecute-function)
- [RshInitialize](#rshinitialize-function)
- [RshLogin](#rshlogin-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshAsyncLogin Function

Establish an asynchronous login session with the specified server

### Prototype

```c
HCLIENT WINAPI RshAsyncLogin(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwReserved,
    LPCTSTR lpszUserName,
    LPCTSTR lpszTerminal,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port should be used. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the username used to authenticate the client session. |
| `lpszTerminal` | `LPCTSTR` | A pointer to a string which specifies the terminal type which the client will be identified as using during the session. If no particular terminal emulation is required, this parameter may be NULL. |
| `hEventWnd` | `HWND` | The handle to the event notification window. This window receives messages which notify the client of various asynchronous network events that occur. If this parameter is NULL, then a synchronous (blocking) connection will be established with the server. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous network event occurs. This value should be greater than WM_USER as defined in the Windows header files. If the hEventWnd parameter is NULL, this parameter should be specified as WM_NULL. |

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call RshGetLastError.

### Remarks

- The RshAsyncLogin function uses host equivalence, where the client is permitted to login without requiring a password. Host equivalence must be configured by the server administrator and it is typically restricted to specific users. Note that if configured improperly, host equivalence can introduce a significant security loophole. Refer to your UNIX system documentation for more information about host equivalence and the various remote command services.
- This protocol should not be used when connecting to a server on a public network because the user credentials are sent as unencrypted text. For secure remote command execution and interactive terminal sessions, it is recommended you use the Secure Shell (SSH) library.
- On UNIX based systems, the terminal name specified by the lpszTerminal parameter corresponds to a termcap or terminfo entry as set in the TERM environment variable. On Windows based systems which implement the rlogin service, this parameter may be ignored and the server will assume that the client is capable of displaying ANSI escape sequences. On VMS systems, the terminal name should correspond to the terminal type used with the SET TERMINAL/DEVICE command.
- If this parameter is passed as NULL pointer or an empty string, a default terminal type named "unknown" will be used. On most UNIX and VMS systems this defines a terminal which is not capable of cursor positioning using control or escape sequences. This terminal type may not be recognized and an error may be displayed when the user logs in indicating that the terminal type is invalid.
- Refer to the documentation for the server system to determine what terminal type names are available to you. Remember that on UNIX systems, the terminal type is case-sensitive. Some of the more common terminal types are:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshAsyncExecute](#rshasyncexecute-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshExecute](#rshexecute-function)
- [RshInitialize](#rshinitialize-function)
- [RshLogin](#rshlogin-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI RshAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The RshAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should RshAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the RshCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the RshUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshCancel](#rshcancel-function)
- [RshAsyncExecute](#rshasyncexecute-function)
- [RshAsyncLogin](#rshasynclogin-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshExecute](#rshexecute-function)
- [RshInitialize](#rshinitialize-function)
- [RshLogin](#rshlogin-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI RshCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero.

### Remarks

- When the RshCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshIsBlocking](#rshisblocking-function)

---

## RshDisableEvents Function

Disable asynchronous event notification

### Prototype

```c
INT WINAPI RshDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- This function affects both event notification and event callbacks. Any outstanding events in the message queue should be ignored by the client application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshEnableEvents](#rshenableevents-function)
- [RshFreezeEvents](#rshfreezeevents-function)
- [RshRegisterEvent](#rshregisterevent-function)

---

## RshDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI RshDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshEnableTrace](#rshenabletrace-function)

---

## RshDisconnect Function

Disconnect from the current server

### Prototype

```c
INT WINAPI RshDisconnect(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshAsyncExecute](#rshasyncexecute-function)
- [RshAsyncLogin](#rshasynclogin-function)
- [RshExecute](#rshexecute-function)
- [RshInitialize](#rshinitialize-function)
- [RshLogin](#rshlogin-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshEnableEvents Function

Enable asynchronous event notification

### Prototype

```c
INT WINAPI RshEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the event notification window. This window receives a user-defined message which specifies the event that has occurred. If this value is NULL, event notification is disabled. |
| `uEventMsg` | `UINT` | An unsigned integer which specifies the user-defined message that is sent when an event occurs. This parameter's value must be greater than the value of WM_USER. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshDisableEvents](#rshdisableevents-function)
- [RshFreezeEvents](#rshfreezeevents-function)
- [RshRegisterEvent](#rshregisterevent-function)

---

## RshEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI RshEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshDisableTrace](#rshdisabletrace-function)

---

## RshEventProc Function

Callback function that processes events generated by the client

### Prototype

```c
VOID __cdecl RshEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | The handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the RshRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the RshRegisterEvent function. The RshEventProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshDisableEvents](#rshdisableevents-function)
- [RshEnableEvents](#rshenableevents-function)
- [RshFreezeEvents](#rshfreezeevents-function)
- [RshRegisterEvent](#rshregisterevent-function)

---

## RshExecute Function

Execute a command on the server

### Prototype

```c
HCLIENT WINAPI RshExecute(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    LPCTSTR lpszCommand
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. One of the following values should be used: |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the username used to authenticate the client session. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password used to authenticate the client session. This parameter is only used when connecting to the RSH_PORT_REXEC port. If the password is not required, this parameter may be NULL. |
| `lpszCommand` | `LPCTSTR` | A pointer to a string which specifies the command to execute on the server. |

### Options

- **nRemotePort** Parameter

    - `RSH_PORT_REXEC`: A connection is established with the server using port 512, the rexec service. This service requires that the client provide a username and password to execute the specified command.
    - `RSH_PORT_RSHELL`: A connection is established with the server using port 514, the rshell service. This service uses host equivalence to authenticate the user. With host equivalence, the server considers the client to be equivalent to itself, and as long as the specified user exists on the server, the client is permitted to execute commands on behalf of the user without requiring a password. Host equivalence is configured by the server administrator.
- **dwOptions** Parameter

    - `RSH_OPTION_RESERVEDPORT`: This option specifies that a reserved port should be used to establish the connection. Reserved ports are those port numbers which are less than 1024. This option should be specified when connecting on the RSH_PORT_RSHELL port.
    - `RSH_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `RSH_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call RshGetLastError.

### Remarks

- This protocol should not be used when connecting to a server on a public network because the user credentials are sent as unencrypted text. For secure remote command execution and interactive terminal sessions, it is recommended you use the Secure Shell (SSH) library.
- The dwOptions argument can be used to specify the threading model that is used by the library when a connection is established. By default, the handle is initially attached to the thread that created it. From that point on, until the it is released, only the owner may call functions using that handle. The ownership of the handle may be transferred from one thread to another using the RshAttachThread function.
- Specifying the RSH_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshAsyncExecute](#rshasyncexecute-function)
- [RshAsyncLogin](#rshasynclogin-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshInitialize](#rshinitialize-function)
- [RshLogin](#rshlogin-function)
- [RshSearch](#rshsearch-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshFreezeEvents Function

Suspend asynchronous event processing

### Prototype

```c
INT WINAPI RshFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four read events occur, once event handling is resumed only one of those read events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshDisableEvents](#rshdisableevents-function)
- [RshEnableEvents](#rshenableevents-function)
- [RshRegisterEvent](#rshregisterevent-function)

---

## RshGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI RshGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The last-error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshGetLastError](#rshgetlasterror-function)
- [RshSetLastError](#rshsetlasterror-function)

---

## RshGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI RshGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the RshSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the RshGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call RshSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or RSH_ERROR. Those functions which call RshSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshGetErrorString](#rshgeterrorstring-function)
- [RshSetLastError](#rshsetlasterror-function)

---

## RshGetStatus Function

Return the current client status

### Prototype

```c
INT WINAPI RshGetStatus(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the client status code. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- The RshGetStatus function returns a numeric code which identifies the current state of the client session. The following values may be returned:
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshIsBlocking](#rshisblocking-function)
- [RshIsConnected](#rshisconnected-function)
- [RshIsReadable](#rshisreadable-function)

---

## RshGetTimeout Function

Return the number of seconds until an operation times out

### Prototype

```c
INT WINAPI RshGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshSetTimeout](#rshsettimeout-function)

---

## RshInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI RshInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call RshGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that RshInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to RshUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the RshInitialize or RshUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshAsyncExecute](#rshasyncexecute-function)
- [RshAsyncLogin](#rshasynclogin-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshExecute](#rshexecute-function)
- [RshLogin](#rshlogin-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI RshIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns TRUE. If the client is not performing a blocking operation, or the client handle is invalid, the function returns FALSE.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The RshIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action (such as warning the user that they must wait for the operation to complete).

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshCancel](#rshcancel-function)
- [RshGetStatus](#rshgetstatus-function)
- [RshIsConnected](#rshisconnected-function)
- [RshIsReadable](#rshisreadable-function)

---

## RshIsConnected Function

Determine if the client is connected to the server

### Prototype

```c
BOOL WINAPI RshIsConnected(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is connected to a server, the function returns a non-zero value. If the client is not connected, or the client handle is invalid, the function returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshGetStatus](#rshgetstatus-function)
- [RshIsBlocking](#rshisblocking-function)
- [RshIsReadable](#rshisreadable-function)
- [RshIsWritable](#rshiswritable-function)

---

## RshIsReadable Function

Determine if data can be read from the server

### Prototype

```c
BOOL WINAPI RshIsReadable(
    HCLIENT hClient,
    INT nTimeout,
    LPDWORD lpdwAvail
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |
| `lpdwAvail` | `LPDWORD` | A pointer to an unsigned integer which will contain the number of bytes available to read. This parameter may be NULL if this information is not required. |

### Return Value

- **Type**: `BOOL`
- If the client can read data from the server within the timeout period, the function returns a non-zero value. If the client cannot read any data, the function returns zero.

### Remarks

- On some platforms, this value will not exceed the size of the receive buffer (typically 64K bytes). Because of differences between TCP/IP stack implementations, it is not recommended that your application exclusively depend on this value to determine the exact number of bytes available. Instead, it should be used as a general indicator that there is data available to be read.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshGetStatus](#rshgetstatus-function)
- [RshIsBlocking](#rshisblocking-function)
- [RshIsConnected](#rshisconnected-function)
- [RshIsWritable](#rshiswritable-function)
- [RshRead](#rshread-function)

---

## RshIsWritable Function

Determine if data can be written to the server

### Prototype

```c
BOOL WINAPI RshIsWritable(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |

### Return Value

- **Type**: `BOOL`
- If the client can write data to the server within the specified timeout period, the function returns a non-zero value. If the client cannot write any data, the function returns zero.

### Remarks

- Although this function can be used to determine if some amount of data can be sent to the remote process it does not indicate the amount of data that can be written without blocking the client. In most cases, it is recommended that large amounts of data be broken into smaller logical blocks, typically some multiple of 512 bytes in length.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshGetStatus](#rshgetstatus-function)
- [RshIsBlocking](#rshisblocking-function)
- [RshIsConnected](#rshisconnected-function)
- [RshIsReadable](#rshisreadable-function)
- [RshWrite](#rshwrite-function)

---

## RshLogin Function

Establish a login session with the specified server

### Prototype

```c
HCLIENT WINAPI RshLogin(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwReserved,
    LPCTSTR lpszUserName,
    LPCTSTR lpszTerminal
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port should be used. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the username used to authenticate the client session. |
| `lpszTerminal` | `LPCTSTR` | A pointer to a string which specifies the terminal type which the client will be identified as using during the session. If no particular terminal emulation is required, this parameter may be NULL. |

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call RshGetLastError.

### Remarks

- The RshLogin function uses host equivalence, where the client is permitted to login without requiring a password. Host equivalence must be configured by the server administrator and it is typically restricted to specific users. Note that if configured improperly, host equivalence can introduce a significant security loophole. Refer to your UNIX system documentation for more information about host equivalence and the various remote command services. Note that if there is no host equivalence for the local host and the user account requires a password, the system will prompt for a password.
- This protocol should not be used when connecting to a server on a public network because the user credentials are sent as unencrypted text. For secure remote command execution and interactive terminal sessions, it is recommended you use the Secure Shell (SSH) library.
- On UNIX based systems, the terminal name specified by the lpszTerminal parameter corresponds to a termcap or terminfo entry as set in the TERM environment variable. On Windows based systems which implement the rlogin service, this parameter may be ignored and the server will assume that the client is capable of displaying ANSI escape sequences. On VMS systems, the terminal name should correspond to the terminal type used with the SET TERMINAL/DEVICE command.
- If this parameter is passed as NULL pointer or an empty string, a default terminal type named "unknown" will be used. On most UNIX and VMS systems this defines a terminal which is not capable of cursor positioning using control or escape sequences. This terminal type may not be recognized and an error may be displayed when the user logs in indicating that the terminal type is invalid.
- Refer to the documentation for the server system to determine what terminal type names are available to you. Remember that on UNIX systems, the terminal type is case-sensitive. Some of the more common terminal types are:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshAsyncExecute](#rshasyncexecute-function)
- [RshAsyncLogin](#rshasynclogin-function)
- [RshDisconnect](#rshdisconnect-function)
- [RshExecute](#rshexecute-function)
- [RshInitialize](#rshinitialize-function)
- [RshSearch](#rshsearch-function)
- [RshUninitialize](#rshuninitialize-function)

---

## RshRead Function

Read data returned by the server

### Prototype

```c
INT WINAPI RshRead(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer in which the data will be copied. |
| `cbBuffer` | `INT` | The maximum number of bytes to read and copy into the specified buffer. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually read. A return value of zero indicates that the server has closed the connection and there is no more data available to be read. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- When RshRead is called and the client is in non-blocking mode, it is possible that the function will fail because there is no available data to read at that time. This should not be considered a fatal error. Instead, the application should simply wait to receive the next asynchronous notification that data is available.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshIsBlocking](#rshisblocking-function)
- [RshIsReadable](#rshisreadable-function)
- [RshSearch](#rshsearch-function)
- [RshWrite](#rshwrite-function)

---

## RshRegisterEvent Function

Register an event callback function

### Prototype

```c
INT WINAPI RshRegisterEvent(
    HCLIENT hClient,
    UINT nEventId,
    INETEVENTPROC lpfnEvent,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to client session. |
| `nEventId` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One or more of the following values may be used: |
| `lpfnEvent` | `INETEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the RshEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEventId** Parameter

    - `RSH_EVENT_CONNECT`: The connection to the server has completed.
    - `RSH_EVENT_DISCONNECT`: The server has closed the connection to the client. The client should read any remaining data and disconnect.
    - `RSH_EVENT_READ`: Data is available to read by the calling process. No additional messages will be posted until the client has read at least some of the data. This event is only generated if the client is in asynchronous mode.
    - `RSH_EVENT_WRITE`: The client can now write data. This notification is sent after a connection has been established, or after a previous attempt to write data has failed because it would result in a blocking operation. This event is only generated if the client is in asynchronous mode.
    - `RSH_EVENT_TIMEOUT`: The network operation has exceeded the specified timeout period. The client application may attempt to retry the operation, or may disconnect from the server and report an error to the user.
    - `RSH_EVENT_CANCEL`: The current operation has been canceled. Under most circumstances the client should disconnect from the server and re-connect if needed. After an operation has been canceled, the server may abort the connection or refuse to accept further commands from the client.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- The RshRegisterEvent function associates a callback function with a specific event. The event handler is an RshEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshDisableEvents](#rshdisableevents-function)
- [RshEnableEvents](#rshenableevents-function)
- [RshEventProc](#rsheventproc-function)
- [RshFreezeEvents](#rshfreezeevents-function)

---

## RshSearch Function

Search for a specific character sequence in the data stream

### Prototype

```c
BOOL WINAPI RshSearch(
    HCLIENT hClient,
    LPCTSTR lpszString,
    LPVOID lpvBuffer,
    LPDWORD lpdwLength,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszString` | `LPCTSTR` | A pointer to a string which specifies the sequence of characters to search for in the data stream. This parameter cannot be NULL or point to an empty string. |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the output from the server, or a pointer to a global memory handle which will reference the output when the function returns. If the output from the server is not required, this parameter may be NULL. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual number of bytes of output stored in the buffer. If the lpvBuffer parameter is NULL, this parameter should also be NULL. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must be zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds and the character sequences was found in the data stream, the return value is non-zero. If the function fails or a timeout occurs before the sequence is found, the return value is zero. To get extended error information, call RshGetLastError.

### Remarks

- The RshSearch function searches for a character sequence in the data stream and stops reading when it is found. This is useful when the client wants to automate responses to the server, such as executing a command and processing the output. The function collects the output from the server and stores it in the buffer specified by the lpvBuffer parameter. When the function returns, the buffer will contain everything sent by the server up to and including the search string.
- The lpvBuffer parameter may be specified in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the a fixed amount of output. In this case, the lpvBuffer parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer. If the server sends more output than can be stored in the buffer, the remaining output will be discarded.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the output when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. This method is preferred if the client application does not have a general idea of how much output will be generated until the search string is found.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RshExecute](#rshexecute-function)
- [RshIsBlocking](#rshisblocking-function)
- [RshIsReadable](#rshisreadable-function)
- [RshLogin](#rshlogin-function)
- [RshRead](#rshread-function)

---

## RshSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI RshSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or RSH_ERROR. Those functions which call RshSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the RshGetLastError function. The use of RshGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshGetErrorString](#rshgeterrorstring-function)
- [RshGetLastError](#rshgetlasterror-function)

---

## RshSetTimeout Function

Set the number of seconds until an operation times out

### Prototype

```c
INT WINAPI RshSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds to wait for a blocking operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- The timeout value is only used with blocking client connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshGetTimeout](#rshgettimeout-function)

---

## RshUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI RshUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful RshInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call RshUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to RshUninitialize for every successful call to RshInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshDisconnect](#rshdisconnect-function)
- [RshInitialize](#rshinitialize-function)

---

## RshWrite Function

Write data to the server

### Prototype

```c
INT WINAPI RshWrite(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | The pointer to the buffer which contains the data that is to be sent to the server. |
| `cbBuffer` | `INT` | The number of bytes to send from the specified buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually written. If the function fails, the return value is RSH_ERROR. To get extended error information, call RshGetLastError.

### Remarks

- The return value may be less than the number of bytes specified by the cbBuffer parameter. In this case, the data has been partially written and it is the responsibility of the client application to send the remaining data at some later point. For non-blocking clients, the client must wait for the RSH_EVENT_WRITE asynchronous notification message before it resumes sending data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrshv11.lib`

### See Also

- [RshIsBlocking](#rshisblocking-function)
- [RshIsWritable](#rshiswritable-function)
- [RshRead](#rshread-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

