# Really Simple Syndication

## Functions

- [RssCloseFeed](#rssclosefeed-function)
- [RssDisableTrace](#rssdisabletrace-function)
- [RssEnableTrace](#rssenabletrace-function)
- [RssFindItem](#rssfinditem-function)
- [RssGetErrorString](#rssgeterrorstring-function)
- [RssGetFirstItem](#rssgetfirstitem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemCount](#rssgetitemcount-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RssGetLastError](#rssgetlasterror-function)
- [RssGetNextItem](#rssgetnextitem-function)
- [RssInitialize](#rssinitialize-function)
- [RssOpenFeed](#rssopenfeed-function)
- [RssParseFeed](#rssparsefeed-function)
- [RssRefreshFeed](#rssrefreshfeed-function)
- [RssSetLastError](#rsssetlasterror-function)
- [RssStoreFeed](#rssstorefeed-function)
- [RssUninitialize](#rssuninitialize-function)
- [RssValidateFeed](#rssvalidatefeed-function)
## Structures

- [INITDATA](#initdata-structure)
- [RSSCHANNEL](#rsschannel-structure)
- [RSSCHANNELITEM](#rsschannelitem-structure)

---

## RssCloseFeed Function

Close the specified news feed and release memory allocated for the channel

### Prototype

```c
INT WINAPI RssCloseFeed(
    HCHANNEL hChannel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- The RssCloseFeed function must be called whenever the application has completed processing the news feed. It is important to note that the memory allocated for the channel will be released when this function is called, which means that any data referenced in the RSSCHANNEL and RSSCHANNELITEM structures will no longer be valid and must not be used by the application after the feed has been closed.
- This function can fail if the feed is currently being updated, such as when the RssRefreshFeed function is called. In this case, the channel handle will not be released and the application must attempt to close the feed at a later time.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`

### See Also

- [RssOpenFeed](#rssopenfeed-function)
- [RssParseFeed](#rssparsefeed-function)
- [RssStoreFeed](#rssstorefeed-function)

---

## RssDisableTrace Function

Disable logging of network function calls

### Prototype

```c
BOOL WINAPI RssDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`

### See Also

- [RssEnableTrace](#rssenabletrace-function)

---

## RssEnableTrace Function

Enable logging of network function calls to a text file

### Prototype

```c
BOOL WINAPI RssEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssDisableTrace](#rssdisabletrace-function)

---

## RssFindItem Function

Find a specific news feed item using its unique identifier (GUID) property

### Prototype

```c
INT WINAPI RssFindItem(
    HCHANNEL hChannel,
    LPCTSTR lpszValue,
    DWORD dwOptions,
    LPRSSCHANNELITEM lpItem
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `lpszValue` | `LPCTSTR` | A pointer to a string which specifies the value of the item being searched for. This value should uniquely identify the item in the feed, and this parameter cannot be an empty string or NULL pointer. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpItem` | `LPRSSCHANNELITEM` | A pointer to a RSSCHANNELITEM structure which contains information about the specified item in the news feed. This structure is initialized by the function and the parameter can never be specified as a NULL pointer. |

### Options

- **dwOptions** Parameter

    - `RSS_FIND_GUID`: Search the feed for items with a matching GUID property value. This is the default option, and is the only item property that is guaranteed to be unique in the feed. The search is case-sensitive, requiring that the lpszValue parameter match the property value exactly.
    - `RSS_FIND_LINK`: Search the feed for items with a matching link property value. For feeds that do not specify a GUID property, this is the recommended option for searching for an item. The search is not case-sensitive.
    - `RSS_FIND_TITLE`: Search the feed for items with a matching title. This option should not be used if you must ensure that the item returned is unique in the feed because there may be multiple items with the same title in the feed. The search is not case-sensitive.
    - `RSS_FIND_PUBDATE`: Search the feed for items with a matching publishing date. This option should not be used if you must ensure that the item returned is unique in the feed because more than one item may have the same publishing date. The format of the date string must match the standard format used with the RSS protocol and the match is not case-sensitive.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- It is recommended that you use the RSS_FIND_GUID option with news feeds that are using version 2.0 or later of the RSS specification. If the feed uses an earlier version, items may not include a GUID property. It is also possible that a feed may omit the GUID property even though it is considered a requirement for the current RSS specification. For the broadest compatibility with all news feeds, an application should not depend on being able to search for a specific news feed item by its GUID.
- Only the GUID property is guaranteed to be unique in the feed. If the feed does not specify GUIDs for the news items, the application must use an alternate criteria such as the item hyperlink or publishing date. If there are multiple items that match the lpszValue value, the first matching item will be returned.
- The data referenced in the RSSCHANNELITEM structure should be considered read-only and never modified by the application. Not all members of the structure may contain valid values, in which case those members will either have a value of zero or will specify NULL pointers. When the feed is closed, the members of this structure will no longer be valid, and therefore should never be stored by the application. If the application needs to store or modify this information, it should create its own private copy of the data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssGetFirstItem](#rssgetfirstitem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RssGetNextItem](#rssgetnextitem-function)
- [RSSCHANNELITEM](#rsschannelitem-function)

---

## RssGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI RssGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The last-error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssGetLastError](#rssgetlasterror-function)
- [RssSetLastError](#rsssetlasterror-function)

---

## RssGetFirstItem Function

Return information about the first item in the news feed channel

### Prototype

```c
BOOL WINAPI RssGetFirstItem(
    HCHANNEL hChannel,
    LPRSSCHANNELITEM lpItem
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `lpItem` | `LPRSSCHANNELITEM` | A pointer to a RSSCHANNELITEM structure which contains information about the first item in the news feed. This structure is initialized by the function and the parameter can never be specified as a NULL pointer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, it returns a non-zero value. If the function fails, the return value is zero. To get extended error information, call RssGetLastError.

### Remarks

- The RssGetFirstItem function is used in conjunction with the RssGetNextItem function to enumerate the available items in the specified news feed channel. If this function fails, it typically indicates that the channel does not contain any valid news items or that the format of the news feed is invalid.
- The data referenced in the RSSCHANNELITEM structure should be considered read-only and never modified by the application. Not all members of the structure may contain valid values, in which case those members will either have a value of zero or will specify NULL pointers. When the feed is closed, the members of this structure will no longer be valid, and therefore should never be stored by the application. If the application needs to store or modify this information, it should create its own private copy of the data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssFindItem](#rssfinditem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RssGetNextItem](#rssgetnextitem-function)
- [RSSCHANNELITEM](#rsschannelitem-function)

---

## RssGetItem Function

Return information about the specified news feed item

### Prototype

```c
BOOL WINAPI RssGetItem(
    HCHANNEL hChannel,
    UINT nItemId,
    LPRSSCHANNELITEM lpItem
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `nItemId` | `UINT` | An integer value which identifies the news feed item. The first item identifier in the news feed has a value of one, and that value is incremented for each additional item in the feed. If this parameter is zero or specifies a value larger than the number of items in the feed, this function will fail. |
| `lpItem` | `LPRSSCHANNELITEM` | A pointer to a RSSCHANNELITEM structure which contains information about the specified item in the news feed. This structure is initialized by the function and the parameter can never be specified as a NULL pointer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, it returns a value of zero. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- The RssGetItem function is used to return information about a specific item in the news feed. If this function fails, it typically indicates that the item ID is invalid or that the feed does not contain any valid news items. The RssGetItemCount function can be used to determine the number of items contained in the feed channel.
- The data referenced in the RSSCHANNELITEM structure should be considered read-only and never modified by the application. Not all members of the structure may contain valid values, in which case those members will either have a value of zero or will specify NULL pointers. When the feed is closed, the members of this structure will no longer be valid, and therefore should never be stored by the application. If the application needs to store or modify this information, it should create its own private copy of the data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssFindItem](#rssfinditem-function)
- [RssGetFirstItem](#rssgetfirstitem-function)
- [RssGetItemCount](#rssgetitemcount-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RssGetNextItem](#rssgetnextitem-function)
- [RSSCHANNELITEM](#rsschannelitem-function)

---

## RssGetItemCount Function

Return the number of news feed items in the channel

### Prototype

```c
INT WINAPI RssGetItemCount(
    HCHANNEL hChannel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of items in the news feed. A value of zero indicates that the feed channel is empty. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- The RssGetItemCount function is used to determine the number of items that are contained in the news feed channel, and therefore determine the maximum value of the item identifier which can be used to reference a specific item in the feed. This value is the same as the value specified by the nItemCount member of the RSSCHANNEL structure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`

### See Also

- [RssFindItem](#rssfinditem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RSSCHANNEL](#rsschannel-function)

---

## RssGetItemProperty Function

Return the value of the specified news feed item property or attribute

### Prototype

```c
INT WINAPI RssGetItemProperty(
    HCHANNEL hChannel,
    UINT nItemId,
    DWORD dwReserved,
    LPCTSTR lpszProperty,
    LPCTSTR lpszAttribute,
    LPTSTR lpszValue,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `nItemId` | `UINT` | An integer value which identifies the news feed item. The first item identifier in the news feed has a value of one, and that value is incremented for each additional item in the feed. If this parameter is zero or specifies a value larger than the number of items in the feed, this function will fail. |
| `dwReserved` | `DWORD` | A parameter reserved for future use. This value should always be zero. |
| `lpszProperty` | `LPCTSTR` | A pointer to a string which specifies the name of the item property. This parameter cannot point to an empty string or specify a NULL pointer. |
| `lpszAttribute` | `LPCTSTR` | A pointer to a string which specifies the name of an attribute for the property. If this parameter is an empty string or NULL pointer, the function will return the value of the property, rather than an attribute of the specified property. |
| `lpszValue` | `LPTSTR` | A pointer to a string buffer which will contain the value of the specified item property or attribute. This string should be large enough to contain the property value. If this parameter is a NULL pointer, it will be ignored and the function will only return the length of value for the specified property. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the property value buffer. If the value of this parameter is zero, then the lpszValue parameter is ignored and the function will only return the length of the value for the specified property. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the property value string. A return value of zero indicates that the property does not contain any value. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- The RssGetItemProperty function is primarily used with custom item properties that may be used with extensions to the news feed. The standard properties for an item such as the title, link and description can be obtained using RssGetItem and related functions. However, if items in the feed contain custom properties that are not part of the standard RSS format, this function can be used to obtain those values.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssFindItem](#rssfinditem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RSSCHANNELITEM](#rsschannelitem-function)

---

## RssGetItemText Function

Return the text description of the specified news feed item

### Prototype

```c
INT WINAPI RssGetItemText(
    HCHANNEL hChannel,
    UINT nItemId,
    LPTSTR lpszBuffer,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `nItemId` | `UINT` | An integer value which identifies the news feed item. The first item identifier in the news feed has a value of one, and that value is incremented for each additional item in the feed. If this parameter is zero or specifies a value larger than the number of items in the feed, this function will fail. |
| `lpszBuffer` | `LPTSTR` | A pointer to a string buffer which will contain the value of the item description. If this parameter is a NULL pointer, it will be ignored and the function will only return the length of the item description. |
| `nMaxLength` | `INT` | The maximum number of characters that may be copied into the description buffer. If the value of this parameter is zero, then the lpszValue parameter is ignored and the function will only return the length of the item description. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the item description. A return value of zero indicates that the item does not have a description. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssFindItem](#rssfinditem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RSSCHANNELITEM](#rsschannelitem-function)

---

## RssGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI RssGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the RssSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the RssGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call RssSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CHANNEL or RSS_ERROR. Those functions which call RssSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`

### See Also

- [RssGetErrorString](#rssgeterrorstring-function)
- [RssSetLastError](#rsssetlasterror-function)

---

## RssGetNextItem Function

Return information about the next item in the news feed channel

### Prototype

```c
BOOL WINAPI RssGetNextItem(
    HCHANNEL hChannel,
    LPRSSCHANNELITEM lpItem
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `lpItem` | `LPRSSCHANNELITEM` | A pointer to a RSSCHANNELITEM structure which contains information about the next item in the news feed. This structure is initialized by the function and the parameter can never be specified as a NULL pointer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, it returns a non-zero value. If the function fails, the return value is zero. To get extended error information, call RssGetLastError.

### Remarks

- The RssGetNextItem function is used in conjunction with the RssGetFirstItem function to enumerate the available items in the specified news feed channel. If this function fails, it typically indicates that there are no more items in the news feed channel.
- The data referenced in the RSSCHANNELITEM structure should be considered read-only and never modified by the application. Not all members of the structure may contain valid values, in which case those members will either have a value of zero or will specify NULL pointers. When the feed is closed, the members of this structure will no longer be valid, and therefore should never be stored by the application. If the application needs to store or modify this information, it should create its own private copy of the data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssFindItem](#rssfinditem-function)
- [RssGetFirstItem](#rssgetfirstitem-function)
- [RssGetItem](#rssgetitem-function)
- [RssGetItemProperty](#rssgetitemproperty-function)
- [RssGetItemText](#rssgetitemtext-function)
- [RSSCHANNELITEM](#rsschannelitem-function)

---

## RssInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI RssInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call RssGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that RssInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to RssUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the RssInitialize or RssUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssCloseFeed](#rssclosefeed-function)
- [RssOpenFeed](#rssopenfeed-function)
- [RssParseFeed](#rssparsefeed-function)
- [RssUninitialize](#rssuninitialize-function)

---

## RssOpenFeed Function

Open the specified news feed and return information about the channel

### Prototype

```c
HCHANNEL WINAPI RssOpenFeed(
    LPCTSTR lpszFeedUrl,
    UINT nTimeout,
    DWORD dwOptions,
    LPRSSCHANNEL lpChannel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFeedUrl` | `LPCTSTR` | A pointer to a string which specifies the URL for the news feed. To access a news feed on a web server, a standard http or https URL may be specified. To access a file on the local system or network share, a file name or file URL may be specified. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. This parameter is ignored if the lpszFeedUrl parameter specifies a local file name or URL. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpChannel` | `LPRSSCHANNEL` | A pointer to an RSSCHANNEL structure which contains information about the news feed channel such as the feed title, hyperlink and description. If the parameter is not NULL, the structure is initialized by the function. If the parameter is NULL, it is ignored and no information is returned. |

### Options

- **dwOptions** Parameter

    - `RSS_OPTION_NONE`: No additional options are specified and the news feed is processed using relaxed rules when checking the validity of the feed. The library will attempt to automatically compensate for a feed that is malformed or does not strictly conform to the RSS standard.
    - `RSS_OPTION_STRICT`: The news feed content should be processed using strict rules to ensure that the feed meets the appropriate RSS standard specification and all feed property values are case-sensitive. By default, relaxed rules are used which allows the application to open a feed that may not strictly conform to the standard specification.

### Return Value

- **Type**: `HCHANNEL`
- If the function succeeds, the return value is a handle to the news feed channel. If the function fails, the return value is INVALID_CHANNEL. To get extended error information, call RssGetLastError.

### Remarks

- A news feed may be local or remote, depending on the URL that is specified. If a local file name or file URL is specified for the feed, then it is opened locally and no network access is required. If an http or https URL is specified, then RssOpenFeed will attempt to download the feed from the server and store it temporarily on the local system. Accessing a remote feed requires that the application has permission to establish a connection with the server and will cause the application to block until the feed has been downloaded, the operation times out or an error occurs.
- Although the RssOpenFeed function will meet the needs of most applications, if you require more complex functionality such as retrieving the feed asynchronously in the background or event notifications for large transfers, you can use the SocketTools Hypertext Transfer Protocol API to download the news feed and then use the RssParseFeed function to parse the contents.
- The data referenced in the RSSCHANNEL structure should be considered read-only and never modified by the application. Not all members of the structure may contain valid values, in which case those members will either have a value of zero or will specify NULL pointers. When the feed is closed, the members of this structure will no longer be valid, and therefore should never be stored by the application. If the application needs to store or modify this information, it should create its own private copy of the data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssCloseFeed](#rssclosefeed-function)
- [RssInitialize](#rssinitialize-function)
- [RssParseFeed](#rssparsefeed-function)
- [RssRefreshFeed](#rssrefreshfeed-function)
- [RssStoreFeed](#rssstorefeed-function)
- [RssUninitialize](#rssuninitialize-function)
- [RssValidateFeed](#rssvalidatefeed-function)
- [RSSCHANNEL](#rsschannel-function)

---

## RssParseFeed Function

Parse the contents of a string and return information about the channel

### Prototype

```c
HCHANNEL WINAPI RssParseFeed(
    LPCTSTR lpszFeedXml,
    DWORD dwOptions,
    LPRSSCHANNEL lpChannel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFeedXml` | `LPCTSTR` | A pointer to a string which contains the contents of the news feed. The string must contain XML formatted data that conforms to the RSS standard specification. This parameter cannot specify an empty string or a NULL pointer. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpChannel` | `LPRSSCHANNEL` | A pointer to an RSSCHANNEL structure which contains information about the news feed channel such as the feed title, hyperlink and description. If the parameter is not NULL, the structure is initialized by the function. If the parameter is NULL, it is ignored and no information is returned. |

### Options

- **dwOptions** Parameter

    - `RSS_OPTION_NONE`: No additional options are specified.
    - `RSS_OPTION_STRICT`: The news feed content should be processed using strict rules to ensure that the feed meets the appropriate RSS standard specification and all feed property values are case-sensitive. By default, relaxed rules are used which allows the application to open a feed that may not strictly conform to the standard specification.

### Return Value

- **Type**: `HCHANNEL`
- If the function succeeds, the return value is a handle to the news feed channel. If the function fails, the return value is INVALID_CHANNEL. To get extended error information, call RssGetLastError.

### Remarks

- The RssParseFeed function is an alternative to the RssOpenFeed function, enabling the application to process a news feed from alternative sources such as a database or compressed file. It is important to note that the string which contains the news feed XML must be properly formatted and conform to the RSS standard specification.
- The data referenced in the RSSCHANNEL structure should be considered read-only and never modified by the application. Not all members of the structure may contain valid values, in which case those members will either have a value of zero or will specify NULL pointers. When the feed is closed, the members of this structure will no longer be valid, and therefore should never be stored by the application. If the application needs to store or modify this information, it should create its own private copy of the data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssCloseFeed](#rssclosefeed-function)
- [RssInitialize](#rssinitialize-function)
- [RssOpenFeed](#rssopenfeed-function)
- [RssRefreshFeed](#rssrefreshfeed-function)
- [RssStoreFeed](#rssstorefeed-function)
- [RssUninitialize](#rssuninitialize-function)
- [RSSCHANNEL](#rsschannel-function)

---

## RssRefreshFeed Function

Refresh the specified news feed, updating the items in the channel

### Prototype

```c
INT WINAPI RssRefreshFeed(
    HCHANNEL hChannel,
    LPRSSCHANNEL lpChannel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `lpChannel` | `LPRSSCHANNEL` | A pointer to an RSSCHANNEL structure which contains information about the news feed channel such as the feed title, hyperlink and description. If the parameter is not NULL, the structure is initialized by the function. If the parameter is NULL, it is ignored and no information is returned. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- When the RssRefreshFeed function is called, the news feed is reloaded from the original source and the items in the channel are updated. For news feeds that are frequently updated, the nTimeToLive member of the RSSCHANNEL structure can provide a hint to the application as to how frequently the feed should be refreshed.
- If the news feed was originally opened using an http or https URL, this function will download the updated feed from the server and store it temporarily on the local system. Accessing a remote feed requires that the application has permission to establish a connection with the server and will cause the application to block until the feed has been downloaded, the operation times out or an error occurs. The same timeout period and options will be used as when the feed was originally opened.
- The RssRefreshFeed function should only be used if the feed was opened using the RssOpenFeed function, otherwise the function will fail with an error indicating that the operation is not supported.
- The data referenced in the RSSCHANNEL structure should be considered read-only and never modified by the application. The members of this structure returned by previous calls to either the RssOpenFeed or RssRefreshFeed functions will no longer be valid and should not be referenced. Likewise, the members of an RSSCHANNELITEM structure will no longer be valid after this function returns.
- It is important that the application does not make any assumptions about the number of news items in the channel, or the content associated with those items after the RssRefreshFeed function has been called. For example, never assume that the number of items in the channel remains the same, or that the item IDs for each item remains the same. If you need to find a specific item in the news feed, use the RssFindItem function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssCloseFeed](#rssclosefeed-function)
- [RssFindItem](#rssfinditem-function)
- [RssGetItem](#rssgetitem-function)
- [RssOpenFeed](#rssopenfeed-function)
- [RssStoreFeed](#rssstorefeed-function)
- [RSSCHANNEL](#rsschannel-function)

---

## RssSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI RssSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or RSS_ERROR. Those functions which call RssSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the RssGetLastError function. The use of RssGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`

### See Also

- [RssGetErrorString](#rssgeterrorstring-function)
- [RssGetLastError](#rssgetlasterror-function)

---

## RssStoreFeed Function

Store the contents of the specified news feed in an XML formatted text file

### Prototype

```c
INT WINAPI RssStoreFeed(
    HCHANNEL hChannel,
    LPCTSTR lpszFileName,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hChannel` | `HCHANNEL` | Handle to the news feed channel. |
| `lpszFileName` | `LPCTSTR` | A pointer to a string which specifies the name of the file on the local system. The contents of the news feed will be stored in this file. If the file does not exist, it will be created; otherwise it will overwrite the contents of the file. |
| `dwReserved` | `DWORD` | A reserved parameter for future use. This value should always be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssOpenFeed](#rssopenfeed-function)
- [RssParseFeed](#rssparsefeed-function)

---

## RssUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI RssUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful RssInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call RssUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to RssUninitialize for every successful call to RssInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`

### See Also

- [RssCloseFeed](#rssclosefeed-function)
- [RssInitialize](#rssinitialize-function)

---

## RssValidateFeed Function

Validate the contents of the specified news feed, returning the number of items in the feed

### Prototype

```c
INT WINAPI RssValidateFeed(
    LPCTSTR lpszFeedUrl,
    UINT nTimeout,
    DWORD dwOptions,
    LPSYSTEMTIME lpstModified
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszFeedUrl` | `LPCTSTR` | A pointer to a string which specifies the URL for the news feed. To access a news feed on a web server, a standard http or https URL may be specified. To access a file on the local system or network share, a file name or file URL may be specified. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. This parameter is ignored if the lpszFeedUrl parameter specifies a local file name or URL. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is reserved for future use an should always have a value of zero. |
| `lpstModified` | `LPSYSTEMTIME` | A pointer to a SYSTEMTIME structure which will specify the date that the feed was last modified when the function returns. If the parameter is NULL, it is ignored and no information is returned. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of items in the news feed channel. If the function fails, the return value is RSS_ERROR. To get extended error information, call RssGetLastError.

### Remarks

- The RssValidateFeed function can be used to check that a news feed exists and is properly formatted. If the contents of the feed are valid, the function will return the number of items in the feed and the date that it was last modified. This can be useful for applications that want to periodically check a news feed and determine if the contents have changed.
- The SYSTEMTIME structure that is populated by the function specifies the date when the feed was last modified. The function first checks the value of the lastBuildDate property of the feed channel. If that property is not defined, then it will use the value of the pubDate property. If neither are defined, then the structure members will have a value of zero.
- The validation process imposes strict checks on the structure of the news feed and requires that it conform to the RSS specification. For example, the feed must have a title, link and description. Each item in the feed must have either a title or description, and the hyperlinks specified in the feed must be valid. If the feed XML is malformed, or a required property of the feed is invalid or missing, this function will fail.
- If a news feed cannot be validated, it still may be possible to open the feed using the RssOpenFeed function. By default, relaxed rules are used when parsing the contents of the feed and it does not check to ensure all required properties are defined and have valid values.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `csrssv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [RssOpenFeed](#rssopenfeed-function)
- [RssParseFeed](#rssparsefeed-function)
- [RssRefreshFeed](#rssrefreshfeed-function)
- [RssStoreFeed](#rssstorefeed-function)
- [SYSTEMTIME](#systemtime-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## RSSCHANNEL Structure

The RSSCHANNEL structure contains information about the properties of the news feed channel.

### Definition

```c
typedef struct _RSSCHANNEL
{
    UINT nItemCount;
    UINT nTimeToLive;
    WORD wVersionMajor;
    WORD wVersionMinor;
    DWORD dwFlags;
    DWORD dwReserved;
    LPCTSTR lpszTitle;
    LPCTSTR lpszLink;
    LPCTSTR lpszDescription;
    LPCTSTR lpszCategory;
    LPCTSTR lpszLanguage;
    LPCTSTR lpszCopyright;
    LPCTSTR lpszEditor;
    LPCTSTR lpszWebmaster;
    LPCTSTR lpszGenerator;
    LPCTSTR lpszImageLink;
    LPCTSTR lpszImageTitle;
    LPCTSTR lpszImageUrl;
    SYSTEMTIME stPublished;
    SYSTEMTIME stLastBuild;
} RSSCHANNEL, *LPRSSCHANNEL;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nItemCount` | `UINT` | An integer value which specifies number of news items in the channel. |
| `nTimeToLive` | `UINT` | An integer value which specifies the frequency in seconds at which the feed should be refreshed to obtain updated information. Not all feeds specify a time-to-live, in which case this member will have a value of zero. |
| `wVersionMajor` | `WORD` | A word value which specifies the major version number for the news feed. |
| `wVersionMinor` | `WORD` | A word value which specifies the minor version number for the news feed. |
| `dwFlags` | `DWORD` | A value which specifies one or more option flags for the news feed channel. Currently there are no option flags defined and this member is reserved for future expansion. |
| `dwReserved` | `DWORD` | A value reserved for future expansion. |
| `lpszTitle` | `LPCTSTR` | A pointer to a string which specifies the name of the channel. If the content of the news feed corresponds to a website, this is typically the same as the title of the website. If a title has not been specified, this member will be NULL. Note that a strictly conforming news feed requires a title. |
| `lpszLink` | `LPCTSTR` | A pointer to a string which specifies a URL to the website corresponding to the channel. Note that this is not the URL of the news feed itself. Typically it is a link to the home page of the site which owns the news feed. If a link has not been specified, this member will be NULL. Note that a strictly conforming news feed requires a valid link URL. |
| `lpszDescription` | `LPCTSTR` | A pointer to a string which describes the channel. This provides an overview of the news feed and the type of information that is provided. If a description of the feed has not been specified, this member will be NULL. Note that a strictly conforming news feed requires a description. |
| `lpszCategory` | `LPCTSTR` | A pointer to a string which defines the category or categories that the channel belongs to. This property is optional and the category names themselves are user-defined. If a category has not been specified, this member will be NULL. |
| `lpszLanguage` | `LPCTSTR` | A pointer to a string which defines the language the channel is written in, using the standard language codes. This property is optional and if this member is NULL, the English language is typically presumed to be the default. |
| `lpszCopyright` | `LPCTSTR` | A pointer to a string which specifies a copyright notice for the content. If a copyright has not been specified, this member will be NULL. |
| `lpszEditor` | `LPCTSTR` | A pointer to a string which identifies the person responsible for managing the content of the news feed. If this property is defined, it is typically the name and email address of the feed editor. If an editor has not been specified, this member will be NULL. |
| `lpszWebmaster` | `LPCTSTR` | A pointer to a string which identifies the person responsible for technical issues related to the news feed. If this property is defined, it is typically the name and email address of a system administrator. If a webmaster has not been specified, this member will be NULL. |
| `lpszGenerator` | `LPCTSTR` | A pointer to a string which identifies the application that was used to create the news feed. If the application that generated the feed has not been specified, this member will be NULL. |
| `lpszImageLink` | `LPCTSTR` | A pointer to a string which specifies a URL to the website corresponding to the channel. In most cases, this is the same URL that is specified by the lpszLink member. If an image link has not been specified, this member will be NULL. |
| `lpszImageTitle` | `LPCTSTR` | A pointer to a string which identifies the image associated with the channel. This is usually a brief description of the image, and may be the same as the value specified by the lpszTitle member. If an image title has not been specified, this member will be NULL. |
| `lpszImageUrl` | `LPCTSTR` | A pointer to a string which specifies a URL for the image associated with the channel. An application can download this image and display it with the contents of the news feed. If an image URL has not been specified, this member will be NULL. |
| `stPublished` | `SYSTEMTIME` | The date that the news feed was published. For example, a feed that is associated with a weekly print publication may update this value once per week. Note that this is not necessarily the date that the feed was last modified. If the channel does not specify the publish date, this structure will contain all zeroes. |
| `stLastBuild` | `SYSTEMTIME` | The date that the content of the channel was last modified. If the channel does not specify the build date, this structure will contain all zeroes. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## RSSCHANNELITEM Structure

The RSSCHANNELITEM structure contains information about the properties of an item in a news feed channel.

### Definition

```c
typedef struct _RSSCHANNELITEM
{
    UINT nItemId;
    DWORD dwFlags;
    DWORD dwReserved;
    LPCTSTR lpszTitle;
    LPCTSTR lpszLink;
    LPCTSTR lpszText;
    LPCTSTR lpszGuid;
    LPCTSTR lpszAuthor;
    LPCTSTR lpszSource;
    LPCTSTR lpszComments;
    LPCTSTR lpszEnclosure;
    SYSTEMTIME stPublished;
} RSSCHANNELITEM, *LPRSSCHANNELITEM;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nItemId` | `UINT` | An integer which identifies this item in the channel. |
| `dwFlags` | `DWORD` | A value which specifies one or more option flags for the item. Currently there are no option flags defined and this member is reserved for future expansion. |
| `dwReserved` | `DWORD` | A value reserved for future expansion. |
| `lpszTitle` | `LPCTSTR` | A pointer to a string which specifies the title of the item. If a title for the item has not been specified, this member will be NULL. |
| `lpszLink` | `LPCTSTR` | A pointer to a string which specifies a URL that typically links to additional information related to the item. If a link for the item has not been specified, this member will be NULL. |
| `lpszText` | `LPCTSTR` | A pointer to a string which specifies a summary or description of the item. This may contain either plain text or HTML formatted text, and there is no fixed limit to the length of the text. If no text has been specified for the item, this member will be NULL. |
| `lpszGuid` | `LPCTSTR` | A pointer to a string which uniquely identifies the item in the channel. If this property is defined, it is guaranteed to be a unique, persistent value. It is important to note that this string does not have to be a standard GUID reference number, it can be any unique string. In many cases it is the same value as the item hyperlink specified by the lpszLink member, although an application should never depend on this behavior. If there is no unique identifier associated with the item, this member will be NULL. |
| `lpszAuthor` | `LPCTSTR` | A pointer to a string which identifies the author of the item. If this property is defined, it is typically the name and email address of the person who created the content that the item links to. If the author is not specified, this member will be NULL. |
| `lpszSource` | `LPCTSTR` | A pointer to a string which identifies the source of the item, specified as a URL for the original news feed that contained it. This typically used to propagate credit for items that are aggregated by a third-party and re-published in their own channel. If the source is not specified, this member will be NULL. |
| `lpszComments` | `LPCTSTR` | A pointer to a string which specifies a URL that links to further discussion about the item. Typically this is a link to the comment area of a weblog or a forum topic specific to the item. If a comment link is not specified, this member will be NULL. |
| `lpszEnclosure` | `LPCTSTR` | A pointer to a string which specifies a URL that links to a file related to the item. This is similar to an attachment in an email message, however instead of the item containing the contents of the attached file, it only specifies a link to the file. Enclosures are most commonly used with podcasting where an item is linked to an audio or video file, however the link may reference any type of file. If there is no enclosure specified for the item, this member will be NULL. |
| `stPublished` | `SYSTEMTIME` | The date that the item was published. If the item does not specify the publish date, this structure will contain all zeroes. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

