# SocketTools 11 Library Edition

The SocketTools Library Edition includes standard Windows dynamic link libraries (DLLs) which can be used in a wide variety of programming languages such as Visual C++, Visual C#, Visual Basic and Delphi. The Library Edition is ideal for the developer who requires the high performance, minimum resource utilization and flexibility of a lower level interface, without the inherent overhead of ActiveX components or the .NET Framework. The SocketTools Library Edition API has over 950 functions which can be used to develop applications 
that meet a wide range of needs. SocketTools covers it all, including uploading and downloading files, sending and retrieving email, remote command execution, terminal emulation, and much more.

The SocketTools Library Edition includes support for the industry standard Transport Security Layer (TLS) and Secure Shell (SSH) protocols which are used to ensure that data exchanged between the local system and a server is secure and encrypted. The Library Edition implements the major secure protocols such as HTTPS, FTPS, SFTP, SMTPS, POP3S, IMAPS and more. Your data is protected with TLS 
1.2 or later using 256-bit encryption and full support for client certificates. SocketTools also includes an FTP and HTTP server API, as well as a general purpose TCP server API that can be used to create custom server applications. There's no need for you to understand the details of certificate management, data encryption or how the security protocols work. All it takes is a few lines of code to enable the security features, and SocketTools handles the rest.

## Technical Reference

### Domain Name Services

The Domain Name Services (DNS) protocol is what applications use to resolve domain names into Internet addresses as well as provide other information about a domain. All of the SocketTools libraries provide basic domain name resolution functionality, but the Domain Name Services library gives an application direct control over what servers are queried, the amount of time spent waiting for a response and the type of information that is returned.

- [Functions](csdnsapi.md#functions)
- [Structures](csdnsapi.md#structures)
- [Error Codes](cserror11.md)

### File Encoding and Compression

The Encoding and Compression library provides functions for encoding and decoding binary files, typically attachments to email messages. The process of encoding converts the contents of a binary file to printable 7-bit ASCII text. Decoding reverses the process, converting a previously encoded text file back into a binary file.

There are two primary types of encoding methods used with various Internet applications: base64 and uucode. The base64 algorithm is most commonly used with email attachments, and is often referred to as MIME encoding since this is the encoding method specified in the MIME standards document. The uucode algorithm (so called because the programs to perform the encoding were called uuencode and uudecode) is often used when attaching binary files to Usenet newsgroup posts. The library also supports an alternate encoding format called yEnc which is also widely used to attach files to Usenet posts.

In addition to encoding and decoding data files, this library includes functions to compress and expand data, as well as encrypt and decrypt files using 256-bit AES encryption.

- [Functions](csncdapi.md#functions)
- [Structures](csncdapi.md#structures)
- [Error Codes](cserror11.md)

### File Transfer Protocol Client

The File Transfer Protocol (FTP) library provides a comprehensive API which supports both high level operations, such as uploading or downloading files, as well as a collection of lower-level file I/O functions. In addition to file transfers, an application can create, rename and delete files and directories, search for files using wildcards and perform other common file management functions.

Files can be stored on the local file system or in memory, depending on the needs of your application and multiple file transfers be performed using a single function call. The library can also be used to manage files on the server and supports many of the common protocol extensions that can be used to access the remote file system. It understands a number of different directory listing formats, including those typically used with UNIX and Linux based systems, Windows server platforms, NetWare servers and VMS systems.

This library supports active and passive mode file transfers, firewall compatibility options, proxy servers and secure file transfers using the standard TLS 1.2 and SSH 2.0 protocols. Secure file transfers support implicit and explicit TLS sessions, client certificates and up to 256-bit AES encryption.

- [Functions](csftpapi.md#functions)
- [Structures](csftpapi.md#structures)
- [Error Codes](cserror11.md)

### File Transfer Protocol Server

This library provides an interface for implementing an embedded, lightweight server that can be used to exchange files with a client using the standard File Transfer Protocol. The server can accept connections from any third-party application or a program developed using the SocketTools FTP client API.

The application specifies an initial server configuration and then responds to events that are raised by the API when the client sends a request to the server. An application may implement only minimal handlers for most events, in which case the default actions are performed for most standard FTP commands. However, an application may also use the event mechanism to filter specific commands or to extend the protocol by providing custom implementations of existing commands or add entirely new commands.

The server supports active and passive mode file transfers, has compatibility options for NAT router and firewall support, and provides support for secure file transfers using explicit TLS. Secure connections require a valid security certificate to be installed on the system.

- [Functions](csftsapi.md#functions)
- [Structures](csftsapi.md#structures)
- [Error Codes](cserror11.md)

### Hypertext Transfer Protocol Client

The Hypertext Transfer Protocol (HTTP) is a lightweight, stateless application protocol that is used to access resources on web servers, as well as send data to those servers for processing. The library provides direct, low-level access to the server and the commands that are used to retrieve resources (i.e.: documents, images, etc.). The library also provides a simple interface for downloading resources to the local host, similar to how the HTTP library can be used to download files.

In a typical session, the library is used to establish a connection, send a request (to download a resource, post data for processing, etc.), read the data returned by the server and then disconnect. It is the responsibility of the client to process the data returned by the server, depending on the type of resource that was requested.

This library supports supports secure, encrypted file transfers using TLS 1.2 and later versions of the protocol.

- [Functions](cshtpapi.md#functions)
- [Structures](cshtpapi.md#structures)
- [Error Codes](cserror11.md)

### Hypertext Transfer Protocol Server

This library provides an interface for implementing an embedded, lightweight server that can be used to provide access to documents and other resources using the Hypertext Transfer Protocol. The server can accept connections from any standard web browser, third-party applications or programs developed using the SocketTools HTTP client API.

The application specifies an initial server configuration and then responds to events that are generated when the client sends a request to the server. An application may implement only minimal handlers for most events, in which case the default actions are performed for most standard HTTP commands. However, an application may also use the event mechanism to filter specific commands or to extend the protocol by providing custom implementations of existing commands or add entirely new commands.

The server includes support for CGI scripting, virtual hosting, client authentication and the creation of virtual directories and files. The server also supports secure connections using TLS 1.2 and later versions. Secure connections require that a valid security certificate be installed on the system.

- [Functions](cshtsapi.md#functions)
- [Structures](cshtsapi.md#structures)
- [Error Codes](cserror11.md)

### Internet Control Message Protocol

The Internet Control Message Protocol (ICMP) is commonly used to determine if a remote host is reachable and how packets of data are routed to that system. Users are most familiar with this protocol as it is implemented in the ping and traceroute command line utilities. The ping command is used to check if a system is reachable and the amount of time that it takes for a packet of data to make a round trip from the local system, to the remote host and then back again. The traceroute command is used to trace the route that a packet of data takes from the local system to the remote host, and can be used to identify potential problems with overall throughput and latency. The library can be used to build in this type of functionality in your own applications, giving you the ability to send and receive ICMP echo datagrams in order to perform your own analysis.

- [Functions](csicmapi.md#functions)
- [Structures](csicmapi.md#structures)
- [Error Codes](cserror11.md)

### Internet Message Access Protocol

The Internet Message Access Protocol (IMAP) is an application protocol which is used to access a user's email messages which are stored on a mail server. However, unlike the Post Office Protocol (POP) where messages are downloaded and processed on the local system, the messages on an IMAP server are retained on the server and processed remotely. This is ideal for users who need access to a centralized store of messages or have limited bandwidth. For example, traveling salesmen who have notebook computers or mobile users on a wireless network would be ideal candidates for using IMAP.

The SocketTools IMAP library implements the current standard for this protocol, and provides functions to retrieve messages, or just certain parts of a message, create and manage mailboxes, search for specific messages based on certain criteria and so on. The API is designed as a superset of the Post Office Protocol API, so developers who are used to working with the POP3 library will find the IMAP library very easy to integrate into an existing application.

This library supports secure connections using the standard SSL and TLS protocols.

- [Functions](csmapapi.md#functions)
- [Structures](csmapapi.md#structures)
- [Error Codes](cserror11.md)

### Mail Message (MIME)

The Mail Message library provides an interface for composing and processing email messages and newsgroup articles which are structured according to the Multipurpose Internet Mail Extensions (MIME) standard. Using this library, an application can easily create complex messages which include multiple alternative content types, such as plain text and styled HTML text, file attachments and customized headers.

It is not required that the developer understand the complex MIME standard; a single function call can be used to create multipart message, complete with a styled HTML text body and support for international character sets. The Mail Message library can be easily integrated with the other mail related protocol libraries, making it extremely easy to create and process MIME formatted messages.

The library also includes an interface for managing a local message storage file that can be used to store and retrieve multiple messages. Functions are provided to open and create storage files, add, remove and extract messages from storage, and search the stored messages for specific header field values.

- [Functions](csmsgapi.md#functions)
- [Structures](csmsgapi.md#structures)
- [Error Codes](cserror11.md)

### Network News Transfer Protocol

The Network News Transfer Protocol (NNTP) is used with servers that provide news services. This is similar in functionality to bulletin boards or message boards, where topics are organized hierarchically into groups, called newsgroups. Users can browse and search for messages, called news articles, which have been posted by other users. On many servers, they can also post their own articles which can be read by others. The largest collection of public newsgroups available is called USENET, a world-wide distributed discussion system. In addition, there are a large number of smaller news servers. For example, Catalyst Development operates a news server which functions as a forum for technical questions and announcements.

The SocketTools library provides a comprehensive interface for accessing newsgroups, retrieving articles and posting new articles. In combination with the Mail Message library to process the news articles, SocketTools can be used to integrate newsgroup access with an existing email application, or you can implement your own full-featured newsgroup client.

This library supports secure connections using the standard SSL and TLS protocols.

- [Functions](csnwsapi.md#functions)
- [Structures](csnwsapi.md#structures)
- [Error Codes](cserror11.md)

### News Feed

Really Simple Syndication (RSS) is a collection of standardized formats that are used to publish information about content that is frequently changed. A news feed is published in XML format, which contains one or more items that includes summary text, hyperlinks to source content and additional metadata that is used to describe the item. News feeds can be used for a variety of purposes, including providing updates for weblogs, news headlines, video and audio content. RSS can also be used for other purposes, such as a software updates, where new updates are listed as items in the feed.

News feeds can be accessed remotely from a web server, or locally as an XML formatted text file. The source of the feed is determined by the URI scheme that is specified. If the http or https scheme is specified, then the feed is retrieved from a web server. If the file scheme is used, the feed is considered to be local and is accessed from the disk or local network. The News Feed library provides an API that enables you to open a feed by URL and iterate through each of the items in the feed or search for a specific feed item. The API also provides a function that can be used to parse a string that contains XML data in RSS format, where the feed may have been retrieved from other sources such as a database.

- [Functions](csnwsapi.md#functions)
- [Structures](csnwsapi.md#structures)
- [Error Codes](cserror11.md)

### Post Office Protocol

The Post Office Protocol (POP3) provides access to a user's new email messages on a mail server. Functions are provided for listing available messages and then retrieving those messages, storing them either in files or in memory. Once a user's messages have been downloaded to the local system, they are typically removed from the server. This is the most popular email protocol used by Internet Service Providers (ISPs) and the library provides a complete interface for managing a user's mailbox. This library is typically used in conjunction with the Mail Message library, which is used to process the messages that are retrieved from the server.

This library supports secure connections using the standard SSL and TLS protocols. Both implicit and explicit SSL connections can be established, enabling the library to work with a wide variety of servers.

- [Functions](csnwsapi.md#functions)
- [Structures](csnwsapi.md#structures)
- [Error Codes](cserror11.md)

### Remote Command Protocol

The Remote Command protocol is used to execute a command on a server and return the output of that command to the client. This is most commonly used with UNIX based servers, although there are implementations of remote command servers for the Windows operating system. The library supports both the rcmd and rshell remote execution protocols and provides methods which can be used to search the data stream for specific sequences of characters. This makes it extremely easy to write Windows applications which serve as light-weight client interfaces to commands being executed on a UNIX server or another Windows system. The library can also be used to establish a remote terminal session using the rlogin protocol, which is similar to the Telnet protocol.

This protocol should not be used when connecting to a server on a public network because the user credentials are sent as unencrypted text. For secure remote command execution and interactive terminal sessions, it is recommended that you use the Secure Shell (SSH) library.

- [Functions](csrshapi.md#functions)
- [Structures](csrshapi.md#structures)
- [Error Codes](cserror11.md)

### Secure Shell Protocol

The Secure Shell (SSH) protocol API is used to establish a secure connection with a server which provides a virtual terminal session for a user. Its functionality is similar to how character based consoles and serial terminals work, enabling a user to login to the server, execute commands and interact with applications running on the server. The library provides an interface for establishing the connection and handling the standard I/O functions needed by the program. It also provides functions that enable a program to easily scan the data stream for specific sequences of characters, making it very simple to write light-weight client interfaces to applications running on the server. This library can be combined with the Terminal Emulation library to provide complete terminal emulation services for a standard ANSI or DEC-VT220 terminal.

- [Functions](cstshapi.md#functions)
- [Structures](cstshapi.md#structures)
- [Error Codes](cserror11.md)

### Simple Mail Transfer Protocol

The Simple Mail Transfer Protocol (SMTP) enables applications to deliver email messages to one or more recipients. The library provides an API for addressing and delivering messages, and extended features such as user authentication and delivery status notification. Unlike Microsoft's Messaging API (MAPI) or Collaboration Data Objects (CDO), there is no requirement to have certain third-party email applications installed or specific types of servers installed on the local system. The library can be used to deliver mail through a wide variety of systems, from standard UNIX based mail servers to Windows systems running Microsoft Exchange.

Using this library, messages can be delivered directly to the recipient, or they can be routed through a relay server, such as an Internet service provider's mail system. The SocketTools Mail Message API can be integrated with this library in order to provide an extremely simple, yet flexible interface for composing and delivering messages.

This library supports secure connections using the standard SSL and TLS protocols. Both implicit and explicit SSL connections are supported, as well as client certificates used for authentication.

- [Functions](csmtpapi.md#functions)
- [Structures](csmtpapi.md#structures)
- [Error Codes](cserror11.md)

### Telnet Protocol

The Telnet protocol is used to establish a connection with a server which provides a virtual terminal session for a user. Its functionality is similar to how character based consoles and serial terminals work, enabling a user to login to the server, execute commands and interact with applications running on the server. The class provides an interface for establishing the connection, negotiating certain options (such as whether characters will be echoed back to the client) and handling the standard I/O functions needed by the program.

The API also includes functions that enable a program to easily scan the data stream for specific sequences of characters, making it very simple to write light-weight client interfaces to applications running on the server. This library can be combined with the SocketTools Terminal Emulation API to provide complete terminal emulation services for a standard ANSI or DEC-VT220 terminal.

This library supports secure connections using the standard SSL and TLS protocols. To establish a secure connection to the server using the Secure Shell (SSH) protocol, use the SocketTools Secure Shell API.

- [Functions](cstntapi.md#functions)
- [Structures](cstntapi.md#structures)
- [Error Codes](cserror11.md)

### Terminal Emulator

The Terminal Emulation library provides a comprehensive API for emulating an ANSI or DEC-VT220 terminal, with full support for all standard escape and control sequences, color mapping and other advanced features. The library functions provide both a high level interface for parsing escape sequences and updating a display, as well as lower level primitives for directly managing the virtual display, such as controlling the individual display cells, moving the cursor position and specifying display attributes.

This library can be used in conjunction with the Remote Command, Secure Shell or Telnet Protocol libraries to provide terminal emulation services for an application, or it can be used independently. For example, this library could be used to provide emulation services for a program that connects to a device using an RS-232 serial port.

- [Functions](csnvtapi.md#functions)
- [Structures](csnvtapi.md#structures)
- [Error Codes](cserror11.md)

### Text Messaging

Short Message Service (SMS) is a text messaging service used by mobile communication devices to exchange brief text messages. Most service providers also provide gateway servers that can be used to send messages to a wireless device on their network using standard email protocols. The Text Message API provides functions that can be used to determine the provider associated with a specific telephone number and send a text message to the device using the provider's mail gateway.

This library has been designed to assist developers in sending text message notifications as part of their application. For example, it can be used to enable your software to automatically send notifications when a specific event occurs, such as an error condition. This library is not designed to be used with software that will send out a large number of text messages to many users, and there are limitations on the number of messages that may be sent to different phone numbers over a short period of time. Because many recipients must pay a fee for each text message they receive, text messages should only be sent to those who explicitly request them.

Note: This library only supports service providers in North America and cannot be used to send text messages to mobile devices that use providers outside of the United States and Canada. Some service providers may prevent messages from being sent through their gateway to a user that does not have unlimited text messaging as part of their service agreement.

- [Functions](cstxtapi.md#functions)
- [Structures](cstxtapi.md#structures)
- [Error Codes](cserror11.md)

### Time Protocol

The Time Protocol library provides an interface for synchronizing the local system's time and date with that of a server. The time values returned are in in Coordinated Universal Time and be adjusted for the local host's timezone. The library enables developers to query a server for the current time and then update the system clock if desired.

- [Functions](cstimapi.md#functions)
- [Structures](cstimapi.md#structures)
- [Error Codes](cserror11.md)

### Web Services

The Web Services library enables an application to store and manage data remotely, and return information about the current physical location of the client. These functions use secure services provided by SocketTools API servers and do not require third-party party APIs or accounts with other cloud service providers.

- [Functions](cswebapi.md#functions)
- [Structures](cswebapi.md#structures)
- [Error Codes](cserror11.md)

### Whois Protocol

The Whois protocol library provides an interface for requesting registration information for an Internet domain name. When a domain name is registered, the organization that registers the domain must provide certain contact information along with technical information such as the primary name servers for that domain. The library provides an API for requesting that information and returning it to the program so that it can be displayed or processed. This library would be most commonly used to query the Whois server at whois.internic.net to obtain information about a specific Internet domain name or an administrative contact at that domain.

- [Functions](cswhoapi.md#functions)
- [Structures](cswhoapi.md#structures)
- [Error Codes](cserror11.md)

### Windows Sockets (SocketWrench)

At the core of all of the SocketTools networking libraries is the Windows Sockets API. This provides a low level interface for sending and receiving data over the Internet or a local intranet using the Transmission Control Protocol (TCP) and/or User Datagram Protocol (UDP). The SocketWrench library provides a simpler interface to the Windows Sockets API, without sacrificing features or functionality. Using SocketWrench, you can easily create client and server applications while avoiding many of the mundane tasks and common problems that developers face when building Internet applications.

This library supports secure connections using the TLS 1.2 protocol and can also be used to create secure, customized server applications. Both implicit and explicit SSL connections are supported, enabling the library to work with a wide variety of client and server applications without requiring that you use third-party libraries or Microsoft's CryptoAPI.

- [Functions](cswskapi.md#functions)
- [Structures](cswskapi.md#structures)
- [Error Codes](cserror11.md)
