# Time Protocol

## Functions

- [ConvertNetworkTime](#convertnetworktime-function)
- [ConvertSystemTime](#convertsystemtime-function)
- [GetNetworkTime](#getnetworktime-function)
- [QueryNetworkTime](#querynetworktime-function)
- [ReadNetworkTime](#readnetworktime-function)
- [TimeDisableTrace](#timedisabletrace-function)
- [TimeEnableTrace](#timeenabletrace-function)
- [TimeGetErrorString](#timegeterrorstring-function)
- [TimeGetLastError](#timegetlasterror-function)
- [TimeInitialize](#timeinitialize-function)
- [TimeSetLastError](#timesetlasterror-function)
- [TimeUninitialize](#timeuninitialize-function)
- [UpdateLocalTime](#updatelocaltime-function)
## Structures

- [INITDATA](#initdata-structure)

---

## ConvertNetworkTime Function

Convert network time to system time

### Prototype

```c
BOOL WINAPI ConvertNetworkTime(
    DWORD dwNetworkTime,
    LPSYSTEMTIME lpSystemTime,
    BOOL bLocalTime
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwNetworkTime` | `DWORD` | The network time to be converted. |
| `lpSystemTime` | `LPSYSTEMTIME` | A pointer to a SYSTEMTIME structure which will be modified for the specified network time. |
| `bLocalTime` | `BOOL` | A boolean flag that is used to specify if the network time should be adjusted for the local timezone. |

### Return Value

- **Type**: `BOOL`
- If the network time could be converted, the function returns a non-zero value. If the network time cannot be converted, or the pointer to the SYSTEMTIME structure is invalid, the function will return zero.

### Remarks

- The network time value can represent a date and time up to the year 2036.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [ConvertSystemTime](#convertsystemtime-function)

---

## ConvertSystemTime Function

Convert system time to network time

### Prototype

```c
DWORD WINAPI ConvertSystemTime(
    LPSYSTEMTIME lpSystemTime
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpSystemTime` | `LPSYSTEMTIME` | A pointer to a SYSTEMTIME structure which will be modified for the specified network time. |

### Return Value

- **Type**: `DWORD`
- If the system time could be converted, the function returns the number of seconds since midnight, 1 January 1900 UTC. If the pointer to the SYSTEMTIME structure is invalid, or the structure contains invalid data, the function will return zero.

### Remarks

- The network time value can represent a date and time up to the year 2036.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [ConvertNetworkTime](#convertnetworktime-function)
- [UpdateLocalTime](#updatelocaltime-function)

---

## GetNetworkTime Function

Return the current network time from the server

### Prototype

```c
DWORD WINAPI GetNetworkTime(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server. The host must be running a time server that complies with the specifications outlined in RFC 868. |
| `nRemotePort` | `UINT` | The port the time server is running on. A value of zero indicates that the default port number for the service should be used. |
| `nTimeout` | `UINT` | The number of seconds that the function will wait for a response from the server. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, it returns the number of seconds since midnight, 1 January 1900 UTC. If the function was unable to obtain the time from the specified host, it returns zero.

### Remarks

- The GetNetworkTime function will cause the calling thread to block until the time is returned by the server, or the operation times out. For applications which require asynchronous operation, the QueryNetworkTime function should be used instead.
- The network time value can represent a date and time up to the year 2036. It is important to note that the network time value is not the same as the UNIX time value that is used the standard C library time functions.
- In the United States, the National Institute of Standards and Technology (NIST) hosts a number of public servers which can be used to obtain the current time. The following table lists the current host names and addresses:
- Time servers are also commonly maintained by Internet service providers and universities. If you are unable to obtain the time from a server, contact the system administrator to determine if they have the standard time service available on port 37.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ConvertNetworkTime](#convertnetworktime-function)
- [QueryNetworkTime](#querynetworktime-function)
- [ReadNetworkTime](#readnetworktime-function)
- [TimeInitialize](#timeinitialize-function)

---

## QueryNetworkTime Function

Query the server for the network time

### Prototype

```c
HCLIENT WINAPI QueryNetworkTime(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server. The host must be running a time server that complies with the specifications outlined in RFC 868. |
| `nRemotePort` | `UINT` | The port the time server is running on. A value of zero indicates that the default port number for the service (37) should be used. |
| `hEventWnd` | `HWND` | A handle to the window that will receive the notification that the network time has been returned by the server. |
| `uEventMsg` | `UINT` | The notification message that will be sent to the window, indicating that the client has received the network time from the server. |

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, it returns a handle which can be used with the obtain the network time. If the function fails, it returns INVALID_CLIENT. To get extended error information, call TimeGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [ConvertNetworkTime](#convertnetworktime-function)
- [GetNetworkTime](#getnetworktime-function)
- [ReadNetworkTime](#readnetworktime-function)
- [UpdateLocalTime](#updatelocaltime-function)

---

## ReadNetworkTime Function

Read the network time returned by the server

### Prototype

```c
DWORD WINAPI ReadNetworkTime(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | The client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, it returns the number of seconds since midnight, 1 January 1900 UTC. If the function was unable to read the time from the specified host, it returns zero.

### Remarks

- The ReadNetworkTime function reads the time value returned by the server using the handle returned by the QueryNetworkTime function. The network time value can represent a date and time up to the year 2036.
- To convert the time value to a SYSTEMTIME structure, use the ConvertNetworkTime function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [ConvertNetworkTime](#convertnetworktime-function)
- [QueryNetworkTime](#querynetworktime-function)

---

## TimeDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI TimeDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [TimeEnableTrace](#timeenabletrace-function)

---

## TimeEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI TimeEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TimeDisableTrace](#timedisabletrace-function)

---

## TimeGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI TimeGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The last-error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is 0, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TimeGetLastError](#timegetlasterror-function)
- [TimeSetLastError](#timesetlasterror-function)

---

## TimeGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI TimeGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the TimeSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the TimeGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call TimeSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or TIME_ERROR. Those functions which call TimeSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [TimeGetErrorString](#timegeterrorstring-function)
- [TimeSetLastError](#timesetlasterror-function)

---

## TimeInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI TimeInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call TimeGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that TimeInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to TimeUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the TimeInitialize or TimeUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [GetNetworkTime](#getnetworktime-function)
- [QueryNetworkTime](#querynetworktime-function)
- [TimeUninitialize](#timeuninitialize-function)

---

## TimeSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI TimeSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or TIME_ERROR. Those functions which call TimeSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the TimeGetLastError function. The use of TimeGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [TimeGetErrorString](#timegeterrorstring-function)
- [TimeGetLastError](#timegetlasterror-function)

---

## TimeUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI TimeUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful TimeInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call TimeUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to TimeUninitialize for every successful call to TimeInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [TimeInitialize](#timeinitialize-function)

---

## UpdateLocalTime Function

Update the local system time with the network time

### Prototype

```c
BOOL WINAPI UpdateLocalTime(
    DWORD dwNetworkTime
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwNetworkTime` | `DWORD` | The date and time the system clock should be set to, represented as the number of seconds since midnight, 1 January 1900. |

### Return Value

- **Type**: `BOOL`
- If the function is able to update the local time, it returns a non-zero value. If the specified time is invalid, or the user does not have the access rights to change the system clock, the function returns zero.

### Remarks

- The network time value can represent a date and time up to the year 2036.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstimv11.lib`

### See Also

- [ConvertNetworkTime](#convertnetworktime-function)
- [GetNetworkTime](#getnetworktime-function)
- [QueryNetworkTime](#querynetworktime-function)
- [TimeInitialize](#timeinitialize-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

