# Telnet Protocol

## Functions

- [TelnetAbort](#telnetabort-function)
- [TelnetAsyncConnect](#telnetasyncconnect-function)
- [TelnetAttachThread](#telnetattachthread-function)
- [TelnetBreak](#telnetbreak-function)
- [TelnetCancel](#telnetcancel-function)
- [TelnetConnect](#telnetconnect-function)
- [TelnetCreateSecurityCredentials](#telnetcreatesecuritycredentials-function)
- [TelnetDeleteSecurityCredentials](#telnetdeletesecuritycredentials-function)
- [TelnetDisableEvents](#telnetdisableevents-function)
- [TelnetDisableTrace](#telnetdisabletrace-function)
- [TelnetDisconnect](#telnetdisconnect-function)
- [TelnetEnableEvents](#telnetenableevents-function)
- [TelnetEnableTrace](#telnetenabletrace-function)
- [TelnetEventProc](#telneteventproc-function)
- [TelnetFreezeEvents](#telnetfreezeevents-function)
- [TelnetGetErrorString](#telnetgeterrorstring-function)
- [TelnetGetLastError](#telnetgetlasterror-function)
- [TelnetGetMode](#telnetgetmode-function)
- [TelnetGetSecurityInformation](#telnetgetsecurityinformation-function)
- [TelnetGetStatus](#telnetgetstatus-function)
- [TelnetGetTerminalType](#telnetgetterminaltype-function)
- [TelnetGetTimeout](#telnetgettimeout-function)
- [TelnetInitialize](#telnetinitialize-function)
- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsConnected](#telnetisconnected-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetIsThere](#telnetisthere-function)
- [TelnetIsWritable](#telnetiswritable-function)
- [TelnetLogin](#telnetlogin-function)
- [TelnetRead](#telnetread-function)
- [TelnetReadLine](#telnetreadline-function)
- [TelnetRegisterEvent](#telnetregisterevent-function)
- [TelnetSearch](#telnetsearch-function)
- [TelnetSetLastError](#telnetsetlasterror-function)
- [TelnetSetMode](#telnetsetmode-function)
- [TelnetSetTerminalType](#telnetsetterminaltype-function)
- [TelnetSetTimeout](#telnetsettimeout-function)
- [TelnetUninitialize](#telnetuninitialize-function)
- [TelnetWrite](#telnetwrite-function)
- [TelnetWriteLine](#telnetwriteline-function)
## Structures

- [INITDATA](#initdata-structure)
- [SECURITYCREDENTIALS](#securitycredentials-structure)
- [SECURITYINFO](#securityinfo-structure)

---

## TelnetAbort Function

Abort the current session and close the connection with the server

### Prototype

```c
INT WINAPI TelnetAbort(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- When the TelnetAbort function is called, the Telnet abort sequence is sent to the server and the connection to the server is terminated. Once this function returns, the client handle is no longer valid. If a program is currently executing on the server at the time this function is called, that program may be terminated as a result of the session being aborted. Applications should normally call TelnetDisconnect to gracefully disconnect from the server and should only use this function when the connection must be aborted immediately.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetBreak](#telnetbreak-function)
- [TelnetCancel](#telnetcancel-function)
- [TelnetIsBlocking](#telnetisblocking-function)

---

## TelnetAsyncConnect Function

Connect asynchronously to the specified server

### Prototype

```c
HCLIENT WINAPI TelnetAsyncConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPSECURITYCREDENTIALS lpCredentials,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on; a value of zero specifies that the default port number should be used. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | Pointer to credentials structure SECURITYCREDENTIALS. This parameter is only used if the TELNET_OPTION_SECURE option is specified for the connection. This parameter may be NULL, in which case no client credentials will be provided to the server. If client credentials are required, the fields dwSize, lpszCertStore, and lpszCertName must be defined, while other fields may be left undefined. Set dwSize to the size of the SECURITYCREDENTIALS structure. |
| `hEventWnd` | `HWND` | The handle to the event notification window. This window receives messages which notify the client of various asynchronous socket events that occur. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous socket event occurs. This value should be greater than WM_USER as defined in the Windows header files. |

### Options

- **dwOptions** Parameter

    - `TELNET_OPTION_NONE`: No connection options specified. A standard connection to the server will be established using the specified host name and port number.
    - `TELNET_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `TELNET_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `TELNET_OPTION_SECURE`: This option specifies the client should attempt to establish a secure connection with the server. Note that the server must support secure connections using the TLS protocol.
    - `TELNET_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server using the START_TLS option. The client initiates a standard connection with the server, then requests a secure connection during the option negotiation process.
    - `TELNET_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `TELNET_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `TELNET_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call TelnetGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetCreateSecurityCredentials](#telnetcreatesecuritycredentials-function)
- [TelnetDeleteSecurityCredentials](#telnetdeletesecuritycredentials-function)
- [TelnetDisconnect](#telnetdisconnect-function)
- [TelnetInitialize](#telnetinitialize-function)
- [TelnetUninitialize](#telnetuninitialize-function)

---

## TelnetAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI TelnetAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The TelnetAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should TelnetAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the TelnetCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the TelnetUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetDisconnect](#telnetdisconnect-function)
- [TelnetUninitialize](#telnetuninitialize-function)

---

## TelnetBreak Function

Send a break signal to the server

### Prototype

```c
INT WINAPI TelnetBreak(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetAbort](#telnetabort-function)
- [TelnetCancel](#telnetcancel-function)
- [TelnetRead](#telnetread-function)
- [TelnetWrite](#telnetwrite-function)

---

## TelnetCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI TelnetCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- When the TelnetCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetAbort](#telnetabort-function)
- [TelnetBreak](#telnetbreak-function)
- [TelnetIsBlocking](#telnetisblocking-function)

---

## TelnetConnect Function

Connect to the specified server

### Prototype

```c
HCLIENT WINAPI TelnetConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    UINT nTimeout,
    DWORD dwOptions,
    LPSECURITYCREDENTIALS lpCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to the name of the server to connect to; this may be a fully-qualified domain name or an IP address. |
| `nRemotePort` | `UINT` | The port number the server is listening on; a value of zero specifies that the default port number should be used. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | Pointer to credentials structure SECURITYCREDENTIALS. This parameter is only used if the TELNET_OPTION_SECURE option is specified for the connection. This parameter may be NULL, in which case no client credentials will be provided to the server. If client credentials are required, the fields dwSize, lpszCertStore, and lpszCertName must be defined, while other fields may be left undefined. Set dwSize to the size of the SECURITYCREDENTIALS structure. |

### Options

- **dwOptions** Parameter

    - `TELNET_OPTION_NONE`: No connection options specified. A standard connection to the server will be established using the specified host name and port number.
    - `TELNET_OPTION_TUNNEL`: This option specifies that a tunneled TCP connection and/or port-forwarding is being used to establish the connection to the server. This changes the behavior of the client with regards to internal checks of the destination IP address and remote port number, default capability selection and how the connection is established.
    - `TELNET_OPTION_TRUSTEDSITE`: This option specifies the server is trusted. The server certificate will not be validated and the connection will always be permitted. This option only affects connections using the TLS protocol.
    - `TELNET_OPTION_SECURE`: This option specifies the client should attempt to establish a secure connection with the server. Note that the server must support secure connections using the TLS protocol.
    - `TELNET_OPTION_SECURE_EXPLICIT`: This option specifies the client should attempt to establish a secure connection with the server using the START_TLS option. The client initiates a standard connection with the server, then requests a secure connection during the option negotiation process.
    - `TELNET_OPTION_SECURE_FALLBACK`: This option specifies the client should permit the use of less secure cipher suites for compatibility with legacy servers. If this option is specified, the client will allow connections using TLS 1.0 and cipher suites that use RC4, MD5 and SHA1.
    - `TELNET_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `TELNET_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call TelnetGetLastError.

### Remarks

- This function will block the current thread until a connection has been established or the timeout period has elapsed. To prevent it from blocking the main user interface thread, the application should create a background worker thread and establish a connection by calling TelnetConnect in that thread. If the application requires multiple simultaneous connections, it is recommended you create a worker thread for each connection.
- The dwOptions argument can be used to specify the threading model that is used by the library when a connection is established. By default, the handle is initially attached to the thread that created it. From that point on, until the it is released, only the owner may call functions using that handle. The ownership of the handle may be transferred from one thread to another using the TelnetAttachThread function.
- Specifying the TELNET_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetCreateSecurityCredentials](#telnetcreatesecuritycredentials-function)
- [TelnetDeleteSecurityCredentials](#telnetdeletesecuritycredentials-function)
- [TelnetDisconnect](#telnetdisconnect-function)
- [TelnetInitialize](#telnetinitialize-function)
- [TelnetUninitialize](#telnetuninitialize-function)

---

## TelnetCreateSecurityCredentials Function

Create a new security credentials structure

### Prototype

```c
BOOL WINAPI TelnetCreateSecurityCredentials(
    DWORD dwProtocol,
    DWORD dwOptions,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    LPCTSTR lpszCertStore,
    LPCTSTR lpszCertName,
    LPVOID lpvReserved,
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. This parameter is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or more options. This member is constructed by using a bitwise operator with any of the following values: |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's username. A value of NULL specifies that no username is required. Currently this parameter is not used and any value specified will be ignored. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the certificate owner's password. A value of NULL specifies that no password is required. This parameter is only used if a PKCS #12 (PFX) certificate file is specified and that certificate has been secured with a password. This value will be ignored the current user or local machine certificate store is specified. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The function will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the function will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the function will return an error indicating that the certificate could not be found. |
| `lpvReserved` | `LPVOID` | Pointer reserved for future use. Set it to NULL when using this function. |
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. The memory for the credentials structure will be allocated by this function and must be released by calling the TelnetDeleteSecurityCredentials function when it is no longer needed. The pointer value must be set to NULL before the function is called. It is important to note that this is a pointer to a pointer variable, not a pointer to the SECURITYCREDENTIALS structure itself. |

### Options

- **dwProtocol** Parameter

    - `SECURITY_PROTOCOL_DEFAULT`: The default security protocol will be used when establishing a connection. This option will always request the latest version of the preferred security protocol and is the recommended value. Currently the default security protocol is TLS 1.2.
    - `SECURITY_PROTOCOL_TLS`: Either the TLS 1.2 or 1.3 protocol should be used when establishing a secure connection. The correct protocol is automatically selected, based on what version of the protocol is supported by the server. If this is the only protocol specified, SSL will be excluded from the list of supported protocols. This may be necessary for some servers which reject any attempt to use the older SSL protocol and require that only TLS be used.
    - `SECURITY_PROTOCOL_TLS10`: The TLS 1.0 protocol should be used when establishing a secure connection. This version is supported on all Windows platforms, however some servers may reject connections using version 1.0 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS11`: The TLS 1.1 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. Some servers may reject connections using version 1.1 in favor of using version 1.2. This version should only be used for backwards compatibility with older servers that have not been updated to use the current version of TLS.
    - `SECURITY_PROTOCOL_TLS12`: The TLS 1.2 protocol should be used when establishing a secure connection. This version is supported on Windows 7 and later desktop platforms, and Windows Server 2008 R2 and later server platforms. This version of TLS offers the broadest compatibility with most servers.
    - `SECURITY_PROTOCOL_TLS13`: The TLS 1.3 protocol should be used when establishing a secure connection. This is the newest version of the protocol and is only supported on Windows 11, Windows Server 2022 and later versions of Windows. If this version is not supported by the operating system, TLS 1.2 will be used instead.
- **dwOptions** Parameter

    - `CREDENTIAL_STORE_CURRENT_USER`: The library will attempt to open a store for the current user using the certificate store name provided in this structure. If no options are specified, then this is the default behavior.
    - `CREDENTIAL_STORE_LOCAL_MACHINE`: The library will attempt to open a local machine store using the certificate store name provided in this structure. If this option is specified, the library will always search for a local machine store before searching for the store by that name for the current user.
    - `CREDENTIAL_STORE_FILENAME`: The certificate store name is a file that contains the certificate and private key that will be used.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call the TelnetGetLastError function.

### Remarks

- The structure that is created by this function may be used as client credentials when establishing a secure connection. This is particularly useful for programming languages other than C/C++ which may not support C structures or pointers. The pointer to the SECURITYCREDENTIALS structure can be declared as an unsigned integer variable which is passed by reference to this function, and then passed by value to the TelnetConnect function.
- A client application only needs to call this function if the server requires that the client provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions parameter should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore parameter should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12.These certificate files typically have an extension of .pfx or .p12. If a password was specified when the certificate was created, it must be provided in the lpszPassword parameter or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword parameters are values which are used to access the certificate store or client certificate. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session. In most cases, these parameters will be NULL.

### Example

```cpp
LPSECURITYCREDENTIALS lpSecCred = NULL;
TelnetCreateSecurityCredentials(SECURITY_PROTOCOL_DEFAULT,
								0,
								NULL,
								NULL,
								lpszCertStore,
								lpszCertName,
								NULL,
								&lpSecCred);

hClient = TelnetConnect(lpszHostName,
						TELNET_PORT_SECURE,
						TELNET_TIMEOUT,
						TELNET_OPTION_SECURE,
						lpSecCred);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetDeleteSecurityCredentials](#telnetdeletesecuritycredentials-function)
- [TelnetGetSecurityInformation](#telnetgetsecurityinformation-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)

---

## TelnetDeleteSecurityCredentials Function

Delete a previously created security credentials structure

### Prototype

```c
VOID WINAPI TelnetDeleteSecurityCredentials(
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. On exit from the function, the pointer will be NULL. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- This function can be used to release the memory allocated to the client or server credentials after a secure connection has been terminated.

### Example

```cpp
if (lpSecCred)
	TelnetDeleteSecurityCredentials(&lpSecCred);

TelnetUninitialize();
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetCreateSecurityCredentials](#telnetcreatesecuritycredentials-function)
- [TelnetUninitialize](#telnetuninitialize-function)

---

## TelnetDisableEvents Function

Disable asynchronous event notification

### Prototype

```c
INT WINAPI TelnetDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- This function affects both event notification and event callbacks. Any outstanding events in the message queue should be ignored by the client application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetEnableEvents](#telnetenableevents-function)
- [TelnetFreezeEvents](#telnetfreezeevents-function)
- [TelnetRegisterEvent](#telnetregisterevent-function)

---

## TelnetDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI TelnetDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetEnableTrace](#telnetenabletrace-function)

---

## TelnetDisconnect Function

Disconnect from the current server

### Prototype

```c
INT WINAPI TelnetDisconnect(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetUninitialize](#telnetuninitialize-function)

---

## TelnetEnableEvents Function

Enable asynchronous event notification

### Prototype

```c
INT WINAPI TelnetEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the event notification window. This window receives a user-defined message which specifies the event that has occurred. If this value is NULL, event notification is disabled. |
| `uEventMsg` | `UINT` | An unsigned integer which specifies the user-defined message that is sent when an event occurs. This parameter's value must be greater than the value of WM_USER. If the hEventWnd parameter is NULL, this value must be specified as WM_NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetDisableEvents](#telnetdisableevents-function)
- [TelnetFreezeEvents](#telnetfreezeevents-function)
- [TelnetRegisterEvent](#telnetregisterevent-function)

---

## TelnetEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI TelnetEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetDisableTrace](#telnetdisabletrace-function)

---

## TelnetEventProc Function

Callback function that processes events generated by the client

### Prototype

```c
VOID __cdecl TelnetEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the TelnetRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the TelnetRegisterEvent function. The TelnetEventProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetDisableEvents](#telnetdisableevents-function)
- [TelnetEnableEvents](#telnetenableevents-function)
- [TelnetFreezeEvents](#telnetfreezeevents-function)
- [TelnetRegisterEvent](#telnetregisterevent-function)

---

## TelnetFreezeEvents Function

Suspend asynchronous event processing

### Prototype

```c
INT WINAPI TelnetFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four read events occur, once event handling is resumed only one of those read events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetDisableEvents](#telnetdisableevents-function)
- [TelnetEnableEvents](#telnetenableevents-function)
- [TelnetRegisterEvent](#telnetregisterevent-function)

---

## TelnetGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI TelnetGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetGetLastError](#telnetgetlasterror-function)
- [TelnetSetLastError](#telnetsetlasterror-function)

---

## TelnetGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI TelnetGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the TelnetSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the TelnetGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call TelnetSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or TELNET_ERROR. Those functions which call TelnetSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetErrorString](#telnetgeterrorstring-function)
- [TelnetSetLastError](#telnetsetlasterror-function)

---

## TelnetGetMode Function

Return the current client mode

### Prototype

```c
UINT WINAPI TelnetGetMode(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | A handle to the client session. |

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the current client mode. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- The client mode is a combination of one or more flags which determines how the client handles local character echo and character processing. The following values are recognized:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetSetMode](#telnetsetmode-function)

---

## TelnetGetSecurityInformation Function

Return security information about the current client connection

### Prototype

```c
BOOL WINAPI TelnetGetSecurityInformation(
    HCLIENT hClient,
    LPSECURITYINFO lpSecurityInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpSecurityInfo` | `LPSECURITYINFO` | A pointer to a SECURITYINFO structure which contains information about the current client connection. The dwSize member of this structure must be initialized to the size of the structure before passing the address of the structure to this function. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call TelnetGetLastError.

### Remarks

- This function is used to obtain security related information about the current client connection to the server. It can be used to determine if a secure connection has been established, what security protocol was selected, and information about the server certificate. Note that a secure connection has not been established, the dwProtocol structure member will contain the value SECURITY_PROTOCOL_NONE.

### Example

```cpp
// Notifies the user if the connection is secure or not
SECURITYINFO securityInfo;

securityInfo.dwSize = sizeof(SECURITYINFO); 
if (TelnetGetSecurityInformation(hClient, &securityInfo)) 
{ 
	if (securityInfo.dwProtocol == SECURITY_PROTOCOL_NONE) 
	{ 
		MessageBox(NULL, _T("The connection is not secure"), 
						_T("Connection"), MB_OK); 
	} 
	else 
	{ 
		if (securityInfo.dwCertStatus == SECURITY_CERTIFICATE_VALID) 
		{ 
			MessageBox(NULL, _T("The connection is secure"), 
							_T("Connection"), MB_OK); 
		} 
		else 
		{ 
			MessageBox(NULL, _T("The server certificate not valid"), 
							_T("Connection"), MB_OK); 
		} 
	} 
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetDisconnect](#telnetdisconnect-function)
- [SECURITYINFO](#securityinfo-function)

---

## TelnetGetStatus Function

Return the current client status

### Prototype

```c
INT WINAPI TelnetGetStatus(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the client status code. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- The TelnetGetStatus function returns a numeric code which identifies the current state of the client session. The following values may be returned:
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsConnected](#telnetisconnected-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetIsWritable](#telnetiswritable-function)

---

## TelnetGetTerminalType Function

Return the current terminal type

### Prototype

```c
INT WINAPI TelnetGetTerminalType(
    HCLIENT hClient,
    LPCTSTR lpszTermType,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszTermType` | `LPCTSTR` | Points to a buffer which the current terminal type is copied into. This buffer should be at least 32 characters in length, including the terminating null character. |
| `nMaxLength` | `INT` | Maximum number of characters that may be copied to the buffer, including the terminating null character. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the terminal type name. A value of zero indicates that no terminal type has been specified. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetSetTerminalType](#telnetsetterminaltype-function)

---

## TelnetGetTimeout Function

Return the number of seconds until an operation times out

### Prototype

```c
INT WINAPI TelnetGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetSetTimeout](#telnetsettimeout-function)

---

## TelnetInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI TelnetInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call TelnetGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that TelnetInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to TelnetUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the TelnetInitialize or TelnetUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetDisconnect](#telnetdisconnect-function)
- [TelnetUninitialize](#telnetuninitialize-function)

---

## TelnetIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI TelnetIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns a non-zero value. If the client is not performing a blocking operation, or the client handle is invalid, the function returns zero.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The TelnetIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action such as warning the user that they must wait for the operation to complete.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetCancel](#telnetcancel-function)

---

## TelnetIsConnected Function

Determine if the client is connected to the server

### Prototype

```c
BOOL WINAPI TelnetIsConnected(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is connected to a server, the function returns a non-zero value. If the client is not connected, or the client handle is invalid, the function returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetIsThere](#telnetisthere-function)
- [TelnetIsWritable](#telnetiswritable-function)

---

## TelnetIsReadable Function

Determine if data can be read from the server

### Prototype

```c
BOOL WINAPI TelnetIsReadable(
    HCLIENT hClient,
    INT nTimeout,
    LPDWORD lpdwAvail
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |
| `lpdwAvail` | `LPDWORD` | A pointer to an unsigned integer which will contain the number of bytes available to read. This parameter may be NULL if this information is not required. |

### Return Value

- **Type**: `BOOL`
- If the client can read data from the server within the specified timeout period, the function returns a non-zero value. If the client cannot read any data, the function returns zero.

### Remarks

- On some platforms, this value will not exceed the size of the receive buffer (typically 64K bytes). Because of differences between TCP/IP stack implementations, it is not recommended that your application exclusively depend on this value to determine the exact number of bytes available. Instead, it should be used as a general indicator that there is data available to be read.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetStatus](#telnetgetstatus-function)
- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsConnected](#telnetisconnected-function)
- [TelnetIsThere](#telnetisthere-function)
- [TelnetIsWritable](#telnetiswritable-function)
- [TelnetRead](#telnetread-function)

---

## TelnetIsThere Function

Determine if the server is available

### Prototype

```c
BOOL WINAPI TelnetIsThere(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- The function returns a non-zero value if the server acknowledges a specific control sequence used to determine if a Telnet server is responsive. If the server does not respond, the function will return a value of zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetStatus](#telnetgetstatus-function)
- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsConnected](#telnetisconnected-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetIsWritable](#telnetiswritable-function)

---

## TelnetIsWritable Function

Determine if data can be written to the server

### Prototype

```c
BOOL WINAPI TelnetIsWritable(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |

### Return Value

- **Type**: `BOOL`
- If the client can write data to the server within the specified timeout period, the function returns a non-zero value. If the client cannot write any data, the function returns zero.

### Remarks

- Although this function can be used to determine if some amount of data can be sent to the remote process it does not indicate the amount of data that can be written without blocking the client. In most cases, it is recommended that large amounts of data be broken into smaller logical blocks, typically some multiple of 512 bytes in length.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetStatus](#telnetgetstatus-function)
- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsConnected](#telnetisconnected-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetIsThere](#telnetisthere-function)
- [TelnetWrite](#telnetwrite-function)

---

## TelnetLogin Function

Login to the server using the specified username and password

### Prototype

```c
BOOL WINAPI TelnetLogin(
    HCLIENT hClient,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the name of the user to authenticate. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password to be used when authenticating the user. If the user does not require a password, this parameter may be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call TelnetGetLastError.

### Remarks

- The TelnetLogin function is used to authenticate a user, logging them into the server. This function is specifically designed to work with most UNIX based servers, and may work with other servers that use a similar login process. The function works by scanning the data stream for a username prompt and then replying with the specified username. If that is successful, it will then scan for a password prompt and provide the specified password. If no recognized prompt is found, or if the server responds with an error indicating that the username or password is invalid, the function will fail.
- If the TelnetLogin function succeeds, the next call to TelnetRead by the client will return any welcome message to the user. This is typically followed by a command prompt where the user can enter commands to be executed on the server. The data sent by the server during the login process is discarded and not available when the function returns. If the client requires this information, use the TelnetSearch function to automate the login process instead.
- Because the TelnetLogin function is designed for UNIX based systems, it may not work with servers running on other operating system platforms such as Windows or VMS. In this case, applications should use the TelnetSearch function to search for the appropriate login prompts in the data stream.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetConnect](#telnetconnect-function)
- [TelnetIsConnected](#telnetisconnected-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetRead](#telnetread-function)
- [TelnetSearch](#telnetsearch-function)

---

## TelnetRead Function

Read data returned by the server

### Prototype

```c
INT WINAPI TelnetRead(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer in which the data will be copied. |
| `cbBuffer` | `INT` | The maximum number of bytes to read and copy into the specified buffer. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually read. A return value of zero indicates that the server has closed the connection and there is no more data available to be read. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- When TelnetRead is called and the client is in non-blocking mode, it is possible that the function will fail because there is no available data to read at that time. This should not be considered a fatal error. Instead, the application should simply wait to receive the next asynchronous notification that data is available.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetSearch](#telnetsearch-function)
- [TelnetWrite](#telnetwrite-function)

---

## TelnetReadLine Function

Read a line of text from the server and return it in a string buffer

### Prototype

```c
BOOL WINAPI TelnetReadLine(
    HCLIENT hClient,
    LPTSTR lpszBuffer,
    LPINT lpnLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszBuffer` | `LPTSTR` | Pointer to the string buffer that will contain the data when the function returns. The string will be terminated with a null byte, and will not contain the end-of-line characters. |
| `lpnLength` | `LPINT` | A pointer to an integer value which specifies the length of the buffer. The value should be initialized to the maximum number of characters that can be copied into the string buffer, including the terminating null character. When the function returns, its value will updated with the actual length of the string. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call TelnetGetLastError.

### Remarks

- The TelnetReadLine function reads data sent by the server and copies it into a specified string buffer. Unlike the TelnetRead function which reads arbitrary bytes of data, this function is specifically designed to return a single line of text data in a string. When an end-of-line character sequence is encountered, the function will stop and return the data up to that point. The string buffer is guaranteed to be null-terminated and will not contain the end-of-line characters.
- There are some limitations when using TelnetReadLine. The function should only be used to read text, never binary data. In particular, the function will discard nulls, linefeed and carriage return control characters. The Unicode version of this function will return a Unicode string, however this function does not support reading raw Unicode data from the server. The data is internally buffered as octets (eight-bit bytes) and converted to Unicode using the MultiByteToWideChar function.
- This function will force the thread to block until an end-of-line character sequence is processed, the read operation times out or the server closes its end of the connection. If this function is called with asynchronous events enabled, it will automatically switch the client into a blocking mode, read the data and then restore the client to asynchronous operation. If another client operation is attempted while TelnetReadLine is blocked waiting for data from the server, an error will occur. It is recommended that this function only be used with blocking (synchronous) client connections; if the application needs to establish multiple simultaneous connections, it should create worker threads to manage each connection.
- The TelnetRead and TelnetReadLine function calls can be intermixed, however be aware that TelnetRead will consume any data that has already been buffered by the TelnetReadLine function and this may have unexpected results.
- Unlike the TelnetRead function, it is possible for data to be returned in the buffer even if the return value is zero. Applications should also check the value of the lpnLength argument to determine if any data was copied into the buffer. For example, if a timeout occurs while the function is waiting for more data to arrive, it will return zero; however, data may have already been copied into the string buffer prior to the error condition. It is the responsibility of the application to process that data, regardless of the function return value.

### Example

```cpp
TCHAR szBuffer[MAXBUFLEN];
INT nLength;
BOOL bResult;

do
{
	nLength = sizeof(szBuffer);
	bResult = TelnetReadLine(hSocket, szBuffer, &nLength);

	if (nLength > 0)
	{
		// Process the line of data returned in the string
		// buffer; the string is always null-terminated
	}
} while (bResult);

DWORD dwError = TelnetGetLastError();
if (dwError == ST_ERROR_CONNECTION_CLOSED)
{
	// The server has closed its side of the connection and
	// there is no more data available to be read
}
else if (dwError != 0)
{
	// An error has occurred while reading a line of data
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetIsReadble](#telnetisreadble-function)
- [TelnetRead](#telnetread-function)
- [TelnetWrite](#telnetwrite-function)
- [TelnetWriteLine](#telnetwriteline-function)

---

## TelnetRegisterEvent Function

Register an event callback function

### Prototype

```c
INT WINAPI TelnetRegisterEvent(
    HCLIENT hClient,
    UINT nEvent,
    TELNETEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One of the following values may be used: |
| `lpEventProc` | `TELNETEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the TelnetEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEvent** Parameter

    - `TELNET_EVENT_CONNECT`: The connection to the server has completed.
    - `TELNET_EVENT_DISCONNECT`: The server has closed the connection to the client. The client should read any remaining data and disconnect.
    - `TELNET_EVENT_READ`: Data is available to read by the calling process. No additional messages will be posted until the client has read at least some of the data. This event is only generated if the client is in asynchronous mode.
    - `TELNET_EVENT_WRITE`: The client can now write data. This notification is sent after a connection has been established, or after a previous attempt to write data has failed because it would result in a blocking operation. This event is only generated if the client is in asynchronous mode.
    - `TELNET_EVENT_TIMEOUT`: The network operation has exceeded the specified timeout period. The client application may attempt to retry the operation, or may disconnect from the server and report an error to the user.
    - `TELNET_EVENT_CANCEL`: The current operation has been canceled. Under most circumstances the client should disconnect from the server and re-connect if needed. After an operation has been canceled, the server may abort the connection or refuse to accept further commands from the client.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- The TelnetRegisterEvent function associates a callback function with a specific event. The event handler is an TelnetEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetDisableEvents](#telnetdisableevents-function)
- [TelnetEnableEvents](#telnetenableevents-function)
- [TelnetEventProc](#telneteventproc-function)
- [TelnetFreezeEvents](#telnetfreezeevents-function)

---

## TelnetSearch Function

Search for a specific character sequence in the data stream

### Prototype

```c
BOOL WINAPI TelnetSearch(
    HCLIENT hClient,
    LPCTSTR lpszString,
    LPVOID lpvBuffer,
    LPDWORD lpdwLength,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszString` | `LPCTSTR` | A pointer to a string which specifies the sequence of characters to search for in the data stream. This parameter cannot be NULL or point to an empty string. |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the output from the server, or a pointer to a global memory handle which will reference the output when the function returns. If the output from the server is not required, this parameter may be NULL. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual number of bytes of output stored in the buffer. If the lpvBuffer parameter is NULL, this parameter should also be NULL. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must be zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds and the character sequences was found in the data stream, the return value is non-zero. If the function fails or a timeout occurs before the sequence is found, the return value is zero. To get extended error information, call TelnetGetLastError.

### Remarks

- The TelnetSearch function searches for a character sequence in the data stream and stops reading when it is found. This is useful when the client wants to automate responses to the server, such as logging in a user and executing a command. The function collects the output from the server and stores it in the buffer specified by the lpvBuffer parameter. When the function returns, the buffer will contain everything sent by the server up to and including the search string.
- The lpvBuffer parameter may be specified in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the a fixed amount of output. In this case, the lpvBuffer parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer. If the server sends more output than can be stored in the buffer, the remaining output will be discarded.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the output when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. This method is preferred if the client application does not have a general idea of how much output will be generated until the search string is found.

### Example

```cpp
LPCTSTR lpszUserName = "abc123\r\n";
LPCTSTR lpszPassword = "secret\r\n";
LPCTSTR lpszCommand = "/bin/ls -l\r\n";
HGLOBAL hgblOutput = NULL;
DWORD cbOutput = 0;
BOOL bResult;

// Search for the login prompt issued by the server

bResult = TelnetSearch(hClient,
					_T("ogin: "),
					NULL,
					NULL,
					0);

// If the Login: prompt was found, then write out the
// username and search for the Password: prompt; note
// that the username, password and command strings are
// terminated with a carriage-return/linefeed sequence
// which the server will see as the user pressing the
// Enter or Return key on the keyboard

if (bResult)
{
	TelnetWrite(hClient,
				(LPBYTE)lpszUserName,
				lstrlen(lpszUserName));

	bResult = TelnetSearch(hClient,
						_T("word: "),
						NULL,
						NULL,
						0);
}

// If the Password: prompt was found, write out the
// password and then search for the shell prompt;
// the prompt may be different, depending on what
// operating system and shell is being used

if (bResult)
{
	TelnetWrite(hClient,
				(LPBYTE)lpszPassword,
				lstrlen(lpszPassword));

	bResult = TelnetSearch(hClient,
						_T("$ "),
						NULL,
						NULL,
						0);
}

// If the shell prompt was found, issue the command
// and capture the output into the hgblBuffer global
// memory buffer; the cbBuffer variable will contain
// the actual number of bytes in the buffer when the
// function returns

if (bResult)
{
	TelnetWrite(hClient,
				(LPBYTE)lpszCommand,
				lstrlen(lpszCommand));

	bResult = TelnetSearch(hClient,
						_T("$ "),
						&hgblOutput,
						&cbOutput,
						0);
}

// Write the contents of the output buffer to the
// standard output stream

if (bResult)
{
	LPBYTE lpBuffer = (LPBYTE)GlobalLock(hgblBuffer);
	
	if (lpBuffer)
		fwrite(lpBuffer, 1, cbBuffer, stdout);

	GlobalUnlock(hgblBuffer);
	GlobalFree(hgblBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetIsBlocking](#telnetisblocking-function)
- [TelnetIsReadable](#telnetisreadable-function)
- [TelnetLogin](#telnetlogin-function)
- [TelnetRead](#telnetread-function)

---

## TelnetSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI TelnetSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or TELNET_ERROR. Those functions which call TelnetSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the TelnetGetLastError function. The use of TelnetGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetErrorString](#telnetgeterrorstring-function)
- [TelnetGetLastError](#telnetgetlasterror-function)

---

## TelnetSetMode Function

Set the current client mode

### Prototype

```c
UINT WINAPI TelnetSetMode(
    HCLIENT hClient,
    UINT nMode,
    BOOL bEnable
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nMode` | `UINT` | The client mode. This value is a combination of one or more flags which determines how the client handles local character echo and character processing. The following values are recognized: |
| `bEnable` | `BOOL` | This boolean flag specifies if the specified mode is to be enabled or disabled. |

### Options

- **nMode** Parameter

    - `TELNET_MODE_LOCALECHO`: The local client is responsible for echoing data entered by the user. By default, this mode is not set which means that the server is responsible for echoing back each character written to it.
    - `TELNET_MODE_BINARY`: Data exchanged between the client and server should not be converted or line buffered. If this option is not specified, the high-bit will be cleared on all characters, and single linefeed characters will be converted to carriage-return/linefeed sequences.

### Return Value

- **Type**: `UINT`
- If the function succeeds, the return value is the previous mode. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetMode](#telnetgetmode-function)

---

## TelnetSetTerminalType Function

Set the current terminal type

### Prototype

```c
INT WINAPI TelnetSetTerminalType(
    HCLIENT hClient,
    LPCTSTR lpszTermType
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszTermType` | `LPCTSTR` | Points to a string which specifies the terminal type. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetGetTerminalType](#telnetgetterminaltype-function)

---

## TelnetSetTimeout Function

Set the number of seconds until an operation times out

### Prototype

```c
INT WINAPI TelnetSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds to wait for a blocking operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- The timeout value is only used with blocking client connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetGetTimeout](#telnetgettimeout-function)

---

## TelnetUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI TelnetUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful TelnetInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call TelnetUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to TelnetUninitialize for every successful call to TelnetInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetDisconnect](#telnetdisconnect-function)
- [TelnetInitialize](#telnetinitialize-function)

---

## TelnetWrite Function

Write data to the server

### Prototype

```c
INT WINAPI TelnetWrite(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | The pointer to the buffer which contains the data that is to be sent to the server. |
| `cbBuffer` | `INT` | The number of bytes to send from the specified buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually written. If the function fails, the return value is TELNET_ERROR. To get extended error information, call TelnetGetLastError.

### Remarks

- The return value may be less than the number of bytes specified by the cbBuffer parameter. In this case, the data has been partially written and it is the responsibility of the client application to send the remaining data at some later point. For non-blocking clients, the client must wait for the TELNET_EVENT_WRITE asynchronous notification message before it resumes sending data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`

### See Also

- [TelnetRead](#telnetread-function)

---

## TelnetWriteLine Function

Write a line of text to the server

### Prototype

```c
BOOL WINAPI TelnetWriteLine(
    HCLIENT hClient,
    LPCTSTR lpszBuffer,
    LPINT lpnLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszBuffer` | `LPCTSTR` | The pointer to a string buffer which contains the data that will be sent to the server. All characters up to, but not including, the terminating null character will be written to the server. The data will always be terminated with a carriage-return and linefeed control character sequence. If this parameter points to an empty string or NULL pointer, then a only a carriage-return and linefeed are written to the server. |
| `lpnLength` | `LPINT` | A pointer to an integer value which will contain the number of characters written to the server, including the carriage-return and linefeed sequence. If this information is not required, a NULL pointer may be specified. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call TelnetGetLastError.

### Remarks

- The TelnetWriteLine function writes a line of text to the server and terminates the line with a carriage-return and linefeed control character sequence. Unlike the TelnetWrite function which writes arbitrary bytes of data to the server, this function is specifically designed to write a single line of text data from a string.
- If the lpszBuffer string is terminated with a linefeed (LF) or carriage return (CR) character, it will be automatically converted to a standard CRLF end-of-line sequence. Because the string will be sent with a terminating CRLF sequence, the value returned in the lpnLength parameter will typically be larger than the original string length (reflecting the additional CR and LF characters), unless the string was already terminated with CRLF.
- There are some limitations when using TelnetWriteLine. The function should only be used to send text, never binary data. In particular, the function will discard nulls and append linefeed and carriage return control characters to the data stream. The Unicode version of this function will accept a Unicode string, however this function does not support sending raw Unicode data to the server. Unicode strings will be automatically converted to UTF-8 encoding using the WideCharToMultiByte function and then written as a stream of bytes.
- This function will force the thread to block until the complete line of text has been written, the write operation times out or the server aborts the connection. If this function is called with asynchronous events enabled, it will automatically switch the client into a blocking mode, send the data and then restore the client to asynchronous operation. If another network operation is attempted while TelnetWriteLine is blocked sending data to the server, an error will occur. It is recommended that this function only be used with blocking (synchronous) connections; if the application needs to establish multiple simultaneous connections, it should create worker threads to manage each connection.
- The TelnetWrite and TelnetWriteLine function calls can be safely intermixed.
- Unlike the TelnetWrite function, it is possible for data to have been written to the server if the return value is zero. For example, if a timeout occurs while the function is waiting to send more data to the server, it will return zero; however, some data may have already been written prior to the error condition. If this is the case, the lpnLength argument will specify the number of characters actually written up to that point.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstntv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [TelnetIsWritable](#telnetiswritable-function)
- [TelnetRead](#telnetread-function)
- [TelnetReadLine](#telnetreadline-function)
- [TelnetWrite](#telnetwrite-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYCREDENTIALS Structure

The SECURITYCREDENTIALS structure specifies the information needed by the library to specify additional security credentials, such as a client certificate, when establishing a secure connection.

### Definition

```c
typedef struct _SECURITYCREDENTIALS
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwOptions;
    DWORD dwReserved;
    LPCTSTR lpszHostName;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
    LPCTSTR lpszCertStore;
    LPCTSTR lpszCertName;
    LPCTSTR lpszKeyFile;
} SECURITYCREDENTIALS, *LPSECURITYCREDENTIALS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. If the structure is being allocated dynamically, this member must be set to the size of the structure and all other unused structure members must be initialized to a value of zero or NULL. |
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. The value of this structure member is constructed by using a bitwise operator with any of the following values: |
| `dwOptions` | `DWORD` | A value which specifies one or options. This member is constructed by using a bitwise operator with any of the following values: |
| `dwReserved` | `DWORD` | This structure member is reserved for future use and should always be initialized to zero. |
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the hostname that will be used when validating the server certificate. If this member is NULL, then the server certificate will be validated against the hostname used to establish the connection. |
| `lpszUserName` | `LPCTSTR` | A pointer to a null terminated string which identifies the owner of client certificate. Currently this member is not used by the library and should always be initialized as a NULL pointer. |
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string which specifies the password needed to access the certificate. Currently this member is only used if the CREDENTIAL_STORE_FILENAME option has been specified. If there is no password associated with the certificate, then this member should be initialized as a NULL pointer. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate store to open. A certificate store is a collection of certificates and their private keys, typically organized by how they are used. If this value is NULL or points to an empty string, the default certificate store "MY" will be used. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. The library will first search the certificate store for a certificate with a matching "friendly name"; this is a name for the certificate that is assigned by the user. Note that the name must match completely, but the comparison is not case sensitive. If no matching certificate is found, the library will then attempt to find a certificate that has a matching common name (also called the certificate subject). This comparison is less stringent, and the first partial match will be returned. If this second search fails, the library will return an error indicating that the certificate could not be found. If the SECURITY_PROTOCOL_SSH protocol has been specified, this member should be NULL. |
| `lpszKeyFile` | `LPCTSTR` | This member is not used and should always be NULL. The private key cannot be specified separately and must be associated with the certificate. |

### Remarks

- An application only needs to create this structure if the server requires the client to provide a certificate as part of the process of negotiating the secure session. If a certificate is required, note that it must have a private key associated with it. Attempting to use a certificate that does not have a private key will result in an error during the connection process indicating that the client credentials could not be established.
- If you are using a local certificate store, with the certificate and private key stored in the registry, you can explicitly specify whether the certificate store for the current user or the local machine (all users) should be used. This is done by prefixing the certificate store name with "HKCU:" for the current user, or "HKLM:" for the local machine. For example, a certificate store name of "HKLM:MY" would specify the personal certificate store for the local machine, rather than the current user. If neither prefix is specified, then it will default to the certificate store for the current user. You can manage these certificates using the CertMgr.msc Microsoft Management Console (MMC) snap-in.
- It is possible to load the certificate from a file rather than from current user's certificate store. The dwOptions member should be set to CREDENTIAL_STORE_FILENAME and the lpszCertStore member should specify the name of the file that contains the certificate and its private key. The file must be in Private Information Exchange (PFX) format, also known as PKCS #12. These certificate files typically have an extension of .pfx or .p12. Note that if a password was specified when the certificate file was created, it must be provided in the lpszPassword member or the library will be unable to access the certificate.
- Note that the lpszUserName and lpszPassword members are values which are used to access the certificate store or private key file. They are not the credentials which are used when establishing the connection with the remote host or authenticating the client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYINFO Structure

The SECURITYINFO structure contains security related information about the connection, including the protocol, cipher suite and certificate.

### Definition

```c
typedef struct _SECURITYINFO
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwCipher;
    DWORD dwCipherStrength;
    DWORD dwHash;
    DWORD dwHashStrength;
    DWORD dwKeyExchange;
    DWORD dwCertStatus;
    SYSTEMTIME stCertIssued;
    SYSTEMTIME stCertExpires;
    LPCTSTR lpszCertIssuer;
    LPCTSTR lpszCertSubject;
    LPCTSTR lpszFingerprint;
} SECURITYINFO, *LPSECURITYINFO;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Specifies the size of the data structure. This member must always be initialized to the size of the structure in bytes prior to passing the address of this structure to the function. If this this member is not initialized, an error will be returned indicating that an invalid parameter has been passed to the function. |
| `dwProtocol` | `DWORD` | A numeric value which specifies the security protocol selected when establishing the connection. |
| `dwCipher` | `DWORD` | A numeric value which specifies the cipher that was selected when establishing the secure connection. |
| `dwCipherStrength` | `DWORD` | A numeric value which specifies the strength (the length of the cipher key in bits) of the cipher that was selected. Typically this value will be 128 or 256. 40-bit and 56-bit key lengths are considered weak encryption, and subject to brute force attacks. 128-bit and 256-bit key lengths are considered to be secure, and are the recommended key length for secure communications. |
| `dwHash` | `DWORD` | A numeric value which specifies the hash algorithm which was selected. |
| `dwHashStrength` | `DWORD` | A numeric value which specifies the strength (the length in bits) of the message digest that was selected. |
| `dwKeyExchange` | `DWORD` | A numeric value which specifies the key exchange algorithm which was selected. |
| `dwCertStatus` | `DWORD` | A numeric value which specifies the status of the certificate returned by the secure server. |
| `stCertIssued` | `SYSTEMTIME` | A structure which contains the date and time that the certificate was issued by the certificate authority. If the issue date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `stCertExpires` | `SYSTEMTIME` | A structure which contains the date and time that the certificate expires. If the expiration date cannot be determined for the certificate, the SYSTEMTIME structure members will have zero values. |
| `lpszCertIssuer` | `LPCTSTR` | A pointer to a string which contains information about the organization that issued the certificate. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate issuer could not be determined, this member may be NULL. |
| `lpszCertSubject` | `LPCTSTR` | A pointer to a string which contains information about the organization that the certificate was issued to. This string consists of one or more comma-separated tokens. Each token consists of an identifier, followed by an equal sign and a value. If the certificate subject could not be determined, this member may be NULL. |
| `lpszFingerprint` | `LPCTSTR` | A pointer to a string which contains a sequence of hexadecimal values that uniquely identify the server. This member may be NULL. |

### Remarks

- The size of this structure is typically determined using the sizeof operator in C/C++ and most other languages have a similar method for obtaining the size of a data structure or user-defined type. It is important to note the size of this structure will be different on 32-bit and 64-bit platforms due to the difference in the length of the structure members which are pointers.
- When evaluating structure members which are string pointers, such as lpszCertIssuer and lpszCertSubject, you should always check to ensure their values are not NULL (zero) before attempting to dereference them. If those values are undefined, it is not guaranteed they will point to zero-length strings.
- There are additional protocols and ciphers defined which are not listed above because they are considered insecure and are no longer supported. It is important to note that most servers will reject attempts to use unsupported protocols such as SSL 3.0 and will either abort the connection or issue an error message and then immediately disconnect.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

