# Secure Shell Protocol

## Functions

- [SshAsyncConnect](#sshasyncconnect-function)
- [SshAttachThread](#sshattachthread-function)
- [SshCancel](#sshcancel-function)
- [SshConnect](#sshconnect-function)
- [SshControl](#sshcontrol-function)
- [SshCreateSecurityCredentials](#sshcreatesecuritycredentials-function)
- [SshDeleteSecurityCredentials](#sshdeletesecuritycredentials-function)
- [SshDisableEvents](#sshdisableevents-function)
- [SshDisableTrace](#sshdisabletrace-function)
- [SshDisconnect](#sshdisconnect-function)
- [SshEnableEvents](#sshenableevents-function)
- [SshEnableTrace](#sshenabletrace-function)
- [SshEventProc](#ssheventproc-function)
- [SshExecute](#sshexecute-function)
- [SshFreezeEvents](#sshfreezeevents-function)
- [SshGetErrorString](#sshgeterrorstring-function)
- [SshGetExitCode](#sshgetexitcode-function)
- [SshGetLastError](#sshgetlasterror-function)
- [SshGetLineMode](#sshgetlinemode-function)
- [SshGetSecurityInformation](#sshgetsecurityinformation-function)
- [SshGetStatus](#sshgetstatus-function)
- [SshGetTimeout](#sshgettimeout-function)
- [SshInitialize](#sshinitialize-function)
- [SshIsBlocking](#sshisblocking-function)
- [SshIsConnected](#sshisconnected-function)
- [SshIsReadable](#sshisreadable-function)
- [SshIsWritable](#sshiswritable-function)
- [SshPeek](#sshpeek-function)
- [SshRead](#sshread-function)
- [SshReadLine](#sshreadline-function)
- [SshRegisterEvent](#sshregisterevent-function)
- [SshSearch](#sshsearch-function)
- [SshSetLastError](#sshsetlasterror-function)
- [SshSetLineMode](#sshsetlinemode-function)
- [SshSetTimeout](#sshsettimeout-function)
- [SshUninitialize](#sshuninitialize-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)
## Structures

- [INITDATA](#initdata-structure)
- [SECURITYCREDENTIALS](#securitycredentials-structure)
- [SECURITYINFO](#securityinfo-structure)
- [SSHOPTIONDATA](#sshoptiondata-structure)

---

## SshAsyncConnect Function

Connect asynchronously to the specified server

### Prototype

```c
HCLIENT WINAPI SshAsyncConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    UINT nTimeout,
    DWORD dwOptions,
    LPSSHOPTIONDATA lpOptions,
    LPSECURITYCREDENTIALS lpCredentials,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to a string which specifies the name of the server to connect to. This may either be a fully-qualified domain name or an IP address. This parameter cannot be NULL. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number 22 should be used. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the user name which will be used to authenticate the client session. This parameter must specify a valid user name and cannot be NULL or an empty string. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password which will be used to authenticate the client session. If the user does not have a password, this parameter can be NULL or an empty string. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpOptions` | `LPSSHOPTIONDATA` | A pointer to a SSHOPTIONDATA structure which specifies additional information for one or more options. If no optional data is required, a NULL pointer may be specified. |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which specifies additional security-related information required to establish the connection. This parameter may be NULL, in which case default values will be used. Note that the dwSize member must be initialized to the size of the SECURITYCREDENTIALS structure that is being passed to the function. You can use the SshCreateSecurityCredentials function to create this structure. |
| `hEventWnd` | `HWND` | The handle to the event notification window. This window receives messages which notify the client of various asynchronous socket events that occur. |
| `uEventMsg` | `UINT` | The message identifier that is used when an asynchronous socket event occurs. This value should be greater than WM_USER as defined in the Windows header files. |

### Options

- **dwOptions** Parameter

    - `SSH_OPTION_NONE`: No options specified. A standard terminal session will be established with the default terminal type.
    - `SSH_OPTION_KEEPALIVE`: This option specifies the library should attempt to maintain an idle client session for long periods of time. This option is only necessary if you expect that the connection will be held open for an extended period of time. The actual client idle timeout period is determined by the server configuration.
    - `SSH_OPTION_NOPTY`: This option specifies that a pseudoterminal (PTY) should not be created for the client session. This option is automatically set if the SSH_OPTION_COMMAND option has been specified. This option should never be used with interactive client sessions.
    - `SSH_OPTION_NOSHELL`: This option specifies that a command shell should not be used when executing a command on the server. You should not use this option unless you require the command to execute without the normal shell environment. In most cases you should not use this option because it may cause commands to fail unexpectedly.
    - `SSH_OPTION_TERMINAL`: This option specifies the client session will use terminal emulation and the SSHOPTIONDATA structure specifies the characteristics of the virtual terminal. This enables the caller to specify the dimensions of the virtual display (in columns and rows) and the type of terminal that will be emulated. If this option is omitted, the session will default to a virtual display that is 80 columns, 25 rows.
    - `SSH_OPTION_COMMAND`: This option specifies the client session will be used to issue a command that is executed on the server, and the output will be returned to the caller. If this option is specified, the session will not be interactive and no pseudoterminal is created for the client. The szCommandLine member of the SSHOPTIONDATA structure specifies the command string that will be sent to the server.
    - `SSH_OPTION_PROXYSERVER`: This option specifies the client should establish a connection through a proxy server. The two protocols that are supported are SSH_PROXY_HTTP and SSH_PROXY_TELNET, which specifies the protocol that the proxy connection is created through. The proxy-related members of the SSHOPTIONDATA structure should be set to the appropriate values.
    - `SSH_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `SSH_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call SshGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Example

```cpp
// Define the asynchronous event notification message ID
#define WM_CLIENT_EVENTS (WM_APP + 1)

HCLIENT hClient;
SSHOPTIONDATA sshOptions;

// Initialize the SSHOPTIONDATA structure and specify the
// command that should be executed on the server
ZeroMemory(&sshOptions, sizeof(sshOptions));
lstrcpyn(sshOptions.szCommandLine, lpszCommand, SSH_MAXCOMMANDLEN);

// Establish a connection with the SSH server

hClient = SshAsyncConnect(lpszHostName,
						SSH_PORT_DEFAULT,
						lpszUserName,
						lpszPassword,
						SSH_TIMEOUT,
						SSH_OPTION_COMMAND,
						&sshOptions,
						NULL,
						hAppWnd,
						WM_CLIENT_EVENTS);

// If the connection attempt fails, then get a description of
// the error and display it in a message box

if (hClient == INVALID_CLIENT)
{
	DWORD dwError;
	TCHAR szError[128];

	dwError = SshGetLastError();
	if (dwError > 0)
	{
		SshGetErrorString(dwError, szError, 128);
		MessageBox(NULL, szError, _T("Error"), MB_ICONEXCLAMATION);
	}

	return;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshConnect](#sshconnect-function)
- [SshCreateSecurityCredentials](#sshcreatesecuritycredentials-function)
- [SshDeleteSecurityCredentials](#sshdeletesecuritycredentials-function)
- [SshDisconnect](#sshdisconnect-function)
- [SshInitialize](#sshinitialize-function)
- [SshUninitialize](#sshuninitialize-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)
- [SSHOPTIONDATA](#sshoptiondata-function)

---

## SshAttachThread Function

Attach the specified client handle to another thread

### Prototype

```c
DWORD WINAPI SshAttachThread(
    HCLIENT hClient,
    DWORD dwThreadId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwThreadId` | `DWORD` | The ID of the thread that will become the new owner of the handle. A value of zero specifies that the current thread should become the owner of the client handle. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the thread ID of the previous owner. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- When a client handle is created, it is associated with the current thread that created it. Normally, if another thread attempts to perform an operation using that handle, an error is returned since it does not own the handle. This is used to ensure that other threads cannot interfere with an operation being performed by the owner thread. In some cases, it may be desirable for one thread in a client application to create the client handle, and then pass that handle to another worker thread. The SshAttachThread function can be used to change the ownership of the handle to the new worker thread. By preserving the return value from the function, the original owner of the handle can be restored before the worker thread terminates.
- This function should be called by the new thread immediately after it has been created, and if the new thread does not release the handle itself, the ownership of the handle should be restored to the parent thread before it terminates. Under no circumstances should SshAttachThread be used to forcibly release a handle allocated by another thread while a blocking operation is in progress. To cancel an operation, use the SshCancel function and then release the handle after the blocking function exits and control is returned to the current thread.
- Note that the dwThreadId parameter is presumed to be a valid thread ID and no checks are performed to ensure that the thread actually exists. Specifying an invalid thread ID will orphan the handle until the SshUninitialize function is called.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshConnect](#sshconnect-function)
- [SshDisconnect](#sshdisconnect-function)
- [SshUninitialize](#sshuninitialize-function)

---

## SshCancel Function

Cancel the current blocking operation

### Prototype

```c
INT WINAPI SshCancel(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- When the SshCancel function is called, the blocking function will not immediately fail. An internal flag is set which causes the blocking operation to exit with an error. This means that the application cannot cancel an operation and immediately perform some other operation. Instead it must allow the calling stack to unwind, returning back to the blocking operation before making any further function calls.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshControl](#sshcontrol-function)
- [SshIsBlocking](#sshisblocking-function)

---

## SshConnect Function

Connect to the specified server

### Prototype

```c
HCLIENT WINAPI SshConnect(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    UINT nTimeout,
    DWORD dwOptions,
    LPSSHOPTIONDATA lpOptions,
    LPSECURITYCREDENTIALS lpCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to a string which specifies the name of the server to connect to. This may either be a fully-qualified domain name or an IP address. This parameter cannot be NULL. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number 22 should be used. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the user name which will be used to authenticate the client session. This parameter must specify a valid user name and cannot be NULL or an empty string. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password which will be used to authenticate the client session. If the user does not have a password, this parameter can be NULL or an empty string. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpOptions` | `LPSSHOPTIONDATA` | A pointer to a SSHOPTIONDATA structure which specifies additional information for one or more options. If no optional data is required, a NULL pointer may be specified. |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which specifies additional security-related information required to establish the connection. This parameter may be NULL, in which case default values will be used. Note that the dwSize member must be initialized to the size of the SECURITYCREDENTIALS structure that is being passed to the function. You can use the SshCreateSecurityCredentials function to create this structure. |

### Options

- **dwOptions** Parameter

    - `SSH_OPTION_NONE`: No options specified. A standard terminal session will be established with the default terminal type.
    - `SSH_OPTION_KEEPALIVE`: This option specifies the library should attempt to maintain an idle client session for long periods of time. This option is only necessary if you expect that the connection will be held open for an extended period of time. The actual client idle timeout period is determined by the server configuration.
    - `SSH_OPTION_NOPTY`: This option specifies that a pseudoterminal (PTY) should not be created for the client session. This option is automatically set if the SSH_OPTION_COMMAND option has been specified. This option should never be used with interactive client sessions.
    - `SSH_OPTION_NOSHELL`: This option specifies that a command shell should not be used when executing a command on the server. You should not use this option unless you require the command to execute without the normal shell environment. In most cases you should not use this option because it may cause commands to fail unexpectedly.
    - `SSH_OPTION_TERMINAL`: This option specifies the client session will use terminal emulation and the SSHOPTIONDATA structure specifies the characteristics of the virtual terminal. This enables the caller to specify the dimensions of the virtual display (in columns and rows) and the type of terminal that will be emulated. If this option is omitted, the session will default to a virtual display that is 80 columns, 25 rows.
    - `SSH_OPTION_COMMAND`: This option specifies the client session will be used to issue a command that is executed on the server, and the output will be returned to the caller. If this option is specified, the session will not be interactive and no pseudoterminal is created for the client. The szCommandLine member of the SSHOPTIONDATA structure specifies the command string that will be sent to the server.
    - `SSH_OPTION_PROXYSERVER`: This option specifies the client should establish a connection through a proxy server. The two protocols that are supported are SSH_PROXY_HTTP and SSH_PROXY_TELNET, which specifies the protocol that the proxy connection is created through. The proxy-related members of the SSHOPTIONDATA structure should be set to the appropriate values.
    - `SSH_OPTION_PREFER_IPV6`: This option specifies the client should prefer the use of IPv6 if the server hostname can be resolved to both an IPv6 and IPv4 address. This option is ignored if the local system does not have IPv6 enabled, or when the hostname can only be resolved to an IPv4 address. If the server hostname can only be resolved to an IPv6 address, the client will attempt to establish a connection using IPv6 regardless if this option has been specified.
    - `SSH_OPTION_FREETHREAD`: This option specifies the handle returned by this function may be used by any thread, and is not limited to the thread which created it. The application is responsible for ensuring that access to the handle is synchronized across multiple threads.

### Return Value

- **Type**: `HCLIENT`
- If the function succeeds, the return value is a handle to a client session. If the function fails, the return value is INVALID_CLIENT. To get extended error information, call SshGetLastError.

### Remarks

- This function will block the current thread until a connection has been established or the timeout period has elapsed. To prevent it from blocking the main user interface thread, the application should create a background worker thread and establish a connection by calling SshConnect in that thread. If the application requires multiple simultaneous connections, it is recommended you create a worker thread for each connection.
- By default, the client session will be authenticated using standard password authentication. If you prefer to authenticate using a private key file, you must create a SECURITYCREDENTIALS structure which specifies the path to the private key and optionally the password used when creating it. The private key file must be in the standard PEM format defined in RFC 1422. Both the standard RSA private key format and the proprietary OpenSSH format are supported. To determine how the private key was created, you can open the file in a text editor and the first line of the file will identify the format.
- The dwOptions argument can be used to specify the threading model that is used by the library when a connection is established. By default, the handle is initially attached to the thread that created it. From that point on, until the it is released, only the owner may call functions using that handle. The ownership of the handle may be transferred from one thread to another using the SshAttachThread function.
- Specifying the SSH_OPTION_FREETHREAD option enables any thread to call any function using the handle, regardless of which thread created it. It is important to note that this option disables certain internal safety checks which are performed by the library and may result in unexpected behavior unless access to the handle is synchronized. If one thread calls a function in the library, it must ensure that no other thread will call another function at the same time using the same handle.

### Example

```cpp
HCLIENT hClient;
SSHOPTIONDATA sshOptions;

// Initialize the SSHOPTIONDATA structure and specify the
// command that should be executed on the server
ZeroMemory(&sshOptions, sizeof(sshOptions));
lstrcpyn(sshOptions.szCommandLine, lpszCommand, SSH_MAXCOMMANDLEN);

// Establish a connection with the SSH server

hClient = SshConnect(lpszHostName,
					SSH_PORT_DEFAULT,
					lpszUserName,
					lpszPassword,
					SSH_TIMEOUT,
					SSH_OPTION_COMMAND,
					&sshOptions,
					NULL);

// If the connection attempt fails, then get a description of
// the error and display it in a message box

if (hClient == INVALID_CLIENT)
{
	DWORD dwError;
	TCHAR szError[128];

	dwError = SshGetLastError();
	if (dwError > 0)
	{
		SshGetErrorString(dwError, szError, 128);
		MessageBox(NULL, szError, _T("Error"), MB_ICONEXCLAMATION);
	}

	return;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshCreateSecurityCredentials](#sshcreatesecuritycredentials-function)
- [SshDeleteSecurityCredentials](#sshdeletesecuritycredentials-function)
- [SshDisconnect](#sshdisconnect-function)
- [SshExecute](#sshexecute-function)
- [SshInitialize](#sshinitialize-function)
- [SshUninitialize](#sshuninitialize-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)
- [SSHOPTIONDATA](#sshoptiondata-function)

---

## SshControl Function

Send a control sequence to the server

### Prototype

```c
INT WINAPI SshControl(
    HCLIENT hClient,
    DWORD dwControlCode,
    LPVOID lpvReserved,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `dwControlCode` | `DWORD` | A numeric control code which specifies the control message which should be sent to the server. This may be one of the following values: |
| `lpvReserved` | `LPVOID` | A reserved parameter which should always be specified as NULL. |
| `dwReserved` | `DWORD` | A reserved parameter which should always be specified as a value of 0. |

### Options

- **dwControlCode** Parameter

    - `SSH_CONTROL_BREAK`: Sends a control message to the server which simulates a break signal on a physical terminal. This is used by some operating systems as an instruction to enter a privileged configuration mode. Note that this is not the same as sending an interrupt character such as Ctrl+C to the server. This control code is ignored for SSH 1.0 sessions.
    - `SSH_CONTROL_NOOP`: Sends a control message to the server, but it does not perform any operation. This is typically used by clients to prevent the server from automatically closing a session that has been idle for a long period of time.
    - `SSH_CONTROL_EOF`: Sends a control message to the server indicating that the client has finished sending data. Note that this option is normally not used with interactive terminal sessions, and should only be used when required by the server.
    - `SSH_CONTROL_PING`: Sends a control message to the server which is used to test whether or not the server is responsive to the client. This is typically used by clients to attempt to detect if the connection to the server is still active.
    - `SSH_CONTROL_REKEY`: Sends a control message to the server requesting that the key exchange be performed again. This control code is ignored for SSH 1.0 sessions.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The SshControl function enables an application to send control messages to the server, which can cause it to take specific actions such as simulate a terminal break or request that the key exchanged be performed again. Some control messages are not supported by the SSH 1.0 protocol, in which case the control message is ignored.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshCancel](#sshcancel-function)
- [SshIsBlocking](#sshisblocking-function)

---

## SshCreateSecurityCredentials Function

Create a new security credentials structure

### Prototype

```c
BOOL WINAPI SshCreateSecurityCredentials(
    DWORD dwProtocol,
    DWORD dwOptions,
    LPCTSTR lpszKeyFile,
    LPCTSTR lpszPassword,
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwProtocol` | `DWORD` | A bitmask of supported security protocols. |
| `dwOptions` | `DWORD` | This parameter is reserved for future use and should always be a value of zero. |
| `lpszKeyFile` | `LPCTSTR` | A pointer to a string which specifies the name of a private key file that used when authenticating the client connection. The private key file must be in the standard PEM format defined in RFC 1422. Both the standard RSA private key format and the proprietary OpenSSH format are supported. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password for the private key file. A value of NULL specifies that no password is required. |
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. The memory for the credentials structure will be allocated by this function and must be released by calling the SshDeleteSecurityCredentials function when it is no longer needed. The pointer value must be set to NULL before the function is called. |

### Options

- **dwProtocol** Parameter

    - `SECURITY_PROTOCOL_SSH2`: Version 2.0 of the protocol will be used when establishing the connection. Currently, this is the only supported protocol option for SSH connections. In older versions of SocketTools, you had the option of selecting either version 1.0 or 2.0; however, version 1.0 is no longer considered secure and is not supported.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SshGetLastError.

### Remarks

- This function is only required if you wish to authenticate the client session using a private key file instead of an account password.
- The lpszKeyFile path can use environment variables enclosed in percent symbols, and the path to the private key file will be normalized. It is recommended you always use an absolute path to the private key file. If you do not include a path, it will use the current working directory for the process. This can produce inconsistent results with multithreaded applications because the current working directory for a process is a global value and it can be changed by any thread at any time.
- The structure that is created by this function may be used as client credentials when establishing a secure connection. This is particularly useful for programming languages other than C/C++ which may not support C structures or pointers. The pointer to the SECURITYCREDENTIALS structure can be declared as an unsigned integer variable which is passed by reference to this function, and then passed by value to the SshConnect function.
- It is safe to call SshDeleteSecurityCredentials after the connection has been established. An internal copy of the credentials are created during the connection process and the pointer to the credentials structure allocated by this function are no longer needed.

### Example

```cpp
HCLIENT hClient = INVALID_CLIENT;
LPSECURITYCREDENTIALS lpCredentials = NULL;

// Define the security credentials
SshCreateSecurityCredentials(SECURITY_PROTOCOL_SSH2,
							SSH_OPTION_NONE,
							_T("%USERPROFILE%\\.ssh\\id_rsa.pem"),
							NULL,
							&lpCredentials);

if (lpCredentials == NULL)
{
	// Something went wrong
	return;
}

// Establish a connection with the SSH server on port 22
hClient = SshConnect(lpszHostName,
					SSH_PORT_DEFAULT,
					lpszUserName,
					NULL, // No password, using a private key file
					SSH_TIMEOUT,
					SSH_OPTION_COMMAND,
					NULL,
					lpCredentials);

// Delete the credentials
SshDeleteSecurityCredentials(&lpCredentials);

if (hClient == INVALID_CLIENT)
{
	DWORD dwError;
	TCHAR szError[128];

	dwError = SshGetLastError();
	if (dwError > 0)
	{
		SshGetErrorString(dwError, szError, 128);
		MessageBox(NULL, szError, _T("Error"), MB_ICONEXCLAMATION);
	}

	return;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshConnect](#sshconnect-function)
- [SshDeleteSecurityCredentials](#sshdeletesecuritycredentials-function)
- [SshGetSecurityInformation](#sshgetsecurityinformation-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)

---

## SshDeleteSecurityCredentials Function

Delete a previously created security credentials structure

### Prototype

```c
VOID WINAPI SshDeleteSecurityCredentials(
    LPSECURITYCREDENTIALS lppCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lppCredentials` | `LPSECURITYCREDENTIALS` | Pointer to an LPSECURITYCREDENTIALS pointer. On exit from the function, the pointer will be NULL. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- This function releases the memory allocated for the client credentials created by the SshCreateSecurityCredentials function. It is safe to call this function immediately after the connection has been established. An internal copy of the credentials are allocated during the connection process and the pointer to the credentials structure is no longer needed.
- You should never call this function using a pointer to a credentials structure you have not explicitly allocated using the SshCreateSecurityCredentials function. Attempting to do so will corrupt the process heap and in most cases will throw an unhandled exception, causing your application to terminate.

### Example

```cpp
HCLIENT hClient = INVALID_CLIENT;
LPSECURITYCREDENTIALS lpCredentials = NULL;

// Define the security credentials using a private key file
SshCreateSecurityCredentials(SECURITY_PROTOCOL_SSH2,
							SSH_OPTION_NONE,
							_T("%USERPROFILE%\\.ssh\\id_rsa.pem"),
							NULL,
							&lpCredentials);

if (lpCredentials == NULL)
{
	// Something went wrong
	return;
}

// Establish a connection with the SSH server on port 22
hClient = SshConnect(lpszHostName,
					SSH_PORT_DEFAULT,
					lpszUserName,
					NULL, // No password, using a private key file
					SSH_TIMEOUT,
					SSH_OPTION_COMMAND,
					NULL,
					lpCredentials);

// Delete the credentials
SshDeleteSecurityCredentials(&lpCredentials);

if (hClient == INVALID_CLIENT)
{
	DWORD dwError;
	TCHAR szError[128];

	dwError = SshGetLastError();
	if (dwError > 0)
	{
		SshGetErrorString(dwError, szError, 128);
		MessageBox(NULL, szError, _T("Error"), MB_ICONEXCLAMATION);
	}

	return;
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshCreateSecurityCredentials](#sshcreatesecuritycredentials-function)
- [SshUninitialize](#sshuninitialize-function)

---

## SshDisableEvents Function

Disable asynchronous event notification

### Prototype

```c
INT WINAPI SshDisableEvents(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- This function affects both event notification and event callbacks. Any outstanding events in the message queue should be ignored by the client application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshEnableEvents](#sshenableevents-function)
- [SshFreezeEvents](#sshfreezeevents-function)
- [SshRegisterEvent](#sshregisterevent-function)

---

## SshDisableTrace Function

Disable logging of network function calls to the trace log

### Prototype

```c
BOOL WINAPI SshDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshEnableTrace](#sshenabletrace-function)

---

## SshDisconnect Function

Disconnect from the current server

### Prototype

```c
INT WINAPI SshDisconnect(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshConnect](#sshconnect-function)
- [SshUninitialize](#sshuninitialize-function)

---

## SshEnableEvents Function

Enable asynchronous event notification

### Prototype

```c
INT WINAPI SshEnableEvents(
    HCLIENT hClient,
    HWND hEventWnd,
    UINT uEventMsg
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `hEventWnd` | `HWND` | Handle to the event notification window. This window receives a user-defined message which specifies the event that has occurred. If this value is NULL, event notification is disabled. |
| `uEventMsg` | `UINT` | An unsigned integer which specifies the user-defined message that is sent when an event occurs. This parameter's value must be greater than the value of WM_USER. If the hEventWnd parameter is NULL, this value must be specified as WM_NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- When a message is posted to the notification window, the low word of the lParam parameter contains the event identifier. The high word of lParam contains the low word of the error code, if an error has occurred. The wParam parameter contains the client handle. One or more of the following event identifiers may be sent:

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshDisableEvents](#sshdisableevents-function)
- [SshFreezeEvents](#sshfreezeevents-function)
- [SshRegisterEvent](#sshregisterevent-function)

---

## SshEnableTrace Function

Enable logging of network function calls to a file

### Prototype

```c
BOOL WINAPI SshEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshDisableTrace](#sshdisabletrace-function)

---

## SshEventProc Function

Callback function that processes events generated by the client

### Prototype

```c
VOID __cdecl SshEventProc(
    HCLIENT hClient,
    UINT nEvent,
    DWORD dwError,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event occurred. For a complete list of events, refer to the SshRegisterEvent function. |
| `dwError` | `DWORD` | An unsigned integer which specifies any error that occurred. If no error occurred, then this parameter will be zero. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value which was specified when the event callback was registered. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application must register this callback function by passing its address to the SshRegisterEvent function. The SshEventProc function is a placeholder for the application-defined function name.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshDisableEvents](#sshdisableevents-function)
- [SshEnableEvents](#sshenableevents-function)
- [SshFreezeEvents](#sshfreezeevents-function)
- [SshRegisterEvent](#sshregisterevent-function)

---

## SshExecute Function

Execute a command on a server and return the output in the specified buffer

### Prototype

```c
INT WINAPI SshExecute(
    LPCTSTR lpszRemoteHost,
    UINT nRemotePort,
    LPCTSTR lpszUserName,
    LPCTSTR lpszPassword,
    UINT nTimeout,
    DWORD dwOptions,
    LPVOID lpvBuffer,
    LPDWORD lpdwLength,
    LPSECURITYCREDENTIALS lpCredentials
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszRemoteHost` | `LPCTSTR` | A pointer to a string which specifies the name of the server. This may either be a fully-qualified domain name, or an IP address. This parameter cannot be NULL. |
| `nRemotePort` | `UINT` | The port number the server is listening on. A value of zero specifies that the default port number 22 should be used. |
| `lpszUserName` | `LPCTSTR` | A pointer to a string which specifies the user name which will be used to authenticate the client session. |
| `lpszPassword` | `LPCTSTR` | A pointer to a string which specifies the password which will be used to authenticate the client session. |
| `nTimeout` | `UINT` | The number of seconds that the client will wait for a response from the server before failing the current operation. |
| `dwOptions` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the file that was downloaded. |
| `lpCredentials` | `LPSECURITYCREDENTIALS` | A pointer to a SECURITYCREDENTIALS structure which specifies additional security-related information required to establish the connection. This parameter may be NULL, in which case default values will be used. Note that the dwSize member must be initialized to the size of the SECURITYCREDENTIALS structure that is being passed to the function. |

### Options

- **dwOptions** Parameter

    - `SSH_OPTION_NONE`: No options specified. A standard terminal session will be established with the default terminal type.
    - `SSH_OPTION_KEEPALIVE`: This option specifies the library should attempt to maintain an idle client session for long periods of time. This option is only necessary if you expect that the connection will be held open for more than two hours.
    - `SSH_OPTION_NOPTY`: This option specifies that a pseudoterminal (PTY) should not be created for the client session. This option is automatically set if the SSH_OPTION_COMMAND option has been specified.
    - `SSH_OPTION_NOSHELL`: This option specifies that a command shell should not be used when executing a command on the server.
    - `SSH_OPTION_NOAUTHRSA`: This option specifies that RSA authentication should not be used with SSH-1 connections. This option is ignored with SSH-2 connections and should only be specified if required by the server.
    - `SSH_OPTION_NOPWDNULL`: This option specifies the user password cannot be terminated with a null character. This option is ignored with SSH-2 connections and should only be specified if required by the server.
    - `SSH_OPTION_NOREKEY`: This option specifies the client should never attempt a repeat key exchange with the server. Some SSH servers do not support rekeying the session, and this can cause the client to become non-responsive or abort the connection after being connected for an hour.
    - `SSH_OPTION_COMPATSID`: This compatibility option changes how the session ID is handled during public key authentication with older SSH servers. This option should only be specified when connecting to servers that use OpenSSH 2.2.0 or earlier versions.
    - `SSH_OPTION_COMPATHMAC`: This compatibility option changes how the HMAC authentication codes are generated. This option should only be specified when connecting to servers that use OpenSSH 2.2.0 or earlier versions.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the exit code from the program that was executed on the server. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The SshExecute function is used to execute a command on a server, read the output from that command and copy it into a local buffer. This function cannot be used if the connection to the server must be established through a proxy server; if a proxy server must be used, then you should use the SshConnect function to establish the connection, and then use either the SshRead or SshReadLine functions to read the output.
- This function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the command output. In this case, the lpvBuffer parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the output when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. See the example code below.
- When the command output is being read from the server, this function will automatically convert the data to match the end-of-line convention used on the Windows platform. This is useful when executing a command on a UNIX based system where the end-of-line is indicated by a single linefeed, while on Windows it is a carriage-return and linefeed pair. If the output contains embedded nulls or escape sequences, then this conversion will not be performed.
- This function will cause the current thread to block until the command completes or a timeout occurs.

### Example

```cpp
HGLOBAL hgblBuffer = (HGLOBAL)NULL;
LPBYTE lpBuffer = (LPBYTE)NULL;
DWORD cbBuffer = 0;

// Execute a command on the server and return the data into block
// of global memory allocated by the GlobalAlloc function; the handle
// to this memory will be returned in the hgblBuffer parameter
nResult = SshExecute(lpszHostName,
					SSH_PORT_DEFAULT,
					lpszUserName,
					lpszPassword,
					lpszCommandLine,
					SSH_TIMEOUT,
					SSH_OPTION_NONE,
					&hgblBuffer,
					&cbBuffer,
					NULL);

if (nResult != SSH_ERROR)
{
	// Lock the global memory handle, returning a pointer to the
	// resource data
	lpBuffer = (LPBYTE)GlobalLock(hgblBuffer);
	
	// After the data has been used, the handle must be unlocked
	// and freed, otherwise a memory leak will occur
	GlobalUnlock(hgblBuffer);
	GlobalFree(hgblBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshConnect](#sshconnect-function)
- [SshGetExitCode](#sshgetexitcode-function)
- [SshRead](#sshread-function)
- [SshReadLine](#sshreadline-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)
- [SECURITYCREDENTIALS](#securitycredentials-function)
- [SSHOPTIONDATA](#sshoptiondata-function)

---

## SshFreezeEvents Function

Suspend asynchronous event processing

### Prototype

```c
INT WINAPI SshFreezeEvents(
    HCLIENT hClient,
    BOOL bFreeze
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `bFreeze` | `BOOL` | A non-zero value specifies that event handling should be suspended by the client. A zero value specifies that event handling should be resumed. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- This function should be used when the application does not want to process events, such as when a modal dialog is being displayed. When events are suspended, all client events are queued. If events are re-enabled at a later point, those queued events will be sent to the application for processing. Note that only one of each event will be generated. For example, if the client has suspended event handling, and four read events occur, once event handling is resumed only one of those read events will be posted to the client. This prevents the application from being flooded by a potentially large number of queued events.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshDisableEvents](#sshdisableevents-function)
- [SshEnableEvents](#sshenableevents-function)
- [SshRegisterEvent](#sshregisterevent-function)

---

## SshGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI SshGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshGetLastError](#sshgetlasterror-function)
- [SshSetLastError](#sshsetlasterror-function)

---

## SshGetExitCode Function

Return the exit code from the remote program

### Prototype

```c
INT WINAPI SshGetExitCode(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the numeric exit code. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- This function should only be called after the command has completed and the SshRead function has returned a value of zero. In most cases, an exit code value of zero indicates success, while any other value indicates an error condition.
- Note that the actual value is application dependent and is only meaningful in the context of that particular program. A program may choose or use exit codes in a non-standard way, such as having certain non-zero values indicate success.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetStatus](#sshgetstatus-function)

---

## SshGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI SshGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the SshSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the SshGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call SshSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or SSH_ERROR. Those functions which call SshSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetErrorString](#sshgeterrorstring-function)
- [SshSetLastError](#sshsetlasterror-function)

---

## SshGetLineMode Function

Return the current mode used to send end-of-line character sequences

### Prototype

```c
INT WINAPI SshGetLineMode(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the current line mode. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The SshGetLineMode function returns an integer value that specifies how end-of-line character sequences are sent to the server. For more information about how newlines are processed by the library and the available options, refer to the SshSetLineMode function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshSetLineMode](#sshsetlinemode-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)

---

## SshGetSecurityInformation Function

Return security information about the current client connection

### Prototype

```c
BOOL WINAPI SshGetSecurityInformation(
    HCLIENT hClient,
    LPSECURITYINFO lpSecurityInfo
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpSecurityInfo` | `LPSECURITYINFO` | A pointer to a SECURITYINFO structure which contains information about the current client connection. The dwSize member of this structure must be initialized to the size of the structure before passing the address of the structure to this function. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SshGetLastError.

### Remarks

- This function is used to obtain security related information about the current client connection to the server. It can be used to determine if a secure connection has been established, what security protocol was selected, and information about the server certificate. Note that a secure connection has not been established, the dwProtocol structure member will contain the value SECURITY_PROTOCOL_NONE.

### Example

```cpp
// Get the server fingerprint
SECURITYINFO securityInfo;

securityInfo.dwSize = sizeof(SECURITYINFO); 
if (SshGetSecurityInformation(hClient, &securityInfo)) 
{ 
	if (securityInfo.lpszFingerprint != NULL)
	{
		TCHAR szMessage[256];
		wsprintf(szMessage, _T("The fingerprint is %s", securityInfo.lpszFingerprint);
		MessageBox(NULL, szMessage, "Connection", MB_OK);
	} 
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshConnect](#sshconnect-function)
- [SshDisconnect](#sshdisconnect-function)
- [SECURITYINFO](#securityinfo-function)

---

## SshGetStatus Function

Return the current client status

### Prototype

```c
INT WINAPI SshGetStatus(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the client status code. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The SshGetStatus function returns a numeric code which identifies the current state of the client session. The following values may be returned:
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified client session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetExitCode](#sshgetexitcode-function)
- [SshIsBlocking](#sshisblocking-function)
- [SshIsConnected](#sshisconnected-function)
- [SshIsReadable](#sshisreadable-function)
- [SshIsWritable](#sshiswritable-function)

---

## SshGetTimeout Function

Return the number of seconds until an operation times out

### Prototype

```c
INT WINAPI SshGetTimeout(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The timeout value is only used with blocking client connections. A value of zero indicates that the default timeout period of 20 seconds should be used.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshSetTimeout](#sshsettimeout-function)

---

## SshInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI SshInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SshGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that SshInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to SshUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the SshInitialize or SshUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshConnect](#sshconnect-function)
- [SshDisconnect](#sshdisconnect-function)
- [SshUninitialize](#sshuninitialize-function)

---

## SshIsBlocking Function

Determine if the client is blocked, waiting for information

### Prototype

```c
BOOL WINAPI SshIsBlocking(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is performing a blocking operation, the function returns a non-zero value. If the client is not performing a blocking operation, or the client handle is invalid, the function returns zero.

### Remarks

- Because a blocking operation can allow the application to be re-entered (for example, by pressing a button while the operation is being performed), it is possible that another blocking function may be called while it is in progress. Since only one thread of execution may perform a blocking operation at any one time, an error would occur. The SshIsBlocking function can be used to determine if the client is already blocked, and if so, take some other action such as warning the user that they must wait for the operation to complete.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshCancel](#sshcancel-function)

---

## SshIsConnected Function

Determine if the client is connected to the server

### Prototype

```c
BOOL WINAPI SshIsConnected(
    HCLIENT hClient
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |

### Return Value

- **Type**: `BOOL`
- If the client is connected to a server, the function returns a non-zero value. If the client is not connected, or the client handle is invalid, the function returns zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshIsBlocking](#sshisblocking-function)
- [SshIsReadable](#sshisreadable-function)
- [SshIsWritable](#sshiswritable-function)

---

## SshIsReadable Function

Determine if data can be read from the server

### Prototype

```c
BOOL WINAPI SshIsReadable(
    HCLIENT hClient,
    INT nTimeout,
    LPDWORD lpdwAvail
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |
| `lpdwAvail` | `LPDWORD` | A pointer to an unsigned integer which will contain the number of bytes available to read. This parameter may be NULL if this information is not required. |

### Return Value

- **Type**: `BOOL`
- If the client can read data from the server within the specified timeout period, the function returns a non-zero value. If the client cannot read any data, the function returns zero.

### Remarks

- On some platforms, this value will not exceed the size of the receive buffer (typically 64K bytes). Because of differences between TCP/IP stack implementations on different versions of Windows, it is not recommended that your application exclusively depend on this value to determine the exact number of bytes available. Instead, it should be used as a general indicator that there is data available to be read.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetStatus](#sshgetstatus-function)
- [SshIsBlocking](#sshisblocking-function)
- [SshIsConnected](#sshisconnected-function)
- [SshIsWritable](#sshiswritable-function)
- [SshRead](#sshread-function)

---

## SshIsWritable Function

Determine if data can be written to the server

### Prototype

```c
BOOL WINAPI SshIsWritable(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | Timeout for server response, in seconds. A value of zero specifies that the connection should be polled without blocking the current thread. |

### Return Value

- **Type**: `BOOL`
- If the client can write data to the server within the specified timeout period, the function returns a non-zero value. If the client cannot write any data, the function returns zero.

### Remarks

- Although this function can be used to determine if some amount of data can be sent to the remote process it does not indicate the amount of data that can be written without blocking the client. In most cases, it is recommended that large amounts of data be broken into smaller logical blocks, typically some multiple of 512 bytes in length.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetStatus](#sshgetstatus-function)
- [SshIsBlocking](#sshisblocking-function)
- [SshIsConnected](#sshisconnected-function)
- [SshIsReadable](#sshisreadable-function)
- [SshWrite](#sshwrite-function)

---

## SshPeek Function

Read data returned by the server, but do not remove it from the receive buffer

### Prototype

```c
INT WINAPI SshPeek(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer in which the data will be copied. |
| `cbBuffer` | `INT` | The maximum number of bytes to read and copy into the specified buffer. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually read. A return value of zero indicates that there is no data available to be read at that time. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The SshPeek function can be used to examine the data that is available to be read from the internal receive buffer. If there is no data in the receive buffer at that time, a value of zero is returned. It should be noted that this differs from the SshRead function, where a return value of zero indicates that there is no more data available to be read and the connection has been closed. The SshPeek function will never cause the client to block, and so may be safely used with asynchronous connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshIsReadable](#sshisreadable-function)
- [SshRead](#sshread-function)
- [SshSearch](#sshsearch-function)
- [SshWrite](#sshwrite-function)

---

## SshRead Function

Read data returned by the server

### Prototype

```c
INT WINAPI SshRead(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | Pointer to the buffer in which the data will be copied. |
| `cbBuffer` | `INT` | The maximum number of bytes to read and copy into the specified buffer. This value must be greater than zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually read. A return value of zero indicates that the server has closed the connection and there is no more data available to be read. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- When SshRead is called and the client is in non-blocking mode, it is possible that the function will fail because there is no available data to read at that time. This should not be considered a fatal error. Instead, the application should simply wait to receive the next asynchronous notification that data is available.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshIsReadable](#sshisreadable-function)
- [SshPeek](#sshpeek-function)
- [SshReadLine](#sshreadline-function)
- [SshSearch](#sshsearch-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)

---

## SshReadLine Function

Read a line of text from the server and return it in a string buffer

### Prototype

```c
BOOL WINAPI SshReadLine(
    HCLIENT hClient,
    LPTSTR lpszBuffer,
    LPINT lpnLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszBuffer` | `LPTSTR` | Pointer to the string buffer that will contain the data when the function returns. The string will be terminated with a null byte, and will not contain the end-of-line characters. |
| `lpnLength` | `LPINT` | A pointer to an integer value which specifies the length of the buffer. The value should be initialized to the maximum number of characters that can be copied into the string buffer, including the terminating null character. When the function returns, its value will updated with the actual length of the string. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SshGetLastError.

### Remarks

- The SshReadLine function reads data sent by the server and copies it into a specified string buffer. Unlike the SshRead function which reads arbitrary bytes of data, this function is specifically designed to return a single line of text data in a string. When an end-of-line character sequence is encountered, the function will stop and return the data up to that point. The string buffer is guaranteed to be null-terminated and will not contain the end-of-line characters.
- There are some limitations when using SshReadLine. The function should only be used to read text, never binary data. In particular, the function will discard nulls, linefeed and carriage return control characters. The Unicode version of this function will return a Unicode string, however this function does not support reading raw Unicode data from the server. The data is internally buffered as octets (eight-bit bytes) and converted to Unicode using the MultiByteToWideChar function.
- This function will force the thread to block until an end-of-line character sequence is processed, the read operation times out or the server closes its end of the connection. If this function is called with asynchronous events enabled, it will automatically switch the client into a blocking mode, read the data and then restore the client to asynchronous operation. If another client operation is attempted while SshReadLine is blocked waiting for data from the server, an error will occur. It is recommended that this function only be used with blocking (synchronous) client connections; if the application needs to establish multiple simultaneous connections, it should create worker threads to manage each connection.
- The SshRead and SshReadLine function calls can be intermixed, however be aware that SshRead will consume any data that has already been buffered by the SshReadLine function and this may have unexpected results.
- Unlike the SshRead function, it is possible for data to be returned in the buffer even if the return value is zero. Applications should also check the value of the lpnLength argument to determine if any data was copied into the buffer. For example, if a timeout occurs while the function is waiting for more data to arrive, it will return zero; however, data may have already been copied into the string buffer prior to the error condition. It is the responsibility of the application to process that data, regardless of the function return value.

### Example

```cpp
TCHAR szBuffer[MAXBUFLEN];
INT nLength;
BOOL bResult;

do
{
	nLength = sizeof(szBuffer);
	bResult = SshReadLine(hSocket, szBuffer, &nLength);

	if (nLength > 0)
	{
		// Process the line of data returned in the string
		// buffer; the string is always null-terminated
	}
} while (bResult);

DWORD dwError = SshGetLastError();
if (dwError == ST_ERROR_CONNECTION_CLOSED)
{
	// The server has closed its side of the connection and
	// there is no more data available to be read
}
else if (dwError != 0)
{
	// An error has occurred while reading a line of data
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshIsReadble](#sshisreadble-function)
- [SshRead](#sshread-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)

---

## SshRegisterEvent Function

Register an event callback function

### Prototype

```c
INT WINAPI SshRegisterEvent(
    HCLIENT hClient,
    UINT nEvent,
    SSHEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nEvent` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. One of the following values may be used: |
| `lpEventProc` | `SSHEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the SshEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEvent** Parameter

    - `SSH_EVENT_CONNECT`: The connection to the server has completed.
    - `SSH_EVENT_DISCONNECT`: The server has closed the connection to the client. The client should read any remaining data and disconnect.
    - `SSH_EVENT_READ`: Data is available to read by the calling process. No additional messages will be posted until the client has read at least some of the data. This event is only generated if the client is in asynchronous mode.
    - `SSH_EVENT_WRITE`: The client can now write data. This notification is sent after a connection has been established, or after a previous attempt to write data has failed because it would result in a blocking operation. This event is only generated if the client is in asynchronous mode.
    - `SSH_EVENT_TIMEOUT`: The network operation has exceeded the specified timeout period. The client application may attempt to retry the operation, or may disconnect from the server and report an error to the user.
    - `SSH_EVENT_CANCEL`: The current operation has been canceled. Under most circumstances the client should disconnect from the server and re-connect if needed. After an operation has been canceled, the server may abort the connection or refuse to accept further commands from the client.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The SshRegisterEvent function associates a callback function with a specific event. The event handler is an SshEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the client session, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshDisableEvents](#sshdisableevents-function)
- [SshEnableEvents](#sshenableevents-function)
- [SshEventProc](#ssheventproc-function)
- [SshFreezeEvents](#sshfreezeevents-function)

---

## SshSearch Function

Search for a specific character sequence in the data stream

### Prototype

```c
BOOL WINAPI SshSearch(
    HCLIENT hClient,
    LPCTSTR lpszString,
    LPVOID lpvBuffer,
    LPDWORD lpdwLength,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszString` | `LPCTSTR` | A pointer to a string which specifies the sequence of characters to search for in the data stream. This parameter cannot be NULL or point to an empty string. |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the output from the server, or a pointer to a global memory handle which will reference the output when the function returns. If the output from the server is not required, this parameter may be NULL. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual number of bytes of output stored in the buffer. If the lpvBuffer parameter is NULL, this parameter should also be NULL. |
| `dwReserved` | `DWORD` | A reserved parameter. This value must be zero. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds and the character sequences was found in the data stream, the return value is non-zero. If the function fails or a timeout occurs before the sequence is found, the return value is zero. To get extended error information, call SshGetLastError.

### Remarks

- The SshSearch function searches for a character sequence in the data stream and stops reading when it is found. This is useful when the client wants to automate responses to the server. The function collects the output from the server and stores it in the buffer specified by the lpvBuffer parameter. When the function returns, the buffer will contain everything sent by the server up to and including the matching string.
- The lpvBuffer parameter may be specified in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the a fixed amount of output. In this case, the lpvBuffer parameter will point to the buffer that was allocated, the value that the lpdwLength parameter points to should be initialized to the size of that buffer. If the server sends more output than can be stored in the buffer, the remaining output will be discarded.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the output when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. This method is preferred if the client application does not have a general idea of how much output will be generated until the search string is found.

### Example

```cpp
LPCTSTR lpszCommand = _T("/bin/ls -l\r\n");
LPCTSTR lpszPrompt = _T("$ ");
HGLOBAL hgblOutput = NULL;
DWORD cbOutput = 0;
BOOL bResult;

// Search for a command prompt issued by the server

bResult = SshSearch(hClient,
					lpszPrompt,
					NULL,
					NULL,
					0);

// If the shell prompt was found, issue the command
// and capture the output into the hgblBuffer global
// memory buffer; the cbBuffer variable will contain
// the actual number of bytes in the buffer when the
// function returns

if (bResult)
{
	SshWrite(hClient,
			(LPBYTE)lpszCommand,
			lstrlen(lpszCommand));

	bResult = SshSearch(hClient,
						lpszPrompt,
						&hgblOutput,
						&cbOutput,
						0);
}

// Write the contents of the output buffer to the
// standard output stream

if (bResult)
{
	LPBYTE lpBuffer = (LPBYTE)GlobalLock(hgblBuffer);
	
	if (lpBuffer)
		fwrite(lpBuffer, 1, cbBuffer, stdout);

	GlobalUnlock(hgblBuffer);
	GlobalFree(hgblBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshIsBlocking](#sshisblocking-function)
- [SshIsReadable](#sshisreadable-function)
- [SshPeek](#sshpeek-function)
- [SshRead](#sshread-function)
- [SshReadLine](#sshreadline-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)

---

## SshSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI SshSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or SSH_ERROR. Those functions which call SshSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the SshGetLastError function. The use of SshGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetErrorString](#sshgeterrorstring-function)
- [SshGetLastError](#sshgetlasterror-function)

---

## SshSetLineMode Function

Change how end-of-line character sequences are sent to the server

### Prototype

```c
INT WINAPI SshSetLineMode(
    HCLIENT hClient,
    INT nLineMode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nLineMode` | `INT` | An integer value which specifies how the newlines are sent by the library. It must be one of the following values: |

### Options

- **nLineMode** Parameter

    - `SSH_NEWLINE_DEFAULT`: There are no changes to how data is sent to the server. Any carriage return or linefeed characters that are sent using the SshWrite function will be sent as-is. The SshWriteLine function will terminate each line of text with a carriage return and linefeed (CRLF) sequence. This is the default line mode that is set when a new connection is established.
    - `SSH_NEWLINE_CR`: A carriage return is used as the end-of-line character. Any data sent using the SshWrite function that contains only a linefeed (LF) character or a carriage return and linefeed (CRLF) sequence to indicate the end-of-line will be replaced by a carriage return (CR) character. The SshWriteLine function will terminate each line of text with a single carriage return character.
    - `SSH_NEWLINE_LF`: A linefeed is used as the end-of-line character. Any data sent using the SshWrite function that contains only a carriage return (CR) character or a carriage return an linefeed (CRLF) sequence to indicate the end-of-line will be replaced by a linefeed (LF) character. The SshWriteLine function will terminate each line of text with a single linefeed character.
    - `SSH_NEWLINE_CRLF`: A carriage return and linefeed (CRLF) character sequence is used to indicate the end-of-line. Any data sent using the SshWrite function that contains only a carriage return (CR) or linefeed (LF) will be replaced by a carriage return and linefeed. The SshWriteLine function will terminate each line of text with a carriage return and linefeed sequence.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the previous line mode for the client session. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- When a connection is initially established with the server, it determines what characters are used to indicate the end-of-line and how they are displayed. On UNIX based systems, this is controlled by the settings for the pseudo-terminal that is allocated for the client session, and can be changed using the stty command. In most cases, the client line mode can be left at the default. However, in some cases you may need to change the line mode, particularly if you intend to send data from a Windows text file or copied from the clipboard.
- Windows uses a carriage return and linefeed (CRLF) sequence to indicate the end-of-line and a UNIX based server may interpret that as multiple newlines. To prevent this, use the SshSetLineMode function to change the current line mode to SSH_NEWLINE_CR and the CRLF sequence in the text will be replaced by a single carriage return.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetLineMode](#sshgetlinemode-function)
- [SshWrite](#sshwrite-function)
- [SshWriteLine](#sshwriteline-function)

---

## SshSetTimeout Function

Set the number of seconds until an operation times out

### Prototype

```c
INT WINAPI SshSetTimeout(
    HCLIENT hClient,
    INT nTimeout
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `nTimeout` | `INT` | The number of seconds to wait for a blocking operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The timeout value is only used with blocking client connections.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshGetTimeout](#sshgettimeout-function)

---

## SshUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI SshUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful SshInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call SshUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to SshUninitialize for every successful call to SshInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshDisconnect](#sshdisconnect-function)
- [SshInitialize](#sshinitialize-function)

---

## SshWrite Function

Write data to the server

### Prototype

```c
INT WINAPI SshWrite(
    HCLIENT hClient,
    LPBYTE lpBuffer,
    INT cbBuffer
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpBuffer` | `LPBYTE` | The pointer to the buffer which contains the data that is to be sent to the server. |
| `cbBuffer` | `INT` | The number of bytes to send from the specified buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of bytes actually written. If the function fails, the return value is SSH_ERROR. To get extended error information, call SshGetLastError.

### Remarks

- The return value may be less than the number of bytes specified by the cbBuffer parameter. In this case, the data has been partially written and it is the responsibility of the client application to send the remaining data at some later point. For non-blocking clients, the client must wait for the SSH_EVENT_WRITE asynchronous notification message before it resumes sending data.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`

### See Also

- [SshIsWritable](#sshiswritable-function)
- [SshRead](#sshread-function)
- [SshReadLine](#sshreadline-function)
- [SshSetLineMode](#sshsetlinemode-function)
- [SshWriteLine](#sshwriteline-function)

---

## SshWriteLine Function

Write a line of text to the server

### Prototype

```c
BOOL WINAPI SshWriteLine(
    HCLIENT hClient,
    LPCTSTR lpszBuffer,
    LPINT lpnLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hClient` | `HCLIENT` | Handle to the client session. |
| `lpszBuffer` | `LPCTSTR` | The pointer to a string buffer which contains the data that will be sent to the server. All characters up to, but not including, the terminating null character will be written to the server. The data will always be terminated with a carriage-return and linefeed control character sequence. If this parameter points to an empty string or NULL pointer, then a only a carriage-return and linefeed are written to the server. |
| `lpnLength` | `LPINT` | A pointer to an integer value which will contain the number of characters written to the server, including the carriage-return and linefeed sequence. If this information is not required, a NULL pointer may be specified. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SshGetLastError.

### Remarks

- The SshWriteLine function writes a line of text to the server and terminates the line with a carriage-return and linefeed control character sequence. Unlike the SshWrite function which writes arbitrary bytes of data to the server, this function is specifically designed to write a single line of text data from a string.
- If the lpszBuffer string is terminated with a linefeed (LF) or carriage return (CR) character, it will be automatically converted to a standard CRLF end-of-line sequence. Because the string will be sent with a terminating CRLF sequence, the value returned in the lpnLength parameter will typically be larger than the original string length (reflecting the additional CR and LF characters), unless the string was already terminated with CRLF.
- There are some limitations when using SshWriteLine. The function should only be used to send text, never binary data. In particular, the function will discard nulls and append linefeed and carriage return control characters to the data stream. The Unicode version of this function will accept a Unicode string, however this function does not support sending raw Unicode data to the server. Unicode strings will be automatically converted to UTF-8 encoding using the WideCharToMultiByte function and then written as a stream of bytes.
- This function will force the thread to block until the complete line of text has been written, the write operation times out or the server aborts the connection. If this function is called with asynchronous events enabled, it will automatically switch the client into a blocking mode, send the data and then restore the client to asynchronous operation. If another network operation is attempted while SshWriteLine is blocked sending data to the server, an error will occur. It is recommended that this function only be used with blocking (synchronous) connections; if the application needs to establish multiple simultaneous connections, it should create worker threads to manage each connection.
- The SshWrite and SshWriteLine function calls can be safely intermixed.
- Unlike the SshWrite function, it is possible for data to have been written to the server if the return value is zero. For example, if a timeout occurs while the function is waiting to send more data to the server, it will return zero; however, some data may have already been written prior to the error condition. If this is the case, the lpnLength argument will specify the number of characters actually written up to that point.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstshv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshIsWritable](#sshiswritable-function)
- [SshRead](#sshread-function)
- [SshReadLine](#sshreadline-function)
- [SshSetLineMode](#sshsetlinemode-function)
- [SshWrite](#sshwrite-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SECURITYCREDENTIALS Structure

The SECURITYCREDENTIALS structure specifies the information needed by the library to specify additional security credentials, such as a client certificate or private key, when establishing a secure connection.

### Definition

```c
typedef struct _SECURITYCREDENTIALS
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwOptions;
    DWORD dwReserved;
    LPCTSTR lpszHostName;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
    LPCTSTR lpszCertStore;
    LPCTSTR lpszCertName;
    LPCTSTR lpszKeyFile;
} SECURITYCREDENTIALS, *LPSECURITYCREDENTIALS;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. If the structure is being allocated dynamically, this member must be set to the size of the structure and all other unused structure members must be initialized to a value of zero or NULL. |
| `dwProtocol` | `DWORD` | A value which specifies which security protocols are supported: |
| `dwOptions` | `DWORD` | This structure member is reserved for use with TLS connections and should always be initialized to zero. |
| `dwReserved` | `DWORD` | This structure member is reserved for future use and should always be initialized to zero. |
| `lpszHostName` | `LPCTSTR` | A pointer to a null terminated string which specifies the hostname that will be used when validating a server certificate. This member should always be initialized as a NULL pointer for connections using the SSH protocol. |
| `lpszUserName` | `LPCTSTR` | A pointer to a null terminated string which identifies the owner of client certificate. Currently this member is not used by the library and should always be initialized as a NULL pointer. |
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string which specifies the password needed to access the certificate. Currently this member is only used if a private key file has been specified. If there is no password associated with the certificate, then this member should be initialized as a NULL pointer. |
| `lpszCertStore` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate store to open. This member should always be initialized as a NULL pointer for connections using the SSH protocol. |
| `lpszCertName` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the certificate to use. This member should always be initialized as a NULL pointer for connections using the SSH protocol. |
| `lpszKeyFile` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the file which contains the private key required to establish the connection. This member is only used with the SSH protocol. If the member is NULL, then no private key is used. The private key file must be in the standard PEM format defined in RFC 1422. Both the standard RSA private key format and the proprietary OpenSSH format are supported. |

### Remarks

- This is a common structure used by several SocketTools APIs when establishing a secure connection. You should always initialize all unused members to zero or NULL and the dwSize member must be initialized to the size of the structure in bytes. In C/C++ this is done using the sizeof operator and other languages typically offer similar methods to determine the size of a user-defined structure.
- You only need to create this structure when authenticating the client session using a private key file instead of an account password. If the private key was protected with a password, the lpszPassword member must be assigned to that value or the connection request will fail with an authentication error.
- The lpszKeyFile path can use environment variables enclosed in percent symbols, and the path to the private key file will be normalized. It is recommended you always use an absolute path to the private key file. If you do not include a path, it will use the current working directory for the process. This can produce inconsistent results with multithreaded applications because the current working directory for a process is a global value and it can be changed by any thread at any time.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SshConnect](#sshconnect-structure)
- [SshCreateSecurityCredentials](#sshcreatesecuritycredentials-structure)
- [SshDeleteSecurityCredentials](#sshdeletesecuritycredentials-structure)
- [SshGetSecurityInformation](#sshgetsecurityinformation-structure)

---

## SECURITYINFO Structure

The SECURITYINFO structure contains security related information about the connection, including the protocol, cipher suite and fingerprint.

### Definition

```c
typedef struct _SECURITYINFO
{
    DWORD dwSize;
    DWORD dwProtocol;
    DWORD dwCipher;
    DWORD dwCipherStrength;
    DWORD dwHash;
    DWORD dwHashStrength;
    DWORD dwKeyExchange;
    DWORD dwCertStatus;
    SYSTEMTIME stCertIssued;
    SYSTEMTIME stCertExpires;
    LPCTSTR lpszCertIssuer;
    LPCTSTR lpszCertSubject;
    LPCTSTR lpszFingerprint;
} SECURITYINFO, *LPSECURITYINFO;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Specifies the size of the data structure. This member must always be initialized to the size of the structure in bytes prior to passing the address of this structure to the function. If this this member is not initialized, an error will be returned indicating that an invalid parameter has been passed to the function. |
| `dwProtocol` | `DWORD` | A numeric value which specifies the protocol that was selected to establish the secure connection. One of the following values will be returned: |
| `dwCipher` | `DWORD` | A numeric value which specifies the cipher that was selected when establishing the secure connection. One of the following values will be returned: |
| `dwCipherStrength` | `DWORD` | A numeric value which specifies the strength (the length of the cipher key in bits) of the cipher that was selected. Typically this value will be 128 or 256. 40-bit and 56-bit key lengths are considered weak encryption, and subject to brute force attacks. 128-bit and 256-bit key lengths are considered to be secure and are the recommended key length for secure communications. |
| `dwHash` | `DWORD` | A numeric value which specifies the hash algorithm which was selected. One of the following values will be returned: |
| `dwHashStrength` | `DWORD` | A numeric value which specifies the strength (the length in bits) of the message digest that was selected. |
| `dwKeyExchange` | `DWORD` | A numeric value which specifies the key exchange algorithm which was selected. One of the following values will be returned: |
| `dwCertStatus` | `DWORD` | This member is not used with SSH connections and will have a value of zero. |
| `stCertIssued` | `SYSTEMTIME` | This member is not used with SSH connections and will have a value of zero. |
| `stCertExpires` | `SYSTEMTIME` | This member is not used with SSH connections and will have a value of zero. |
| `lpszCertIssuer` | `LPCTSTR` | This member is not used with SSH connections and will have a value of zero. |
| `lpszCertSubject` | `LPCTSTR` | This member is not used with SSH connections and will have a value of zero. |
| `lpszFingerprint` | `LPCTSTR` | A pointer to a string which contains a sequence of hexadecimal values that uniquely identify the server. |

### Remarks

- The size of this structure is typically determined using the sizeof operator in C/C++ and most other languages have a similar method for obtaining the size of a data structure or user-defined type. It is important to note the size of this structure will be different on 32-bit and 64-bit platforms due to the difference in the length of the structure members which are pointers.
- When evaluating structure members which are string pointers, such as lpszCertIssuer and lpszCertSubject, you should always check to ensure their values are not NULL (zero) before attempting to dereference them. If those values are undefined, it is not guaranteed they will point to zero-length strings.
- There are additional protocols and ciphers defined which are not listed above because they are considered insecure and cannot be used when establishing a connection with the server. Although SSH protocol version 1.0 is defined in the header file, it is important to note that it is considered insecure and is no longer supported.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## SSHOPTIONDATA Structure

The SSHOPTIONDATA structure specifies additional option information for the client session.

### Definition

```c
typedef struct _SSHOPTIONDATA
{
    DWORD dwSize;
    DWORD dwReserved;
    UINT nProxyType;
    UINT nProxyPort;
    TCHAR szProxyHost;
    TCHAR szProxyUser;
    TCHAR szProxyPassword;
    UINT nTermCols;
    UINT nTermRows;
    TCHAR szTermName;
    TCHAR szCommandLine;
} SSHOPTIONDATA, *LPSSHOPTIONDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | An unsigned integer value which specifies the size of the SSHOPTIONDATA structure. This member must be initialized prior to passing the structure to the SshAsyncConnect or SshConnect functions. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for internal use, and should always be initialized to a value of zero. |
| `nProxyType` | `UINT` | An unsigned integer value that specifies the type of proxy that the client should connect through. This structure member is only used if the option SSH_OPTION_PROXYSERVER has been specified. Possible values are: |
| `nProxyPort` | `UINT` | An unsigned integer value that specifies the port number which should be used to establish the proxy connection. A value of zero specifies that the default port number appropriate for the selected protocol should be used. This structure member is only used if the option SSH_OPTION_PROXYSERVER has been specified. |
| `szProxyHost` | `TCHAR` | A null terminated string which specifies the host name or IP address of the proxy server. This structure member is only used if the option SSH_OPTION_PROXYSERVER has been specified. |
| `szProxyUser` | `TCHAR` | A null terminated string which specifies the user name which is used to authenticate the connection through the proxy server. This structure member is only used if the option SSH_OPTION_PROXYSERVER has been specified. |
| `szProxyPassword` | `TCHAR` | A null terminated string which specifies the password which is used to authenticate the connection through the proxy server. This structure member is only used if the option SSH_OPTION_PROXYSERVER has been specified. |
| `nTermCols` | `UINT` | An unsigned integer value which specifies the number of columns for the virtual terminal allocated for the client session. The default number of columns is 80. This structure member is only used if the option SSH_OPTION_TERMINAL has been specified. |
| `nTermRows` | `UINT` | An unsigned integer value which specifies the number of rows for the virtual terminal allocated for the client session. The default number of rows is 25. This structure member is only used if the option SSH_OPTION_TERMINAL has been specified. |
| `szTermName` | `TCHAR` | A null terminated string which specifies the name of the terminal emulation type. On UNIX based systems, this name typically corresponds to an entry in the terminal capability database (either termcap or terminfo). If the name is not specified, then the default name terminal name of "unknown" will be used. This structure member is only used if the option SSH_OPTION_TERMINAL has been specified. |
| `szCommandLine` | `TCHAR` | A null terminated string which specifies the command that should be executed on the server. The output from the command is returned to the client, and the session is terminated. This structure member is only used if the option SSH_OPTION_COMMAND has been specified. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

