# Web Services

## Functions

- [WebCancelTransfer](#webcanceltransfer-function)
- [WebCloseStorage](#webclosestorage-function)
- [WebCompareFile](#webcomparefile-function)
- [WebCompareObject](#webcompareobject-function)
- [WebCompareText](#webcomparetext-function)
- [WebCopyObject](#webcopyobject-function)
- [WebDeleteObject](#webdeleteobject-function)
- [WebDisableTrace](#webdisabletrace-function)
- [WebDownloadFile](#webdownloadfile-function)
- [WebDownloadFileEx](#webdownloadfileex-function)
- [WebEnableTrace](#webenabletrace-function)
- [WebEnumApplications](#webenumapplications-function)
- [WebEnumObjects](#webenumobjects-function)
- [WebGetAccountId](#webgetaccountid-function)
- [WebGetErrorString](#webgeterrorstring-function)
- [WebGetFile](#webgetfile-function)
- [WebGetFileEx](#webgetfileex-function)
- [WebGetFirstApplication](#webgetfirstapplication-function)
- [WebGetFirstObject](#webgetfirstobject-function)
- [WebGetLastError](#webgetlasterror-function)
- [WebGetLocation](#webgetlocation-function)
- [WebGetNextApplication](#webgetnextapplication-function)
- [WebGetNextObject](#webgetnextobject-function)
- [WebGetObject](#webgetobject-function)
- [WebGetObjectEx](#webgetobjectex-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)
- [WebGetObjectSize](#webgetobjectsize-function)
- [WebGetObjectTime](#webgetobjecttime-function)
- [WebGetStorageId](#webgetstorageid-function)
- [WebGetStorageQuota](#webgetstoragequota-function)
- [WebGetStorageTimeout](#webgetstoragetimeout-function)
- [WebGetTextObject](#webgettextobject-function)
- [WebGetTransferStatus](#webgettransferstatus-function)
- [WebInitialize](#webinitialize-function)
- [WebMoveObject](#webmoveobject-function)
- [WebOpenStorage](#webopenstorage-function)
- [WebPutFile](#webputfile-function)
- [WebPutFileEx](#webputfileex-function)
- [WebPutObject](#webputobject-function)
- [WebPutObjectEx](#webputobjectex-function)
- [WebPutTextObject](#webputtextobject-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebRegisterEvent](#webregisterevent-function)
- [WebRenameObject](#webrenameobject-function)
- [WebResetStorage](#webresetstorage-function)
- [WebResetStorageEx](#webresetstorageex-function)
- [WebSetLastError](#websetlasterror-function)
- [WebSetStorageTimeout](#websetstoragetimeout-function)
- [WebUninitialize](#webuninitialize-function)
- [WebUnregisterAppId](#webunregisterappid-function)
- [WebUploadFile](#webuploadfile-function)
- [WebUploadFileEx](#webuploadfileex-function)
- [WebValidateAppId](#webvalidateappid-function)
- [WebValidateLabel](#webvalidatelabel-function)
## Structures

- [INITDATA](#initdata-structure)
- [WEB_LOCATION](#web_location-structure)
- [WEB_STORAGE_APPLICATION](#web_storage_application-structure)
- [WEB_STORAGE_OBJECT](#web_storage_object-structure)
- [WEB_STORAGE_QUOTA](#web_storage_quota-structure)
- [WEB_STORAGE_TRANSFER](#web_storage_transfer-structure)

---

## WebCancelTransfer Function

Cancel a storage operation that is currently in progress

### Prototype

```c
BOOL WINAPI WebCancelTransfer(
    HSTORAGE hStorage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebCancelTransfer function will cancel the current data transfer and abort the connection to the storage server. This function will fail if an active data transfer (either an upload or download) is not in progress.
- This would typically be used within an event handler to cancel an operation as the contents of an object are being read or written. There is no mechanism to resume a canceled data transfer and this function should only be used when absolutely necessary.

### Requirements

- Minimum Desktop Platform: Windows 7 (Service Pack 1)
- Minimum Server Platform: Windows Server 2008 R2 (Service Pack 1)
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebGetTransferStatus](#webgettransferstatus-function)
- [WebRegisterEvent](#webregisterevent-function)

---

## WebCloseStorage Function

Close a storage container and release resources allocated for the client session

### Prototype

```c
BOOL WINAPI WebCloseStorage(
    HSTORAGE hStorage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebCloseStorage function must be called after all operations using the storage container have completed. The access token granted to the application will be released and the memory allocated for the session cache will be freed. Failure to call this function can result in a memory leak.

### Requirements

- Minimum Desktop Platform: Windows 7 (Service Pack 1)
- Minimum Server Platform: Windows Server 2008 R2 (Service Pack 1)
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebOpenStorage](#webopenstorage-function)

---

## WebCompareFile Function

Compare the contents of a stored object with a local file

### Prototype

```c
BOOL WINAPI WebCompareFile(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszLocalFile
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the label of the object to be compared. |
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file. If a path is not specified, the file will be created in the current working directory. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero and the contents of the file matches the stored object. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebCompareFile function performs a binary comparison of the contents of a local file with a stored object on the server. The contents of the file must be identical to the contents of the stored object or the function will fail.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCompareObject](#webcompareobject-function)
- [WebCompareText](#webcomparetext-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)

---

## WebCompareObject Function

Compare the contents of a stored object with a memory buffer

### Prototype

```c
BOOL WINAPI WebCompareObject(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPCVOID lpvBuffer,
    DWORD dwLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the label of the object to be compared. |
| `lpvBuffer` | `LPCVOID` | A pointer to a buffer that contains the data to be compared against the storage object. |
| `dwLength` | `DWORD` | An unsigned integer which specifies the length of the data buffer to be compared. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero and the data matches the contents of the stored object. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebCompareObject function performs a binary comparison of the data in the specified buffer with the contents of the storage object on the server. The dwLength parameter must match the size of the stored object exactly, or this function will fail. Partial comparisons are not supported by this function.
- If you wish to compare the contents of a text object, it is recommended that you use WebCompareText. This function ensures that Unicode text is compared correctly.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCompareFile](#webcomparefile-function)
- [WebCompareText](#webcomparetext-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)

---

## WebCompareText Function

Compare the contents of a stored object with a string

### Prototype

```c
BOOL WINAPI WebCompareText(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszObjectText,
    INT cchObjectText
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the label of the object to be compared. |
| `lpszObjectText` | `LPCTSTR` | A pointer to a null terminated string which contains the text to be compared with the stored object. |
| `cchObjectText` | `INT` | The number of characters in the lpszObjectText string to be compared. This value may be -1, in which case the string length will be determined by counting the number of characters up to the terminating null. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero and the data matches the contents of the stored object. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebCompareText function performs a text comparison of the characters in the string with the contents of the storage object on the server. The string length must match the amount of text in the stored object exactly, or this function will fail. Partial comparisons are not supported by this function.
- The Unicode version of this function will automatically convert the UTF-16 string to UTF-8 encoding in the same way the WebPutTextObject function does.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCompareFile](#webcomparefile-function)
- [WebCompareObject](#webcompareobject-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)
- [WebGetTextObject](#webgettextobject-function)
- [WebPutTextObject](#webputtextobject-function)

---

## WebCopyObject Function

Copy a storage object to a new location and optionally rename the object label

### Prototype

```c
BOOL WINAPI WebCopyObject(
    HSTORAGE hStorage,
    LPCTSTR lpszOldLabel,
    LPCTSTR lpszNewLabel,
    DWORD dwStorageType,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszOldLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the label of the existing storage object to be copied. This parameter must specify a valid object label and cannot be a NULL pointer or an empty string. |
| `lpszNewLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the new storage object that will be created. This parameter may be NULL or point to an empty string, in which case the label name is not changed. In this case, the dwStorageType parameter cannot be WEB_STORAGE_DEFAULT. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. One of the following values should be specified: |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain information about the new storage object. If this information is not required, this parameter may be a NULL pointer. |

### Options

- **dwStorageType** Parameter

    - `WEB_STORAGE_DEFAULT`: The default storage type. If this value is specified, the new object will be created using the same storage type as the original storage object.
    - `WEB_STORAGE_GLOBAL`: Global storage. Objects stored using this storage type are available to all users. Any changes made to objects using this storage type will affect all users of the application. Unless there is a specific need to limit access to the objects stored by the application to specific domains, local machines or users, it is recommended that you use this storage type when creating new objects.
    - `WEB_STORAGE_DOMAIN`: Local domain storage. Objects stored using this storage type are only available to users in the same local domain, as defined by the domain name or workgroup name assigned to the local system. If the domain or workgroup name changes, objects previously stored using this storage type will not be available to the application.
    - `WEB_STORAGE_MACHINE`: Local machine storage. Objects stored using this storage type are only available to users on the same local machine. The local machine is identified by unique characteristics of the system, including the boot volume GUID. Objects previously stored using this storage type will not be available on that system if the boot disk is reformatted.
    - `WEB_STORAGE_USER`: Current user storage. Objects stored using this storage type are only available to the current user logged in on the local machine. The user identifier is based on the Windows user SID that is assigned when the user account is created. If the user account is deleted, the objects previously stored using this storage type will not be available to the application.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebCopyObject function is used to create a copy of an existing storage object. It may be used to duplicate an object with a different label, or it may be used to copy the object to a new storage container type. For example, it can copy an object originally created using WEB_STORAGE_USER to a new object stored using WEB_STORAGE_MACHINE.
- Copied objects are assigned their own unique ID and are not linked to one another. Any subsequent changes made to the original object will not affect the copied object. Attempting to copy an object to itself or another existing object will result in an error.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDeleteObject](#webdeleteobject-function)
- [WebMoveObject](#webmoveobject-function)
- [WebRenameObject](#webrenameobject-function)

---

## WebDeleteObject Function

Delete an existing storage object

### Prototype

```c
BOOL WINAPI WebDeleteObject(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the label of the object to be deleted. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebDeleteObject function permanently deletes the storage object and its associated data from the server. Deleted objects cannot be recovered by the application. To remove all objects stored in the container, use the WebResetStorage function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCopyObject](#webcopyobject-function)
- [WebMoveObject](#webmoveobject-function)
- [WebRenameObject](#webrenameobject-function)
- [WebResetStorage](#webresetstorage-function)

---

## WebDisableTrace Function

Disable logging of function calls to the trace log

### Prototype

```c
BOOL WINAPI WebDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, a value of zero is returned.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebEnableTrace](#webenabletrace-function)

---

## WebDownloadFile Function

Download the contents of a stored object to a local file

### Prototype

```c
BOOL WINAPI WebDownloadFile(
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszLocalFile,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. This parameter cannot be NULL or an empty string. Refer to the WebValidateLabel function for more information about object labels. |
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be created or overwritten with the contents of the storage object. If a path is not specified, the file will be created in the current working directory. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object that has been downloaded. This parameter may be NULL if the information is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebDownloadFile function downloads the contents of the storage object and stores it in a local file. The object must have been previously created in the storage container WEB_STORAGE_GLOBAL using the application ID SocketTools.Storage.Default. This is the same default container used by the WebUploadFile function.
- Unlike the WebGetFile and WebGetFileEx functions, it is not required that you open a storage container using the WebOpenStorage function prior to calling WebDownloadFile.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.
- The WebDownloadFileEx function provides similar functionality with a more complex interface that supports custom application IDs, additional options and the ability to provide a callback function that is invoked during the download process.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Download the object from global storage to a local file
if (WebDownloadFile(lpszLocalFile, lpszObjectLabel, &webObject))
{
	// The object was downloaded, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object could not be downloaded, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to download \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFileEx](#webdownloadfileex-function)
- [WebGetFile](#webgetfile-function)
- [WebUploadFile](#webuploadfile-function)
- [WebValidateLabel](#webvalidatelabel-function)

---

## WebDownloadFileEx Function

Download the contents of a stored object with progress notifications

### Prototype

```c
BOOL WINAPI WebDownloadFileEx(
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszAppId,
    DWORD dwStorageType,
    DWORD dwReserved,
    DWORD dwTimeout,
    LPWEB_STORAGE_OBJECT lpObject,
    WEBEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be created or overwritten with the contents of the storage object. If a path is not specified, the file will be created in the current working directory. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. |
| `lpszAppId` | `LPCTSTR` | A pointer to null terminated string which specifies the application ID for the storage container. The application ID is a string that uniquely identifies the application and can only contain letters, numbers, the period and the underscore character. If this parameter is NULL or an empty string, the default identifier SocketTools.Storage.Default will be used. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. One of the following values should be specified: |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value should always be zero. |
| `dwTimeout` | `DWORD` | An unsigned integer value that specifies a timeout period in seconds. If this value is zero, a default timeout period will be used. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object that has been downloaded. This parameter may be NULL if the information is not required. |
| `lpEventProc` | `WEBEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the WebEventProc callback function. If this parameter is NULL, no callback function will be invoked during the data transfer. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function specified by lpEventProc. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **dwStorageType** Parameter

    - `WEB_STORAGE_GLOBAL`: Global storage. Objects stored using this storage type are available to all users. Any changes made to objects using this storage type will affect all users of the application. Unless there is a specific need to limit access to the objects stored by the application to specific domains, local machines or users, it is recommended that you use this storage type when creating new objects.
    - `WEB_STORAGE_DOMAIN`: Local domain storage. Objects stored using this storage type are only available to users in the same local domain, as defined by the domain name or workgroup name assigned to the local system. If the domain or workgroup name changes, objects previously stored using this storage type will not be available to the application.
    - `WEB_STORAGE_MACHINE`: Local machine storage. Objects stored using this storage type are only available to users on the same local machine. The local machine is identified by unique characteristics of the system, including the boot volume GUID. Objects previously stored using this storage type will not be available on that system if the boot disk is reformatted.
    - `WEB_STORAGE_USER`: Current user storage. Objects stored using this storage type are only available to the current user logged in on the local machine. The user identifier is based on the Windows user SID that is assigned when the user account is created. If the user account is deleted, the objects previously stored using this storage type will not be available to the application.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebDownloadFileEx function downloads the contents of the storage object and stores it in a local file. The WebDownloadFile function provides a simpler interface that defaults to downloading an object from the global storage container.
- Unlike the WebGetFile and WebGetFileEx functions, it is not required that you open a storage container using the WebOpenStorage function prior to calling WebDownloadFileEx.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.
- If you are downloading a large object and want your application to receive progress updates during the data transfer, provide a pointer to a callback function as the lpEventProc parameter. That function will receive event notifications as the data is being downloaded.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Download the object from global storage to a local file
if (WebDownloadFileEx(lpszLocalFile,
					lpszObjectLabel,
					lpszAppId,
					WEB_STORAGE_GLOBAL,
					0,
					&webObject,
					NULL, 0))
{
	// The object was downloaded, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object could not be retrieved, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to retrieve \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFileEx](#webgetfileex-function)
- [WebPutFileEx](#webputfileex-function)
- [WebUploadFile](#webuploadfile-function)
- [WebUploadFileEx](#webuploadfileex-function)

---

## WebEnableTrace Function

Enable logging of function calls to a file

### Prototype

```c
BOOL WINAPI WebEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDisableTrace](#webdisabletrace-function)

---

## WebEnumApplications Function

Enumerate all registered storage applications

### Prototype

```c
BOOL WINAPI WebEnumApplications(
    LPWEB_STORAGE_APPLICATION lpApplications,
    LPDWORD lpdwRegistered
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpApplications` | `LPWEB_STORAGE_APPLICATION` | A pointer to an array of WEB_STORAGE_APPLICATION structures that will contain information about the enumerated application IDs. If this parameter is NULL, then no information is returned. |
| `lpdwRegistered` | `LPDWORD` | A pointer to an unsigned integer that will contain the number of registered applications enumerated by this function. If the lpApplications parameter points to an array of WEB_STORAGE_APPLICATION structures, this parameter must be initialized to the maximum size of the array being passed to the function. If the lpApplications parameter is NULL, this value must be initialized to zero and when the function returns it will contain the number of registered applications. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebEnumApplications function can be used to enumerate the application IDs registered with the storage account using the WebRegisterAppId function.
- This function can be used in two ways. If the lpApplications parameter is NULL and the value passed by reference in the lpdwRegistered parameter is initialized to zero, the function will return the number of registered applications in lpdwRegistered. This can be used to dynamically determine the size of the lpApplications array instead of declaring a fixed-size array in your application.
- If the lpApplications parameter is not NULL, then the value referenced by lpdwRegistered must be initialized to the maximum size of the array that lpApplications points to. It is important to note that if either parameter is not initialized correctly, it can result in memory corruption and/or an unhandled exception.
- The WebGetFirstApplication and WebGetNextApplication functions can be used to iterate through all registered application IDs without requiring you to preallocate memory for an array. This can be more efficient if a large number of application IDs have been registered.

### Example

```cpp
DWORD dwAppCount = 0;

if (WebEnumApplications(NULL, &dwAppCount ))
{
	LPWEB_STORAGE_APPLICATION lpAppArray = NULL;
	DWORD dwIndex;

	// Return if there are no registered applications
	if (dwAppCount == 0)
		return;

	// Allocate memory for an array of WEB_STORAGE_APPLICATION structures
	lpAppArray = (LPWEB_STORAGE_APPLICATION)LocalAlloc(LPTR, sizeof(WEB_STORAGE_APPLICATION) * dwAppCount);

	if (lpAppArray == NULL)
		return; // Exhausted virtual memory?

	// Enumerate the registered application IDs
	if (!WebEnumApplications(lpAppArray, &dwAppCount))
		return;

	// Print information about each application
	for (dwIndex = 0; dwIndex < dwAppCount ; dwIndex++)
	{
		_tprintf(_T("AppId: %s\n"), lpAppArray[dwIndex].szAppId);
		_tprintf(_T("Key: %s\n"), lpAppArray[dwIndex].szApiKey);
		_tprintf(_T("LUID: %s\n"), lpAppArray[dwIndex].szLuid);
		_tprintf(_T("Tokens: %lu\n"), lpAppArray[dwIndex].dwTokens);

		if (dwAppCount > 1 && dwIndex < dwAppCount - 1)
			_tprintf(_T("\n"));
	}

	LocalFree((HLOCAL)lpAppArray);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetFirstApplication](#webgetfirstapplication-function)
- [WebGetNextApplication](#webgetnextapplication-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## WebEnumObjects Function

Enumerate all storage objects that match the specified label or content type

### Prototype

```c
BOOL WINAPI WebEnumObjects(
    HSTORAGE hStorage,
    LPCTSTR lpszMatchLabel,
    LPCTSTR lpszContentType,
    DWORD dwReserved,
    LPWEB_STORAGE_OBJECT lpObjects,
    LPDWORD lpdwObjects
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszMatchLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the value to match against the object labels in the container. The string may contain wildcard characters similar to those use with the Windows filesystem. A "?" character matches any single character, and "*" matches any number of characters in the label. If this value is a NULL pointer or an empty string, all objects in the container will be matched. |
| `lpszContentType` | `LPCTSTR` | A pointer to a null terminated string which specifies the content type of the objects to be enumerated. If this value is a NULL pointer or an empty string, the content type is ignored and all matching objects are returned. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value must be zero. |
| `lpObjects` | `LPWEB_STORAGE_OBJECT` | A pointer to an array of WEB_STORAGE_OBJECT structures that will contain information about the enumerated objects. If this parameter is NULL, then no object information is returned. |
| `lpdwObjects` | `LPDWORD` | A pointer to an unsigned integer that will contain the number of objects enumerated by this function. If the lpObjects parameter points to an array of WEB_STORAGE_OBJECT structures, this parameter must be initialized to the maximum size of the array being passed to the function. If the lpObjects parameter is NULL, this value must be initialized to zero, and when the function returns it will contain the number of matching objects. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebEnumObjects function can be used to enumerate the number of objects that match a given label, content type or both. If a content type is specified, it must be a valid MIME media content type designated using the type/subtype nomenclature. For example, "text/plain" or "image/jpeg". An invalid MIME type will cause the function to fail.
- This function can be used in two ways. If the lpObjects parameter is NULL and the value passed by reference in the lpdwObjects parameter is initialized to zero, the function will return the number of matching objects in lpdwObjects. This can be used to dynamically determine the size of the lpObjects array instead of declaring a fixed-size array in your application.
- If the lpObjects parameter is not NULL, then the value referenced by lpdwObjects must be initialized to the maximum size of the array that lpObjects points to. It is important to note that if either parameter is not initialized correctly, it can result in memory corruption and/or an unhandled exception.
- The WebGetFirstObject and WebGetNextObject functions can be used to iterate through all matching storage objects without requiring you to preallocate memory for an array. Using WebGetFirstObject and WebGetNextObject is more efficient when the container contains a large number of objects that match the specified label and/or content type.

### Example

```cpp
DWORD dwObjects = 0;

if (WebEnumObjects(hStorage, _T("*.pdf"), NULL, 0, NULL, &dwObjects))
{
	LPWEB_STORAGE_OBJECT lpObjects = NULL;
	DWORD dwIndex;

	// Return if there are no matching objects in the container
	if (dwObjects == 0)
		return;

	// Allocate memory for an array of WEB_STORAGE_OBJECT structures
	lpObjects = (LPWEB_STORAGE_OBJECT)LocalAlloc(LPTR, sizeof(WEB_STORAGE_OBJECT) * dwObjects);

	if (lpObjects == NULL)
		return; // Exhausted virtual memory?

	// Enumerate the matching objects
	if (!WebEnumObjects(hStorage, NULL, NULL, 0, lpObjects, &dwObjects))
		return;

	// Print information about each object
	for (dwIndex = 0; dwIndex < dwObjects; dwIndex++)
	{
		_tprintf(_T("Object: %s\n"), lpObjects[dwIndex].szObjectId);
		_tprintf(_T("Label: %s\n"), lpObjects[dwIndex].szLabel);
		_tprintf(_T("Size: %lu\n"), lpObjects[dwIndex].dwObjectSize);
		_tprintf(_T("Digest: %s\n"), lpObjects[dwIndex].szDigest);
		_tprintf(_T("Content: %s\n"), lpObjects[dwIndex].szContent);

		if (dwObjects > 1 && dwIndex < dwObjects - 1)
			_tprintf(_T("\n"));
	}
	
	LocalFree((HLOCAL)lpObjects);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetFirstObject](#webgetfirstobject-function)
- [WebGetNextObject](#webgetnextobject-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)

---

## WebGetAccountId Function

Return the current web services account identifier

### Prototype

```c
BOOL WINAPI WebGetAccountId(
    LPCTSTR lpszAccountId,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAccountId` | `LPCTSTR` | A pointer to a null-terminated string that will contain the account ID when the function returns. This parameter cannot be NULL and must be at least 35 characters in length. |
| `nMaxLength` | `INT` | An integer value that specifies the maximum number of characters that can be copied into the lpszAccountId string parameter, including the terminating null character. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The account ID is a string that uniquely identifies the web services account that is associated with the session. The account ID corresponds with your product serial number and runtime license key, but it is not identical to either of those values.
- If you are using an evaluation license, the account ID is temporary and only valid during the evaluation period. After the evaluation period has expired, the account ID is revoked and objects stored using this ID will be deleted. It is not recommended that you store critical application data using an evaluation license.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetStorageId](#webgetstorageid-function)

---

## WebGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI WebGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetLastError](#webgetlasterror-function)
- [WebSetLastError](#websetlasterror-function)

---

## WebGetFile Function

Download the contents of a storage object and copy to a local file

### Prototype

```c
BOOL WINAPI WebGetFile(
    HSTORAGE hStorage,
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be created or overwritten with the contents of the storage object. If a path is not specified, the file will be created in the current working directory. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetFile function downloads the contents of the storage object and stores it in a local file. For information about the stored object that has been downloaded, use the WebGetFileEx function.
- If you are downloading a large object and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFileEx](#webgetfileex-function)
- [WebGetObject](#webgetobject-function)
- [WebPutFile](#webputfile-function)
- [WebPutObject](#webputobject-function)
- [WebRegisterEvent](#webregisterevent-function)
- [WebUploadFile](#webuploadfile-function)

---

## WebGetFileEx Function

Download the contents of a storage object and return information about the object

### Prototype

```c
BOOL WINAPI WebGetFileEx(
    HSTORAGE hStorage,
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel,
    DWORD dwReserved,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be created or overwritten with the contents of the storage object. If a path is not specified, the file will be created in the current working directory. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value should always be zero. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object. This parameter may be NULL if the information is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetFileEx function downloads the contents of the storage object and stores it in a local file. Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.
- If you are downloading a large object and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications.
- The WebDownloadFile and WebDownloadFileEx functions provide similar functionality, but do not require a handle to an open storage container.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Download the object and store the contents in a local file
if (WebGetFileEx(hStorage, lpszLocalFile, lpszObjectLabel, 0, &webObject))
{
	// The object was downloaded, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %I64u\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object could not be retrieved, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to retrieve \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFile](#webgetfile-function)
- [WebGetObjectEx](#webgetobjectex-function)
- [WebPutFileEx](#webputfileex-function)
- [WebPutObjectEx](#webputobjectex-function)
- [WebRegisterEvent](#webregisterevent-function)
- [WebUploadFile](#webuploadfile-function)

---

## WebGetFirstApplication Function

Return information about the first registered storage application

### Prototype

```c
BOOL WINAPI WebGetFirstApplication(
    LPWEB_STORAGE_APPLICATION lpAppInfo,
    LPDWORD lpdwContext
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpAppInfo` | `LPWEB_STORAGE_APPLICATION` | A pointer to a WEB_STORAGE_APPLICATION structure that will contain information about the registered application when the function returns. This parameter cannot be NULL. |
| `lpdwContext` | `LPDWORD` | A pointer to an unsigned integer that is used with subsequent calls to the WebGetNextApplication function. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetFirstApplication function returns information about the first registered application. It is used in conjunction with the WebGetNextApplication function to enumerate all of the registered application IDs associated with your storage account.
- This function provides an alternative to the WebEnumApplications function, which populates an array of WEB_STORAGE_APPLICATION structures. In some cases, iterating through each AppID in a loop may be preferred to preallocating memory for an array to store every registered application. Using WebGetFirstApplication and WebGetNextApplication is more efficient when you have registered a large number of application IDs.
- You cannot intermix calls between WebGetFirstApplication and WebEnumApplications. The WebEnumApplications function will reset the internal application ID cache for the client session and subsequent calls to WebGetNextApplication will fail. The cached application ID information used by this function is shared by the entire process. Attempting to enumerate all registered application IDs in multiple threads at the same time can yield unexpected results. It is recommended that multithreaded clients use a critical section to ensure that only a single thread is enumerating the AppIDs at any one time.

### Example

```cpp
WEB_STORAGE_APPLICATION webApp;
DWORD dwContext = 0;

if (WebGetFirstApplication(&webApp, &dwContext))
{
	do
	{
		// Print information for each registered application
		_tprintf(_T("AppId: %s\n"), webApp.szAppId);
		_tprintf(_T("Key: %s\n"), webApp.szApiKey);
		_tprintf(_T("LUID: %s\n"), webApp.szLuid);
		_tprintf(_T("Tokens: %lu\n"), webApp.dwTokens);
		_tprintf(_T("\n"));
	}
	while (WebGetNextApplication(&webApp, &dwContext));
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebEnumApplications](#webenumapplications-function)
- [WebGetNextApplication](#webgetnextapplication-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## WebGetFirstObject Function

Return information about the first object that matches search criteria

### Prototype

```c
BOOL WINAPI WebGetFirstObject(
    HSTORAGE hStorage,
    LPCTSTR lpszMatchLabel,
    LPCTSTR lpszContentType,
    DWORD dwReserved,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszMatchLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the value to match against the object labels in the container. The string may contain wildcard characters similar to those use with the Windows filesystem. A "?" character matches any single character, and "*" matches any number of characters in the label. If this value is a NULL pointer or an empty string, all objects in the container will be matched. |
| `lpszContentType` | `LPCTSTR` | A pointer to a null terminated string which specifies the content type of the objects to be enumerated. If this value is a NULL pointer or an empty string, the content type is ignored and all matching objects are returned. If a content type is specified, it must be a valid MIME media content type designated using the type/subtype nomenclature. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value must be zero. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain information about the storage object when the function returns. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetFirstObject function returns information about the first object that matches a given label, content type or both. It is used in conjunction with the WebGetNextObject function to enumerate all of the matching objects in the storage container.
- This function provides an alternative to the WebEnumObjects function, which populates an array of WEB_STORAGE_OBJECT structures. In some cases, iterating over each object in a loop may be preferred to preallocating memory for an array to store every matching object. Using WebGetFirstObject and WebGetNextObject is more efficient when the container contains a large number of objects that match the specified label and/or content type.
- You cannot intermix calls between WebGetFirstObject and WebEnumObjects. The WebEnumObjects function will reset the internal object cache for the client session and subsequent calls to WebGetNextObject will fail.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

if (WebGetFirstObject(hStorage, _T("*.pdf"), NULL, 0, &webObject))
{
	do
	{
		// Print information about each object
		_tprintf(_T("Object: %s\n"), webObject.szObjectId);
		_tprintf(_T("Label: %s\n"), webObject.szLabel);
		_tprintf(_T("Size: %lu\n"), webObject.dwObjectSize);
		_tprintf(_T("Digest: %s\n"), webObject.szDigest);
		_tprintf(_T("Content: %s\n"), webObject.szContent);
		_tprintf(_T("\n"));
	}
	while (WebGetNextObject(hStorage, &webObject));
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebEnumObjects](#webenumobjects-function)
- [WebGetNextObject](#webgetnextobject-function)

---

## WebGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI WebGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the WebSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the WebGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call WebSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or WEB_ERROR. Those functions which call WebSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebGetErrorString](#webgeterrorstring-function)
- [WebSetLastError](#websetlasterror-function)

---

## WebGetLocation Function

Return the current physical location of the local computer system

### Prototype

```c
BOOL WINAPI WebGetLocation(
    LPWEB_LOCATION lpLocation
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpLocation` | `LPWEB_LOCATION` | A pointer to a WEB_LOCATION structure that will contain information about the location when the function returns. The structure members will be automatically initialized to zero and zero-length strings when the function is called. This parameter cannot be a NULL pointer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetLocation function is used to obtain physical location information associated with the with the external IP address of the local system. The accuracy of this information can vary depending on the location, with the most detailed information being available for North America. The country and time zone information for all locations is generally accurate. However, as the location information becomes more precise, details such as city names, postal codes and specific geographic locations (e.g.: longitude and latitude) may have reduced accuracy.
- This location information should not be used by programs that require extremely accurate map coordinates, such as navigation applications. The location information in North America should be generally accurate within a 25 mile (40km) radius. However, given the nature of how IP address location works, there is no guarantee that location information for any specific IP address or network will be accurate.
- Software that is designed to protect the privacy of users, such as those which route all Internet traffic through proxy servers or VPNs, can significantly impact the accuracy of this information. In this case, the data returned in this structure may reflect the location of the network or proxy server, and not the location of the person using your application. It is recommended that you always request permission from the user before acquiring their location, have them confirm that the location is correct and provide a mechanism for them to update that information.

### Example

```cpp
WEB_LOCATION webLocation;

// Return information about the location of the local system 
if (WebGetLocation(&webLocation))
{
	_tprintf(_T("Location ID: %s\n"), webLocation.szLocationId);
	_tprintf(_T("IP address: %s\n"), webLocation.szIPAddress);
	_tprintf(_T("ASN: %u\n"), webLocation.nAutonomousSystemNumber);
	_tprintf(_T("Organization: %s\n"), webLocation.szOrganization);
	_tprintf(_T("Region Name: %s\n"), webLocation.szRegionName);
	_tprintf(_T("Region Code: %03u\n"), webLocation.nRegionCode);
	_tprintf(_T("Country Name: %s\n"), webLocation.szCountryName);
	_tprintf(_T("Country Code: %03u\n"), webLocation.nCountryCode);
	_tprintf(_T("Subdivision: %s\n"), webLocation.szSubdivision);
	_tprintf(_T("City Name: %s\n"), webLocation.szCityName);
	_tprintf(_T("Postal Code: %s\n"), webLocation.szPostalCode);
	_tprintf(_T("Timezone: %s (%ld)\n"), webLocation.szTimezone, webLocation.nTimezoneOffset);
	_tprintf(_T("Local Time: %04d-%02d-%02d %02d:%02d:%02d %s\n"),
			webLocation.stLocalTime.wYear,
			webLocation.stLocalTime.wMonth,
			webLocation.stLocalTime.wDay,
			webLocation.stLocalTime.wHour,
			webLocation.stLocalTime.wMinute,
			webLocation.stLocalTime.wSecond,
			webLocation.szTzShortName);
	_tprintf(_T("Coordinates: %s\n"), webLocation.szCoordinates);
	_tprintf(_T("Latitude: %.4f\n"), webLocation.dLatitude);
	_tprintf(_T("Longitude: %.4f\n"), webLocation.dLongitude);
}
else
{
	_tprintf(_T("Unable to determine the current location\n"));
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

---

## WebGetNextApplication Function

Return information about the next registered storage application

### Prototype

```c
BOOL WINAPI WebGetNextApplication(
    LPWEB_STORAGE_APPLICATION lpAppInfo,
    LPDWORD lpdwContext
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpAppInfo` | `LPWEB_STORAGE_APPLICATION` | A pointer to a WEB_STORAGE_APPLICATION structure that will contain information about the registered application when the function returns. This parameter cannot be NULL. |
| `lpdwContext` | `LPDWORD` | A pointer to an unsigned integer that is used with subsequent calls to the WebGetNextApplication function. The value of this parameter is initialized by calling the WebGetFirstApplication function. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetNextApplication function returns information about the next registered application after initially calling the WebGetFirstApplication function. It is used to enumerate all of the registered application IDs associated with your storage account.
- This function provides an alternative to the WebEnumApplications function, which populates an array of WEB_STORAGE_APPLICATION structures. In some cases, iterating through each AppID in a loop may be preferred to preallocating memory for an array to store every registered application. Using WebGetFirstApplication and WebGetNextApplication is more efficient when you have registered a large number of application IDs.
- You cannot intermix calls between WebGetNextApplication and WebEnumApplications. The WebEnumApplications function will reset the internal application ID cache for the client session and subsequent calls to WebGetNextApplication will fail. The cached application ID information used by this function is shared by the entire process. Attempting to enumerate all registered application IDs in multiple threads at the same time can yield unexpected results. It is recommended that multithreaded clients use a critical section to ensure that only a single thread is enumerating the AppIDs at any one time.

### Example

```cpp
WEB_STORAGE_APPLICATION webApp;
DWORD dwContext = 0;

if (WebGetFirstApplication(&webApp, &dwContext))
{
	do
	{
		// Print information for each registered application
		_tprintf(_T("AppId: %s\n"), webApp.szAppId);
		_tprintf(_T("Key: %s\n"), webApp.szApiKey);
		_tprintf(_T("LUID: %s\n"), webApp.szLuid);
		_tprintf(_T("Tokens: %lu\n"), webApp.dwTokens);
		_tprintf(_T("\n"));
	}
	while (WebGetNextApplication(&webApp, &dwContext));
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebEnumApplications](#webenumapplications-function)
- [WebGetFirstApplication](#webgetfirstapplication-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## WebGetNextObject Function

Return information about the next object that matches search criteria

### Prototype

```c
BOOL WINAPI WebGetNextObject(
    HSTORAGE hStorage,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain information about the storage object when the function returns. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetNextObject function returns information about the next object that matches the label and/or content type that was specified by the WebGetFirstObject function call. This function may only be called after WebGetFirstObject has been called, otherwise it will fail.
- When all matching objects have been returned to the caller, this function will return zero (FALSE) and the WebGetLastError function will return NO_ERROR. Any other error code indicates an underlying problem with the request, such as an invalid parameter passed to the function.
- You cannot intermix calls between WebGetNextObject and WebEnumObjects. The WebEnumObjects function will reset the internal object cache for the client session and subsequent calls to WebGetNextObject will fail.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

if (WebGetFirstObject(hStorage, _T("*.pdf"), NULL, 0, &webObject))
{
	do
	{
		// Print information about each object
		_tprintf(_T("Object: %s\n"), webObject.szObjectId);
		_tprintf(_T("Label: %s\n"), webObject.szLabel);
		_tprintf(_T("Size: %lu\n"), webObject.dwObjectSize);
		_tprintf(_T("Digest: %s\n"), webObject.szDigest);
		_tprintf(_T("Content: %s\n"), webObject.szContent);
		_tprintf(_T("\n"));
	}
	while (WebGetNextObject(hStorage, &webObject));
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebEnumObjects](#webenumobjects-function)
- [WebGetFirstObject](#webgetfirstobject-function)

---

## WebGetObject Function

Download the contents of a storage object to a buffer

### Prototype

```c
BOOL WINAPI WebGetObject(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPVOID lpvBuffer,
    DWORD lpdwLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `DWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the file that was downloaded. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetObject function is used to retrieve a stored object from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the object. In this case, the lpvBuffer parameter will point to the buffer that was allocated and the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the file data when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. See the example code below.

### Example

```cpp
HGLOBAL hgblBuffer = (HGLOBAL)NULL;
DWORD dwLength = 0;

// Return the file data into block of global memory allocated by
// the GlobalAlloc function; the handle to this memory will be
// returned in the hgblBuffer parameter
if (WebGetObject(hStorage, lpszObjectLabel, &hgblBuffer, &dwLength))
{
	// Lock the global memory handle, returning a pointer to the
	// resource data
	LPBYTE lpBuffer = (LPBYTE)GlobalLock(hgblBuffer);
	
	// After the data has been used, the handle must be unlocked
	// and freed, otherwise a memory leak will occur
	GlobalUnlock(hgblBuffer);
	GlobalFree(hgblBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetFile](#webgetfile-function)
- [WebGetObjectEx](#webgetobjectex-function)
- [WebPutFile](#webputfile-function)
- [WebPutObject](#webputobject-function)

---

## WebGetObjectEx Function

Download the contents of a storage object to buffer and return information about the object

### Prototype

```c
BOOL WINAPI WebGetObjectEx(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    DWORD dwReserved,
    LPVOID lpvBuffer,
    LPDWORD lpdwLength,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. |
| `dwReserved` | `DWORD` | An unsigned integer that is reserved for future use. This value should always be zero. |
| `lpvBuffer` | `LPVOID` | A pointer to a byte buffer which will contain the data transferred from the server, or a pointer to a global memory handle which will reference the data when the function returns. |
| `lpdwLength` | `LPDWORD` | A pointer to an unsigned integer which should be initialized to the maximum number of bytes that can be copied to the buffer specified by the lpvBuffer parameter. If the lpvBuffer parameter points to a global memory handle, the length value should be initialized to zero. When the function returns, this value will be updated with the actual length of the file that was downloaded. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object. This parameter may be NULL if the information is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetObjectEx function is used to retrieve a stored object from the server and copy it into a local buffer. The function may be used in one of two ways, depending on the needs of the application. The first method is to pre-allocate a buffer large enough to store the contents of the object. In this case, the lpvBuffer parameter will point to the buffer that was allocated and the value that the lpdwLength parameter points to should be initialized to the size of that buffer.
- The second method that can be used is have the lpvBuffer parameter point to a global memory handle which will contain the file data when the function returns. In this case, the value that the lpdwLength parameter points to must be initialized to zero. It is important to note that the memory handle returned by the function must be freed by the application, otherwise a memory leak will occur. See the example code below.
- If you wish to retrieve the contents of a text object and store it in a null terminated string buffer, use the WebGetTextObject function. If you wish to download the contents of a stored file, use the WebGetFileEx function.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.

### Example

```cpp
HGLOBAL hgblBuffer = (HGLOBAL)NULL;
DWORD dwLength = 0;
WEB_STORAGE_OBJECT webObject;

// Return the file data into block of global memory allocated by
// the GlobalAlloc function; the handle to this memory will be
// returned in the hgblBuffer parameter
if (WebGetObjectEx(hStorage, lpszObjectLabel, 0, &hgblBuffer, &dwLength, &webObject))
{
	// Lock the global memory handle, returning a pointer to the
	// resource data
	LPBYTE lpBuffer = (LPBYTE)GlobalLock(hgblBuffer);
	
	// After the data has been used, the handle must be unlocked
	// and freed, otherwise a memory leak will occur
	GlobalUnlock(hgblBuffer);
	GlobalFree(hgblBuffer);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetFileEx](#webgetfileex-function)
- [WebGetObject](#webgetobject-function)
- [WebGetTextObject](#webgettextobject-function)
- [WebPutFileEx](#webputfileex-function)
- [WebPutObjectEx](#webputobjectex-function)

---

## WebGetObjectInformation Function

Retrieve the metadata for the specified storage object

### Prototype

```c
BOOL WINAPI WebGetObjectInformation(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    DWORD dwReserved,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object. |
| `dwReserved` | `DWORD` | An unsigned integer that is reserved for future use. This value should always be zero. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object. This parameter cannot be a NULL pointer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetObjectInformation function is used to retrieve the metadata for a stored object on the server, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents. This function can also be used as a simple method to determine if the specified object exists without the overhead of requesting the server attempt to retrieve the contents of an object.
- Although storage object labels are similar to Windows file names, they are case-sensitive. When requesting information about an object, your application must specify the label name exactly as it was created. The object label cannot contain wildcard characters.
- If you are only interested in obtaining the size of a stored object, you can use the WebGetObjectSize function.
- To obtain information about how much storage your applications are using and the total number of stored objects, use the WebGetStorageQuota function.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Get information about the object
if (WebGetObjectInformation(hStorage, lpszObjectLabel, 0, &webObject))
{
	// The object exists, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object does not exist, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to get information on \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetFile](#webgetfile-function)
- [WebGetObject](#webgetobject-function)
- [WebGetObjectSize](#webgetobjectsize-function)
- [WebGetObjectTime](#webgetobjecttime-function)
- [WebGetStorageQuota](#webgetstoragequota-function)
- [WebPutFile](#webputfile-function)
- [WebPutObject](#webputobject-function)

---

## WebGetObjectSize Function

Return the size of the specified storage object

### Prototype

```c
BOOL WINAPI WebGetObjectSize(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPDWORD lpdwObjectSize
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object. This parameter cannot be a NULL pointer. |
| `lpdwObjectSize` | `LPDWORD` | A pointer to an unsigned integer value that will contain the size of the object. If this parameter is NULL, the parameter is ignored and the function only checks for the existence of an object that matches the specified label. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetObjectSize function is used to retrieve the size of a stored object on the server. This function can also be used as a simple method to determine if the specified object exists. An object size of zero means the object exists, but currently has no data associated with it.
- If the lpObjectSize parameter is not NULL, its value is initialized to zero when the function is called, and updated with the object size when the function returns.
- Although storage object labels are similar to Windows file names, they are case-sensitive. When requesting the size of an object, your application must specify the label name exactly as it was created. The object label cannot contain wildcard characters.
- The WebGetObjectInformation function can be used to obtain the metadata associated with the storage object, including the size, content type, creation date and a SHA-256 digest of the data.
- To obtain information about how much storage your applications are using and the total number of stored objects, use the WebGetStorageQuota function.

### Example

```cpp
// Check if the object exists
DWORD dwObjectSize;

if (WebGetObjectSize(hStorage, lpszObjectLabel, &dwObjectSize))
{
	// The object exists, display the size in bytes
	_tprintf(_T("The size of \"%s\" is %lu bytes\n"), lpszObjectLabel, dwObjectSize);
}
else
{
	// The object size could not be determined, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to get the size of \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetObject](#webgetobject-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)
- [WebGetObjectTime](#webgetobjecttime-function)
- [WebGetStorageQuota](#webgetstoragequota-function)
- [WebPutObject](#webputobject-function)

---

## WebGetObjectTime Function

Return the creation and last modification time for the specified storage object

### Prototype

```c
BOOL WINAPI WebGetObjectTime(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPFILETIME lpTimeCreated,
    LPFILETIME lpTimeModified
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object. This parameter cannot be a NULL pointer. |
| `lpTimeCreated` | `LPFILETIME` | A pointer to a FILETIME structure which will contain the time the storage object was created when the function returns. If this information is not required, the parameter may be NULL. |
| `lpTimeModified` | `LPFILETIME` | A pointer to a FILETIME structure which will contain the time the storage object was last modified when the function returns. If this information is not required, the parameter may be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetObjectTime function is used to retrieve the creation and modification times for a stored object on the server. This function can also be used as a simple method to determine if the specified object exists. Each of the LPFILETIME parameters are optional and may be NULL if either the creation or modification times are not needed. If a pointer to a FILETIME structure is provided, its members will be initialized to zero and then updated with the creation and/or modification time if the function succeeds.
- Although storage object labels are similar to Windows file names, they are case-sensitive. When requesting the size of an object, your application must specify the label name exactly as it was created. The object label cannot contain wildcard characters.
- If you wish to obtain all of the metadata associated with a specific object with a single function call, including the size, content type, creation and modification times, use the WebGetObjectInformation function.

### Example

```cpp
// Check if the object exists
FILETIME ftCreated;
FILETIME ftModified;

if (WebGetObjectTime(hStorage, lpszObjectLabel, &ftCreated, &ftModified))
{
	// The object exists, display the last modification time
	TCHAR szLocalDate[128];
	TCHAR szLocalTime[128];
	FILETIME ftLocalTime;
	SYSTEMTIME stLocalTime;
	
	FileTimeToLocalFileTime(&ftModified, &ftLocalTime);
	FileTimeToSystemTime(&ftLocalTime, &stLocalTime);
	
	GetDateFormat(LOCALE_USER_DEFAULT,
				DATE_SHORTDATE,
				&stLocalTime,
				NULL,
				szLocalDate, 128);

	GetTimeFormat(LOCALE_USER_DEFAULT,
				TIME_FORCE24HOURFORMAT,
				&stLocalTime,
				NULL,
				szLocalTime, 128);    
	
	_tprintf(_T("The object \"%s\" was lasted modified on %s %s\n"), lpszObjectLabel, szLocalDate, szLocalTime);
}
else
{
	// The object times could not be determined, display an error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to get the size of \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetObject](#webgetobject-function)
- [WebGetObjectInformation](#webgetobjectinformation-function)
- [WebGetStorageQuota](#webgetstoragequota-function)
- [WebPutObject](#webputobject-function)

---

## WebGetStorageId Function

Return the returns the storage container ID

### Prototype

```c
BOOL WINAPI WebGetStorageId(
    HSTORAGE hStorage,
    LPCTSTR lpszStorageId,
    INT nMaxLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszStorageId` | `LPCTSTR` | A pointer to a null-terminated string that will contain the storage ID when the function returns. This parameter cannot be NULL and must be at least 35 characters in length. |
| `nMaxLength` | `INT` | An integer value that specifies the maximum number of characters that can be copied into the lpszStorageId string parameter, including the terminating null character. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The storage ID is a string that identifies the storage container that was opened with the WebOpenStorage function. The storage ID is associated with your development license and is guaranteed to be a unique value.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetAccountId](#webgetaccountid-function)
- [WebOpenStorage](#webopenstorage-function)

---

## WebGetStorageQuota Function

Return quota limits assigned to your storage account

### Prototype

```c
BOOL WINAPI WebGetStorageQuota(
    LPWEB_STORAGE_QUOTA lpQuota
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpQuota` | `LPWEB_STORAGE_QUOTA` | A pointer to a WEB_STORAGE_QUOTA structure that will contain information the quota limits for your account when the function returns. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetStorageQuota function can be used to determine how much storage space is available to your application. The ulBytesFree structure member will tell you how many bytes of storage you have available, and the dwObjectSize member will tell you the maximum size of any individual storage object. In addition to the bytes allocated for storage, there is also a limit on the total number of objects that your application may create which is specified by the dwObjectLimit member.
- Accounts that are created with an evaluation license have much lower quota limits than a standard account and should be used for testing purposes only. After the evaluation period has ended, all objects stored using the evaluation license will be deleted.

### Example

```cpp
WEB_STORAGE_QUOTA appQuota;

// Initialize the library
if (WebInitialize(CSTOOLS11_LICENSE_KEY, NULL) == FALSE)
{
	_tprintf(_T("Unable to initialize the SocketTools library\n"));
	return;
}

// Display storage account usage and limits
if (WebGetStorageQuota(&appQuota))
{
	_tprintf(_T("Objects Used:  %lu\n"), appQuota.dwObjects);
	_tprintf(_T("Object Limit:  %lu\n"), appQuota.dwObjectLimit);
	_tprintf(_T("Object Size:   %lu\n"), appQuota.dwObjectSize);
	_tprintf(_T("Bytes Used:    %I64u\n"), appQuota.ulBytesUsed);
	_tprintf(_T("Bytes Free:    %I64u\n"), appQuota.ulBytesFree);
	_tprintf(_T("Storage Limit: %I64u\n"), appQuota.ulStorageLimit);
}
else
{
	// Unable to get the quota information for this account
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to get the account quota (%s)\n"), szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebGetObjectInformation](#webgetobjectinformation-function)
- [WebGetObjectSize](#webgetobjectsize-function)
- [WEB_STORAGE_QUOTA](#web_storage_quota-function)

---

## WebGetStorageTimeout Function

Get the number of seconds until a storage operation times out

### Prototype

```c
DWORD WINAPI WebGetStorageTimeout(
    HSTORAGE hStorage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |

### Return Value

- **Type**: `DWORD`
- If the function succeeds, the return value is the timeout period in seconds. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The default timeout period is 10 seconds.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebOpenStorage](#webopenstorage-function)
- [WebSetStorageTimeout](#websetstoragetimeout-function)

---

## WebGetTextObject Function

Download the contents of a text object into a string buffer

### Prototype

```c
BOOL WINAPI WebGetTextObject(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPTSTR lpszBuffer,
    INT nMaxLength,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the text object that should be retrieved from the server. |
| `lpszBuffer` | `LPTSTR` | A pointer to a string buffer which will contain the text stored in the object. The string must be large enough to store the terminating null character. |
| `nMaxLength` | `INT` | An integer value that specifies the maximum number of characters that can be copied into the string buffer. This value must be large enough to store the complete text and the terminating null character. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object. This parameter may be NULL if the information is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetTextObject function is used to retrieve the text from a stored object and copy it into a string buffer. The string buffer must be pre-allocated and large enough to contain the entire text and a terminating null character.
- If you use the WebGetObjectSize function to determine the size of the object so your application can dynamically allocate memory for the string buffer, be sure to allocate a string buffer that is larger than the reported size. The size of the stored text object may differ slightly from what is returned in the string, depending on how the text has been encoded.
- Text objects that are created or updated using the Unicode version of WebPutTextObject function will always be converted and stored as UTF-8 encoded text. If your application uses the ANSI version of this function to retrieve that text, it will be UTF-8 encoded and your application will need to convert it back to UTF-16 encoded text, if needed. The Unicode version of this function will perform the conversion automatically.
- This function should never be used to retrieve data from an object that does not contain text. If the WebPutObject function was previously used to store UTF-16 encoded text, you must use WebGetObject to retrieve that data. This function only recognizes UTF-8 encoded text and considers UTF-16 encoded text to be binary data.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.

### Example

```cpp
LPTSTR lpszBuffer = (LPTSTR)NULL;
INT nMaxLength = 8192;
WEB_STORAGE_OBJECT webObject;

lpszBuffer = (LPTSTR)LocalAlloc(LPTR, nMaxLength);
if (lpszBuffer == NULL)
{
	// Unable to allocate memory for the string
	_ftprintf(stderr, _T("Unable to allocate %d bytes of memory\n"), nMaxLength);
	return;
}

if (WebGetTextObject(hStorage, lpszObjectLabel, lpszBuffer, nMaxLength, &webObject))
{
	// Output the text contained in the object
	_fputts(lpszBuffer, stdout);
}
else
{
	DWORD dwError = WebGetLastError();
	TCHAR szError[128];

	WebGetErrorString(dwError, szError, 128);
	_ftprintf(stderr, _T("Unable to get \"%s\": %s\n"), lpszObjectLabel, szError);
}

// Free memory allocated for the text when no longer needed
LocalFree((HLOCAL)lpszBuffer);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCompareText](#webcomparetext-function)
- [WebGetFileEx](#webgetfileex-function)
- [WebGetObject](#webgetobject-function)
- [WebPutFileEx](#webputfileex-function)
- [WebPutObjectEx](#webputobjectex-function)
- [WebPutTextObject](#webputtextobject-function)

---

## WebGetTransferStatus Function

Return status information about the progress of a data transfer

### Prototype

```c
BOOL WINAPI WebGetTransferStatus(
    HSTORAGE hStorage,
    LPWEB_STORAGE_TRANSFER lpStatus
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | Handle to the storage container. |
| `lpStatus` | `LPWEB_STORAGE_TRANSFER` | A pointer to a WEB_STORAGE_TRANSFER structure which contains information about the status of the current data transfer. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebGetTransferStatus function returns information about the current data transfer, including the average number of bytes transferred per second and the estimated amount of time until the transfer completes. If there is no data currently being transferred, this function will return the status of the last successful data transfer.
- In a multithreaded application, any thread in the current process may call this function to obtain status information for the specified storage session.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCancelTransfer](#webcanceltransfer-function)
- [WebRegisterEvent](#webregisterevent-function)

---

## WebInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI WebInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that WebInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to WebUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the WebInitialize or WebUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCloseStorage](#webclosestorage-function)
- [WebOpenStorage](#webopenstorage-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebUninitialize](#webuninitialize-function)

---

## WebMoveObject Function

Move a storage object to a new location and optionally rename the object label

### Prototype

```c
BOOL WINAPI WebMoveObject(
    HSTORAGE hStorage,
    LPCTSTR lpszOldLabel,
    LPCTSTR lpszNewLabel,
    DWORD dwStorageType,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszOldLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the existing storage object to be moved. This parameter must specify a valid object label and cannot be a NULL pointer or an empty string. |
| `lpszNewLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies a new label for the storage object being moved. This parameter may be NULL or point to an empty string, in which case the label name is not changed. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. One of the following values should be specified: |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain information about the new storage object. If this information is not required, this parameter may be a NULL pointer. |

### Options

- **dwStorageType** Parameter

    - `WEB_STORAGE_GLOBAL`: Global storage. Objects stored using this storage type are available to all users. Any changes made to objects using this storage type will affect all users of the application. Unless there is a specific need to limit access to the objects stored by the application to specific domains, local machines or users, it is recommended that you use this storage type when creating new objects.
    - `WEB_STORAGE_DOMAIN`: Local domain storage. Objects stored using this storage type are only available to users in the same local domain, as defined by the domain name or workgroup name assigned to the local system. If the domain or workgroup name changes, objects previously stored using this storage type will not be available to the application.
    - `WEB_STORAGE_MACHINE`: Local machine storage. Objects stored using this storage type are only available to users on the same local machine. The local machine is identified by unique characteristics of the system, including the boot volume GUID. Objects previously stored using this storage type will not be available on that system if the boot disk is reformatted.
    - `WEB_STORAGE_USER`: Current user storage. Objects stored using this storage type are only available to the current user logged in on the local machine. The user identifier is based on the Windows user SID that is assigned when the user account is created. If the user account is deleted, the objects previously stored using this storage type will not be available to the application.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebMoveObject function is used to move an existing storage object to a new container. For example, it can move an object originally created using WEB_STORAGE_USER to the WEB_STORAGE_MACHINE container. The dwStorageType parameter must specify a valid storage container type and cannot be WEB_STORAGE_DEFAULT.
- If the dwStorageType parameter specifies the same container that the object is currently in, and the lpszNewLabel parameter specifies a new label name, this function will simply rename the existing object and is effectively the same as calling the WebRenameObject function.
- To duplicate an existing storage object, use the WebCopyObject function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCopyObject](#webcopyobject-function)
- [WebDeleteObject](#webdeleteobject-function)
- [WebRenameObject](#webrenameobject-function)

---

## WebOpenStorage Function

Open a storage container and return a handle for the client session

### Prototype

```c
HSTORAGE WINAPI WebOpenStorage(
    LPCTSTR lpszAppId,
    DWORD dwStorageType
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAppId` | `LPCTSTR` | A pointer to null terminated string which specifies the application ID for the storage container. The application ID is a string that uniquely identifies the application and can only contain letters, numbers, the period and the underscore character. If this parameter is NULL or an empty string, the default identifier SocketTools.Storage.Default will be used. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. One of the following values should be specified: |

### Options

- **dwStorageType** Parameter

    - `WEB_STORAGE_GLOBAL`: Global storage. Objects stored using this storage type are available to all users. Any changes made to objects using this storage type will affect all users of the application. Unless there is a specific need to limit access to the objects stored by the application to specific domains, local machines or users, it is recommended that you use this storage type when creating new objects.
    - `WEB_STORAGE_DOMAIN`: Local domain storage. Objects stored using this storage type are only available to users in the same local domain, as defined by the domain name or workgroup name assigned to the local system. If the domain or workgroup name changes, objects previously stored using this storage type will not be available to the application.
    - `WEB_STORAGE_MACHINE`: Local machine storage. Objects stored using this storage type are only available to users on the same local machine. The local machine is identified by unique characteristics of the system, including the boot volume GUID. Objects previously stored using this storage type will not be available on that system if the boot disk is reformatted.
    - `WEB_STORAGE_USER`: Current user storage. Objects stored using this storage type are only available to the current user logged in on the local machine. The user identifier is based on the Windows user SID that is assigned when the user account is created. If the user account is deleted, the objects previously stored using this storage type will not be available to the application.

### Return Value

- **Type**: `HSTORAGE`
- If the function succeeds, the return value is a handle to the storage container. If the function fails, the return value is INVALID_HANDLE. To get extended error information, call WebGetLastError.

### Remarks

- The WebOpenStorage function opens the specified storage container and requests an access token for the application. This is the first function that must be called prior to accessing any stored objects.
- The application ID is a string that uniquely identifies the application requesting the access and must have been previously registered with the server by calling the WebRegisterAppId function. If the lpszAppId parameter is NULL or an empty string, the function will use a default internal ID that is allocated for each storage account. You can use this default ID if you wish to share data between all of the applications you create.
- The storage type specifies the type of container that objects will be stored in. In most cases, we recommend using WEB_STORAGE_GLOBAL which means that stored objects will be accessible to all users of your application. However, you can limit access to the stored objects based on the local domain, local machine ID or the current user SID.
- If you specify anything other than global storage, objects can be orphaned if the system configuration changes. For example, if WEB_STORAGE_MACHINE is specified, the objects that are stored there can only be accessed from that computer system. If the system is reconfigured (for example, the boot volume formatted and Windows is reinstalled) the unique identifier for that system will change and the previous objects that were stored by your application can no longer be accessed.
- It is advisable is to store critical application data and configuration information using WEB_STORAGE_GLOBAL and use other non-global storage containers for configuration information that is unique to that system and/or user which is not critical and can be easily recreated.

### Example

```cpp
HSTORAGE hStorage = INVALID_HANDLE;
LPCTSTR lpszAppId = _T("MyCompany.WebTest.1");
LPCTSTR lpszLocalFile = _T("TestDocument.pdf");
LPCTSTR lpszObjectLabel = _T("TestDocument.pdf");
WEB_STORAGE_OBJECT webObject = { 0, };
BOOL bSuccess = FALSE;

// Register the application ID which identifies the application
if (!WebRegisterAppId(lpszAppId))
{
	tprintf(_T("Unable to register the application ID"));
	_exit(0);
}

// Open the application storage container
hStorage = WebOpenStorage(lpszAppId, WEB_STORAGE_GLOBAL);

if (hStorage == INVALID_HANDLE)
{
	tprintf(_T("Unable to open global storage for this application"));
	_exit(0);
}

// Upload a local file to the storage server and return information
// about the stored object when it completes
bSuccess = WebPutFileEx(hStorage,
						lpszLocalFile,
						lpszObjectLabel,
						NULL,
						WEB_OBJECT_DEFAULT,
						0,
						&webObject);

if (bSuccess)
{
	// Print information about the object that was created
	_tprintf(_T("Object: %s\n"), webObject.szObjectId);
	_tprintf(_T("Label: %s\n"), webObject.szLabel);
	_tprintf(_T("Size: %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest: %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The upload failed, display error information
	DWORD dwError = WebGetLastError();
	TCHAR szError[128];

	WebGetErrorString(dwError, szError, 128);
	_tprintf(_T("WebPutFileEx failed, error 0x%08lX (%s)\n"), dwError, szError);
}

// Release the handle allocated for this storage session
WebCloseStorage(hStorage);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCloseStorage](#webclosestorage-function)
- [WebGetFile](#webgetfile-function)
- [WebGetObject](#webgetobject-function)
- [WebGetStorageId](#webgetstorageid-function)
- [WebPutFile](#webputfile-function)
- [WebPutObject](#webputobject-function)
- [WebRegisterAppId](#webregisterappid-function)

---

## WebPutFile Function

Upload the contents of a local file to a new storage object

### Prototype

```c
BOOL WINAPI WebPutFile(
    HSTORAGE hStorage,
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be uploaded. If a path is not specified, the file will be read from the current working directory. The current user must have read access to the file, and an error will be returned if the function cannot obtain an exclusive lock on the file during the upload process. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object that will be created or replaced. This parameter cannot be NULL or a zero-length string. The function will fail if the label contains any illegal characters. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebPutFile function uploads the contents of a local file to the storage container specified by the hStorage handle. If an object with this name already exists, its data will be replaced with the contents of the file. The object will be created using the default attributes that permit read and write access, and will automatically determine the content type based on the type of file being uploaded.
- If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must first move, rename or delete the existing object.
- If you want to specify the object content type, its attributes, or obtain additional information about the storage object that was created, use the WebPutFileEx function.
- The WebValidateLabel function can be used to ensure a label is valid prior to calling this function. Refer to that function for more information about the difference between Windows file names and object labels.
- If you are uploading a large file and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications.

### Example

```cpp
// Upload a local file to the storage container
if (WebPutFile(hStorage, lpszLocalFile, lpszObjectLabel)
{
	// The file was uploaded successfully
	_tprintf(_T("The file was stored as \"%s\"\n"), lpszObjectLabel);
}
else
{
	// The file could not be uploaded, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to store \"%s\" (%s)\n"), lpszLocalFile, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFile](#webgetfile-function)
- [WebPutFileEx](#webputfileex-function)
- [WebRegisterEvent](#webregisterevent-function)
- [WebUploadFile](#webuploadfile-function)
- [WebValidateLabel](#webvalidatelabel-function)

---

## WebPutFileEx Function

Upload the contents of a local file and return information about the new object

### Prototype

```c
BOOL WINAPI WebPutFileEx(
    HSTORAGE hStorage,
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszContentType,
    DWORD dwAttributes,
    DWORD dwReserved,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be uploaded. If a path is not specified, the file will be read from the current working directory. The current user must have read access to the file, and an error will be returned if the function cannot obtain an exclusive lock on the file during the upload process. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object that will be created or replaced. This parameter cannot be NULL or a zero-length string. The function will fail if the label contains any illegal characters. |
| `lpszContentType` | `LPCTSTR` | A pointer to a null terminated string that identifies the contents of the file being uploaded. If this parameter is a NULL pointer, or specifies a zero-length string, the function will attempt to automatically determine the content type based on the file name extension and the contents of the file. |
| `dwAttributes` | `DWORD` | An unsigned integer that specifies the attributes associated with the storage object. This value can be a combination of one or more of the following bit flags using a bitwise OR operation: |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value should always be zero. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object. This parameter may be NULL if the information is not required. |

### Options

- **dwAttributes** Parameter

    - `WEB_OBJECT_DEFAULT`: Default object attributes. This value is used to indicate the object can be modified, or that the attributes for a previously existing object should not be changed.
    - `WEB_OBJECT_NORMAL`: A normal object that that can be read and modified by the application. This is the default attribute for new objects that are created by the application.
    - `WEB_OBJECT_READONLY`: A read-only object that can only be read by the application. Attempts to modify or replace the contents of the object will fail. Read-only objects can be deleted.
    - `WEB_OBJECT_HIDDEN`: A hidden object. Objects with this attribute are not returned when enumerated using the WebEnumObjects function. The object can only be accessed directly when specifying its label.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebPutFileEx function uploads the contents of a local file to the storage container specified by the hStorage handle. The caller is responsible for specifying the object's content type and the attributes for the object being created.
- If a content type is provided, it must specify a valid MIME media type and subtype. For example, normal text files have a content type of text/plain while an XML-formatted text file would have a content type of text/xml. Files that contain unstructured binary data are typically identified as application/octet-stream. If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must first move, rename or delete the existing object. The WebValidateLabel function can be used to ensure a label is valid prior to calling this function. Refer to that function for more information about the difference between Windows file names and object labels If you are uploading a large file and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications. The WebUploadFile and WebUploadFileEx functions provide similar functionality, but do not require a handle to an open storage container.
- If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must first move, rename or delete the existing object.
- The WebValidateLabel function can be used to ensure a label is valid prior to calling this function. Refer to that function for more information about the difference between Windows file names and object labels
- If you are uploading a large file and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications.
- The WebUploadFile and WebUploadFileEx functions provide similar functionality, but do not require a handle to an open storage container.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Upload a local file to the storage container, automatically
// determining the content type with normal read/write access
if (WebPutFileEx(hStorage, lpszLocalFile, lpszObjectLabel, NULL, WEB_OBJECT_NORMAL, 0, &webObject))
{
	// The file was uploaded, display the object metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The file could not be uploaded, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to store \"%s\" (%s)\n"), lpszLocalFile, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFile](#webgetfile-function)
- [WebPutFile](#webputfile-function)
- [WebRegisterEvent](#webregisterevent-function)
- [WebUploadFile](#webuploadfile-function)
- [WebValidateLabel](#webvalidatelabel-function)

---

## WebPutObject Function

Upload the contents of a memory buffer to a new storage object

### Prototype

```c
BOOL WINAPI WebPutObject(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPCVOID lpvBuffer,
    DWORD dwLength
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object that will be created or replaced. This parameter cannot be NULL or a zero-length string. The function will fail if the label contains any illegal characters. |
| `lpvBuffer` | `LPCVOID` | A pointer to a buffer that contains the data to be stored. If this parameter is NULL, the dwLength parameter must have a value of zero and a storage object will be created which has no data associated with it. |
| `dwLength` | `DWORD` | An unsigned integer value that specifies the number of bytes that will be copied from the lpvBuffer parameter and stored in the object. If the lpvBuffer parameter is NULL, this value must be zero or the function will fail. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebPutObject function stores the contents of a buffer to the storage container specified by the hStorage handle. If an object with this name already exists, its data will be replaced with the contents of the file. The object will be created using the default attributes that permit read and write access, and it will automatically determine the content type based on the data being stored.
- Although storage object labels are similar to Windows file names, they are case-sensitive. When requesting information about an object, your application must specify the label name exactly as it was created using this function. The object label cannot contain wildcard characters.
- If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must first move, rename or delete the existing object.
- If you want to specify the object content type, its attributes, or obtain additional information about the storage object that was created, use the WebPutObjectEx function.
- If you want to upload the contents of a file, the WebPutFile function simplifies this process. The example code below demonstrates how the Windows API can be used to read from a local file and store the contents using WebPutObject.
- If you are storing a large amount of data and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications.

### Example

```cpp
HANDLE hFile = INVALID_HANDLE_VALUE;
LPBYTE lpContents = NULL;
DWORD dwLength = 0;
BOOL bFileRead = FALSE;

// Open a file on the local system and read the contents
// into a buffer that will be stored on the server
hFile = CreateFile(lpszLocalFile,
				GENERIC_READ,
				0,
				NULL,
				OPEN_EXISTING,
				FILE_ATTRIBUTE_NORMAL,
				NULL);

if (hFile == INVALID_HANDLE_VALUE)
{
	// Unable to open the file
	return;
}

// Get the size of the file and allocate a buffer large
// enough to store the contents of the file
dwLength = GetFileSize(hFile, NULL);
lpContents = (LPBYTE)LocalAlloc(LPTR, dwLength + 1);

if (lpContents == NULL)
{
	// Memory allocation failed
	return;
}

bFileRead = ReadFile(hFile, lpContents, dwLength, &dwLength, NULL);
CloseHandle(hFile);

if (!bFileRead)
{
	// Unable to read the contents of the file
	return;
}

// Store the contents of the buffer
if (WebPutObject(hStorage, lpszObjectLabel, lpContents, dwLength))
{
	_tprintf(_T("WebPutObject stored %lu bytes\n"), dwLength);
}
else
{
	// The object could not be created
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to create \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebPutFile](#webputfile-function)
- [WebPutObjectEx](#webputobjectex-function)
- [WebRegisterEvent](#webregisterevent-function)

---

## WebPutObjectEx Function

Upload the contents of a memory buffer and return information about the new object

### Prototype

```c
BOOL WINAPI WebPutObjectEx(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszContentType,
    DWORD dwAttributes,
    DWORD dwReserved,
    LPCVOID lpvBuffer,
    DWORD dwLength,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the storage object that will be created or replaced. This parameter cannot be NULL or a zero-length string. The function will fail if the label contains any illegal characters. |
| `lpszContentType` | `LPCTSTR` | A pointer to a null terminated string that identifies the contents of the buffer being stored. If this parameter is a NULL pointer, or specifies a zero-length string, the function will attempt to automatically determine the content type based on the object label and the contents of the buffer. |
| `dwAttributes` | `DWORD` | An unsigned integer that specifies the attributes associated with the storage object. This value can be a combination of one or more of the following bit flags using a bitwise OR operation: |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value should always be zero. |
| `lpvBuffer` | `LPCVOID` | A pointer to a buffer that contains the data to be stored. If this parameter is NULL, the dwLength parameter must have a value of zero and a storage object will be created which has no data associated with it. |
| `dwLength` | `DWORD` | An unsigned integer value that specifies the number of bytes that will be copied from the lpvBuffer parameter and stored in the object. If the lpvBuffer parameter is NULL, this value must be zero or the function will fail. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object that was created or replaced. This parameter may be NULL if the information is not required. |

### Options

- **dwAttributes** Parameter

    - `WEB_OBJECT_DEFAULT`: Default object attributes. This value is used to indicate the object can be modified, or that the attributes for a previously existing object should not be changed.
    - `WEB_OBJECT_NORMAL`: A normal object that that can be read and modified by the application. This is the default attribute for new objects that are created by the application.
    - `WEB_OBJECT_READONLY`: A read-only object that can only be read by the application. Attempts to modify or replace the contents of the object will fail. Read-only objects can be deleted.
    - `WEB_OBJECT_HIDDEN`: A hidden object. Objects with this attribute are not returned when enumerated using the WebEnumObjects function. The object can only be accessed directly when specifying its label.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebPutObjectEx function uploads the contents of a buffer to the storage container specified by the hStorage handle. The caller is responsible for specifying the object's content type and the attributes for the object being created.
- Although storage object labels are similar to Windows file names, they are case-sensitive. When requesting information about an object, your application must specify the label name exactly as it was created using this function. The object label cannot contain wildcard characters.
- If a content type is provided, it must specify a valid MIME media type and subtype. For example, typical text data has a content type of text/plain while XML-formatted text would have a content type of text/xml. Data which contains unstructured binary data is typically identified as application/octet-stream.
- If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must explicitly move, rename or delete the existing object.
- If you wish to store the contents of a null terminated string, you can use the WebPutTextObject function. If you want to upload the contents of a file, the WebPutFileEx function simplifies this process. The example code below demonstrates how the Windows API can be used to read from a local file and store the contents using WebPutObjectEx.
- If you are storing a large amount of data and want your application to receive progress updates during the data transfer, use the WebRegisterEvent function and provide a pointer to a callback function that will receive event notifications.

### Example

```cpp
HANDLE hFile = INVALID_HANDLE_VALUE;
LPBYTE lpContents = NULL;
DWORD dwLength = 0;
BOOL bFileRead = FALSE;
WEB_STORAGE_OBJECT webObject;

// Open a file on the local system and read the contents
// into a buffer that will be stored on the server
hFile = CreateFile(lpszLocalFile,
				GENERIC_READ,
				0,
				NULL,
				OPEN_EXISTING,
				FILE_ATTRIBUTE_NORMAL,
				NULL);

if (hFile == INVALID_HANDLE_VALUE)
{
	// Unable to open the file
	return;
}

// Get the size of the file and allocate a buffer large
// enough to store the contents of the file 
dwLength = GetFileSize(hFile, NULL);
lpContents = (LPBYTE)LocalAlloc(LPTR, dwLength + 1);

if (lpContents == NULL)
{
	// Memory allocation failed
	return;
}

bFileRead = ReadFile(hFile, lpContents, dwLength, &dwLength, NULL);
CloseHandle(hFile);

if (!bFileRead)
{
	// Unable to read the contents of the file
	return;
}

// Store the contents of the buffer, identifying it as an unstructured
// stream of bytes, and the object is created with read/write access
if (WebPutObjectEx(hStorage,
				lpszObjectLabel,
				_T("application/octet-stream"),
				WEB_OBJECT_NORMAL,
				0,
				lpContents,
				dwLength,
				&webObject))
{
	// The object was created or replaced, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object could not be created
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to create \"%s\" (%s)\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebPutFileEx](#webputfileex-function)
- [WebPutObject](#webputobject-function)
- [WebPutTextObject](#webputtextobject-function)
- [WebRegisterEvent](#webregisterevent-function)

---

## WebPutTextObject Function

Create or replace a text object with the contents of a string buffer

### Prototype

```c
BOOL WINAPI WebPutTextObject(
    HSTORAGE hStorage,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszObjectText,
    INT cchObjectText,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the text object that should be created or updated. |
| `lpszObjectText` | `LPCTSTR` | A pointer to a string which contains the text that should be uploaded to the storage server. This parameter may be NULL, in which case the object is created but no data is stored. |
| `cchObjectText` | `INT` | The number of characters that are stored in the lpszObjectText string. If this value is -1, the size of the string will be determined automatically by counting the number of characters up to the terminating null character. If the lpszObjectText parameter is NULL, this value must be zero. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object. This parameter may be NULL if the information is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebPutTextObject function is used to store the contents of a string as a text object. The string should be terminated with a null character and cannot contain embedded nulls. If the Unicode version of this function is called, the contents of the string will be normalized prior to being converted to UTF-8 using canonical composition, where decomposed characters are combined to create their canonical precomposed equivalent.
- The size of the text object that is created can be different from the length of the string buffer passed to this function, depending on how the text has been encoded and stored. This will almost always be the case when the Unicode version of this function is called because the UTF-16 string will be converted and stored as UTF-8 encoded text. This encoding will typically increase the size of the stored object unless the string only contains ASCII characters.
- This function should only be used to store textual data in a null terminated string, and should never be used to store binary data. If you wish to create or update an object which contains binary data, use the WebPutObjectEx function instead. If you want to upload the contents of a file, the WebPutFileEx function simplifies this process.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.

### Example

```cpp
LPCTSTR lpszObjectLabel = _T("LoremIpsum");
LPCTSTR lpszObjectText = _T("Lorem ipsum dolor sit amet, consectetur adipiscing elit.");
WEB_STORAGE_OBJECT webObject;

if (WebPutTextObject(hStorage, lpszObjectLabel, lpszObjectText, -1, &webObject))
{
	// The object was created or replaced, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	DWORD dwError = WebGetLastError();
	TCHAR szError[128];

	WebGetErrorString(dwError, szError, 128);
	_ftprintf(stderr, _T("Unable to store \"%s\": %s\n"), lpszObjectLabel, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCompareText](#webcomparetext-function)
- [WebGetObjectEx](#webgetobjectex-function)
- [WebGetTextObject](#webgettextobject-function)
- [WebPutFileEx](#webputfileex-function)
- [WebPutObjectEx](#webputobjectex-function)

---

## WebRegisterAppId Function

Register a new application identifier used to store and retrieve data

### Prototype

```c
BOOL WINAPI WebRegisterAppId(
    LPCTSTR lpszAppId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAppId` | `LPCTSTR` | A pointer to a null terminated string which identifies the application requesting access. This parameter cannot be NULL or point to a zero-length string. If the application ID contains illegal characters, the function will fail. See the remarks below on the recommended method for identifying your application. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebRegisterAppId function registers an application ID with the server which uniquely identifies the application that is requesting access to the storage container. The ID must only consist of ASCII letters, numbers, the period and underscore character. Whitespace characters and non-ASCII Unicode characters are not permitted. The maximum length of an application ID string is 64 characters, including the terminating null character.
- It is recommended that you use a standard format for the application ID that consists of your company name, application name and optionally a version number. For example:
- It is important to note that with these two example IDs, although they are similar, they reference two different applications. Objects stored using the first ID will not be accessible using the second ID. If you want to store objects that should be shared between all versions of the application, it is recommended that you use the first form, without the version number. If you want to store objects that should only be accessible to a specific version of your application, then it is recommended that you use the second form that includes the version number.
- It is safe to call this function with an application ID that was previously registered. If the provided application ID has already been registered, this function will succeed. You can choose to call this function every time your application starts to ensure the application has been registered correctly.
- If you no longer wish to use an application ID you have previously registered, you can call the WebUnregisterAppId function. Exercise caution when unregistering an application. This will cause all objects stored using that ID to be deleted by the storage server. Once an application ID has been unregistered, the operation is permanent. Calling WebUnregisterAppId and then WebRegisterAppId again using the same ID will force the system to create new access tokens for your application. You will not be able to regain access to the objects that were previously stored using that ID.
- The application ID is intended to be an application defined human-readable string that uniquely identifies your application. If you want to obtain the internal storage ID associated with your application, use the WebGetStorageId function. The storage ID is a fixed-length string of letters and numbers guaranteed to be unique across all applications that you register.
- It is not required for your application to create a unique application ID. Each storage account has a default internal application ID named SocketTools.Storage.Default. This default ID is used if a NULL pointer or an empty string is specified to functions like WebOpenStorage. It is intended to identify storage available to all applications that you create.
- To enumerate the application IDs that you have created, use the WebGetFirstApplication and WebGetNextApplication functions.

### Example

```cpp
HSTORAGE hStorage = INVALID_HANDLE;
LPCTSTR lpszAppId = _T("MyCompany.WebTest.1");
LPCTSTR lpszLocalFile = _T("TestDocument.pdf");
LPCTSTR lpszObjectLabel = _T("TestDocument.pdf");
WEB_STORAGE_OBJECT webObject = { 0, };
BOOL bSuccess = FALSE;

// Register the application ID which identifies the application
if (!WebRegisterAppId(lpszAppId))
{
	tprintf(_T("Unable to register the application ID"));
	_exit(0);
}

// Open the application storage container
hStorage = WebOpenStorage(lpszAppId, WEB_STORAGE_GLOBAL);

if (hStorage == INVALID_HANDLE)
{
	tprintf(_T("Unable to open global storage for this application"));
	_exit(0);
}

// Upload a local file to the storage server and return information
// about the stored object when it completes
bSuccess = WebPutFileEx(hStorage,
						lpszLocalFile,
						lpszObjectLabel,
						NULL,
						WEB_OBJECT_DEFAULT,
						0,
						&webObject);

if (bSuccess)
{
	// Print information about the object that was created
	_tprintf(_T("Object: %s\n"), webObject.szObjectId);
	_tprintf(_T("Label: %s\n"), webObject.szLabel);
	_tprintf(_T("Size: %I64u\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest: %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The upload failed, display error information
	DWORD dwError = WebGetLastError();
	TCHAR szError[128];

	WebGetErrorString(dwError, szError, 128);
	_tprintf(_T("WebPutFileEx failed, error 0x%08lX (%s)\n"), dwError, szError);
}

// Release the handle allocated for this storage session
WebCloseStorage(hStorage);
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCloseStorage](#webclosestorage-function)
- [WebOpenStorage](#webopenstorage-function)
- [WebGetFirstApplication](#webgetfirstapplication-function)
- [WebGetNextApplication](#webgetnextapplication-function)
- [WebGetStorageId](#webgetstorageid-function)
- [WebUnregisterAppId](#webunregisterappid-function)
- [WebValidateAppId](#webvalidateappid-function)

---

## WebRegisterEvent Function

Register an event handler to receive notifications for the session

### Prototype

```c
INT WINAPI WebRegisterEvent(
    HSTORAGE hStorage,
    UINT nEventId,
    WEBEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `nEventId` | `UINT` | An unsigned integer which specifies which event should be registered with the specified callback function. This parameter cannot be zero. |
| `lpEventProc` | `WEBEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the WebEventProc callback function. If this parameter is NULL, the callback for the specified event is disabled. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function specified by lpEventProc. If the application targets x86 (32-bit) platforms, this parameter must be a 32-bit unsigned integer. If the application targets x64 (64-bit) platforms, this parameter must be a 64-bit unsigned integer. |

### Options

- **nEventId** Parameter

    - `WEB_EVENT_CONNECT`: The connection to the storage server has been established and the session has been authenticated. This is the first event that occurs when initiating an operation to create or retrieve a storage object.
    - `WEB_EVENT_DISCONNECT`: The connection to the storage server has been closed and the session is terminating. This is the last event that occurs after completing an operation to create or retrieve a storage object.
    - `WEB_EVENT_READ`: The contents of an object is being read from the storage container. This event occurs only once after the operation has been initiated by the application. This event can also be generated if there is an error during the process of retrieving the object contents from the container.
    - `WEB_EVENT_WRITE`: The contents of an object is being written to the storage container. This event occurs only once after the operation has been initiated by the application. This event can also be generated if there is an error during the process of submitting the object contents to the container.
    - `WEB_EVENT_TIMEOUT`: The operation has exceeded the specified timeout period. The application may attempt to retry the operation or report an error to the user. This event typically indicates a connectivity problem with the storage server.
    - `WEB_EVENT_CANCEL`: The operation has been canceled. This event occurs after the application calls WebCancelTransfer while an object is being stored or retrieved.
    - `WEB_EVENT_PROGRESS`: A storage operation is in progress. This event periodically occurs as the contents of a storage object is being read or written from the container. To retrieve information about the status of the operation, the application should register a handler for this event and call the WebGetTransferStatus function from within that handler.

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebRegisterEvent function associates a callback function with a specific event. The event handler is a WebEventProc function that is invoked when the event occurs. Arguments are passed to the function to identify the storage handle, the event type and the user-defined value specified when the event handler is registered. If the event occurs because of an error condition, the error code will be provided to the handler.
- Events are only generated as the result of a call to the WebGetFile, WebGetFileEx, WebPutFile and WebPutFileEx functions. Other storage functions will not generate event notifications.
- This function is typically used to register an event handler that is invoked while the contents of a storage object is being transferred. The WEB_EVENT_PROGRESS event will only be generated periodically during the transfer to ensure the application is not flooded with event notifications. It is guaranteed that at least one WEB_EVENT_PROGRESS notification will occur at the beginning of the transfer, and one at the end of the transfer when it has completed.
- The callback function specified by the lpEventProc parameter must be declared using the __stdcall calling convention. This ensures the arguments passed to the event handler are pushed on to the stack in the correct order. Failure to use the correct calling convention will corrupt the stack and cause the application to terminate abnormally.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebCancelTransfer](#webcanceltransfer-function)
- [WebEventProc](#webeventproc-function)
- [WebGetTransferStatus](#webgettransferstatus-function)

---

## WebRenameObject Function

Change the label associated with a storage object

### Prototype

```c
BOOL WINAPI WebRenameObject(
    HSTORAGE hStorage,
    LPCTSTR lpszOldLabel,
    LPCTSTR lpszNewLabel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `lpszOldLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the existing storage object to be renamed. This parameter must specify a valid object label and cannot be a NULL pointer or an empty string. |
| `lpszNewLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies a new label for the storage object being moved. This parameter may not be NULL or point to an empty string. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebRenameObject function is used to change the label for an existing storage object. Although storage object labels are similar to Windows file names, they are case-sensitive. When renaming an object, your application must specify the original label name exactly as it was created. The object label cannot contain wildcard characters.
- To duplicate an existing storage object, use the WebCopyObject function. If you need to move the object to a different storage container, use the WebMoveObject function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebCopyObject](#webcopyobject-function)
- [WebDeleteObject](#webdeleteobject-function)
- [WebMoveObject](#webmoveobject-function)

---

## WebResetStorage Function

Resets the application storage container and deletes all stored objects

### Prototype

```c
BOOL WINAPI WebResetStorage(
    HSTORAGE hStorage
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The storage container contains information for each of the objects that have been stored using the handle returned by the WebOpenStorage function. Each of these objects are associated with both the application ID and the storage type that was specified by the caller. This function instructs the server to reset the container back to its initial state, deleting all of the objects that were stored in it.
- Exercise caution when using this function. The reset operation is immediate and the objects that are stored in the container are permanently deleted. They cannot be recovered by your application.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebDeleteObject](#webdeleteobject-function)
- [WebOpenStorage](#webopenstorage-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## WebResetStorageEx Function

Resets the specified storage container for an application and deletes all stored objects

### Prototype

```c
BOOL WINAPI WebResetStorageEx(
    HSTORAGE hStorage,
    DWORD dwStorageType,
    LPVOID lpReserved,
    LPDWORD lpdwObjects
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. One of the following values should be specified: |
| `lpReserved` | `LPVOID` | A reserved parameter that should always be NULL. If your project targets the x64 platform, this must be a 64-bit pointer with a value of zero. If this parameter is not a NULL pointer, the function will fail. |
| `lpdwObjects` | `LPDWORD` | A pointer to an unsigned integer value which will contain the number of objects that were deleted when the contents of the container were expunged. |

### Options

- **dwStorageType** Parameter

    - `WEB_STORAGE_GLOBAL`: Global storage. Objects stored using this storage type are available to all users. Any changes made to objects using this storage type will affect all users of the application. Unless there is a specific need to limit access to the objects stored by the application to specific domains, local machines or users, it is recommended that you use this storage type when creating new objects.
    - `WEB_STORAGE_DOMAIN`: Local domain storage. Objects stored using this storage type are only available to users in the same local domain, as defined by the domain name or workgroup name assigned to the local system. If the domain or workgroup name changes, objects previously stored using this storage type will not be available to the application.
    - `WEB_STORAGE_MACHINE`: Local machine storage. Objects stored using this storage type are only available to users on the same local machine. The local machine is identified by unique characteristics of the system, including the boot volume GUID. Objects previously stored using this storage type will not be available on that system if the boot disk is reformatted.
    - `WEB_STORAGE_USER`: Current user storage. Objects stored using this storage type are only available to the current user logged in on the local machine. The user identifier is based on the Windows user SID that is assigned when the user account is created. If the user account is deleted, the objects previously stored using this storage type will not be available to the application.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The storage container contains information for each of the objects that have been stored using the handle returned by the WebOpenStorage function. Each of these objects are associated with both the application ID and the storage type that was specified by the caller. This function instructs the server to reset the container back to its initial state, deleting all of the objects that were stored in it.
- Exercise caution when using this function. The reset operation is immediate and the objects that are stored in the container are permanently deleted. They cannot be recovered by your application.
- Unlike the WebResetStorage function which deletes all of the objects in the container opened using WebOpenStorage, this function enables you reset the contents of a different container without explicitly opening it. For example, if you used WebOpenStorage to open the WEB_STORAGE_GLOBAL container, you could call this version of the function using that handle, but specifying the WEB_STORAGE_USER container to remove all of the objects created by the current user without affecting the stored objects in the global container.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebDeleteObject](#webdeleteobject-function)
- [WebOpenStorage](#webopenstorage-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebResetStorage](#webresetstorage-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## WebSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI WebSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the last error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or WEB_ERROR. Those functions which call WebSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the WebGetLastError function. The use of WebGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebGetErrorString](#webgeterrorstring-function)
- [WebGetLastError](#webgetlasterror-function)

---

## WebSetStorageTimeout Function

Set the number of seconds until a storage operation times out

### Prototype

```c
INT WINAPI WebSetStorageTimeout(
    HSTORAGE hStorage,
    INT nSeconds
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `hStorage` | `HSTORAGE` | A handle to the storage container. |
| `nSeconds` | `INT` | The number of seconds to wait for a storage operation to complete. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the previous timeout period set for this storage handle. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The default timeout period is 10 seconds. The minimum timeout period is 5 seconds, and the maximum timeout period is 60 seconds. Values outside of this range will be normalized internally and will not cause the function to fail.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebGetStorageTimeout](#webgetstoragetimeout-function)
- [WebOpenStorage](#webopenstorage-function)

---

## WebUninitialize Function

Terminates the use of the library

### Prototype

```c
VOID WINAPI WebUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful WebInitialize call before it can call any of the other library functions. When it has completed the use of library, the application must call WebUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to WebUninitialize for every successful call to WebInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`

### See Also

- [WebCloseStorage](#webclosestorage-function)
- [WebInitialize](#webinitialize-function)
- [WebOpenStorage](#webopenstorage-function)
- [WebRegisterAppId](#webregisterappid-function)

---

## WebUnregisterAppId Function

Unregister the application identifier and delete all associated storage objects

### Prototype

```c
BOOL WINAPI WebUnregisterAppId(
    LPCTSTR lpszAppId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAppId` | `LPCTSTR` | A pointer to a null terminated string which specifies the application ID to be deleted. This parameter cannot be NULL or point to a zero-length string. If the application ID contains illegal characters, the function will fail. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebUnregisterAppId function deletes the internal storage identifier associated with the application ID and revokes all access tokens that were granted for the application. This operation is immediate and permanent.
- Exercise caution when using this function. This will permanently delete all objects that were stored for the specified application. Calling WebUnregisterAppId and then WebRegisterAppId again using the same ID will force the system to create new access tokens for your application. You will not be able to regain access to the objects that were previously stored using that ID.
- This function cannot be used to unregister the default storage application identifier SocketTools.Storage.Default. If this ID is specified, the function will fail with an error indicating that the ID is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebOpenStorage](#webopenstorage-function)
- [WebRegisterAppId](#webregisterappid-function)
- [WebResetStorage](#webresetstorage-function)
- [WebValidateAppId](#webvalidateappid-function)

---

## WebUploadFile Function

Upload the contents of a local file, creating or overwriting a stored object

### Prototype

```c
BOOL WINAPI WebUploadFile(
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel,
    LPWEB_STORAGE_OBJECT lpObject
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be created or overwritten with the contents of the storage object. If a path is not specified, the file will be created in the current working directory. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. This parameter cannot be NULL or an empty string. Refer to the WebValidateLabel function for more information about object labels. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object that has been downloaded. This parameter may be NULL if the information is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebUploadFile function uploads the contents of a local file and either creates a new storage object, or replaces an object if one already exists with the same label. The object will be created in the storage container WEB_STORAGE_GLOBAL using the application ID SocketTools.Storage.Default. This is the same default container used by the WebDownloadFile function.
- If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must explicitly move, rename or delete the existing object.
- Unlike the WebPutFile and WebPutFileEx functions, it is not required that you open a storage container using the WebOpenStorage function prior to calling WebUploadFile.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.
- The WebUploadFileEx function provides similar functionality with a more complex interface that supports custom application IDs, additional options and the ability to provide a callback function that is invoked during the upload process.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Upload a local file to the global storage container
if (WebUploadFile(lpszLocalFile, lpszObjectLabel, &webObject))
{
	// The object was uploaded, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object could not be uploaded, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to upload \"%s\" (%s)\n"), lpszLocalFile, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFile](#webgetfile-function)
- [WebPutFile](#webputfile-function)
- [WebUploadFileEx](#webuploadfileex-function)
- [WebValidateLabel](#webvalidatelabel-function)

---

## WebUploadFileEx Function

Upload the contents of a local file with progress notifications

### Prototype

```c
BOOL WINAPI WebUploadFileEx(
    LPCTSTR lpszLocalFile,
    LPCTSTR lpszObjectLabel,
    LPCTSTR lpszAppId,
    DWORD dwStorageType,
    DWORD dwAttributes,
    DWORD dwReserved,
    DWORD dwTimeout,
    LPWEB_STORAGE_OBJECT lpObject,
    WEBEVENTPROC lpEventProc,
    DWORD_PTR dwParam
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLocalFile` | `LPCTSTR` | A pointer to a null terminated string that specifies the name of the local file that will be created or overwritten with the contents of the storage object. If a path is not specified, the file will be created in the current working directory. |
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string that specifies the label of the object that should be retrieved from the server. |
| `lpszAppId` | `LPCTSTR` | A pointer to null terminated string which specifies the application ID for the storage container. The application ID is a string that uniquely identifies the application and can only contain letters, numbers, the period and the underscore character. If this parameter is NULL or an empty string, the default identifier SocketTools.Storage.Default will be used. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. One of the following values should be specified: |
| `dwAttributes` | `DWORD` | An unsigned integer that specifies the attributes associated with the storage object. This value can be a combination of one or more of the following bit flags using a bitwise OR operation: |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value should always be zero. |
| `dwTimeout` | `DWORD` | An unsigned integer value that specifies a timeout period in seconds. If this value is zero, a default timeout period will be used. |
| `lpObject` | `LPWEB_STORAGE_OBJECT` | A pointer to a WEB_STORAGE_OBJECT structure that will contain additional information about the object that has been downloaded. This parameter may be NULL if the information is not required. |
| `lpEventProc` | `WEBEVENTPROC` | Specifies the procedure-instance address of the application defined callback function. For more information about the callback function, see the description of the WebEventProc callback function. If this parameter is NULL, no callback function will be invoked during the data transfer. |
| `dwParam` | `DWORD_PTR` | A user-defined integer value that is passed to the callback function specified by lpEventProc. If the application targets the x86 (32-bit) platform, this parameter must be a 32-bit unsigned integer. If the application targets the x64 (64-bit) platform, this parameter must be a 64-bit unsigned integer. |

### Options

- **dwStorageType** Parameter

    - `WEB_STORAGE_GLOBAL`: Global storage. Objects stored using this storage type are available to all users. Any changes made to objects using this storage type will affect all users of the application. Unless there is a specific need to limit access to the objects stored by the application to specific domains, local machines or users, it is recommended that you use this storage type when creating new objects.
    - `WEB_STORAGE_DOMAIN`: Local domain storage. Objects stored using this storage type are only available to users in the same local domain, as defined by the domain name or workgroup name assigned to the local system. If the domain or workgroup name changes, objects previously stored using this storage type will not be available to the application.
    - `WEB_STORAGE_MACHINE`: Local machine storage. Objects stored using this storage type are only available to users on the same local machine. The local machine is identified by unique characteristics of the system, including the boot volume GUID. Objects previously stored using this storage type will not be available on that system if the boot disk is reformatted.
    - `WEB_STORAGE_USER`: Current user storage. Objects stored using this storage type are only available to the current user logged in on the local machine. The user identifier is based on the Windows user SID that is assigned when the user account is created. If the user account is deleted, the objects previously stored using this storage type will not be available to the application.
- **dwAttributes** Parameter

    - `WEB_OBJECT_DEFAULT`: Default object attributes. This value is used to indicate the object can be modified, or that the attributes for a previously existing object should not be changed.
    - `WEB_OBJECT_NORMAL`: A normal object that that can be read and modified by the application. This is the default attribute for new objects that are created by the application.
    - `WEB_OBJECT_READONLY`: A read-only object that can only be read by the application. Attempts to modify or replace the contents of the object will fail. Read-only objects can be deleted.
    - `WEB_OBJECT_HIDDEN`: A hidden object. Objects with this attribute are not returned when enumerated using the WebEnumObjects function. The object can only be accessed directly when specifying its label.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebUploadFileEx function uploads the contents of a local file and either creates a new storage object, or replaces an object if one already exists with the same label. The WebUploadFile function provides a simpler interface that defaults to uploading an object to the global storage container.
- If the label identifies an object that already exists in the container, and that object was created with the WEB_OBJECT_READONLY attribute, this function will fail. To replace a read-only object, the application must explicitly move, rename or delete the existing object.
- Unlike the WebPutFile and WebPutFileEx functions, it is not required that you open a storage container using the WebOpenStorage function prior to calling WebUploadFileEx.
- Additional metadata about the object will be returned in the WEB_STORAGE_OBJECT structure provided by the caller, such as the date and time the object was created, the content type and the SHA-256 hash of the object contents.
- If you are uploading a large file and want your application to receive progress updates during the data transfer, provide a pointer to a callback function as the lpEventProc parameter. That function will receive event notifications as the data is being uploaded.

### Example

```cpp
WEB_STORAGE_OBJECT webObject;

// Upload a local file to the global storage container
if (WebUploadFileEx(lpszLocalFile,
					lpszObjectLabel,
					lpszAppId,
					WEB_STORAGE_GLOBAL,
					WEB_OBJECT_DEFAULT,
					0,
					&webObject,
					NULL, 0))
{
	// The object was uploaded, display the metadata
	_tprintf(_T("Object:  %s\n"), webObject.szObjectId);
	_tprintf(_T("Label:   %s\n"), webObject.szLabel);
	_tprintf(_T("Size:    %lu\n"), webObject.dwObjectSize);
	_tprintf(_T("Digest:  %s\n"), webObject.szDigest);
	_tprintf(_T("Content: %s\n"), webObject.szContent);
}
else
{
	// The object could not be uploaded, display the error
	TCHAR szError[128];

	WebGetErrorString(WebGetLastError(), szError, 128);    
	_tprintf(_T("Unable to upload \"%s\" (%s)\n"), lpszLocalFile, szError);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebDownloadFile](#webdownloadfile-function)
- [WebGetFileEx](#webgetfileex-function)
- [WebPutFileEx](#webputfileex-function)
- [WebUploadFile](#webuploadfile-function)

---

## WebValidateAppId Function

Validate the specified application identifier

### Prototype

```c
BOOL WINAPI WebValidateAppId(
    LPCTSTR lpszAppId
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszAppId` | `LPCTSTR` | A pointer to a null terminated string which specifies the application ID to be validated. This parameter cannot be NULL or point to a zero-length string. If the application ID contains illegal characters, the function will fail. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- The WebValidateAppId function is used to determine if the specified application identifier is valid and has been previously registered using the WebRegisterAppId function. The ID must only consist of ASCII letters, numbers, the period and underscore character. Whitespace characters and non-ASCII Unicode characters are not permitted. The maximum length of an application ID string is 64 characters, including the terminating null character.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebRegisterAppId](#webregisterappid-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## WebValidateLabel Function

Check the specified string to ensure it is a valid object label

### Prototype

```c
BOOL WINAPI WebValidateLabel(
    LPCTSTR lpszObjectLabel
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszObjectLabel` | `LPCTSTR` | A pointer to a null terminated string which specifies the object label to be validated. This parameter cannot be NULL or point to a zero-length string. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is a non-zero. If the function fails, the return value is zero. To get extended error information, call WebGetLastError.

### Remarks

- Object labels are similar to Windows file names, except they are case-sensitive. The maximum length of a label string is 512 characters, including the terminating null character. Leading and trailing whitespace (spaces, tabs, linebreaks, etc.) are ignored in label names.
- Illegal characters include ASCII and Unicode control characters 1 through 31, single quotes (39), double quotes (34), less than symbol (60), greater than symbol (62), pipe (124), asterisk (42) and question mark (63). A null character (0) specifies the end of the label and any subsequent characters are ignored. It is not possible to embed null characters in the label name.
- Label names may contain forward slash (47) characters and backslash (92) characters, however it is important to note that objects are not stored in a hierarchical structure. An application can create its own folder-like structure to the labels it creates, but this structure is not imposed or enforced by the library.
- If the application is built to use Unicode, labels can contain Unicode characters which are internally encoded as UTF-8. This is important to consider if you have an project built using a multi-byte (ANSI) character set and it needs to access an object that was created using Unicode characters. In that case, the ANSI application must be prepared to handle UTF-8 encoded names and display them appropriately.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cswebv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebRegisterAppId](#webregisterappid-function)
- [WebUnregisterAppId](#webunregisterappid-function)

---

## INITDATA Structure

This structure contains information about the SocketTools library when the initialization function returns.

### Definition

```c
typedef struct _INITDATA
{
    DWORD dwSize;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwVersionBuild;
    DWORD dwOptions;
    DWORD_PTR dwReserved1;
    DWORD_PTR dwReserved2;
    TCHAR szDescription;
} INITDATA, *LPINITDATA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwSize` | `DWORD` | Size of this structure. This structure member must be set to the size of the structure prior to calling the initialization function. Failure to do so will cause the function to fail. |
| `dwVersionMajor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the major version number for the library. |
| `dwVersionMinor` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the minor version number for the library. |
| `dwVersionBuild` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. When the function returns, it will contain the build number for the library. |
| `dwOptions` | `DWORD` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved1` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `dwReserved2` | `DWORD_PTR` | This member must be initialized to a value of zero prior to calling the initialization function. This member is reserved for future use. |
| `szDescription` | `TCHAR` | A null terminated string which describes the library. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## WEB_LOCATION Structure

The WEB_LOCATION structure contains information about the current location of the local computer system.

### Definition

```c
typedef struct _WEB_LOCATION
{
    TCHAR szLocationId;
    TCHAR szIPAddress;
    UINT nAutonomousSytemNumber;
    TCHAR szOrganization;
    TCHAR szRegionName;
    UINT nRegionCode;
    TCHAR szCountryName;
    TCHAR szCountryAlpha2;
    TCHAR szCountryAlpha3;
    UINT nCountryCode;
    TCHAR szSubdivision;
    TCHAR szSubdivisionCode;
    TCHAR szCityName;
    TCHAR szPostalCode;
    TCHAR szCoordinates;
    TCHAR szTimezone;
    TCHAR szTzShortName;
    LONG nTimezoneOffset;
    DOUBLE dLatitude;
    DOUBLE dLongitude;
    SYSTEMTIME stLocalTime;
} WEB_LOCATION, *LPWEB_LOCATION;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `szLocationId` | `TCHAR` | A null-terminated string which contains contains a string of hexadecimal characters which uniquely identifies the location for this computer system. This value is used internally by the location service, and may also be used by the application for its own purposes. If this value changes in subsequent queries, it indicates the external IP address for the local system has changed. |
| `szIPAddress` | `TCHAR` |  |
| `nAutonomousSytemNumber` | `UINT` |  |
| `szOrganization` | `TCHAR` | A null-terminated string which identifies the organization associated with the local system's external IP address. For residential end-users this is typically the name of their Internet Service provider, however it may also identify a private company such as Microsoft, Google or Amazon. Because of the nature of how this information is updated, the organization names can change over time due to acquisitions or changes of ownership. If the owner of the network cannot be determined, this member may contain an empty string. |
| `szRegionName` | `TCHAR` | A null-terminated string which identifies the region in which the external IP address is located. This refers to a broad geographical area, such as "North America" or "Southeast Asia" and uses the conventions for supranational regions as defined by UN M49 codes. These names will always be in English, regardless of the current system locale. |
| `nRegionCode` | `UINT` | An unsigned integer value which identifies the geographical region in which the external IP address is located. This value corresponds to the name returned in the szRegionName structure member. The numeric values use the UN M49 standard established by the United Nations Statistics Division. |
| `szCountryName` | `TCHAR` | A null-terminated string which contains the full name of the country in which the external IP address is located. These names will always be in English, regardless of the current system locale. |
| `szCountryAlpha2` | `TCHAR` | A null-terminated string which contains the ISO 3166-1 alpha-2 code for the country the external IP address is located in. This is a two-letter country code established by the International Organization for Standardization (ISO). For example, the code for the United States is "US". |
| `szCountryAlpha3` | `TCHAR` | A null-terminated string which contains the ISO 3166-1 alpha-3 code for the country the external IP address is located in. This is a three-letter country code established by the International Organization for Standardization (ISO). For example, the code for the United States is "USA". |
| `nCountryCode` | `UINT` | An unsigned integer value which identifies the country where the external IP address is located. These codes can be up to three digits (usually displayed with leading zeros as necessary) and correspond to the country codes assigned by the United Nations. For example, the code for the United States is 840. It is important to note that these are not international dialing codes and should not be used with telephony applications. |
| `szSubdivision` | `TCHAR` | A null-terminated string which identifies the geopolitical subdivision within a country where the external IP address is located. In the United States, this will contain the full name of the state or commonwealth. In Canada, this will contain the name of the province or territory. These names will always be in English, regardless of the current system locale. |
| `szSubdivisionCode` | `TCHAR` | A null-terminated string which is either a two- or three-letter code which identifies a geopolitical subdivision within the country where the external IP address is located. These codes are defined by the ISO 3166-2 standard. For example, the code for the state of California in the United States is "CA". For specificity, these subdivision codes are often combined with the ISO 3166-1 alpha-2 or alpha-3 code for the country the subdivision is located within. Using the previous example of California, ISO alpha-2 code and subdivision code would be combined as "US-CA". |
| `szCityName` | `TCHAR` | A null-terminated string which identifies the city in which the external IP address is located. These names will always be in English, regardless of the current system locale. If the city name cannot be determined, this member may contain an empty string. |
| `szPostalCode` | `TCHAR` | A null-terminated string which contains the postal code associated with the area where the IP address is located. In the United States, this is a 5-digit numeric code. In Canada, this will contain the forward sortation area (the first three characters of the six character postal code). Local delivery portions of a postal code (such as the ZIP+4 code in the United States, or the local delivery unit in Canada) are not included. This information will be most accurate within the geographic region of North America. Postal codes are locale specific, and this structure member may contain an empty string if the postal code for the location cannot be determined. |
| `szCoordinates` | `TCHAR` |  |
| `szTimezone` | `TCHAR` | A null-terminated string which specifies the full time zone name in which the external IP address is located. These names are defined by the Internet Assigned Numbers Authority (IANA) and have values like "America/Los_Angeles" and "Europe/London". These time zones may also be defined as "Etc/GMT+10" if there is not a regional name associated with the time zone. |
| `szTzShortName` | `TCHAR` | A null-terminated string which specifies the abbreviated time zone code in which the external IP address is located. If daylight savings time is used within the time zone, then this value can change based on whether or not daylight savings is in effect. For example, if the IP address is located within the Pacific time zone in the United States, this will return "PDT" when daylight savings is in effect and "PST" when it is not. If a short time zone code cannot be determined, a value such as "UTC+9" may be returned, indicating the number of hours ahead or behind UTC. |
| `nTimezoneOffset` | `LONG` | A signed integer which specifies the number of seconds east or west of the prime meridian (UTC). A positive value indicates a time zone that is east of the prime meridian and a negative value indicates a time zone that is west of the prime meridian. For example, the Pacific time zone in the western United States during daylight savings time would have a value of -25200 (7 hours). |
| `dLatitude` | `DOUBLE` | A floating point value which specifies the latitude of the location in decimal format. A positive value indicates a location that is north of the equator, while a negative value is a location that is south of the equator. |
| `dLongitude` | `DOUBLE` | A floating point value which specifies the longitude of the location in decimal format. A positive value indicates a location that is east of the prime meridian, while a negative value is a location that is west of the prime meridian. |
| `stLocalTime` | `SYSTEMTIME` | A SYSTEMTIME structure that contains information about the current date and time at the location, adjusted for its time zone and whether or not it's in daylight savings time. |

### Remarks

- Use the WebGetLocation function to populate this structure with information about the physical location of the local computer system.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [WebGetLocation](#webgetlocation-structure)

---

## WEB_STORAGE_APPLICATION Structure

The WEB_STORAGE_APPLICATION structure contains information about a registered application.

### Definition

```c
typedef struct _WEB_STORAGE_APPLICATION
{
    TCHAR szAppId;
    TCHAR szApiKey;
    TCHAR szLuid;
    DWORD dwTokens;
    DWORD dwReserved;
    SYSTEMTIME stCreated;
    SYSTEMTIME stUpdated;
} WEB_STORAGE_APPLICATION, *LPWEB_STORAGE_APPLICATION;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `szAppId` | `TCHAR` | A null-terminated string which contains the application ID that was registered using the WebRegisterAppId function. |
| `szApiKey` | `TCHAR` | A null-terminated string which contains the API key value that is used internally by the storage services. The AppID is effectively the human readable alias for this key value used to identify the stored objects for the application. This value is guaranteed to be unique across all applications registered with the storage service. |
| `szLuid` | `TCHAR` | A null-terminated string which specifies a locally unique value associated with the registered application. This value is used internally by the storage service and guaranteed to be unique to the storage account that has registered the application. Note that there are no functions that currently accept the application LUID as parameter, but you may choose to use this value in your own application for other purposes. |
| `dwTokens` | `DWORD` | An unsigned integer value which specifies the number of access tokens associated with the registered application. Typically there is only one access token associated with a given AppID at any one time, although it is possible that the API may allocate additional access tokens under some circumstances. |
| `dwReserved` | `DWORD` | An unsigned integer value that is reserved for future use. This value will always be zero. |
| `stCreated` | `SYSTEMTIME` | A SYSTEMTIME structure that specifies the date and time the AppID was registered. This value is represented using Coordinated Universal Time (UTC) and is not adjusted for the local time zone. |
| `stUpdated` | `SYSTEMTIME` | A SYSTEMTIME structure that specifies the date and time the AppID object was last updated. This value is represented using Coordinated Universal Time (UTC) and is not adjusted for the local time zone. When a storage object is first created, this value will be the same as the object creation time. |

### Remarks

- This structure is used in conjunction with the WebEnumApplications function which is used to enumerate all of the registered applications for the storage account. It is also used with the WebGetFirstApplication and WebGetNextApplication functions which provide an alternative method for enumerating registered applications.
- To adjust the creation and update times to account for the local time zone, use the SystemTimeToTzSpecificLocalTime function. If you prefer to use FILETIME values, use the SystemTimeToFileTime function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## WEB_STORAGE_OBJECT Structure

The WEB_STORAGE_OBJECT structure contains information about an individual storage object.

### Definition

```c
typedef struct _WEB_STORAGE_OBJECT
{
    TCHAR szObjectId;
    TCHAR szLabel;
    TCHAR szDigest;
    TCHAR szContent;
    DWORD dwAttributes;
    DWORD dwObjectSize;
    SYSTEMTIME stCreated;
    SYSTEMTIME stModified;
} WEB_STORAGE_OBJECT, *LPWEB_STORAGE_OBJECT;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `szObjectId` | `TCHAR` | A null-terminated string which contains the unique identifier associated with this object. Object IDs are guaranteed to be unique for each storage object that is created by the application. The maximum length of an object ID is 64 characters, including the terminating null character. |
| `szLabel` | `TCHAR` | A null-terminated string which contains the label assigned to the object by the application. Object labels are case-sensitive and must be unique for each object. An application uses labels to reference an object with a human-recognizable name, rather than referencing them by their object ID. The maximum length of an object label is 512 characters, including the terminating null character. |
| `szDigest` | `TCHAR` | A null-terminated string which specifies the digest of the object contents, computed using an SHA-256 hash. The maximum length of the szDigest string is 128 characters, including the terminating null character. However, the digest value is always represented as a string of hexadecimal numbers that is exactly 64 characters long. It is important to note that even a zero-length object will have a digest, which is the standard SHA-256 NULL hash value. |
| `szContent` | `TCHAR` | A null-terminated string which specifies the MIME content type for the storage object. The content type is determined by the object label and evaluating the contents of the object. It is also possible for the application to explicitly specify the content type of the object when it is created. |
| `dwAttributes` | `DWORD` | An unsigned integer value that specifies the attributes for the storage object. The object attributes are comprised of one or more bit flags: |
| `dwObjectSize` | `DWORD` | An unsigned integer value that specifies the size of the storage object in bytes. The maximum size of an individual object is determined by the storage quota limits established for the account. |
| `stCreated` | `SYSTEMTIME` | A SYSTEMTIME structure that specifies the date and time the storage object was created. This value is represented using Coordinated Universal Time (UTC) and is not adjusted for the local time zone. |
| `stModified` | `SYSTEMTIME` | A SYSTEMTIME structure that specifies the date and time the storage object was last modified. This value is represented using Coordinated Universal Time (UTC) and is not adjusted for the local time zone. When a storage object is first created, this value will be the same as the object creation time. |

### Remarks

- The object content type will always be in the format type/subtype where the type specifies a common media type (e.g.: text, audio, video, etc.) and subtype specifies the specific content. The most common content type for text files is text/plain. If the content type is unknown, the default content type is application/octet-stream.
- Text objects may also optionally include the character encoding as part of the content type. For example, if an object contains UTF-8 encoded text, the content type may be returned as text/plain; charset=utf-8. If your application is parsing the content types, you must check if a character encoding was also included in the value. Text objects that do not specify an encoding either contain ASCII or text which uses the system code page. Unicode text will always be stored using UTF-8 encoding.
- To adjust the object creation and modification times to account for the local time zone, use the SystemTimeToTzSpecificLocalTime function. If you prefer to use FILETIME values, use the SystemTimeToFileTime function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

---

## WEB_STORAGE_QUOTA Structure

The WEB_STORAGE_QUOTA structure contains information about the current storage account data usage and limits.

### Definition

```c
typedef struct _WEB_STORAGE_QUOTA
{
    DWORD dwObjects;
    DWORD dwObjectLimit;
    DWORD dwObjectSize;
    ULONGLONG ulBytesUsed;
    ULONGLONG ulBytesFree;
    ULONGLONG ulStorageLimit;
} WEB_STORAGE_QUOTA, *LPWEB_STORAGE_QUOTA;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwObjects` | `DWORD` | An unsigned integer value which specifies the number of storage objects allocated for the account. This value may not exceed the total number of objects specified by the dwObjectLimit member. |
| `dwObjectLimit` | `DWORD` | An unsigned integer value which specifies the maximum number of storage objects that may be created. In addition to the limit on the total amount of storage that may be used, there is a limit on the total number of objects that may be created by all applications. |
| `dwObjectSize` | `DWORD` | An unsigned integer value which specifies the maximum size of an individual storage object. In addition to a limit on the total amount of storage used and the number of objects created, each object stored by the application cannot exceed this size. |
| `ulBytesUsed` | `ULONGLONG` | An unsigned 64-bit integer value which specifies the total number of bytes of data allocated for all storage objects. This value may not exceed the total number of bytes of storage available, which is specified by the ulStorageLimit member. |
| `ulBytesFree` | `ULONGLONG` | An unsigned 64-bit integer value which specifies the number of bytes available for the storage of new objects. This value reflects the total amount of available storage across all applications registered with the development account. If this value is zero, your storage account has reached its storage limit. |
| `ulStorageLimit` | `ULONGLONG` | An unsigned 64-bit integer value which specifies the maximum number of bytes of data storage available. This limit applies to all applications registered with the development account. |

### Remarks

- Storage quota limits are assigned for each SocketTools development account. The WebGetStorageQuota function will populate this structure with information about the limits on your account. Accounts that are created with an evaluation license have much lower quota limits than a standard account and should be used for testing purposes only. After the evaluation period has ended, all objects stored using the evaluation license will be deleted.
- These values do not represent limits on storage usage by a specific application. Quotas limits apply to all applications that are registered with the development account, which is identified with the runtime license key passed to the WebInitialize function.
- If your storage quota has been exceeded, either because the total number of objects or the total bytes of storage has reached their limit, your applications will be unable to create new objects. Your application can continue to access existing objects, regardless of your current quota limits.
- To free storage space, use the WebDeleteObject function to delete individual storage objects that are no longer needed by your application, or use the WebResetStorage function to delete all objects in a container.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

### See Also

- [WebGetStorageQuota](#webgetstoragequota-structure)

---

## WEB_STORAGE_TRANSFER Structure

This structure is used by the WebGetTransferStatus function to return information about a data transfer in progress.

### Definition

```c
typedef struct _WEB_STORAGE_TRANSFER
{
    TCHAR szObjectLabel;
    DWORD dwStorageType;
    DWORD dwBytesTotal;
    DWORD dwBytesCopied;
    DWORD dwBytesPerSecond;
    DWORD dwTimeElapsed;
    DWORD dwTimeEstimated;
} WEB_STORAGE_TRANSFER, *LPWEB_STORAGE_TRANSFER;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `szObjectLabel` | `TCHAR` | A null-terminated string that specifies the label for the object that is being retrieved, created or replaced. |
| `dwStorageType` | `DWORD` | An integer value that identifies the storage container type. This will be one of the following values: |
| `dwBytesTotal` | `DWORD` | An unsigned integer which specifies the total number of bytes that will be transferred. If the object is being downloaded to the local host, this is the size of the stored object. If the data is being uploaded from the local host to be stored on the server, it is the size of the buffer or local file. |
| `dwBytesCopied` | `DWORD` | An unsigned integer which specifies the total number of bytes that have been copied. |
| `dwBytesPerSecond` | `DWORD` | The average number of bytes that have been copied per second. |
| `dwTimeElapsed` | `DWORD` | The number of seconds that have elapsed since the file transfer started. |
| `dwTimeEstimated` | `DWORD` | The estimated number of seconds until the data transfer is completed. This is based on the average number of bytes transferred per second. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

